var require = meteorInstall({"imports":{"server":{"accounts":{"ldap.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/accounts/ldap.js                                                                                     //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  LDAP: function () {                                                                                                  // 1
    return LDAP;                                                                                                       // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var ldapjs = void 0;                                                                                                   // 1
module.watch(require("ldapjs"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    ldapjs = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 1
                                                                                                                       //
// At a minimum, set up LDAP_DEFAULTS.url and .dn according to                                                         // 4
// your needs. url should appear as 'ldap://your.url.here'                                                             // 5
// dn should appear in normal ldap format of comma separated attribute=value                                           // 6
// e.g. 'uid=someuser,cn=users,dc=somevalue'                                                                           // 7
LDAP_DEFAULTS = {                                                                                                      // 8
  url: false,                                                                                                          // 9
  port: "389",                                                                                                         // 10
  dn: false,                                                                                                           // 11
  searchDN: false,                                                                                                     // 12
  searchCredentials: false,                                                                                            // 13
  createNewUser: true,                                                                                                 // 14
  defaultDomain: false,                                                                                                // 15
  searchResultsProfileMap: false,                                                                                      // 16
  base: null,                                                                                                          // 17
  search: "(objectclass=*)",                                                                                           // 18
  ldapsCertificate: false                                                                                              // 19
}; /**                                                                                                                 // 8
    @class LDAP                                                                                                        //
    @constructor                                                                                                       //
    */                                                                                                                 //
                                                                                                                       //
function LDAP() {                                                                                                      // 26
  // Set options                                                                                                       // 27
  this.options = _.clone(LDAP_DEFAULTS);                                                                               // 28
}                                                                                                                      // 29
                                                                                                                       //
; /**                                                                                                                  // 29
   * Attempt to bind (authenticate) ldap                                                                               //
   * and perform a dn search if specified                                                                              //
   *                                                                                                                   //
   * @method ldapCheck                                                                                                 //
   *                                                                                                                   //
   * @param {Object} options  Object with username, ldapPass and overrides for LDAP_DEFAULTS object.                   //
   * Additionally the searchBeforeBind parameter can be specified, which is used to search for the DN                  //
   * if not provided.                                                                                                  //
   */                                                                                                                  //
                                                                                                                       //
LDAP.prototype.ldapCheck = function (db, options) {                                                                    // 41
  var _this = this;                                                                                                    // 43
                                                                                                                       //
  options = options || {};                                                                                             // 45
                                                                                                                       //
  if (options.hasOwnProperty("username") && options.hasOwnProperty("ldapPass") || options.hasOwnProperty("searchUsername")) {
    _this.options.base = db.getLdapBase();                                                                             // 49
    _this.options.url = db.getLdapUrl();                                                                               // 50
    _this.options.searchBeforeBind = {};                                                                               // 51
    _this.options.searchBeforeBind[options.searchUsernameField || db.getLdapSearchUsername()] = options.searchUsername || options.username;
    _this.options.filter = db.getLdapFilter() || "(objectclass=*)";                                                    // 54
    _this.options.searchBindDn = db.getLdapSearchBindDn();                                                             // 55
    _this.options.searchBindPassword = db.getLdapSearchBindPassword();                                                 // 56
    var resolved = false;                                                                                              // 58
    var ldapAsyncFut = new Future(); // Create ldap client                                                             // 59
                                                                                                                       //
    var fullUrl = db.getLdapUrl();                                                                                     // 62
    var client = null;                                                                                                 // 63
                                                                                                                       //
    var errorFunc = function (err) {                                                                                   // 65
      if (err) {                                                                                                       // 66
        if (resolved) return;                                                                                          // 67
        resolved = true;                                                                                               // 68
        ldapAsyncFut.return({                                                                                          // 69
          error: err                                                                                                   // 70
        });                                                                                                            // 69
      }                                                                                                                // 72
    };                                                                                                                 // 73
                                                                                                                       //
    if (fullUrl.indexOf("ldaps://") === 0) {                                                                           // 75
      var tlsOptions = {};                                                                                             // 76
      var cert = db.getLdapCaCert();                                                                                   // 77
                                                                                                                       //
      if (cert) {                                                                                                      // 78
        tlsOptions.ca = cert;                                                                                          // 79
      }                                                                                                                // 80
                                                                                                                       //
      client = ldapjs.createClient({                                                                                   // 82
        url: fullUrl,                                                                                                  // 83
        tlsOptions: tlsOptions                                                                                         // 84
      }, errorFunc);                                                                                                   // 82
    } else {                                                                                                           // 86
      client = ldapjs.createClient({                                                                                   // 87
        url: fullUrl                                                                                                   // 88
      }, errorFunc);                                                                                                   // 87
    }                                                                                                                  // 90
                                                                                                                       //
    client.on("error", errorFunc);                                                                                     // 92
    var username = options.username;                                                                                   // 94
    var domain = _this.options.defaultDomain;                                                                          // 95
                                                                                                                       //
    if (!options.hasOwnProperty("searchUsername")) {                                                                   // 97
      // Slide @xyz.whatever from username if it was passed in                                                         // 98
      // and replace it with the domain specified in defaults                                                          // 99
      var emailSliceIndex = options.username.indexOf("@"); // If user appended email domain, strip it out              // 100
      // And use the defaults.defaultDomain if set                                                                     // 103
                                                                                                                       //
      if (emailSliceIndex !== -1) {                                                                                    // 104
        username = options.username.substring(0, emailSliceIndex);                                                     // 105
        domain = domain || options.username.substring(emailSliceIndex + 1, options.username.length);                   // 106
      } else {                                                                                                         // 107
        username = options.username;                                                                                   // 108
      }                                                                                                                // 109
    }                                                                                                                  // 110
                                                                                                                       //
    if (_this.options.searchBindDn) {                                                                                  // 112
      var ldapBindFut = new Future();                                                                                  // 113
      client.bind(_this.options.searchBindDn, _this.options.searchBindPassword, function (err) {                       // 114
        if (err) {                                                                                                     // 116
          ldapBindFut.throw(err);                                                                                      // 117
        } else {                                                                                                       // 118
          ldapBindFut.return();                                                                                        // 119
        }                                                                                                              // 120
      });                                                                                                              // 121
                                                                                                                       //
      try {                                                                                                            // 124
        ldapBindFut.wait();                                                                                            // 125
      } catch (err) {                                                                                                  // 126
        return {                                                                                                       // 127
          error: err                                                                                                   // 127
        };                                                                                                             // 127
      }                                                                                                                // 128
    } // initialize result                                                                                             // 129
                                                                                                                       //
                                                                                                                       //
    var retObject = {                                                                                                  // 131
      username: username,                                                                                              // 132
      email: domain ? username + "@" + domain : false,                                                                 // 133
      emptySearch: true,                                                                                               // 134
      searchResults: {}                                                                                                // 135
    };                                                                                                                 // 131
    var filter = _this.options.filter;                                                                                 // 138
    Object.keys(_this.options.searchBeforeBind).forEach(function (searchKey) {                                         // 139
      filter = "&" + filter + "(" + searchKey + "=" + _this.options.searchBeforeBind[searchKey] + ")";                 // 140
    });                                                                                                                // 141
    var searchOptions = {                                                                                              // 143
      scope: "sub",                                                                                                    // 144
      sizeLimit: 1,                                                                                                    // 145
      filter: filter                                                                                                   // 146
    }; // perform LDAP search to determine DN                                                                          // 143
                                                                                                                       //
    client.search(_this.options.base, searchOptions, function (err, res) {                                             // 150
      if (err) {                                                                                                       // 151
        if (resolved) return;                                                                                          // 152
        resolved = true;                                                                                               // 153
        ldapAsyncFut.return({                                                                                          // 154
          error: err                                                                                                   // 155
        });                                                                                                            // 154
        return;                                                                                                        // 157
      }                                                                                                                // 158
                                                                                                                       //
      retObject.emptySearch = true;                                                                                    // 160
      res.on("searchEntry", function (entry) {                                                                         // 161
        retObject.dn = entry.objectName;                                                                               // 162
        retObject.username = retObject.dn;                                                                             // 163
        retObject.emptySearch = false;                                                                                 // 164
        retObject.searchResults = _.omit(entry.object, "userPassword");                                                // 166
                                                                                                                       //
        if (options.hasOwnProperty("searchUsername")) {                                                                // 168
          // This was only a search, return immediately                                                                // 169
          resolved = true;                                                                                             // 170
          ldapAsyncFut.return(retObject);                                                                              // 171
          return;                                                                                                      // 172
        } // use the determined DN to bind                                                                             // 173
                                                                                                                       //
                                                                                                                       //
        client.bind(entry.objectName, options.ldapPass, function (err) {                                               // 176
          try {                                                                                                        // 177
            if (err) {                                                                                                 // 178
              throw new Meteor.Error(err.code, err.message);                                                           // 179
            } else {                                                                                                   // 180
              resolved = true;                                                                                         // 181
              ldapAsyncFut.return(retObject);                                                                          // 182
            }                                                                                                          // 183
          } catch (e) {                                                                                                // 184
            if (resolved) return;                                                                                      // 186
            resolved = true;                                                                                           // 187
            ldapAsyncFut.return({                                                                                      // 188
              error: e                                                                                                 // 189
            });                                                                                                        // 188
          }                                                                                                            // 191
        });                                                                                                            // 192
      });                                                                                                              // 193
      res.on("error", errorFunc); // If no dn is found, return as is.                                                  // 195
                                                                                                                       //
      res.on("end", function (result) {                                                                                // 198
        if (retObject.dn === undefined) {                                                                              // 199
          resolved = true;                                                                                             // 200
          ldapAsyncFut.return(retObject);                                                                              // 201
        }                                                                                                              // 202
      });                                                                                                              // 203
    });                                                                                                                // 204
    return ldapAsyncFut.wait();                                                                                        // 206
  } else {                                                                                                             // 208
    throw new Meteor.Error(403, "Missing LDAP Auth Parameter");                                                        // 209
  }                                                                                                                    // 210
};                                                                                                                     // 212
                                                                                                                       //
LDAP.prototype.updateUserQuota = function (db, user) {                                                                 // 214
  var fallback = {                                                                                                     // 215
    storage: user.cachedStorageQuota || 0,                                                                             // 216
    grains: Infinity,                                                                                                  // 217
    compute: Infinity                                                                                                  // 218
  };                                                                                                                   // 215
  var setting = db.collections.settings.findOne({                                                                      // 221
    _id: "quotaLdapAttribute"                                                                                          // 221
  });                                                                                                                  // 221
  if (!setting || !setting.value) return fallback; // TODO(someday): don't just assume the first login identity is the primary identity?
                                                                                                                       //
  var email = db.getPrimaryEmail(user._id, user.loginCredentials[0].id);                                               // 225
  if (!email) return fallback;                                                                                         // 226
  var ldapUser = void 0;                                                                                               // 228
                                                                                                                       //
  try {                                                                                                                // 229
    ldapUser = this.ldapCheck(db, {                                                                                    // 230
      searchUsername: email,                                                                                           // 230
      searchUsernameField: "mail"                                                                                      // 230
    });                                                                                                                // 230
  } catch (err) {                                                                                                      // 231
    console.error("Error looking up quota from LDAP");                                                                 // 232
    return fallback;                                                                                                   // 233
  }                                                                                                                    // 234
                                                                                                                       //
  if (!ldapUser || !ldapUser.searchResults) return fallback;                                                           // 236
  var newStorageQuota = +ldapUser.searchResults[setting.value];                                                        // 238
                                                                                                                       //
  if (newStorageQuota !== user.cachedStorageQuota) {                                                                   // 239
    Meteor.users.update({                                                                                              // 240
      _id: user._id                                                                                                    // 240
    }, {                                                                                                               // 240
      $set: {                                                                                                          // 240
        cachedStorageQuota: newStorageQuota                                                                            // 240
      }                                                                                                                // 240
    });                                                                                                                // 240
  } // TODO(someday): cache timestamp as well and only check/update if greater than 60s ago                            // 241
                                                                                                                       //
                                                                                                                       //
  return {                                                                                                             // 244
    storage: newStorageQuota,                                                                                          // 245
    grains: Infinity,                                                                                                  // 246
    compute: Infinity                                                                                                  // 247
  };                                                                                                                   // 244
};                                                                                                                     // 249
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"picture.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/accounts/picture.js                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  userPictureUrl: function () {                                                                                        // 1
    return userPictureUrl;                                                                                             // 1
  },                                                                                                                   // 1
  fetchPicture: function () {                                                                                          // 1
    return fetchPicture;                                                                                               // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var HTTP = void 0;                                                                                                     // 1
module.watch(require("meteor/http"), {                                                                                 // 1
  HTTP: function (v) {                                                                                                 // 1
    HTTP = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var userPictureUrl = function (user) {                                                                                 // 3
  if (user.services && !(user.profile && user.profile.picture)) {                                                      // 4
    // Try to determine user's avatar URL from login service.                                                          // 5
    var google = user.services.google;                                                                                 // 7
                                                                                                                       //
    if (google && google.picture) {                                                                                    // 8
      return google.picture;                                                                                           // 9
    }                                                                                                                  // 10
                                                                                                                       //
    var github = user.services.github;                                                                                 // 12
                                                                                                                       //
    if (github && github.id) {                                                                                         // 13
      return "https://avatars.githubusercontent.com/u/" + github.id;                                                   // 14
    } // Note that we do NOT support Gravatar for email addresses because pinging Gravatar would be                    // 15
    // a data leak, revealing that the user has logged into this Sandstorm server. Google and                          // 18
    // Github are different because they are actually the identity providers, so they already know                     // 19
    // the user logged in.                                                                                             // 20
                                                                                                                       //
  }                                                                                                                    // 21
};                                                                                                                     // 22
                                                                                                                       //
var fetchPicture = function (db, url) {                                                                                // 24
  try {                                                                                                                // 25
    var result = HTTP.get(url, {                                                                                       // 26
      npmRequestOptions: {                                                                                             // 27
        encoding: null                                                                                                 // 27
      },                                                                                                               // 27
      timeout: 5000                                                                                                    // 28
    });                                                                                                                // 26
    var metadata = {};                                                                                                 // 31
    metadata.mimeType = result.headers["content-type"];                                                                // 33
                                                                                                                       //
    if (metadata.mimeType.lastIndexOf("image/png", 0) === -1 && metadata.mimeType.lastIndexOf("image/jpeg", 0) === -1) {
      throw new Error("unexpected Content-Type:", metadata.mimeType);                                                  // 36
    }                                                                                                                  // 37
                                                                                                                       //
    var enc = result.headers["content-encoding"];                                                                      // 39
                                                                                                                       //
    if (enc && enc !== "identity") {                                                                                   // 40
      metadata.encoding = enc;                                                                                         // 41
    }                                                                                                                  // 42
                                                                                                                       //
    return db.addStaticAsset(metadata, result.content);                                                                // 44
  } catch (err) {                                                                                                      // 45
    console.error("failed to fetch user profile picture:", url, err.stack);                                            // 46
  }                                                                                                                    // 47
};                                                                                                                     // 48
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"saml-utils.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/accounts/saml-utils.js                                                                               //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  SAML: function () {                                                                                                  // 1
    return SAML;                                                                                                       // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var crypto = void 0;                                                                                                   // 1
module.watch(require("crypto"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    crypto = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var querystring = void 0;                                                                                              // 1
module.watch(require("querystring"), {                                                                                 // 1
  "default": function (v) {                                                                                            // 1
    querystring = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Url = void 0;                                                                                                      // 1
module.watch(require("url"), {                                                                                         // 1
  "default": function (v) {                                                                                            // 1
    Url = v;                                                                                                           // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var xml2js = void 0;                                                                                                   // 1
module.watch(require("xml2js"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    xml2js = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var xmlbuilder = void 0;                                                                                               // 1
module.watch(require("xmlbuilder"), {                                                                                  // 1
  "default": function (v) {                                                                                            // 1
    xmlbuilder = v;                                                                                                    // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var xmlCrypto = void 0;                                                                                                // 1
module.watch(require("xml-crypto"), {                                                                                  // 1
  "default": function (v) {                                                                                            // 1
    xmlCrypto = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
var xmldom = void 0;                                                                                                   // 1
module.watch(require("xmldom"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    xmldom = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 6);                                                                                                                 // 1
var zlib = void 0;                                                                                                     // 1
module.watch(require("zlib"), {                                                                                        // 1
  "default": function (v) {                                                                                            // 1
    zlib = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 7);                                                                                                                 // 1
var HOSTNAME = Url.parse(process.env.ROOT_URL).hostname;                                                               // 10
                                                                                                                       //
var SAML = function (options) {                                                                                        // 12
  this.options = this.initialize(options);                                                                             // 13
};                                                                                                                     // 14
                                                                                                                       //
SAML.prototype.initialize = function (options) {                                                                       // 16
  if (!options) {                                                                                                      // 17
    options = {};                                                                                                      // 18
  }                                                                                                                    // 19
                                                                                                                       //
  if (!options.protocol) {                                                                                             // 21
    options.protocol = "https://";                                                                                     // 22
  }                                                                                                                    // 23
                                                                                                                       //
  if (!options.path) {                                                                                                 // 25
    options.path = "/saml/consume";                                                                                    // 26
  }                                                                                                                    // 27
                                                                                                                       //
  if (!options.issuer) {                                                                                               // 29
    options.issuer = "onelogin_saml";                                                                                  // 30
  }                                                                                                                    // 31
                                                                                                                       //
  if (options.identifierFormat === undefined) {                                                                        // 33
    options.identifierFormat = "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent";                                 // 34
  }                                                                                                                    // 35
                                                                                                                       //
  return options;                                                                                                      // 37
};                                                                                                                     // 38
                                                                                                                       //
SAML.prototype.generateUniqueID = function () {                                                                        // 40
  var chars = "abcdef0123456789";                                                                                      // 41
  var uniqueID = "";                                                                                                   // 42
                                                                                                                       //
  for (var i = 0; i < 20; i++) {                                                                                       // 43
    uniqueID += chars.substr(Math.floor(Math.random() * 15), 1);                                                       // 44
  }                                                                                                                    // 45
                                                                                                                       //
  return uniqueID;                                                                                                     // 47
};                                                                                                                     // 48
                                                                                                                       //
SAML.prototype.generateInstant = function () {                                                                         // 50
  var date = new Date();                                                                                               // 51
  return date.getUTCFullYear() + "-" + ("0" + (date.getUTCMonth() + 1)).slice(-2) + "-" + ("0" + date.getUTCDate()).slice(-2) + "T" + ("0" + date.getUTCHours()).slice(-2) + ":" + ("0" + date.getUTCMinutes()).slice(-2) + ":" + ("0" + date.getUTCSeconds()).slice(-2) + "Z";
};                                                                                                                     // 53
                                                                                                                       //
SAML.prototype.signRequest = function (xml) {                                                                          // 55
  var signer = crypto.createSign("RSA-SHA1");                                                                          // 56
  signer.update(xml);                                                                                                  // 57
  return signer.sign(this.options.privateCert, "base64");                                                              // 58
};                                                                                                                     // 59
                                                                                                                       //
SAML.prototype.generateAuthorizeRequest = function (req) {                                                             // 61
  var id = "_" + this.generateUniqueID();                                                                              // 62
  var instant = this.generateInstant();                                                                                // 63
  var callbackUrl = void 0; // Post-auth destination                                                                   // 65
                                                                                                                       //
  if (this.options.callbackUrl) {                                                                                      // 67
    callbackUrl = this.options.callbackUrl;                                                                            // 68
  } else {                                                                                                             // 69
    callbackUrl = this.options.protocol + req.headers.host + this.options.path;                                        // 70
  }                                                                                                                    // 71
                                                                                                                       //
  if (this.options.id) id = this.options.id;                                                                           // 73
  var request = "<samlp:AuthnRequest xmlns:samlp=\"urn:oasis:names:tc:SAML:2.0:protocol\" ID=\"" + id + "\" Version=\"2.0\" IssueInstant=\"" + instant + "\" ProtocolBinding=\"urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST\" AssertionConsumerServiceURL=\"" + callbackUrl + "\" Destination=\"" + this.options.entryPoint + "\">" + "<saml:Issuer xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\">" + this.options.issuer + "</saml:Issuer>\n";
                                                                                                                       //
  if (this.options.identifierFormat) {                                                                                 // 82
    request += "<samlp:NameIDPolicy xmlns:samlp=\"urn:oasis:names:tc:SAML:2.0:protocol\" Format=\"" + this.options.identifierFormat + "\" AllowCreate=\"true\"></samlp:NameIDPolicy>\n";
  }                                                                                                                    // 85
                                                                                                                       //
  request += "</samlp:AuthnRequest>";                                                                                  // 87
  return request;                                                                                                      // 89
};                                                                                                                     // 90
                                                                                                                       //
SAML.prototype.generateLogoutRequest = function (req) {                                                                // 92
  var id = "_" + this.generateUniqueID();                                                                              // 93
  var instant = this.generateInstant(); //samlp:LogoutRequest xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion"
  // ID="_135ad2fd-b275-4428-b5d6-3ac3361c3a7f" Version="2.0" Destination="https://idphost/adfs/ls/"                   // 97
  //IssueInstant="2008-06-03T12:59:57Z"><saml:Issuer>myhost</saml:Issuer><NameID Format="urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress"
  //NameQualifier="https://idphost/adfs/ls/">myemail@mydomain.com</NameID<samlp:SessionIndex>_0628125f-7f95-42cc-ad8e-fde86ae90bbe
  //</samlp:SessionIndex></samlp:LogoutRequest>                                                                        // 100
                                                                                                                       //
  var request = "<samlp:LogoutRequest xmlns:samlp=\"urn:oasis:names:tc:SAML:2.0:protocol\" " + "xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\" ID=\"" + id + "\" Version=\"2.0\" IssueInstant=\"" + instant + "\" Destination=\"" + this.options.entryPoint + "\">" + "<saml:Issuer xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\">" + this.options.issuer + "</saml:Issuer>" + "<saml:NameID Format=\"" + req.user.nameIDFormat + "\">" + req.user.nameID + "</saml:NameID>" + "</samlp:LogoutRequest>";
  return request;                                                                                                      // 108
};                                                                                                                     // 109
                                                                                                                       //
SAML.prototype.requestToUrl = function (request, operation, callback) {                                                // 111
  var _this = this;                                                                                                    // 112
                                                                                                                       //
  zlib.deflateRaw(request, function (err, buffer) {                                                                    // 113
    if (err) {                                                                                                         // 114
      return callback(err);                                                                                            // 115
    }                                                                                                                  // 116
                                                                                                                       //
    var base64 = buffer.toString("base64");                                                                            // 118
    var target = _this.options.entryPoint;                                                                             // 119
                                                                                                                       //
    if (operation === "logout") {                                                                                      // 121
      if (_this.options.logoutUrl) {                                                                                   // 122
        target = _this.options.logoutUrl;                                                                              // 123
      }                                                                                                                // 124
    }                                                                                                                  // 125
                                                                                                                       //
    if (target.indexOf("?") > 0) target += "&";else target += "?";                                                     // 127
    var samlRequest = {                                                                                                // 132
      SAMLRequest: base64                                                                                              // 133
    };                                                                                                                 // 132
                                                                                                                       //
    if (_this.options.privateCert) {                                                                                   // 136
      samlRequest.SigAlg = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";                                               // 137
      samlRequest.Signature = _this.signRequest(querystring.stringify(samlRequest));                                   // 138
    }                                                                                                                  // 139
                                                                                                                       //
    target += querystring.stringify(samlRequest);                                                                      // 141
    callback(null, target);                                                                                            // 143
  });                                                                                                                  // 144
};                                                                                                                     // 145
                                                                                                                       //
SAML.prototype.getAuthorizeUrl = function (req, callback) {                                                            // 147
  var request = this.generateAuthorizeRequest(req);                                                                    // 148
  this.requestToUrl(request, "authorize", callback);                                                                   // 150
};                                                                                                                     // 151
                                                                                                                       //
SAML.prototype.getLogoutUrl = function (req, callback) {                                                               // 153
  var request = this.generateLogoutRequest(req);                                                                       // 154
  this.requestToUrl(request, "logout", callback);                                                                      // 156
};                                                                                                                     // 157
                                                                                                                       //
SAML.prototype.certToPEM = function (cert) {                                                                           // 159
  cert = cert.match(/.{1,64}/g).join("\n");                                                                            // 160
  cert = "-----BEGIN CERTIFICATE-----\n" + cert;                                                                       // 161
  cert = cert + "\n-----END CERTIFICATE-----\n";                                                                       // 162
  return cert;                                                                                                         // 163
};                                                                                                                     // 164
                                                                                                                       //
SAML.prototype.validateSignature = function (xml, cert) {                                                              // 166
  var _this = this;                                                                                                    // 167
                                                                                                                       //
  var doc = new xmldom.DOMParser().parseFromString(xml);                                                               // 168
  var signature = xmlCrypto.xpath(doc, "//*[local-name(.)='Signature' and namespace-uri(.)='http://www.w3.org/2000/09/xmldsig#']")[0];
  var sig = new xmlCrypto.SignedXml();                                                                                 // 170
  sig.keyInfoProvider = {                                                                                              // 171
    getKeyInfo: function (key) {                                                                                       // 172
      return "<X509Data></X509Data>";                                                                                  // 173
    },                                                                                                                 // 174
    getKey: function (keyInfo) {                                                                                       // 176
      return _this.certToPEM(cert);                                                                                    // 177
    }                                                                                                                  // 178
  };                                                                                                                   // 171
  sig.loadSignature(signature.toString());                                                                             // 180
  return sig.checkSignature(xml);                                                                                      // 181
};                                                                                                                     // 182
                                                                                                                       //
SAML.prototype.getElement = function (parentElement, elementName) {                                                    // 184
  if (parentElement["saml:" + elementName]) {                                                                          // 185
    return parentElement["saml:" + elementName];                                                                       // 186
  } else if (parentElement["samlp:" + elementName]) {                                                                  // 187
    return parentElement["samlp:" + elementName];                                                                      // 188
  } else if (parentElement["saml2p:" + elementName]) {                                                                 // 189
    return parentElement["saml2p:" + elementName];                                                                     // 190
  } else if (parentElement["saml2:" + elementName]) {                                                                  // 191
    return parentElement["saml2:" + elementName];                                                                      // 192
  }                                                                                                                    // 193
                                                                                                                       //
  return parentElement[elementName];                                                                                   // 195
};                                                                                                                     // 196
                                                                                                                       //
SAML.prototype.parseLogoutRequest = function (xml, callback) {                                                         // 198
  var _this = this;                                                                                                    // 199
                                                                                                                       //
  var parser = new xml2js.Parser({                                                                                     // 200
    explicitRoot: true                                                                                                 // 200
  });                                                                                                                  // 200
  parser.parseString(xml, function (err, doc) {                                                                        // 201
    if (err) {                                                                                                         // 202
      callback(err);                                                                                                   // 203
      return;                                                                                                          // 204
    }                                                                                                                  // 205
                                                                                                                       //
    var request = _this.getElement(doc, "LogoutRequest");                                                              // 207
                                                                                                                       //
    var nameId = _this.getElement(request, "NameID")[0];                                                               // 208
                                                                                                                       //
    callback(null, nameId && nameId._);                                                                                // 209
  });                                                                                                                  // 210
};                                                                                                                     // 211
                                                                                                                       //
SAML.prototype.validateResponse = function (samlResponse, callback) {                                                  // 213
  var _this = this;                                                                                                    // 214
                                                                                                                       //
  var xml = new Buffer(samlResponse, "base64").toString("utf8");                                                       // 215
  var parser = new xml2js.Parser({                                                                                     // 216
    explicitRoot: true                                                                                                 // 216
  });                                                                                                                  // 216
  parser.parseString(xml, function (err, doc) {                                                                        // 217
    // Verify signature                                                                                                // 218
    if (_this.options.cert && !_this.validateSignature(xml, _this.options.cert)) {                                     // 219
      return callback(new Error("Invalid signature"), null, false, xml);                                               // 220
    }                                                                                                                  // 221
                                                                                                                       //
    var response = _this.getElement(doc, "Response");                                                                  // 223
                                                                                                                       //
    if (response) {                                                                                                    // 224
      var assertion = _this.getElement(response, "Assertion");                                                         // 225
                                                                                                                       //
      if (!assertion) {                                                                                                // 226
        return callback(new Error("Missing SAML assertion"), null, false, xml);                                        // 227
      }                                                                                                                // 228
                                                                                                                       //
      profile = {};                                                                                                    // 230
                                                                                                                       //
      if (response.$ && response.$.InResponseTo) {                                                                     // 232
        profile.inResponseToId = response.$.InResponseTo;                                                              // 233
      }                                                                                                                // 234
                                                                                                                       //
      if (response.$ && response.$.Destination) {                                                                      // 236
        if (!response.$.Destination.startsWith(process.env.ROOT_URL)) {                                                // 237
          return callback(new Error("SAML Response received with invalid Destination: " + response.$.Destination), null, false, xml);
        }                                                                                                              // 240
      }                                                                                                                // 241
                                                                                                                       //
      var issuer = _this.getElement(assertion[0], "Issuer");                                                           // 243
                                                                                                                       //
      if (issuer) {                                                                                                    // 244
        profile.issuer = issuer[0];                                                                                    // 245
      }                                                                                                                // 246
                                                                                                                       //
      var subject = _this.getElement(assertion[0], "Subject");                                                         // 248
                                                                                                                       //
      if (subject) {                                                                                                   // 249
        var nameID = _this.getElement(subject[0], "NameID");                                                           // 250
                                                                                                                       //
        if (nameID) {                                                                                                  // 251
          profile.nameID = nameID[0]._;                                                                                // 252
                                                                                                                       //
          if (nameID[0].$.Format) {                                                                                    // 254
            profile.nameIDFormat = nameID[0].$.Format;                                                                 // 255
                                                                                                                       //
            if (profile.nameIDFormat.toLowerCase().indexOf("transient") !== -1) {                                      // 256
              return callback(new Error("SAML returned a transient NameID. Sandstorm requires a persistent NameID. " + "Please check your IdP config."), null, false, xml);
            }                                                                                                          // 260
          }                                                                                                            // 261
        }                                                                                                              // 262
                                                                                                                       //
        var subjectConfirmation = _this.getElement(subject[0], "SubjectConfirmation");                                 // 264
                                                                                                                       //
        if (subjectConfirmation) {                                                                                     // 265
          var subjectConfirmationData = _this.getElement(subjectConfirmation[0], "SubjectConfirmationData")[0];        // 266
                                                                                                                       //
          if (subjectConfirmationData) {                                                                               // 268
            var recipient = subjectConfirmationData.$.Recipient;                                                       // 269
                                                                                                                       //
            if (recipient && !recipient.startsWith(process.env.ROOT_URL)) {                                            // 270
              return callback(new Error("SAML sent to wrong recipient"), null, false, xml);                            // 271
            }                                                                                                          // 272
                                                                                                                       //
            var nowMs = Date.now();                                                                                    // 274
            var notBefore = subjectConfirmationData.$.NotBefore;                                                       // 275
                                                                                                                       //
            if (notBefore && nowMs < Date.parse(notBefore)) {                                                          // 276
              return callback(new Error("SAML assertion was signed for the future."), null, false, xml);               // 277
            }                                                                                                          // 278
                                                                                                                       //
            var notOnOrAfter = subjectConfirmationData.$.NotOnOrAfter;                                                 // 280
                                                                                                                       //
            if (notOnOrAfter && nowMs >= Date.parse(notOnOrAfter)) {                                                   // 281
              return callback(new Error("SAML assertion was signed for the past."), null, false, xml);                 // 282
            }                                                                                                          // 283
          }                                                                                                            // 284
        }                                                                                                              // 285
      }                                                                                                                // 286
                                                                                                                       //
      var conditions = _this.getElement(assertion[0], "Conditions")[0];                                                // 288
                                                                                                                       //
      if (conditions) {                                                                                                // 289
        for (var key in meteorBabelHelpers.sanitizeForInObject(conditions)) {                                          // 290
          if (conditions.hasOwnProperty(key)) {                                                                        // 291
            var value = conditions[key];                                                                               // 292
                                                                                                                       //
            if (key === "$") {                                                                                         // 293
              var _nowMs = Date.now();                                                                                 // 294
                                                                                                                       //
              var _notBefore = value.NotBefore;                                                                        // 295
                                                                                                                       //
              if (_notBefore && _nowMs < Date.parse(_notBefore)) {                                                     // 296
                return callback(new Error("SAML condition NotBefore is in the future."), null, false, xml);            // 297
              }                                                                                                        // 299
                                                                                                                       //
              var _notOnOrAfter = value.NotOnOrAfter;                                                                  // 301
                                                                                                                       //
              if (_notOnOrAfter && _nowMs >= Date.parse(_notOnOrAfter)) {                                              // 302
                return callback(new Error("SAML condition notOnOrAfter is in the past."), null, false, xml);           // 303
              }                                                                                                        // 305
            } else if (key.endsWith("AudienceRestriction") || key.endsWith("OneTimeUse") || key.endsWith("ProxyRestriction")) {
              continue; // Do nothing.                                                                                 // 309
              // We already check both Destination and SubjectConfirmation.Recipient, so                               // 311
              // the Audience constraint isn't as important, and is tricky to handle                                   // 312
              // correctly.                                                                                            // 313
              // OneTimeUse is actually very tricky to get right, and requires a cache of                              // 314
              // previously seen assertions (or at least their hashes). In Sandstorm's case,                           // 315
              // we would probably need to stuff it in Mongo to make it robust across multiple                         // 316
              // front-ends. It is a very rarely used constraint anyways.                                              // 317
              // ProxyRestriction constraints only govern the future use of the assertion by the                       // 318
              // SP. We don't fall under this constraint so it's meaningless to us. As per the                         // 319
              // spec, it is always considered valid.                                                                  // 320
            } else {                                                                                                   // 321
              return callback(new Error("Unrecognized SAML constraint: " + key), null, false, xml);                    // 322
            }                                                                                                          // 323
          }                                                                                                            // 324
        }                                                                                                              // 325
      }                                                                                                                // 326
                                                                                                                       //
      var attributeStatement = _this.getElement(assertion[0], "AttributeStatement");                                   // 328
                                                                                                                       //
      if (attributeStatement) {                                                                                        // 329
        var attributes = _this.getElement(attributeStatement[0], "Attribute");                                         // 330
                                                                                                                       //
        if (attributes) {                                                                                              // 332
          attributes.forEach(function (attribute) {                                                                    // 333
            var value = _this.getElement(attribute, "AttributeValue");                                                 // 334
                                                                                                                       //
            if (typeof value[0] === "string") {                                                                        // 335
              profile[attribute.$.Name] = value[0];                                                                    // 336
            } else {                                                                                                   // 337
              profile[attribute.$.Name] = value[0]._;                                                                  // 338
            }                                                                                                          // 339
          });                                                                                                          // 340
        }                                                                                                              // 341
                                                                                                                       //
        if (!profile.mail && profile["urn:oid:0.9.2342.19200300.100.1.3"]) {                                           // 343
          // See http://www.incommonfederation.org/attributesummary.html for definition of attribute OIDs              // 344
          profile.mail = profile["urn:oid:0.9.2342.19200300.100.1.3"];                                                 // 345
        }                                                                                                              // 346
                                                                                                                       //
        if (!profile.email && profile.mail) {                                                                          // 348
          profile.email = profile.mail;                                                                                // 349
        }                                                                                                              // 350
                                                                                                                       //
        var microsoftEmail = profile["http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress"];            // 352
                                                                                                                       //
        if (!profile.email && microsoftEmail) {                                                                        // 353
          profile.email = microsoftEmail;                                                                              // 354
        }                                                                                                              // 355
                                                                                                                       //
        var azureAdEmail = profile["http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name"];                      // 357
                                                                                                                       //
        if (!profile.email && azureAdEmail) {                                                                          // 358
          profile.email = azureAdEmail;                                                                                // 359
        }                                                                                                              // 360
                                                                                                                       //
        var microsoftDisplayName = profile["http://schemas.microsoft.com/identity/claims/displayname"];                // 362
                                                                                                                       //
        if (!profile.displayName && microsoftDisplayName) {                                                            // 363
          profile.displayName = microsoftDisplayName;                                                                  // 364
        }                                                                                                              // 365
      }                                                                                                                // 366
                                                                                                                       //
      if (!profile.email && profile.nameID && profile.nameIDFormat && profile.nameIDFormat.indexOf("emailAddress") >= 0) {
        profile.email = profile.nameID;                                                                                // 369
      }                                                                                                                // 370
                                                                                                                       //
      callback(null, profile, false, xml);                                                                             // 372
    } else {                                                                                                           // 373
      var logoutResponse = _this.getElement(doc, "LogoutResponse");                                                    // 374
                                                                                                                       //
      if (logoutResponse) {                                                                                            // 376
        callback(null, null, true, xml);                                                                               // 377
      } else {                                                                                                         // 378
        return callback(new Error("Unknown SAML response message"), null, false, xml);                                 // 379
      }                                                                                                                // 380
    }                                                                                                                  // 382
  });                                                                                                                  // 384
};                                                                                                                     // 385
                                                                                                                       //
SAML.prototype.generateServiceProviderMetadata = function () {                                                         // 387
  var entityId = this.options.issuer;                                                                                  // 388
  var metadata = {                                                                                                     // 389
    "EntityDescriptor": {                                                                                              // 390
      "@xmlns": "urn:oasis:names:tc:SAML:2.0:metadata",                                                                // 391
      "@xmlns:ds": "http://www.w3.org/2000/09/xmldsig#",                                                               // 392
      "@entityID": entityId,                                                                                           // 393
      "@ID": entityId.replace(/\W/g, "_"),                                                                             // 394
      "SPSSODescriptor": {                                                                                             // 395
        "@protocolSupportEnumeration": "urn:oasis:names:tc:SAML:2.0:protocol"                                          // 396
      }                                                                                                                // 395
    }                                                                                                                  // 390
  };                                                                                                                   // 389
  var decryptionCert = this.options.localCertPublic; // TODO(someday); support local certs                             // 401
                                                                                                                       //
  if (decryptionCert) {                                                                                                // 402
    decryptionCert = decryptionCert.replace(/-+BEGIN CERTIFICATE-+\r?\n?/, "");                                        // 403
    decryptionCert = decryptionCert.replace(/-+END CERTIFICATE-+\r?\n?/, "");                                          // 404
    decryptionCert = decryptionCert.replace(/\r\n/g, "\n");                                                            // 405
    metadata.EntityDescriptor.SPSSODescriptor.KeyDescriptor = {                                                        // 407
      "ds:KeyInfo": {                                                                                                  // 408
        "ds:X509Data": {                                                                                               // 409
          "ds:X509Certificate": {                                                                                      // 410
            "#text": decryptionCert                                                                                    // 411
          }                                                                                                            // 410
        }                                                                                                              // 409
      },                                                                                                               // 408
      "EncryptionMethod": [// this should be the set that the xmlenc library supports                                  // 415
      {                                                                                                                // 417
        "@Algorithm": "http://www.w3.org/2001/04/xmlenc#aes256-cbc"                                                    // 417
      }, {                                                                                                             // 417
        "@Algorithm": "http://www.w3.org/2001/04/xmlenc#aes128-cbc"                                                    // 418
      }, {                                                                                                             // 418
        "@Algorithm": "http://www.w3.org/2001/04/xmlenc#tripledes-cbc"                                                 // 419
      }]                                                                                                               // 419
    };                                                                                                                 // 407
  }                                                                                                                    // 422
                                                                                                                       //
  metadata.EntityDescriptor.SPSSODescriptor.SingleLogoutService = {                                                    // 424
    "@Binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect",                                                  // 425
    "@Location": Meteor.absoluteUrl("saml/logout/default")                                                             // 426
  };                                                                                                                   // 424
  metadata.EntityDescriptor.SPSSODescriptor.NameIDFormat = this.options.identifierFormat;                              // 429
  metadata.EntityDescriptor.SPSSODescriptor.AssertionConsumerService = {                                               // 430
    "@index": "1",                                                                                                     // 431
    "@isDefault": "true",                                                                                              // 432
    "@Binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST",                                                      // 433
    "@Location": Meteor.absoluteUrl("_saml/validate/default")                                                          // 434
  };                                                                                                                   // 430
  return xmlbuilder.create(metadata).end({                                                                             // 437
    pretty: true,                                                                                                      // 437
    indent: "  ",                                                                                                      // 437
    newline: "\n"                                                                                                      // 437
  });                                                                                                                  // 437
};                                                                                                                     // 438
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"async-helpers.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/async-helpers.js                                                                                     //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  inMeteor: function () {                                                                                              // 1
    return inMeteor;                                                                                                   // 1
  },                                                                                                                   // 1
  promiseToFuture: function () {                                                                                       // 1
    return promiseToFuture;                                                                                            // 1
  },                                                                                                                   // 1
  waitPromise: function () {                                                                                           // 1
    return waitPromise;                                                                                                // 1
  },                                                                                                                   // 1
  onInMeteor: function () {                                                                                            // 1
    return onInMeteor;                                                                                                 // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 3
                                                                                                                       //
var inMeteorListener = undefined;                                                                                      // 5
                                                                                                                       //
var onInMeteor = function (callback) {                                                                                 // 6
  inMeteorListener = callback;                                                                                         // 7
}; // Meteor context <-> Async Node.js context adapters                                                                // 8
                                                                                                                       //
                                                                                                                       //
var inMeteorInternal = Meteor.bindEnvironment(function (callback) {                                                    // 11
  callback();                                                                                                          // 12
});                                                                                                                    // 13
                                                                                                                       //
var inMeteor = function (callback) {                                                                                   // 15
  if (inMeteorListener) {                                                                                              // 16
    inMeteorListener();                                                                                                // 17
  } // Calls the callback in a Meteor context.  Returns a Promise for its result.                                      // 18
                                                                                                                       //
                                                                                                                       //
  return new Promise(function (resolve, reject) {                                                                      // 21
    inMeteorInternal(function () {                                                                                     // 22
      try {                                                                                                            // 23
        resolve(callback());                                                                                           // 24
      } catch (err) {                                                                                                  // 25
        reject(err);                                                                                                   // 26
      }                                                                                                                // 27
    });                                                                                                                // 28
  });                                                                                                                  // 29
};                                                                                                                     // 30
                                                                                                                       //
var promiseToFuture = function (promise) {                                                                             // 32
  var result = new Future();                                                                                           // 33
  promise.then(result.return.bind(result), result.throw.bind(result));                                                 // 34
  return result;                                                                                                       // 35
};                                                                                                                     // 36
                                                                                                                       //
var waitPromise = function (promise) {                                                                                 // 38
  return promiseToFuture(promise).wait();                                                                              // 39
};                                                                                                                     // 40
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"auth.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/auth.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  checkAuth: function () {                                                                                             // 1
    return checkAuth;                                                                                                  // 1
  },                                                                                                                   // 1
  clearAdminToken: function () {                                                                                       // 1
    return clearAdminToken;                                                                                            // 1
  },                                                                                                                   // 1
  tokenIsValid: function () {                                                                                          // 1
    return tokenIsValid;                                                                                               // 1
  },                                                                                                                   // 1
  tokenIsSetupSession: function () {                                                                                   // 1
    return tokenIsSetupSession;                                                                                        // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var Match = void 0,                                                                                                    // 1
    check = void 0;                                                                                                    // 1
module.watch(require("meteor/check"), {                                                                                // 1
  Match: function (v) {                                                                                                // 1
    Match = v;                                                                                                         // 1
  },                                                                                                                   // 1
  check: function (v) {                                                                                                // 1
    check = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Crypto = void 0;                                                                                                   // 1
module.watch(require("crypto"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    Crypto = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var Fs = void 0;                                                                                                       // 1
module.watch(require("fs"), {                                                                                          // 1
  "default": function (v) {                                                                                            // 1
    Fs = v;                                                                                                            // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var SANDSTORM_VARDIR = void 0;                                                                                         // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  SANDSTORM_VARDIR: function (v) {                                                                                     // 1
    SANDSTORM_VARDIR = v;                                                                                              // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
// TODO(cleanup): globalDb is still an unbound global, but extracting it is Hard.                                      // 6
var ADMIN_TOKEN_EXPIRATION_TIME = 60 * 60 * 1000;                                                                      // 8
var SANDSTORM_ADMIN_TOKEN = SANDSTORM_VARDIR + "/adminToken";                                                          // 9
                                                                                                                       //
function clearAdminToken(token) {                                                                                      // 11
  if (tokenIsSetupSession(token)) {                                                                                    // 12
    var hash = Crypto.createHash("sha256").update(token).digest("base64");                                             // 13
    globalDb.collections.setupSession.remove({                                                                         // 14
      _id: "current-session",                                                                                          // 15
      hashedSessionId: hash                                                                                            // 16
    });                                                                                                                // 14
  }                                                                                                                    // 18
                                                                                                                       //
  if (tokenIsValid(token)) {                                                                                           // 20
    Fs.unlinkSync(SANDSTORM_ADMIN_TOKEN);                                                                              // 21
    console.log("Admin token deleted.");                                                                               // 22
  }                                                                                                                    // 23
}                                                                                                                      // 24
                                                                                                                       //
function tokenIsValid(token) {                                                                                         // 26
  if (token && Fs.existsSync(SANDSTORM_ADMIN_TOKEN)) {                                                                 // 27
    var stats = Fs.statSync(SANDSTORM_ADMIN_TOKEN);                                                                    // 28
    var expireTime = new Date(Date.now() - ADMIN_TOKEN_EXPIRATION_TIME);                                               // 29
                                                                                                                       //
    if (stats.mtime < expireTime) {                                                                                    // 30
      return false;                                                                                                    // 31
    } else {                                                                                                           // 32
      return Fs.readFileSync(SANDSTORM_ADMIN_TOKEN, {                                                                  // 33
        encoding: "utf8"                                                                                               // 33
      }) === token;                                                                                                    // 33
    }                                                                                                                  // 34
  } else {                                                                                                             // 35
    return false;                                                                                                      // 36
  }                                                                                                                    // 37
}                                                                                                                      // 38
                                                                                                                       //
function tokenIsSetupSession(token) {                                                                                  // 40
  if (token) {                                                                                                         // 41
    var setupSession = globalDb.collections.setupSession.findOne({                                                     // 42
      _id: "current-session"                                                                                           // 42
    });                                                                                                                // 42
                                                                                                                       //
    if (setupSession) {                                                                                                // 43
      var hash = Crypto.createHash("sha256").update(token).digest("base64");                                           // 44
      var now = new Date();                                                                                            // 45
      var sessionLifetime = 24 * 60 * 60 * 1000; // length of setup session validity, in milliseconds: 1 day           // 46
                                                                                                                       //
      if (setupSession.hashedSessionId === hash && now - setupSession.creationDate < sessionLifetime) {                // 47
        return true;                                                                                                   // 48
      }                                                                                                                // 49
    }                                                                                                                  // 50
  }                                                                                                                    // 51
                                                                                                                       //
  return false;                                                                                                        // 53
}                                                                                                                      // 54
                                                                                                                       //
function checkAuth(token) {                                                                                            // 56
  check(token, Match.OneOf(undefined, null, String));                                                                  // 57
                                                                                                                       //
  if (!isAdmin() && !tokenIsValid(token) && !tokenIsSetupSession(token)) {                                             // 58
    throw new Meteor.Error(403, "User must be admin or provide a valid token");                                        // 59
  }                                                                                                                    // 60
}                                                                                                                      // 61
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"backend.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/backend.js                                                                                           //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
module.export({                                                                                                        // 1
  SandstormBackend: function () {                                                                                      // 1
    return SandstormBackend;                                                                                           // 1
  },                                                                                                                   // 1
  shouldRestartGrain: function () {                                                                                    // 1
    return shouldRestartGrain;                                                                                         // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var SANDSTORM_ALTHOME = void 0;                                                                                        // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  SANDSTORM_ALTHOME: function (v) {                                                                                    // 1
    SANDSTORM_ALTHOME = v;                                                                                             // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var inMeteor = void 0,                                                                                                 // 1
    promiseToFuture = void 0,                                                                                          // 1
    waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  },                                                                                                                   // 1
  promiseToFuture: function (v) {                                                                                      // 1
    promiseToFuture = v;                                                                                               // 1
  },                                                                                                                   // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Backend = Capnp.importSystem("sandstorm/backend.capnp").Backend;                                                   // 20
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 21
                                                                                                                       //
var storageUsageUnimplemented = false;                                                                                 // 23
                                                                                                                       //
function generateSessionId(grainId, userId, packageSalt, clientSalt) {                                                 // 25
  var sessionParts = [grainId, clientSalt];                                                                            // 26
                                                                                                                       //
  if (userId) {                                                                                                        // 27
    sessionParts.push(userId);                                                                                         // 28
  }                                                                                                                    // 29
                                                                                                                       //
  if (packageSalt) {                                                                                                   // 31
    sessionParts.push(packageSalt);                                                                                    // 32
  }                                                                                                                    // 33
                                                                                                                       //
  var sessionInput = sessionParts.join(":");                                                                           // 35
  return Crypto.createHash("sha256").update(sessionInput).digest("hex");                                               // 36
}                                                                                                                      // 37
                                                                                                                       //
var shouldRestartGrain = function (error, retryCount) {                                                                // 39
  // Given an error thrown by an RPC call to a grain, return whether or not it makes sense to try                      // 40
  // to restart the grain and retry. `retryCount` is the number of times that the request has                          // 41
  // already gone through this cycle (should be zero for the first call).                                              // 42
  return error.kjType === "disconnected" && retryCount < 1;                                                            // 43
};                                                                                                                     // 44
                                                                                                                       //
var SandstormBackend = function () {                                                                                   //
  function SandstormBackend(db, backendCap) {                                                                          // 47
    (0, _classCallCheck3.default)(this, SandstormBackend);                                                             // 47
    this._db = db;                                                                                                     // 48
    this._backendCap = backendCap;                                                                                     // 49
  }                                                                                                                    // 50
                                                                                                                       //
  SandstormBackend.prototype.cap = function () {                                                                       //
    function cap() {                                                                                                   //
      return this._backendCap;                                                                                         // 53
    }                                                                                                                  // 54
                                                                                                                       //
    return cap;                                                                                                        //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.deleteUser = function () {                                                                //
    function deleteUser(userId) {                                                                                      //
      return waitPromise(this._backendCap.deleteUser(userId));                                                         // 57
    }                                                                                                                  // 58
                                                                                                                       //
    return deleteUser;                                                                                                 //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.shutdownGrain = function () {                                                             //
    function shutdownGrain(grainId, ownerId, keepSessions) {                                                           //
      if (!keepSessions) {                                                                                             // 61
        Sessions.remove({                                                                                              // 62
          grainId: grainId                                                                                             // 62
        });                                                                                                            // 62
      }                                                                                                                // 63
                                                                                                                       //
      var grain = this._backendCap.getGrain(ownerId, grainId).supervisor;                                              // 65
                                                                                                                       //
      return grain.shutdown().then(function () {                                                                       // 66
        grain.close();                                                                                                 // 67
        throw new Error("expected shutdown() to throw disconnected");                                                  // 68
      }, function (err) {                                                                                              // 69
        grain.close();                                                                                                 // 71
                                                                                                                       //
        if (err.kjType !== "disconnected") {                                                                           // 72
          throw err;                                                                                                   // 73
        }                                                                                                              // 74
      });                                                                                                              // 75
    }                                                                                                                  // 76
                                                                                                                       //
    return shutdownGrain;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.deleteGrain = function () {                                                               //
    function deleteGrain(grainId, ownerId) {                                                                           //
      // We leave it up to the caller if they want to actually wait, but some don't so we report                       // 79
      // exceptions.                                                                                                   // 80
      return this._backendCap.deleteGrain(ownerId, grainId).catch(function (err) {                                     // 81
        console.error("problem deleting grain " + grainId + ":", err.message);                                         // 82
        throw err;                                                                                                     // 83
      });                                                                                                              // 84
    }                                                                                                                  // 85
                                                                                                                       //
    return deleteGrain;                                                                                                //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.maybeRetryUseGrain = function () {                                                        //
    function maybeRetryUseGrain(grainId, cb, retryCount, err) {                                                        //
      var _this = this;                                                                                                // 87
                                                                                                                       //
      if (shouldRestartGrain(err, retryCount)) {                                                                       // 88
        return inMeteor(function () {                                                                                  // 89
          return cb(_this.continueGrain(grainId).supervisor).catch(_this.maybeRetryUseGrain.bind(_this, grainId, cb, retryCount + 1));
        });                                                                                                            // 92
      } else {                                                                                                         // 93
        throw err;                                                                                                     // 94
      }                                                                                                                // 95
    }                                                                                                                  // 96
                                                                                                                       //
    return maybeRetryUseGrain;                                                                                         //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.useGrain = function () {                                                                  //
    function useGrain(grainId, cb) {                                                                                   //
      var _this2 = this;                                                                                               // 98
                                                                                                                       //
      // This will open a grain for you, handling restarts if needed, and call the passed function with                // 99
      // the supervisor capability as the only parameter. The callback must return a promise that used                 // 100
      // the supervisor, so that we can check if a disconnect error occurred, and retry if possible.                   // 101
      // This function returns the same promise that your callback returns.                                            // 102
      //                                                                                                               // 103
      // This function is NOT expected to be run in a meteor context.                                                  // 104
      return inMeteor(function () {                                                                                    // 105
        return cb(_this2.continueGrain(grainId).supervisor).catch(_this2.maybeRetryUseGrain.bind(_this2, grainId, cb, 0));
      });                                                                                                              // 108
    }                                                                                                                  // 109
                                                                                                                       //
    return useGrain;                                                                                                   //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.continueGrain = function () {                                                             //
    function continueGrain(grainId) {                                                                                  //
      var grain = Grains.findOne(grainId);                                                                             // 112
                                                                                                                       //
      if (!grain) {                                                                                                    // 113
        throw new Meteor.Error(404, "Grain Not Found", "Grain ID: " + grainId);                                        // 114
      }                                                                                                                // 115
                                                                                                                       //
      if (grain.trashed) {                                                                                             // 117
        throw new Meteor.Error(403, "Grain is in the trash bin", "Grain ID: " + grainId);                              // 118
      } // If a DevPackage with the same app ID is currently active, we let it override the installed                  // 119
      // package, so that the grain runs using the dev app.                                                            // 122
                                                                                                                       //
                                                                                                                       //
      var devPackage = DevPackages.findOne({                                                                           // 123
        appId: grain.appId                                                                                             // 123
      });                                                                                                              // 123
      var isDev = void 0;                                                                                              // 124
      var mountProc = void 0;                                                                                          // 125
      var pkg = void 0;                                                                                                // 126
                                                                                                                       //
      if (devPackage) {                                                                                                // 127
        isDev = true;                                                                                                  // 128
        pkg = devPackage;                                                                                              // 129
        mountProc = pkg.mountProc;                                                                                     // 130
      } else {                                                                                                         // 131
        pkg = Packages.findOne(grain.packageId);                                                                       // 132
                                                                                                                       //
        if (!pkg) {                                                                                                    // 133
          throw new Meteor.Error(500, "Grain's package not installed", "Package ID: " + grain.packageId);              // 134
        }                                                                                                              // 136
      }                                                                                                                // 137
                                                                                                                       //
      var manifest = pkg.manifest;                                                                                     // 139
      var packageId = pkg._id;                                                                                         // 140
                                                                                                                       //
      if (!("continueCommand" in manifest)) {                                                                          // 142
        throw new Meteor.Error(500, "Package manifest defines no continueCommand.", "Package ID: " + packageId);       // 143
      }                                                                                                                // 145
                                                                                                                       //
      var result = this.startGrainInternal(packageId, grainId, grain.userId, manifest.continueCommand, false, isDev, mountProc);
      result.packageSalt = isDev ? pkg._id : grain.packageSalt;                                                        // 149
      return result;                                                                                                   // 150
    }                                                                                                                  // 151
                                                                                                                       //
    return continueGrain;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.startGrainInternal = function () {                                                        //
    function startGrainInternal(packageId, grainId, ownerId, command, isNew, isDev, mountProc) {                       //
      // Starts the grain supervisor.  Must be executed in a Meteor context.  Blocks until grain is                    // 154
      // started. Returns a promise for an object containing two fields: `owner` (the ID of the owning                 // 155
      // user) and `supervisor` (the supervisor capability).                                                           // 156
      if (isUserExcessivelyOverQuota(Meteor.users.findOne(ownerId))) {                                                 // 158
        throw new Meteor.Error(402, "Cannot start grain because owner's storage is exhausted.\n" + "Please ask them to upgrade.");
      } // Ugly: Stay backwards-compatible with old manifests that had "executablePath" and "args" rather              // 162
      //   than just "argv".                                                                                           // 165
                                                                                                                       //
                                                                                                                       //
      if ("args" in command) {                                                                                         // 166
        if (!("argv" in command)) {                                                                                    // 167
          command.argv = command.args;                                                                                 // 168
        }                                                                                                              // 169
                                                                                                                       //
        delete command.args;                                                                                           // 171
      }                                                                                                                // 172
                                                                                                                       //
      if ("executablePath" in command) {                                                                               // 174
        if (!("deprecatedExecutablePath" in command)) {                                                                // 175
          command.deprecatedExecutablePath = command.executablePath;                                                   // 176
        }                                                                                                              // 177
                                                                                                                       //
        delete command.executablePath;                                                                                 // 179
      }                                                                                                                // 180
                                                                                                                       //
      return this._backendCap.startGrain(ownerId, grainId, packageId, command, isNew, isDev, mountProc);               // 182
    }                                                                                                                  // 183
                                                                                                                       //
    return startGrainInternal;                                                                                         //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.updateLastActive = function () {                                                          //
    function updateLastActive(grainId, userId, obsolete) {                                                             //
      // Update the lastActive date on the grain, any relevant API tokens, and the user,                               // 186
      // and also update the user's storage usage.                                                                     // 187
      var storagePromise = undefined;                                                                                  // 189
      var ownerId = undefined;                                                                                         // 190
                                                                                                                       //
      if (this._db.isQuotaEnabled() && !storageUsageUnimplemented) {                                                   // 191
        ownerId = Grains.findOne(grainId).userId;                                                                      // 192
        storagePromise = this._backendCap.getUserStorageUsage(ownerId);                                                // 193
      }                                                                                                                // 194
                                                                                                                       //
      var now = new Date();                                                                                            // 196
                                                                                                                       //
      if (Grains.update(grainId, {                                                                                     // 197
        $set: {                                                                                                        // 197
          lastUsed: now                                                                                                // 197
        }                                                                                                              // 197
      }) === 0) {                                                                                                      // 197
        // Grain must have been deleted. Ignore.                                                                       // 198
        return;                                                                                                        // 199
      }                                                                                                                // 200
                                                                                                                       //
      if (userId) {                                                                                                    // 202
        Meteor.users.update(userId, {                                                                                  // 203
          $set: {                                                                                                      // 203
            lastActive: now                                                                                            // 203
          }                                                                                                            // 203
        }); // Update any API tokens that match this user/grain pairing as well                                        // 203
                                                                                                                       //
        ApiTokens.update({                                                                                             // 206
          grainId: grainId,                                                                                            // 206
          "owner.user.accountId": userId                                                                               // 206
        }, {                                                                                                           // 206
          $set: {                                                                                                      // 207
            lastUsed: now                                                                                              // 207
          }                                                                                                            // 207
        }, {                                                                                                           // 207
          multi: true                                                                                                  // 208
        });                                                                                                            // 208
      }                                                                                                                // 209
                                                                                                                       //
      if (storagePromise) {                                                                                            // 211
        try {                                                                                                          // 212
          var size = parseInt(waitPromise(storagePromise).size);                                                       // 213
          Meteor.users.update(ownerId, {                                                                               // 214
            $set: {                                                                                                    // 214
              storageUsage: size                                                                                       // 214
            }                                                                                                          // 214
          }); // TODO(security): Consider actively killing grains if the user is excessively over quota?               // 214
          //   Otherwise a constantly-active grain could consume arbitrary space without being stopped.                // 216
        } catch (err) {                                                                                                // 217
          if (err.kjType === "unimplemented") {                                                                        // 218
            storageUsageUnimplemented = true;                                                                          // 219
          } else {                                                                                                     // 220
            console.error("error getting user storage usage:", err.stack);                                             // 221
          }                                                                                                            // 222
        }                                                                                                              // 223
      }                                                                                                                // 224
    }                                                                                                                  // 225
                                                                                                                       //
    return updateLastActive;                                                                                           //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormBackend.prototype.openSessionInternal = function () {                                                       //
    function openSessionInternal(grain, userId, revealIdentity, title, apiToken, cachedSalt, sessionFields) {          //
      var grainId = grain._id; // Start the grain if it is not running. This is an optimization: if we didn't start it here,
      // it would start on the first request to the session host, but we'd like to get started before                  // 231
      // the round trip.                                                                                               // 232
                                                                                                                       //
      var _continueGrain = this.continueGrain(grainId),                                                                // 227
          supervisor = _continueGrain.supervisor,                                                                      // 227
          packageSalt = _continueGrain.packageSalt;                                                                    // 227
                                                                                                                       //
      this.updateLastActive(grainId, userId);                                                                          // 235
                                                                                                                       //
      var identityId = userId && revealIdentity && this._db.getOrGenerateIdentityId(userId, grain);                    // 237
                                                                                                                       //
      cachedSalt = cachedSalt || Random.id(22);                                                                        // 239
      var sessionId = generateSessionId(grainId, userId, packageSalt, cachedSalt);                                     // 240
      var session = Sessions.findOne({                                                                                 // 241
        _id: sessionId                                                                                                 // 241
      });                                                                                                              // 241
                                                                                                                       //
      if (session) {                                                                                                   // 242
        // TODO(someday): also do some more checks for anonymous sessions (sessions without a userId).                 // 243
        if (session.userId && session.userId !== userId || session.identityId && session.identityId !== identityId || session.grainId !== grainId) {
          var e = new Meteor.Error(500, "Duplicate SessionId");                                                        // 247
          console.error(e);                                                                                            // 248
          throw e;                                                                                                     // 249
        } else {                                                                                                       // 250
          return {                                                                                                     // 251
            supervisor: supervisor,                                                                                    // 252
            methodResult: {                                                                                            // 253
              sessionId: session._id,                                                                                  // 254
              title: title,                                                                                            // 255
              grainId: grainId,                                                                                        // 256
              hostId: session.hostId,                                                                                  // 257
              tabId: session.tabId,                                                                                    // 258
              salt: cachedSalt                                                                                         // 259
            }                                                                                                          // 253
          };                                                                                                           // 251
        }                                                                                                              // 262
      } // TODO(someday): Allow caller to specify the parent session from which to inherit the tab ID, or              // 263
      //   something.                                                                                                  // 266
                                                                                                                       //
                                                                                                                       //
      session = {                                                                                                      // 268
        _id: sessionId,                                                                                                // 269
        grainId: grainId,                                                                                              // 270
        hostId: Crypto.createHash("sha256").update(sessionId).digest("hex").slice(0, 32),                              // 271
        tabId: Crypto.createHash("sha256").update("tab:").update(sessionId).digest("hex").slice(0, 32),                // 272
        timestamp: new Date().getTime(),                                                                               // 273
        hasLoaded: false                                                                                               // 274
      };                                                                                                               // 268
                                                                                                                       //
      if (userId) {                                                                                                    // 277
        session.userId = userId;                                                                                       // 278
      }                                                                                                                // 279
                                                                                                                       //
      if (identityId) {                                                                                                // 281
        session.identityId = identityId;                                                                               // 282
      }                                                                                                                // 283
                                                                                                                       //
      if (apiToken) {                                                                                                  // 285
        session.hashedToken = apiToken._id;                                                                            // 286
      }                                                                                                                // 287
                                                                                                                       //
      if (sessionFields) {                                                                                             // 289
        _.extend(session, sessionFields);                                                                              // 290
      }                                                                                                                // 291
                                                                                                                       //
      Sessions.insert(session);                                                                                        // 293
      return {                                                                                                         // 295
        supervisor: supervisor,                                                                                        // 296
        methodResult: {                                                                                                // 297
          sessionId: session._id,                                                                                      // 298
          title: title,                                                                                                // 299
          grainId: grainId,                                                                                            // 300
          hostId: session.hostId,                                                                                      // 301
          tabId: session.tabId,                                                                                        // 302
          salt: cachedSalt,                                                                                            // 303
          identityId: identityId                                                                                       // 304
        }                                                                                                              // 297
      };                                                                                                               // 295
    }                                                                                                                  // 307
                                                                                                                       //
    return openSessionInternal;                                                                                        //
  }();                                                                                                                 //
                                                                                                                       //
  return SandstormBackend;                                                                                             //
}();                                                                                                                   //
                                                                                                                       //
;                                                                                                                      // 308
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"constants.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/constants.js                                                                                         //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  SANDSTORM_ALTHOME: function () {                                                                                     // 1
    return SANDSTORM_ALTHOME;                                                                                          // 1
  },                                                                                                                   // 1
  SANDSTORM_LOGDIR: function () {                                                                                      // 1
    return SANDSTORM_LOGDIR;                                                                                           // 1
  },                                                                                                                   // 1
  SANDSTORM_VARDIR: function () {                                                                                      // 1
    return SANDSTORM_VARDIR;                                                                                           // 1
  },                                                                                                                   // 1
  staticAssetHost: function () {                                                                                       // 1
    return staticAssetHost;                                                                                            // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var Url = void 0;                                                                                                      // 1
module.watch(require("url"), {                                                                                         // 1
  "default": function (v) {                                                                                            // 1
    Url = v;                                                                                                           // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var SANDSTORM_ALTHOME = Meteor.settings && Meteor.settings.home;                                                       // 22
var SANDSTORM_LOGDIR = (SANDSTORM_ALTHOME || "") + "/var/log";                                                         // 23
var SANDSTORM_VARDIR = (SANDSTORM_ALTHOME || "") + "/var/sandstorm"; // TODO: dedupe this and the equivalent copy in sandstorm-db
                                                                                                                       //
var wildcardHost = Meteor.settings.public.wildcardHost.toLowerCase().split("*");                                       // 27
var staticAssetHost = Url.parse(process.env.ROOT_URL).protocol + "//" + wildcardHost[0] + "static" + wildcardHost[1];  // 29
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"desktop-notifications.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/desktop-notifications.js                                                                             //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  createAppActivityDesktopNotification: function () {                                                                  // 1
    return createAppActivityDesktopNotification;                                                                       // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var Match = void 0,                                                                                                    // 1
    check = void 0;                                                                                                    // 1
module.watch(require("meteor/check"), {                                                                                // 1
  Match: function (v) {                                                                                                // 1
    Match = v;                                                                                                         // 1
  },                                                                                                                   // 1
  check: function (v) {                                                                                                // 1
    check = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var createAppActivityDesktopNotification = function (options) {                                                        // 19
  check(options, {                                                                                                     // 20
    userId: String,                                                                                                    // 21
    notificationId: String,                                                                                            // 22
    appActivity: {                                                                                                     // 23
      user: Match.OneOf(undefined, {                                                                                   // 24
        accountId: String,                                                                                             // 27
        name: String,                                                                                                  // 28
        avatarUrl: String                                                                                              // 29
      }, {                                                                                                             // 26
        anonymous: true                                                                                                // 31
      }),                                                                                                              // 31
      grainId: String,                                                                                                 // 33
      path: String,                                                                                                    // 34
      body: Match.ObjectIncluding({                                                                                    // 35
        defaultText: String                                                                                            // 36
      }),                                                                                                              // 35
      actionText: Match.ObjectIncluding({                                                                              // 38
        defaultText: String                                                                                            // 39
      })                                                                                                               // 38
    }                                                                                                                  // 23
  });                                                                                                                  // 20
  globalDb.collections.desktopNotifications.insert({                                                                   // 44
    userId: options.userId,                                                                                            // 45
    notificationId: options.notificationId,                                                                            // 46
    creationDate: new Date(),                                                                                          // 47
    appActivity: options.appActivity                                                                                   // 48
  });                                                                                                                  // 44
};                                                                                                                     // 50
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"email.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/email.js                                                                                             //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _typeof2 = require("babel-runtime/helpers/typeof");                                                                //
                                                                                                                       //
var _typeof3 = _interopRequireDefault(_typeof2);                                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
module.export({                                                                                                        // 1
  send: function () {                                                                                                  // 1
    return send;                                                                                                       // 1
  },                                                                                                                   // 1
  rawSend: function () {                                                                                               // 1
    return rawSend;                                                                                                    // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var nodemailer = void 0;                                                                                               // 1
module.watch(require("nodemailer"), {                                                                                  // 1
  "default": function (v) {                                                                                            // 1
    nodemailer = v;                                                                                                    // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var smtpPool = void 0;                                                                                                 // 1
module.watch(require("nodemailer-smtp-pool"), {                                                                        // 1
  "default": function (v) {                                                                                            // 1
    smtpPool = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 5
                                                                                                                       //
var getSmtpConfig = function () {                                                                                      // 7
  var config = Settings.findOne({                                                                                      // 8
    _id: "smtpConfig"                                                                                                  // 8
  });                                                                                                                  // 8
  return config && config.value;                                                                                       // 9
};                                                                                                                     // 10
                                                                                                                       //
var makePool = function (mailConfig) {                                                                                 // 12
  if (!mailConfig.hostname) {                                                                                          // 13
    throw new Error("This Sandstorm server has not been configured to send email.");                                   // 14
  }                                                                                                                    // 15
                                                                                                                       //
  var auth = false;                                                                                                    // 17
                                                                                                                       //
  if (mailConfig.auth && (mailConfig.auth.user || mailConfig.auth.pass)) {                                             // 18
    auth = mailConfig.auth;                                                                                            // 19
  }                                                                                                                    // 20
                                                                                                                       //
  var secure = mailConfig.port === 465;                                                                                // 22
  var tlsOptions = {                                                                                                   // 23
    // Previously, node 0.10 did not attempt to validate certificates received when connecting                         // 24
    // with STARTTLS, so to avoid regressing we need to preserve that behavior here for now.                           // 25
    rejectUnauthorized: false                                                                                          // 26
  };                                                                                                                   // 23
  var pool = nodemailer.createTransport(smtpPool({                                                                     // 29
    host: mailConfig.hostname,                                                                                         // 30
    port: mailConfig.port,                                                                                             // 31
    secure: secure,                                                                                                    // 32
    tls: tlsOptions,                                                                                                   // 33
    auth: auth // TODO(someday): allow maxConnections to be configured?                                                // 34
                                                                                                                       //
  }));                                                                                                                 // 29
  pool._futureWrappedSendMail = _.bind(Future.wrap(pool.sendMail), pool);                                              // 38
  return pool;                                                                                                         // 39
}; // We construct the SMTP pool at the first call to Email.send, so that                                              // 40
// other code like migrations can modify the SMTP configuration.                                                       // 43
                                                                                                                       //
                                                                                                                       //
var pool = void 0;                                                                                                     // 44
var configured = false;                                                                                                // 45
Meteor.startup(function () {                                                                                           // 47
  Settings.find({                                                                                                      // 48
    _id: "smtpConfig"                                                                                                  // 48
  }).observeChanges({                                                                                                  // 48
    removed: function () {                                                                                             // 49
      configured = false;                                                                                              // 50
    },                                                                                                                 // 51
    changed: function () {                                                                                             // 53
      configured = false;                                                                                              // 54
    },                                                                                                                 // 55
    added: function () {                                                                                               // 57
      configured = false;                                                                                              // 58
    }                                                                                                                  // 59
  });                                                                                                                  // 48
});                                                                                                                    // 61
                                                                                                                       //
var getPool = function (smtpConfig) {                                                                                  // 63
  if (smtpConfig) {                                                                                                    // 64
    return makePool(smtpConfig);                                                                                       // 65
  } else if (!configured) {                                                                                            // 66
    configured = true;                                                                                                 // 67
    var config = getSmtpConfig();                                                                                      // 68
                                                                                                                       //
    if (config) {                                                                                                      // 69
      pool = makePool(config);                                                                                         // 70
    }                                                                                                                  // 71
  }                                                                                                                    // 72
                                                                                                                       //
  return pool;                                                                                                         // 74
};                                                                                                                     // 75
                                                                                                                       //
var smtpSend = function (pool, mailOptions) {                                                                          // 77
  pool._futureWrappedSendMail(mailOptions).wait();                                                                     // 78
}; // From http://emailregex.com/, which claims this is the W3C standard for the HTML input element,                   // 79
// although their link is broken and I can find no evidence that this is a standard. The page                          // 82
// lists several regexes, ostensibly in syntaxes intended for different programming languages,                         // 83
// but each regex is in fact substantially different for no apparent reason.                                           // 84
//                                                                                                                     // 85
// The most important thing here is that we disallow separators that might allow a user to confuse                     // 86
// nodemailer into thinking the address is a list. Unfortunately, nodemailer will happily separate                     // 87
// strings into lists splitting on all kinds of separator characters, such as commas, semicolons,                      // 88
// etc. This regex should accomplish that both by disallowing the separators, and by disallowing                       // 89
// multiple @ signs. The rest is for show.                                                                             // 90
                                                                                                                       //
                                                                                                                       //
var EMAIL_REGEX = /^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/;                              // 91
                                                                                                                       //
function validateEmail(email) {                                                                                        // 93
  if (email instanceof Array) {                                                                                        // 94
    email.forEach(validateEmail);                                                                                      // 95
  } else if ((typeof email === "undefined" ? "undefined" : (0, _typeof3.default)(email)) === "object" && "address" in email) {
    validateEmail(email.address);                                                                                      // 97
  } else if (email) {                                                                                                  // 98
    check(email, String);                                                                                              // 99
                                                                                                                       //
    if (!email.match(EMAIL_REGEX)) {                                                                                   // 101
      throw new Meteor.Error(400, "invalid e-mail address");                                                           // 102
    }                                                                                                                  // 103
  }                                                                                                                    // 104
}                                                                                                                      // 105
                                                                                                                       //
var rawSend = function (mailOptions, smtpConfig) {                                                                     // 107
  // Sends an email mailOptions object structured as described in                                                      // 108
  // https://github.com/nodemailer/mailcomposer#e-mail-message-fields                                                  // 109
  // across the transport described by smtpConfig.                                                                     // 110
  // For fields that are supposed to be lists of addresses, if only a single string is provided,                       // 112
  // wrap it in an array. This prevents nodemailer from interpreting the address as a                                  // 113
  // comma-separated list.                                                                                             // 114
  ["from", "to", "cc", "bcc", "replyTo"].forEach(function (field) {                                                    // 115
    validateEmail(mailOptions[field]);                                                                                 // 116
  });                                                                                                                  // 117
  var pool = getPool(smtpConfig);                                                                                      // 119
                                                                                                                       //
  if (pool) {                                                                                                          // 120
    smtpSend(pool, mailOptions);                                                                                       // 121
  } else {                                                                                                             // 122
    throw new Error("SMTP pool is misconfigured.");                                                                    // 123
  }                                                                                                                    // 124
}; // Old comment below                                                                                                // 125
/**                                                                                                                    // 128
 * Send an email.                                                                                                      //
 *                                                                                                                     //
 * Connects to the mail server configured via the MAIL_URL environment                                                 //
 * variable. If unset, prints formatted message to stdout. The "from" option                                           //
 * is required, and at least one of "to", "cc", and "bcc" must be provided;                                            //
 * all other options are optional.                                                                                     //
 *                                                                                                                     //
 * @param options                                                                                                      //
 * @param options.from {String} RFC5322 "From:" address                                                                //
 * @param options.to {String|String[]} RFC5322 "To:" address[es]                                                       //
 * @param options.cc {String|String[]} RFC5322 "Cc:" address[es]                                                       //
 * @param options.bcc {String|String[]} RFC5322 "Bcc:" address[es]                                                     //
 * @param options.replyTo {String|String[]} RFC5322 "Reply-To:" address[es]                                            //
 * @param options.subject {String} RFC5322 "Subject:" line                                                             //
 * @param options.text {String} RFC5322 mail body (plain text)                                                         //
 * @param options.html {String} RFC5322 mail body (HTML)                                                               //
 * @param options.headers {Object} custom RFC5322 headers (dictionary)                                                 //
 */ // New API doc comment below                                                                                       //
/**                                                                                                                    // 149
 * @summary Send an email. Throws an `Error` on failure to contact mail server                                         //
 * or if mail server returns an error. All fields should match                                                         //
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                                                        //
 * @locus Server                                                                                                       //
 * @param {Object} options                                                                                             //
 * @param {String} options.from "From:" address (required)                                                             //
 * @param {String|String[]} options.to,cc,bcc,replyTo                                                                  //
 *   "To:", "Cc:", "Bcc:", and "Reply-To:" addresses                                                                   //
 * @param {String} [options.subject]  "Subject:" line                                                                  //
 * @param {String} [options.text|html] Mail body (in plain text or HTML)                                               //
 * @param {Object} [options.headers] Dictionary of custom headers                                                      //
 * @param {Object} [options.smtpConfig] SMTP server to use. Otherwise defaults to configured one.                      //
 * @param {String} [options.smtpConfig.hostname] SMTP server hostname.                                                 //
 * @param {Number} [options.smtpConfig.port] SMTP server port.                                                         //
 * @param {Object} [options.smtpConfig.auth] SMTP server authentication tokens.  Optional.                             //
 * @param {String} [options.smtpConfig.auth.user] Username of user to log in to SMTP server as.  Optional.             //
 * @param {String} [options.smtpConfig.auth.pass] Password of user to log in to SMTP server as.  Optional.             //
 * @param {Object} [options.attachments] Attachments. See:                                                             //
 *   https://github.com/nodemailer/mailcomposer/tree/v0.1.15#add-attachments                                           //
 * @param {String} [options.envelopeFrom] Envelope sender.                                                             //
 */                                                                                                                    //
                                                                                                                       //
var send = function (options) {                                                                                        // 171
  // Unpack options                                                                                                    // 172
  var from = options.from,                                                                                             // 171
      to = options.to,                                                                                                 // 171
      cc = options.cc,                                                                                                 // 171
      bcc = options.bcc,                                                                                               // 171
      replyTo = options.replyTo,                                                                                       // 171
      subject = options.subject,                                                                                       // 171
      text = options.text,                                                                                             // 171
      html = options.html,                                                                                             // 171
      envelopeFrom = options.envelopeFrom,                                                                             // 171
      headers = options.headers,                                                                                       // 171
      attachments = options.attachments,                                                                               // 171
      smtpConfig = options.smtpConfig;                                                                                 // 171
  var opts = {                                                                                                         // 188
    from: from,                                                                                                        // 189
    to: to,                                                                                                            // 190
    cc: cc,                                                                                                            // 191
    bcc: bcc,                                                                                                          // 192
    replyTo: replyTo,                                                                                                  // 193
    subject: subject,                                                                                                  // 194
    text: text,                                                                                                        // 195
    html: html,                                                                                                        // 196
    headers: headers,                                                                                                  // 197
    attachments: attachments                                                                                           // 198
  };                                                                                                                   // 188
                                                                                                                       //
  if (envelopeFrom) {                                                                                                  // 201
    opts.envelope = {                                                                                                  // 202
      from: envelopeFrom,                                                                                              // 203
      to: to,                                                                                                          // 204
      cc: cc,                                                                                                          // 205
      bcc: bcc                                                                                                         // 206
    };                                                                                                                 // 202
  }                                                                                                                    // 208
                                                                                                                       //
  rawSend(opts, smtpConfig);                                                                                           // 210
};                                                                                                                     // 211
                                                                                                                       //
// TODO(cleanup): Remove this once BlackrockPayments code finds a better way to import it.                             // 215
global.SandstormEmail = {                                                                                              // 216
  send: send                                                                                                           // 216
};                                                                                                                     // 216
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"frontend-ref.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/frontend-ref.js                                                                                      //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
module.export({                                                                                                        // 1
  FrontendRefRegistry: function () {                                                                                   // 1
    return FrontendRefRegistry;                                                                                        // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var checkRequirements = void 0;                                                                                        // 1
module.watch(require("./persistent.js"), {                                                                             // 1
  checkRequirements: function (v) {                                                                                    // 1
    checkRequirements = v;                                                                                             // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var FrontendRefRegistry = function () {                                                                                //
  function FrontendRefRegistry() {                                                                                     // 20
    (0, _classCallCheck3.default)(this, FrontendRefRegistry);                                                          // 20
    this._frontendRefHandlers = {};                                                                                    // 21
    this._typeIdHandlers = {};                                                                                         // 22
  }                                                                                                                    // 23
                                                                                                                       //
  FrontendRefRegistry.prototype.create = function () {                                                                 //
    function create(db, frontendRef, requirements) {                                                                   //
      // Construct a fresh capability based on a `frontendRef` value.                                                  // 26
      checkRequirements(requirements);                                                                                 // 28
      var saveTemplate = {                                                                                             // 30
        frontendRef: frontendRef                                                                                       // 30
      };                                                                                                               // 30
                                                                                                                       //
      if (requirements) {                                                                                              // 31
        saveTemplate.requirements = requirements;                                                                      // 32
      }                                                                                                                // 33
                                                                                                                       //
      return this.restore(db, saveTemplate, frontendRef);                                                              // 35
    }                                                                                                                  // 36
                                                                                                                       //
    return create;                                                                                                     //
  }();                                                                                                                 //
                                                                                                                       //
  FrontendRefRegistry.prototype.restore = function () {                                                                //
    function restore(db, saveTemplate, frontendRef) {                                                                  //
      // Restores a frontendRef capability using the appropriate registered handler.                                   // 39
      var keys = Object.keys(frontendRef);                                                                             // 41
                                                                                                                       //
      if (keys.length != 1) {                                                                                          // 42
        throw new Error("invalid frontendRef: " + JSON.stringify(frontendRef));                                        // 43
      }                                                                                                                // 44
                                                                                                                       //
      var key = keys[0];                                                                                               // 46
      var handler = this._frontendRefHandlers[key];                                                                    // 47
                                                                                                                       //
      if (!handler) {                                                                                                  // 48
        throw new Error("invalid frontendRef: " + JSON.stringify(frontendRef));                                        // 49
      }                                                                                                                // 50
                                                                                                                       //
      return handler.restore(db, saveTemplate, frontendRef[key]);                                                      // 52
    }                                                                                                                  // 53
                                                                                                                       //
    return restore;                                                                                                    //
  }();                                                                                                                 //
                                                                                                                       //
  FrontendRefRegistry.prototype.query = function () {                                                                  //
    function query(db, userAccountId, tag) {                                                                           //
      // Performs a powerbox query using the appropriate registered handler.                                           // 56
      var handler = this._typeIdHandlers[tag.id];                                                                      // 58
                                                                                                                       //
      if (handler) {                                                                                                   // 59
        return handler.query(db, userAccountId, tag.value);                                                            // 60
      } else {                                                                                                         // 61
        return []; // no matches                                                                                       // 62
      }                                                                                                                // 63
    }                                                                                                                  // 64
                                                                                                                       //
    return query;                                                                                                      //
  }();                                                                                                                 //
                                                                                                                       //
  FrontendRefRegistry.prototype.validate = function () {                                                               //
    function validate(db, session, frontendRefRequest) {                                                               //
      // Validates a powerbox request on the given `session` (a record from the Sessions table)                        // 67
      // requesting the creation of the given frontendRef type. `frontendRefRequest` is expected to                    // 68
      // come directly from the client; calling validate() verifies that it is well-formed and                         // 69
      // carries out any type-specific server-side work needed to construct the desired frontendRef.                   // 70
      // `validate()` also returns an array of MembraneRequirements that shall apply to the                            // 71
      // capability. These requirements will not yet have been checked, so the caller should check                     // 72
      // them before proceeding.                                                                                       // 73
      var keys = Object.keys(frontendRefRequest);                                                                      // 75
                                                                                                                       //
      if (keys.length != 1) {                                                                                          // 76
        throw new Error("invalid frontendRefRequest: " + JSON.stringify(frontendRefRequest));                          // 77
      }                                                                                                                // 78
                                                                                                                       //
      var key = keys[0];                                                                                               // 80
      var handler = this._frontendRefHandlers[key];                                                                    // 81
                                                                                                                       //
      if (!handler) {                                                                                                  // 82
        throw new Error("invalid frontendRefRequest: " + JSON.stringify(frontendRefRequest));                          // 83
      }                                                                                                                // 84
                                                                                                                       //
      if (!handler.validate) {                                                                                         // 86
        throw new Error("frontendRef type cannot be created via powerbox: " + JSON.stringify(frontendRefRequest));     // 87
      }                                                                                                                // 89
                                                                                                                       //
      var _handler$validate = handler.validate(db, session, frontendRefRequest[key]),                                  // 66
          descriptor = _handler$validate.descriptor,                                                                   // 66
          requirements = _handler$validate.requirements,                                                               // 66
          frontendRef = _handler$validate.frontendRef;                                                                 // 66
                                                                                                                       //
      var result = {                                                                                                   // 94
        descriptor: descriptor,                                                                                        // 94
        requirements: requirements,                                                                                    // 94
        frontendRef: {}                                                                                                // 94
      };                                                                                                               // 94
      result.frontendRef[key] = frontendRef;                                                                           // 95
      return result;                                                                                                   // 96
    }                                                                                                                  // 97
                                                                                                                       //
    return validate;                                                                                                   //
  }();                                                                                                                 //
                                                                                                                       //
  FrontendRefRegistry.prototype.register = function () {                                                               //
    function register(object) {                                                                                        //
      // Register callbacks related to a particular frontendRef type. The object has the fields:                       // 100
      //   `frontendRefField`: Name of the field of `ApiTokens.frontendRef` that is filled in for                      // 101
      //       this type. Only needed if `create` and/or `validate` handlers are defined.                              // 102
      //   `typeId`: Type ID of powerbox tags handled by the `query` callback. Stringified decimal                     // 103
      //       64-bit integer. Only needed if `query` is defined.                                                      // 104
      //   `restore`: Callback to construct a capability of this type when restoring a saved                           // 105
      //       capability. Has signature `(db, saveTemplate, value) -> capability`, where:                             // 106
      //     `value`: The value of the single field of `ApiTokens.frontendRef` for this capability.                    // 107
      //     `saveTemplate`: The token template to pass to the PersistentImpl constructor.                             // 108
      //     `capability` (returned): A Cap'n Proto capability implementing SystemPersistent along                     // 109
      //         with whatever other interfaces are appropriate for the ref type.                                      // 110
      //   `validate`: Callback to validate a powerbox request for a new capability of this type.                      // 111
      //       Has signature `(db, session, request) -> {descriptor, requirements, frontendRef}`,                      // 112
      //       where:                                                                                                  // 113
      //     `request` is type-specific information describing the requested capability. The                           // 114
      //         callback *must* type-check this value, and should throw an exception if it is                         // 115
      //         not valid.                                                                                            // 116
      //     `session` is the record from the Sessions table of the UI session where the powerbox                      // 117
      //         request occurred.                                                                                     // 118
      //     `descriptor` (returned) is the JSON-encoded PowerboxDescriptor for the capability. Note                   // 119
      //         that if the descriptor contains any `tag.value`s, they of course need to be                           // 120
      //         presented as capnp-encoded Buffers.                                                                   // 121
      //     `requirements` (returned) is an array of MembraneRequirements which should apply to the                   // 122
      //         new capability. Note that these requirements will be checked immediately and the                      // 123
      //         powerbox request will fail if they aren't met.                                                        // 124
      //     `frontendRef` (returned) is the value that will be written for the key specified                          // 125
      //         by `frontendRefField` in the single-key object `ApiTokens.frontendRef`.                               // 126
      //    `query`: Callback to populate options for a powerbox request for this type ID. Has                         // 127
      //        signature `(db, userAccountId, tagValue) -> options`, where:                                           // 128
      //      `tagValue`: A Buffer of the Cap'n-Proto-encoded `PowerboxDescriptor.Tag.value`.                          // 129
      //      `options` (returned): An array of objects representing the options that should be                        // 130
      //          offered to the user for this query. See the `powerboxOptions` Meteor publish in                      // 131
      //          powerbox-server.js for a full description of the fields of each option.                              // 132
      if (object.frontendRefField) {                                                                                   // 134
        if (object.frontendRefField in this._frontendRefHandlers) {                                                    // 135
          throw new Error("frontendRef handler already registered: " + object.frontendRefField);                       // 136
        }                                                                                                              // 137
                                                                                                                       //
        this._frontendRefHandlers[object.frontendRefField] = object;                                                   // 139
      }                                                                                                                // 140
                                                                                                                       //
      if (object.typeId) {                                                                                             // 142
        if (object.typeId in this._typeIdHandlers) {                                                                   // 143
          throw new Error("typeId handler already registered: " + object.typeId);                                      // 144
        }                                                                                                              // 145
                                                                                                                       //
        this._typeIdHandlers[object.typeId] = object;                                                                  // 147
      }                                                                                                                // 148
    }                                                                                                                  // 149
                                                                                                                       //
    return register;                                                                                                   //
  }();                                                                                                                 //
                                                                                                                       //
  return FrontendRefRegistry;                                                                                          //
}();                                                                                                                   //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"header-whitelist.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/header-whitelist.js                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
module.export({                                                                                                        // 1
  HeaderWhitelist: function () {                                                                                       // 1
    return HeaderWhitelist;                                                                                            // 1
  },                                                                                                                   // 1
  REQUEST_HEADER_WHITELIST: function () {                                                                              // 1
    return REQUEST_HEADER_WHITELIST;                                                                                   // 1
  },                                                                                                                   // 1
  RESPONSE_HEADER_WHITELIST: function () {                                                                             // 1
    return RESPONSE_HEADER_WHITELIST;                                                                                  // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2017 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var WebSession = Capnp.importSystem("sandstorm/web-session.capnp").WebSession;                                         // 17
                                                                                                                       //
var HeaderWhitelist = function () {                                                                                    //
  function HeaderWhitelist(list) {                                                                                     // 20
    var _this = this;                                                                                                  // 20
                                                                                                                       //
    (0, _classCallCheck3.default)(this, HeaderWhitelist);                                                              // 20
    this._headers = {};                                                                                                // 21
    this._prefixes = [];                                                                                               // 22
    list.forEach(function (pattern) {                                                                                  // 24
      if (pattern.endsWith("*")) {                                                                                     // 25
        _this._prefixes.push(pattern.slice(0, -1));                                                                    // 26
      } else {                                                                                                         // 27
        _this._headers[pattern] = true;                                                                                // 28
      }                                                                                                                // 29
    });                                                                                                                // 30
  }                                                                                                                    // 31
                                                                                                                       //
  HeaderWhitelist.prototype.matches = function () {                                                                    //
    function matches(header) {                                                                                         //
      header = header.toLowerCase();                                                                                   // 34
      if (this._headers[header]) return true;                                                                          // 35
                                                                                                                       //
      for (var i in meteorBabelHelpers.sanitizeForInObject(this._prefixes)) {                                          // 37
        if (header.startsWith(this._prefixes[i])) {                                                                    // 38
          return true;                                                                                                 // 39
        }                                                                                                              // 40
      }                                                                                                                // 41
                                                                                                                       //
      ;                                                                                                                // 41
      return false;                                                                                                    // 43
    }                                                                                                                  // 44
                                                                                                                       //
    return matches;                                                                                                    //
  }();                                                                                                                 //
                                                                                                                       //
  return HeaderWhitelist;                                                                                              //
}();                                                                                                                   //
                                                                                                                       //
var REQUEST_HEADER_WHITELIST = new HeaderWhitelist(WebSession.Context.headerWhitelist);                                // 47
var RESPONSE_HEADER_WHITELIST = new HeaderWhitelist(WebSession.Response.headerWhitelist);                              // 48
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"migrations.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/migrations.js                                                                                        //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  migrateToLatest: function () {                                                                                       // 1
    return migrateToLatest;                                                                                            // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var _ = void 0;                                                                                                        // 1
                                                                                                                       //
module.watch(require("meteor/underscore"), {                                                                           // 1
  _: function (v) {                                                                                                    // 1
    _ = v;                                                                                                             // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Match = void 0;                                                                                                    // 1
module.watch(require("meteor/check"), {                                                                                // 1
  Match: function (v) {                                                                                                // 1
    Match = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var userPictureUrl = void 0,                                                                                           // 1
    fetchPicture = void 0;                                                                                             // 1
module.watch(require("/imports/server/accounts/picture.js"), {                                                         // 1
  userPictureUrl: function (v) {                                                                                       // 1
    userPictureUrl = v;                                                                                                // 1
  },                                                                                                                   // 1
  fetchPicture: function (v) {                                                                                         // 1
    fetchPicture = v;                                                                                                  // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var PRIVATE_IPV4_ADDRESSES = void 0,                                                                                   // 1
    PRIVATE_IPV6_ADDRESSES = void 0;                                                                                   // 1
module.watch(require("/imports/constants.js"), {                                                                       // 1
  PRIVATE_IPV4_ADDRESSES: function (v) {                                                                               // 1
    PRIVATE_IPV4_ADDRESSES = v;                                                                                        // 1
  },                                                                                                                   // 1
  PRIVATE_IPV6_ADDRESSES: function (v) {                                                                               // 1
    PRIVATE_IPV6_ADDRESSES = v;                                                                                        // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 14
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 15
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 16
                                                                                                                       //
var updateLoginStyleToRedirect = function (db, backend) {                                                              // 18
  var configurations = Package["service-configuration"].ServiceConfiguration.configurations;                           // 19
  ["google", "github"].forEach(function (serviceName) {                                                                // 20
    var config = configurations.findOne({                                                                              // 21
      service: serviceName                                                                                             // 21
    });                                                                                                                // 21
                                                                                                                       //
    if (config && config.loginStyle !== "redirect") {                                                                  // 22
      configurations.update({                                                                                          // 23
        service: serviceName                                                                                           // 23
      }, {                                                                                                             // 23
        $set: {                                                                                                        // 23
          loginStyle: "redirect"                                                                                       // 23
        }                                                                                                              // 23
      });                                                                                                              // 23
    }                                                                                                                  // 24
  });                                                                                                                  // 25
};                                                                                                                     // 26
                                                                                                                       //
var enableLegacyOAuthProvidersIfNotInSettings = function (db, backend) {                                               // 28
  // In the before time, Google and Github login were enabled by default.                                              // 29
  //                                                                                                                   // 30
  // This actually didn't make much sense, required the first user to configure                                        // 31
  // OAuth, and had some trust-the-first-user properties that weren't totally                                          // 32
  // secure.                                                                                                           // 33
  //                                                                                                                   // 34
  // Now, we have admin-token, so we wish to disable all logins by default                                             // 35
  // (since they need to be configured anyway) but since users may have never                                          // 36
  // explicitly told Sandstorm that Google or Github login should be enabled,                                          // 37
  // we can't just use the value in the Settings collection, since it might                                            // 38
  // never have been set.                                                                                              // 39
  //                                                                                                                   // 40
  // Thus, if the service is configured but absent from Settings, we should                                            // 41
  // explicitly enable it in Settings, and then the rest of the logic can just                                         // 42
  // depend on what value is in Settings and default to false without breaking                                         // 43
  // user installations.                                                                                               // 44
  var configurations = ServiceConfiguration.configurations;                                                            // 45
  ["google", "github"].forEach(function (serviceName) {                                                                // 46
    var config = configurations.findOne({                                                                              // 47
      service: serviceName                                                                                             // 47
    });                                                                                                                // 47
    var serviceConfig = db.collections.settings.findOne({                                                              // 48
      _id: serviceName                                                                                                 // 48
    });                                                                                                                // 48
                                                                                                                       //
    if (config && !serviceConfig) {                                                                                    // 49
      // Only explicitly enable the login service if:                                                                  // 50
      // 1) the service is already configured                                                                          // 51
      // 2) there is no sandstorm configuration already present (the user was                                          // 52
      //    using the previous default behavior).                                                                      // 53
      db.collections.settings.insert({                                                                                 // 54
        _id: serviceName,                                                                                              // 54
        value: true                                                                                                    // 54
      });                                                                                                              // 54
    }                                                                                                                  // 55
  });                                                                                                                  // 56
};                                                                                                                     // 57
                                                                                                                       //
var denormalizeInviteInfo = function (db, backend) {                                                                   // 59
  // When a user is invited via a signup token, the `signupKey` field of their user table entry                        // 60
  // has always been populated to indicate the key they used. This points into the SignupKeys table                    // 61
  // which has more information about the key, namely a freeform note entered by the admin when                        // 62
  // they created the key. In the case that the email invite form was used, the note has the form                      // 63
  // "E-mail invite to <address>".                                                                                     // 64
  //                                                                                                                   // 65
  // Later, we decided it was useful to indicate in the users table visible to the admin                               // 66
  // information about the invite terms. Namely, for email invites we want to show the address                         // 67
  // and for others we want to show the note. To make this efficient, fields `signupNote` and                          // 68
  // `signupEmail` were added to the users table. We can backfill these values by denormalizing                        // 69
  // from the SignupKeys table.                                                                                        // 70
  db.collections.users.find().forEach(function (user) {                                                                // 72
    if (user.signupKey && typeof user.signupKey === "string" && user.signupKey !== "admin") {                          // 73
      var signupInfo = db.collections.signupKeys.findOne(user.signupKey);                                              // 74
                                                                                                                       //
      if (signupInfo && signupInfo.note) {                                                                             // 75
        var newFields = {                                                                                              // 76
          signupNote: signupInfo.note                                                                                  // 76
        };                                                                                                             // 76
        var prefix = "E-mail invite to ";                                                                              // 78
                                                                                                                       //
        if (signupInfo.note.lastIndexOf(prefix) === 0) {                                                               // 79
          newFields.signupEmail = signupInfo.note.slice(prefix.length);                                                // 80
        }                                                                                                              // 81
                                                                                                                       //
        db.collections.users.update(user._id, {                                                                        // 83
          $set: newFields                                                                                              // 83
        });                                                                                                            // 83
      }                                                                                                                // 84
    }                                                                                                                  // 85
  });                                                                                                                  // 86
};                                                                                                                     // 87
                                                                                                                       //
var mergeRoleAssignmentsIntoApiTokens = function (db, backend) {                                                       // 89
  db.collections.roleAssignments.find().forEach(function (roleAssignment) {                                            // 90
    db.collections.apiTokens.insert({                                                                                  // 91
      grainId: roleAssignment.grainId,                                                                                 // 92
      userId: roleAssignment.sharer,                                                                                   // 93
      roleAssignment: roleAssignment.roleAssignment,                                                                   // 94
      petname: roleAssignment.petname,                                                                                 // 95
      created: roleAssignment.created,                                                                                 // 96
      owner: {                                                                                                         // 97
        user: {                                                                                                        // 98
          userId: roleAssignment.recipient,                                                                            // 99
          title: roleAssignment.title                                                                                  // 100
        }                                                                                                              // 98
      }                                                                                                                // 97
    });                                                                                                                // 91
  });                                                                                                                  // 104
};                                                                                                                     // 105
                                                                                                                       //
var fixOasisStorageUsageStats = function (db, backend) {}; // This migration only pertained to Oasis and it was successfully applied there. Since it referred
// to some global variables that we later wanted to remove and/or rename, we've since replaced it                      // 109
// with a no-op.                                                                                                       // 110
                                                                                                                       //
                                                                                                                       //
var fetchProfilePictures = function (db, backend) {                                                                    // 112
  db.collections.users.find({}).forEach(function (user) {                                                              // 113
    var url = userPictureUrl(user);                                                                                    // 114
                                                                                                                       //
    if (url) {                                                                                                         // 115
      console.log("Fetching user picture:", url);                                                                      // 116
      var assetId = fetchPicture(db, url);                                                                             // 117
                                                                                                                       //
      if (assetId) {                                                                                                   // 118
        db.collections.users.update(user._id, {                                                                        // 119
          $set: {                                                                                                      // 119
            "profile.picture": assetId                                                                                 // 119
          }                                                                                                            // 119
        });                                                                                                            // 119
      }                                                                                                                // 120
    }                                                                                                                  // 121
  });                                                                                                                  // 122
};                                                                                                                     // 123
                                                                                                                       //
var assignPlans = function (db, backend) {// This was a one-time migration intended to be applied on Oasis to existing users.
  // It has run, so we only need this stub function here.                                                              // 127
};                                                                                                                     // 128
                                                                                                                       //
var removeKeyrings = function (db, backend) {                                                                          // 130
  // These blobs full of public keys were not intended to find their way into mongo and while                          // 131
  // harmless they slow things down because they're huge. Remove them.                                                 // 132
  db.collections.packages.update({                                                                                     // 133
    "manifest.metadata.pgpKeyring": {                                                                                  // 133
      $exists: true                                                                                                    // 133
    }                                                                                                                  // 133
  }, {                                                                                                                 // 133
    $unset: {                                                                                                          // 134
      "manifest.metadata.pgpKeyring": ""                                                                               // 134
    }                                                                                                                  // 134
  }, {                                                                                                                 // 134
    multi: true                                                                                                        // 135
  });                                                                                                                  // 135
};                                                                                                                     // 136
                                                                                                                       //
var useLocalizedTextInUserActions = function (db, backend) {                                                           // 138
  function toLocalizedText(newObj, oldObj, field) {                                                                    // 139
    if (field in oldObj) {                                                                                             // 140
      if (typeof oldObj[field] === "string") {                                                                         // 141
        newObj[field] = {                                                                                              // 142
          defaultText: oldObj[field]                                                                                   // 142
        };                                                                                                             // 142
      } else {                                                                                                         // 143
        newObj[field] = oldObj[field];                                                                                 // 144
      }                                                                                                                // 145
    }                                                                                                                  // 146
  }                                                                                                                    // 147
                                                                                                                       //
  db.collections.userActions.find({}).forEach(function (userAction) {                                                  // 149
    var fields = {};                                                                                                   // 150
    toLocalizedText(fields, userAction, "appTitle");                                                                   // 151
    toLocalizedText(fields, userAction, "title");                                                                      // 152
    toLocalizedText(fields, userAction, "nounPhrase");                                                                 // 153
    db.collections.userActions.update(userAction._id, {                                                                // 154
      $set: fields                                                                                                     // 154
    });                                                                                                                // 154
  });                                                                                                                  // 155
};                                                                                                                     // 156
                                                                                                                       //
var verifyAllPgpSignatures = function (db, backend) {                                                                  // 158
  db.collections.packages.find({}).forEach(function (pkg) {                                                            // 159
    try {                                                                                                              // 160
      console.log("checking PGP signature for package:", pkg._id);                                                     // 161
      var info = waitPromise(backend.cap().tryGetPackage(pkg._id));                                                    // 162
                                                                                                                       //
      if (info.authorPgpKeyFingerprint) {                                                                              // 163
        console.log("  " + info.authorPgpKeyFingerprint);                                                              // 164
        db.collections.packages.update(pkg._id, {                                                                      // 165
          $set: {                                                                                                      // 166
            authorPgpKeyFingerprint: info.authorPgpKeyFingerprint                                                      // 166
          }                                                                                                            // 166
        });                                                                                                            // 166
      } else {                                                                                                         // 167
        console.log("  no signature");                                                                                 // 168
      }                                                                                                                // 169
    } catch (err) {                                                                                                    // 170
      console.error(err.stack);                                                                                        // 171
    }                                                                                                                  // 172
  });                                                                                                                  // 173
};                                                                                                                     // 174
                                                                                                                       //
var splitUserIdsIntoAccountIdsAndIdentityIds = function (db, backend) {                                                // 176
  db.collections.users.find().forEach(function (user) {                                                                // 177
    var identity = {};                                                                                                 // 178
    var serviceUserId = void 0;                                                                                        // 179
                                                                                                                       //
    if ("devName" in user) {                                                                                           // 180
      identity.service = "dev";                                                                                        // 181
      serviceUserId = user.devName;                                                                                    // 182
    } else if ("expires" in user) {                                                                                    // 183
      identity.service = "demo";                                                                                       // 184
      serviceUserId = user._id;                                                                                        // 185
    } else if (user.services && "google" in user.services) {                                                           // 186
      identity.service = "google";                                                                                     // 187
                                                                                                                       //
      if (user.services.google.email && user.services.google.verified_email) {                                         // 188
        // jscs:ignore requireCamelCaseOrUpperCaseIdentifiers                                                          // 188
        identity.verifiedEmail = user.services.google.email;                                                           // 189
      }                                                                                                                // 190
                                                                                                                       //
      serviceUserId = user.services.google.id;                                                                         // 192
    } else if (user.services && "github" in user.services) {                                                           // 193
      identity.service = "github";                                                                                     // 194
      identity.unverifiedEmail = user.services.github.email;                                                           // 195
      serviceUserId = user.services.github.id;                                                                         // 196
    } else if (user.services && "emailToken" in user.services) {                                                       // 197
      identity.service = "emailToken";                                                                                 // 198
      identity.verifiedEmail = user.services.emailToken.email;                                                         // 199
      serviceUserId = user.services.emailToken.email;                                                                  // 200
    }                                                                                                                  // 201
                                                                                                                       //
    identity.id = Crypto.createHash("sha256").update(identity.service + ":" + serviceUserId).digest("hex");            // 203
                                                                                                                       //
    if (user.profile) {                                                                                                // 206
      if (user.profile.name) {                                                                                         // 207
        identity.name = user.profile.name;                                                                             // 208
      }                                                                                                                // 209
                                                                                                                       //
      if (user.profile.handle) {                                                                                       // 211
        identity.handle = user.profile.handle;                                                                         // 212
      }                                                                                                                // 213
                                                                                                                       //
      if (user.profile.picture) {                                                                                      // 215
        identity.picture = user.profile.picture;                                                                       // 216
      }                                                                                                                // 217
                                                                                                                       //
      if (user.profile.pronoun) {                                                                                      // 219
        identity.pronoun = user.profile.pronoun;                                                                       // 220
      }                                                                                                                // 221
                                                                                                                       //
      if (user.profile.email) {                                                                                        // 223
        identity.unverifiedEmail = user.profile.email;                                                                 // 224
      }                                                                                                                // 225
    }                                                                                                                  // 226
                                                                                                                       //
    identity.main = true;                                                                                              // 228
    db.collections.users.update(user._id, {                                                                            // 230
      $set: {                                                                                                          // 230
        identities: [identity]                                                                                         // 230
      }                                                                                                                // 230
    });                                                                                                                // 230
    db.collections.grains.update({                                                                                     // 232
      userId: user._id                                                                                                 // 232
    }, {                                                                                                               // 232
      $set: {                                                                                                          // 232
        identityId: identity.id                                                                                        // 232
      }                                                                                                                // 232
    }, {                                                                                                               // 232
      multi: true                                                                                                      // 232
    });                                                                                                                // 232
    db.collections.sessions.update({                                                                                   // 233
      userId: user._id                                                                                                 // 233
    }, {                                                                                                               // 233
      $set: {                                                                                                          // 233
        identityId: identity.id                                                                                        // 233
      }                                                                                                                // 233
    }, {                                                                                                               // 233
      multi: true                                                                                                      // 233
    });                                                                                                                // 233
    db.collections.apiTokens.update({                                                                                  // 234
      userId: user._id                                                                                                 // 234
    }, {                                                                                                               // 234
      $set: {                                                                                                          // 235
        identityId: identity.id                                                                                        // 235
      }                                                                                                                // 235
    }, {                                                                                                               // 235
      multi: true                                                                                                      // 236
    });                                                                                                                // 236
    db.collections.apiTokens.update({                                                                                  // 237
      "owner.user.userId": user._id                                                                                    // 237
    }, {                                                                                                               // 237
      $set: {                                                                                                          // 238
        "owner.user.identityId": identity.id                                                                           // 238
      }                                                                                                                // 238
    }, {                                                                                                               // 238
      multi: true                                                                                                      // 239
    });                                                                                                                // 239
    db.collections.apiTokens.update({                                                                                  // 240
      "owner.grain.introducerUser": user._id                                                                           // 240
    }, {                                                                                                               // 240
      $set: {                                                                                                          // 241
        "owner.grain.introducerIdentity": identity.id                                                                  // 241
      }                                                                                                                // 241
    }, {                                                                                                               // 241
      multi: true                                                                                                      // 242
    });                                                                                                                // 242
                                                                                                                       //
    while (db.collections.apiTokens.update({                                                                           // 244
      "requirements.permissionsHeld.userId": user._id                                                                  // 245
    }, {                                                                                                               // 244
      $set: {                                                                                                          // 247
        "requirements.$.permissionsHeld.identityId": identity.id                                                       // 247
      },                                                                                                               // 247
      $unset: {                                                                                                        // 248
        "requirements.$.permissionsHeld.userId": 1                                                                     // 248
      }                                                                                                                // 248
    }, {                                                                                                               // 246
      multi: true                                                                                                      // 250
    }) > 0) {} // The `$` operator modifies the first element in the array that matches the query. Since               // 249
    // there may be many matches, we need to repeat until no documents are modified.                                   // 253
                                                                                                                       //
  });                                                                                                                  // 255
  db.collections.apiTokens.remove({                                                                                    // 257
    userInfo: {                                                                                                        // 257
      $exists: true                                                                                                    // 257
    }                                                                                                                  // 257
  }); // We've renamed `Grain.UserInfo.userId` to `Grain.userInfo.identityId`. The only place                          // 257
  // that this field could show up in the database was in this deprecated, no-longer-functional                        // 259
  // form of API token.                                                                                                // 260
};                                                                                                                     // 261
                                                                                                                       //
var appUpdateSettings = function (db, backend) {                                                                       // 263
  db.collections.settings.insert({                                                                                     // 264
    _id: "appMarketUrl",                                                                                               // 264
    value: "https://apps.sandstorm.io"                                                                                 // 264
  });                                                                                                                  // 264
  db.collections.settings.insert({                                                                                     // 265
    _id: "appIndexUrl",                                                                                                // 265
    value: "https://app-index.sandstorm.io"                                                                            // 265
  });                                                                                                                  // 265
  db.collections.settings.insert({                                                                                     // 266
    _id: "appUpdatesEnabled",                                                                                          // 266
    value: true                                                                                                        // 266
  });                                                                                                                  // 266
};                                                                                                                     // 267
                                                                                                                       //
var moveDevAndEmailLoginDataIntoIdentities = function (db, backend) {                                                  // 269
  db.collections.users.find().forEach(function (user) {                                                                // 270
    if (user.identities.length !== 1) {                                                                                // 271
      throw new Error("User does not have exactly one identity: ", user);                                              // 272
    }                                                                                                                  // 273
                                                                                                                       //
    var identity = user.identities[0];                                                                                 // 275
                                                                                                                       //
    if (Match.test(identity.service, Object)) {                                                                        // 276
      return;                                                                                                          // 276
    } // Already migrated.                                                                                             // 276
                                                                                                                       //
                                                                                                                       //
    var newIdentity = _.pick(identity, "id", "main", "noLogin", "verifiedEmail", "unverifiedEmail");                   // 278
                                                                                                                       //
    newIdentity.profile = _.pick(identity, "name", "handle", "picture", "pronoun");                                    // 279
    var serviceObject = {};                                                                                            // 281
    var fieldsToUnset = {};                                                                                            // 282
                                                                                                                       //
    if (identity.service === "dev") {                                                                                  // 284
      serviceObject.name = user.devName;                                                                               // 285
      fieldsToUnset.devName = 1;                                                                                       // 286
    } else if (identity.service === "emailToken") {                                                                    // 287
      serviceObject.tokens = user.services.emailToken.tokens;                                                          // 288
      serviceObject.email = user.services.emailToken.email;                                                            // 289
      fieldsToUnset["services.emailToken"] = 1;                                                                        // 290
    }                                                                                                                  // 291
                                                                                                                       //
    newIdentity.service = {};                                                                                          // 293
    newIdentity.service[identity.service] = serviceObject;                                                             // 294
    var modifier = {                                                                                                   // 296
      $set: {                                                                                                          // 296
        identities: [newIdentity]                                                                                      // 296
      }                                                                                                                // 296
    };                                                                                                                 // 296
                                                                                                                       //
    if (Object.keys(fieldsToUnset).length > 0) {                                                                       // 297
      modifier.$unset = fieldsToUnset;                                                                                 // 298
    }                                                                                                                  // 299
                                                                                                                       //
    db.collections.users.update({                                                                                      // 301
      _id: user._id                                                                                                    // 301
    }, modifier);                                                                                                      // 301
  });                                                                                                                  // 302
};                                                                                                                     // 303
                                                                                                                       //
var repairEmailIdentityIds = function (db, backend) {                                                                  // 305
  db.collections.users.find({                                                                                          // 306
    "identities.service.emailToken": {                                                                                 // 306
      $exists: 1                                                                                                       // 306
    }                                                                                                                  // 306
  }).forEach(function (user) {                                                                                         // 306
    if (user.identities.length !== 1) {                                                                                // 307
      throw new Error("User does not have exactly one identity: ", user);                                              // 308
    }                                                                                                                  // 309
                                                                                                                       //
    var identity = user.identities[0];                                                                                 // 311
                                                                                                                       //
    var newIdentity = _.pick(identity, "main", "noLogin", "verifiedEmail", "unverifiedMail", "profile");               // 312
                                                                                                                       //
    newIdentity.service = {                                                                                            // 314
      email: identity.service.emailToken                                                                               // 314
    };                                                                                                                 // 314
    newIdentity.id = Crypto.createHash("sha256").update("email:" + identity.service.emailToken.email).digest("hex");   // 315
    db.collections.grains.update({                                                                                     // 318
      identityId: identity.id                                                                                          // 318
    }, {                                                                                                               // 318
      $set: {                                                                                                          // 318
        identityId: newIdentity.id                                                                                     // 318
      }                                                                                                                // 318
    }, {                                                                                                               // 318
      multi: true                                                                                                      // 318
    });                                                                                                                // 318
    db.collections.sessions.update({                                                                                   // 319
      identityId: identity.id                                                                                          // 319
    }, {                                                                                                               // 319
      $set: {                                                                                                          // 319
        identityId: newIdentity.id                                                                                     // 319
      }                                                                                                                // 319
    }, {                                                                                                               // 319
      multi: true                                                                                                      // 319
    });                                                                                                                // 319
    db.collections.apiTokens.update({                                                                                  // 320
      identityId: identity.id                                                                                          // 320
    }, {                                                                                                               // 320
      $set: {                                                                                                          // 321
        identityId: newIdentity.id                                                                                     // 321
      }                                                                                                                // 321
    }, {                                                                                                               // 321
      multi: true                                                                                                      // 322
    });                                                                                                                // 322
    db.collections.apiTokens.update({                                                                                  // 323
      "owner.user.identityId": identity.id                                                                             // 323
    }, {                                                                                                               // 323
      $set: {                                                                                                          // 324
        "owner.user.identityId": newIdentity.id                                                                        // 324
      }                                                                                                                // 324
    }, {                                                                                                               // 324
      multi: true                                                                                                      // 325
    });                                                                                                                // 325
    db.collections.apiTokens.update({                                                                                  // 326
      "owner.grain.introducerIdentity": identity.id                                                                    // 326
    }, {                                                                                                               // 326
      $set: {                                                                                                          // 327
        "owner.grain.introducerIdentity": newIdentity.id                                                               // 327
      }                                                                                                                // 327
    }, {                                                                                                               // 327
      multi: true                                                                                                      // 328
    });                                                                                                                // 328
                                                                                                                       //
    while (db.collections.apiTokens.update({                                                                           // 330
      "requirements.permissionsHeld.identityId": identity.id                                                           // 330
    }, {                                                                                                               // 330
      $set: {                                                                                                          // 331
        "requirements.$.permissionsHeld.identityId": newIdentity.id                                                    // 331
      }                                                                                                                // 331
    }, {                                                                                                               // 331
      multi: true                                                                                                      // 332
    }) > 0) {}                                                                                                         // 332
                                                                                                                       //
    db.collections.users.update({                                                                                      // 334
      _id: user._id                                                                                                    // 334
    }, {                                                                                                               // 334
      $set: {                                                                                                          // 334
        identities: [newIdentity]                                                                                      // 334
      }                                                                                                                // 334
    });                                                                                                                // 334
  });                                                                                                                  // 335
};                                                                                                                     // 336
                                                                                                                       //
var splitAccountUsersAndIdentityUsers = function (db, backend) {                                                       // 338
  db.collections.users.find({                                                                                          // 339
    identities: {                                                                                                      // 339
      $exists: true                                                                                                    // 339
    }                                                                                                                  // 339
  }).forEach(function (user) {                                                                                         // 339
    if (user.identities.length !== 1) {                                                                                // 340
      throw new Error("User does not have exactly one identity: ", user);                                              // 341
    }                                                                                                                  // 342
                                                                                                                       //
    var identity = user.identities[0];                                                                                 // 344
                                                                                                                       //
    var identityUser = _.pick(user, "createdAt", "lastActive", "expires");                                             // 345
                                                                                                                       //
    identityUser._id = identity.id;                                                                                    // 346
    identityUser.profile = identity.profile;                                                                           // 347
                                                                                                                       //
    _.extend(identityUser, _.pick(identity, "unverifiedEmail"));                                                       // 348
                                                                                                                       //
    identityUser.profile.service = Object.keys(identity.service)[0]; // Updating this user needs to be a two step process because the `services` field typically
    // contains subfields that are constrained to be unique by Mongo indices.                                          // 352
                                                                                                                       //
    identityUser.stagedServices = _.omit(user.services, "resume");                                                     // 353
                                                                                                                       //
    if (identity.service.dev) {                                                                                        // 354
      identityUser.stagedServices.dev = identity.service.dev;                                                          // 355
    } else if (identity.service.email) {                                                                               // 356
      identityUser.stagedServices.email = identity.service.email;                                                      // 357
    }                                                                                                                  // 358
                                                                                                                       //
    var accountUser = _.pick(user, "_id", "createdAt", "lastActive", "expires", "isAdmin", "signupKey", "signupNote", "signupEmail", "plan", "storageUsage", "isAppDemoUser", "appDemoId", "payments", "dailySentMailCount", "hasCompletedSignup");
                                                                                                                       //
    accountUser.loginIdentities = [_.pick(identity, "id")];                                                            // 364
    accountUser.nonloginIdentities = [];                                                                               // 365
                                                                                                                       //
    if (user.services && user.services.resume) {                                                                       // 366
      accountUser.services = {                                                                                         // 367
        resume: user.services.resume                                                                                   // 367
      };                                                                                                               // 367
    }                                                                                                                  // 368
                                                                                                                       //
    accountUser.stashedOldUser = user;                                                                                 // 370
    db.collections.apiTokens.update({                                                                                  // 372
      identityId: identityUser._id                                                                                     // 373
    }, {                                                                                                               // 373
      $set: {                                                                                                          // 374
        accountId: user._id                                                                                            // 374
      }                                                                                                                // 374
    }, {                                                                                                               // 374
      multi: true                                                                                                      // 375
    });                                                                                                                // 375
    db.collections.users.upsert({                                                                                      // 377
      _id: identityUser._id                                                                                            // 377
    }, identityUser);                                                                                                  // 377
    db.collections.users.update({                                                                                      // 378
      _id: user._id                                                                                                    // 378
    }, accountUser);                                                                                                   // 378
  });                                                                                                                  // 379
  db.collections.users.find({                                                                                          // 381
    stagedServices: {                                                                                                  // 381
      $exists: true                                                                                                    // 381
    }                                                                                                                  // 381
  }).forEach(function (identity) {                                                                                     // 381
    db.collections.users.update({                                                                                      // 382
      _id: identity._id                                                                                                // 382
    }, {                                                                                                               // 382
      $unset: {                                                                                                        // 383
        stagedServices: 1                                                                                              // 383
      },                                                                                                               // 383
      $set: {                                                                                                          // 384
        services: identity.stagedServices                                                                              // 384
      }                                                                                                                // 384
    });                                                                                                                // 382
  });                                                                                                                  // 386
};                                                                                                                     // 387
                                                                                                                       //
var populateContactsFromApiTokens = function (db, backend) {                                                           // 389
  db.collections.apiTokens.find({                                                                                      // 390
    "owner.user.identityId": {                                                                                         // 391
      $exists: 1                                                                                                       // 391
    },                                                                                                                 // 391
    accountId: {                                                                                                       // 392
      $exists: 1                                                                                                       // 392
    }                                                                                                                  // 392
  }).forEach(function (token) {                                                                                        // 390
    var identityId = token.owner.user.identityId;                                                                      // 394
    var identity = Meteor.users.findOne({                                                                              // 395
      _id: identityId                                                                                                  // 395
    });                                                                                                                // 395
                                                                                                                       //
    if (identity) {                                                                                                    // 396
      var profile = identity.profile;                                                                                  // 397
      SandstormDb.fillInProfileDefaults(identity, profile);                                                            // 398
      db.collections.contacts.upsert({                                                                                 // 399
        ownerId: token.accountId,                                                                                      // 399
        identityId: identityId                                                                                         // 399
      }, {                                                                                                             // 399
        ownerId: token.accountId,                                                                                      // 400
        petname: profile && profile.name,                                                                              // 401
        created: new Date(),                                                                                           // 402
        identityId: identityId,                                                                                        // 403
        profile: profile                                                                                               // 404
      });                                                                                                              // 399
    }                                                                                                                  // 406
  });                                                                                                                  // 407
};                                                                                                                     // 408
                                                                                                                       //
var cleanUpApiTokens = function (db, backend) {                                                                        // 410
  // The `splitUserIdsIntoAccountIdsAndIdentityIds()` migration only added `identityId` in cases                       // 411
  // where the user still existed in the database.                                                                     // 412
  db.collections.apiTokens.remove({                                                                                    // 413
    userId: {                                                                                                          // 414
      $exists: true                                                                                                    // 414
    },                                                                                                                 // 414
    identityId: {                                                                                                      // 415
      $exists: false                                                                                                   // 415
    }                                                                                                                  // 415
  });                                                                                                                  // 413
  db.collections.apiTokens.remove({                                                                                    // 417
    "owner.user.userId": {                                                                                             // 418
      $exists: true                                                                                                    // 418
    },                                                                                                                 // 418
    "owner.user.identityId": {                                                                                         // 419
      $exists: false                                                                                                   // 419
    }                                                                                                                  // 419
  }); // For a while we were accidentally setting `appIcon` instead of `icon`.                                         // 417
                                                                                                                       //
  db.collections.apiTokens.find({                                                                                      // 423
    "owner.user.denormalizedGrainMetadata.appIcon": {                                                                  // 424
      $exists: true                                                                                                    // 424
    }                                                                                                                  // 424
  }).forEach(function (apiToken) {                                                                                     // 423
    var icon = apiToken.owner.user.denormalizedGrainMetadata.appIcon;                                                  // 426
    db.collections.apiTokens.update({                                                                                  // 427
      _id: apiToken._id                                                                                                // 427
    }, {                                                                                                               // 427
      $set: {                                                                                                          // 428
        "owner.user.denormalizedGrainMetadata.icon": icon                                                              // 428
      },                                                                                                               // 428
      $unset: {                                                                                                        // 429
        "owner.user.denormalizedGrainMetadata.appIcon": true                                                           // 429
      }                                                                                                                // 429
    });                                                                                                                // 427
  }); // For a while the `identityId` field of child UiView tokens was not getting set.                                // 431
                                                                                                                       //
  function repairChain(parentToken) {                                                                                  // 434
    db.collections.apiTokens.find({                                                                                    // 435
      parentToken: parentToken._id,                                                                                    // 436
      grainId: {                                                                                                       // 437
        $exists: true                                                                                                  // 437
      },                                                                                                               // 437
      identityId: {                                                                                                    // 438
        $exists: false                                                                                                 // 438
      }                                                                                                                // 438
    }).forEach(function (childToken) {                                                                                 // 435
      db.collections.apiTokens.update({                                                                                // 440
        _id: childToken._id                                                                                            // 440
      }, {                                                                                                             // 440
        $set: {                                                                                                        // 440
          identityId: parentToken.identityId                                                                           // 440
        }                                                                                                              // 440
      });                                                                                                              // 440
      repairChain(childToken);                                                                                         // 441
    });                                                                                                                // 442
  }                                                                                                                    // 443
                                                                                                                       //
  db.collections.apiTokens.find({                                                                                      // 445
    grainId: {                                                                                                         // 446
      $exists: true                                                                                                    // 446
    },                                                                                                                 // 446
    identityId: {                                                                                                      // 447
      $exists: true                                                                                                    // 447
    },                                                                                                                 // 447
    parentToken: {                                                                                                     // 448
      $exists: false                                                                                                   // 448
    }                                                                                                                  // 448
  }).forEach(repairChain);                                                                                             // 445
};                                                                                                                     // 450
                                                                                                                       //
var initServerTitleAndReturnAddress = function (db, backend) {                                                         // 452
  var hostname = Url.parse(process.env.ROOT_URL).hostname;                                                             // 453
  db.collections.settings.insert({                                                                                     // 454
    _id: "serverTitle",                                                                                                // 454
    value: hostname                                                                                                    // 454
  });                                                                                                                  // 454
  db.collections.settings.insert({                                                                                     // 455
    _id: "returnAddress",                                                                                              // 455
    value: "no-reply@" + hostname                                                                                      // 455
  });                                                                                                                  // 455
};                                                                                                                     // 456
                                                                                                                       //
var sendReferralNotifications = function (db, backend) {                                                               // 458
  if (db.isReferralEnabled()) {                                                                                        // 459
    db.collections.users.find({                                                                                        // 460
      loginIdentities: {                                                                                               // 461
        $exists: true                                                                                                  // 461
      },                                                                                                               // 461
      expires: {                                                                                                       // 462
        $exists: false                                                                                                 // 462
      }                                                                                                                // 462
    }, {                                                                                                               // 460
      fields: {                                                                                                        // 463
        _id: 1                                                                                                         // 463
      }                                                                                                                // 463
    }).forEach(function (user) {                                                                                       // 463
      db.sendReferralProgramNotification(user._id);                                                                    // 464
    });                                                                                                                // 465
  }                                                                                                                    // 466
};                                                                                                                     // 467
                                                                                                                       //
var assignBonuses = function (db, backend) {// This was a one-time migration intended to be applied on Oasis to existing users.
  // It has run, so we only need this stub function here.                                                              // 471
};                                                                                                                     // 472
                                                                                                                       //
var splitSmtpUrl = function (db, backend) {                                                                            // 474
  var smtpUrlSetting = db.collections.settings.findOne({                                                               // 475
    _id: "smtpUrl"                                                                                                     // 475
  });                                                                                                                  // 475
  var smtpUrl = smtpUrlSetting ? smtpUrlSetting.value : process.env.MAIL_URL;                                          // 476
  var returnAddress = db.collections.settings.findOne({                                                                // 477
    _id: "returnAddress"                                                                                               // 477
  }); // Default values.                                                                                               // 477
                                                                                                                       //
  var smtpConfig = {                                                                                                   // 480
    hostname: "localhost",                                                                                             // 481
    port: "25",                                                                                                        // 482
    auth: undefined,                                                                                                   // 483
    returnAddress: returnAddress.value                                                                                 // 484
  };                                                                                                                   // 480
  var parsed = void 0;                                                                                                 // 487
                                                                                                                       //
  try {                                                                                                                // 488
    parsed = smtpUrl && Url.parse(smtpUrl);                                                                            // 489
  } catch (e) {}                                                                                                       // 490
                                                                                                                       //
  if (parsed) {                                                                                                        // 492
    // If there was a SMTP URL previously defined, import its data.                                                    // 493
    var auth = undefined;                                                                                              // 494
                                                                                                                       //
    if (parsed.auth) {                                                                                                 // 495
      var colonIndex = parsed.auth.indexOf(":");                                                                       // 496
      var user = undefined;                                                                                            // 497
      var pass = undefined;                                                                                            // 498
                                                                                                                       //
      if (colonIndex !== -1) {                                                                                         // 499
        user = parsed.auth.slice(0, colonIndex);                                                                       // 500
        pass = parsed.auth.slice(colonIndex + 1);                                                                      // 501
      }                                                                                                                // 502
                                                                                                                       //
      auth = {                                                                                                         // 504
        user: user,                                                                                                    // 505
        pass: pass                                                                                                     // 506
      };                                                                                                               // 504
    } // Override defaults with previous config's values.                                                              // 508
                                                                                                                       //
                                                                                                                       //
    smtpConfig.hostname = parsed.hostname || "localhost";                                                              // 511
    smtpConfig.port = parsed.port || "25";                                                                             // 512
    smtpConfig.auth = auth;                                                                                            // 513
  }                                                                                                                    // 514
                                                                                                                       //
  db.collections.settings.upsert({                                                                                     // 516
    _id: "smtpConfig"                                                                                                  // 516
  }, {                                                                                                                 // 516
    value: smtpConfig                                                                                                  // 516
  });                                                                                                                  // 516
  db.collections.settings.remove({                                                                                     // 517
    _id: "returnAddress"                                                                                               // 517
  });                                                                                                                  // 517
  db.collections.settings.remove({                                                                                     // 518
    _id: "smtpUrl"                                                                                                     // 518
  });                                                                                                                  // 518
};                                                                                                                     // 519
                                                                                                                       //
var smtpPortShouldBeNumber = function (db, backend) {                                                                  // 521
  var entry = db.collections.settings.findOne({                                                                        // 522
    _id: "smtpConfig"                                                                                                  // 522
  });                                                                                                                  // 522
                                                                                                                       //
  if (entry) {                                                                                                         // 523
    var setting = entry.value;                                                                                         // 524
                                                                                                                       //
    if (setting.port) {                                                                                                // 525
      setting.port = _.isNumber(setting.port) ? setting.port : parseInt(setting.port);                                 // 526
      db.collections.settings.upsert({                                                                                 // 527
        _id: "smtpConfig"                                                                                              // 527
      }, {                                                                                                             // 527
        value: setting                                                                                                 // 527
      });                                                                                                              // 527
    }                                                                                                                  // 528
  }                                                                                                                    // 529
};                                                                                                                     // 530
                                                                                                                       //
var consolidateOrgSettings = function (db, backend) {                                                                  // 532
  var settings = db.collections.settings;                                                                              // 533
  var orgGoogleDomain = settings.findOne({                                                                             // 534
    _id: "organizationGoogle"                                                                                          // 534
  });                                                                                                                  // 534
  var orgEmailDomain = settings.findOne({                                                                              // 535
    _id: "organizationEmail"                                                                                           // 535
  });                                                                                                                  // 535
  var orgLdap = settings.findOne({                                                                                     // 536
    _id: "organizationLdap"                                                                                            // 536
  });                                                                                                                  // 536
  var orgSaml = settings.findOne({                                                                                     // 537
    _id: "organizationSaml"                                                                                            // 537
  });                                                                                                                  // 537
  var orgMembership = {                                                                                                // 539
    google: {                                                                                                          // 540
      enabled: orgGoogleDomain ? !!orgGoogleDomain.value : false,                                                      // 541
      domain: orgGoogleDomain ? orgGoogleDomain.value : ""                                                             // 542
    },                                                                                                                 // 540
    email: {                                                                                                           // 544
      enabled: orgEmailDomain ? !!orgEmailDomain.value : false,                                                        // 545
      domain: orgEmailDomain ? orgEmailDomain.value : ""                                                               // 546
    },                                                                                                                 // 544
    ldap: {                                                                                                            // 548
      enabled: orgLdap ? orgLdap.value : false                                                                         // 549
    },                                                                                                                 // 548
    saml: {                                                                                                            // 551
      enabled: orgSaml ? orgSaml.value : false                                                                         // 552
    }                                                                                                                  // 551
  };                                                                                                                   // 539
  settings.upsert({                                                                                                    // 556
    _id: "organizationMembership"                                                                                      // 556
  }, {                                                                                                                 // 556
    value: orgMembership                                                                                               // 556
  });                                                                                                                  // 556
  settings.remove({                                                                                                    // 557
    _id: "organizationGoogle"                                                                                          // 557
  });                                                                                                                  // 557
  settings.remove({                                                                                                    // 558
    _id: "organizationEmail"                                                                                           // 558
  });                                                                                                                  // 558
  settings.remove({                                                                                                    // 559
    _id: "organizationLdap"                                                                                            // 559
  });                                                                                                                  // 559
  settings.remove({                                                                                                    // 560
    _id: "organizationSaml"                                                                                            // 560
  });                                                                                                                  // 560
};                                                                                                                     // 561
                                                                                                                       //
var unsetSmtpDefaultHostnameIfNoUsersExist = function (db, backend) {                                                  // 563
  // We don't actually want to have the default hostname "localhost" set.                                              // 564
  // If the user has already finished configuring their server, then this migration should do                          // 565
  // nothing (since we might break their deployment), but for new installs (which will have no users                   // 566
  // at the time this migration runs) we'll unset the hostname if it's still the previously-filled                     // 567
  // default value.                                                                                                    // 568
  var hasUsers = db.collections.users.findOne();                                                                       // 569
                                                                                                                       //
  if (!hasUsers) {                                                                                                     // 570
    var entry = db.collections.settings.findOne({                                                                      // 571
      _id: "smtpConfig"                                                                                                // 571
    });                                                                                                                // 571
    var smtpConfig = entry.value;                                                                                      // 572
                                                                                                                       //
    if (smtpConfig.hostname === "localhost") {                                                                         // 573
      smtpConfig.hostname = "";                                                                                        // 574
      db.collections.settings.upsert({                                                                                 // 575
        _id: "smtpConfig"                                                                                              // 575
      }, {                                                                                                             // 575
        value: smtpConfig                                                                                              // 575
      });                                                                                                              // 575
    }                                                                                                                  // 576
  }                                                                                                                    // 577
};                                                                                                                     // 578
                                                                                                                       //
var extractLastUsedFromApiTokenOwner = function (db, backend) {                                                        // 580
  // We used to store lastUsed as a field on owner.user.  It makes more sense to store lastUsed on                     // 581
  // the apiToken as a whole.  This migration hoists such values from owner.user onto the apiToken                     // 582
  // itself.                                                                                                           // 583
  db.collections.apiTokens.find({                                                                                      // 584
    "owner.user": {                                                                                                    // 584
      $exists: true                                                                                                    // 584
    }                                                                                                                  // 584
  }).forEach(function (token) {                                                                                        // 584
    var lastUsed = token.owner.user.lastUsed;                                                                          // 585
    db.collections.apiTokens.update({                                                                                  // 586
      _id: token._id                                                                                                   // 586
    }, {                                                                                                               // 586
      $set: {                                                                                                          // 587
        lastUsed: lastUsed                                                                                             // 587
      },                                                                                                               // 587
      $unset: {                                                                                                        // 588
        "owner.user.lastUsed": true                                                                                    // 588
      }                                                                                                                // 588
    });                                                                                                                // 586
  });                                                                                                                  // 590
};                                                                                                                     // 591
                                                                                                                       //
var setUpstreamTitles = function (db, backend) {                                                                       // 593
  // Initializes the `upstreamTitle` and `renamed` fields of `ApiToken.owner.user`.                                    // 594
  var apiTokensRaw = db.collections.apiTokens.rawCollection();                                                         // 596
  var aggregateApiTokens = Meteor.wrapAsync(apiTokensRaw.aggregate, apiTokensRaw); // First, construct a list of all *shared* grains. We will need to do a separate update()
  // for each of these, so we'd like to skip those which have nothing to update.                                       // 600
                                                                                                                       //
  var grainIds = aggregateApiTokens([{                                                                                 // 601
    $match: {                                                                                                          // 602
      "owner.user": {                                                                                                  // 602
        $exists: true                                                                                                  // 602
      },                                                                                                               // 602
      grainId: {                                                                                                       // 602
        $exists: true                                                                                                  // 602
      }                                                                                                                // 602
    }                                                                                                                  // 602
  }, {                                                                                                                 // 602
    $group: {                                                                                                          // 603
      _id: "$grainId"                                                                                                  // 603
    }                                                                                                                  // 603
  }]).map(function (grain) {                                                                                           // 603
    return grain._id;                                                                                                  // 604
  });                                                                                                                  // 604
  var count = 0;                                                                                                       // 606
  db.collections.grains.find({                                                                                         // 607
    _id: {                                                                                                             // 607
      $in: grainIds                                                                                                    // 607
    }                                                                                                                  // 607
  }, {                                                                                                                 // 607
    fields: {                                                                                                          // 607
      title: 1                                                                                                         // 607
    }                                                                                                                  // 607
  }).forEach(function (grain) {                                                                                        // 607
    if (count % 100 == 0) {                                                                                            // 608
      console.log(count + " / " + grainIds.length);                                                                    // 609
    }                                                                                                                  // 610
                                                                                                                       //
    ++count; // For ApiTokens whose petname titles do not match the upstream title, we need to set                     // 612
    // `upstreamTitle` and `renamed`. We have no way of knowing which of the two names was the                         // 615
    // original name, so we don't know whether it was the owner or the receiver who renamed their                      // 616
    // copy post-sharaing. We assume it was the owner, because:                                                        // 617
    // 1. It's probably unusual for people to try to rename a grain they don't own.                                    // 618
    // 2. This results in UI that is reasonably non-confusing even if we guessed wrong. Namely,                        // 619
    //    the user sees "Owner's title (was: User's title)", which is reasonably OK even if it                         // 620
    //    was the user who had renamed their copy. They can rename it again if they like. On the                       // 621
    //    other hand, if we guessed wrongly in the other direction, the user would see                                 // 622
    //    "User's title (renamed from: Owners title)", which would be wrong if it was in fact the                      // 623
    //    owner who renamed post-sharing.                                                                              // 624
                                                                                                                       //
    db.collections.apiTokens.update({                                                                                  // 625
      grainId: grain._id,                                                                                              // 626
      "owner.user.title": {                                                                                            // 627
        $exists: true,                                                                                                 // 627
        $ne: grain.title                                                                                               // 627
      }                                                                                                                // 627
    }, {                                                                                                               // 625
      $set: {                                                                                                          // 628
        "owner.user.upstreamTitle": grain.title                                                                        // 628
      }                                                                                                                // 628
    }, {                                                                                                               // 628
      multi: true                                                                                                      // 628
    });                                                                                                                // 628
  });                                                                                                                  // 629
};                                                                                                                     // 630
                                                                                                                       //
var markAllRead = function (db, backend) {                                                                             // 632
  // Mark as "read" all grains and tokens that predate the creation of read/unread status.                             // 633
  // Otherwise it's pretty annoying to see all your old grains look like they have activity.                           // 634
  db.collections.grains.update({}, {                                                                                   // 636
    $set: {                                                                                                            // 636
      ownerSeenAllActivity: true                                                                                       // 636
    }                                                                                                                  // 636
  }, {                                                                                                                 // 636
    multi: true                                                                                                        // 636
  });                                                                                                                  // 636
  db.collections.apiTokens.update({                                                                                    // 637
    "owner.user": {                                                                                                    // 637
      $exists: true                                                                                                    // 637
    }                                                                                                                  // 637
  }, {                                                                                                                 // 637
    $set: {                                                                                                            // 638
      "owner.user.seenAllActivity": true                                                                               // 638
    }                                                                                                                  // 638
  }, {                                                                                                                 // 638
    multi: true                                                                                                        // 639
  });                                                                                                                  // 639
};                                                                                                                     // 640
                                                                                                                       //
var clearAppIndex = function (db, backend) {                                                                           // 642
  // Due to a bug in the app update code, some app update notifications that the user accepted                         // 643
  // around July 9-16, 2016 may not have applied. We have no way of knowing exactly which updates                      // 644
  // the user accepted but didn't receive. Instead, to recover, we are clearing the local cache of                     // 645
  // the app index one time. This way, the next time the app index is re-fetched, the server will                      // 646
  // interpret all of the apps in the index as updated and will re-deliver notifications to                            // 647
  // everyone. This may mean users get notifications that they previously dismissed, but they can                      // 648
  // click "dismiss" again easily enough.                                                                              // 649
  db.collections.appIndex.remove({});                                                                                  // 651
};                                                                                                                     // 652
                                                                                                                       //
var assignEmailVerifierIds = function (db, backend) {                                                                  // 654
  // Originally, the ID of an EmailVerifier was actually the _id of the root token from which it                       // 655
  // was restored. This was broken, though: Conceptually, it meant that you couldn't have a working                    // 656
  // EmailVerifier that had not been restore()d from disk. In practice, that wasn't a problem due                      // 657
  // to the fact that powerbox selections always involve a restore(), but a different problem                          // 658
  // existed: the capability returned by claimRequest() would return one ID, but after save()ing                       // 659
  // and restore()ing that capability, the ID would be different, because the new capability would                     // 660
  // be a copy -- not a child -- of the original. Additionally, the whole thing made various code                      // 661
  // ugly because it was puncturing layers of abstraction. So, we switched to doing the right                          // 662
  // thing: assigning an ID to the EmailVerifier on first creation and storing it separately.                          // 663
  db.collections.apiTokens.find({                                                                                      // 665
    "frontendRef.emailVerifier": {                                                                                     // 665
      $exists: true                                                                                                    // 665
    }                                                                                                                  // 665
  }).forEach(function (token) {                                                                                        // 665
    db.collections.apiTokens.update(token._id, {                                                                       // 666
      $set: {                                                                                                          // 666
        "frontendRef.emailVerifier.id": token._id                                                                      // 666
      }                                                                                                                // 666
    });                                                                                                                // 666
  });                                                                                                                  // 667
};                                                                                                                     // 668
                                                                                                                       //
var startPreinstallingApps = function (db, backend) {                                                                  // 670
  // This isn't really a normal migration. It will run only on brand new servers, and it has to                        // 671
  // run after the `clearAppIndex` migration because it relies on populating AppIndex.                                 // 672
  var startPreinstallingAppsHelper = function () {                                                                     // 674
    db.updateAppIndex();                                                                                               // 675
    var preinstalledApps = db.collections.appIndex.find({                                                              // 677
      _id: {                                                                                                           // 677
        $in: db.getProductivitySuiteAppIds().concat(db.getSystemSuiteAppIds())                                         // 678
      }                                                                                                                // 677
    }).fetch();                                                                                                        // 677
                                                                                                                       //
    var appAndPackageIds = _.map(preinstalledApps, function (app) {                                                    // 681
      return {                                                                                                         // 682
        appId: app.appId,                                                                                              // 683
        packageId: app.packageId                                                                                       // 684
      };                                                                                                               // 682
    });                                                                                                                // 686
                                                                                                                       //
    db.setPreinstalledApps(appAndPackageIds);                                                                          // 688
  };                                                                                                                   // 689
                                                                                                                       //
  if (!Meteor.settings.public.isTesting && !db.allowDevAccounts()) {                                                   // 691
    // We want preinstalling apps to run async and not block startup.                                                  // 692
    Meteor.setTimeout(startPreinstallingAppsHelper, 0);                                                                // 693
  }                                                                                                                    // 694
};                                                                                                                     // 695
                                                                                                                       //
var setNewServer = function (db, backend) {                                                                            // 697
  // This migration only applies to "old" servers. New servers will set                                                // 698
  // new_server_migrations_applied to false before any migrations run.                                                 // 699
  if (!db.collections.migrations.findOne({                                                                             // 700
    _id: "new_server_migrations_applied"                                                                               // 700
  })) {                                                                                                                // 700
    db.collections.migrations.insert({                                                                                 // 701
      _id: "new_server_migrations_applied",                                                                            // 701
      value: true                                                                                                      // 701
    });                                                                                                                // 701
  }                                                                                                                    // 702
};                                                                                                                     // 703
                                                                                                                       //
var addMembraneRequirementsToIdentities = function (db, backend) {                                                     // 705
  var query = {                                                                                                        // 706
    "frontendRef.identity": {                                                                                          // 707
      $exists: true                                                                                                    // 707
    },                                                                                                                 // 707
    "owner.grain.grainId": {                                                                                           // 708
      $exists: true                                                                                                    // 708
    },                                                                                                                 // 708
    "requirements.0": {                                                                                                // 709
      $exists: false                                                                                                   // 709
    }                                                                                                                  // 709
  };                                                                                                                   // 706
  db.collections.apiTokens.find(query).map(function (apiToken) {                                                       // 712
    db.collections.apiTokens.update({                                                                                  // 713
      _id: apiToken._id                                                                                                // 714
    }, {                                                                                                               // 714
      $push: {                                                                                                         // 716
        requirements: {                                                                                                // 717
          permissionsHeld: {                                                                                           // 718
            grainId: apiToken.owner.grain.grainId,                                                                     // 719
            identityId: apiToken.frontendRef.identity,                                                                 // 720
            permissions: []                                                                                            // 721
          }                                                                                                            // 718
        }                                                                                                              // 717
      }                                                                                                                // 716
    });                                                                                                                // 715
  });                                                                                                                  // 727
};                                                                                                                     // 728
                                                                                                                       //
var addEncryptionToFrontendRefIpNetwork = function (db, backend) {                                                     // 730
  db.collections.apiTokens.find({                                                                                      // 731
    "frontendRef.ipNetwork": true                                                                                      // 731
  }).map(function (apiToken) {                                                                                         // 731
    db.collections.apiTokens.update({                                                                                  // 732
      _id: apiToken._id                                                                                                // 733
    }, {                                                                                                               // 733
      $set: {                                                                                                          // 734
        "frontendRef.ipNetwork": {                                                                                     // 734
          encryption: {                                                                                                // 734
            none: null                                                                                                 // 734
          }                                                                                                            // 734
        }                                                                                                              // 734
      }                                                                                                                // 734
    });                                                                                                                // 734
  });                                                                                                                  // 735
};                                                                                                                     // 736
                                                                                                                       //
function backgroundFillInGrainSizes(db, backend) {                                                                     // 738
  // Fill in sizes for all grains that don't have them. Since computing a grain size requires a                        // 739
  // directory walk, we don't want to do them all at once. Instead, we compute one a second until                      // 740
  // all grains have sizes.                                                                                            // 741
  try {                                                                                                                // 743
    var grain = db.collections.grains.findOne({                                                                        // 744
      size: {                                                                                                          // 744
        $exists: false                                                                                                 // 744
      }                                                                                                                // 744
    }, {                                                                                                               // 744
      fields: {                                                                                                        // 744
        _id: 1,                                                                                                        // 744
        userId: 1                                                                                                      // 744
      }                                                                                                                // 744
    });                                                                                                                // 744
                                                                                                                       //
    if (grain) {                                                                                                       // 746
      // Compute size!                                                                                                 // 747
      try {                                                                                                            // 748
        var result = waitPromise(backend.cap().getGrainStorageUsage(grain.userId, grain._id));                         // 749
        db.collections.grains.update({                                                                                 // 751
          _id: grain._id,                                                                                              // 751
          size: {                                                                                                      // 751
            $exists: false                                                                                             // 751
          }                                                                                                            // 751
        }, {                                                                                                           // 751
          $set: {                                                                                                      // 752
            size: parseInt(result.size)                                                                                // 752
          }                                                                                                            // 752
        });                                                                                                            // 752
      } catch (err) {                                                                                                  // 753
        if (err.kjType === "failed") {                                                                                 // 754
          // Backend had a problem. Maybe the grain doesn't actually exist on disk and the database                    // 755
          // is messed up. We'll set the size to zero and move on.                                                     // 756
          console.error("Error while backfilling grain size for", grain._id, ":", err.stack);                          // 757
          db.collections.grains.update({                                                                               // 758
            _id: grain._id,                                                                                            // 758
            size: {                                                                                                    // 758
              $exists: false                                                                                           // 758
            }                                                                                                          // 758
          }, {                                                                                                         // 758
            $set: {                                                                                                    // 758
              size: 0                                                                                                  // 758
            }                                                                                                          // 758
          });                                                                                                          // 758
        } else {                                                                                                       // 759
          // Rethrow on disconnected / overloaded / unimplemented.                                                     // 760
          throw err;                                                                                                   // 761
        }                                                                                                              // 762
      } // Do another one in a second.                                                                                 // 763
                                                                                                                       //
                                                                                                                       //
      Meteor.setTimeout(backgroundFillInGrainSizes.bind(this, db, backend), 1000);                                     // 766
    }                                                                                                                  // 767
  } catch (err) {                                                                                                      // 768
    // We'll just stop for now, to avoid spamming logs if this error persists. Next time the server                    // 769
    // restarts, the migration will continue.                                                                          // 770
    console.error("Error while backfilling grain sizes:", err.stack);                                                  // 771
  }                                                                                                                    // 772
}                                                                                                                      // 773
                                                                                                                       //
function removeFeatureKeys(db, backend) {                                                                              // 775
  // Remove obsolete data related to the Sandstorm for Work paywall, which was eliminated.                             // 776
  db.notifications.remove({                                                                                            // 778
    "admin.type": "cantRenewFeatureKey"                                                                                // 778
  });                                                                                                                  // 778
  db.notifications.remove({                                                                                            // 779
    "admin.type": "trialFeatureKeyExpired"                                                                             // 779
  });                                                                                                                  // 779
}                                                                                                                      // 780
                                                                                                                       //
function setIpBlacklist(db, backend) {                                                                                 // 782
  if (Meteor.settings.public.isTesting) {                                                                              // 783
    db.collections.settings.insert({                                                                                   // 784
      _id: "ipBlacklist",                                                                                              // 784
      value: "192.168.0.0/16"                                                                                          // 784
    });                                                                                                                // 784
  } else {                                                                                                             // 785
    var defaultIpBlacklist = PRIVATE_IPV4_ADDRESSES.concat(PRIVATE_IPV6_ADDRESSES).join("\n");                         // 786
    db.collections.settings.insert({                                                                                   // 787
      _id: "ipBlacklist",                                                                                              // 787
      value: defaultIpBlacklist                                                                                        // 787
    });                                                                                                                // 787
  }                                                                                                                    // 788
}                                                                                                                      // 789
                                                                                                                       //
function getUserIdentityIds(user) {                                                                                    // 791
  // Formerly SandstormDb.getUserIdentityIds(), from before the identity refactor.                                     // 792
  if (user && user.loginIdentities) {                                                                                  // 794
    return _.pluck(user.nonloginIdentities.concat(user.loginIdentities), "id").reverse();                              // 795
  } else {                                                                                                             // 796
    return [];                                                                                                         // 797
  }                                                                                                                    // 798
}                                                                                                                      // 799
                                                                                                                       //
function notifyIdentityChanges(db, backend) {                                                                          // 801
  // Notify users who might be affected by the identity model changes.                                                 // 802
  //                                                                                                                   // 803
  // Two types of users are affected:                                                                                  // 804
  // - Users who have multiple identities with differing names.                                                        // 805
  // - Users who have identities that are shared with other users.                                                     // 806
  //                                                                                                                   // 807
  // However, the second group seems like it must be a subset of the first group. So we only check                     // 808
  // for the first.                                                                                                    // 809
  var names = {};                                                                                                      // 811
  Meteor.users.find({                                                                                                  // 812
    "profile.name": {                                                                                                  // 812
      $exists: true                                                                                                    // 812
    }                                                                                                                  // 812
  }, {                                                                                                                 // 812
    fields: {                                                                                                          // 812
      "profile.name": 1                                                                                                // 812
    }                                                                                                                  // 812
  }).forEach(function (user) {                                                                                         // 812
    names[user._id] = user.profile.name;                                                                               // 814
  });                                                                                                                  // 815
  Meteor.users.find({                                                                                                  // 817
    loginIdentities: {                                                                                                 // 817
      $exists: true                                                                                                    // 817
    }                                                                                                                  // 817
  }, {                                                                                                                 // 817
    fields: {                                                                                                          // 818
      loginIdentities: 1,                                                                                              // 818
      nonloginIdentities: 1                                                                                            // 818
    }                                                                                                                  // 818
  }).forEach(function (user) {                                                                                         // 818
    var previousName = null;                                                                                           // 819
    var needsNotification = false;                                                                                     // 820
    getUserIdentityIds(user).forEach(function (identityId) {                                                           // 821
      var name = names[identityId];                                                                                    // 822
                                                                                                                       //
      if (!name || previousName && previousName !== name) {                                                            // 823
        needsNotification = true;                                                                                      // 824
      }                                                                                                                // 825
                                                                                                                       //
      previousName = name;                                                                                             // 827
    });                                                                                                                // 828
                                                                                                                       //
    if (needsNotification) {                                                                                           // 830
      db.collections.notifications.upsert({                                                                            // 831
        userId: user._id,                                                                                              // 832
        identityChanges: true                                                                                          // 833
      }, {                                                                                                             // 831
        userId: user._id,                                                                                              // 835
        identityChanges: true,                                                                                         // 836
        timestamp: new Date(),                                                                                         // 837
        isUnread: true                                                                                                 // 838
      });                                                                                                              // 834
    }                                                                                                                  // 840
  });                                                                                                                  // 841
}                                                                                                                      // 842
                                                                                                                       //
Mongo.Collection.prototype.ensureDroppedIndex = function () {                                                          // 844
  try {                                                                                                                // 845
    this._dropIndex.apply(this, arguments);                                                                            // 846
  } catch (err) {// ignore (probably, index didn't exist)                                                              // 847
  }                                                                                                                    // 849
};                                                                                                                     // 850
                                                                                                                       //
function onePersonaPerAccountPreCleanup(db, backend) {                                                                 // 852
  // Removes some already-obsolete data from the database before attempting the                                        // 853
  // one-persona-per-account migration.                                                                                // 854
  // Remove long-obsolete index.                                                                                       // 856
  Meteor.users.ensureDroppedIndex({                                                                                    // 857
    "identities.id": 1                                                                                                 // 857
  }); // Remove `stashedOldUser`, which is long-obsolete.                                                              // 857
                                                                                                                       //
  Meteor.users.update({                                                                                                // 860
    stashedOldUser: {                                                                                                  // 860
      $exists: true                                                                                                    // 860
    }                                                                                                                  // 860
  }, {                                                                                                                 // 860
    $unset: {                                                                                                          // 861
      stashedOldUser: 1                                                                                                // 861
    }                                                                                                                  // 861
  }, {                                                                                                                 // 861
    multi: true                                                                                                        // 862
  }); // Remove ApiTokens which have an identityId but not an accountId. These tokens could only exist                 // 862
  // if they were created in between `splitUserIdsIntoAccountIdsAndIdentityIds` and                                    // 865
  // `splitAccountUsersAndIdentityUsers` (late 2015), and if, during that time, the user who                           // 866
  // created the ApiToken was deleted (which, at the time, was only possible for demo users, or                        // 867
  // through direct database manipulation). These tokens are all invalid and couldn't possibly have                    // 868
  // been used since the user was deleted.                                                                             // 869
                                                                                                                       //
  db.collections.apiTokens.remove({                                                                                    // 870
    identityId: {                                                                                                      // 870
      $exists: true                                                                                                    // 870
    },                                                                                                                 // 870
    accountId: {                                                                                                       // 870
      $exists: false                                                                                                   // 870
    }                                                                                                                  // 870
  }); // Remove ApiTokens that have the obsolete owner.grain.introducerIdentity field as these tokens                  // 870
  // are not allowed to be restored anyway.                                                                            // 873
                                                                                                                       //
  db.collections.apiTokens.remove({                                                                                    // 874
    "owner.grain.introducerIdentity": {                                                                                // 874
      $exists: true                                                                                                    // 874
    }                                                                                                                  // 874
  }); // Make sure all demo credentials have a "services.demo" entry, to be consistent with all other                  // 874
  // service types.                                                                                                    // 877
                                                                                                                       //
  Meteor.users.update({                                                                                                // 878
    "profile.service": "demo"                                                                                          // 878
  }, {                                                                                                                 // 878
    $set: {                                                                                                            // 879
      "services.demo": {}                                                                                              // 879
    }                                                                                                                  // 879
  }, {                                                                                                                 // 879
    multi: true                                                                                                        // 880
  });                                                                                                                  // 880
}                                                                                                                      // 882
                                                                                                                       //
function forEachProgress(title, cursor, func) {                                                                        // 884
  console.log(title);                                                                                                  // 885
  var total = cursor.count();                                                                                          // 887
  var count = 0;                                                                                                       // 888
  cursor.forEach(function (doc) {                                                                                      // 890
    func(doc);                                                                                                         // 891
    if (++count % 100 == 0) console.log("   ", count, "/", total);                                                     // 892
  });                                                                                                                  // 893
}                                                                                                                      // 894
                                                                                                                       //
function onePersonaPerAccount(db, backend) {                                                                           // 896
  // THIS IS A MAJOR CHANGE: https://sandstorm.io/news/2017-05-08-refactoring-identities                               // 897
  console.log("** Migrating to new identity model! **");                                                               // 899
  console.log("see: https://sandstorm.io/news/2017-05-08-refactoring-identities");                                     // 900
  console.log("tagging accounts...");                                                                                  // 902
  Meteor.users.update({                                                                                                // 903
    type: {                                                                                                            // 903
      $exists: false                                                                                                   // 903
    },                                                                                                                 // 903
    loginIdentities: {                                                                                                 // 903
      $exists: true                                                                                                    // 903
    }                                                                                                                  // 903
  }, {                                                                                                                 // 903
    $set: {                                                                                                            // 904
      type: "account"                                                                                                  // 904
    }                                                                                                                  // 904
  }, {                                                                                                                 // 904
    multi: true                                                                                                        // 905
  });                                                                                                                  // 905
  console.log("tagging credentials...");                                                                               // 907
  Meteor.users.update({                                                                                                // 908
    type: {                                                                                                            // 908
      $exists: false                                                                                                   // 908
    },                                                                                                                 // 908
    profile: {                                                                                                         // 908
      $exists: true                                                                                                    // 908
    }                                                                                                                  // 908
  }, {                                                                                                                 // 908
    $set: {                                                                                                            // 909
      type: "credential"                                                                                               // 909
    }                                                                                                                  // 909
  }, {                                                                                                                 // 909
    multi: true                                                                                                        // 910
  }); // Map each identity ID to the list of connected accounts.                                                       // 910
                                                                                                                       //
  console.log("building identity map...");                                                                             // 913
  var identityToAccount = {};                                                                                          // 914
  var needSort = [];                                                                                                   // 915
  Meteor.users.find({                                                                                                  // 916
    type: "account"                                                                                                    // 916
  }).forEach(function (user) {                                                                                         // 916
    var userInfo = {                                                                                                   // 917
      id: user._id,                                                                                                    // 917
      lastActive: user.lastActive || user.createdAt                                                                    // 917
    };                                                                                                                 // 917
                                                                                                                       //
    function handleIdentity(identity) {                                                                                // 919
      if (!identityToAccount[identity.id]) {                                                                           // 920
        identityToAccount[identity.id] = [userInfo];                                                                   // 921
      } else {                                                                                                         // 922
        var list = identityToAccount[identity.id];                                                                     // 923
        if (list.length == 1) needSort.push(list);                                                                     // 924
        list.push(userInfo);                                                                                           // 925
      }                                                                                                                // 926
    }                                                                                                                  // 927
                                                                                                                       //
    user.loginIdentities.forEach(handleIdentity);                                                                      // 928
    user.nonloginIdentities.forEach(handleIdentity);                                                                   // 929
  }); // For identities attached to multiple accounts, we want the most-recently-active account to sort                // 930
  // first. In cases where we need to replace an identity with exactly one account, we'll use that                     // 933
  // first option.                                                                                                     // 934
                                                                                                                       //
  needSort.forEach(function (item) {                                                                                   // 935
    item.sort(function (a, b) {                                                                                        // 936
      if (a.lastActive > b.lastActive) {                                                                               // 937
        return -1;                                                                                                     // 938
      } else if (a.lastActive < b.lastActive) {                                                                        // 939
        return 1;                                                                                                      // 940
      } else {                                                                                                         // 941
        return 0;                                                                                                      // 942
      }                                                                                                                // 943
    });                                                                                                                // 944
  });                                                                                                                  // 945
                                                                                                                       //
  function accountForIdentity(identityId) {                                                                            // 947
    var ids = identityToAccount[identityId];                                                                           // 948
                                                                                                                       //
    if (ids) {                                                                                                         // 949
      return ids[0].id;                                                                                                // 950
    } else {                                                                                                           // 951
      console.error("no such identity:", identityId);                                                                  // 952
      return "invalid-" + identityId;                                                                                  // 953
    }                                                                                                                  // 954
  }                                                                                                                    // 955
                                                                                                                       //
  function accountListForIdentity(identityId) {                                                                        // 957
    var ids = identityToAccount[identityId];                                                                           // 958
                                                                                                                       //
    if (ids) {                                                                                                         // 959
      return ids.map(function (i) {                                                                                    // 960
        return i.id;                                                                                                   // 960
      });                                                                                                              // 960
    } else {                                                                                                           // 961
      console.error("no such identity:", identityId);                                                                  // 962
      return ["invalid-" + identityId];                                                                                // 963
    }                                                                                                                  // 964
  }                                                                                                                    // 965
                                                                                                                       //
  forEachProgress("migrating users...", Meteor.users.find({                                                            // 967
    type: "account"                                                                                                    // 968
  }), function (user) {                                                                                                // 968
    // Fetch all the user's login identities.                                                                          // 970
    var identities = Meteor.users.find({                                                                               // 971
      _id: {                                                                                                           // 972
        $in: user.loginIdentities.map(function (identity) {                                                            // 972
          return identity.id;                                                                                          // 972
        })                                                                                                             // 972
      },                                                                                                               // 972
      profile: {                                                                                                       // 973
        $exists: true                                                                                                  // 973
      }                                                                                                                // 973
    }).fetch(); // Fill out the profiles for each identity.                                                            // 972
                                                                                                                       //
    identities.forEach(function (identity) {                                                                           // 976
      SandstormDb.fillInProfileDefaults(identity, identity.profile);                                                   // 977
    }); // Find the best profile among them.                                                                           // 978
                                                                                                                       //
    var profile = void 0;                                                                                              // 981
                                                                                                                       //
    if (identities.length == 0) {                                                                                      // 982
      // no profiles???                                                                                                // 983
      profile = null;                                                                                                  // 984
    } else if (identities.length == 1) {                                                                               // 985
      profile = identities[0].profile;                                                                                 // 986
    } else {                                                                                                           // 987
      // Multi-identity user. Try to find the "best" identity.                                                         // 988
      var maxScore = -1;                                                                                               // 990
      identities.forEach(function (identity) {                                                                         // 992
        // Count total grains using this identity.                                                                     // 993
        var score = db.collections.grains.find({                                                                       // 994
          userId: user._id,                                                                                            // 995
          identityId: identity._id                                                                                     // 995
        }).count() + db.collections.apiTokens.find({                                                                   // 995
          "owner.user.identityId": identity._id                                                                        // 996
        }).count(); // Avoid choosing demo user, unless they've really used it a lot.                                  // 996
                                                                                                                       //
        if (identity.profile.name !== "Demo User") {                                                                   // 999
          score += 10;                                                                                                 // 1000
        }                                                                                                              // 1001
                                                                                                                       //
        if (score > maxScore) {                                                                                        // 1003
          profile = identity.profile;                                                                                  // 1004
          maxScore = score;                                                                                            // 1005
        }                                                                                                              // 1006
      });                                                                                                              // 1007
    }                                                                                                                  // 1008
                                                                                                                       //
    if (profile) {                                                                                                     // 1010
      delete profile.service;                                                                                          // 1011
    } else {                                                                                                           // 1012
      console.warn("no suitable profile found for user account:", user._id);                                           // 1013
      profile = {                                                                                                      // 1014
        name: "Unknown",                                                                                               // 1015
        handle: "unknown",                                                                                             // 1016
        pronoun: "neutral",                                                                                            // 1017
        identicon: Crypto.randomBytes(32).toString("hex")                                                              // 1018
      };                                                                                                               // 1014
    }                                                                                                                  // 1020
                                                                                                                       //
    var mod = {                                                                                                        // 1022
      profile: profile                                                                                                 // 1022
    }; // Also figure out referrals.                                                                                   // 1022
                                                                                                                       //
    var referrers = Meteor.users.find({                                                                                // 1025
      _id: {                                                                                                           // 1026
        $in: getUserIdentityIds(user)                                                                                  // 1026
      },                                                                                                               // 1026
      referredBy: {                                                                                                    // 1026
        $exists: true                                                                                                  // 1026
      }                                                                                                                // 1026
    }, {                                                                                                               // 1026
      fields: {                                                                                                        // 1027
        referredBy: 1                                                                                                  // 1027
      }                                                                                                                // 1027
    }).map(function (id) {                                                                                             // 1027
      return id.referredBy;                                                                                            // 1028
    });                                                                                                                // 1028
                                                                                                                       //
    if (referrers.length > 0) {                                                                                        // 1029
      mod.referredBy = referrers[0];                                                                                   // 1030
    }                                                                                                                  // 1031
                                                                                                                       //
    if (user.referredIdentityIds) {                                                                                    // 1033
      mod.referredAccountIds = user.referredIdentityIds.map(accountForIdentity);                                       // 1034
    }                                                                                                                  // 1035
                                                                                                                       //
    Meteor.users.update({                                                                                              // 1037
      _id: user._id                                                                                                    // 1037
    }, {                                                                                                               // 1037
      $set: mod                                                                                                        // 1037
    });                                                                                                                // 1037
  });                                                                                                                  // 1038
  forEachProgress("migrating ApiTokens...", db.collections.apiTokens.find({                                            // 1040
    "owner.user.identityId": {                                                                                         // 1041
      $exists: true                                                                                                    // 1041
    }                                                                                                                  // 1041
  }), function (token) {                                                                                               // 1041
    var accounts = accountListForIdentity(token.owner.user.identityId);                                                // 1043
    db.collections.apiTokens.update({                                                                                  // 1045
      _id: token._id                                                                                                   // 1045
    }, {                                                                                                               // 1045
      $set: {                                                                                                          // 1046
        "owner.user.accountId": accounts[0]                                                                            // 1046
      }                                                                                                                // 1046
    });                                                                                                                // 1046
                                                                                                                       //
    if (accounts.length > 1 && token.grainId && token.accountId) {                                                     // 1048
      // Shared to an identity that has multiple accounts. We need to denormalize the share to                         // 1049
      // target each account individually.                                                                             // 1050
      //                                                                                                               // 1051
      // Note: A token with owner.user.identityId is always a UiView share token and so always has                     // 1052
      //   accountId and grainId. There are zero counterexamples in Oasis. But if a counterexample                     // 1053
      //   existed, the code below might compound the confusion, so we skip it.                                        // 1054
      delete token._id;                                                                                                // 1056
      accounts.slice(1).forEach(function (account) {                                                                   // 1057
        // For idempotency purposes, don't insert if a similar token already exists.                                   // 1058
        if (!db.collections.apiTokens.findOne({                                                                        // 1059
          grainId: token.grainId,                                                                                      // 1060
          accountId: token.accountId,                                                                                  // 1061
          "owner.user.accountId": account                                                                              // 1062
        })) {                                                                                                          // 1059
          token.owner.user.accountId = account;                                                                        // 1064
          db.collections.apiTokens.insert(token);                                                                      // 1065
        }                                                                                                              // 1066
      });                                                                                                              // 1067
    }                                                                                                                  // 1068
  });                                                                                                                  // 1069
  forEachProgress("migrating membrane requirements...", db.collections.apiTokens.find({                                // 1071
    "requirements.permissionsHeld.identityId": {                                                                       // 1072
      $exists: true                                                                                                    // 1072
    }                                                                                                                  // 1072
  }), function (token) {                                                                                               // 1072
    token.requirements.forEach(function (requirement) {                                                                // 1074
      if (requirement.permissionsHeld && requirement.permissionsHeld.identityId) {                                     // 1075
        requirement.permissionsHeld.accountId = accountForIdentity(requirement.permissionsHeld.identityId);            // 1076
      }                                                                                                                // 1078
    });                                                                                                                // 1079
    db.collections.apiTokens.update({                                                                                  // 1081
      _id: token._id                                                                                                   // 1081
    }, {                                                                                                               // 1081
      $set: {                                                                                                          // 1082
        requirements: token.requirements                                                                               // 1082
      }                                                                                                                // 1082
    });                                                                                                                // 1082
  });                                                                                                                  // 1083
  forEachProgress("migrating identity capabilities...", db.collections.apiTokens.find({                                // 1085
    "frontendRef.identity": {                                                                                          // 1086
      $exists: true                                                                                                    // 1086
    }                                                                                                                  // 1086
  }), function (token) {                                                                                               // 1086
    db.collections.apiTokens.update({                                                                                  // 1088
      _id: token._id                                                                                                   // 1088
    }, {                                                                                                               // 1088
      $set: {                                                                                                          // 1089
        "frontendRef.identity": accountForIdentity(token.frontendRef.identity)                                         // 1089
      }                                                                                                                // 1089
    });                                                                                                                // 1089
  });                                                                                                                  // 1090
  forEachProgress("migrating contacts...", db.collections.contacts.find(), function (contact) {                        // 1092
    db.collections.contacts.update({                                                                                   // 1095
      _id: contact._id                                                                                                 // 1095
    }, {                                                                                                               // 1095
      $set: {                                                                                                          // 1096
        accountId: accountForIdentity(contact.identityId)                                                              // 1096
      }                                                                                                                // 1096
    });                                                                                                                // 1096
  });                                                                                                                  // 1097
  forEachProgress("migrating notifications...", db.collections.notifications.find({                                    // 1099
    initiatingIdentity: {                                                                                              // 1100
      $exists: true                                                                                                    // 1100
    }                                                                                                                  // 1100
  }), function (notification) {                                                                                        // 1100
    db.collections.notifications.update({                                                                              // 1102
      _id: notification._id                                                                                            // 1102
    }, {                                                                                                               // 1102
      $set: {                                                                                                          // 1103
        initiatingAccount: accountForIdentity(notification.initiatingIdentity)                                         // 1103
      }                                                                                                                // 1103
    });                                                                                                                // 1103
  });                                                                                                                  // 1104
  forEachProgress("migrating desktop notifications...", db.collections.desktopNotifications.find({                     // 1106
    "appActivity.user.identityId": {                                                                                   // 1108
      $exists: true                                                                                                    // 1108
    }                                                                                                                  // 1108
  }), function (notification) {                                                                                        // 1108
    db.collections.desktopNotifications.update({                                                                       // 1110
      _id: notification._id                                                                                            // 1110
    }, {                                                                                                               // 1110
      $set: {                                                                                                          // 1111
        "appActivity.user.accountId": accountForIdentity(notification.appActivity.user.identityId)                     // 1111
      }                                                                                                                // 1111
    });                                                                                                                // 1111
  });                                                                                                                  // 1113
  forEachProgress("migrating subscriptions...", db.collections.activitySubscriptions.find({                            // 1115
    identityId: {                                                                                                      // 1116
      $exists: true                                                                                                    // 1116
    }                                                                                                                  // 1116
  }), function (subscription) {                                                                                        // 1116
    db.collections.activitySubscriptions.update({                                                                      // 1118
      _id: subscription._id                                                                                            // 1118
    }, {                                                                                                               // 1118
      $set: {                                                                                                          // 1119
        accountId: accountForIdentity(subscription.identityId)                                                         // 1119
      }                                                                                                                // 1119
    });                                                                                                                // 1119
  });                                                                                                                  // 1120
}                                                                                                                      // 1121
                                                                                                                       //
function onePersonaPerAccountPostCleanup(db, backend) {                                                                // 1123
  // Drop obsolete indices.                                                                                            // 1124
  db.collections.apiTokens.ensureDroppedIndex({                                                                        // 1125
    "owner.user.identityId": 1                                                                                         // 1125
  });                                                                                                                  // 1125
  db.collections.activitySubscriptions.ensureDroppedIndex({                                                            // 1126
    "identityId": 1                                                                                                    // 1126
  });                                                                                                                  // 1126
  Meteor.users.ensureDroppedIndex({                                                                                    // 1127
    "loginIdentities.id": 1                                                                                            // 1127
  });                                                                                                                  // 1127
  Meteor.users.ensureDroppedIndex({                                                                                    // 1128
    "nonloginIdentities.id": 1                                                                                         // 1128
  });                                                                                                                  // 1128
  Meteor.users.update({                                                                                                // 1130
    type: "account"                                                                                                    // 1130
  }, {                                                                                                                 // 1130
    $rename: {                                                                                                         // 1131
      loginIdentities: "loginCredentials",                                                                             // 1131
      nonloginIdentities: "nonloginCredentials"                                                                        // 1132
    },                                                                                                                 // 1131
    $unset: {                                                                                                          // 1133
      referredIdentityIds: 1                                                                                           // 1133
    }                                                                                                                  // 1133
  }, {                                                                                                                 // 1131
    multi: true                                                                                                        // 1134
  }); // Note that we intentionally don't unset profiles from credentials for now, as this is a place                  // 1134
  // where we could legitimately be losing data since only of the user's identity profiles gets                        // 1137
  // promoted to their account. We could remove them in a later pass, once we're sure we no longer                     // 1138
  // need that data. (Also unclear if Meteor's accounts system will get confused if profiles                           // 1139
  // disappear...) (Also SansdtormDb.fillInIntrinsicName() is specific to credentials and still                        // 1140
  // reads the profile a bit...)                                                                                       // 1141
                                                                                                                       //
  db.collections.notifications.update({}, {                                                                            // 1143
    $unset: {                                                                                                          // 1144
      initiatingIdentity: 1                                                                                            // 1144
    }                                                                                                                  // 1144
  }, {                                                                                                                 // 1144
    multi: true                                                                                                        // 1145
  });                                                                                                                  // 1145
  db.collections.apiTokens.update({                                                                                    // 1147
    identityId: {                                                                                                      // 1147
      $exists: true                                                                                                    // 1147
    }                                                                                                                  // 1147
  }, {                                                                                                                 // 1147
    $unset: {                                                                                                          // 1148
      identityId: 1                                                                                                    // 1148
    }                                                                                                                  // 1148
  }, {                                                                                                                 // 1148
    multi: true                                                                                                        // 1149
  });                                                                                                                  // 1149
  db.collections.apiTokens.update({                                                                                    // 1151
    "requirements.permissionsHeld.identityId": {                                                                       // 1151
      $exists: true                                                                                                    // 1151
    }                                                                                                                  // 1151
  }, {                                                                                                                 // 1151
    $unset: {                                                                                                          // 1152
      "requirements.permissionsHeld.identityId": 1                                                                     // 1152
    }                                                                                                                  // 1152
  }, {                                                                                                                 // 1152
    multi: true                                                                                                        // 1153
  });                                                                                                                  // 1153
} // TODO(cleanup): Delete profiles from credentials. (Make sure nothing depends on them.)                             // 1154
// TODO(cleanup): Delete all demo credentials since they aren't really needed anymore. Remove them                     // 1157
//   from associated account nonloginCredentials.                                                                      // 1158
// This must come after all the functions named within are defined.                                                    // 1160
// Only append to this list!  Do not modify or remove list entries;                                                    // 1161
// doing so is likely change the meaning and semantics of user databases.                                              // 1162
                                                                                                                       //
                                                                                                                       //
var MIGRATIONS = [updateLoginStyleToRedirect, enableLegacyOAuthProvidersIfNotInSettings, denormalizeInviteInfo, mergeRoleAssignmentsIntoApiTokens, fixOasisStorageUsageStats, fetchProfilePictures, assignPlans, removeKeyrings, useLocalizedTextInUserActions, verifyAllPgpSignatures, splitUserIdsIntoAccountIdsAndIdentityIds, appUpdateSettings, moveDevAndEmailLoginDataIntoIdentities, repairEmailIdentityIds, splitAccountUsersAndIdentityUsers, populateContactsFromApiTokens, cleanUpApiTokens, initServerTitleAndReturnAddress, sendReferralNotifications, assignBonuses, splitSmtpUrl, smtpPortShouldBeNumber, consolidateOrgSettings, unsetSmtpDefaultHostnameIfNoUsersExist, extractLastUsedFromApiTokenOwner, setUpstreamTitles, markAllRead, clearAppIndex, assignEmailVerifierIds, setNewServer, addMembraneRequirementsToIdentities, addEncryptionToFrontendRefIpNetwork, setIpBlacklist, notifyIdentityChanges, onePersonaPerAccountPreCleanup, onePersonaPerAccount, onePersonaPerAccountPostCleanup];
var NEW_SERVER_STARTUP = [startPreinstallingApps];                                                                     // 1203
                                                                                                                       //
var migrateToLatest = function (db, backend) {                                                                         // 1207
  if (Meteor.settings.replicaNumber) {                                                                                 // 1208
    // This is a replica. Wait for the first replica to perform migrations.                                            // 1209
    console.log("Waiting for migrations on replica zero...");                                                          // 1211
    var done = new Future();                                                                                           // 1213
                                                                                                                       //
    var change = function (doc) {                                                                                      // 1214
      console.log("Migrations applied elsewhere: " + doc.value + "/" + MIGRATIONS.length);                             // 1215
      if (doc.value >= MIGRATIONS.length) done.return();                                                               // 1216
    };                                                                                                                 // 1217
                                                                                                                       //
    var observer = db.collections.migrations.find({                                                                    // 1219
      _id: "migrations_applied"                                                                                        // 1219
    }).observe({                                                                                                       // 1219
      added: change,                                                                                                   // 1220
      changed: change                                                                                                  // 1221
    });                                                                                                                // 1219
    var newServerDone = new Future();                                                                                  // 1224
                                                                                                                       //
    var newServerChange = function (doc) {                                                                             // 1225
      if (doc.value) {                                                                                                 // 1226
        console.log("New server migrations applied elsewhere");                                                        // 1227
        newServerDone.return();                                                                                        // 1228
      }                                                                                                                // 1229
    };                                                                                                                 // 1230
                                                                                                                       //
    var newServerObserver = db.collections.migrations.find({                                                           // 1232
      _id: "new_server_migrations_applied"                                                                             // 1232
    }).observe({                                                                                                       // 1232
      added: newServerChange,                                                                                          // 1233
      changed: newServerChange                                                                                         // 1234
    });                                                                                                                // 1232
    done.wait();                                                                                                       // 1237
    observer.stop();                                                                                                   // 1238
    newServerDone.wait();                                                                                              // 1239
    newServerObserver.stop();                                                                                          // 1240
    console.log("Migrations have completed on replica zero.");                                                         // 1241
  } else {                                                                                                             // 1242
    var applied = db.collections.migrations.findOne({                                                                  // 1243
      _id: "migrations_applied"                                                                                        // 1243
    });                                                                                                                // 1243
    var start = void 0;                                                                                                // 1244
                                                                                                                       //
    if (!applied) {                                                                                                    // 1245
      // Migrations table is not yet seeded with a value.  This means it has                                           // 1246
      // applied 0 migrations.  Persist this.                                                                          // 1247
      db.collections.migrations.insert({                                                                               // 1248
        _id: "migrations_applied",                                                                                     // 1248
        value: 0                                                                                                       // 1248
      });                                                                                                              // 1248
      start = 0; // This also means this is a brand new server                                                         // 1249
                                                                                                                       //
      db.collections.migrations.insert({                                                                               // 1252
        _id: "new_server_migrations_applied",                                                                          // 1252
        value: false                                                                                                   // 1252
      });                                                                                                              // 1252
    } else {                                                                                                           // 1253
      start = applied.value;                                                                                           // 1254
    }                                                                                                                  // 1255
                                                                                                                       //
    console.log("Migrations already applied: " + start + "/" + MIGRATIONS.length);                                     // 1257
                                                                                                                       //
    for (var i = start; i < MIGRATIONS.length; i++) {                                                                  // 1259
      // Apply migration i, then record that migration i was successfully run.                                         // 1260
      console.log("Applying migration " + (i + 1));                                                                    // 1261
      MIGRATIONS[i](db, backend);                                                                                      // 1262
      db.collections.migrations.update({                                                                               // 1263
        _id: "migrations_applied"                                                                                      // 1263
      }, {                                                                                                             // 1263
        $set: {                                                                                                        // 1263
          value: i + 1                                                                                                 // 1263
        }                                                                                                              // 1263
      });                                                                                                              // 1263
      console.log("Applied migration " + (i + 1));                                                                     // 1264
    }                                                                                                                  // 1265
                                                                                                                       //
    if (!db.collections.migrations.findOne({                                                                           // 1267
      _id: "new_server_migrations_applied"                                                                             // 1267
    }).value) {                                                                                                        // 1267
      // new_server_migrations_applied is guaranteed to exist since we have a migration that                           // 1268
      // ensures it.                                                                                                   // 1269
      for (var _i = 0; _i < NEW_SERVER_STARTUP.length; _i++) {                                                         // 1270
        console.log("Running new server startup function " + (_i + 1));                                                // 1271
                                                                                                                       //
        NEW_SERVER_STARTUP[_i](db, backend);                                                                           // 1272
                                                                                                                       //
        console.log("Running new server startup function " + (_i + 1));                                                // 1273
      }                                                                                                                // 1274
                                                                                                                       //
      db.collections.migrations.update({                                                                               // 1276
        _id: "new_server_migrations_applied"                                                                           // 1276
      }, {                                                                                                             // 1276
        $set: {                                                                                                        // 1276
          value: true                                                                                                  // 1276
        }                                                                                                              // 1276
      });                                                                                                              // 1276
    } // Start background migrations.                                                                                  // 1277
                                                                                                                       //
                                                                                                                       //
    backgroundFillInGrainSizes(db, backend);                                                                           // 1280
  }                                                                                                                    // 1281
};                                                                                                                     // 1282
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"networking.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/networking.js                                                                                        //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _typeof2 = require("babel-runtime/helpers/typeof");                                                                //
                                                                                                                       //
var _typeof3 = _interopRequireDefault(_typeof2);                                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var module1 = module;                                                                                                  // 1
module1.export({                                                                                                       // 1
  ssrfSafeLookup: function () {                                                                                        // 1
    return ssrfSafeLookup;                                                                                             // 1
  },                                                                                                                   // 1
  ssrfSafeLookupOrProxy: function () {                                                                                 // 1
    return ssrfSafeLookupOrProxy;                                                                                      // 1
  },                                                                                                                   // 1
  monkeyPatchHttp: function () {                                                                                       // 1
    return monkeyPatchHttp;                                                                                            // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var SPECIAL_IPV4_ADDRESSES = void 0,                                                                                   // 1
    SPECIAL_IPV6_ADDRESSES = void 0;                                                                                   // 1
module1.watch(require("/imports/constants.js"), {                                                                      // 1
  SPECIAL_IPV4_ADDRESSES: function (v) {                                                                               // 1
    SPECIAL_IPV4_ADDRESSES = v;                                                                                        // 1
  },                                                                                                                   // 1
  SPECIAL_IPV6_ADDRESSES: function (v) {                                                                               // 1
    SPECIAL_IPV6_ADDRESSES = v;                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var Dns = Npm.require("dns");                                                                                          // 19
                                                                                                                       //
var Ip = Npm.require("ip");                                                                                            // 20
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 21
                                                                                                                       //
var lookupInFiber = Meteor.wrapAsync(Dns.lookup, Dns);                                                                 // 23
                                                                                                                       //
function getNetwork(addr, bits) {                                                                                      // 25
  // npm ip's "mask" and "cidr" functions are broken for ipv6. :(                                                      // 26
  var parsed = Ip.toBuffer(addr);                                                                                      // 28
                                                                                                                       //
  for (var i = Math.ceil(bits / 8); i < parsed.length; i++) {                                                          // 30
    parsed[i] = 0;                                                                                                     // 31
  }                                                                                                                    // 32
                                                                                                                       //
  var n = Math.floor(bits / 8);                                                                                        // 34
                                                                                                                       //
  if (n < parsed.length) {                                                                                             // 35
    parsed[n] = parsed[n] & 0xff << 8 - bits % 8;                                                                      // 36
  }                                                                                                                    // 37
                                                                                                                       //
  return parsed;                                                                                                       // 39
}                                                                                                                      // 40
                                                                                                                       //
function parseCidr(cidr) {                                                                                             // 42
  // Given a CIDR-format network specification, return a function which, given an address string,                      // 43
  // returns true if the address is in this network.                                                                   // 44
  // The "ip" NPM module's CIDR handling unfortunately is very broken for IPv6. Many bugs have been                    // 46
  // filed but it remains broken.                                                                                      // 47
  try {                                                                                                                // 49
    cidr = cidr.trim();                                                                                                // 50
    if (cidr === "") return null;                                                                                      // 51
    var parts = cidr.split("/");                                                                                       // 53
                                                                                                                       //
    if (parts.length === 1) {                                                                                          // 54
      // Bare address.                                                                                                 // 55
      return function (addr) {                                                                                         // 56
        return Ip.isEqual(cidr, addr);                                                                                 // 56
      };                                                                                                               // 56
    } else if (parts.length === 2) {                                                                                   // 57
      var bits = parseInt(parts[1], 10);                                                                               // 58
      if (bits !== bits) throw new Error("value after slash must be an integer");                                      // 59
      var network = getNetwork(parts[0], bits);                                                                        // 60
      return function (addr) {                                                                                         // 61
        return network.equals(getNetwork(addr, bits));                                                                 // 62
      };                                                                                                               // 63
    } else {                                                                                                           // 64
      throw new Error("too many slashes");                                                                             // 65
    }                                                                                                                  // 66
  } catch (err) {                                                                                                      // 67
    console.error("invalid network specification in IP blacklist:", cidr, err);                                        // 68
    return null;                                                                                                       // 69
  }                                                                                                                    // 70
}                                                                                                                      // 71
                                                                                                                       //
SPECIAL_FILTERS = SPECIAL_IPV4_ADDRESSES.concat(SPECIAL_IPV6_ADDRESSES).map(parseCidr);                                // 73
                                                                                                                       //
function ssrfSafeLookup(db, url) {                                                                                     // 75
  // Given an HTTP/HTTPS URL, look up the hostname, verify it doesn't point to a blacklisted IP,                       // 76
  // then return an object of {url, host}, where `url` has the original hostname substituted with                      // 77
  // an IP address, and `host` is the original hostname suitable for sending in the `Host` header.                     // 78
  var parsedUrl = Url.parse(url);                                                                                      // 80
                                                                                                                       //
  if (parsedUrl.protocol !== "http:" && parsedUrl.protocol !== "https:") {                                             // 82
    throw new Error("not an HTTP nor HTTPS URL: " + url);                                                              // 83
  }                                                                                                                    // 84
                                                                                                                       //
  var addresses = lookupInFiber(parsedUrl.hostname, {                                                                  // 86
    all: true,                                                                                                         // 86
    hints: Dns.ADDRCONFIG                                                                                              // 86
  }); // TODO(perf): Subscribe to blacklist changes so that we don't have to do a new lookup and                       // 86
  //   parse each time.                                                                                                // 89
                                                                                                                       //
  var blacklist = db.getSettingWithFallback("ipBlacklist", "").split("\n").map(parseCidr).filter(function (x) {        // 90
    return x;                                                                                                          // 91
  });                                                                                                                  // 91
                                                                                                                       //
  var _loop = function (i) {                                                                                           // 75
    var address = addresses[i];                                                                                        // 94
    if (address.family !== 4 && address.family !== 6) return "continue";                                               // 95
    var ok = true;                                                                                                     // 97
    blacklist.forEach(function (test) {                                                                                // 98
      if (test(address.address)) {                                                                                     // 98
        ok = false;                                                                                                    // 98
      }                                                                                                                // 98
    });                                                                                                                // 98
    SPECIAL_FILTERS.forEach(function (test) {                                                                          // 100
      if (test(address.address)) {                                                                                     // 100
        ok = false;                                                                                                    // 100
      }                                                                                                                // 100
    });                                                                                                                // 100
                                                                                                                       //
    if (ok) {                                                                                                          // 102
      var host = parsedUrl.host;                                                                                       // 103
      delete parsedUrl.host;                                                                                           // 104
      parsedUrl.hostname = address.address;                                                                            // 105
      return {                                                                                                         // 106
        v: {                                                                                                           // 106
          url: Url.format(parsedUrl),                                                                                  // 106
          host: host                                                                                                   // 106
        }                                                                                                              // 106
      };                                                                                                               // 106
    }                                                                                                                  // 107
  };                                                                                                                   // 75
                                                                                                                       //
  for (var i in meteorBabelHelpers.sanitizeForInObject(addresses)) {                                                   // 93
    var _ret = _loop(i);                                                                                               // 93
                                                                                                                       //
    switch (_ret) {                                                                                                    // 93
      case "continue":                                                                                                 // 93
        continue;                                                                                                      // 95
                                                                                                                       //
      default:                                                                                                         // 93
        if ((typeof _ret === "undefined" ? "undefined" : (0, _typeof3.default)(_ret)) === "object") return _ret.v;     // 93
    }                                                                                                                  // 93
  }                                                                                                                    // 108
                                                                                                                       //
  if (addresses.length > 0) {                                                                                          // 110
    throw new Meteor.Error(403, "can't connect to blacklisted private network address: " + parsedUrl.hostname + "; the Sandstorm server admin can change the blacklist in the admin settings");
  } else {                                                                                                             // 114
    throw new Meteor.Error(404, "host not found: " + parsedUrl.hostname);                                              // 115
  }                                                                                                                    // 116
}                                                                                                                      // 117
                                                                                                                       //
function ssrfSafeLookupOrProxy(db, url) {                                                                              // 119
  // If there is an HTTP proxy, then it will have to do the work of blacklisting IPs, because it's                     // 120
  // the proxy that does the DNS lookup.                                                                               // 121
  var httpProxy = process.env.HTTP_PROXY || process.env.http_proxy;                                                    // 122
  var httpsProxy = process.env.HTTPS_PROXY || process.env.https_proxy;                                                 // 123
                                                                                                                       //
  if (httpProxy && url.startsWith("http:")) {                                                                          // 125
    return {                                                                                                           // 126
      proxy: httpProxy                                                                                                 // 126
    };                                                                                                                 // 126
  } else if (httpsProxy && url.startsWith("https:")) {                                                                 // 127
    return {                                                                                                           // 128
      proxy: httpsProxy                                                                                                // 128
    };                                                                                                                 // 128
  } else {                                                                                                             // 129
    return ssrfSafeLookup(db, url);                                                                                    // 130
  }                                                                                                                    // 131
}                                                                                                                      // 132
                                                                                                                       //
function ssrfSafeHttp(originalHttpCall, db, method, url, options, callback) {                                          // 134
  if (typeof options === "function") {                                                                                 // 135
    callback = options;                                                                                                // 136
    options = undefined;                                                                                               // 137
  }                                                                                                                    // 138
                                                                                                                       //
  if (!options) options = {};                                                                                          // 140
                                                                                                                       //
  if (options.npmRequestOptions && options.npmRequestOptions.proxy) {                                                  // 142
    // Request already specifies a different proxy.                                                                    // 143
    return originalHttpCall(method, url, options, callback);                                                           // 144
  }                                                                                                                    // 145
                                                                                                                       //
  var safe = ssrfSafeLookupOrProxy(db, url);                                                                           // 147
                                                                                                                       //
  if (safe.proxy) {                                                                                                    // 149
    if (!options.npmRequestOptions) options.npmRequestOptions = {};                                                    // 150
    options.npmRequestOptions.proxy = safe.proxy;                                                                      // 151
    return originalHttpCall(method, url, options, callback);                                                           // 152
  } else {                                                                                                             // 153
    var _safe = ssrfSafeLookup(db, url);                                                                               // 154
                                                                                                                       //
    if (!options.headers) options.headers = {};                                                                        // 155
    options.headers.host = _safe.host;                                                                                 // 156
    options.servername = _safe.host.split(":")[0];                                                                     // 157
    return originalHttpCall(method, _safe.url, options, callback);                                                     // 158
  }                                                                                                                    // 159
}                                                                                                                      // 160
                                                                                                                       //
function monkeyPatchHttp(db, HTTP) {                                                                                   // 162
  var original = HTTP.call.bind(HTTP);                                                                                 // 163
  HTTP.call = ssrfSafeHttp.bind(this, original, db);                                                                   // 164
}                                                                                                                      // 165
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"observe-helpers.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/observe-helpers.js                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  fillUndefinedForChangedDoc: function () {                                                                            // 1
    return fillUndefinedForChangedDoc;                                                                                 // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
function fillUndefinedForChangedDoc(newDoc, oldDoc) {                                                                  // 17
  // In publishes, setting a field to `undefined` specifies that it is no longer present.                              // 18
  // Annoyingly, in the changed callback of observing a collection, it does not do this for you.                       // 19
  Object.keys(oldDoc).forEach(function (key) {                                                                         // 20
    if (!(key in newDoc)) {                                                                                            // 21
      newDoc[key] = undefined;                                                                                         // 22
    }                                                                                                                  // 23
  });                                                                                                                  // 24
}                                                                                                                      // 25
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"persistent.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/persistent.js                                                                                        //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
module.export({                                                                                                        // 1
  PersistentImpl: function () {                                                                                        // 1
    return PersistentImpl;                                                                                             // 1
  },                                                                                                                   // 1
  hashSturdyRef: function () {                                                                                         // 1
    return hashSturdyRef;                                                                                              // 1
  },                                                                                                                   // 1
  generateSturdyRef: function () {                                                                                     // 1
    return generateSturdyRef;                                                                                          // 1
  },                                                                                                                   // 1
  checkRequirements: function () {                                                                                     // 1
    return checkRequirements;                                                                                          // 1
  },                                                                                                                   // 1
  fetchApiToken: function () {                                                                                         // 1
    return fetchApiToken;                                                                                              // 1
  },                                                                                                                   // 1
  insertApiToken: function () {                                                                                        // 1
    return insertApiToken;                                                                                             // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
var inMeteor = void 0;                                                                                                 // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 19
                                                                                                                       //
var privateDb = Symbol("PersistentImpl.db");                                                                           // 21
var privateTemplate = Symbol("PersistentImpl.template");                                                               // 22
var privateIsSaved = Symbol("PersistentImpl.isSaved");                                                                 // 23
                                                                                                                       //
var PersistentImpl = function () {                                                                                     //
  function PersistentImpl(db, saveTemplate) {                                                                          // 26
    (0, _classCallCheck3.default)(this, PersistentImpl);                                                               // 26
    this[privateDb] = db;                                                                                              // 27
    this[privateTemplate] = saveTemplate; // TODO(someday): This may or may not be quite right in cases where we're saving a root token
    //   by copy. Is the copy the "same" capability or not? Today, isSaved() is only used by                           // 31
    //   wakelock notification handles which are not issued through the powerbox, therefore do not                     // 32
    //   go through claimRequest(), therefore are not saved by copy, so the point is moot.                             // 33
                                                                                                                       //
    this[privateIsSaved] = !!saveTemplate.parentToken;                                                                 // 34
  }                                                                                                                    // 35
                                                                                                                       //
  PersistentImpl.prototype.isSaved = function () {                                                                     //
    function isSaved() {                                                                                               //
      // Can be called by subclasses to ask whether or not a record currently exists in the ApiTokens                  // 38
      // table representing this capability.                                                                           // 39
      return this[privateIsSaved];                                                                                     // 41
    }                                                                                                                  // 42
                                                                                                                       //
    return isSaved;                                                                                                    //
  }();                                                                                                                 //
                                                                                                                       //
  PersistentImpl.prototype.save = function () {                                                                        //
    function save(params) {                                                                                            //
      var _this = this;                                                                                                // 44
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 45
        if (!params.sealFor) {                                                                                         // 46
          throw new Error("must specify 'sealFor'");                                                                   // 47
        }                                                                                                              // 48
                                                                                                                       //
        var db = _this[privateDb];                                                                                     // 50
                                                                                                                       //
        var newToken = _.clone(_this[privateTemplate]);                                                                // 52
                                                                                                                       //
        newToken.owner = params.sealFor;                                                                               // 53
                                                                                                                       //
        if (newToken.owner.user) {                                                                                     // 54
          if (!newToken.accountId || !newToken.grainId) {                                                              // 55
            throw new Error("can't save non-UiView with user as owner");                                               // 56
          } // Only "accountId" and "title" are allowed to be passed to save().                                        // 57
                                                                                                                       //
                                                                                                                       //
          var userOwner = _.pick(newToken.owner.user, "accountId", "title");                                           // 60
                                                                                                                       //
          var grain = db.getGrain(newToken.grainId);                                                                   // 62
                                                                                                                       //
          if (!grain) {                                                                                                // 63
            throw new Error("unknown grain ID");                                                                       // 64
          }                                                                                                            // 65
                                                                                                                       //
          userOwner.identityId = db.getOrGenerateIdentityId(userOwner.accountId, grain); // Fill in denormalizedGrainMetadata and upstreamTitle ourselves.
                                                                                                                       //
          userOwner.denormalizedGrainMetadata = db.getDenormalizedGrainInfo(newToken.grainId);                         // 70
                                                                                                                       //
          if (grain.title !== userOwner.title) {                                                                       // 72
            userOwner.upstreamTitle = grain.title;                                                                     // 73
          }                                                                                                            // 74
                                                                                                                       //
          newToken.owner.user = userOwner;                                                                             // 76
        }                                                                                                              // 77
                                                                                                                       //
        newToken.created = new Date();                                                                                 // 79
        var sturdyRef = insertApiToken(db, newToken);                                                                  // 80
        _this[privateIsSaved] = true;                                                                                  // 82
        return {                                                                                                       // 83
          sturdyRef: new Buffer(sturdyRef)                                                                             // 83
        };                                                                                                             // 83
      });                                                                                                              // 84
    }                                                                                                                  // 85
                                                                                                                       //
    return save;                                                                                                       //
  }(); // TODO(someday): Implement SystemPersistent.addRequirements().                                                 //
                                                                                                                       //
                                                                                                                       //
  return PersistentImpl;                                                                                               //
}();                                                                                                                   //
                                                                                                                       //
function hashSturdyRef(sturdyRef) {                                                                                    // 90
  return Crypto.createHash("sha256").update(sturdyRef).digest("base64");                                               // 91
}                                                                                                                      // 92
                                                                                                                       //
;                                                                                                                      // 92
                                                                                                                       //
function generateSturdyRef() {                                                                                         // 94
  return Random.secret();                                                                                              // 95
}                                                                                                                      // 96
                                                                                                                       //
function cryptApiToken(key, entry, cryptIn, cryptOut) {                                                                // 98
  // Encrypts or decrypts all fields of an ApiToken.                                                                   // 99
  // `cryptIn` translates a token in the input to a buffer.                                                            // 100
  // `cryptOut` translates a buffer to a token in the output.                                                          // 101
  check(key, String);                                                                                                  // 103
  check(entry, Object);                                                                                                // 104
  var nonce0 = new Buffer(8);                                                                                          // 106
  nonce0.fill(0);                                                                                                      // 107
  var keyBuf = new Buffer(32);                                                                                         // 109
  keyBuf.fill(0);                                                                                                      // 110
  keyBuf.write(key, 0, 32, "base64");                                                                                  // 111
                                                                                                                       //
  function encrypt0(token) {                                                                                           // 113
    return cryptOut(Capnp.chacha20(cryptIn(token), nonce0, keyBuf));                                                   // 114
  }                                                                                                                    // 115
                                                                                                                       //
  if (entry.parentTokenKey) {                                                                                          // 117
    entry.parentTokenKey = encrypt0(entry.parentTokenKey);                                                             // 118
  } else if (entry.frontendRef && entry.frontendRef.http) {                                                            // 119
    var http = entry.frontendRef.http;                                                                                 // 120
                                                                                                                       //
    if (http.auth) {                                                                                                   // 121
      var auth = http.auth;                                                                                            // 122
                                                                                                                       //
      if (auth.bearer) {                                                                                               // 123
        auth.bearer = encrypt0(auth.bearer);                                                                           // 124
      } else if (auth.basic) {                                                                                         // 125
        auth.basic.password = encrypt0(auth.basic.password);                                                           // 126
      } else if (auth.refresh) {                                                                                       // 127
        auth.refresh = encrypt0(auth.refresh);                                                                         // 128
      }                                                                                                                // 129
    }                                                                                                                  // 130
  }                                                                                                                    // 131
}                                                                                                                      // 132
                                                                                                                       //
function fetchApiToken(db, key, moreQuery) {                                                                           // 134
  // Reads an ApiToken from the database and decrypts its encrypted fields.                                            // 135
  function bufferToString(buf) {                                                                                       // 137
    // un-pad short secrets                                                                                            // 138
    var size = buf.length;                                                                                             // 139
                                                                                                                       //
    while (size > 0 && buf[size - 1] == 0) {                                                                           // 140
      --size;                                                                                                          // 141
    }                                                                                                                  // 142
                                                                                                                       //
    return buf.slice(0, size).toString("utf8");                                                                        // 144
  }                                                                                                                    // 145
                                                                                                                       //
  var query = {                                                                                                        // 147
    _id: hashSturdyRef(key)                                                                                            // 147
  };                                                                                                                   // 147
  Object.assign(query, moreQuery || {});                                                                               // 148
  var entry = db.collections.apiTokens.findOne(query);                                                                 // 149
                                                                                                                       //
  if (entry) {                                                                                                         // 150
    cryptApiToken(key, entry, function (x) {                                                                           // 151
      return x;                                                                                                        // 151
    }, bufferToString);                                                                                                // 151
  }                                                                                                                    // 152
                                                                                                                       //
  return entry;                                                                                                        // 154
}                                                                                                                      // 155
                                                                                                                       //
function insertApiToken(db, entry, key) {                                                                              // 157
  // Adds a new ApiToken to the database. `key`, if specified, *must* be a base64-encoded 256-bit                      // 158
  // value. If omitted, a key will be generated. Either way, the key is returned, and entry._id                        // 159
  // is filled in. Also, as a side effect, some fields of `entry` will become encrypted, but                           // 160
  // ideally callers should not depend on this behavior.                                                               // 161
  function stringToBuffer(str) {                                                                                       // 163
    var buf = new Buffer(str, "utf8");                                                                                 // 164
    if (buf.length >= 32) return buf;                                                                                  // 165
    var padded = new Buffer(32);                                                                                       // 167
    padded.fill(0);                                                                                                    // 168
    buf.copy(padded);                                                                                                  // 169
    return padded;                                                                                                     // 170
  }                                                                                                                    // 171
                                                                                                                       //
  if (!key) key = generateSturdyRef();                                                                                 // 173
  entry._id = hashSturdyRef(key);                                                                                      // 174
  cryptApiToken(key, entry, stringToBuffer, function (x) {                                                             // 175
    return x;                                                                                                          // 175
  });                                                                                                                  // 175
  db.collections.apiTokens.insert(entry);                                                                              // 176
  return key;                                                                                                          // 177
}                                                                                                                      // 178
                                                                                                                       //
function checkRequirements(db, requirements) {                                                                         // 180
  // Checks if the given list of MembraneRequirements are all satisfied, returning true if so and                      // 181
  // false otherwise.                                                                                                  // 182
  // TODO(cleanup): SandstormPermissions has a different way of checking the same requirements.                        // 184
  //   Reuse?                                                                                                          // 185
  // TODO(security): Eventually we want checkRequirements(), when it passes, to produce an observer                    // 187
  //   that can be used to receive a notification when the requirements may no longer be satisfied,                    // 188
  //   in order to revoke the live object. See RequirementObserver in supervisor.capnp.                                // 189
  if (!requirements) {                                                                                                 // 191
    return true;                                                                                                       // 192
  }                                                                                                                    // 193
                                                                                                                       //
  requirements.forEach(function (requirement) {                                                                        // 195
    if (requirement.tokenValid) {                                                                                      // 196
      var token = db.collections.apiTokens.findOne({                                                                   // 197
        _id: requirement.tokenValid,                                                                                   // 198
        revoked: {                                                                                                     // 198
          $ne: true                                                                                                    // 198
        }                                                                                                              // 198
      }, {                                                                                                             // 198
        fields: {                                                                                                      // 199
          requirements: 1                                                                                              // 199
        }                                                                                                              // 199
      });                                                                                                              // 199
                                                                                                                       //
      if (!token) {                                                                                                    // 200
        throw new Meteor.Error(403, "Capability revoked because the link through which it was introduced has been " + "revoked or deleted.");
      }                                                                                                                // 204
                                                                                                                       //
      checkRequirements(db, token.requirements);                                                                       // 206
                                                                                                                       //
      if (token.parentToken) {                                                                                         // 208
        checkRequirements(db, [{                                                                                       // 209
          tokenValid: token.parentToken                                                                                // 209
        }]);                                                                                                           // 209
      }                                                                                                                // 210
    } else if (requirement.permissionsHeld) {                                                                          // 211
      var p = requirement.permissionsHeld;                                                                             // 212
      var viewInfo = db.collections.grains.findOne(p.grainId, {                                                        // 213
        fields: {                                                                                                      // 214
          cachedViewInfo: 1                                                                                            // 214
        }                                                                                                              // 214
      }).cachedViewInfo;                                                                                               // 214
      var vertex = void 0;                                                                                             // 215
                                                                                                                       //
      if (p.accountId) {                                                                                               // 216
        vertex = {                                                                                                     // 217
          grain: {                                                                                                     // 217
            _id: p.grainId,                                                                                            // 217
            accountId: p.accountId                                                                                     // 217
          }                                                                                                            // 217
        };                                                                                                             // 217
      } else {                                                                                                         // 218
        vertex = {                                                                                                     // 219
          token: {                                                                                                     // 219
            _id: p.tokenId,                                                                                            // 219
            grainId: p.grainId                                                                                         // 219
          }                                                                                                            // 219
        };                                                                                                             // 219
      }                                                                                                                // 220
                                                                                                                       //
      var currentPermissions = SandstormPermissions.grainPermissions(db, vertex, viewInfo || {}).permissions;          // 222
                                                                                                                       //
      if (!currentPermissions) {                                                                                       // 224
        throw new Meteor.Error(403, "Capability revoked because a user involved in introducing it no longer has " + "the necessary permissions.");
      }                                                                                                                // 228
                                                                                                                       //
      var requiredPermissions = p.permissions || [];                                                                   // 230
                                                                                                                       //
      for (var ii = 0; ii < requiredPermissions.length; ++ii) {                                                        // 231
        if (requiredPermissions[ii] && !currentPermissions[ii]) {                                                      // 232
          throw new Meteor.Error(403, "Capability revoked because a user involved in introducing it no longer has " + "the necessary permissions.");
        }                                                                                                              // 236
      }                                                                                                                // 237
    } else if (requirement.userIsAdmin) {                                                                              // 238
      if (!db.isAdminById(requirement.userIsAdmin)) {                                                                  // 239
        throw new Meteor.Error(403, "Capability revoked because the user who created it has lost their admin " + "rights.");
      }                                                                                                                // 243
    } else {                                                                                                           // 244
      throw new Meteor.Error(403, "Unknown requirement type.");                                                        // 245
    }                                                                                                                  // 246
  });                                                                                                                  // 247
}                                                                                                                      // 248
                                                                                                                       //
;                                                                                                                      // 248
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"static-asset.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/server/static-asset.js                                                                                      //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
module.export({                                                                                                        // 1
  StaticAssetImpl: function () {                                                                                       // 1
    return StaticAssetImpl;                                                                                            // 1
  },                                                                                                                   // 1
  IdenticonStaticAssetImpl: function () {                                                                              // 1
    return IdenticonStaticAssetImpl;                                                                                   // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014-2016 Sandstorm Development Group, Inc. and contributors                                          // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var Url = Npm.require("url");                                                                                          // 17
                                                                                                                       //
var StaticAsset = Capnp.importSystem("sandstorm/grain.capnp").StaticAsset;                                             // 18
var PROTOCOL = Url.parse(process.env.ROOT_URL).protocol;                                                               // 20
                                                                                                                       //
var StaticAssetImpl = function () {                                                                                    //
  function StaticAssetImpl(assetId) {                                                                                  // 23
    (0, _classCallCheck3.default)(this, StaticAssetImpl);                                                              // 23
    check(assetId, String);                                                                                            // 24
    this._protocol = PROTOCOL.slice(0, -1);                                                                            // 25
    this._hostPath = makeWildcardHost("static") + "/" + assetId;                                                       // 26
  }                                                                                                                    // 27
                                                                                                                       //
  StaticAssetImpl.prototype.getUrl = function () {                                                                     //
    function getUrl() {                                                                                                //
      return {                                                                                                         // 30
        protocol: this._protocol,                                                                                      // 30
        hostPath: this._hostPath                                                                                       // 30
      };                                                                                                               // 30
    }                                                                                                                  // 31
                                                                                                                       //
    return getUrl;                                                                                                     //
  }();                                                                                                                 //
                                                                                                                       //
  return StaticAssetImpl;                                                                                              //
}();                                                                                                                   //
                                                                                                                       //
;                                                                                                                      // 32
                                                                                                                       //
var IdenticonStaticAssetImpl = function () {                                                                           //
  function IdenticonStaticAssetImpl(hash, size) {                                                                      // 35
    (0, _classCallCheck3.default)(this, IdenticonStaticAssetImpl);                                                     // 35
    check(hash, String);                                                                                               // 36
    check(size, Match.Integer);                                                                                        // 37
    this._protocol = PROTOCOL.slice(0, -1);                                                                            // 38
    this._hostPath = makeWildcardHost("static") + "/identicon/" + hash + "?s=" + size;                                 // 39
  }                                                                                                                    // 40
                                                                                                                       //
  IdenticonStaticAssetImpl.prototype.getUrl = function () {                                                            //
    function getUrl() {                                                                                                //
      return {                                                                                                         // 43
        protocol: this._protocol,                                                                                      // 43
        hostPath: this._hostPath                                                                                       // 43
      };                                                                                                               // 43
    }                                                                                                                  // 44
                                                                                                                       //
    return getUrl;                                                                                                     //
  }();                                                                                                                 //
                                                                                                                       //
  return IdenticonStaticAssetImpl;                                                                                     //
}();                                                                                                                   //
                                                                                                                       //
;                                                                                                                      // 45
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"constants.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/constants.js                                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
  ACCOUNT_DELETION_SUSPENSION_TIME: function () {                                                                      // 1
    return ACCOUNT_DELETION_SUSPENSION_TIME;                                                                           // 1
  },                                                                                                                   // 1
  PRIVATE_IPV4_ADDRESSES: function () {                                                                                // 1
    return PRIVATE_IPV4_ADDRESSES;                                                                                     // 1
  },                                                                                                                   // 1
  PRIVATE_IPV6_ADDRESSES: function () {                                                                                // 1
    return PRIVATE_IPV6_ADDRESSES;                                                                                     // 1
  },                                                                                                                   // 1
  SPECIAL_IPV4_ADDRESSES: function () {                                                                                // 1
    return SPECIAL_IPV4_ADDRESSES;                                                                                     // 1
  },                                                                                                                   // 1
  SPECIAL_IPV6_ADDRESSES: function () {                                                                                // 1
    return SPECIAL_IPV6_ADDRESSES;                                                                                     // 1
  }                                                                                                                    // 1
});                                                                                                                    // 1
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var ACCOUNT_DELETION_SUSPENSION_TIME = 7 * 60 * 60 * 24 * 1000; // 7 days in ms                                        // 17
// Lists below developed from RFC6890, which is an overview of all special addresses.                                  // 19
                                                                                                                       //
var PRIVATE_IPV4_ADDRESSES = ["10.0.0.0/8", // RFC1918 reserved for internal network                                   // 20
"127.0.0.0/8", // RFC1122 loopback / localhost                                                                         // 22
"169.254.0.0/16", // RFC3927 "link local" (auto-configured LAN in absence of DHCP)                                     // 23
"172.16.0.0/12", // RFC1918 reserved for internal network                                                              // 24
"192.168.0.0/16"];                                                                                                     // 25
var PRIVATE_IPV6_ADDRESSES = ["::1/128", // RFC4291 loopback / localhost                                               // 28
"fc00::/7", // RFC4193 unique private network                                                                          // 30
"fe80::/10"];                                                                                                          // 31
var SPECIAL_IPV4_ADDRESSES = ["0.0.0.0/8", // RFC1122 "this host" / wildcard                                           // 34
"100.64.0.0/10", // RFC6598 "shared address space" for carrier-grade NAT                                               // 36
"192.0.0.0/24", // RFC6890 reserved for special protocols                                                              // 37
"192.0.2.0/24", // RFC5737 "example address" block 1 -- like example.com for IPs                                       // 38
"192.88.99.0/24", // RFC3068 6to4 relay                                                                                // 39
"198.18.0.0/15", // RFC2544 standard benchmarks                                                                        // 40
"198.51.100.0/24", // RFC5737 "example address" block 2 -- like example.com for IPs                                    // 41
"203.0.113.0/24", // RFC5737 "example address" block 3 -- like example.com for IPs                                     // 42
"224.0.0.0/4", // RFC1112 multicast                                                                                    // 43
"240.0.0.0/4", // RFC1112 multicast / reserved for future use                                                          // 44
"255.255.255.255/32"];                                                                                                 // 45
var SPECIAL_IPV6_ADDRESSES = ["::/128", // RFC4291 unspecified address / wildcard                                      // 48
"64:ff9b::/96", // RFC6052 IPv4-IPv6 translation                                                                       // 50
"::ffff:0:0/96", // RFC4291 IPv4-mapped address                                                                        // 51
// TODO(someday): I don't understand the difference between the above                                                  // 52
//     two. Both are described as mapping ip4 addresses into the ip6                                                   // 53
//     space. Perhaps this should be allowed, however, we'd need to                                                    // 54
//     filter the ip4 address against the ip4 blacklist, so special                                                    // 55
//     handling would be needed.                                                                                       // 56
"100::/64", // RFC6666 discard-only address block                                                                      // 57
"2001::/23", // RFC2928 reserved for special protocols                                                                 // 58
"2001:2::/48", // RFC5180 standard benchmarks                                                                          // 59
"2001:db8::/32", // RFC3849 "example address" block -- like example.com for IPs                                        // 60
"2001:10::/28", // RFC4843 ORCHID                                                                                      // 61
"2002::/16", // RFC3056 6to4 relay                                                                                     // 62
"ff00::/8"];                                                                                                           // 63
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"db-deprecated.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/db-deprecated.js                                                                                            //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
// This file imports symbols from the sandstorm-db package. New code should use SandstormDb                            // 17
// directly.                                                                                                           // 18
// TODO(cleanup): Over time, eliminate the use of each of these assignments by using `globalDb`                        // 20
//   directly. For collections, prefer to update SandstormDb to provide methods for querying the                       // 21
//   collection rather than use `globalDb.collections` directly. When code is moved into packages,                     // 22
//   the `globalDb` global should NOT go with it; the package should expect a SandstormDb to be                        // 23
//   passed in, thus allowing mocking the database for unit tests.                                                     // 24
var quotaManager = void 0;                                                                                             // 26
                                                                                                                       //
if (Meteor.isServer) {                                                                                                 // 27
  var LDAP = void 0;                                                                                                   // 1
  module.watch(require("/imports/server/accounts/ldap.js"), {                                                          // 1
    LDAP: function (v) {                                                                                               // 1
      LDAP = v;                                                                                                        // 1
    }                                                                                                                  // 1
  }, 0);                                                                                                               // 1
  // Imports are usually not allowed to occur in a block. However, it is the only way to do                            // 29
  // this under Meteor. Using // jscs:disable doesn't work for what it considers syntax violations,                    // 30
  // and so we've added this file to .jscsrc's excludedFiles explicitly.                                               // 31
  quotaManager = new LDAP();                                                                                           // 33
} else {                                                                                                               // 34
  quotaManager = {                                                                                                     // 35
    updateUserQuota: function (db, user) {                                                                             // 36
      return {                                                                                                         // 37
        storage: user.cachedStorageQuota || 0,                                                                         // 38
        grains: Infinity,                                                                                              // 39
        compute: Infinity                                                                                              // 40
      };                                                                                                               // 37
    }                                                                                                                  // 42
  };                                                                                                                   // 35
}                                                                                                                      // 44
                                                                                                                       //
globalDb = new SandstormDb(quotaManager);                                                                              // 46
Packages = globalDb.collections.packages;                                                                              // 48
DevPackages = globalDb.collections.devPackages;                                                                        // 49
UserActions = globalDb.collections.userActions;                                                                        // 50
Grains = globalDb.collections.grains;                                                                                  // 51
Contacts = globalDb.collections.contacts;                                                                              // 52
Sessions = globalDb.collections.sessions;                                                                              // 53
SignupKeys = globalDb.collections.signupKeys;                                                                          // 54
ActivityStats = globalDb.collections.activityStats;                                                                    // 55
DeleteStats = globalDb.collections.deleteStats;                                                                        // 56
FileTokens = globalDb.collections.fileTokens;                                                                          // 57
ApiTokens = globalDb.collections.apiTokens;                                                                            // 58
Notifications = globalDb.collections.notifications;                                                                    // 59
StatsTokens = globalDb.collections.statsTokens;                                                                        // 60
Misc = globalDb.collections.misc;                                                                                      // 61
Settings = globalDb.collections.settings;                                                                              // 62
currentUserGrains = globalDb.currentUserGrains.bind(globalDb);                                                         // 64
isDemoUser = globalDb.isDemoUser.bind(globalDb);                                                                       // 65
isSignedUp = globalDb.isSignedUp.bind(globalDb);                                                                       // 66
isSignedUpOrDemo = globalDb.isSignedUpOrDemo.bind(globalDb);                                                           // 67
isUserOverQuota = globalDb.isUserOverQuota.bind(globalDb);                                                             // 68
isUserExcessivelyOverQuota = globalDb.isUserExcessivelyOverQuota.bind(globalDb);                                       // 69
isAdmin = globalDb.isAdmin.bind(globalDb);                                                                             // 70
isAdminById = globalDb.isAdminById.bind(globalDb);                                                                     // 71
findAdminUserForToken = globalDb.findAdminUserForToken.bind(globalDb);                                                 // 72
matchWildcardHost = globalDb.matchWildcardHost.bind(globalDb);                                                         // 73
makeWildcardHost = globalDb.makeWildcardHost.bind(globalDb);                                                           // 74
allowDevAccounts = globalDb.allowDevAccounts.bind(globalDb);                                                           // 75
roleAssignmentPattern = globalDb.roleAssignmentPattern;                                                                // 76
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"demo.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// imports/demo.js                                                                                                     //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
module.export({                                                                                                        // 1
                  allowDemo: function () {                                                                             // 1
                                    return allowDemo;                                                                  // 1
                  }                                                                                                    // 1
});                                                                                                                    // 1
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var allowDemo = Meteor.settings && Meteor.settings.public && Meteor.settings.public.allowDemoAccounts;                 // 17
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"server":{"accounts":{"credentials":{"credentials-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/credentials/credentials-server.js                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var SandstormBackend = void 0;                                                                                         // 1
module.watch(require("/imports/server/backend.js"), {                                                                  // 1
  SandstormBackend: function (v) {                                                                                     // 1
    SandstormBackend = v;                                                                                              // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var linkCredentialToAccountInternal = function (db, backend, credentialId, accountId, allowLogin) {                    // 19
  // Links the credential to the account. If `allowLogin` is true, grants the credential login access                  // 20
  // if possible. Makes the account durable if it is a demo account.                                                   // 21
  check(db, SandstormDb);                                                                                              // 23
  check(backend, SandstormBackend);                                                                                    // 24
  check(credentialId, String);                                                                                         // 25
  check(accountId, String);                                                                                            // 26
  var accountUser = Meteor.users.findOne({                                                                             // 28
    _id: accountId                                                                                                     // 28
  });                                                                                                                  // 28
                                                                                                                       //
  if (!accountUser) {                                                                                                  // 29
    throw new Meteor.Error(404, "No account found with ID " + accountId);                                              // 30
  }                                                                                                                    // 31
                                                                                                                       //
  if (accountUser.type !== "account") {                                                                                // 33
    throw new Meteor.Error(400, "Cannot link a credential to another credential.");                                    // 34
  }                                                                                                                    // 35
                                                                                                                       //
  if (!!_.findWhere(accountUser.loginCredentials, {                                                                    // 37
    id: credentialId                                                                                                   // 37
  }) || !!_.findWhere(accountUser.nonloginCredentials, {                                                               // 37
    id: credentialId                                                                                                   // 38
  })) {                                                                                                                // 38
    throw new Meteor.Error("alreadyLinked", "Cannot link a credential that's alread linked to this account.");         // 39
  }                                                                                                                    // 41
                                                                                                                       //
  var credentialUser = Meteor.users.findOne({                                                                          // 43
    _id: credentialId                                                                                                  // 43
  });                                                                                                                  // 43
                                                                                                                       //
  if (!credentialUser) {                                                                                               // 45
    throw new Meteor.Error(404, "No credential found with ID " + credentialId);                                        // 46
  }                                                                                                                    // 47
                                                                                                                       //
  if (!credentialUser.profile) {                                                                                       // 49
    throw new Meteor.Error(400, "Cannot link an account to another account");                                          // 50
  }                                                                                                                    // 51
                                                                                                                       //
  db.deleteUnusedAccount(backend, credentialUser._id);                                                                 // 53
                                                                                                                       //
  if (Meteor.users.findOne({                                                                                           // 54
    "loginCredentials.id": credentialUser._id                                                                          // 54
  })) {                                                                                                                // 54
    throw new Meteor.Error(403, "Cannot link a credential that can already log into another account");                 // 55
  }                                                                                                                    // 57
                                                                                                                       //
  var alreadyLinked = !!Meteor.users.findOne({                                                                         // 59
    "nonloginCredentials.id": credentialUser._id                                                                       // 59
  });                                                                                                                  // 59
  var pushModifier = alreadyLinked || !allowLogin ? {                                                                  // 61
    nonloginCredentials: {                                                                                             // 62
      id: credentialUser._id                                                                                           // 62
    }                                                                                                                  // 62
  } : {                                                                                                                // 62
    loginCredentials: {                                                                                                // 63
      id: credentialUser._id                                                                                           // 63
    }                                                                                                                  // 63
  };                                                                                                                   // 63
  var modifier = void 0;                                                                                               // 65
                                                                                                                       //
  if (accountUser.expires) {                                                                                           // 66
    if (alreadyLinked) {                                                                                               // 67
      throw new Meteor.Error(403, "Cannot create an account for a credential that's " + "already linked to another account.");
    }                                                                                                                  // 70
                                                                                                                       //
    modifier = {                                                                                                       // 72
      $push: pushModifier,                                                                                             // 73
      $unset: {                                                                                                        // 74
        expires: 1                                                                                                     // 74
      },                                                                                                               // 74
      $set: {                                                                                                          // 75
        upgradedFromDemo: Date.now()                                                                                   // 75
      }                                                                                                                // 75
    };                                                                                                                 // 72
                                                                                                                       //
    if (db.isReferralEnabled()) {                                                                                      // 77
      // Demo users never got the referral notification. Send it now:                                                  // 78
      db.sendReferralProgramNotification(accountUser._id);                                                             // 79
    }                                                                                                                  // 80
  } else {                                                                                                             // 82
    modifier = {                                                                                                       // 83
      $push: pushModifier                                                                                              // 83
    };                                                                                                                 // 83
  } // Make sure not to add the same credential twice.                                                                 // 84
                                                                                                                       //
                                                                                                                       //
  Meteor.users.update({                                                                                                // 87
    _id: accountUser._id,                                                                                              // 87
    "nonloginCredentials.id": {                                                                                        // 88
      $ne: credentialUser._id                                                                                          // 88
    },                                                                                                                 // 88
    "loginCredentials.id": {                                                                                           // 89
      $ne: credentialUser._id                                                                                          // 89
    }                                                                                                                  // 89
  }, modifier);                                                                                                        // 87
                                                                                                                       //
  if (accountUser.expires) {                                                                                           // 92
    var demoCredentialId = SandstormDb.getUserCredentialIds(accountUser)[0];                                           // 93
    Meteor.users.update({                                                                                              // 94
      _id: demoCredentialId                                                                                            // 94
    }, {                                                                                                               // 94
      $unset: {                                                                                                        // 95
        expires: 1                                                                                                     // 95
      },                                                                                                               // 95
      $set: {                                                                                                          // 96
        upgradedFromDemo: Date.now()                                                                                   // 96
      }                                                                                                                // 96
    }); // The account's existing profile is just "Demo User". Import the new credential's profile.                    // 95
                                                                                                                       //
    SandstormDb.fillInProfileDefaults(credentialUser, credentialUser.profile); // Mark the demo credential as nonlogin. It'd be nicer if the credential started out as nonlogin,
    // but to get that to work we would need to adjust the account creation and first login logic.                     // 102
                                                                                                                       //
    Meteor.users.update({                                                                                              // 103
      _id: accountUser._id,                                                                                            // 103
      "loginCredentials.id": demoCredentialId,                                                                         // 104
      "nonloginCredentials.id": {                                                                                      // 105
        $not: {                                                                                                        // 105
          $eq: demoCredentialId                                                                                        // 105
        }                                                                                                              // 105
      }                                                                                                                // 105
    }, {                                                                                                               // 103
      $pull: {                                                                                                         // 106
        loginCredentials: {                                                                                            // 106
          id: demoCredentialId                                                                                         // 106
        }                                                                                                              // 106
      },                                                                                                               // 106
      $push: {                                                                                                         // 107
        nonloginCredentials: {                                                                                         // 107
          id: demoCredentialId                                                                                         // 107
        }                                                                                                              // 107
      },                                                                                                               // 107
      $set: {                                                                                                          // 108
        profile: credentialUser.profile                                                                                // 108
      }                                                                                                                // 108
    });                                                                                                                // 106
  }                                                                                                                    // 109
};                                                                                                                     // 110
                                                                                                                       //
Meteor.methods({                                                                                                       // 112
  loginWithCredential: function (accountUserId) {                                                                      // 113
    // Logs into the account with ID `accountUserId`. Throws an exception if the current user is                       // 114
    // not a credential user listed in the account's `loginCredentials` field. This method is not                      // 115
    // intended to be called directly; client-side code should only invoke it through                                  // 116
    // `Meteor.loginWithCredential()`, which additionally maintains the standard Meteor client-side                    // 117
    // login state.                                                                                                    // 118
    check(accountUserId, String);                                                                                      // 120
    var credentialUser = Meteor.user();                                                                                // 122
                                                                                                                       //
    if (!credentialUser || !credentialUser.profile) {                                                                  // 123
      throw new Meteor.Error(403, "Must be already logged in as an credential.");                                      // 124
    }                                                                                                                  // 125
                                                                                                                       //
    var accountUser = Meteor.users.findOne(accountUserId);                                                             // 127
                                                                                                                       //
    if (!accountUser) {                                                                                                // 128
      throw new Meteor.Error(404, "No such user found: " + accountUserId);                                             // 129
    }                                                                                                                  // 130
                                                                                                                       //
    var linkedCredential = _.findWhere(accountUser.loginCredentials, {                                                 // 132
      id: credentialUser._id                                                                                           // 132
    });                                                                                                                // 132
                                                                                                                       //
    if (!linkedCredential) {                                                                                           // 134
      throw new Meteor.Error(403, "Current credential is not a login credential for account " + accountUserId);        // 135
    }                                                                                                                  // 137
                                                                                                                       //
    return Accounts._loginMethod(this, "loginWithCredential", [accountUserId], "credential", function () {             // 139
      return {                                                                                                         // 140
        userId: accountUserId                                                                                          // 140
      };                                                                                                               // 140
    });                                                                                                                // 140
  },                                                                                                                   // 141
  createAccountForCredential: function () {                                                                            // 143
    // Creates a new account for the currently-logged-in credential.                                                   // 144
    var user = Meteor.user();                                                                                          // 146
                                                                                                                       //
    if (user.type !== "credential") {                                                                                  // 147
      throw new Meteor.Error(403, "Must be logged in as a credential in order to create an account.");                 // 148
    }                                                                                                                  // 149
                                                                                                                       //
    if (Meteor.users.findOne({                                                                                         // 151
      $or: [{                                                                                                          // 152
        "loginCredentials.id": user._id                                                                                // 153
      }, {                                                                                                             // 153
        "nonloginCredentials.id": user._id                                                                             // 154
      }]                                                                                                               // 154
    })) {                                                                                                              // 151
      throw new Meteor.Error(403, "Cannot create an account for a credential that's already " + "linked to another account.");
    }                                                                                                                  // 159
                                                                                                                       //
    var newUser = {                                                                                                    // 161
      type: "account",                                                                                                 // 162
      loginCredentials: [{                                                                                             // 163
        id: user._id                                                                                                   // 163
      }],                                                                                                              // 163
      nonloginCredentials: []                                                                                          // 164
    };                                                                                                                 // 161
                                                                                                                       //
    if (user.services.dev) {                                                                                           // 166
      newUser.signupKey = "devAccounts";                                                                               // 167
                                                                                                                       //
      if (user.services.dev.isAdmin) {                                                                                 // 168
        newUser.isAdmin = true;                                                                                        // 169
      }                                                                                                                // 170
                                                                                                                       //
      if (user.services.dev.hasCompletedSignup) {                                                                      // 172
        newUser.hasCompletedSignup = true;                                                                             // 173
      }                                                                                                                // 174
    } else if (user.expires) {                                                                                         // 175
      // Demo user.                                                                                                    // 176
      newUser.expires = user.expires;                                                                                  // 177
                                                                                                                       //
      if (!!user.appDemoId) {                                                                                          // 178
        newUser.appDemoId = user.appDemoId;                                                                            // 179
      }                                                                                                                // 180
    }                                                                                                                  // 181
                                                                                                                       //
    SandstormDb.fillInProfileDefaults(user, user.profile);                                                             // 183
    var options = {                                                                                                    // 184
      profile: user.profile                                                                                            // 184
    }; // This will throw an error if the credential has been added as a login credential to some                      // 184
    // other account while we were executing the body of this method.                                                  // 187
                                                                                                                       //
    return Accounts.insertUserDoc(options, newUser);                                                                   // 188
  },                                                                                                                   // 189
  linkCredentialToAccount: function (token) {                                                                          // 191
    // Links the credential of the current user to the account that has `token` as a resume token.                     // 192
    // If the account is a demo account, makes the account durable and gives the credential login                      // 193
    // access to it.                                                                                                   // 194
    check(token, String);                                                                                              // 196
                                                                                                                       //
    if (!this.userId) {                                                                                                // 198
      throw new Meteor.Error(403, "Cannot link to account if not logged in.");                                         // 199
    }                                                                                                                  // 200
                                                                                                                       //
    var hashed = Accounts._hashLoginToken(token);                                                                      // 202
                                                                                                                       //
    var accountUser = Meteor.users.findOne({                                                                           // 203
      "services.resume.loginTokens.hashedToken": hashed                                                                // 203
    });                                                                                                                // 203
    linkCredentialToAccountInternal(this.connection.sandstormDb, this.connection.sandstormBackend, this.userId, accountUser._id, true);
  },                                                                                                                   // 207
  unlinkCredential: function (accountUserId, credentialId) {                                                           // 209
    // Unlinks the credential with ID `credentialId` from the account with ID `accountUserId`.                         // 210
    check(credentialId, String);                                                                                       // 212
    check(accountUserId, String);                                                                                      // 213
                                                                                                                       //
    if (!this.userId) {                                                                                                // 215
      throw new Meteor.Error(403, "Not logged in.");                                                                   // 216
    }                                                                                                                  // 217
                                                                                                                       //
    if (!this.connection.sandstormDb.userHasCredential(this.userId, credentialId)) {                                   // 219
      throw new Meteor.Error(403, "Current user does not own credential " + credentialId);                             // 220
    }                                                                                                                  // 221
                                                                                                                       //
    var credentialUser = Meteor.users.findOne({                                                                        // 223
      _id: credentialId                                                                                                // 223
    });                                                                                                                // 223
    Meteor.users.update({                                                                                              // 224
      _id: accountUserId                                                                                               // 225
    }, {                                                                                                               // 224
      $pull: {                                                                                                         // 227
        nonloginCredentials: {                                                                                         // 228
          id: credentialId                                                                                             // 228
        },                                                                                                             // 228
        loginCredentials: {                                                                                            // 229
          id: credentialId                                                                                             // 229
        }                                                                                                              // 229
      }                                                                                                                // 227
    });                                                                                                                // 226
  },                                                                                                                   // 232
  setCredentialAllowsLogin: function (credentialId, allowLogin) {                                                      // 234
    // Sets whether the current account allows the credential with ID `credentialId` to log in.                        // 235
    check(credentialId, String);                                                                                       // 237
    check(allowLogin, Boolean);                                                                                        // 238
                                                                                                                       //
    if (!this.userId) {                                                                                                // 239
      throw new Meteor.Error(403, "Not logged in.");                                                                   // 240
    }                                                                                                                  // 241
                                                                                                                       //
    if (!this.connection.sandstormDb.userHasCredential(this.userId, credentialId)) {                                   // 243
      throw new Meteor.Error(403, "Current user does not own credential " + credentialId);                             // 244
    }                                                                                                                  // 245
                                                                                                                       //
    if (allowLogin) {                                                                                                  // 247
      Meteor.users.update({                                                                                            // 248
        _id: this.userId,                                                                                              // 248
        "nonloginCredentials.id": credentialId,                                                                        // 249
        "loginCredentials.id": {                                                                                       // 250
          $not: {                                                                                                      // 250
            $eq: credentialId                                                                                          // 250
          }                                                                                                            // 250
        }                                                                                                              // 250
      }, {                                                                                                             // 248
        $pull: {                                                                                                       // 251
          nonloginCredentials: {                                                                                       // 251
            id: credentialId                                                                                           // 251
          }                                                                                                            // 251
        },                                                                                                             // 251
        $push: {                                                                                                       // 252
          loginCredentials: {                                                                                          // 252
            id: credentialId                                                                                           // 252
          }                                                                                                            // 252
        }                                                                                                              // 252
      });                                                                                                              // 251
    } else {                                                                                                           // 253
      Meteor.users.update({                                                                                            // 254
        _id: this.userId,                                                                                              // 254
        "loginCredentials.id": credentialId,                                                                           // 255
        "nonloginCredentials.id": {                                                                                    // 256
          $not: {                                                                                                      // 256
            $eq: credentialId                                                                                          // 256
          }                                                                                                            // 256
        }                                                                                                              // 256
      }, {                                                                                                             // 254
        $pull: {                                                                                                       // 257
          loginCredentials: {                                                                                          // 257
            id: credentialId                                                                                           // 257
          }                                                                                                            // 257
        },                                                                                                             // 257
        $push: {                                                                                                       // 258
          nonloginCredentials: {                                                                                       // 258
            id: credentialId                                                                                           // 258
          }                                                                                                            // 258
        }                                                                                                              // 258
      });                                                                                                              // 257
    }                                                                                                                  // 259
  },                                                                                                                   // 260
  logoutCredentialsOfCurrentAccount: function () {                                                                     // 262
    // Logs out all credentials that are allowed to log in to the current account.                                     // 263
    var user = Meteor.user();                                                                                          // 264
                                                                                                                       //
    if (user && user.loginCredentials) {                                                                               // 265
      user.loginCredentials.forEach(function (credential) {                                                            // 266
        Meteor.users.update({                                                                                          // 267
          _id: credential.id                                                                                           // 267
        }, {                                                                                                           // 267
          $set: {                                                                                                      // 267
            "services.resume.loginTokens": []                                                                          // 267
          }                                                                                                            // 267
        });                                                                                                            // 267
      });                                                                                                              // 268
    }                                                                                                                  // 269
  }                                                                                                                    // 270
});                                                                                                                    // 112
                                                                                                                       //
Accounts.linkCredentialToAccount = function (db, backend, credentialId, accountId, allowLogin) {                       // 273
  // Links the credential to the account. If the account is a demo account, makes it durable.                          // 274
  // If `allowLogin` is true, attempts to give the credential login access.                                            // 275
  check(db, SandstormDb);                                                                                              // 276
  check(backend, SandstormBackend);                                                                                    // 277
  check(credentialId, String);                                                                                         // 278
  check(accountId, String);                                                                                            // 279
  check(allowLogin, Boolean);                                                                                          // 280
  linkCredentialToAccountInternal(db, backend, credentialId, accountId, allowLogin);                                   // 281
};                                                                                                                     // 282
                                                                                                                       //
Meteor.publish("accountsOfCredential", function (credentialId) {                                                       // 284
  check(credentialId, String);                                                                                         // 285
  if (!SandstormDb.ensureSubscriberHasCredential(this, credentialId)) return; // Map from credential ID to `true` for each credential we've published already.
                                                                                                                       //
  var loginCredentials = {};                                                                                           // 289
                                                                                                                       //
  var _this = this;                                                                                                    // 291
                                                                                                                       //
  function addCredentialsOfAccount(account) {                                                                          // 292
    account.loginCredentials.forEach(function (credential) {                                                           // 293
      if (!(credential.id in loginCredentials)) {                                                                      // 294
        var user = Meteor.users.findOne({                                                                              // 295
          _id: credential.id                                                                                           // 295
        });                                                                                                            // 295
                                                                                                                       //
        if (user) {                                                                                                    // 296
          user.intrinsicName = SandstormDb.getIntrinsicName(user);                                                     // 297
          user.loginId = SandstormDb.getLoginId(user);                                                                 // 298
          user.serviceName = SandstormDb.getServiceName(user);                                                         // 299
                                                                                                                       //
          var filteredUser = _.pick(user, "_id", "intrinsicName", "loginId", "serviceName");                           // 301
                                                                                                                       //
          filteredUser.loginAccountId = account._id;                                                                   // 302
          filteredUser.sourceCredentialId = credentialId;                                                              // 303
                                                                                                                       //
          _this.added("loginCredentialsOfLinkedAccounts", user._id, filteredUser);                                     // 304
        }                                                                                                              // 305
                                                                                                                       //
        loginCredentials[credential.id] = true;                                                                        // 307
      }                                                                                                                // 308
    });                                                                                                                // 309
  }                                                                                                                    // 310
                                                                                                                       //
  var cursor = Meteor.users.find({                                                                                     // 312
    $or: [{                                                                                                            // 313
      "loginCredentials.id": credentialId                                                                              // 314
    }, {                                                                                                               // 314
      "nonloginCredentials.id": credentialId                                                                           // 315
    }]                                                                                                                 // 315
  });                                                                                                                  // 312
  var handle = cursor.observe({                                                                                        // 319
    added: function (account) {                                                                                        // 320
      addCredentialsOfAccount(account);                                                                                // 321
    },                                                                                                                 // 322
    changed: function (newAccount, oldAccount) {                                                                       // 324
      addCredentialsOfAccount(newAccount);                                                                             // 325
    },                                                                                                                 // 326
    removed: function (account) {                                                                                      // 328
      account.loginCredentials.forEach(function (credential) {                                                         // 329
        if (credential.id in loginCredentials) {                                                                       // 330
          _this.removed("loginCredentialsOfLinkedAccounts", credential.id);                                            // 331
                                                                                                                       //
          delete loginCredentials[credential.id];                                                                      // 332
        }                                                                                                              // 333
      });                                                                                                              // 334
    }                                                                                                                  // 335
  });                                                                                                                  // 319
  this.ready();                                                                                                        // 337
  this.onStop(function () {                                                                                            // 339
    handle.stop();                                                                                                     // 340
    Object.keys(loginCredentials).forEach(function (credentialId) {                                                    // 341
      delete loginCredentials[credentialId];                                                                           // 342
    });                                                                                                                // 343
  });                                                                                                                  // 344
});                                                                                                                    // 345
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"email-token":{"token-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/email-token/token-server.js                                                                         //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var crypto = void 0;                                                                                                   // 1
module.watch(require("crypto"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    crypto = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var sendEmail = void 0;                                                                                                // 1
module.watch(require("/imports/server/email.js"), {                                                                    // 1
  send: function (v) {                                                                                                 // 1
    sendEmail = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 5
                                                                                                                       //
var V1_ROUNDS = 4096; // Selected to take ~5msec at creation time (2016) on a developer's laptop.                      // 7
                                                                                                                       //
var V1_KEYSIZE = 32; // 256 bits / 8 bits/byte = 32 bytes                                                              // 8
                                                                                                                       //
var V1_HASHFUNC = "sha512"; // ^ hash function used with pbkdf2.  Chosen to be different from the function which maps the token
// to the value stored in the database.  Note that the first thing that pbkdf2 does is                                 // 11
// HMAC(HASHFUNC, key, salt), and the first thing that HMAC does is either pad or hash the key to                      // 12
// make it the appropriate width.  The result is that knowing sha256(key) and the salt is possibly                     // 13
// sufficient to reconstruct the output of pbkdf2().                                                                   // 14
                                                                                                                       //
var V1_CIPHER = "AES-256-CTR"; // cipher used                                                                          // 15
                                                                                                                       //
var TOKEN_EXPIRATION_MS = 60 * 60 * 1000;                                                                              // 17
                                                                                                                       //
var cleanupExpiredTokens = function () {                                                                               // 19
  Meteor.users.update({                                                                                                // 20
    "services.email.tokens.createdAt": {                                                                               // 21
      $lt: new Date(Date.now() - TOKEN_EXPIRATION_MS)                                                                  // 22
    }                                                                                                                  // 21
  }, {                                                                                                                 // 20
    $pull: {                                                                                                           // 25
      "services.email.tokens": {                                                                                       // 26
        createdAt: {                                                                                                   // 27
          $lt: new Date(Date.now() - TOKEN_EXPIRATION_MS)                                                              // 27
        }                                                                                                              // 27
      }                                                                                                                // 26
    }                                                                                                                  // 25
  }, {                                                                                                                 // 24
    multi: true                                                                                                        // 31
  });                                                                                                                  // 30
};                                                                                                                     // 33
                                                                                                                       //
Meteor.startup(cleanupExpiredTokens); // Tokens can actually last up to 2 * TOKEN_EXPIRATION_MS                        // 35
                                                                                                                       //
SandstormDb.periodicCleanup(TOKEN_EXPIRATION_MS, cleanupExpiredTokens);                                                // 37
                                                                                                                       //
var hashToken = function (token) {                                                                                     // 39
  return {                                                                                                             // 40
    digest: SHA256(token),                                                                                             // 41
    algorithm: "sha-256"                                                                                               // 42
  };                                                                                                                   // 40
};                                                                                                                     // 44
                                                                                                                       //
var checkToken = function (tokens, token) {                                                                            // 46
  // Looks for an object in `tokens` with `algorithm` and `digest` fields matching those in `token`.                   // 47
  // Returns the matching object, if one is found, or undefined if none match.                                         // 48
  var foundToken = undefined;                                                                                          // 49
  tokens.forEach(function (userToken) {                                                                                // 50
    if (userToken.algorithm === token.algorithm && userToken.digest === token.digest) {                                // 51
      foundToken = userToken;                                                                                          // 53
    }                                                                                                                  // 54
  });                                                                                                                  // 55
  return foundToken;                                                                                                   // 57
};                                                                                                                     // 58
                                                                                                                       //
var consumeToken = function (user, token) {                                                                            // 60
  var hashedToken = hashToken(token);                                                                                  // 61
  var foundToken = checkToken(user.services.email.tokens, hashedToken);                                                // 62
                                                                                                                       //
  if (foundToken !== undefined) {                                                                                      // 64
    Meteor.users.update({                                                                                              // 65
      _id: user._id                                                                                                    // 65
    }, {                                                                                                               // 65
      $pull: {                                                                                                         // 65
        "services.email.tokens": hashedToken                                                                           // 65
      }                                                                                                                // 65
    });                                                                                                                // 65
  }                                                                                                                    // 66
                                                                                                                       //
  return foundToken;                                                                                                   // 68
};                                                                                                                     // 69
                                                                                                                       //
var makeBox = function (token, plaintext) {                                                                            // 71
  // Encrypt plaintext symmetrically with a key derived from token.  Returns an object with                            // 72
  // ciphertext and associated data needed to decrypt later.                                                           // 73
  // Produce a symmetric key.  Note that the token itself does not have sufficient entropy to                          // 75
  // be used as a key directly, so we need to use a KDF with a strong random salt.                                     // 76
  // In the fullness of time, it might be nice to move away from using a KDF (which blocks the whole                   // 77
  // node process) in favor of the token itself having enough entropy to serve as the key itself.                      // 78
  // This would require lengthening the token, which would make the manual-code-entry workflow                         // 79
  // worse, so I'm punting on that for now.                                                                            // 80
  var salt = Random.secret(16);                                                                                        // 81
  var key = crypto.pbkdf2Sync(token, salt, V1_ROUNDS, V1_KEYSIZE, V1_HASHFUNC);                                        // 82
  var iv = crypto.randomBytes(16);                                                                                     // 83
  var cipher = crypto.createCipheriv(V1_CIPHER, key, iv);                                                              // 84
  var ciphertext = cipher.update(new Buffer(plaintext, "binary"));                                                     // 85
  return {                                                                                                             // 86
    version: 1,                                                                                                        // 87
    salt: salt,                                                                                                        // 88
    iv: iv.toString("base64"),                                                                                         // 89
    boxedValue: ciphertext.toString("base64")                                                                          // 90
  };                                                                                                                   // 86
};                                                                                                                     // 92
                                                                                                                       //
var tryUnbox = function (box, secret) {                                                                                // 94
  if (box) {                                                                                                           // 95
    if (box.version === 1) {                                                                                           // 96
      var key = crypto.pbkdf2Sync(secret, box.salt, V1_ROUNDS, V1_KEYSIZE, V1_HASHFUNC);                               // 97
      var iv = new Buffer(box.iv, "base64");                                                                           // 98
      var cipher = crypto.createDecipheriv(V1_CIPHER, key, iv);                                                        // 99
      var cipherText = new Buffer(box.boxedValue, "base64");                                                           // 100
      var plaintext = cipher.update(cipherText);                                                                       // 101
      return plaintext.toString("binary");                                                                             // 102
    }                                                                                                                  // 103
  } // If no box was provided, or it was of an unknown version, return no data.                                        // 104
                                                                                                                       //
                                                                                                                       //
  return;                                                                                                              // 107
}; // Handler to login with a token.                                                                                   // 108
                                                                                                                       //
                                                                                                                       //
Accounts.registerLoginHandler("email", function (options) {                                                            // 111
  if (!options.email) {                                                                                                // 112
    return undefined; // don't handle                                                                                  // 113
  }                                                                                                                    // 114
                                                                                                                       //
  if (!Accounts.loginServices.email.isEnabled()) {                                                                     // 116
    throw new Meteor.Error(403, "Email login service is disabled.");                                                   // 117
  }                                                                                                                    // 118
                                                                                                                       //
  options = options.email;                                                                                             // 120
  check(options, {                                                                                                     // 121
    email: String,                                                                                                     // 122
    token: String                                                                                                      // 123
  });                                                                                                                  // 121
  var user = Meteor.users.findOne({                                                                                    // 126
    "services.email.email": options.email                                                                              // 127
  }, {                                                                                                                 // 126
    fields: {                                                                                                          // 129
      "services.email": 1                                                                                              // 130
    }                                                                                                                  // 129
  });                                                                                                                  // 128
                                                                                                                       //
  if (!user) {                                                                                                         // 134
    console.error("User not found:", options.email);                                                                   // 135
    return {                                                                                                           // 136
      error: new Meteor.Error(403, "User not found")                                                                   // 137
    };                                                                                                                 // 136
  }                                                                                                                    // 139
                                                                                                                       //
  if (!user.services.email.tokens) {                                                                                   // 141
    console.error("User has no token set:", options.email);                                                            // 142
    return {                                                                                                           // 143
      error: new Meteor.Error(403, "User has no token set")                                                            // 144
    };                                                                                                                 // 143
  }                                                                                                                    // 146
                                                                                                                       //
  var tokenString = options.token.trim();                                                                              // 148
  var maybeToken = consumeToken(user, tokenString);                                                                    // 149
                                                                                                                       //
  if (!maybeToken) {                                                                                                   // 150
    console.error("Token not found:", options.email);                                                                  // 151
    return {                                                                                                           // 152
      error: new Meteor.Error(403, "Invalid authentication code")                                                      // 153
    };                                                                                                                 // 152
  } // Attempt to decrypt the resumePath, if provided.                                                                 // 155
                                                                                                                       //
                                                                                                                       //
  var resumePath = tryUnbox(maybeToken.secureBox, tokenString);                                                        // 158
  return {                                                                                                             // 160
    userId: user._id,                                                                                                  // 161
    options: {                                                                                                         // 162
      resumePath: resumePath                                                                                           // 163
    }                                                                                                                  // 162
  };                                                                                                                   // 160
});                                                                                                                    // 166
                                                                                                                       //
var makeTokenUrl = function (email, token, options) {                                                                  // 168
  if (options.linking) {                                                                                               // 169
    return options.rootUrl + "/_emailLinkCredential/" + encodeURIComponent(email) + "/" + encodeURIComponent(token) + "/" + Meteor.userId() + "?allowLogin=" + options.linking.allowLogin;
  } else {                                                                                                             // 173
    return options.rootUrl + "/_emailLogin/" + encodeURIComponent(email) + "/" + encodeURIComponent(token);            // 174
  }                                                                                                                    // 175
}; ///                                                                                                                 // 176
/// EMAIL VERIFICATION                                                                                                 // 179
///                                                                                                                    // 180
                                                                                                                       //
                                                                                                                       //
var sendTokenEmail = function (db, email, token, options) {                                                            // 181
  var subject = void 0;                                                                                                // 182
  var text = void 0;                                                                                                   // 183
  var rootHostname = Url.parse(options.rootUrl).hostname;                                                              // 185
                                                                                                                       //
  if (!options.linking) {                                                                                              // 187
    subject = "Log in to " + rootHostname;                                                                             // 188
    text = "To confirm this email address on ";                                                                        // 189
  } else {                                                                                                             // 190
    subject = "Confirm this email address on " + rootHostname;                                                         // 191
    text = "To confirm this email address on ";                                                                        // 192
  }                                                                                                                    // 193
                                                                                                                       //
  text = text + rootHostname + ", click on the following link:\n\n" + makeTokenUrl(email, token, options) + "\n\n" + "Alternatively, enter the following one-time authentication code into the log-in form:\n\n" + token;
  var sendOptions = {                                                                                                  // 200
    to: email,                                                                                                         // 201
    from: {                                                                                                            // 202
      name: globalDb.getServerTitle(),                                                                                 // 202
      address: db.getReturnAddress()                                                                                   // 202
    },                                                                                                                 // 202
    subject: subject,                                                                                                  // 203
    text: text                                                                                                         // 204
  };                                                                                                                   // 200
  sendEmail(sendOptions);                                                                                              // 207
};                                                                                                                     // 208
                                                                                                                       //
var parsedRootUrl = Url.parse(process.env.ROOT_URL); ///                                                               // 210
/// CREATING USERS                                                                                                     // 212
///                                                                                                                    // 213
// returns the user id                                                                                                 // 214
                                                                                                                       //
var createAndEmailTokenForUser = function (db, email, options) {                                                       // 215
  check(email, String);                                                                                                // 216
  check(options, {                                                                                                     // 217
    resumePath: String,                                                                                                // 218
    linking: Match.Optional({                                                                                          // 219
      allowLogin: Boolean                                                                                              // 219
    }),                                                                                                                // 219
    rootUrl: String                                                                                                    // 220
  });                                                                                                                  // 217
  var parsedUrl = Url.parse(options.rootUrl);                                                                          // 223
                                                                                                                       //
  if ((parsedUrl.hostname !== parsedRootUrl.hostname || parsedUrl.protocol !== parsedRootUrl.protocol) && !db.hostIsStandalone(parsedUrl.hostname)) {
    // Ignore port and only check hostname/protocol since IE will differ from other browsers and                       // 227
    // sometimes include port 80/443 and sometimes won't                                                               // 228
    throw new Meteor.Error(400, "rootUrl is not valid");                                                               // 229
  }                                                                                                                    // 230
                                                                                                                       //
  var atIndex = email.indexOf("@");                                                                                    // 232
                                                                                                                       //
  if (atIndex === -1) {                                                                                                // 233
    throw new Meteor.Error(400, "No @ symbol was found in your email");                                                // 234
  }                                                                                                                    // 235
                                                                                                                       //
  var user = Meteor.users.findOne({                                                                                    // 237
    "services.email.email": email                                                                                      // 237
  }, {                                                                                                                 // 237
    fields: {                                                                                                          // 238
      "services.email": 1                                                                                              // 238
    }                                                                                                                  // 238
  });                                                                                                                  // 238
  var userId = void 0; // TODO(someday): make this shorter, and handle requests that try to brute force it.            // 239
  // Alternately, require using the link over copy/pasting the code, and crank up the entropy.                         // 242
                                                                                                                       //
  var token = Random.id(12);                                                                                           // 243
  var tokenObj = hashToken(token);                                                                                     // 244
  tokenObj.createdAt = new Date();                                                                                     // 245
  tokenObj.secureBox = makeBox(token, options.resumePath);                                                             // 246
                                                                                                                       //
  if (user) {                                                                                                          // 248
    if (user.services.email.tokens && user.services.email.tokens.length > 2) {                                         // 249
      throw new Meteor.Error("alreadySentEmailToken", "It looks like we sent a log in email to this address not long " + "ago. Please use the one that was already sent (check your spam folder if you can't find " + "it), or wait a while and try again.");
    }                                                                                                                  // 255
                                                                                                                       //
    userId = user._id;                                                                                                 // 257
    Meteor.users.update({                                                                                              // 259
      _id: user._id                                                                                                    // 259
    }, {                                                                                                               // 259
      $push: {                                                                                                         // 259
        "services.email.tokens": tokenObj                                                                              // 259
      }                                                                                                                // 259
    });                                                                                                                // 259
  } else {                                                                                                             // 260
    var _options = {};                                                                                                 // 261
    user = {                                                                                                           // 262
      services: {                                                                                                      // 263
        email: {                                                                                                       // 264
          tokens: [tokenObj],                                                                                          // 265
          email: email                                                                                                 // 266
        }                                                                                                              // 264
      }                                                                                                                // 263
    };                                                                                                                 // 262
    userId = Accounts.insertUserDoc(_options, user);                                                                   // 271
  }                                                                                                                    // 272
                                                                                                                       //
  sendTokenEmail(db, email, token, options);                                                                           // 274
  return userId;                                                                                                       // 276
};                                                                                                                     // 277
                                                                                                                       //
Meteor.methods({                                                                                                       // 279
  createAndEmailTokenForUser: function (email, options) {                                                              // 280
    // method for create user. Requests come from the client.                                                          // 281
    // This method will create a user if it doesn't exist, otherwise it will generate a token.                         // 282
    // It will always send an email to the user                                                                        // 283
    check(email, String);                                                                                              // 285
    check(options, {                                                                                                   // 286
      resumePath: String,                                                                                              // 287
      linking: Match.Optional({                                                                                        // 288
        allowLogin: Boolean                                                                                            // 288
      }),                                                                                                              // 288
      rootUrl: String                                                                                                  // 289
    });                                                                                                                // 286
                                                                                                                       //
    if (!Accounts.loginServices.email.isEnabled()) {                                                                   // 292
      throw new Meteor.Error(403, "Email login service is disabled.");                                                 // 293
    } // Create user. result contains id and token.                                                                    // 294
                                                                                                                       //
                                                                                                                       //
    var user = createAndEmailTokenForUser(this.connection.sandstormDb, email, options);                                // 296
  },                                                                                                                   // 297
  linkEmailCredentialToAccount: function (email, token, allowLogin) {                                                  // 299
    // Links the email credential with address `email` and login token `token` to the current                          // 300
    // account.                                                                                                        // 301
    check(email, String);                                                                                              // 302
    check(token, String);                                                                                              // 303
    check(allowLogin, Boolean);                                                                                        // 304
    var account = Meteor.user();                                                                                       // 305
                                                                                                                       //
    if (!account || !account.loginCredentials) {                                                                       // 306
      throw new Meteor.Error(403, "Must be logged in to an account to link an email credential.");                     // 307
    }                                                                                                                  // 308
                                                                                                                       //
    var credential = Meteor.users.findOne({                                                                            // 310
      "services.email.email": email                                                                                    // 310
    }, {                                                                                                               // 310
      fields: {                                                                                                        // 311
        "services.email": 1                                                                                            // 311
      }                                                                                                                // 311
    });                                                                                                                // 311
                                                                                                                       //
    if (!credential) {                                                                                                 // 312
      throw new Meteor.Error(403, "Invalid authentication code.");                                                     // 313
    }                                                                                                                  // 314
                                                                                                                       //
    var maybeToken = consumeToken(credential, token.trim());                                                           // 316
                                                                                                                       //
    if (!maybeToken) {                                                                                                 // 317
      throw new Meteor.Error(403, "Invalid authentication code.");                                                     // 318
    }                                                                                                                  // 319
                                                                                                                       //
    Accounts.linkCredentialToAccount(this.connection.sandstormDb, this.connection.sandstormBackend, credential._id, account._id, allowLogin); // Return the resume path, if we have one.
                                                                                                                       //
    var resumePath = tryUnbox(maybeToken.secureBox, token);                                                            // 325
    return resumePath;                                                                                                 // 326
  }                                                                                                                    // 327
});                                                                                                                    // 279
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"ldap":{"ldap-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/ldap/ldap-server.js                                                                                 //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var LDAP = void 0;                                                                                                     // 1
module.watch(require("/imports/server/accounts/ldap.js"), {                                                            // 1
  LDAP: function (v) {                                                                                                 // 1
    LDAP = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
// Register login handler with Meteor                                                                                  // 3
Accounts.registerLoginHandler("ldap", function (loginRequest) {                                                        // 4
  // If 'ldap' isn't set in loginRequest object,                                                                       // 5
  // then this isn't the proper handler (return undefined)                                                             // 6
  if (!loginRequest.ldap) {                                                                                            // 7
    return undefined;                                                                                                  // 8
  }                                                                                                                    // 9
                                                                                                                       //
  if (!Accounts.loginServices.ldap.isEnabled()) {                                                                      // 11
    throw new Meteor.Error(403, "LDAP service is disabled.");                                                          // 12
  }                                                                                                                    // 13
                                                                                                                       //
  check(loginRequest, {                                                                                                // 15
    ldap: true,                                                                                                        // 16
    username: String,                                                                                                  // 17
    ldapPass: String                                                                                                   // 18
  }); // Instantiate LDAP                                                                                              // 15
                                                                                                                       //
  var ldapObj = new LDAP(); // Call ldapCheck and get response                                                         // 22
                                                                                                                       //
  var ldapResponse = ldapObj.ldapCheck(this.connection.sandstormDb, loginRequest);                                     // 25
                                                                                                                       //
  if (ldapResponse.error) {                                                                                            // 27
    return {                                                                                                           // 28
      userId: null,                                                                                                    // 29
      error: ldapResponse.error                                                                                        // 30
    };                                                                                                                 // 28
  } else if (ldapResponse.emptySearch) {                                                                               // 32
    return {                                                                                                           // 33
      userId: null,                                                                                                    // 34
      error: new Meteor.Error(403, "User not found in LDAP")                                                           // 35
    };                                                                                                                 // 33
  } else {                                                                                                             // 37
    // Set initial userId and token vals                                                                               // 38
    return Accounts.updateOrCreateUserFromExternalService("ldap", {                                                    // 39
      id: ldapResponse.dn,                                                                                             // 40
      username: loginRequest.username,                                                                                 // 40
      rawAttrs: ldapResponse.searchResults                                                                             // 41
    }, {});                                                                                                            // 40
  }                                                                                                                    // 42
});                                                                                                                    // 44
Meteor.methods({                                                                                                       // 46
  updateQuota: function () {                                                                                           // 47
    return this.connection.sandstormDb.updateUserQuota(Meteor.user()); // This is a no-op if settings aren't enabled   // 48
  }                                                                                                                    // 50
});                                                                                                                    // 46
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"saml":{"saml-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/saml/saml-server.js                                                                                 //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var module1 = module;                                                                                                  // 1
var Url = void 0;                                                                                                      // 1
module1.watch(require("url"), {                                                                                        // 1
  "default": function (v) {                                                                                            // 1
    Url = v;                                                                                                           // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var zlib = void 0;                                                                                                     // 1
module1.watch(require("zlib"), {                                                                                       // 1
  "default": function (v) {                                                                                            // 1
    zlib = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var SAML = void 0;                                                                                                     // 1
module1.watch(require("/imports/server/accounts/saml-utils.js"), {                                                     // 1
  SAML: function (v) {                                                                                                 // 1
    SAML = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
                                                                                                                       //
var Fiber = Npm.require("fibers");                                                                                     // 5
                                                                                                                       //
if (!Accounts.saml) {                                                                                                  // 7
  Accounts.saml = {};                                                                                                  // 8
} // We need to use connect. Let's make sure we're using the same version as Meteor's WebApp module                    // 9
// uses. Fortunately, they let us extract it.                                                                          // 12
                                                                                                                       //
                                                                                                                       //
var connect = WebAppInternals.NpmModules.connect.module;                                                               // 13
RoutePolicy.declare("/_saml/", "network");                                                                             // 15
var HOSTNAME = Url.parse(process.env.ROOT_URL).hostname; // TODO(soon): This may need to be a Mongo collection in order to work when the frontend is
//   replicated (but currently SAML is not used on any Blackrock servers).                                             // 20
                                                                                                                       //
var _loginResultForCredentialToken = {};                                                                               // 21
                                                                                                                       //
var retrieveCredential = function (credentialToken) {                                                                  // 23
  var result = _loginResultForCredentialToken[credentialToken];                                                        // 24
  delete _loginResultForCredentialToken[credentialToken];                                                              // 25
  return result;                                                                                                       // 26
};                                                                                                                     // 27
                                                                                                                       //
Accounts.registerLoginHandler(function (loginRequest) {                                                                // 29
  if (!loginRequest.saml || !loginRequest.credentialToken) {                                                           // 30
    return undefined;                                                                                                  // 31
  }                                                                                                                    // 32
                                                                                                                       //
  if (!Accounts.loginServices.saml.isEnabled()) {                                                                      // 34
    throw new Meteor.Error(403, "SAML service is disabled.");                                                          // 35
  }                                                                                                                    // 36
                                                                                                                       //
  var loginResult = retrieveCredential(loginRequest.credentialToken);                                                  // 38
                                                                                                                       //
  if (!loginResult) {                                                                                                  // 39
    throw new Meteor.Error(500, "SAML login did not complete.");                                                       // 40
  } else if (loginResult.profile && loginResult.profile.email) {                                                       // 41
    var user = _.pick(loginResult.profile, "displayName", "email", "nameIDFormat");                                    // 42
                                                                                                                       //
    user.id = loginResult.profile.nameID;                                                                              // 43
    return Accounts.updateOrCreateUserFromExternalService("saml", user, {});                                           // 44
  } else {                                                                                                             // 45
    throw new Meteor.Error(500, "SAML profile did not contain an email address");                                      // 46
  }                                                                                                                    // 47
});                                                                                                                    // 48
                                                                                                                       //
var samlUrlToObject = function (url) {                                                                                 // 50
  // req.url will be "/_saml/<action>/<service name>/<credentialToken>"                                                // 51
  if (!url) return null;                                                                                               // 52
  var splitPath = url.split("/"); // Any non-saml request will continue down the default                               // 55
  // middlewares.                                                                                                      // 58
                                                                                                                       //
  if (splitPath[1] !== "_saml") return null;                                                                           // 59
  return {                                                                                                             // 62
    actionName: splitPath[2],                                                                                          // 63
    serviceName: splitPath[3],                                                                                         // 64
    credentialToken: splitPath[4]                                                                                      // 65
  };                                                                                                                   // 62
};                                                                                                                     // 67
                                                                                                                       //
var closePopup = function (res, err) {                                                                                 // 69
  res.writeHead(200, {                                                                                                 // 70
    "Content-Type": "text/html"                                                                                        // 70
  });                                                                                                                  // 70
  var content = "<html><head><script>window.close()</script></head></html>";                                           // 71
  if (err) content = "<html><body><h2>Sorry, an error occured</h2><div>" + err + '</div><a onclick="window.close();">Close Window</a> <a href="/admin/status" target="_blank">View system log (admin only)</a></body></html>';
  res.end(content, "utf-8");                                                                                           // 75
};                                                                                                                     // 76
                                                                                                                       //
var generateService = function () {                                                                                    // 78
  // TODO(cleanup): Inject the db.                                                                                     // 79
  var db = new SandstormDb();                                                                                          // 80
  var entityId = db.getSamlEntityId();                                                                                 // 82
  var service = {                                                                                                      // 83
    "provider": "default",                                                                                             // 84
    "entryPoint": db.getSamlEntryPoint(),                                                                              // 85
    "logoutUrl": db.getSamlLogout(),                                                                                   // 86
    // TODO(someday): find a better way to inject the DB                                                               // 87
    "issuer": entityId || HOSTNAME,                                                                                    // 88
    // If the certificate has "-----BEGIN CERTIFICATE-----" markers, automatically remove those.                       // 89
    "cert": db.getSamlPublicCert().replace(/-[^\n]*-/g, "").trim()                                                     // 90
  };                                                                                                                   // 83
  return service;                                                                                                      // 92
};                                                                                                                     // 93
                                                                                                                       //
var middleware = function (req, res, next) {                                                                           // 95
  // Make sure to catch any exceptions because otherwise we'd crash                                                    // 96
  // the runner                                                                                                        // 97
  try {                                                                                                                // 98
    var samlObject = samlUrlToObject(req.url);                                                                         // 99
                                                                                                                       //
    if (!samlObject || !samlObject.serviceName) {                                                                      // 100
      next();                                                                                                          // 101
      return;                                                                                                          // 102
    }                                                                                                                  // 103
                                                                                                                       //
    if (samlObject.actionName === "config") {                                                                          // 105
      var _saml = new SAML(generateService());                                                                         // 106
                                                                                                                       //
      res.writeHead(200, {                                                                                             // 107
        "Content-Type": "text/xml"                                                                                     // 107
      });                                                                                                              // 107
      res.end(_saml.generateServiceProviderMetadata());                                                                // 108
      return;                                                                                                          // 109
    }                                                                                                                  // 110
                                                                                                                       //
    if (!Accounts.loginServices.saml.isEnabled()) {                                                                    // 112
      next();                                                                                                          // 113
      return;                                                                                                          // 114
    }                                                                                                                  // 115
                                                                                                                       //
    if (!samlObject.actionName) throw new Error("Missing SAML action");                                                // 117
    var service = generateService(); // Skip everything if there's no service set by the saml middleware               // 120
                                                                                                                       //
    if (!service || samlObject.serviceName !== service.provider) throw new Error("Unexpected SAML service " + samlObject.serviceName);
                                                                                                                       //
    if (samlObject.actionName === "authorize") {                                                                       // 126
      service.callbackUrl = Meteor.absoluteUrl("_saml/validate/" + service.provider);                                  // 127
      service.id = samlObject.credentialToken;                                                                         // 128
                                                                                                                       //
      var _saml2 = new SAML(service);                                                                                  // 129
                                                                                                                       //
      _saml2.getAuthorizeUrl(req, function (err, url) {                                                                // 130
        if (err) throw new Error("Unable to generate authorize url");                                                  // 131
        res.writeHead(302, {                                                                                           // 133
          "Location": url                                                                                              // 133
        });                                                                                                            // 133
        res.end();                                                                                                     // 134
      });                                                                                                              // 135
    } else if (samlObject.actionName === "validate") {                                                                 // 136
      var _saml3 = new SAML(service);                                                                                  // 137
                                                                                                                       //
      _saml3.validateResponse(req.body.SAMLResponse, function (err, profile, loggedOut, responseText) {                // 138
        if (err) {                                                                                                     // 140
          console.error("Error validating SAML response:", err.toString(), "\nFull SAML response XML:\n", responseText);
          throw new Error("Unable to validate SAML response.");                                                        // 143
        } // Do NOT use samlObject.credentialToken; it isn't signed!                                                   // 144
                                                                                                                       //
                                                                                                                       //
        var credentialToken = profile.inResponseToId || profile.InResponseTo;                                          // 147
                                                                                                                       //
        if (!credentialToken) {                                                                                        // 148
          throw new Error("SAML response missing InResponseTo attribute. Sandstorm does not support " + "IdP-initiated authentication; authentication requests must start " + "from the user choosing SAML login in the Sandstorm UI.");
        }                                                                                                              // 153
                                                                                                                       //
        _loginResultForCredentialToken[credentialToken] = {                                                            // 155
          profile: profile                                                                                             // 156
        };                                                                                                             // 155
        closePopup(res);                                                                                               // 159
      });                                                                                                              // 160
    } else {                                                                                                           // 161
      throw new Error("Unexpected SAML action " + samlObject.actionName);                                              // 162
    }                                                                                                                  // 163
  } catch (err) {                                                                                                      // 164
    console.error(err.stack);                                                                                          // 165
    closePopup(res, err);                                                                                              // 166
  }                                                                                                                    // 167
}; // Listen to incoming OAuth http requests                                                                           // 168
                                                                                                                       //
                                                                                                                       //
WebApp.connectHandlers.use(connect.urlencoded()).use(function (req, res, next) {                                       // 171
  // Need to create a Fiber since we're using synchronous http calls and nothing                                       // 172
  // else is wrapping this in a fiber automatically                                                                    // 173
  Fiber(function () {                                                                                                  // 174
    middleware(req, res, next);                                                                                        // 175
  }).run();                                                                                                            // 176
});                                                                                                                    // 177
Meteor.methods({                                                                                                       // 179
  generateSamlLogout: function () {                                                                                    // 180
    var service = generateService();                                                                                   // 181
                                                                                                                       //
    if (!service.logoutUrl) {                                                                                          // 182
      throw new Meteor.Error(500, "No SAML logout url specified");                                                     // 183
    }                                                                                                                  // 184
                                                                                                                       //
    var _saml = new SAML(service);                                                                                     // 186
                                                                                                                       //
    var credential = void 0;                                                                                           // 187
    Meteor.user().loginCredentials.forEach(function (_credential) {                                                    // 188
      var currCredential = Meteor.users.findOne({                                                                      // 189
        _id: _credential.id                                                                                            // 189
      });                                                                                                              // 189
                                                                                                                       //
      if (currCredential.services.saml) {                                                                              // 190
        credential = currCredential;                                                                                   // 191
      }                                                                                                                // 192
    }); // TODO(someday): handle user having more than one SAML credential                                             // 193
                                                                                                                       //
    return Meteor.wrapAsync(_saml.getLogoutUrl.bind(_saml))({                                                          // 196
      user: {                                                                                                          // 197
        nameID: credential.services.saml.id,                                                                           // 198
        nameIDFormat: credential.services.saml.nameIDFormat || "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent"  // 199
      }                                                                                                                // 197
    });                                                                                                                // 196
  },                                                                                                                   // 203
  validateSamlLogout: function (samlRequest) {                                                                         // 205
    check(samlRequest, String);                                                                                        // 206
    var db = this.connection.sandstormDb;                                                                              // 208
    var userId = Meteor.userId();                                                                                      // 209
                                                                                                                       //
    if (!userId) {                                                                                                     // 210
      return new Meteor.Error(403, "Non-logged in users can't logout.");                                               // 211
    }                                                                                                                  // 212
                                                                                                                       //
    var service = generateService();                                                                                   // 214
                                                                                                                       //
    var _saml = new SAML(service);                                                                                     // 215
                                                                                                                       //
    if (samlRequest) {                                                                                                 // 216
      var buf = new Buffer(samlRequest, "base64");                                                                     // 217
      var xml = zlib.inflateRawSync(buf).toString();                                                                   // 218
                                                                                                                       //
      _saml.parseLogoutRequest(xml, function (err, nameId) {                                                           // 219
        if (err) {                                                                                                     // 220
          console.error("Error validating SAML logout response:", err.toString(), "\nFull SAML response XML:\n", responseText);
          throw new Error("Unable to validate SAML logout response.");                                                 // 223
        }                                                                                                              // 224
                                                                                                                       //
        check(nameId, String);                                                                                         // 226
        var credential = db.collections.users.findOne({                                                                // 227
          "services.saml.id": nameId                                                                                   // 227
        }, {                                                                                                           // 227
          fields: {                                                                                                    // 228
            _id: 1                                                                                                     // 228
          }                                                                                                            // 228
        });                                                                                                            // 228
                                                                                                                       //
        if (!credential) {                                                                                             // 229
          return new Meteor.Error(400, "No credential found matching SAML nameID.");                                   // 230
        }                                                                                                              // 231
                                                                                                                       //
        var user = db.collections.users.findOne({                                                                      // 233
          "loginCredentials.id": credential._id                                                                        // 233
        }, {                                                                                                           // 233
          fields: {                                                                                                    // 234
            _id: 1                                                                                                     // 234
          }                                                                                                            // 234
        });                                                                                                            // 234
                                                                                                                       //
        if (!user) {                                                                                                   // 235
          return new Meteor.Error(403, "No user found for expected SAML credential.");                                 // 236
        }                                                                                                              // 237
                                                                                                                       //
        if (user._id !== userId) {                                                                                     // 239
          var txt = "SAML logout requested for wrong user: " + nameId + ", " + userId;                                 // 240
          console.error(txt);                                                                                          // 241
          throw new Error(txt);                                                                                        // 242
        }                                                                                                              // 243
      });                                                                                                              // 244
    }                                                                                                                  // 245
  }                                                                                                                    // 246
});                                                                                                                    // 179
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"accounts-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/accounts-server.js                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var Accounts = void 0;                                                                                                 // 1
module.watch(require("meteor/accounts-base"), {                                                                        // 1
  Accounts: function (v) {                                                                                             // 1
    Accounts = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var fetchPicture = void 0,                                                                                             // 1
    userPictureUrl = void 0;                                                                                           // 1
module.watch(require("/imports/server/accounts/picture.js"), {                                                         // 1
  fetchPicture: function (v) {                                                                                         // 1
    fetchPicture = v;                                                                                                  // 1
  },                                                                                                                   // 1
  userPictureUrl: function (v) {                                                                                       // 1
    userPictureUrl = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 20
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 21
                                                                                                                       //
var ValidHandle = Match.Where(function (handle) {                                                                      // 23
  check(handle, String);                                                                                               // 24
  return !!handle.match(/^[a-z_][a-z0-9_]*$/);                                                                         // 25
});                                                                                                                    // 26
Accounts.onCreateUser(function (options, user) {                                                                       // 28
  if (!user.type) {                                                                                                    // 29
    if (user.loginCredentials) throw new Error("please set user.type");                                                // 30
    user.type = "credential";                                                                                          // 32
  }                                                                                                                    // 33
                                                                                                                       //
  if (user.type === "account") {                                                                                       // 35
    // it's an account                                                                                                 // 36
    check(user, {                                                                                                      // 37
      _id: String,                                                                                                     // 37
      type: String,                                                                                                    // 38
      createdAt: Date,                                                                                                 // 39
      isAdmin: Match.Optional(Boolean),                                                                                // 40
      hasCompletedSignup: Match.Optional(Boolean),                                                                     // 41
      signupKey: Match.Optional(String),                                                                               // 42
      signupNote: Match.Optional(String),                                                                              // 43
      signupEmail: Match.Optional(String),                                                                             // 44
      expires: Match.Optional(Date),                                                                                   // 45
      appDemoId: Match.Optional(String),                                                                               // 46
      loginCredentials: [{                                                                                             // 47
        id: String                                                                                                     // 47
      }],                                                                                                              // 47
      nonloginCredentials: [{                                                                                          // 48
        id: String                                                                                                     // 48
      }]                                                                                                               // 48
    });                                                                                                                // 37
                                                                                                                       //
    if (globalDb.isReferralEnabled()) {                                                                                // 50
      user.experiments = user.experiments || {};                                                                       // 51
      user.experiments = {                                                                                             // 52
        freeGrainLimit: Math.random() < 0.1 ? 100 : "control"                                                          // 53
      };                                                                                                               // 52
                                                                                                                       //
      if (!("expires" in user)) {                                                                                      // 55
        globalDb.sendReferralProgramNotification(user._id);                                                            // 56
      }                                                                                                                // 57
    }                                                                                                                  // 58
                                                                                                                       //
    if (!options.profile) throw new Error("missing profile");                                                          // 60
    user.profile = options.profile;                                                                                    // 62
    globalDb.preinstallAppsForUser(user._id);                                                                          // 64
    return user;                                                                                                       // 66
  }                                                                                                                    // 67
                                                                                                                       //
  if (globalDb.getOrganizationDisallowGuests() && !globalDb.isCredentialInOrganization(user)) {                        // 69
    throw new Meteor.Error(400, "User not in organization.");                                                          // 71
  } // Check profile.                                                                                                  // 72
                                                                                                                       //
                                                                                                                       //
  if (options.profile) {                                                                                               // 75
    // TODO(cleanup): This check also appears in accounts-ui-methods.js.                                               // 76
    check(options.profile, Match.ObjectIncluding({                                                                     // 77
      name: Match.OneOf(null, Match.Optional(String)),                                                                 // 78
      handle: Match.Optional(ValidHandle),                                                                             // 79
      pronoun: Match.Optional(Match.OneOf("male", "female", "neutral", "robot"))                                       // 80
    }));                                                                                                               // 77
  }                                                                                                                    // 82
                                                                                                                       //
  check(options.unverifiedEmail, Match.Optional(String));                                                              // 84
                                                                                                                       //
  if (options.unverifiedEmail) {                                                                                       // 86
    user.unverifiedEmail = options.unverifiedEmail;                                                                    // 87
  }                                                                                                                    // 88
                                                                                                                       //
  user.profile = _.pick(options.profile || {}, "name", "handle", "pronoun"); // Try downloading avatar.                // 90
                                                                                                                       //
  var url = userPictureUrl(user);                                                                                      // 93
                                                                                                                       //
  if (url) {                                                                                                           // 94
    var assetId = fetchPicture(globalDb, url);                                                                         // 95
                                                                                                                       //
    if (assetId) {                                                                                                     // 96
      user.profile.picture = assetId;                                                                                  // 97
    }                                                                                                                  // 98
  }                                                                                                                    // 99
                                                                                                                       //
  var serviceUserId = void 0;                                                                                          // 101
                                                                                                                       //
  if (user.services && user.services.dev) {                                                                            // 102
    check(user.services.dev, {                                                                                         // 103
      name: String,                                                                                                    // 103
      isAdmin: Boolean,                                                                                                // 103
      hasCompletedSignup: Boolean                                                                                      // 103
    });                                                                                                                // 103
    serviceUserId = user.services.dev.name;                                                                            // 104
  } else if ("expires" in user) {                                                                                      // 105
    serviceUserId = user._id;                                                                                          // 106
    user.services = {                                                                                                  // 107
      demo: {}                                                                                                         // 107
    };                                                                                                                 // 107
  } else if (user.services && user.services.email) {                                                                   // 108
    check(user.services.email, {                                                                                       // 109
      email: String,                                                                                                   // 110
      tokens: [{                                                                                                       // 111
        digest: String,                                                                                                // 113
        algorithm: String,                                                                                             // 114
        createdAt: Date,                                                                                               // 115
        secureBox: Match.Optional({                                                                                    // 116
          version: Number,                                                                                             // 117
          salt: String,                                                                                                // 118
          iv: String,                                                                                                  // 119
          boxedValue: String                                                                                           // 120
        })                                                                                                             // 116
      }]                                                                                                               // 112
    });                                                                                                                // 109
    serviceUserId = user.services.email.email;                                                                         // 125
  } else if (user.services && "google" in user.services) {                                                             // 126
    serviceUserId = user.services.google.id;                                                                           // 127
  } else if (user.services && "github" in user.services) {                                                             // 128
    serviceUserId = user.services.github.id;                                                                           // 129
  } else if (user.services && "ldap" in user.services) {                                                               // 130
    serviceUserId = user.services.ldap.id;                                                                             // 131
  } else if (user.services && "saml" in user.services) {                                                               // 132
    serviceUserId = user.services.saml.id;                                                                             // 133
  } else {                                                                                                             // 134
    throw new Meteor.Error(400, "user does not have a recognized login provider: " + JSON.stringify(user));            // 135
  }                                                                                                                    // 137
                                                                                                                       //
  user._id = Crypto.createHash("sha256").update(SandstormDb.getServiceName(user) + ":" + serviceUserId).digest("hex");
  return user;                                                                                                         // 142
});                                                                                                                    // 143
Accounts.validateLoginAttempt(function (attempt) {                                                                     // 145
  if (!attempt.allowed) {                                                                                              // 146
    return false;                                                                                                      // 147
  }                                                                                                                    // 148
                                                                                                                       //
  var db = attempt.connection.sandstormDb;                                                                             // 150
  var user = attempt.user;                                                                                             // 151
                                                                                                                       //
  if (user.suspended && user.suspended.admin) {                                                                        // 153
    throw new Meteor.Error(403, "User has been suspended. Please contact the administrator " + "of this server if you believe this to be in error.");
  }                                                                                                                    // 156
                                                                                                                       //
  if (user.loginCredentials) {                                                                                         // 158
    // it's an account                                                                                                 // 159
    if (db.getOrganizationDisallowGuests() && !db.isUserInOrganization(user)) {                                        // 160
      throw new Meteor.Error(403, "User not in organization.");                                                        // 162
    }                                                                                                                  // 163
                                                                                                                       //
    db.updateUserQuota(user); // This is a no-op if settings aren't enabled                                            // 165
  } else {                                                                                                             // 166
    if (db.getOrganizationDisallowGuests() && !db.isCredentialInOrganization(user)) {                                  // 167
      throw new Meteor.Error(403, "User not in organization.");                                                        // 169
    }                                                                                                                  // 170
  }                                                                                                                    // 171
                                                                                                                       //
  return true;                                                                                                         // 173
});                                                                                                                    // 174
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"accounts-ui-methods.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/accounts-ui-methods.js                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
// This file contains method definitions which are available on both client and server (on the                         // 17
// client for prediction purposes, on the server for actual execution).                                                // 18
var ValidHandle = Match.Where(function (handle) {                                                                      // 20
  check(handle, String);                                                                                               // 21
  return !!handle.match(/^[a-z_][a-z0-9_]*$/);                                                                         // 22
});                                                                                                                    // 23
Meteor.methods({                                                                                                       // 25
  updateProfile: function (obsolete, profile) {                                                                        // 26
    check(profile, {                                                                                                   // 27
      name: String,                                                                                                    // 28
      handle: ValidHandle,                                                                                             // 29
      pronoun: Match.OneOf("male", "female", "neutral", "robot"),                                                      // 30
      unverifiedEmail: Match.Optional(String)                                                                          // 31
    });                                                                                                                // 27
                                                                                                                       //
    if (!this.userId) {                                                                                                // 34
      throw new Meteor.Error(403, "not logged in");                                                                    // 35
    }                                                                                                                  // 36
                                                                                                                       //
    var userToUpdate = Meteor.user();                                                                                  // 38
    var newValues = {                                                                                                  // 40
      "profile.name": profile.name,                                                                                    // 41
      "profile.handle": profile.handle,                                                                                // 42
      "profile.pronoun": profile.pronoun                                                                               // 43
    };                                                                                                                 // 40
    Meteor.users.update({                                                                                              // 46
      _id: userToUpdate._id                                                                                            // 46
    }, {                                                                                                               // 46
      $set: newValues                                                                                                  // 46
    });                                                                                                                // 46
                                                                                                                       //
    if (!Meteor.user().hasCompletedSignup) {                                                                           // 48
      Meteor.users.update({                                                                                            // 49
        _id: this.userId                                                                                               // 49
      }, {                                                                                                             // 49
        $set: {                                                                                                        // 49
          hasCompletedSignup: true                                                                                     // 49
        }                                                                                                              // 49
      });                                                                                                              // 49
    }                                                                                                                  // 50
  },                                                                                                                   // 51
  testFirstSignup: function (profile) {                                                                                // 53
    if (!this.userId) {                                                                                                // 54
      throw new Meteor.Error(403, "not logged in");                                                                    // 55
    }                                                                                                                  // 56
                                                                                                                       //
    Meteor.users.update(this.userId, {                                                                                 // 58
      $unset: {                                                                                                        // 58
        hasCompletedSignup: ""                                                                                         // 58
      }                                                                                                                // 58
    });                                                                                                                // 58
  },                                                                                                                   // 59
  uploadProfilePicture: function (obsolete) {                                                                          // 61
    if (!this.userId) {                                                                                                // 62
      throw new Meteor.Error(403, "not logged in");                                                                    // 63
    }                                                                                                                  // 64
                                                                                                                       //
    return this.connection.sandstormDb.newAssetUpload({                                                                // 66
      profilePicture: {                                                                                                // 67
        userId: this.userId                                                                                            // 67
      }                                                                                                                // 67
    });                                                                                                                // 66
  },                                                                                                                   // 69
  cancelUploadProfilePicture: function (id) {                                                                          // 71
    check(id, String);                                                                                                 // 72
    this.connection.sandstormDb.fulfillAssetUpload(id);                                                                // 73
  },                                                                                                                   // 74
  setPrimaryEmail: function (email) {                                                                                  // 76
    check(email, String);                                                                                              // 77
                                                                                                                       //
    if (!this.userId) {                                                                                                // 78
      throw new Meteor.Error(403, "Not logged in.");                                                                   // 79
    }                                                                                                                  // 80
                                                                                                                       //
    var emails = SandstormDb.getUserEmails(Meteor.user());                                                             // 82
                                                                                                                       //
    if (!_.findWhere(emails, {                                                                                         // 83
      email: email,                                                                                                    // 83
      verified: true                                                                                                   // 83
    })) {                                                                                                              // 83
      throw new Meteor.Error(403, "Not a verified email of the current user: " + email);                               // 84
    }                                                                                                                  // 85
                                                                                                                       //
    Meteor.users.update({                                                                                              // 87
      _id: this.userId                                                                                                 // 87
    }, {                                                                                                               // 87
      $set: {                                                                                                          // 87
        primaryEmail: email                                                                                            // 87
      }                                                                                                                // 87
    });                                                                                                                // 87
  }                                                                                                                    // 88
});                                                                                                                    // 25
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"accounts-ui-server.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/accounts/accounts-ui-server.js                                                                               //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
// Meteor permits users to modify their own profile by default, for some reason.                                       // 17
Meteor.users.deny({                                                                                                    // 18
  insert: function () {                                                                                                // 19
    return true;                                                                                                       // 19
  },                                                                                                                   // 19
  update: function () {                                                                                                // 21
    return true;                                                                                                       // 21
  },                                                                                                                   // 21
  remove: function () {                                                                                                // 23
    return true;                                                                                                       // 23
  },                                                                                                                   // 23
  fetch: []                                                                                                            // 25
});                                                                                                                    // 18
Meteor.publish("getMyUsage", function () {                                                                             // 28
  var db = this.connection.sandstormDb;                                                                                // 29
                                                                                                                       //
  if (this.userId) {                                                                                                   // 30
    // TODO(someday): Make this reactive.                                                                              // 31
    var user = Meteor.users.findOne(this.userId);                                                                      // 32
    var usage = db.getMyUsage(user);                                                                                   // 33
    this.added("users", this.userId, {                                                                                 // 34
      pseudoUsage: usage                                                                                               // 35
    });                                                                                                                // 34
  }                                                                                                                    // 37
                                                                                                                       //
  this.ready();                                                                                                        // 39
});                                                                                                                    // 40
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"admin":{"admin-alert-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/admin/admin-alert-server.js                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var Match = void 0,                                                                                                    // 1
    check = void 0;                                                                                                    // 1
module.watch(require("meteor/check"), {                                                                                // 1
  Match: function (v) {                                                                                                // 1
    Match = v;                                                                                                         // 1
  },                                                                                                                   // 1
  check: function (v) {                                                                                                // 1
    check = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var checkAuth = void 0;                                                                                                // 1
module.watch(require("/imports/server/auth.js"), {                                                                     // 1
  checkAuth: function (v) {                                                                                            // 1
    checkAuth = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var maintenanceMessageShape = {                                                                                        // 5
  text: String,                                                                                                        // 6
  time: Match.OneOf(Date, undefined, null),                                                                            // 7
  url: Match.Optional(String)                                                                                          // 8
};                                                                                                                     // 5
Meteor.methods({                                                                                                       // 11
  setMaintenanceMessage: function (params) {                                                                           // 12
    checkAuth(undefined);                                                                                              // 13
    check(params, maintenanceMessageShape);                                                                            // 14
    var db = this.connection.sandstormDb; // TODO(soon): make this a single write to a single settings object          // 15
                                                                                                                       //
    db.collections.settings.upsert({                                                                                   // 17
      _id: "adminAlertTime"                                                                                            // 17
    }, {                                                                                                               // 17
      value: params.time                                                                                               // 17
    });                                                                                                                // 17
    db.collections.settings.upsert({                                                                                   // 18
      _id: "adminAlertUrl"                                                                                             // 18
    }, {                                                                                                               // 18
      value: params.url                                                                                                // 18
    });                                                                                                                // 18
    db.collections.settings.upsert({                                                                                   // 19
      _id: "adminAlert"                                                                                                // 19
    }, {                                                                                                               // 19
      value: params.text                                                                                               // 19
    });                                                                                                                // 19
  }                                                                                                                    // 20
});                                                                                                                    // 11
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"admin-user-invite.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/admin/admin-user-invite.js                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
Meteor.publish("allInviteTokens", function () {                                                                        // 1
  var db = this.connection.sandstormDb;                                                                                // 2
                                                                                                                       //
  if (!db.isAdminById(this.userId)) {                                                                                  // 3
    throw new Meteor.Error(403, "User must be admin to list invite tokens.");                                          // 4
  }                                                                                                                    // 5
                                                                                                                       //
  return db.collections.signupKeys.find({});                                                                           // 7
});                                                                                                                    // 8
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"network-capabilities-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/admin/network-capabilities-server.js                                                                         //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var check = void 0;                                                                                                    // 1
module.watch(require("meteor/check"), {                                                                                // 1
  check: function (v) {                                                                                                // 1
    check = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var checkAuth = void 0;                                                                                                // 1
module.watch(require("/imports/server/auth.js"), {                                                                     // 1
  checkAuth: function (v) {                                                                                            // 1
    checkAuth = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
Meteor.publish("adminGrains", function (grainIds) {                                                                    // 5
  // If the caller is an admin, publishes the Grains referred to by the provided list of grain IDs.                    // 6
  // Otherwise, does nothing.                                                                                          // 7
  if (!isAdminById(this.userId)) return [];                                                                            // 8
  check(grainIds, [String]);                                                                                           // 9
  var db = this.connection.sandstormDb;                                                                                // 11
  return db.collections.grains.find({                                                                                  // 12
    _id: {                                                                                                             // 13
      $in: grainIds                                                                                                    // 14
    }                                                                                                                  // 13
  }, {                                                                                                                 // 12
    fields: {                                                                                                          // 17
      title: 1,                                                                                                        // 18
      packageId: 1                                                                                                     // 19
    }                                                                                                                  // 17
  });                                                                                                                  // 16
});                                                                                                                    // 22
Meteor.publish("adminPackages", function (packageIds) {                                                                // 24
  // If the caller is an admin, publishes the Packages referred to by the provided list of package                     // 25
  // IDs.  Otherwise, does nothing.                                                                                    // 26
  if (!isAdminById(this.userId)) return [];                                                                            // 27
  check(packageIds, [String]);                                                                                         // 28
  var db = this.connection.sandstormDb;                                                                                // 30
  return db.collections.packages.find({                                                                                // 31
    _id: {                                                                                                             // 32
      $in: packageIds                                                                                                  // 33
    }                                                                                                                  // 32
  }, {                                                                                                                 // 31
    fields: {                                                                                                          // 36
      manifest: 1                                                                                                      // 37
    }                                                                                                                  // 36
  });                                                                                                                  // 35
});                                                                                                                    // 40
Meteor.publish("adminProfiles", function (userIds) {                                                                   // 42
  // If the caller is an admin, publishes the Users listed by ID in userIds.                                           // 43
  // Otherwise, does nothing.                                                                                          // 44
  if (!isAdminById(this.userId)) return [];                                                                            // 45
  check(userIds, [String]);                                                                                            // 46
  var db = this.connection.sandstormDb;                                                                                // 48
  return db.collections.users.find({                                                                                   // 49
    _id: {                                                                                                             // 50
      $in: userIds                                                                                                     // 51
    }                                                                                                                  // 50
  }, {                                                                                                                 // 49
    fields: {                                                                                                          // 53
      profile: 1                                                                                                       // 53
    }                                                                                                                  // 53
  });                                                                                                                  // 53
});                                                                                                                    // 54
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"personalization-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/admin/personalization-server.js                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var check = void 0;                                                                                                    // 1
module.watch(require("meteor/check"), {                                                                                // 1
  check: function (v) {                                                                                                // 1
    check = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var checkAuth = void 0;                                                                                                // 1
module.watch(require("/imports/server/auth.js"), {                                                                     // 1
  checkAuth: function (v) {                                                                                            // 1
    checkAuth = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var personalizationMessageShape = {                                                                                    // 5
  serverTitle: String,                                                                                                 // 6
  splashUrl: String,                                                                                                   // 7
  signupDialog: String,                                                                                                // 8
  termsOfServiceUrl: String,                                                                                           // 9
  privacyPolicyUrl: String,                                                                                            // 10
  whitelabelCustomLoginProviderName: String,                                                                           // 12
  whitelabelHideSendFeedback: Boolean,                                                                                 // 13
  whitelabelHideTroubleshooting: Boolean,                                                                              // 14
  whiteLabelHideAbout: Boolean,                                                                                        // 15
  whitelabelUseServerTitleForHomeText: Boolean                                                                         // 16
};                                                                                                                     // 5
Meteor.methods({                                                                                                       // 19
  setPersonalizationSettings: function (params) {                                                                      // 20
    checkAuth(undefined);                                                                                              // 21
    check(params, personalizationMessageShape);                                                                        // 22
    var db = this.connection.sandstormDb; // TODO(soon): make this a single write to a single settings object          // 23
                                                                                                                       //
    db.collections.settings.upsert({                                                                                   // 25
      _id: "serverTitle"                                                                                               // 25
    }, {                                                                                                               // 25
      value: params.serverTitle                                                                                        // 25
    });                                                                                                                // 25
    db.collections.settings.upsert({                                                                                   // 26
      _id: "splashUrl"                                                                                                 // 26
    }, {                                                                                                               // 26
      value: params.splashUrl                                                                                          // 26
    });                                                                                                                // 26
    db.collections.settings.upsert({                                                                                   // 27
      _id: "signupDialog"                                                                                              // 27
    }, {                                                                                                               // 27
      value: params.signupDialog                                                                                       // 27
    });                                                                                                                // 27
    db.collections.settings.upsert({                                                                                   // 28
      _id: "termsUrl"                                                                                                  // 28
    }, {                                                                                                               // 28
      value: params.termsOfServiceUrl                                                                                  // 28
    });                                                                                                                // 28
    db.collections.settings.upsert({                                                                                   // 29
      _id: "privacyUrl"                                                                                                // 29
    }, {                                                                                                               // 29
      value: params.privacyPolicyUrl                                                                                   // 29
    });                                                                                                                // 29
    db.collections.settings.upsert({                                                                                   // 31
      _id: "whitelabelCustomLoginProviderName"                                                                         // 31
    }, {                                                                                                               // 31
      value: params.whitelabelCustomLoginProviderName                                                                  // 32
    });                                                                                                                // 32
    db.collections.settings.upsert({                                                                                   // 33
      _id: "whitelabelHideSendFeedback"                                                                                // 33
    }, {                                                                                                               // 33
      value: params.whitelabelHideSendFeedback                                                                         // 34
    });                                                                                                                // 34
    db.collections.settings.upsert({                                                                                   // 35
      _id: "whitelabelHideTroubleshooting"                                                                             // 35
    }, {                                                                                                               // 35
      value: params.whitelabelHideTroubleshooting                                                                      // 36
    });                                                                                                                // 36
    db.collections.settings.upsert({                                                                                   // 37
      _id: "whiteLabelHideAbout"                                                                                       // 37
    }, {                                                                                                               // 37
      value: params.whiteLabelHideAbout                                                                                // 38
    });                                                                                                                // 38
    db.collections.settings.upsert({                                                                                   // 39
      _id: "whitelabelUseServerTitleForHomeText"                                                                       // 39
    }, {                                                                                                               // 39
      value: params.whitelabelUseServerTitleForHomeText                                                                // 40
    });                                                                                                                // 40
  },                                                                                                                   // 41
  getWhitelabelLogoUploadToken: function () {                                                                          // 43
    checkAuth(undefined);                                                                                              // 44
    var db = this.connection.sandstormDb;                                                                              // 45
    return db.newAssetUpload({                                                                                         // 46
      loginLogo: {}                                                                                                    // 46
    });                                                                                                                // 46
  },                                                                                                                   // 47
  resetWhitelabelLogo: function () {                                                                                   // 49
    checkAuth(undefined);                                                                                              // 50
    var db = this.connection.sandstormDb;                                                                              // 51
    var result = globalDb.collections.settings.findAndModify({                                                         // 52
      query: {                                                                                                         // 53
        _id: "whitelabelCustomLogoAssetId"                                                                             // 53
      },                                                                                                               // 53
      remove: true,                                                                                                    // 54
      fields: {                                                                                                        // 55
        value: 1                                                                                                       // 55
      }                                                                                                                // 55
    });                                                                                                                // 52
                                                                                                                       //
    if (result.ok) {                                                                                                   // 58
      var old = result.value;                                                                                          // 59
                                                                                                                       //
      if (old) {                                                                                                       // 60
        db.unrefStaticAsset(old.value);                                                                                // 61
      }                                                                                                                // 62
    } else {                                                                                                           // 63
      throw new Meteor.Error(500, "Couldn't remove whitelabelCustomLogoAssetId");                                      // 64
    }                                                                                                                  // 65
  }                                                                                                                    // 66
});                                                                                                                    // 19
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"system-status-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/admin/system-status-server.js                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var Router = void 0;                                                                                                   // 1
module.watch(require("meteor/iron:router"), {                                                                          // 1
  Router: function (v) {                                                                                               // 1
    Router = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Random = void 0;                                                                                                   // 1
module.watch(require("meteor/random"), {                                                                               // 1
  Random: function (v) {                                                                                               // 1
    Random = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var Crypto = void 0;                                                                                                   // 1
module.watch(require("crypto"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    Crypto = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var Fs = void 0;                                                                                                       // 1
module.watch(require("fs"), {                                                                                          // 1
  "default": function (v) {                                                                                            // 1
    Fs = v;                                                                                                            // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var checkAuth = void 0;                                                                                                // 1
module.watch(require("/imports/server/auth.js"), {                                                                     // 1
  checkAuth: function (v) {                                                                                            // 1
    checkAuth = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
var SANDSTORM_LOGDIR = void 0;                                                                                         // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  SANDSTORM_LOGDIR: function (v) {                                                                                     // 1
    SANDSTORM_LOGDIR = v;                                                                                              // 1
  }                                                                                                                    // 1
}, 6);                                                                                                                 // 1
                                                                                                                       //
var hashSessionId = function (sessionId) {                                                                             // 10
  return Crypto.createHash("sha256").update(sessionId).digest("base64");                                               // 11
}; // An in-memory map of random id -> timestamp, used to authorize requests to download the system                    // 12
// log.                                                                                                                // 15
                                                                                                                       //
                                                                                                                       //
var SYSTEM_LOG_DOWNLOAD_TOKENS = {};                                                                                   // 16
var SYSTEM_LOG_DOWNLOAD_TOKEN_VALIDITY_DURATION = 60000; // 60 seconds                                                 // 17
                                                                                                                       //
Meteor.methods({                                                                                                       // 19
  adminGetServerLogDownloadToken: function (setupToken) {                                                              // 20
    checkAuth(setupToken);                                                                                             // 21
    var token = Random.id(22);                                                                                         // 23
    SYSTEM_LOG_DOWNLOAD_TOKENS[token] = Date.now();                                                                    // 24
    setTimeout(function () {                                                                                           // 25
      if (SYSTEM_LOG_DOWNLOAD_TOKENS[token]) {                                                                         // 26
        delete SYSTEM_LOG_DOWNLOAD_TOKENS[token];                                                                      // 27
      }                                                                                                                // 28
    }, SYSTEM_LOG_DOWNLOAD_TOKEN_VALIDITY_DURATION * 2);                                                               // 29
    return token;                                                                                                      // 30
  }                                                                                                                    // 31
});                                                                                                                    // 19
Router.map(function () {                                                                                               // 34
  this.route("adminDownloadServerLog", {                                                                               // 35
    where: "server",                                                                                                   // 36
    path: "/admin/status/server-log/:tokenId",                                                                         // 37
    action: function () {                                                                                              // 38
      var token = this.params.tokenId;                                                                                 // 39
      var response = this.response;                                                                                    // 40
      var issueDate = SYSTEM_LOG_DOWNLOAD_TOKENS[token];                                                               // 41
      delete SYSTEM_LOG_DOWNLOAD_TOKENS[token]; // Clear the token from the in-memory map.                             // 42
                                                                                                                       //
      if (issueDate === undefined || issueDate + SYSTEM_LOG_DOWNLOAD_TOKEN_VALIDITY_DURATION < Date.now()) {           // 43
        // Require download to start within 60 seconds of the adminGetServerLogDownloadToken                           // 45
        // method being called.                                                                                        // 46
        response.writeHead(404, {                                                                                      // 47
          "Content-Type": "text/plain"                                                                                 // 47
        });                                                                                                            // 47
        return response.end("Invalid server log download token.");                                                     // 48
      }                                                                                                                // 49
                                                                                                                       //
      var logFilePath = SANDSTORM_LOGDIR + "/sandstorm.log";                                                           // 51
      var fd = Fs.openSync(logFilePath, "r");                                                                          // 52
      var stats = Fs.fstatSync(fd);                                                                                    // 53
      var initialSize = stats.size;                                                                                    // 54
      var readStream = Fs.createReadStream(undefined, {                                                                // 55
        fd: fd,                                                                                                        // 56
        start: 0,                                                                                                      // 57
        end: initialSize                                                                                               // 58
      });                                                                                                              // 55
      response.writeHead(200, {                                                                                        // 61
        "Content-Length": initialSize,                                                                                 // 62
        "Content-Type": "text/plain",                                                                                  // 63
        "content-Disposition": "attachment;filename=\"sandstorm.log\""                                                 // 64
      });                                                                                                              // 61
      readStream.pipe(response);                                                                                       // 66
    }                                                                                                                  // 67
  });                                                                                                                  // 35
});                                                                                                                    // 69
Meteor.publish("systemStatus", function () {                                                                           // 71
  // A pseudocollection containing the number of active grain sessions and unique                                      // 72
  // userIds associated with these sessions                                                                            // 73
  var db = this.connection.sandstormDb;                                                                                // 74
                                                                                                                       //
  if (!db.isAdminById(this.userId)) {                                                                                  // 75
    throw new Meteor.Error(403, "User must be admin to view system status.");                                          // 76
  }                                                                                                                    // 77
                                                                                                                       //
  var _this = this;                                                                                                    // 79
                                                                                                                       //
  var userIdToSessionHashes = {}; // Maps userId => list of (sha256(sessionId))                                        // 80
                                                                                                                       //
  var grainIdToSessionHashes = {}; // Maps grainId => list of (sha256(sessionId))                                      // 81
                                                                                                                       //
  var userIdCount = 0;                                                                                                 // 83
  var grainIdCount = 0;                                                                                                // 84
  this.added("systemStatus", "globalStatus", {                                                                         // 86
    activeUsers: userIdCount,                                                                                          // 87
    activeGrains: grainIdCount                                                                                         // 88
  });                                                                                                                  // 86
  var query = db.collections.sessions.find();                                                                          // 91
  var handle = query.observe({                                                                                         // 92
    added: function (session) {                                                                                        // 93
      var hashedSessionId = hashSessionId(session._id);                                                                // 94
      var changed = {}; // Update userId cache.                                                                        // 95
                                                                                                                       //
      if (session.userId) {                                                                                            // 98
        if (userIdToSessionHashes[session.userId] === undefined) {                                                     // 99
          userIdToSessionHashes[session.userId] = [];                                                                  // 100
          userIdCount = userIdCount + 1;                                                                               // 101
          changed.activeUsers = userIdCount;                                                                           // 102
        }                                                                                                              // 103
                                                                                                                       //
        userIdToSessionHashes[session.userId] = _.union(userIdToSessionHashes[session.userId], [hashedSessionId]);     // 105
      } // Update grainId cache.                                                                                       // 107
                                                                                                                       //
                                                                                                                       //
      if (grainIdToSessionHashes[session.grainId] === undefined) {                                                     // 110
        grainIdToSessionHashes[session.grainId] = [];                                                                  // 111
        grainIdCount = grainIdCount + 1;                                                                               // 112
        changed.activeGrains = grainIdCount;                                                                           // 113
      }                                                                                                                // 114
                                                                                                                       //
      grainIdToSessionHashes[session.grainId] = _.union(grainIdToSessionHashes[session.grainId], [hashedSessionId]);   // 116
                                                                                                                       //
      if (!_.isEmpty(changed)) {                                                                                       // 119
        _this.changed("systemStatus", "globalStatus", changed);                                                        // 120
      }                                                                                                                // 121
    },                                                                                                                 // 122
    removed: function (session) {                                                                                      // 124
      var hashedSessionId = hashSessionId(session._id);                                                                // 125
      var changed = {};                                                                                                // 126
                                                                                                                       //
      if (session.userId) {                                                                                            // 128
        userIdToSessionHashes[session.userId] = _.without(userIdToSessionHashes[session.userId], hashedSessionId);     // 129
                                                                                                                       //
        if (userIdToSessionHashes[session.userId].length === 0) {                                                      // 132
          delete userIdToSessionHashes[session.userId];                                                                // 133
          userIdCount = userIdCount - 1;                                                                               // 134
          changed.activeUsers = userIdCount;                                                                           // 135
        }                                                                                                              // 136
      }                                                                                                                // 137
                                                                                                                       //
      grainIdToSessionHashes[session.grainId] = _.without(grainIdToSessionHashes[session.grainId], hashedSessionId);   // 139
                                                                                                                       //
      if (grainIdToSessionHashes[session.grainId].length === 0) {                                                      // 142
        delete grainIdToSessionHashes[session.grainId];                                                                // 143
        grainIdCount = grainIdCount - 1;                                                                               // 144
        changed.activeGrains = grainIdCount;                                                                           // 145
      }                                                                                                                // 146
                                                                                                                       //
      if (!_.isEmpty(changed)) {                                                                                       // 148
        _this.changed("systemStatus", "globalStatus", changed);                                                        // 149
      }                                                                                                                // 150
    }                                                                                                                  // 151
  });                                                                                                                  // 92
  this.onStop(function () {                                                                                            // 154
    handle.stop();                                                                                                     // 155
  });                                                                                                                  // 156
  this.ready();                                                                                                        // 157
});                                                                                                                    // 158
Meteor.publish("adminDemoUsers", function () {                                                                         // 160
  // Publishes expiry information about demo accounts to admins, so demo users can be counted                          // 161
  // clientside and reactively updated.                                                                                // 162
  var db = this.connection.sandstormDb;                                                                                // 163
                                                                                                                       //
  if (!db.isAdminById(this.userId)) {                                                                                  // 164
    throw new Meteor.Error(403, "User must be admin to view system status.");                                          // 165
  }                                                                                                                    // 166
                                                                                                                       //
  return db.collections.users.find({                                                                                   // 168
    expires: {                                                                                                         // 169
      $gt: new Date()                                                                                                  // 170
    },                                                                                                                 // 169
    loginCredentials: {                                                                                                // 172
      $exists: true                                                                                                    // 173
    }                                                                                                                  // 172
  }, {                                                                                                                 // 168
    expires: 1,                                                                                                        // 176
    loginCredentials: 1                                                                                                // 177
  });                                                                                                                  // 175
});                                                                                                                    // 179
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"drivers":{"external-ui-view.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/drivers/external-ui-view.js                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");                          //
                                                                                                                       //
var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);                                 //
                                                                                                                       //
var _inherits2 = require("babel-runtime/helpers/inherits");                                                            //
                                                                                                                       //
var _inherits3 = _interopRequireDefault(_inherits2);                                                                   //
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var PersistentImpl = void 0;                                                                                           // 1
module.watch(require("/imports/server/persistent.js"), {                                                               // 1
  PersistentImpl: function (v) {                                                                                       // 1
    PersistentImpl = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var ssrfSafeLookup = void 0;                                                                                           // 1
module.watch(require("/imports/server/networking.js"), {                                                               // 1
  ssrfSafeLookup: function (v) {                                                                                       // 1
    ssrfSafeLookup = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var REQUEST_HEADER_WHITELIST = void 0,                                                                                 // 1
    RESPONSE_HEADER_WHITELIST = void 0;                                                                                // 1
module.watch(require("/imports/server/header-whitelist.js"), {                                                         // 1
  REQUEST_HEADER_WHITELIST: function (v) {                                                                             // 1
    REQUEST_HEADER_WHITELIST = v;                                                                                      // 1
  },                                                                                                                   // 1
  RESPONSE_HEADER_WHITELIST: function (v) {                                                                            // 1
    RESPONSE_HEADER_WHITELIST = v;                                                                                     // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 22
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 23
                                                                                                                       //
var Http = Npm.require("http");                                                                                        // 24
                                                                                                                       //
var Https = Npm.require("https");                                                                                      // 25
                                                                                                                       //
var ApiSession = Capnp.importSystem("sandstorm/api-session.capnp").ApiSession;                                         // 26
var PersistentApiSession = Capnp.importSystem("sandstorm/api-session-impl.capnp").PersistentApiSession;                // 27
                                                                                                                       //
ExternalUiView = function () {                                                                                         // 30
  function ExternalUiView(url, token) {                                                                                // 31
    (0, _classCallCheck3.default)(this, ExternalUiView);                                                               // 31
    this.url = url;                                                                                                    // 32
    this.token = token;                                                                                                // 33
  }                                                                                                                    // 34
                                                                                                                       //
  ExternalUiView.prototype.newSession = function () {                                                                  // 30
    function newSession(userInfo, context, sessionType, sessionParams) {                                               // 30
      var _this2 = this;                                                                                               // 36
                                                                                                                       //
      if (sessionType !== ApiSession.typeId) {                                                                         // 37
        throw new Error("SessionType must be ApiSession.");                                                            // 38
      }                                                                                                                // 39
                                                                                                                       //
      var options = {};                                                                                                // 41
                                                                                                                       //
      if (this.token) {                                                                                                // 43
        options.headers = {                                                                                            // 44
          authorization: "Bearer " + this.token                                                                        // 45
        };                                                                                                             // 44
      }                                                                                                                // 47
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 49
        return {                                                                                                       // 50
          session: new Capnp.Capability(new ExternalWebSession(_this2.url, options, globalDb), ApiSession)             // 51
        };                                                                                                             // 50
      });                                                                                                              // 54
    }                                                                                                                  // 55
                                                                                                                       //
    return newSession;                                                                                                 // 30
  }();                                                                                                                 // 30
                                                                                                                       //
  return ExternalUiView;                                                                                               // 30
}();                                                                                                                   // 30
                                                                                                                       //
function getOAuthServiceInfo(url) {                                                                                    // 58
  // TODO(soon): Define a table somewhere (probably in a .capnp file) mapping API hosts to OAuth                       // 59
  //   metadata.                                                                                                       // 60
  if (url.startsWith("https://apidata.googleusercontent.com/") || url.startsWith("https://www.googleapis.com/")) {     // 61
    return {                                                                                                           // 63
      service: "google",                                                                                               // 64
      endpoint: "https://www.googleapis.com/oauth2/v4/token"                                                           // 65
    };                                                                                                                 // 63
  } else if (url.startsWith("https://api.github.com/users")) {                                                         // 67
    return {                                                                                                           // 68
      service: "github",                                                                                               // 69
      endpoint: "https://github.com/login/oauth/access_token"                                                          // 70
    };                                                                                                                 // 68
  } else {                                                                                                             // 72
    return null;                                                                                                       // 73
  }                                                                                                                    // 74
}                                                                                                                      // 75
                                                                                                                       //
function refreshOAuth(url, refreshToken) {                                                                             // 77
  // TODO(perf): Cache access tokens until they expire? Currently we re-do the refresh on every                        // 78
  //   restore. In particular, this means we always drop the first access token returned (which                        // 79
  //   is returned together with the refresh token) and then immediately request a new one.                            // 80
  var serviceInfo = getOAuthServiceInfo(url);                                                                          // 82
                                                                                                                       //
  if (!serviceInfo) {                                                                                                  // 83
    throw new Error("Don't know how to OAuth for: " + url);                                                            // 84
  }                                                                                                                    // 85
                                                                                                                       //
  var config = ServiceConfiguration.configurations.findOne({                                                           // 87
    service: serviceInfo.service                                                                                       // 87
  });                                                                                                                  // 87
                                                                                                                       //
  if (!config) {                                                                                                       // 88
    throw new Error("can't refresh OAuth token for service that isn't configured: " + serviceInfo.service);            // 89
  }                                                                                                                    // 91
                                                                                                                       //
  var response = HTTP.post(serviceInfo.endpoint, {                                                                     // 93
    headers: {                                                                                                         // 94
      "Content-Type": "application/x-www-form-urlencoded",                                                             // 95
      "Accept": "application/json"                                                                                     // 96
    },                                                                                                                 // 94
    content: "client_id=" + encodeURIComponent(config.clientId) + "&client_secret=" + encodeURIComponent(config.secret) + "&refresh_token=" + encodeURIComponent(refreshToken) + "&grant_type=refresh_token"
  });                                                                                                                  // 93
  return response.data;                                                                                                // 104
}                                                                                                                      // 105
                                                                                                                       //
function newExternalHttpSession(url, auth, db, saveTemplate) {                                                         // 107
  // `url` and `auth` are the corresponding members of `ApiToken.frontendRef.http`.                                    // 108
  var createCap = function (authorization) {                                                                           // 110
    return new Capnp.Capability(new ExternalWebSession(url, authorization ? {                                          // 111
      headers: {                                                                                                       // 112
        authorization: authorization                                                                                   // 112
      }                                                                                                                // 112
    } : {}, db, saveTemplate), PersistentApiSession);                                                                  // 112
  };                                                                                                                   // 114
                                                                                                                       //
  if (auth.refresh) {                                                                                                  // 116
    return createCap("Bearer " + refreshOAuth(url, auth.refresh).access_token);                                        // 117
  } else if (auth.bearer) {                                                                                            // 118
    return createCap("Bearer " + auth.bearer);                                                                         // 119
  } else if (auth.basic) {                                                                                             // 120
    var userpass = [auth.basic.username, auth.basic.password].join(":");                                               // 121
    return createCap("Basic " + new Buffer(userpass, "utf8").toString("base64"));                                      // 122
  } else {                                                                                                             // 123
    return createCap(null);                                                                                            // 124
  }                                                                                                                    // 125
}                                                                                                                      // 126
                                                                                                                       //
function registerHttpApiFrontendRef(registry) {                                                                        // 128
  registry.register({                                                                                                  // 129
    frontendRefField: "http",                                                                                          // 130
    typeId: ApiSession.typeId,                                                                                         // 131
    restore: function (db, saveTemplate, value) {                                                                      // 133
      return newExternalHttpSession(value.url, value.auth, db, saveTemplate);                                          // 134
    },                                                                                                                 // 135
    validate: function (db, session, request) {                                                                        // 137
      check(request, {                                                                                                 // 138
        url: String,                                                                                                   // 139
        auth: Match.OneOf({                                                                                            // 140
          none: null                                                                                                   // 141
        }, {                                                                                                           // 141
          bearer: String                                                                                               // 142
        }, {                                                                                                           // 142
          basic: {                                                                                                     // 143
            username: String,                                                                                          // 143
            password: String                                                                                           // 143
          }                                                                                                            // 143
        }, {                                                                                                           // 143
          oauth: {                                                                                                     // 144
            credentialToken: String,                                                                                   // 144
            credentialSecret: String                                                                                   // 144
          }                                                                                                            // 144
        })                                                                                                             // 144
      });                                                                                                              // 138
                                                                                                                       //
      if (!request.url.startsWith("https://") && !request.url.startsWith("http://")) {                                 // 147
        throw new Meteor.Error(400, "URL must be HTTP or HTTPS.");                                                     // 149
      } // Check for URL patterns.                                                                                     // 150
                                                                                                                       //
                                                                                                                       //
      var parsedUrl = Url.parse(request.url);                                                                          // 153
                                                                                                                       //
      if (parsedUrl.auth) {                                                                                            // 155
        var parts = parsedUrl.auth.split(":");                                                                         // 156
                                                                                                                       //
        if (parts.length === 2) {                                                                                      // 157
          if ("none" in request.auth) {                                                                                // 158
            request.auth = {                                                                                           // 159
              basic: {                                                                                                 // 159
                username: parts[0],                                                                                    // 159
                password: parts[1]                                                                                     // 159
              }                                                                                                        // 159
            };                                                                                                         // 159
          } else {                                                                                                     // 160
            throw new Meteor.Error(400, "Can't support multiple authentication mechanisms at once");                   // 161
          }                                                                                                            // 162
        }                                                                                                              // 163
                                                                                                                       //
        parsedUrl.auth = null;                                                                                         // 165
        request.url = Url.format(parsedUrl);                                                                           // 166
      }                                                                                                                // 167
                                                                                                                       //
      if (parsedUrl.hash) {                                                                                            // 169
        if ("none" in request.auth) {                                                                                  // 170
          request.auth = {                                                                                             // 171
            bearer: parsedUrl.hash.slice(1)                                                                            // 171
          };                                                                                                           // 171
        } else {                                                                                                       // 172
          throw new Meteor.Error(400, "Can't support multiple authentication mechanisms at once");                     // 173
        }                                                                                                              // 174
                                                                                                                       //
        parsedUrl.hash = null;                                                                                         // 176
        request.url = Url.format(parsedUrl);                                                                           // 177
      }                                                                                                                // 178
                                                                                                                       //
      if (request.auth.oauth) {                                                                                        // 180
        // We did an OAuth handshake client-side.                                                                      // 181
        var oauthResult = OAuth.retrieveCredential(request.auth.oauth.credentialToken, request.auth.oauth.credentialSecret);
                                                                                                                       //
        if (oauthResult instanceof Error) {                                                                            // 185
          throw oauthResult;                                                                                           // 186
        }                                                                                                              // 187
                                                                                                                       //
        var serviceData = oauthResult.serviceData;                                                                     // 189
                                                                                                                       //
        if (serviceData.refreshToken) {                                                                                // 190
          request.auth = {                                                                                             // 191
            refresh: serviceData.refreshToken                                                                          // 191
          };                                                                                                           // 191
        } else {                                                                                                       // 192
          request.auth = {                                                                                             // 193
            bearer: serviceData.accessToken                                                                            // 193
          };                                                                                                           // 193
        } // TODO(security): We could maybe add a MembraneRequirement that this user account                           // 194
        //   possesses credentials for this OAuth service. Conversely, perhaps if an authCode was                      // 197
        //   specified, we should automatically add the associated credential to the user's                            // 198
        //   account? (As a non-login credential.)                                                                     // 199
                                                                                                                       //
      }                                                                                                                // 200
                                                                                                                       //
      var descriptor = {                                                                                               // 202
        tags: [{                                                                                                       // 202
          id: ApiSession.typeId                                                                                        // 202
        }]                                                                                                             // 202
      };                                                                                                               // 202
      return {                                                                                                         // 203
        descriptor: descriptor,                                                                                        // 203
        requirements: [],                                                                                              // 203
        frontendRef: request                                                                                           // 203
      };                                                                                                               // 203
    },                                                                                                                 // 204
    query: function (db, userAccountId, tagValue) {                                                                    // 206
      var tag = tagValue ? Capnp.parse(ApiSession.PowerboxTag, tagValue) : {};                                         // 207
      var options = [];                                                                                                // 209
                                                                                                                       //
      if (tag.canonicalUrl && (tag.canonicalUrl.startsWith("https://") || tag.canonicalUrl.startsWith("http://"))) {   // 211
        var serviceInfo = getOAuthServiceInfo(tag.canonicalUrl);                                                       // 214
                                                                                                                       //
        if (serviceInfo && tag.oauthScopes) {                                                                          // 215
          // Note: We don't check if the service is configured, because it's useful to show the                        // 216
          //   user that the option exists but inform them that it will only work if the admin                         // 217
          //   configures this service.                                                                                // 218
          options.push({                                                                                               // 219
            _id: "http-oauth-" + tag.canonicalUrl,                                                                     // 220
            cardTemplate: "httpOAuthPowerboxCard",                                                                     // 221
            configureTemplate: "httpOAuthPowerboxConfiguration",                                                       // 222
            httpUrl: tag.canonicalUrl,                                                                                 // 223
            oauthServiceInfo: serviceInfo,                                                                             // 224
            oauthScopes: tag.oauthScopes                                                                               // 225
          });                                                                                                          // 219
        } else {                                                                                                       // 227
          // TODO(soon): Support tag.authentication.                                                                   // 228
          options.push({                                                                                               // 230
            _id: "http-url-" + tag.canonicalUrl,                                                                       // 231
            frontendRef: {                                                                                             // 232
              http: {                                                                                                  // 232
                url: tag.canonicalUrl,                                                                                 // 232
                auth: {                                                                                                // 232
                  none: null                                                                                           // 232
                }                                                                                                      // 232
              }                                                                                                        // 232
            },                                                                                                         // 232
            cardTemplate: "httpUrlPowerboxCard"                                                                        // 233
          });                                                                                                          // 230
        }                                                                                                              // 235
      } // Always offer the user the option to connect to an arbitrary URL of their choosing, even                     // 236
      // if canonicalUrl exists.                                                                                       // 239
                                                                                                                       //
                                                                                                                       //
      options.push({                                                                                                   // 240
        _id: "http-arbitrary",                                                                                         // 241
        cardTemplate: "httpArbitraryPowerboxCard",                                                                     // 242
        configureTemplate: "httpArbitraryPowerboxConfiguration"                                                        // 243
      });                                                                                                              // 240
      return options;                                                                                                  // 246
    }                                                                                                                  // 247
  });                                                                                                                  // 129
}                                                                                                                      // 249
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 251
  registerHttpApiFrontendRef(globalFrontendRefRegistry);                                                               // 251
}); // =======================================================================================                         // 251
                                                                                                                       //
var responseCodes = {                                                                                                  // 255
  200: {                                                                                                               // 256
    type: "content",                                                                                                   // 256
    code: "ok"                                                                                                         // 256
  },                                                                                                                   // 256
  201: {                                                                                                               // 257
    type: "content",                                                                                                   // 257
    code: "created"                                                                                                    // 257
  },                                                                                                                   // 257
  202: {                                                                                                               // 258
    type: "content",                                                                                                   // 258
    code: "accepted"                                                                                                   // 258
  },                                                                                                                   // 258
  204: {                                                                                                               // 259
    type: "noContent",                                                                                                 // 259
    shouldResetForm: false                                                                                             // 259
  },                                                                                                                   // 259
  205: {                                                                                                               // 260
    type: "noContent",                                                                                                 // 260
    shouldResetForm: true                                                                                              // 260
  },                                                                                                                   // 260
  // Unsupported until something demonstrates need.                                                                    // 262
  // 206: {type: 'noContent'},                                                                                         // 263
  // 300: {type: 'redirect'},                                                                                          // 264
  301: {                                                                                                               // 265
    type: "redirect",                                                                                                  // 265
    switchToGet: true,                                                                                                 // 265
    isPermanent: true                                                                                                  // 265
  },                                                                                                                   // 265
  302: {                                                                                                               // 266
    type: "redirect",                                                                                                  // 266
    switchToGet: true,                                                                                                 // 266
    isPermanent: false                                                                                                 // 266
  },                                                                                                                   // 266
  303: {                                                                                                               // 267
    type: "redirect",                                                                                                  // 267
    switchToGet: true,                                                                                                 // 267
    isPermanent: false                                                                                                 // 267
  },                                                                                                                   // 267
  304: {                                                                                                               // 269
    type: "preconditionFailed"                                                                                         // 269
  },                                                                                                                   // 269
  // Unsupported until something demonstrates need.                                                                    // 271
  // 305: {type: 'redirect'},                                                                                          // 272
  307: {                                                                                                               // 273
    type: "redirect",                                                                                                  // 273
    switchToGet: false,                                                                                                // 273
    isPermanent: false                                                                                                 // 273
  },                                                                                                                   // 273
  308: {                                                                                                               // 274
    type: "redirect",                                                                                                  // 274
    switchToGet: false,                                                                                                // 274
    isPermanent: true                                                                                                  // 274
  },                                                                                                                   // 274
  400: {                                                                                                               // 275
    type: "clientError",                                                                                               // 275
    clientErrorCode: "badRequest",                                                                                     // 275
    descriptionHtml: "Bad Request"                                                                                     // 275
  },                                                                                                                   // 275
  403: {                                                                                                               // 276
    type: "clientError",                                                                                               // 276
    clientErrorCode: "forbidden",                                                                                      // 276
    descriptionHtml: "Forbidden"                                                                                       // 276
  },                                                                                                                   // 276
  404: {                                                                                                               // 277
    type: "clientError",                                                                                               // 277
    clientErrorCode: "notFound",                                                                                       // 277
    descriptionHtml: "Not Found"                                                                                       // 277
  },                                                                                                                   // 277
  405: {                                                                                                               // 278
    type: "clientError",                                                                                               // 278
    clientErrorCode: "methodNotAllowed",                                                                               // 278
    descriptionHtml: "Method Not Allowed"                                                                              // 278
  },                                                                                                                   // 278
  406: {                                                                                                               // 279
    type: "clientError",                                                                                               // 279
    clientErrorCode: "notAcceptable",                                                                                  // 279
    descriptionHtml: "Not Acceptable"                                                                                  // 279
  },                                                                                                                   // 279
  409: {                                                                                                               // 280
    type: "clientError",                                                                                               // 280
    clientErrorCode: "conflict",                                                                                       // 280
    descriptionHtml: "Conflict"                                                                                        // 280
  },                                                                                                                   // 280
  410: {                                                                                                               // 281
    type: "clientError",                                                                                               // 281
    clientErrorCode: "gone",                                                                                           // 281
    descriptionHtml: "Gone"                                                                                            // 281
  },                                                                                                                   // 281
  412: {                                                                                                               // 282
    type: "preconditionFailed"                                                                                         // 282
  },                                                                                                                   // 282
  413: {                                                                                                               // 283
    type: "clientError",                                                                                               // 283
    clientErrorCode: "requestEntityTooLarge",                                                                          // 283
    descriptionHtml: "Request Entity Too Large"                                                                        // 283
  },                                                                                                                   // 283
  414: {                                                                                                               // 284
    type: "clientError",                                                                                               // 284
    clientErrorCode: "requestUriTooLong",                                                                              // 284
    descriptionHtml: "Request-URI Too Long"                                                                            // 284
  },                                                                                                                   // 284
  415: {                                                                                                               // 285
    type: "clientError",                                                                                               // 285
    clientErrorCode: "unsupportedMediaType",                                                                           // 285
    descriptionHtml: "Unsupported Media Type"                                                                          // 285
  },                                                                                                                   // 285
  418: {                                                                                                               // 286
    type: "clientError",                                                                                               // 286
    clientErrorCode: "imATeapot",                                                                                      // 286
    descriptionHtml: "I'm a teapot"                                                                                    // 286
  },                                                                                                                   // 286
  500: {                                                                                                               // 287
    type: "serverError"                                                                                                // 287
  },                                                                                                                   // 287
  501: {                                                                                                               // 288
    type: "serverError"                                                                                                // 288
  },                                                                                                                   // 288
  502: {                                                                                                               // 289
    type: "serverError"                                                                                                // 289
  },                                                                                                                   // 289
  503: {                                                                                                               // 290
    type: "serverError"                                                                                                // 290
  },                                                                                                                   // 290
  504: {                                                                                                               // 291
    type: "serverError"                                                                                                // 291
  },                                                                                                                   // 291
  505: {                                                                                                               // 292
    type: "serverError"                                                                                                // 292
  }                                                                                                                    // 292
};                                                                                                                     // 255
                                                                                                                       //
function composeETag(etag) {                                                                                           // 295
  if (etag.weak) {                                                                                                     // 296
    return "W/\"" + etag.value + "\"";                                                                                 // 297
  } else {                                                                                                             // 298
    return "\"" + etag.value + "\"";                                                                                   // 299
  }                                                                                                                    // 300
}                                                                                                                      // 301
                                                                                                                       //
function parseETag(input) {                                                                                            // 303
  var etag = {                                                                                                         // 304
    value: "",                                                                                                         // 304
    weak: false                                                                                                        // 304
  };                                                                                                                   // 304
  input = input.trim();                                                                                                // 306
                                                                                                                       //
  if (input.startsWith("W/")) {                                                                                        // 307
    input = input.slice(2);                                                                                            // 308
    etag.weak = true;                                                                                                  // 309
  }                                                                                                                    // 310
                                                                                                                       //
  if (!(input.startsWith("\"") && input.endsWith("\"") && input.length >= 2)) {                                        // 312
    // Invalid etag. Drop.                                                                                             // 313
    // (It would be nice to tell the developer about this but... how?)                                                 // 314
    return undefined;                                                                                                  // 315
  }                                                                                                                    // 316
                                                                                                                       //
  try {                                                                                                                // 318
    // Since the text starts with a quote, the only way it could parse as JSON is if it is a single                    // 319
    // string. This nicely handles escape sequences for us.                                                            // 320
    etag.value = JSON.parse(input);                                                                                    // 321
    return etag;                                                                                                       // 322
  } catch (err) {                                                                                                      // 323
    // Invalid etag.                                                                                                   // 324
    return undefined;                                                                                                  // 325
  }                                                                                                                    // 326
}                                                                                                                      // 327
                                                                                                                       //
ExternalWebSession = function (_PersistentImpl) {                                                                      // 329
  (0, _inherits3.default)(ExternalWebSession, _PersistentImpl);                                                        // 329
                                                                                                                       //
  function ExternalWebSession(url, options, db, saveTemplate) {                                                        // 330
    (0, _classCallCheck3.default)(this, ExternalWebSession);                                                           // 330
                                                                                                                       //
    // TODO(soon): Support HTTP proxy.                                                                                 // 333
    var _this3 = (0, _possibleConstructorReturn3.default)(this, _PersistentImpl.call(this, db, saveTemplate));         // 330
                                                                                                                       //
    var safe = ssrfSafeLookup(db, url);                                                                                // 334
    if (!options) options = {};                                                                                        // 336
    if (!options.headers) options.headers = {};                                                                        // 337
    options.headers.host = safe.host;                                                                                  // 338
    options.servername = safe.host.split(":")[0];                                                                      // 339
                                                                                                                       //
    if (!saveTemplate) {                                                                                               // 341
      // enable backwards-compatibilty tweaks.                                                                         // 342
      _this3.fromHackSession = true;                                                                                   // 343
    }                                                                                                                  // 344
                                                                                                                       //
    var parsedUrl = Url.parse(safe.url);                                                                               // 346
    _this3.host = parsedUrl.hostname;                                                                                  // 347
                                                                                                                       //
    if (_this3.fromHackSession) {// HackSessionContext.getExternalUiView() apparently ignored any path on the URL. Whoops.
    } else {                                                                                                           // 350
      if (parsedUrl.path === "/") {                                                                                    // 351
        // The URL parser says path = "/" for both "http://foo" and "http://foo/". We want to be                       // 352
        // strict, though.                                                                                             // 353
        _this3.path = url.endsWith("/") ? "/" : "";                                                                    // 354
      } else {                                                                                                         // 355
        _this3.path = parsedUrl.path;                                                                                  // 356
      }                                                                                                                // 357
    }                                                                                                                  // 358
                                                                                                                       //
    _this3.port = parsedUrl.port;                                                                                      // 360
    _this3.protocol = parsedUrl.protocol;                                                                              // 361
    _this3.options = options;                                                                                          // 362
    return _this3;                                                                                                     // 330
  }                                                                                                                    // 363
                                                                                                                       //
  ExternalWebSession.prototype.get = function () {                                                                     // 329
    function get(path, context) {                                                                                      // 329
      return this._requestHelper("GET", path, context);                                                                // 366
    }                                                                                                                  // 367
                                                                                                                       //
    return get;                                                                                                        // 329
  }();                                                                                                                 // 329
                                                                                                                       //
  ExternalWebSession.prototype.post = function () {                                                                    // 329
    function post(path, content, context) {                                                                            // 329
      return this._requestHelper("POST", path, context, content.content, content.mimeType);                            // 370
    }                                                                                                                  // 371
                                                                                                                       //
    return post;                                                                                                       // 329
  }();                                                                                                                 // 329
                                                                                                                       //
  ExternalWebSession.prototype.put = function () {                                                                     // 329
    function put(path, content, context) {                                                                             // 329
      return this._requestHelper("PUT", path, context, content.content, content.mimeType);                             // 374
    }                                                                                                                  // 375
                                                                                                                       //
    return put;                                                                                                        // 329
  }();                                                                                                                 // 329
                                                                                                                       //
  ExternalWebSession.prototype.delete = function () {                                                                  // 329
    function _delete(path, context) {                                                                                  // 329
      return this._requestHelper("DELETE", path, context);                                                             // 378
    }                                                                                                                  // 379
                                                                                                                       //
    return _delete;                                                                                                    // 329
  }(); // TODO(someday): implement streaming and websockets for ExternalWebSession                                     // 329
  //postStreaming(path, mimeType, context) {                                                                           // 382
  //}                                                                                                                  // 383
  //putStreaming(path, mimeType, context) {                                                                            // 385
  //}                                                                                                                  // 386
  //openWebSocket(path, context, protocol, clientStream) {                                                             // 388
  //}                                                                                                                  // 389
                                                                                                                       //
                                                                                                                       //
  ExternalWebSession.prototype._requestHelper = function () {                                                          // 329
    function _requestHelper(method, path, context, content, contentType) {                                             // 329
      var _this4 = this;                                                                                               // 391
                                                                                                                       //
      var _this = this;                                                                                                // 392
                                                                                                                       //
      var session = _this;                                                                                             // 393
      return new Promise(function (resolve, reject) {                                                                  // 394
        var options = _.clone(session.options);                                                                        // 395
                                                                                                                       //
        options.headers = options.headers || {};                                                                       // 396
                                                                                                                       //
        if (!options.headers["user-agent"]) {                                                                          // 398
          options.headers["user-agent"] = "sandstorm app";                                                             // 399
        }                                                                                                              // 400
                                                                                                                       //
        if (_this4.fromHackSession) {                                                                                  // 402
          // According to the specification of `WebSession`, `path` should not contain a                               // 403
          // leading slash, and therefore we need to prepend "/". However, for a long time                             // 404
          // this implementation did not in fact prepend a "/". Since some apps might rely on                          // 405
          // that behavior, we only prepend "/" if the path does not start with "/".                                   // 406
          //                                                                                                           // 407
          // TODO(soon): Once apps have updated, prepend "/" unconditionally.                                          // 408
          options.path = path.startsWith("/") ? path : "/" + path;                                                     // 409
        } else {                                                                                                       // 410
          options.path = _this4.path + "/" + path;                                                                     // 411
        }                                                                                                              // 412
                                                                                                                       //
        options.method = method;                                                                                       // 414
                                                                                                                       //
        if (contentType) {                                                                                             // 415
          options.headers["content-type"] = contentType;                                                               // 416
        } // set accept header                                                                                         // 417
                                                                                                                       //
                                                                                                                       //
        if ("accept" in context) {                                                                                     // 420
          options.headers.accept = context.accept.map(function (acceptedType) {                                        // 421
            return acceptedType.mimeType + "; " + acceptedType.qValue;                                                 // 422
          }).join(", ");                                                                                               // 423
        } else if (!("accept" in options.headers)) {                                                                   // 424
          options.headers.accept = "*/*";                                                                              // 425
        } // set cookies                                                                                               // 426
                                                                                                                       //
                                                                                                                       //
        if (context.cookies && context.cookies.length > 0) {                                                           // 429
          options.headers.cookies = options.headers.cookies || "";                                                     // 430
          context.cookies.forEach(function (keyVal) {                                                                  // 431
            options.headers.cookies += keyVal.key + "=" + keyVal.val + ",";                                            // 432
          });                                                                                                          // 433
          options.headers.cookies = options.headers.cookies.slice(0, -1);                                              // 434
        } // set precondition                                                                                          // 435
                                                                                                                       //
                                                                                                                       //
        if (context.eTagPrecondition) {                                                                                // 438
          if (context.eTagPrecondition.none) {// nothing                                                               // 439
          } else if (context.eTagPrecondition.exists) {                                                                // 441
            options.headers["if-match"] = "*";                                                                         // 442
          } else if (context.eTagPrecondition.doesntExist) {                                                           // 443
            options.headers["if-none-match"] = "*";                                                                    // 444
          } else if (context.eTagPrecondition.matchesOneOf) {                                                          // 445
            options.headers["if-match"] = context.eTagPrecondition.matchesOneOf.map(composeETag).join(", ");           // 446
          } else if (context.eTagPrecondition.matchesNoneOf) {                                                         // 448
            options.headers["if-none-match"] = context.eTagPrecondition.matchesNoneOf.map(composeETag).join(", ");     // 449
          }                                                                                                            // 451
        } // set additional headers                                                                                    // 452
                                                                                                                       //
                                                                                                                       //
        (context.additionalHeaders || []).forEach(function (header) {                                                  // 455
          if (REQUEST_HEADER_WHITELIST.matches(header.name)) {                                                         // 456
            options.headers[header.name] = header.value;                                                               // 457
          }                                                                                                            // 458
        });                                                                                                            // 459
        options.host = session.host;                                                                                   // 461
        options.port = session.port;                                                                                   // 462
        var requestMethod = Http.request;                                                                              // 464
                                                                                                                       //
        if (session.protocol === "https:") {                                                                           // 465
          requestMethod = Https.request;                                                                               // 466
        }                                                                                                              // 467
                                                                                                                       //
        req = requestMethod(options, function (resp) {                                                                 // 469
          try {                                                                                                        // 470
            var buffers = [];                                                                                          // 471
            var statusInfo = responseCodes[resp.statusCode];                                                           // 472
            var rpcResponse = {};                                                                                      // 474
            rpcResponse.additionalHeaders = [];                                                                        // 476
                                                                                                                       //
            for (var headerName in meteorBabelHelpers.sanitizeForInObject(resp.headers)) {                             // 477
              if (RESPONSE_HEADER_WHITELIST.matches(headerName)) {                                                     // 478
                rpcResponse.additionalHeaders.push({                                                                   // 479
                  name: headerName,                                                                                    // 480
                  value: resp.headers[headerName]                                                                      // 481
                });                                                                                                    // 479
              }                                                                                                        // 483
            }                                                                                                          // 484
                                                                                                                       //
            resp.on("data", function (buf) {                                                                           // 486
              buffers.push(buf);                                                                                       // 487
            });                                                                                                        // 488
            resp.on("end", function () {                                                                               // 490
              try {                                                                                                    // 491
                var fillInErrorBody = function (error) {                                                               // 491
                  var contentType = resp.headers["content-type"];                                                      // 495
                                                                                                                       //
                  if (contentType && (contentType == "text/html" || contentType.startsWith("text/html;"))) {           // 496
                    // TODO(someday): Check for non-UTF-8 charset and translate?                                       // 498
                    error.descriptionHtml = data.toString("utf8");                                                     // 499
                  } else if (contentType || data.length > 0) {                                                         // 500
                    var _content = {                                                                                   // 501
                      data: data                                                                                       // 501
                    };                                                                                                 // 501
                    if (contentType) _content.mimeType = contentType;                                                  // 502
                    if ("content-encoding" in resp.headers) _content.encoding = resp.headers["content-encoding"];      // 503
                    if ("content-language" in resp.headers) _content.language = resp.headers["content-language"];      // 504
                    error.nonHtmlBody = _content;                                                                      // 505
                  }                                                                                                    // 506
                };                                                                                                     // 507
                                                                                                                       //
                var data = Buffer.concat(buffers);                                                                     // 492
                                                                                                                       //
                switch (statusInfo ? statusInfo.type : resp.statusCode) {                                              // 509
                  case "content":                                                                                      // 510
                    var _content2 = {};                                                                                // 511
                    rpcResponse.content = _content2;                                                                   // 512
                    _content2.statusCode = statusInfo.code;                                                            // 514
                    if ("content-encoding" in resp.headers) _content2.encoding = resp.headers["content-encoding"];     // 515
                    if ("content-language" in resp.headers) _content2.language = resp.headers["content-language"];     // 516
                    if ("content-type" in resp.headers) _content2.mimeType = resp.headers["content-type"];             // 517
                                                                                                                       //
                    if ("content-disposition" in resp.headers) {                                                       // 518
                      var disposition = resp.headers["content-disposition"];                                           // 519
                      var parts = disposition.split(";");                                                              // 520
                                                                                                                       //
                      if (parts[0].toLowerCase().trim() === "attachment") {                                            // 521
                        parts.forEach(function (part) {                                                                // 522
                          var splitPart = part.split("=");                                                             // 523
                                                                                                                       //
                          if (splitPart[0].toLowerCase().trim() === "filename") {                                      // 524
                            _content2.disposition = {                                                                  // 525
                              download: splitPart[1].trim()                                                            // 525
                            };                                                                                         // 525
                          }                                                                                            // 526
                        });                                                                                            // 527
                      }                                                                                                // 528
                    }                                                                                                  // 529
                                                                                                                       //
                    if (resp.headers.etag) {                                                                           // 531
                      _content2.eTag = parseETag(resp.headers.etag);                                                   // 532
                    }                                                                                                  // 533
                                                                                                                       //
                    _content2.body = {};                                                                               // 535
                    _content2.body.bytes = data;                                                                       // 536
                    resolve(rpcResponse);                                                                              // 538
                    break;                                                                                             // 539
                                                                                                                       //
                  case "noContent":                                                                                    // 540
                    var noContent = {};                                                                                // 541
                    rpcResponse.noContent = noContent;                                                                 // 542
                    noContent.setShouldResetForm = statusInfo.shouldResetForm;                                         // 543
                                                                                                                       //
                    if (resp.headers.etag) {                                                                           // 544
                      noContent.eTag = parseETag(resp.headers.etag);                                                   // 545
                    }                                                                                                  // 546
                                                                                                                       //
                    resolve(rpcResponse);                                                                              // 548
                    break;                                                                                             // 549
                                                                                                                       //
                  case "redirect":                                                                                     // 550
                    var redirect = {};                                                                                 // 551
                    rpcResponse.redirect = redirect;                                                                   // 552
                    redirect.isPermanent = statusInfo.isPermanent;                                                     // 553
                    redirect.switchToGet = statusInfo.switchToGet;                                                     // 554
                    if ("location" in resp.headers) redirect.location = resp.headers.location;                         // 555
                    resolve(rpcResponse);                                                                              // 556
                    break;                                                                                             // 557
                                                                                                                       //
                  case "clientError":                                                                                  // 558
                    var clientError = {};                                                                              // 559
                    rpcResponse.clientError = clientError;                                                             // 560
                    clientError.statusCode = statusInfo.clientErrorCode;                                               // 561
                    fillInErrorBody(clientError);                                                                      // 563
                    resolve(rpcResponse);                                                                              // 564
                    break;                                                                                             // 565
                                                                                                                       //
                  case "serverError":                                                                                  // 566
                    var serverError = {};                                                                              // 567
                    rpcResponse.serverError = serverError;                                                             // 568
                    fillInErrorBody(serverError);                                                                      // 569
                    resolve(rpcResponse);                                                                              // 570
                    break;                                                                                             // 571
                                                                                                                       //
                  case "preconditionFailed":                                                                           // 572
                    var preconditionFailed = {};                                                                       // 573
                    rpcResponse.preconditionFailed = preconditionFailed;                                               // 574
                                                                                                                       //
                    if (resp.headers.etag) {                                                                           // 575
                      preconditionFailed.matchingETag = parseETag(resp.headers.etag);                                  // 576
                    }                                                                                                  // 577
                                                                                                                       //
                    resolve(rpcResponse);                                                                              // 579
                    break;                                                                                             // 580
                  // TODO(soon): Handle token-expired errors by throwing DISCONNECTED -- this will                     // 582
                  //   force the client to reload the capability which will refresh the token.                         // 583
                                                                                                                       //
                  default:                                                                                             // 585
                    // ???                                                                                             // 585
                    var _err = new Error("Invalid status code " + resp.statusCode + " received in response.");         // 586
                                                                                                                       //
                    reject(_err);                                                                                      // 588
                    break;                                                                                             // 589
                }                                                                                                      // 509
              } catch (err) {                                                                                          // 591
                reject(err);                                                                                           // 592
              }                                                                                                        // 593
            });                                                                                                        // 594
          } catch (err) {                                                                                              // 595
            reject(err);                                                                                               // 596
          }                                                                                                            // 597
        });                                                                                                            // 598
        req.on("error", function (e) {                                                                                 // 600
          reject(e);                                                                                                   // 601
        });                                                                                                            // 602
        req.setTimeout(15000, function () {                                                                            // 604
          req.abort();                                                                                                 // 605
          err = new Error("Request timed out.");                                                                       // 606
          err.kjType = "overloaded";                                                                                   // 607
          reject(err);                                                                                                 // 608
        });                                                                                                            // 609
                                                                                                                       //
        if (content) {                                                                                                 // 611
          req.end(content);                                                                                            // 612
        } else {                                                                                                       // 613
          req.end();                                                                                                   // 614
        }                                                                                                              // 615
      });                                                                                                              // 616
    }                                                                                                                  // 617
                                                                                                                       //
    return _requestHelper;                                                                                             // 329
  }();                                                                                                                 // 329
                                                                                                                       //
  return ExternalWebSession;                                                                                           // 329
}(PersistentImpl);                                                                                                     // 329
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"ip.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/drivers/ip.js                                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");                          //
                                                                                                                       //
var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);                                 //
                                                                                                                       //
var _inherits2 = require("babel-runtime/helpers/inherits");                                                            //
                                                                                                                       //
var _inherits3 = _interopRequireDefault(_inherits2);                                                                   //
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var Bignum = void 0;                                                                                                   // 1
module.watch(require("bignum"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    Bignum = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var PersistentImpl = void 0;                                                                                           // 1
module.watch(require("/imports/server/persistent.js"), {                                                               // 1
  PersistentImpl: function (v) {                                                                                       // 1
    PersistentImpl = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 19
                                                                                                                       //
var Net = Npm.require("net");                                                                                          // 20
                                                                                                                       //
var Tls = Npm.require("tls");                                                                                          // 21
                                                                                                                       //
var Dgram = Npm.require("dgram");                                                                                      // 22
                                                                                                                       //
var IpRpc = Capnp.importSystem("sandstorm/ip.capnp");                                                                  // 24
                                                                                                                       //
ByteStreamConnection = function () {                                                                                   // 26
  function ByteStreamConnection(connection) {                                                                          // 27
    (0, _classCallCheck3.default)(this, ByteStreamConnection);                                                         // 27
    this.connection = connection;                                                                                      // 28
  }                                                                                                                    // 29
                                                                                                                       //
  ByteStreamConnection.prototype.done = function () {                                                                  // 26
    function done() {                                                                                                  // 26
      this.connection.end();                                                                                           // 32
    }                                                                                                                  // 33
                                                                                                                       //
    return done;                                                                                                       // 26
  }();                                                                                                                 // 26
                                                                                                                       //
  ByteStreamConnection.prototype.write = function () {                                                                 // 26
    function write(data) {                                                                                             // 26
      this.connection.write(data);                                                                                     // 36
    }                                                                                                                  // 37
                                                                                                                       //
    return write;                                                                                                      // 26
  }(); // expectSize not implemented                                                                                   // 26
  // expectSize(size) { }                                                                                              // 40
                                                                                                                       //
                                                                                                                       //
  return ByteStreamConnection;                                                                                         // 26
}();                                                                                                                   // 26
                                                                                                                       //
var IpInterfaceImpl = function (_PersistentImpl) {                                                                     //
  (0, _inherits3.default)(IpInterfaceImpl, _PersistentImpl);                                                           //
                                                                                                                       //
  function IpInterfaceImpl(db, saveTemplate) {                                                                         // 44
    (0, _classCallCheck3.default)(this, IpInterfaceImpl);                                                              // 44
    return (0, _possibleConstructorReturn3.default)(this, _PersistentImpl.call(this, db, saveTemplate));               // 44
  }                                                                                                                    // 46
                                                                                                                       //
  IpInterfaceImpl.prototype.listenTcp = function () {                                                                  //
    function listenTcp(portNum, port) {                                                                                //
      return new Promise(function (resolve, reject) {                                                                  // 49
        var resolved = false;                                                                                          // 50
        var server = Net.createServer(function (connection) {                                                          // 51
          var wrappedConnection = new ByteStreamConnection(connection);                                                // 52
          var upstream = port.connect(wrappedConnection).upstream;                                                     // 53
          connection.on("data", function (data) {                                                                      // 55
            upstream.write(data);                                                                                      // 56
          });                                                                                                          // 57
          connection.on("close", function (hadError) {                                                                 // 59
            upstream.done();                                                                                           // 60
          });                                                                                                          // 61
          connection.on("error", function (err) {                                                                      // 63
            if (resolved) {                                                                                            // 64
              connection.write = errorWrite;                                                                           // 65
            } else {                                                                                                   // 66
              // upstream hasn't been resolved yet, so it's safe to reject                                             // 67
              reject(err);                                                                                             // 68
            }                                                                                                          // 69
          });                                                                                                          // 70
        });                                                                                                            // 71
        server.listen(portNum, function () {                                                                           // 73
          resolved = true;                                                                                             // 74
          resolve({                                                                                                    // 75
            handle: server                                                                                             // 75
          }); // server has a close method which is all we want from a handle                                          // 75
        });                                                                                                            // 76
        server.on("error", function (err) {                                                                            // 78
          if (!resolved) {                                                                                             // 79
            reject(err);                                                                                               // 80
          }                                                                                                            // 81
        });                                                                                                            // 82
      });                                                                                                              // 83
    }                                                                                                                  // 84
                                                                                                                       //
    return listenTcp;                                                                                                  //
  }();                                                                                                                 //
                                                                                                                       //
  IpInterfaceImpl.prototype.listenUdp = function () {                                                                  //
    function listenUdp(portNum, port) {                                                                                //
      return new Promise(function (resolve, reject) {                                                                  // 87
        var portMap = {};                                                                                              // 88
        var resolved = false;                                                                                          // 89
        var server = Dgram.createSocket("udp4"); // TODO(someday): handle ipv6 sockets too                             // 90
                                                                                                                       //
        server.bind(portNum);                                                                                          // 91
        server.on("listening", function () {                                                                           // 93
          // Although UDP is connectionless, we don't resolve until here so that we can handle bind                    // 94
          // errors such as invalid host                                                                               // 95
          resolved = true;                                                                                             // 96
          resolve({                                                                                                    // 97
            handle: server                                                                                             // 97
          }); // server has a close method which is all we want from a handle                                          // 97
        });                                                                                                            // 98
        server.on("error", function (err) {                                                                            // 100
          // TODO(someday): do something about errors after the promise is resolved                                    // 101
          if (!resolved) {                                                                                             // 102
            reject(err);                                                                                               // 103
          } else {                                                                                                     // 104
            console.error("error in listenUdp: " + err);                                                               // 105
          }                                                                                                            // 106
        });                                                                                                            // 107
        var returnMap = {};                                                                                            // 109
        server.on("message", function (msg, rinfo) {                                                                   // 110
          var address = rinfo.address + "]:" + rinfo.port;                                                             // 111
          var returnPort = returnMap[address];                                                                         // 112
                                                                                                                       //
          if (!returnPort) {                                                                                           // 114
            returnMap[address] = returnPort = new BoundUdpPortImpl(server, rinfo.address, rinfo.port);                 // 115
          }                                                                                                            // 116
                                                                                                                       //
          port.send(msg, returnPort);                                                                                  // 118
        });                                                                                                            // 119
      });                                                                                                              // 120
    }                                                                                                                  // 121
                                                                                                                       //
    return listenUdp;                                                                                                  //
  }();                                                                                                                 //
                                                                                                                       //
  return IpInterfaceImpl;                                                                                              //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
; // TODO(cleanup): Meteor.startup() needed because 00-startup.js runs *after* code in subdirectories                  // 122
//   (ugh).                                                                                                            // 125
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 126
  globalFrontendRefRegistry.register({                                                                                 // 127
    frontendRefField: "ipInterface",                                                                                   // 128
    typeId: IpRpc.IpInterface.typeId,                                                                                  // 129
    restore: function (db, saveTemplate) {                                                                             // 131
      return new Capnp.Capability(new IpInterfaceImpl(db, saveTemplate), IpRpc.PersistentIpInterface);                 // 132
    },                                                                                                                 // 134
    validate: function (db, session, value) {                                                                          // 136
      check(value, true);                                                                                              // 137
                                                                                                                       //
      if (!session.userId) {                                                                                           // 139
        throw new Meteor.Error(403, "Not logged in.");                                                                 // 140
      }                                                                                                                // 141
                                                                                                                       //
      return {                                                                                                         // 143
        descriptor: {                                                                                                  // 144
          tags: [{                                                                                                     // 144
            id: IpRpc.IpInterface.typeId                                                                               // 144
          }]                                                                                                           // 144
        },                                                                                                             // 144
        requirements: [{                                                                                               // 145
          userIsAdmin: session.userId                                                                                  // 145
        }],                                                                                                            // 145
        frontendRef: value                                                                                             // 146
      };                                                                                                               // 143
    },                                                                                                                 // 148
    query: function (db, userId, value) {                                                                              // 150
      if (userId && Meteor.users.findOne(userId).isAdmin) {                                                            // 151
        return [{                                                                                                      // 152
          _id: "frontendref-ipinterface",                                                                              // 154
          frontendRef: {                                                                                               // 155
            ipInterface: true                                                                                          // 155
          },                                                                                                           // 155
          cardTemplate: "ipInterfacePowerboxCard"                                                                      // 156
        }];                                                                                                            // 153
      } else {                                                                                                         // 159
        return [];                                                                                                     // 160
      }                                                                                                                // 161
    }                                                                                                                  // 162
  });                                                                                                                  // 127
});                                                                                                                    // 164
                                                                                                                       //
BoundUdpPortImpl = function () {                                                                                       // 166
  function BoundUdpPortImpl(server, address, port) {                                                                   // 167
    (0, _classCallCheck3.default)(this, BoundUdpPortImpl);                                                             // 167
    this.server = server;                                                                                              // 168
    this.address = address;                                                                                            // 169
    this.port = port;                                                                                                  // 170
  }                                                                                                                    // 171
                                                                                                                       //
  BoundUdpPortImpl.prototype.send = function () {                                                                      // 166
    function send(message, returnPort) {                                                                               // 166
      // TODO(someday): this whole class is a hack to deal with the fact that we can't compare                         // 174
      // capabilities or build a map with them. What we should be doing is mapping all ports to                        // 175
      // their raw physical address/port, and using that here                                                          // 176
      this.server.send(message, 0, message.length, this.port, this.address);                                           // 177
    }                                                                                                                  // 178
                                                                                                                       //
    return send;                                                                                                       // 166
  }();                                                                                                                 // 166
                                                                                                                       //
  return BoundUdpPortImpl;                                                                                             // 166
}();                                                                                                                   // 166
                                                                                                                       //
var bits16 = Bignum(1).shiftLeft(16).sub(1);                                                                           // 181
var bits32 = Bignum(1).shiftLeft(32).sub(1);                                                                           // 182
                                                                                                                       //
var intToIpv4 = function (num) {                                                                                       // 184
  var part1 = num & 255;                                                                                               // 185
  var part2 = num >> 8 & 255;                                                                                          // 186
  var part3 = num >> 16 & 255;                                                                                         // 187
  var part4 = num >> 24 & 255;                                                                                         // 188
  return part4 + "." + part3 + "." + part2 + "." + part1;                                                              // 190
};                                                                                                                     // 191
                                                                                                                       //
var addressToString = function (address) {                                                                             // 193
  var ipv6num = Bignum(address.upper64).shiftLeft(64).add(Bignum(address.lower64));                                    // 194
                                                                                                                       //
  if (ipv6num.shiftRight(32).eq(bits16)) {                                                                             // 196
    // this is an ipv4 address, we should return it as such                                                            // 197
    var ipv4num = ipv6num.and(bits32).toNumber();                                                                      // 198
    return intToIpv4(ipv4num);                                                                                         // 199
  }                                                                                                                    // 200
                                                                                                                       //
  var hex = ipv6num.toString(16);                                                                                      // 202
  var numColons = 0;                                                                                                   // 203
  var out = "";                                                                                                        // 204
                                                                                                                       //
  for (var i = 0; i < hex.length; ++i) {                                                                               // 206
    // start with lower bits of address and build the output in reverse                                                // 207
    // this ensures that we can place a colon every 4 characters                                                       // 208
    out += hex[hex.length - 1 - i];                                                                                    // 209
                                                                                                                       //
    if ((i + 1) % 4 === 0) {                                                                                           // 210
      out += ":";                                                                                                      // 211
      ++numColons;                                                                                                     // 212
    }                                                                                                                  // 213
  } // Double colon represents all bits being 0                                                                        // 214
                                                                                                                       //
                                                                                                                       //
  if (numColons < 7) {                                                                                                 // 217
    out += "::";                                                                                                       // 218
  }                                                                                                                    // 219
                                                                                                                       //
  return out.split("").reverse().join("");                                                                             // 221
};                                                                                                                     // 222
                                                                                                                       //
var addressType = function (address) {                                                                                 // 224
  var type = "udp4"; // Check if it's an ipv6 address                                                                  // 225
  // TODO(someday): make this less hacky and change address to explicitly pass this information                        // 228
                                                                                                                       //
  if (address.indexOf(":") != -1) {                                                                                    // 229
    type = "udp6";                                                                                                     // 230
  }                                                                                                                    // 231
                                                                                                                       //
  return type;                                                                                                         // 233
};                                                                                                                     // 234
                                                                                                                       //
var IpNetworkImpl = function (_PersistentImpl2) {                                                                      //
  (0, _inherits3.default)(IpNetworkImpl, _PersistentImpl2);                                                            //
                                                                                                                       //
  function IpNetworkImpl(db, saveTemplate, tls) {                                                                      // 237
    (0, _classCallCheck3.default)(this, IpNetworkImpl);                                                                // 237
                                                                                                                       //
    var _this3 = (0, _possibleConstructorReturn3.default)(this, _PersistentImpl2.call(this, db, saveTemplate));        // 237
                                                                                                                       //
    _this3.tls = tls;                                                                                                  // 239
    return _this3;                                                                                                     // 237
  }                                                                                                                    // 240
                                                                                                                       //
  IpNetworkImpl.prototype.getRemoteHost = function () {                                                                //
    function getRemoteHost(address) {                                                                                  //
      // Note that TLS typically authenticates hostnames, not raw IP addresses. So if `this.tls`                       // 243
      // is true, then the returned `IpRemoteHost` will likely refuse to make any connections.                         // 244
      // However, we keep that code path active because there is in principle no reason why it                         // 245
      // should always fail, and we wish to allow for a possible future in which we let users                          // 246
      // specify custom certificate authorities, in which case it might be more likely for TLS                         // 247
      // to be expected to authenticate raw IP addresses.                                                              // 248
      return {                                                                                                         // 250
        host: new IpRemoteHostImpl(addressToString(address), this.tls)                                                 // 250
      };                                                                                                               // 250
    }                                                                                                                  // 251
                                                                                                                       //
    return getRemoteHost;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  IpNetworkImpl.prototype.getRemoteHostByName = function () {                                                          //
    function getRemoteHostByName(address) {                                                                            //
      return {                                                                                                         // 254
        host: new IpRemoteHostImpl(address, this.tls)                                                                  // 254
      };                                                                                                               // 254
    }                                                                                                                  // 255
                                                                                                                       //
    return getRemoteHostByName;                                                                                        //
  }();                                                                                                                 //
                                                                                                                       //
  return IpNetworkImpl;                                                                                                //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
; // TODO(cleanup): Meteor.startup() needed because 00-startup.js runs *after* code in subdirectories                  // 256
//   (ugh).                                                                                                            // 259
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 260
  globalFrontendRefRegistry.register({                                                                                 // 261
    frontendRefField: "ipNetwork",                                                                                     // 262
    typeId: IpRpc.IpNetwork.typeId,                                                                                    // 263
    restore: function (db, saveTemplate, value) {                                                                      // 265
      return new Capnp.Capability(new IpNetworkImpl(db, saveTemplate, "tls" in value.encryption), IpRpc.PersistentIpNetwork);
    },                                                                                                                 // 269
    validate: function (db, session, value) {                                                                          // 271
      check(value, {                                                                                                   // 272
        encryption: Match.OneOf({                                                                                      // 272
          none: null                                                                                                   // 272
        }, {                                                                                                           // 272
          tls: null                                                                                                    // 272
        })                                                                                                             // 272
      });                                                                                                              // 272
                                                                                                                       //
      if (!session.userId) {                                                                                           // 274
        throw new Meteor.Error(403, "Not logged in.");                                                                 // 275
      }                                                                                                                // 276
                                                                                                                       //
      return {                                                                                                         // 278
        descriptor: {                                                                                                  // 279
          tags: [{                                                                                                     // 280
            id: IpRpc.IpNetwork.typeId,                                                                                // 282
            value: Capnp.serialize(IpRpc.IpNetwork.PowerboxTag, value)                                                 // 283
          }]                                                                                                           // 281
        },                                                                                                             // 279
        requirements: [{                                                                                               // 289
          userIsAdmin: session.userId                                                                                  // 289
        }],                                                                                                            // 289
        frontendRef: value                                                                                             // 290
      };                                                                                                               // 278
    },                                                                                                                 // 292
    query: function (db, userId, value) {                                                                              // 294
      var encryption = {                                                                                               // 295
        none: null                                                                                                     // 295
      };                                                                                                               // 295
                                                                                                                       //
      if (value) {                                                                                                     // 296
        encryption = Capnp.parse(IpRpc.IpNetwork.PowerboxTag, value).encryption || encryption;                         // 297
      }                                                                                                                // 298
                                                                                                                       //
      if (userId && Meteor.users.findOne(userId).isAdmin) {                                                            // 300
        return [{                                                                                                      // 301
          _id: "frontendref-ipnetwork",                                                                                // 303
          frontendRef: {                                                                                               // 304
            ipNetwork: {                                                                                               // 304
              encryption: encryption                                                                                   // 304
            }                                                                                                          // 304
          },                                                                                                           // 304
          cardTemplate: "ipNetworkPowerboxCard"                                                                        // 305
        }];                                                                                                            // 302
      } else {                                                                                                         // 308
        return [];                                                                                                     // 309
      }                                                                                                                // 310
    }                                                                                                                  // 311
  });                                                                                                                  // 261
});                                                                                                                    // 313
                                                                                                                       //
var IpRemoteHostImpl = function () {                                                                                   //
  function IpRemoteHostImpl(address, tls) {                                                                            // 316
    (0, _classCallCheck3.default)(this, IpRemoteHostImpl);                                                             // 316
    this.address = address;                                                                                            // 317
    this.tls = tls;                                                                                                    // 318
  }                                                                                                                    // 319
                                                                                                                       //
  IpRemoteHostImpl.prototype.getTcpPort = function () {                                                                //
    function getTcpPort(portNum) {                                                                                     //
      return {                                                                                                         // 322
        port: new TcpPortImpl(this.address, portNum, this.tls)                                                         // 322
      };                                                                                                               // 322
    }                                                                                                                  // 323
                                                                                                                       //
    return getTcpPort;                                                                                                 //
  }();                                                                                                                 //
                                                                                                                       //
  IpRemoteHostImpl.prototype.getUdpPort = function () {                                                                //
    function getUdpPort(portNum) {                                                                                     //
      if (this.tls) {                                                                                                  // 326
        var error = new Error("Datagram Transport Layer Security is not yet supported");                               // 327
        error.kjType = "unimplemented";                                                                                // 328
        throw error;                                                                                                   // 329
      }                                                                                                                // 330
                                                                                                                       //
      return {                                                                                                         // 332
        port: new UdpPortImpl(this.address, portNum)                                                                   // 332
      };                                                                                                               // 332
    }                                                                                                                  // 333
                                                                                                                       //
    return getUdpPort;                                                                                                 //
  }();                                                                                                                 //
                                                                                                                       //
  return IpRemoteHostImpl;                                                                                             //
}();                                                                                                                   //
                                                                                                                       //
;                                                                                                                      // 334
                                                                                                                       //
TcpPortImpl = function () {                                                                                            // 336
  function TcpPortImpl(address, portNum, tls) {                                                                        // 337
    (0, _classCallCheck3.default)(this, TcpPortImpl);                                                                  // 337
    this.address = address;                                                                                            // 338
    this.port = portNum;                                                                                               // 339
    this.tls = tls;                                                                                                    // 340
  }                                                                                                                    // 341
                                                                                                                       //
  TcpPortImpl.prototype.connect = function () {                                                                        // 336
    function connect(downstream) {                                                                                     // 336
      var _this = this;                                                                                                // 344
                                                                                                                       //
      var resolved = false;                                                                                            // 345
      var connectMethod = Net.connect;                                                                                 // 346
                                                                                                                       //
      if (this.tls) {                                                                                                  // 347
        connectMethod = Tls.connect;                                                                                   // 348
      }                                                                                                                // 349
                                                                                                                       //
      return new Promise(function (resolve, reject) {                                                                  // 351
        var client = connectMethod({                                                                                   // 352
          host: _this.address,                                                                                         // 352
          port: _this.port                                                                                             // 352
        }, function () {                                                                                               // 352
          resolved = true;                                                                                             // 353
          resolve({                                                                                                    // 354
            upstream: new ByteStreamConnection(client)                                                                 // 354
          });                                                                                                          // 354
        });                                                                                                            // 355
        client.on("data", function (data) {                                                                            // 357
          downstream.write(data);                                                                                      // 358
        });                                                                                                            // 359
        client.on("close", function (hadError) {                                                                       // 361
          downstream.done();                                                                                           // 362
        });                                                                                                            // 363
        client.on("error", function (err) {                                                                            // 365
          if (resolved) {                                                                                              // 366
            client.write = errorWrite;                                                                                 // 367
          } else {                                                                                                     // 368
            // upstream hasn't been resolved yet, so it's safe to reject                                               // 369
            reject(err);                                                                                               // 370
          }                                                                                                            // 371
        });                                                                                                            // 372
      });                                                                                                              // 373
    }                                                                                                                  // 374
                                                                                                                       //
    return connect;                                                                                                    // 336
  }();                                                                                                                 // 336
                                                                                                                       //
  return TcpPortImpl;                                                                                                  // 336
}();                                                                                                                   // 336
                                                                                                                       //
var errorWrite = function (data) {                                                                                     // 377
  throw new Error("error occurred in connection");                                                                     // 378
};                                                                                                                     // 379
                                                                                                                       //
UdpPortImpl = function () {                                                                                            // 381
  function UdpPortImpl(address, portNum) {                                                                             // 382
    (0, _classCallCheck3.default)(this, UdpPortImpl);                                                                  // 382
    this.address = address;                                                                                            // 383
    this.port = portNum;                                                                                               // 384
    var type = addressType(address);                                                                                   // 386
    this.socket = Dgram.createSocket(type); // TODO(someday): close socket after a certain time of inactivity?         // 387
    // This may be pointless since grains are killed frequently when not in use anyways                                // 390
    // Temporary hack. We only expect clients to pass in a single return port, so we'll store it                       // 392
    // and only send replies here.                                                                                     // 393
    // This will be changed to be correct when equality comparisons are added to capabilities.                         // 394
                                                                                                                       //
    this.returnPort = null;                                                                                            // 395
                                                                                                                       //
    var _this = this;                                                                                                  // 397
                                                                                                                       //
    this.socket.on("message", function (msg, rinfo) {                                                                  // 398
      if (_this.returnPort) {                                                                                          // 399
        _this.returnPort.send(msg, _this);                                                                             // 400
      }                                                                                                                // 401
    });                                                                                                                // 402
  }                                                                                                                    // 403
                                                                                                                       //
  UdpPortImpl.prototype.send = function () {                                                                           // 381
    function send(message, returnPort) {                                                                               // 381
      this.returnPort = returnPort;                                                                                    // 406
      this.socket.send(message, 0, message.length, this.port, this.address); // TODO(someday): use callback to catch errors and do something with them
    }                                                                                                                  // 410
                                                                                                                       //
    return send;                                                                                                       // 381
  }();                                                                                                                 // 381
                                                                                                                       //
  return UdpPortImpl;                                                                                                  // 381
}();                                                                                                                   // 381
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"mail.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/drivers/mail.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");                          //
                                                                                                                       //
var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);                                 //
                                                                                                                       //
var _inherits2 = require("babel-runtime/helpers/inherits");                                                            //
                                                                                                                       //
var _inherits3 = _interopRequireDefault(_inherits2);                                                                   //
                                                                                                                       //
var _this = this;                                                                                                      //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var SMTPServer = void 0;                                                                                               // 1
module.watch(require("smtp-server"), {                                                                                 // 1
  SMTPServer: function (v) {                                                                                           // 1
    SMTPServer = v;                                                                                                    // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var MailParser = void 0;                                                                                               // 1
module.watch(require("mailparser"), {                                                                                  // 1
  MailParser: function (v) {                                                                                           // 1
    MailParser = v;                                                                                                    // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var PersistentImpl = void 0;                                                                                           // 1
module.watch(require("/imports/server/persistent.js"), {                                                               // 1
  PersistentImpl: function (v) {                                                                                       // 1
    PersistentImpl = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var rawSend = void 0;                                                                                                  // 1
module.watch(require("/imports/server/email.js"), {                                                                    // 1
  rawSend: function (v) {                                                                                              // 1
    rawSend = v;                                                                                                       // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var shouldRestartGrain = void 0;                                                                                       // 1
module.watch(require("/imports/server/backend.js"), {                                                                  // 1
  shouldRestartGrain: function (v) {                                                                                   // 1
    shouldRestartGrain = v;                                                                                            // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var inMeteor = void 0;                                                                                                 // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 25
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 26
                                                                                                                       //
var EmailRpc = Capnp.importSystem("sandstorm/email.capnp");                                                            // 28
var EmailImpl = Capnp.importSystem("sandstorm/email-impl.capnp");                                                      // 29
var HackSessionContext = Capnp.importSystem("sandstorm/hack-session.capnp").HackSessionContext;                        // 30
var Supervisor = Capnp.importSystem("sandstorm/supervisor.capnp").Supervisor;                                          // 31
var EmailSendPort = EmailRpc.EmailSendPort;                                                                            // 32
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 34
                                                                                                                       //
var ROOT_URL = Url.parse(process.env.ROOT_URL);                                                                        // 36
var HOSTNAME = ROOT_URL.hostname;                                                                                      // 37
var RECIPIENT_LIMIT = 20;                                                                                              // 39
var CLIENT_TIMEOUT = 15000; // 15s                                                                                     // 41
// Every day, reset all per-user sent counts to zero.                                                                  // 43
// TODO(cleanup): Consider a more granular approach. For example, each user could have a timer                         // 44
//   after which their count will reset. We'd only check the timer when that user is trying to                         // 45
//   send a new message. This avoids a global query.                                                                   // 46
                                                                                                                       //
if (!Meteor.settings.replicaNumber) {                                                                                  // 47
  // only first replica                                                                                                // 47
  SandstormDb.periodicCleanup(86400000, function () {                                                                  // 48
    Meteor.users.update({                                                                                              // 49
      dailySentMailCount: {                                                                                            // 49
        $exists: true                                                                                                  // 49
      }                                                                                                                // 49
    }, {                                                                                                               // 49
      $unset: {                                                                                                        // 50
        dailySentMailCount: ""                                                                                         // 50
      }                                                                                                                // 50
    }, {                                                                                                               // 50
      multi: true                                                                                                      // 51
    });                                                                                                                // 51
  });                                                                                                                  // 52
}                                                                                                                      // 53
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 55
  var server = new SMTPServer({                                                                                        // 56
    banner: "Sandstorm Mail Server",                                                                                   // 57
    authOptional: true,                                                                                                // 58
    disabledCommands: ["STARTTLS", "AUTH"],                                                                            // 59
    onData: function (req, session, callback) {                                                                        // 60
      var mailparser = new MailParser();                                                                               // 61
      req.pipe(mailparser);                                                                                            // 62
      mailparser.on("end", function (mail) {                                                                           // 64
        // Wrap in outer promise for easier error handling.                                                            // 65
        Promise.resolve().then(function () {                                                                           // 66
          // Extract the 'from' address.                                                                               // 67
          var from = void 0;                                                                                           // 68
                                                                                                                       //
          if (mail.from && mail.from.length > 0) {                                                                     // 69
            // It's theoretically possible for the message to have multiple 'from' headers, but this                   // 70
            // never really happens in legitimate practice so we'll just take the first one.                           // 71
            from = mail.from[0];                                                                                       // 72
          } else {                                                                                                     // 73
            // The mail body is missing a 'From:' header. We'll use the bounce address instead,                        // 74
            // with the caveat that this is sometimes *not* the original sender                                        // 75
            // but rather some intermediate agent (e.g. a mailing list daemon). See:                                   // 76
            //   http://en.wikipedia.org/wiki/Bounce_address                                                           // 77
            // TODO(someday): Is this really right, or should we report a blank address instead?                       // 78
            from = {                                                                                                   // 79
              address: session.envelope.mailFrom,                                                                      // 79
              name: false                                                                                              // 79
            };                                                                                                         // 79
          }                                                                                                            // 80
                                                                                                                       //
          var attachments = [];                                                                                        // 82
                                                                                                                       //
          if (mail.attachments) {                                                                                      // 83
            attachments = mail.attachments.map(function (attachment) {                                                 // 84
              var disposition = attachment.contentDisposition || "attachment";                                         // 85
              disposition += ';\n\tfilename="' + (attachment.fileName || attachment.generatedFileName) + '"';          // 86
              return {                                                                                                 // 87
                contentType: attachment.contentType,                                                                   // 88
                contentDisposition: disposition,                                                                       // 89
                contentId: attachment.contentId,                                                                       // 90
                content: attachment.content                                                                            // 91
              };                                                                                                       // 87
            });                                                                                                        // 93
          }                                                                                                            // 94
                                                                                                                       //
          if (mail.replyTo && mail.replyTo.length > 1) {                                                               // 96
            console.error("More than one reply-to address address was received in an email.");                         // 97
          }                                                                                                            // 98
                                                                                                                       //
          var mailMessage = {                                                                                          // 100
            // Note that converting the date to nanoseconds actually goes outside the range of                         // 101
            // integers that Javascript can represent precisely. But this is OK because dates aren't                   // 102
            // precise anyway.                                                                                         // 103
            date: (mail.date || new Date()).getTime() * 1000000,                                                       // 104
            from: from,                                                                                                // 105
            to: mail.to,                                                                                               // 106
            cc: mail.cc || [],                                                                                         // 107
            bcc: mail.bcc || [],                                                                                       // 108
            replyTo: mail.replyTo && mail.replyTo[0] || {},                                                            // 109
            messageId: mail.headers["message-id"] || Meteor.uuid() + "@" + HOSTNAME,                                   // 110
            references: mail.references || [],                                                                         // 111
            inReplyTo: mail.inReplyTo || [],                                                                           // 112
            subject: mail.subject || "",                                                                               // 113
            text: mail.text || null,                                                                                   // 114
            html: mail.html || null,                                                                                   // 115
            attachments: attachments                                                                                   // 116
          }; // Get list of grain IDs.                                                                                 // 100
                                                                                                                       //
          var grainPublicIds = _.uniq(session.envelope.rcptTo.map(function (deliverTo) {                               // 120
            // smtp-server already validates that the address contains an @.                                           // 121
            // To simplify things, we ignore the hostname part of the address and assume that the                      // 122
            // message would not have been sent here if it weren't intended for our host. Usually                      // 123
            // there will be an nginx frontend verifying hostnames anyway. Grain public IDs are                        // 124
            // globally unique anyway, so an e-mail meant for another server presumably won't match                    // 125
            // any ID at this one anyway.                                                                              // 126
            var address = deliverTo.address;                                                                           // 120
            return address.slice(0, address.indexOf("@"));                                                             // 128
          })); // Deliver to each grain in parallel.                                                                   // 129
                                                                                                                       //
                                                                                                                       //
          return Promise.all(grainPublicIds.map(function (publicId) {                                                  // 132
            // Wrap in a function so that we can call it recursively to retry.                                         // 133
            var tryDeliver = function (retryCount) {                                                                   // 134
              var grainId = void 0;                                                                                    // 135
              return inMeteor(function () {                                                                            // 136
                var grain = Grains.findOne({                                                                           // 137
                  publicId: publicId                                                                                   // 137
                }, {                                                                                                   // 137
                  fields: {}                                                                                           // 137
                });                                                                                                    // 137
                                                                                                                       //
                if (grain) {                                                                                           // 138
                  grainId = grain._id;                                                                                 // 139
                  return globalBackend.continueGrain(grainId, retryCount > 0);                                         // 140
                } else {                                                                                               // 141
                  // TODO(someday): We really ought to rig things up so that the 'RCPT TO' SMTP command                // 142
                  // fails in this case, by adding an onRcptTo() callback.                                             // 143
                  throw new Error("No such grain: " + publicId);                                                       // 144
                }                                                                                                      // 145
              }).then(function (grainInfo) {                                                                           // 146
                var supervisor = grainInfo.supervisor;                                                                 // 147
                var uiView = supervisor.getMainView().view; // Create an arbitrary struct to use as the session params. E-mail sessions actually
                // require no params, but node-capnp won't let us pass null and we don't have an                       // 151
                // EmptyStruct type available, so we just use EmailAddress, but any struct type would                  // 152
                // work.                                                                                               // 153
                // TODO(cleanup): Fix node-capnp to accept null.                                                       // 154
                                                                                                                       //
                var emptyParams = Capnp.serialize(EmailRpc.EmailAddress, {}); // Create a new session of type HackEmailSession. This is a short-term hack until
                // persistent capabilities and the Powerbox are implemented. We need to pass along a                   // 158
                // HackSessionContext because old versions of sandstorm-http-bridge always use                         // 159
                // the context of the most recent session; if an old RoundCube grain has a                             // 160
                // WebSession open, receives an email, and then tries to send an email, the request                    // 161
                // will go out on the SessionContext associated with the HackEmailSession that                         // 162
                // delivered the email.                                                                                // 163
                                                                                                                       //
                var session = uiView.newSession({}, makeHackSessionContext(grainId), "0xc3b5ced7344b04a6", emptyParams).session.castAs(EmailSendPort);
                return session.send(mailMessage);                                                                      // 168
              }).catch(function (err) {                                                                                // 169
                if (shouldRestartGrain(err, retryCount)) {                                                             // 170
                  return tryDeliver(retryCount + 1);                                                                   // 171
                } else {                                                                                               // 172
                  throw err;                                                                                           // 173
                }                                                                                                      // 174
              });                                                                                                      // 175
            };                                                                                                         // 176
                                                                                                                       //
            return tryDeliver(0);                                                                                      // 178
          }));                                                                                                         // 179
        }).then(function () {                                                                                          // 180
          callback(null, "Message delivered");                                                                         // 181
        }, function (err) {                                                                                            // 182
          console.error("E-mail delivery failure:", err.stack);                                                        // 183
          var mailErr = new Error(err.message);                                                                        // 184
          err.responseCode = 451;                                                                                      // 185
          callback(err.message);                                                                                       // 186
        });                                                                                                            // 187
      });                                                                                                              // 188
    }                                                                                                                  // 189
  });                                                                                                                  // 56
                                                                                                                       //
  if (global.SANDSTORM_SMTP_LISTEN_HANDLE) {                                                                           // 192
    server.listen(global.SANDSTORM_SMTP_LISTEN_HANDLE);                                                                // 193
  } else {                                                                                                             // 194
    // We must be running `run-dev.sh`.                                                                                // 195
    var BIND_IP = process.env.BIND_IP || "127.0.0.1";                                                                  // 196
    var SMTP_LISTEN_PORT = Meteor.settings.public.smtpListenPort || 30025;                                             // 197
    server.listen(SMTP_LISTEN_PORT, BIND_IP);                                                                          // 198
  }                                                                                                                    // 199
});                                                                                                                    // 200
                                                                                                                       //
hackSendEmail = function (session, email) {                                                                            // 202
  return inMeteor(function () {                                                                                        // 203
    var recipientCount = 0;                                                                                            // 204
    recipientCount += email.to ? email.to.length : 0;                                                                  // 205
    recipientCount += email.cc ? email.cc.length : 0;                                                                  // 206
    recipientCount += email.bcc ? email.bcc.length : 0;                                                                // 207
                                                                                                                       //
    if (recipientCount > RECIPIENT_LIMIT) {                                                                            // 208
      throw new Error("Sorry, Sandstorm currently only allows you to send an e-mail to " + RECIPIENT_LIMIT + " recipients at a time, for spam control. Consider setting up a mailing list. " + "Please feel free to contact us if this is a problem for you.");
    }                                                                                                                  // 213
                                                                                                                       //
    var grainAddress = session._getAddress();                                                                          // 215
                                                                                                                       //
    var userAddress = session._getUserAddress(); // Overwrite the 'from' address with the grain's address.             // 216
                                                                                                                       //
                                                                                                                       //
    if (!email.from) {                                                                                                 // 219
      email.from = {                                                                                                   // 220
        address: grainAddress                                                                                          // 221
      };                                                                                                               // 220
    }                                                                                                                  // 223
                                                                                                                       //
    if (email.from.address !== grainAddress && email.from.address !== userAddress.address) {                           // 225
      throw new Error("FROM header in outgoing emails need to equal either " + grainAddress + " or " + userAddress.address + ". Yours was: " + email.from.address);
    } // Unpack fields                                                                                                 // 229
                                                                                                                       //
                                                                                                                       //
    var from = email.from,                                                                                             // 203
        to = email.to,                                                                                                 // 203
        cc = email.cc,                                                                                                 // 203
        bcc = email.bcc,                                                                                               // 203
        replyTo = email.replyTo,                                                                                       // 203
        subject = email.subject,                                                                                       // 203
        text = email.text,                                                                                             // 203
        html = email.html;                                                                                             // 203
    var options = {                                                                                                    // 234
      from: from,                                                                                                      // 235
      to: to,                                                                                                          // 236
      cc: cc,                                                                                                          // 237
      bcc: bcc,                                                                                                        // 238
      replyTo: replyTo,                                                                                                // 239
      subject: subject,                                                                                                // 240
      text: text,                                                                                                      // 241
      html: html,                                                                                                      // 242
      envelope: {                                                                                                      // 243
        from: grainAddress,                                                                                            // 244
        to: to,                                                                                                        // 245
        cc: cc,                                                                                                        // 246
        bcc: bcc                                                                                                       // 247
      }                                                                                                                // 243
    };                                                                                                                 // 234
    var headers = {};                                                                                                  // 251
                                                                                                                       //
    if (email.messageId) {                                                                                             // 252
      headers["message-id"] = email.messageId;                                                                         // 253
    }                                                                                                                  // 254
                                                                                                                       //
    if (email.references) {                                                                                            // 256
      headers["references"] = email.references; // jscs:ignore requireDotNotation                                      // 257
    }                                                                                                                  // 258
                                                                                                                       //
    if (email.inReplyTo) {                                                                                             // 260
      headers["in-reply-to"] = email.inReplyTo;                                                                        // 261
    }                                                                                                                  // 262
                                                                                                                       //
    if (email.date) {                                                                                                  // 264
      var date = new Date(email.date / 1000000);                                                                       // 265
                                                                                                                       //
      if (!isNaN(date.getTime())) {                                                                                    // 266
        // Check to make sure date is valid                                                                            // 266
        headers["date"] = date.toUTCString(); // jscs:ignore requireDotNotation                                        // 267
      }                                                                                                                // 268
    }                                                                                                                  // 269
                                                                                                                       //
    options.headers = headers;                                                                                         // 271
                                                                                                                       //
    if (email.attachments) {                                                                                           // 273
      var attachments = [];                                                                                            // 274
      email.attachments.forEach(function (attachment) {                                                                // 275
        attachments.push({                                                                                             // 276
          filename: false,                                                                                             // 277
          // Tell nodemailer not to generate a new filename.                                                           // 277
          cid: attachment.contentId,                                                                                   // 278
          contentType: attachment.contentType,                                                                         // 279
          contentDisposition: attachment.contentDisposition,                                                           // 280
          content: attachment.content                                                                                  // 281
        });                                                                                                            // 276
      });                                                                                                              // 283
      options.attachments = attachments;                                                                               // 284
    }                                                                                                                  // 285
                                                                                                                       //
    var grain = Grains.findOne(session.grainId);                                                                       // 287
    if (!grain) throw new Error("Grain does not exist.");                                                              // 288
    globalDb.incrementDailySentMailCount(grain.userId);                                                                // 289
    rawSend(options);                                                                                                  // 291
  }.bind(_this)).catch(function (err) {                                                                                // 292
    console.error("Error sending e-mail:", err.stack);                                                                 // 293
    throw err;                                                                                                         // 294
  });                                                                                                                  // 295
};                                                                                                                     // 296
                                                                                                                       //
var EmailVerifierImpl = function (_PersistentImpl) {                                                                   //
  (0, _inherits3.default)(EmailVerifierImpl, _PersistentImpl);                                                         //
                                                                                                                       //
  function EmailVerifierImpl(db, saveTemplate, params) {                                                               // 299
    (0, _classCallCheck3.default)(this, EmailVerifierImpl);                                                            // 299
                                                                                                                       //
    var _this2 = (0, _possibleConstructorReturn3.default)(this, _PersistentImpl.call(this, db, saveTemplate));         // 299
                                                                                                                       //
    _this2._id = params.id;                                                                                            // 301
    _this2._services = params.services;                                                                                // 302
    return _this2;                                                                                                     // 299
  }                                                                                                                    // 303
                                                                                                                       //
  EmailVerifierImpl.prototype.getId = function () {                                                                    //
    function getId() {                                                                                                 //
      return {                                                                                                         // 306
        id: new Buffer(this._id, "base64")                                                                             // 306
      };                                                                                                               // 306
    }                                                                                                                  // 307
                                                                                                                       //
    return getId;                                                                                                      //
  }();                                                                                                                 //
                                                                                                                       //
  EmailVerifierImpl.prototype.verifyEmail = function () {                                                              //
    function verifyEmail(tabId, verification) {                                                                        //
      var _this3 = this;                                                                                               // 309
                                                                                                                       //
      return unwrapFrontendCap(verification, "verifiedEmail", function (verification) {                                // 310
        if (verification.tabId !== tabId.toString("hex")) {                                                            // 311
          throw new Error("VerifiedEmail is from a different tab");                                                    // 312
        }                                                                                                              // 313
                                                                                                                       //
        if (_this3._services) {                                                                                        // 315
          // Since this verifier is restricted to specific services, only indicate a match if the                      // 316
          // VerifiedEmail was for the correct verifier ID. (If our _services is null, then we                         // 317
          // match all services, and therefore all VerifiedEmails.)                                                    // 318
          if (verification.verifierId !== _this3._id) {                                                                // 319
            throw new Error("VerifierEmail is for a different EmailVerifier.");                                        // 320
          }                                                                                                            // 321
        }                                                                                                              // 322
                                                                                                                       //
        return verification.address;                                                                                   // 324
      });                                                                                                              // 325
    }                                                                                                                  // 326
                                                                                                                       //
    return verifyEmail;                                                                                                //
  }();                                                                                                                 //
                                                                                                                       //
  return EmailVerifierImpl;                                                                                            //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
;                                                                                                                      // 327
                                                                                                                       //
var VerifiedEmailImpl = function (_PersistentImpl2) {                                                                  //
  (0, _inherits3.default)(VerifiedEmailImpl, _PersistentImpl2);                                                        //
                                                                                                                       //
  function VerifiedEmailImpl(db, saveTemplate) {                                                                       // 330
    (0, _classCallCheck3.default)(this, VerifiedEmailImpl);                                                            // 330
    return (0, _possibleConstructorReturn3.default)(this, _PersistentImpl2.call(this, db, saveTemplate));              // 330
  }                                                                                                                    // 332
                                                                                                                       //
  return VerifiedEmailImpl;                                                                                            //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
function getVerifiedEmails(db, userId, verifierId) {                                                                   // 335
  // Get all of the email addresses verified as belonging to the given user using the given                            // 336
  // verifier.                                                                                                         // 337
  var services = null;                                                                                                 // 339
  var couldAddAnother = true;                                                                                          // 340
                                                                                                                       //
  if (verifierId) {                                                                                                    // 342
    var verifier = db.collections.apiTokens.findOne({                                                                  // 343
      "frontendRef.emailVerifier.id": verifierId                                                                       // 344
    });                                                                                                                // 344
    if (!verifier) return []; // invalid verifier                                                                      // 345
                                                                                                                       //
    var verifierInfo = verifier.frontendRef.emailVerifier;                                                             // 346
                                                                                                                       //
    if (verifierInfo.services) {                                                                                       // 348
      // Limit to the listed services.                                                                                 // 349
      services = {};                                                                                                   // 350
      verifierInfo.services.forEach(function (service) {                                                               // 351
        return services[service] = true;                                                                               // 351
      });                                                                                                              // 351
      couldAddAnother = !!services.email;                                                                              // 352
    }                                                                                                                  // 353
  }                                                                                                                    // 354
                                                                                                                       //
  var user = Meteor.users.findOne(userId);                                                                             // 356
  var emails = {}; // map address -> true, for uniquification                                                          // 357
                                                                                                                       //
  Meteor.users.find({                                                                                                  // 359
    _id: {                                                                                                             // 359
      $in: SandstormDb.getUserCredentialIds(user)                                                                      // 359
    }                                                                                                                  // 359
  }).forEach(function (credential) {                                                                                   // 359
    if (!services || services[SandstormDb.getServiceName(credential)]) {                                               // 360
      SandstormDb.getVerifiedEmailsForCredential(credential).forEach(function (email) {                                // 361
        emails[email.email] = true;                                                                                    // 362
      });                                                                                                              // 362
    }                                                                                                                  // 363
  });                                                                                                                  // 364
  return {                                                                                                             // 366
    emails: Object.keys(emails),                                                                                       // 366
    couldAddAnother: couldAddAnother                                                                                   // 366
  };                                                                                                                   // 366
} // TODO(cleanup): Meteor.startup() needed because 00-startup.js runs *after* code in subdirectories                  // 367
//   (ugh).                                                                                                            // 370
                                                                                                                       //
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 371
  globalFrontendRefRegistry.register({                                                                                 // 372
    frontendRefField: "emailVerifier",                                                                                 // 373
    typeId: EmailRpc.EmailVerifier.typeId,                                                                             // 374
    restore: function (db, saveTemplate, params) {                                                                     // 376
      return new Capnp.Capability(new EmailVerifierImpl(db, saveTemplate, params), EmailImpl.PersistentEmailVerifier);
    },                                                                                                                 // 379
    validate: function (db, session, value) {                                                                          // 381
      check(value, {                                                                                                   // 382
        services: Match.Optional([String])                                                                             // 382
      });                                                                                                              // 382
      var services = value.services;                                                                                   // 384
                                                                                                                       //
      if (services) {                                                                                                  // 385
        services.forEach(function (service) {                                                                          // 386
          if (!Accounts.loginServices[service]) {                                                                      // 387
            throw new Error("No such login service: " + service);                                                      // 388
          }                                                                                                            // 389
        });                                                                                                            // 390
      }                                                                                                                // 391
                                                                                                                       //
      value.id = Crypto.randomBytes(16).toString("base64");                                                            // 393
      return {                                                                                                         // 395
        descriptor: {                                                                                                  // 396
          tags: [{                                                                                                     // 396
            id: EmailRpc.EmailVerifier.typeId                                                                          // 396
          }]                                                                                                           // 396
        },                                                                                                             // 396
        requirements: [],                                                                                              // 397
        frontendRef: value                                                                                             // 398
      };                                                                                                               // 395
    },                                                                                                                 // 400
    query: function (db, userId, value) {                                                                              // 402
      var results = [];                                                                                                // 403
      results.push({                                                                                                   // 405
        _id: "emailverifier-all",                                                                                      // 406
        frontendRef: {                                                                                                 // 407
          emailVerifier: {}                                                                                            // 407
        },                                                                                                             // 407
        cardTemplate: "emailVerifierPowerboxCard"                                                                      // 408
      });                                                                                                              // 405
                                                                                                                       //
      for (var name in meteorBabelHelpers.sanitizeForInObject(Accounts.loginServices)) {                               // 411
        if (Accounts.loginServices[name].isEnabled()) {                                                                // 412
          results.push({                                                                                               // 413
            _id: "emailverifier-" + name,                                                                              // 414
            frontendRef: {                                                                                             // 415
              emailVerifier: {                                                                                         // 415
                services: [name]                                                                                       // 415
              }                                                                                                        // 415
            },                                                                                                         // 415
            cardTemplate: "emailVerifierPowerboxCard"                                                                  // 416
          });                                                                                                          // 413
        }                                                                                                              // 418
      }                                                                                                                // 419
                                                                                                                       //
      ;                                                                                                                // 419
      return results;                                                                                                  // 421
    }                                                                                                                  // 422
  });                                                                                                                  // 372
  globalFrontendRefRegistry.register({                                                                                 // 425
    frontendRefField: "verifiedEmail",                                                                                 // 426
    typeId: EmailRpc.VerifiedEmail.typeId,                                                                             // 427
    restore: function (db, saveTemplate) {                                                                             // 429
      return new Capnp.Capability(new VerifiedEmailImpl(db, saveTemplate), EmailImpl.PersistentVerifiedEmail);         // 430
    },                                                                                                                 // 432
    validate: function (db, session, value) {                                                                          // 434
      check(value, {                                                                                                   // 435
        verifierId: Match.Optional(String),                                                                            // 435
        address: String                                                                                                // 435
      });                                                                                                              // 435
                                                                                                                       //
      if (!session.userId) {                                                                                           // 437
        throw new Meteor.Error(403, "Not logged in.");                                                                 // 438
      } // Verify that the address actually belongs to the user.                                                       // 439
                                                                                                                       //
                                                                                                                       //
      if (!_.contains(getVerifiedEmails(db, session.userId, value.verifierId).emails, value.address)) {                // 443
        throw new Meteor.Error(403, "User has no such verified address");                                              // 445
      } // Add the session's tabId.                                                                                    // 446
                                                                                                                       //
                                                                                                                       //
      value.tabId = session.tabId; // Build the descriptor, which contains the verifier ID.                            // 449
                                                                                                                       //
      var tagValue = value.verifierId && Capnp.serialize(EmailRpc.VerifiedEmail.PowerboxTag, {                         // 452
        verifierId: new Buffer(value.verifierId, "hex")                                                                // 454
      });                                                                                                              // 454
      return {                                                                                                         // 456
        descriptor: {                                                                                                  // 457
          tags: [{                                                                                                     // 457
            id: EmailRpc.VerifiedEmail.typeId,                                                                         // 457
            value: tagValue                                                                                            // 457
          }]                                                                                                           // 457
        },                                                                                                             // 457
        requirements: [],                                                                                              // 458
        frontendRef: value                                                                                             // 459
      };                                                                                                               // 456
    },                                                                                                                 // 461
    query: function (db, userId, value) {                                                                              // 463
      var verifierId = value && Capnp.parse(EmailRpc.VerifiedEmail.PowerboxTag, value).verifierId.toString("base64");  // 464
      var verified = getVerifiedEmails(db, userId, verifierId);                                                        // 466
      var result = verified.emails.map(function (address) {                                                            // 467
        return {                                                                                                       // 467
          _id: "email-" + address,                                                                                     // 468
          frontendRef: {                                                                                               // 469
            verifiedEmail: {                                                                                           // 469
              verifierId: verifierId,                                                                                  // 469
              address: address                                                                                         // 469
            }                                                                                                          // 469
          },                                                                                                           // 469
          cardTemplate: "verifiedEmailPowerboxCard",                                                                   // 470
          searchTerms: [address]                                                                                       // 471
        };                                                                                                             // 467
      });                                                                                                              // 467
                                                                                                                       //
      if (verified.couldAddAnother) {                                                                                  // 474
        result.push({                                                                                                  // 475
          _id: "email-add-a-new-one",                                                                                  // 476
          frontendRef: {                                                                                               // 477
            verifiedEmail: {                                                                                           // 477
              verifierId: verifierId                                                                                   // 477
            }                                                                                                          // 477
          },                                                                                                           // 477
          cardTemplate: "addNewVerifiedEmailPowerboxCard",                                                             // 478
          configureTemplate: "addNewVerifiedEmailPowerboxConfiguration"                                                // 479
        });                                                                                                            // 475
      }                                                                                                                // 481
                                                                                                                       //
      return result;                                                                                                   // 483
    }                                                                                                                  // 484
  });                                                                                                                  // 425
});                                                                                                                    // 486
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"00-startup.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/00-startup.js                                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var module1 = module;                                                                                                  // 1
module1.watch(require("/imports/db-deprecated.js"));                                                                   // 1
var FrontendRefRegistry = void 0;                                                                                      // 1
module1.watch(require("/imports/server/frontend-ref.js"), {                                                            // 1
  FrontendRefRegistry: function (v) {                                                                                  // 1
    FrontendRefRegistry = v;                                                                                           // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var PersistentImpl = void 0;                                                                                           // 1
module1.watch(require("/imports/server/persistent.js"), {                                                              // 1
  PersistentImpl: function (v) {                                                                                       // 1
    PersistentImpl = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var migrateToLatest = void 0;                                                                                          // 1
module1.watch(require("/imports/server/migrations.js"), {                                                              // 1
  migrateToLatest: function (v) {                                                                                      // 1
    migrateToLatest = v;                                                                                               // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var ACCOUNT_DELETION_SUSPENSION_TIME = void 0;                                                                         // 1
module1.watch(require("/imports/constants.js"), {                                                                      // 1
  ACCOUNT_DELETION_SUSPENSION_TIME: function (v) {                                                                     // 1
    ACCOUNT_DELETION_SUSPENSION_TIME = v;                                                                              // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var onInMeteor = void 0;                                                                                               // 1
module1.watch(require("/imports/server/async-helpers.js"), {                                                           // 1
  onInMeteor: function (v) {                                                                                           // 1
    onInMeteor = v;                                                                                                    // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var monkeyPatchHttp = void 0;                                                                                          // 1
module1.watch(require("/imports/server/networking.js"), {                                                              // 1
  monkeyPatchHttp: function (v) {                                                                                      // 1
    monkeyPatchHttp = v;                                                                                               // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
var Fiber = void 0;                                                                                                    // 1
module1.watch(require("fibers"), {                                                                                     // 1
  "default": function (v) {                                                                                            // 1
    Fiber = v;                                                                                                         // 1
  }                                                                                                                    // 1
}, 6);                                                                                                                 // 1
                                                                                                                       //
var url = require("url");                                                                                              // 24
                                                                                                                       //
globalFrontendRefRegistry = new FrontendRefRegistry();                                                                 // 26
SandstormPowerbox.registerUiViewQueryHandler(globalFrontendRefRegistry);                                               // 28
                                                                                                                       //
if (global.BlackrockPayments && BlackrockPayments.registerPaymentsApi) {                                               // 30
  // TODO(cleanup): Meteor.startup() needed because unwrapFrontendCap is not defined yet when this                     // 31
  //   first runs. Move it into an import.                                                                             // 32
  Meteor.startup(function () {                                                                                         // 33
    BlackrockPayments.registerPaymentsApi(globalFrontendRefRegistry, PersistentImpl, unwrapFrontendCap);               // 34
  });                                                                                                                  // 36
}                                                                                                                      // 37
                                                                                                                       //
getWildcardOrigin = globalDb.getWildcardOrigin.bind(globalDb);                                                         // 39
Meteor.onConnection(function (connection) {                                                                            // 41
  // TODO(cleanup): This is the best way I've thought of so far to allow methods declared in                           // 42
  //   packages to actually use the DB, but it's pretty sad.                                                           // 43
  connection.sandstormDb = globalDb;                                                                                   // 44
  connection.frontendRefRegistry = globalFrontendRefRegistry;                                                          // 45
});                                                                                                                    // 46
SandstormDb.periodicCleanup(5 * 60 * 1000, SandstormPermissions.cleanupSelfDestructing(globalDb));                     // 47
SandstormDb.periodicCleanup(10 * 60 * 1000, SandstormPermissions.cleanupClientPowerboxTokens(globalDb));               // 48
SandstormDb.periodicCleanup(60 * 60 * 1000, function () {                                                              // 50
  globalDb.cleanupExpiredAssetUploads();                                                                               // 51
});                                                                                                                    // 52
SandstormDb.periodicCleanup(24 * 60 * 60 * 1000, function () {                                                         // 53
  SandstormAutoupdateApps.updateAppIndex(globalDb);                                                                    // 54
});                                                                                                                    // 55
var deleteAccount = global.BlackrockPayments && global.BlackrockPayments.deleteAccount;                                // 56
SandstormDb.periodicCleanup(24 * 60 * 60 * 1000, function () {                                                         // 57
  globalDb.deletePendingAccounts(ACCOUNT_DELETION_SUSPENSION_TIME, globalBackend, deleteAccount);                      // 58
});                                                                                                                    // 60
monkeyPatchHttp(globalDb, HTTP);                                                                                       // 62
Meteor.startup(function () {                                                                                           // 64
  migrateToLatest(globalDb, globalBackend);                                                                            // 64
}); // If there are multiple replicas, prefix every log message with our replica number.                               // 64
                                                                                                                       //
if ("replicaNumber" in Meteor.settings) {                                                                              // 67
  var patchConsole = function (name) {                                                                                 // 67
    var old = console[name];                                                                                           // 71
                                                                                                                       //
    console[name] = function () {                                                                                      // 72
      // Meteor in dev mode writes "LISTENING" to tell the dev runner that it's ready to accept                        // 73
      // connections.                                                                                                  // 74
      if (arguments.length == 1 && arguments[0] == "LISTENING") {                                                      // 75
        old.apply(this, arguments);                                                                                    // 76
      } else {                                                                                                         // 77
        old.apply(this, [prefix].concat(Array.prototype.slice.call(arguments)));                                       // 78
      }                                                                                                                // 79
    };                                                                                                                 // 80
  };                                                                                                                   // 81
                                                                                                                       //
  var prefix = "replica" + Meteor.settings.replicaNumber.toString() + ":";                                             // 68
  patchConsole("log");                                                                                                 // 83
  patchConsole("info");                                                                                                // 84
  patchConsole("warn");                                                                                                // 85
  patchConsole("error");                                                                                               // 86
} // Make the fiber pool size infinite(ish)!                                                                           // 87
//                                                                                                                     // 90
// Each fiber created adds an entry to `v8::Isolate::ThreadDataTable`. Unfortunatley, items are                        // 91
// never deleted from this table. So if we create and delete a lot of fibers, then we leak memory.                     // 92
// Worse yet, the table is represented as a linked list, and v8 performs a linear scan of this                         // 93
// linked list every time we switch fibers. We've seen cases where Sandstorm was spending 65% of                       // 94
// its CPU time just scanning this list! The v8 people say this is "working as intended".                              // 95
//                                                                                                                     // 96
// The fibers package has implemented a work-around by maintaining a fiber pool. Fibers are not                        // 97
// deleted; they are returned to the pool. Unfortunately the pool has a default size of 120. So                        // 98
// if we pass 120 simultaneous fibers, we start leaking and slowing down. It's very easy to hit                        // 99
// this number with a few dozen users present.                                                                         // 100
//                                                                                                                     // 101
// Up until it hits the limit, the fibers module will grow the pool dynamically, starting with an                      // 102
// empty pool and adding each new fiber to it. Therefore, if we set the pool size to an impossibly                     // 103
// large number, we effectively get a pool size equal to the maximum number of simultaneous fibers                     // 104
// seen. This is exactly what we want! Now no fibers are ever deleted, so we never leak. A                             // 105
// Sandstorm server that sees a brief surge of traffic may end up holding on to unused RAM                             // 106
// long-term, but this is a relatively obscure problem.                                                                // 107
//                                                                                                                     // 108
// I initially tried to use the value `Infinity` here, but somehow when this made its way down into                    // 109
// the C++ code and was converted to an integer, that integer ended up being zero. So instead we                       // 110
// use 1e9 (1 billion), which ought to be enough for anyone.                                                           // 111
//                                                                                                                     // 112
// Third-party issues, for reference:                                                                                  // 113
//                                                                                                                     // 114
//    https://bugs.chromium.org/p/v8/issues/detail?id=5338                                                             // 115
//    https://bugs.chromium.org/p/v8/issues/detail?id=3777                                                             // 116
//    https://github.com/laverdet/node-fibers/issues/305                                                               // 117
                                                                                                                       //
                                                                                                                       //
Fiber.poolSize = 1e9; // Special debugging enabled on Blackrock only.                                                  // 119
                                                                                                                       //
if ("replicaNumber" in Meteor.settings) {                                                                              // 122
  var sampleStackIfTooManyFibers = function () {                                                                       // 122
    if (Fiber.fibersCreated > TOO_MANY_FIBERS && sampleCounter++ % FIBER_SAMPLING_PERIOD == 0) {                       // 147
      var stack = new Error().stack;                                                                                   // 148
      console.warn("There are too many fibers!", stack);                                                               // 149
    }                                                                                                                  // 150
  }; // Sample calls to inMeteor().                                                                                    // 151
                                                                                                                       //
                                                                                                                       //
  // Also take samples from various Meteor infrastructure that schedules fibers.                                       // 156
  var addFiberSampling = function (prototype, functionName) {                                                          // 122
    var old = prototype[functionName];                                                                                 // 158
                                                                                                                       //
    prototype[functionName] = function () {                                                                            // 159
      sampleStackIfTooManyFibers();                                                                                    // 160
      return old.apply(this, arguments);                                                                               // 161
    };                                                                                                                 // 162
  };                                                                                                                   // 163
                                                                                                                       //
  console.warn("Fiber bomb defense enabled."); // TEMPORARY: Monitor the number of fibers created and kill the process any time it goes over
  //   2000. Unfortunately, due to the aforementioned linked list in ThreadDataTable, the process                      // 126
  //   will become unreasonably slow once the list gets this big. It's better to kill the process                      // 127
  //   so that it restarts fresh rather than to let the Sandstorm server become unresponsive.                          // 128
  // TODO(soon): Remove this when the bug is fixed.                                                                    // 129
                                                                                                                       //
  setInterval(function () {                                                                                            // 130
    if (Fiber.fibersCreated > 2000) {                                                                                  // 131
      console.error("Process has allocated more than 2000 concurrent fibers. Due to " + "https://bugs.chromium.org/p/v8/issues/detail?id=5338 it will become extremely slow " + "unless we restart it. ABORTING");
      process.abort();                                                                                                 // 136
    }                                                                                                                  // 137
  }, 5000); // Let's also log some stack traces when fiber count gets high so we can find out what's                   // 138
  // happening.                                                                                                        // 141
                                                                                                                       //
  var TOO_MANY_FIBERS = 1000; // Start sampling when we have more than this many fibers.                               // 142
                                                                                                                       //
  var FIBER_SAMPLING_PERIOD = 100; // Sample one stack per this many fibers created.                                   // 143
                                                                                                                       //
  var sampleCounter = 0;                                                                                               // 145
  onInMeteor(sampleStackIfTooManyFibers);                                                                              // 154
  addFiberSampling(Meteor._SynchronousQueue.prototype, "runTask");                                                     // 165
  addFiberSampling(Meteor._SynchronousQueue.prototype, "queueTask");                                                   // 166
  addFiberSampling(Meteor, "bindEnvironment");                                                                         // 167
}                                                                                                                      // 168
                                                                                                                       //
if ("replicaNumber" in Meteor.settings) {                                                                              // 170
  // On Blackrock, dump debug info about local capability counts every minute.                                         // 171
  Meteor.setInterval(Capnp.dumpLocalCapTypeCounts, 60000);                                                             // 172
}                                                                                                                      // 173
                                                                                                                       //
OAuth._checkRedirectUrlOrigin = function (redirectUrl) {                                                               // 175
  // Mostly copied from meteor/packages/oauth/oauth_server.js                                                          // 176
  // We override this method in order to support login from stand-alone grain domains.                                 // 177
  var appHost = Meteor.absoluteUrl();                                                                                  // 178
  var appHostReplacedLocalhost = Meteor.absoluteUrl(undefined, {                                                       // 179
    replaceLocalhost: true                                                                                             // 180
  });                                                                                                                  // 179
  var redirectParsed = url.parse(redirectUrl);                                                                         // 183
  return !(redirectUrl.substr(0, appHost.length) === appHost || redirectUrl.substr(0, appHostReplacedLocalhost.length) === appHostReplacedLocalhost || globalDb.hostIsStandalone(redirectParsed.hostname));
};                                                                                                                     // 190
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"account-suspension.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/account-suspension.js                                                                                        //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var send = void 0;                                                                                                     // 1
module.watch(require("/imports/server/email.js"), {                                                                    // 1
  send: function (v) {                                                                                                 // 1
    send = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
function sendDeletionEmails(db, deletedUserId, byAdminUserId, feedback) {                                              // 19
  var deletedUser = db.getUser(deletedUserId);                                                                         // 20
                                                                                                                       //
  var userEmail = _.findWhere(SandstormDb.getUserEmails(deletedUser), {                                                // 22
    primary: true                                                                                                      // 22
  });                                                                                                                  // 22
                                                                                                                       //
  if (!byAdminUserId) {                                                                                                // 23
    // This was initiated by the user, send them an email                                                              // 23
    if (!userEmail) {                                                                                                  // 24
      console.error("Couldn't send deletion email. No address found for user with userId:", deletedUser._id);          // 25
    } else {                                                                                                           // 27
      var _emailOptions = {                                                                                            // 28
        from: db.getReturnAddress(),                                                                                   // 29
        subject: "Your account on " + db.getServerTitle() + " will be deleted in 7 days.",                             // 30
        text: "You have requested that your Sandstorm account on " + db.getServerTitle() + " be deleted. Your account has been suspended and will be fully deleted in seven days. If you change your mind, log into " + process.env.ROOT_URL + " to cancel the process.\n\nIf you did not request this deletion, please contact the server administrator immediately."
      };                                                                                                               // 28
                                                                                                                       //
      try {                                                                                                            // 35
        _emailOptions.to = userEmail.email;                                                                            // 36
        send(_emailOptions);                                                                                           // 37
      } catch (err) {                                                                                                  // 38
        console.error("Failed to send deletion email to user (id=" + deletedUser._id + ") with error: " + err);        // 39
      }                                                                                                                // 41
    }                                                                                                                  // 42
  }                                                                                                                    // 43
                                                                                                                       //
  var deleteUserString = userEmail && userEmail.email || "userId=" + deletedUser._id;                                  // 45
  var emailOptions = {                                                                                                 // 46
    from: db.getReturnAddress(),                                                                                       // 47
    subject: "Account for " + deleteUserString + " on " + db.getServerTitle() + " will be deleted in 7 days"           // 48
  };                                                                                                                   // 46
                                                                                                                       //
  if (byAdminUserId) {                                                                                                 // 51
    var initiatingAdmin = db.getUser(byAdminUserId);                                                                   // 52
    var adminName = initiatingAdmin.profile.name;                                                                      // 53
    emailOptions.text = adminName + " has requested that the Sandstorm account held by " + deleteUserString + " on " + db.getServerTitle() + " be deleted. The account has been suspended and will be fully deleted in seven days. To cancel the deletion, go to: " + process.env.ROOT_URL + "/admin/users/" + deletedUser._id;
  } else {                                                                                                             // 55
    emailOptions.text = deleteUserString + " has requested that their account be deleted on " + db.getServerTitle() + ". The account has been suspended and will be fully deleted in seven days. To cancel the deletion, go to: " + process.env.ROOT_URL + "/admin/users/" + deletedUser._id;
                                                                                                                       //
    if (feedback) {                                                                                                    // 57
      emailOptions.text += "\nUser gave the following feedback: " + feedback;                                          // 58
    }                                                                                                                  // 59
  }                                                                                                                    // 60
                                                                                                                       //
  Meteor.users.find({                                                                                                  // 62
    isAdmin: true                                                                                                      // 62
  }).forEach(function (user) {                                                                                         // 62
    var email = _.findWhere(SandstormDb.getUserEmails(user), {                                                         // 63
      primary: true                                                                                                    // 63
    });                                                                                                                // 63
                                                                                                                       //
    if (!email) {                                                                                                      // 64
      console.error("No email found for admin with userId:", user._id);                                                // 65
      return;                                                                                                          // 66
    }                                                                                                                  // 67
                                                                                                                       //
    try {                                                                                                              // 69
      emailOptions.to = email.email;                                                                                   // 70
      send(emailOptions);                                                                                              // 71
    } catch (err) {                                                                                                    // 72
      console.error("Failed to send deletion email to admin (id=" + user._id + ") with error: " + err);                // 73
    }                                                                                                                  // 75
  });                                                                                                                  // 76
}                                                                                                                      // 77
                                                                                                                       //
Meteor.methods({                                                                                                       // 79
  suspendAccount: function (userId, willDelete) {                                                                      // 80
    check(userId, String);                                                                                             // 81
    check(willDelete, Boolean);                                                                                        // 82
                                                                                                                       //
    if (!isAdmin()) {                                                                                                  // 84
      throw new Meteor.Error(403, "Only admins can suspend other users.");                                             // 85
    }                                                                                                                  // 86
                                                                                                                       //
    if (userId === Meteor.userId()) {                                                                                  // 88
      throw new Meteor.Error(400, "Admins cannot suspend their own accounts from the admin page. Please go to your account setttings.");
    }                                                                                                                  // 90
                                                                                                                       //
    var db = this.connection.sandstormDb;                                                                              // 92
                                                                                                                       //
    if (global.BlackrockPayments) {                                                                                    // 94
      BlackrockPayments.suspendAccount(db, userId);                                                                    // 95
    }                                                                                                                  // 96
                                                                                                                       //
    db.suspendAccount(userId, Meteor.userId(), willDelete);                                                            // 98
                                                                                                                       //
    if (willDelete) {                                                                                                  // 100
      sendDeletionEmails(db, userId, Meteor.userId());                                                                 // 101
    }                                                                                                                  // 102
  },                                                                                                                   // 103
  deleteOwnAccount: function (feedback) {                                                                              // 105
    var db = this.connection.sandstormDb;                                                                              // 106
                                                                                                                       //
    if (!Meteor.userId()) {                                                                                            // 107
      throw new Meteor.Error(403, "Must be logged in to delete an account");                                           // 108
    }                                                                                                                  // 109
                                                                                                                       //
    if (db.isUserInOrganization(Meteor.user())) {                                                                      // 111
      throw new Meteor.Error(403, "Users in an organization cannot delete their own account. " + "Please ask your admin to do it for you.");
    }                                                                                                                  // 114
                                                                                                                       //
    if (global.BlackrockPayments) {                                                                                    // 116
      BlackrockPayments.suspendAccount(db, Meteor.userId());                                                           // 117
    }                                                                                                                  // 118
                                                                                                                       //
    db.suspendAccount(Meteor.userId(), null, true);                                                                    // 120
    sendDeletionEmails(db, Meteor.userId(), null, feedback);                                                           // 122
  },                                                                                                                   // 123
  unsuspendAccount: function (userId) {                                                                                // 125
    check(userId, String);                                                                                             // 126
                                                                                                                       //
    if (!isAdmin()) {                                                                                                  // 128
      throw new Meteor.Error(403, "Only admins can unsuspend other users.");                                           // 129
    }                                                                                                                  // 130
                                                                                                                       //
    this.connection.sandstormDb.unsuspendAccount(userId, Meteor.userId());                                             // 132
  },                                                                                                                   // 133
  unsuspendOwnAccount: function () {                                                                                   // 135
    if (!Meteor.userId()) {                                                                                            // 136
      throw new Meteor.Error(403, "Must be logged in to unsuspend an account");                                        // 137
    }                                                                                                                  // 138
                                                                                                                       //
    this.connection.sandstormDb.unsuspendAccount(Meteor.userId());                                                     // 140
  }                                                                                                                    // 141
});                                                                                                                    // 79
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"admin-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/admin-server.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _objectWithoutProperties2 = require("babel-runtime/helpers/objectWithoutProperties");                              //
                                                                                                                       //
var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);                                     //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var Fs = void 0;                                                                                                       // 1
module.watch(require("fs"), {                                                                                          // 1
  "default": function (v) {                                                                                            // 1
    Fs = v;                                                                                                            // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Crypto = void 0;                                                                                                   // 1
module.watch(require("crypto"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    Crypto = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var Heapdump = void 0;                                                                                                 // 1
module.watch(require("heapdump"), {                                                                                    // 1
  "default": function (v) {                                                                                            // 1
    Heapdump = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var SANDSTORM_LOGDIR = void 0;                                                                                         // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  SANDSTORM_LOGDIR: function (v) {                                                                                     // 1
    SANDSTORM_LOGDIR = v;                                                                                              // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var clearAdminToken = void 0,                                                                                          // 1
    checkAuth = void 0,                                                                                                // 1
    tokenIsValid = void 0,                                                                                             // 1
    tokenIsSetupSession = void 0;                                                                                      // 1
module.watch(require("/imports/server/auth.js"), {                                                                     // 1
  clearAdminToken: function (v) {                                                                                      // 1
    clearAdminToken = v;                                                                                               // 1
  },                                                                                                                   // 1
  checkAuth: function (v) {                                                                                            // 1
    checkAuth = v;                                                                                                     // 1
  },                                                                                                                   // 1
  tokenIsValid: function (v) {                                                                                         // 1
    tokenIsValid = v;                                                                                                  // 1
  },                                                                                                                   // 1
  tokenIsSetupSession: function (v) {                                                                                  // 1
    tokenIsSetupSession = v;                                                                                           // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
var sendEmail = void 0;                                                                                                // 1
module.watch(require("/imports/server/email.js"), {                                                                    // 1
  send: function (v) {                                                                                                 // 1
    sendEmail = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 6);                                                                                                                 // 1
var fillUndefinedForChangedDoc = void 0;                                                                               // 1
module.watch(require("/imports/server/observe-helpers.js"), {                                                          // 1
  fillUndefinedForChangedDoc: function (v) {                                                                           // 1
    fillUndefinedForChangedDoc = v;                                                                                    // 1
  }                                                                                                                    // 1
}, 7);                                                                                                                 // 1
var publicAdminSettings = ["google", "github", "ldap", "saml", "emailToken", "splashUrl", "signupDialog", "adminAlert", "adminAlertTime", "adminAlertUrl", "termsUrl", "privacyUrl", "appMarketUrl", "appIndexUrl", "appUpdatesEnabled", "serverTitle", "returnAddress", "ldapNameField", "organizationMembership", "organizationSettings", "whitelabelCustomLoginProviderName", "whitelabelCustomLogoAssetId", "whitelabelHideSendFeedback", "whitelabelHideTroubleshooting", "whiteLabelHideAbout", "whitelabelUseServerTitleForHomeText", "quotaEnabled", "quotaLdapEnabled", "billingPromptUrl"];
var smtpConfigShape = {                                                                                                // 43
  hostname: String,                                                                                                    // 44
  port: Number,                                                                                                        // 45
  auth: {                                                                                                              // 46
    user: String,                                                                                                      // 47
    pass: String                                                                                                       // 48
  },                                                                                                                   // 46
  returnAddress: String                                                                                                // 50
};                                                                                                                     // 43
Meteor.methods({                                                                                                       // 53
  setAccountSetting: function (token, serviceName, value) {                                                            // 54
    checkAuth(token);                                                                                                  // 55
    check(serviceName, String);                                                                                        // 56
    check(value, Boolean); // Only check configurations for OAuth services.                                            // 57
                                                                                                                       //
    var oauthServices = ["google", "github"];                                                                          // 60
                                                                                                                       //
    if (value && oauthServices.indexOf(serviceName) != -1) {                                                           // 61
      var config = ServiceConfiguration.configurations.findOne({                                                       // 62
        service: serviceName                                                                                           // 62
      });                                                                                                              // 62
                                                                                                                       //
      if (!config) {                                                                                                   // 63
        throw new Meteor.Error(403, "You must configure the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
      }                                                                                                                // 66
                                                                                                                       //
      if (!config.clientId || !config.secret) {                                                                        // 68
        throw new Meteor.Error(403, "You must provide a non-empty clientId and secret for the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
      }                                                                                                                // 71
    }                                                                                                                  // 72
                                                                                                                       //
    Settings.upsert({                                                                                                  // 74
      _id: serviceName                                                                                                 // 74
    }, {                                                                                                               // 74
      $set: {                                                                                                          // 74
        value: value                                                                                                   // 74
      }                                                                                                                // 74
    });                                                                                                                // 74
                                                                                                                       //
    if (value) {                                                                                                       // 75
      Settings.update({                                                                                                // 76
        _id: serviceName                                                                                               // 76
      }, {                                                                                                             // 76
        $unset: {                                                                                                      // 76
          automaticallyReset: 1                                                                                        // 76
        }                                                                                                              // 76
      });                                                                                                              // 76
    }                                                                                                                  // 77
  },                                                                                                                   // 78
  setSmtpConfig: function (token, config) {                                                                            // 80
    checkAuth(token);                                                                                                  // 81
    check(config, smtpConfigShape);                                                                                    // 82
    Settings.upsert({                                                                                                  // 84
      _id: "smtpConfig"                                                                                                // 84
    }, {                                                                                                               // 84
      $set: {                                                                                                          // 84
        value: config                                                                                                  // 84
      }                                                                                                                // 84
    });                                                                                                                // 84
  },                                                                                                                   // 85
  disableEmail: function (token) {                                                                                     // 87
    checkAuth(token);                                                                                                  // 88
    var db = this.connection.sandstormDb;                                                                              // 90
    db.collections.settings.update({                                                                                   // 91
      _id: "smtpConfig"                                                                                                // 91
    }, {                                                                                                               // 91
      $set: {                                                                                                          // 91
        "value.hostname": ""                                                                                           // 91
      }                                                                                                                // 91
    });                                                                                                                // 91
  },                                                                                                                   // 92
  setSetting: function (token, name, value) {                                                                          // 94
    checkAuth(token);                                                                                                  // 95
    check(name, String);                                                                                               // 96
    check(value, Match.OneOf(null, String, Date, Boolean));                                                            // 97
    Settings.upsert({                                                                                                  // 99
      _id: name                                                                                                        // 99
    }, {                                                                                                               // 99
      $set: {                                                                                                          // 99
        value: value                                                                                                   // 99
      }                                                                                                                // 99
    });                                                                                                                // 99
  },                                                                                                                   // 100
  saveOrganizationSettings: function (token, params) {                                                                 // 102
    checkAuth(token);                                                                                                  // 103
    check(params, {                                                                                                    // 104
      membership: {                                                                                                    // 105
        emailToken: {                                                                                                  // 106
          enabled: Boolean,                                                                                            // 107
          domain: String                                                                                               // 108
        },                                                                                                             // 106
        google: {                                                                                                      // 110
          enabled: Boolean,                                                                                            // 111
          domain: String                                                                                               // 112
        },                                                                                                             // 110
        ldap: {                                                                                                        // 114
          enabled: Boolean                                                                                             // 115
        },                                                                                                             // 114
        saml: {                                                                                                        // 117
          enabled: Boolean                                                                                             // 118
        }                                                                                                              // 117
      },                                                                                                               // 105
      settings: {                                                                                                      // 121
        disallowGuests: Boolean,                                                                                       // 122
        shareContacts: Boolean                                                                                         // 123
      }                                                                                                                // 121
    });                                                                                                                // 104
    this.connection.sandstormDb.collections.settings.upsert({                                                          // 127
      _id: "organizationMembership"                                                                                    // 127
    }, {                                                                                                               // 127
      value: params.membership                                                                                         // 127
    });                                                                                                                // 127
    this.connection.sandstormDb.collections.settings.upsert({                                                          // 128
      _id: "organizationSettings"                                                                                      // 128
    }, {                                                                                                               // 128
      value: params.settings                                                                                           // 128
    });                                                                                                                // 128
  },                                                                                                                   // 129
  adminConfigureLoginService: function (token, options) {                                                              // 131
    checkAuth(token);                                                                                                  // 132
    check(options, Match.ObjectIncluding({                                                                             // 133
      service: String                                                                                                  // 133
    }));                                                                                                               // 133
    ServiceConfiguration.configurations.upsert({                                                                       // 135
      service: options.service                                                                                         // 135
    }, options);                                                                                                       // 135
  },                                                                                                                   // 136
  clearResumeTokensForService: function (token, serviceName) {                                                         // 138
    checkAuth(token);                                                                                                  // 139
    check(serviceName, String);                                                                                        // 140
    var query = {};                                                                                                    // 142
    query["services." + serviceName] = {                                                                               // 143
      $exists: true                                                                                                    // 143
    };                                                                                                                 // 143
    Meteor.users.find(query).forEach(function (credential) {                                                           // 144
      if (credential.services.resume && credential.services.resume.loginTokens && credential.services.resume.loginTokens.length > 0) {
        Meteor.users.update({                                                                                          // 147
          _id: credential._id                                                                                          // 147
        }, {                                                                                                           // 147
          $set: {                                                                                                      // 147
            "services.resume.loginTokens": []                                                                          // 147
          }                                                                                                            // 147
        });                                                                                                            // 147
      }                                                                                                                // 148
                                                                                                                       //
      Meteor.users.update({                                                                                            // 150
        "loginCredentials.id": credential._id                                                                          // 150
      }, {                                                                                                             // 150
        $set: {                                                                                                        // 151
          "services.resume.loginTokens": []                                                                            // 151
        }                                                                                                              // 151
      });                                                                                                              // 151
    });                                                                                                                // 152
  },                                                                                                                   // 153
  adminUpdateUser: function (token, userInfo) {                                                                        // 155
    checkAuth(token);                                                                                                  // 156
    check(userInfo, {                                                                                                  // 157
      userId: String,                                                                                                  // 158
      signupKey: Boolean,                                                                                              // 159
      isAdmin: Boolean                                                                                                 // 160
    });                                                                                                                // 157
    var userId = userInfo.userId;                                                                                      // 163
                                                                                                                       //
    if (userId === Meteor.userId() && !userInfo.isAdmin) {                                                             // 164
      throw new Meteor.Error(403, "User cannot remove admin permissions from itself.");                                // 165
    }                                                                                                                  // 166
                                                                                                                       //
    Meteor.users.update({                                                                                              // 168
      _id: userId                                                                                                      // 168
    }, {                                                                                                               // 168
      $set: _.omit(userInfo, ["_id", "userId"])                                                                        // 168
    });                                                                                                                // 168
  },                                                                                                                   // 169
  testSend: function (token, smtpConfig, to) {                                                                         // 171
    checkAuth(token);                                                                                                  // 172
    check(smtpConfig, smtpConfigShape);                                                                                // 173
    check(to, String);                                                                                                 // 174
    var returnAddress = smtpConfig.returnAddress,                                                                      // 171
        restConfig = (0, _objectWithoutProperties3.default)(smtpConfig, ["returnAddress"]);                            // 171
                                                                                                                       //
    try {                                                                                                              // 177
      sendEmail({                                                                                                      // 178
        to: to,                                                                                                        // 179
        from: {                                                                                                        // 180
          name: globalDb.getServerTitle(),                                                                             // 180
          address: returnAddress                                                                                       // 180
        },                                                                                                             // 180
        subject: "Testing your Sandstorm's SMTP setting",                                                              // 181
        text: "Success! Your outgoing SMTP is working.",                                                               // 182
        smtpConfig: restConfig                                                                                         // 183
      });                                                                                                              // 178
    } catch (e) {                                                                                                      // 185
      // Attempt to give more accurate error messages for a variety of known failure modes,                            // 186
      // and the actual exception data in the event a user hits a new failure mode.                                    // 187
      if (e.syscall === "getaddrinfo") {                                                                               // 188
        if (e.code === "EIO" || e.code === "ENOTFOUND") {                                                              // 189
          throw new Meteor.Error("getaddrinfo " + e.code, "Couldn't resolve \"" + smtpConfig.hostname + "\" - check for typos or broken DNS.");
        }                                                                                                              // 191
      } else if (e.syscall === "connect") {                                                                            // 192
        if (e.code === "ECONNREFUSED") {                                                                               // 193
          throw new Meteor.Error("connect ECONNREFUSED", "Server at " + smtpConfig.hostname + ":" + smtpConfig.port + " refused connection.  Check your settings, firewall rules, and that your mail server is up.");
        }                                                                                                              // 195
      } else if (e.name === "AuthError") {                                                                             // 196
        throw new Meteor.Error("auth error", "Authentication failed.  Check your credentials.  Message from " + smtpConfig.hostname + ": " + e.data);
      }                                                                                                                // 199
                                                                                                                       //
      throw new Meteor.Error("other-email-sending-error", "Error while trying to send test email: " + JSON.stringify(e));
    }                                                                                                                  // 202
  },                                                                                                                   // 203
  createSignupKey: function (token, note, quota) {                                                                     // 205
    checkAuth(token);                                                                                                  // 206
    check(note, String);                                                                                               // 207
    check(quota, Match.OneOf(undefined, null, Number));                                                                // 208
    var key = Random.id();                                                                                             // 210
    var content = {                                                                                                    // 211
      _id: key,                                                                                                        // 211
      used: false,                                                                                                     // 211
      note: note                                                                                                       // 211
    };                                                                                                                 // 211
    if (typeof quota === "number") content.quota = quota;                                                              // 212
    SignupKeys.insert(content);                                                                                        // 213
    return key;                                                                                                        // 214
  },                                                                                                                   // 215
  sendInvites: function (token, origin, from, list, subject, message, quota) {                                         // 217
    checkAuth(token);                                                                                                  // 218
    check(from, {                                                                                                      // 219
      name: String,                                                                                                    // 219
      address: String                                                                                                  // 219
    });                                                                                                                // 219
    check([origin, list, subject, message], [String]);                                                                 // 220
    check(quota, Match.OneOf(undefined, null, Number));                                                                // 221
                                                                                                                       //
    if (!from.address.trim()) {                                                                                        // 223
      throw new Meteor.Error(403, "Must enter 'from' address.");                                                       // 224
    }                                                                                                                  // 225
                                                                                                                       //
    if (!list.trim()) {                                                                                                // 227
      throw new Meteor.Error(403, "Must enter 'to' addresses.");                                                       // 228
    }                                                                                                                  // 229
                                                                                                                       //
    this.unblock();                                                                                                    // 231
    list = list.split("\n");                                                                                           // 233
                                                                                                                       //
    for (var i in meteorBabelHelpers.sanitizeForInObject(list)) {                                                      // 234
      var email = list[i].trim();                                                                                      // 235
                                                                                                                       //
      if (email) {                                                                                                     // 237
        var key = Random.id();                                                                                         // 238
        var content = {                                                                                                // 240
          _id: key,                                                                                                    // 241
          used: false,                                                                                                 // 242
          note: "E-mail invite to " + email,                                                                           // 243
          email: email,                                                                                                // 244
          definitelySent: false                                                                                        // 245
        };                                                                                                             // 240
        if (typeof quota === "number") content.quota = quota;                                                          // 247
        SignupKeys.insert(content);                                                                                    // 248
        sendEmail({                                                                                                    // 249
          to: email,                                                                                                   // 250
          from: from,                                                                                                  // 251
          envelopeFrom: globalDb.getReturnAddress(),                                                                   // 252
          subject: subject,                                                                                            // 253
          text: message.replace(/\$KEY/g, origin + "/signup/" + key)                                                   // 254
        });                                                                                                            // 249
        SignupKeys.update(key, {                                                                                       // 256
          $set: {                                                                                                      // 256
            definitelySent: true                                                                                       // 256
          }                                                                                                            // 256
        });                                                                                                            // 256
      }                                                                                                                // 257
    }                                                                                                                  // 258
                                                                                                                       //
    return {                                                                                                           // 260
      sent: true                                                                                                       // 260
    };                                                                                                                 // 260
  },                                                                                                                   // 261
  adminToggleDisableCap: function (token, capId, value) {                                                              // 263
    checkAuth(token);                                                                                                  // 264
    check(capId, String);                                                                                              // 265
    check(value, Boolean);                                                                                             // 266
                                                                                                                       //
    if (value) {                                                                                                       // 268
      ApiTokens.update({                                                                                               // 269
        _id: capId                                                                                                     // 269
      }, {                                                                                                             // 269
        $set: {                                                                                                        // 269
          revoked: true                                                                                                // 269
        }                                                                                                              // 269
      });                                                                                                              // 269
    } else {                                                                                                           // 270
      ApiTokens.update({                                                                                               // 271
        _id: capId                                                                                                     // 271
      }, {                                                                                                             // 271
        $set: {                                                                                                        // 271
          revoked: false                                                                                               // 271
        }                                                                                                              // 271
      });                                                                                                              // 271
    }                                                                                                                  // 272
  },                                                                                                                   // 273
  updateQuotas: function (token, list, quota) {                                                                        // 275
    checkAuth(token);                                                                                                  // 276
    check(list, String);                                                                                               // 277
    check(quota, Match.OneOf(undefined, null, Number));                                                                // 278
                                                                                                                       //
    if (!list.trim()) {                                                                                                // 280
      throw new Meteor.Error(400, "Must enter addresses.");                                                            // 281
    }                                                                                                                  // 282
                                                                                                                       //
    var items = list.split("\n");                                                                                      // 284
    var invalid = [];                                                                                                  // 285
                                                                                                                       //
    for (var i in meteorBabelHelpers.sanitizeForInObject(items)) {                                                     // 286
      var modifier = typeof quota === "number" ? {                                                                     // 287
        $set: {                                                                                                        // 287
          quota: quota                                                                                                 // 287
        }                                                                                                              // 287
      } : {                                                                                                            // 287
        $unset: {                                                                                                      // 288
          quota: ""                                                                                                    // 288
        }                                                                                                              // 288
      };                                                                                                               // 288
      var n = SignupKeys.update({                                                                                      // 289
        email: items[i]                                                                                                // 289
      }, modifier, {                                                                                                   // 289
        multi: true                                                                                                    // 289
      });                                                                                                              // 289
      n += Meteor.users.update({                                                                                       // 290
        signupEmail: items[i]                                                                                          // 290
      }, modifier, {                                                                                                   // 290
        multi: true                                                                                                    // 290
      });                                                                                                              // 290
      if (n < 1) invalid.push(items[i]);                                                                               // 292
    }                                                                                                                  // 293
                                                                                                                       //
    if (invalid.length > 0) {                                                                                          // 295
      throw new Meteor.Error(404, "These addresses did not map to any user nor invite: " + invalid.join(", "));        // 296
    }                                                                                                                  // 298
  },                                                                                                                   // 299
  dismissAdminStatsNotifications: function (token) {                                                                   // 301
    checkAuth(token);                                                                                                  // 302
    globalDb.collections.notifications.remove({                                                                        // 303
      "admin.type": "reportStats"                                                                                      // 303
    });                                                                                                                // 303
  },                                                                                                                   // 304
  signUpAsAdmin: function (token) {                                                                                    // 306
    check(token, String);                                                                                              // 307
    checkAuth(token);                                                                                                  // 308
                                                                                                                       //
    if (!this.userId) {                                                                                                // 309
      throw new Meteor.Error(403, "Must be logged in to sign up as admin.");                                           // 310
    }                                                                                                                  // 311
                                                                                                                       //
    if (!Meteor.user().loginCredentials) {                                                                             // 313
      throw new Meteor.Error(403, "Must be logged into an account to sign up as admin.");                              // 314
    }                                                                                                                  // 315
                                                                                                                       //
    Meteor.users.update({                                                                                              // 317
      _id: this.userId                                                                                                 // 317
    }, {                                                                                                               // 317
      $set: {                                                                                                          // 317
        isAdmin: true,                                                                                                 // 317
        signupKey: "admin"                                                                                             // 317
      }                                                                                                                // 317
    });                                                                                                                // 317
    clearAdminToken(token);                                                                                            // 318
  },                                                                                                                   // 319
  redeemSetupToken: function (token) {                                                                                 // 321
    // Redeem an admin token into a setup session.                                                                     // 322
    check(token, String);                                                                                              // 323
                                                                                                                       //
    if (tokenIsValid(token)) {                                                                                         // 324
      var sessId = Random.secret();                                                                                    // 325
      var creationDate = new Date();                                                                                   // 326
      var hashedSessionId = Crypto.createHash("sha256").update(sessId).digest("base64");                               // 327
      this.connection.sandstormDb.collections.setupSession.upsert({                                                    // 328
        _id: "current-session"                                                                                         // 329
      }, {                                                                                                             // 328
        creationDate: creationDate,                                                                                    // 331
        hashedSessionId: hashedSessionId                                                                               // 332
      }); // Then, invalidate the token, so one one else can use it.                                                   // 330
                                                                                                                       //
      clearAdminToken(token);                                                                                          // 335
      return sessId;                                                                                                   // 336
    } else {                                                                                                           // 337
      throw new Meteor.Error(401, "Invalid setup token");                                                              // 338
    }                                                                                                                  // 339
  },                                                                                                                   // 340
  heapdump: function () {                                                                                              // 342
    // Requests a heap dump. Intended for use by Sandstorm developers. Requires admin.                                 // 343
    //                                                                                                                 // 344
    // Call this from the JS console like:                                                                             // 345
    //   Meteor.call("heapdump");                                                                                      // 346
    checkAuth(); // We use /var/log because it's a location in the container to which the front-end is allowed         // 348
    // to write.                                                                                                       // 351
                                                                                                                       //
    var name = "/var/log/" + Date.now() + ".heapsnapshot";                                                             // 352
    Heapdump.writeSnapshot(name);                                                                                      // 353
    console.log("Wrote heapdump: /opt/sandstorm" + name);                                                              // 354
    return name;                                                                                                       // 355
  },                                                                                                                   // 356
  setPreinstalledApps: function (appAndPackageIds) {                                                                   // 358
    checkAuth();                                                                                                       // 359
    check(appAndPackageIds, [{                                                                                         // 360
      appId: String,                                                                                                   // 360
      packageId: String                                                                                                // 360
    }]);                                                                                                               // 360
    this.connection.sandstormDb.setPreinstalledApps(appAndPackageIds);                                                 // 362
  }                                                                                                                    // 363
});                                                                                                                    // 53
                                                                                                                       //
var authorizedAsAdmin = function (token, userId) {                                                                     // 366
  return Match.test(token, Match.OneOf(undefined, null, String)) && (userId && isAdminById(userId) || tokenIsValid(token) || tokenIsSetupSession(token));
};                                                                                                                     // 369
                                                                                                                       //
Meteor.publish("admin", function (token) {                                                                             // 371
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 372
  return Settings.find();                                                                                              // 373
});                                                                                                                    // 374
Meteor.publish("adminServiceConfiguration", function (token) {                                                         // 376
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 377
  return ServiceConfiguration.configurations.find();                                                                   // 378
});                                                                                                                    // 379
Meteor.publish("publicAdminSettings", function () {                                                                    // 381
  return Settings.find({                                                                                               // 382
    _id: {                                                                                                             // 382
      $in: publicAdminSettings                                                                                         // 382
    }                                                                                                                  // 382
  });                                                                                                                  // 382
});                                                                                                                    // 383
Meteor.publish("adminToken", function (token) {                                                                        // 385
  check(token, String);                                                                                                // 386
  this.added("adminToken", "adminToken", {                                                                             // 387
    tokenIsValid: tokenIsValid(token) || tokenIsSetupSession(token)                                                    // 387
  });                                                                                                                  // 387
  this.ready();                                                                                                        // 388
});                                                                                                                    // 389
Meteor.publish("allUsers", function (token) {                                                                          // 391
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 392
  return Meteor.users.find();                                                                                          // 393
});                                                                                                                    // 394
Meteor.publish("adminUserDetails", function (userId) {                                                                 // 396
  var _this2 = this;                                                                                                   // 396
                                                                                                                       //
  if (!authorizedAsAdmin(undefined, this.userId)) return []; // Reactive publish of any credentials owned by the account with id userId,
  // as well as that user object itself.                                                                               // 400
                                                                                                                       //
  var credentialSubs = {};                                                                                             // 401
  var accountId = userId;                                                                                              // 402
                                                                                                                       //
  var unrefCredential = function (credentialId) {                                                                      // 404
    if (!credentialSubs[credentialId]) {                                                                               // 405
      // should never happen, but if somehow you attempt to unref an credential that we don't have a                   // 406
      // subscription to, then don't crash                                                                             // 407
      console.error("attempted to unref untracked credential id:", credentialId);                                      // 408
      return;                                                                                                          // 409
    }                                                                                                                  // 410
                                                                                                                       //
    var observeHandle = credentialSubs[credentialId];                                                                  // 412
    delete credentialSubs[credentialId];                                                                               // 413
    observeHandle.stop();                                                                                              // 414
                                                                                                                       //
    _this2.removed("users", credentialId);                                                                             // 415
  };                                                                                                                   // 416
                                                                                                                       //
  var refCredential = function (credentialId) {                                                                        // 418
    if (credentialSubs[credentialId]) {                                                                                // 419
      // should never happen, but if somehow an account wound up with a duplicate credential ID,                       // 420
      // avoid leaking a subscription                                                                                  // 421
      console.error("duplicate credential id:", credentialId);                                                         // 422
      return;                                                                                                          // 423
    }                                                                                                                  // 424
                                                                                                                       //
    var cursor = Meteor.users.find({                                                                                   // 426
      _id: credentialId                                                                                                // 426
    });                                                                                                                // 426
    var observeHandle = cursor.observe({                                                                               // 427
      added: function (doc) {                                                                                          // 428
        _this2.added("users", doc._id, doc);                                                                           // 429
      },                                                                                                               // 430
      changed: function (newDoc, oldDoc) {                                                                             // 432
        fillUndefinedForChangedDoc(newDoc, oldDoc);                                                                    // 433
                                                                                                                       //
        _this2.changed("users", newDoc._id, newDoc);                                                                   // 434
      },                                                                                                               // 435
      removed: function (oldDoc) {                                                                                     // 437
        _this2.removed("users", oldDoc._id);                                                                           // 438
      }                                                                                                                // 439
    });                                                                                                                // 427
    credentialSubs[credentialId] = observeHandle;                                                                      // 442
  };                                                                                                                   // 443
                                                                                                                       //
  var accountCursor = Meteor.users.find({                                                                              // 445
    _id: accountId                                                                                                     // 445
  });                                                                                                                  // 445
  var accountSubHandle = accountCursor.observe({                                                                       // 446
    added: function (newDoc) {                                                                                         // 447
      var newCredentials = SandstormDb.getUserCredentialIds(newDoc);                                                   // 448
      newCredentials.forEach(function (credentialId) {                                                                 // 449
        refCredential(credentialId);                                                                                   // 450
      });                                                                                                              // 451
                                                                                                                       //
      _this2.added("users", newDoc._id, newDoc);                                                                       // 453
    },                                                                                                                 // 454
    changed: function (newDoc, oldDoc) {                                                                               // 456
      var newCredentials = SandstormDb.getUserCredentialIds(newDoc);                                                   // 457
      var oldCredentials = SandstormDb.getUserCredentialIds(oldDoc); // Those in newDoc - oldDoc, ref.                 // 458
                                                                                                                       //
      var credentialsAdded = _.difference(newCredentials, oldCredentials);                                             // 461
                                                                                                                       //
      credentialsAdded.forEach(function (credentialId) {                                                               // 462
        refCredential(credentialId);                                                                                   // 463
      }); // Those in oldDoc - newDoc, unref.                                                                          // 464
                                                                                                                       //
      var credentialsRemoved = _.difference(oldCredentials, newCredentials);                                           // 467
                                                                                                                       //
      credentialsRemoved.forEach(function (credentialId) {                                                             // 468
        unrefCredential(credentialId);                                                                                 // 469
      });                                                                                                              // 470
      fillUndefinedForChangedDoc(newDoc, oldDoc);                                                                      // 472
                                                                                                                       //
      _this2.changed("users", newDoc._id, newDoc);                                                                     // 474
    },                                                                                                                 // 475
    removed: function (oldDoc) {                                                                                       // 477
      _this2.removed("users", oldDoc._id);                                                                             // 478
                                                                                                                       //
      var oldCredentials = SandstormDb.getUserCredentialIds(oldDoc);                                                   // 479
      oldCredentials.forEach(function (credentialId) {                                                                 // 480
        unrefCredential(credentialId);                                                                                 // 481
      });                                                                                                              // 482
    }                                                                                                                  // 483
  });                                                                                                                  // 446
  this.onStop(function () {                                                                                            // 486
    accountSubHandle.stop(); // Also stop all the credential subscriptions.                                            // 487
                                                                                                                       //
    var subs = _.values(credentialSubs);                                                                               // 489
                                                                                                                       //
    subs.forEach(function (sub) {                                                                                      // 490
      sub.stop();                                                                                                      // 491
    });                                                                                                                // 492
  }); // Meteor's cursor.observe() will synchronously call all of the added() callbacks from the initial               // 493
  // query, so by the time we get here we can report readiness.                                                        // 496
                                                                                                                       //
  this.ready();                                                                                                        // 497
});                                                                                                                    // 498
Meteor.publish("activityStats", function (token) {                                                                     // 500
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 501
  return ActivityStats.find();                                                                                         // 502
});                                                                                                                    // 503
Meteor.publish("statsTokens", function (token) {                                                                       // 505
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 506
  return StatsTokens.find();                                                                                           // 507
});                                                                                                                    // 508
Meteor.publish("allPackages", function (token) {                                                                       // 510
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 511
  return Packages.find({                                                                                               // 512
    manifest: {                                                                                                        // 512
      $exists: true                                                                                                    // 512
    }                                                                                                                  // 512
  }, {                                                                                                                 // 512
    fields: {                                                                                                          // 513
      appId: 1,                                                                                                        // 513
      "manifest.appVersion": 1,                                                                                        // 513
      "manifest.actions": 1,                                                                                           // 514
      "manifest.appTitle": 1,                                                                                          // 514
      progress: 1,                                                                                                     // 514
      status: 1                                                                                                        // 514
    }                                                                                                                  // 513
  });                                                                                                                  // 513
});                                                                                                                    // 515
Meteor.publish("realTimeStats", function (token) {                                                                     // 517
  if (!authorizedAsAdmin(token, this.userId)) return []; // Last five minutes.                                         // 518
                                                                                                                       //
  this.added("realTimeStats", "now", computeStats(new Date(Date.now() - 5 * 60 * 1000))); // Since last sample.        // 521
                                                                                                                       //
  var lastSample = ActivityStats.findOne({}, {                                                                         // 524
    sort: {                                                                                                            // 524
      timestamp: -1                                                                                                    // 524
    }                                                                                                                  // 524
  });                                                                                                                  // 524
  var lastSampleTime = lastSample ? lastSample.timestamp : new Date(0);                                                // 525
  this.added("realTimeStats", "today", computeStats(lastSampleTime)); // TODO(someday): Update every few minutes?      // 526
                                                                                                                       //
  this.ready();                                                                                                        // 530
});                                                                                                                    // 531
Meteor.publish("adminLog", function (token) {                                                                          // 533
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 534
  var logfile = SANDSTORM_LOGDIR + "/sandstorm.log";                                                                   // 536
  var fd = Fs.openSync(logfile, "r");                                                                                  // 538
  var startSize = Fs.fstatSync(fd).size; // Difference between the current file offset and the subscription offset. Can be non-zero when
  // logs have rotated.                                                                                                // 542
                                                                                                                       //
  var extraOffset = 0;                                                                                                 // 543
                                                                                                                       //
  if (startSize < 8192) {                                                                                              // 545
    // Log size is less than window size. Check for rotated log and grab its tail.                                     // 546
    var logfile1 = SANDSTORM_LOGDIR + "/sandstorm.log.1";                                                              // 547
                                                                                                                       //
    if (Fs.existsSync(logfile1)) {                                                                                     // 548
      var fd1 = Fs.openSync(logfile1, "r");                                                                            // 549
      var startSize1 = Fs.fstatSync(fd1).size;                                                                         // 550
      var amountFromLog1 = Math.min(startSize1, 8192 - startSize);                                                     // 551
      var offset1 = startSize1 - amountFromLog1;                                                                       // 552
      var buf = new Buffer(amountFromLog1);                                                                            // 553
      var n = Fs.readSync(fd1, buf, 0, buf.length, offset);                                                            // 554
                                                                                                                       //
      if (n > 0) {                                                                                                     // 555
        this.added("adminLog", 0, {                                                                                    // 556
          text: buf.toString("utf8", 0, n)                                                                             // 556
        });                                                                                                            // 556
        extraOffset += n;                                                                                              // 557
      }                                                                                                                // 558
    }                                                                                                                  // 559
  } // Start tailing at EOF - 8k.                                                                                      // 560
                                                                                                                       //
                                                                                                                       //
  var offset = Math.max(0, startSize - 8192);                                                                          // 563
                                                                                                                       //
  var _this = this;                                                                                                    // 565
                                                                                                                       //
  function doTail() {                                                                                                  // 566
    if (Fs.fstatSync(fd).size < offset) {                                                                              // 567
      extraOffset += offset;                                                                                           // 568
      offset = 0;                                                                                                      // 569
    }                                                                                                                  // 570
                                                                                                                       //
    for (;;) {                                                                                                         // 572
      var _buf = new Buffer(Math.max(1024, startSize - offset));                                                       // 573
                                                                                                                       //
      var _n = Fs.readSync(fd, _buf, 0, _buf.length, offset);                                                          // 574
                                                                                                                       //
      if (_n <= 0) break;                                                                                              // 575
                                                                                                                       //
      _this.added("adminLog", offset + extraOffset, {                                                                  // 576
        text: _buf.toString("utf8", 0, _n)                                                                             // 576
      });                                                                                                              // 576
                                                                                                                       //
      offset += _n;                                                                                                    // 577
    }                                                                                                                  // 578
  } // Watch the file for changes.                                                                                     // 579
                                                                                                                       //
                                                                                                                       //
  var watcher = Fs.watch(logfile, {                                                                                    // 582
    persistent: false                                                                                                  // 582
  }, Meteor.bindEnvironment(doTail)); // When the subscription stops, stop watching the file.                          // 582
                                                                                                                       //
  this.onStop(function () {                                                                                            // 585
    watcher.close();                                                                                                   // 586
    Fs.closeSync(fd);                                                                                                  // 587
  }); // Read initial 8k tail data immediately.                                                                        // 588
                                                                                                                       //
  doTail(); // Notify ready.                                                                                           // 591
                                                                                                                       //
  this.ready();                                                                                                        // 594
});                                                                                                                    // 595
Meteor.publish("adminApiTokens", function (token) {                                                                    // 597
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 598
  return ApiTokens.find({                                                                                              // 599
    $or: [{                                                                                                            // 600
      "frontendRef.ipNetwork": {                                                                                       // 601
        $exists: true                                                                                                  // 601
      }                                                                                                                // 601
    }, {                                                                                                               // 601
      "frontendRef.ipInterface": {                                                                                     // 602
        $exists: true                                                                                                  // 602
      }                                                                                                                // 602
    }]                                                                                                                 // 602
  }, {                                                                                                                 // 599
    fields: {                                                                                                          // 605
      frontendRef: 1,                                                                                                  // 606
      created: 1,                                                                                                      // 607
      requirements: 1,                                                                                                 // 608
      revoked: 1,                                                                                                      // 609
      owner: 1                                                                                                         // 610
    }                                                                                                                  // 605
  });                                                                                                                  // 604
});                                                                                                                    // 613
Meteor.publish("hasAdmin", function (token) {                                                                          // 615
  var _this3 = this;                                                                                                   // 615
                                                                                                                       //
  // Like hasUsers, but for admins, and with token auth required.                                                      // 616
  if (!authorizedAsAdmin(token, this.userId)) return []; // Query if there are any admin users.                        // 617
                                                                                                                       //
  var cursor = Meteor.users.find({                                                                                     // 620
    isAdmin: true                                                                                                      // 620
  });                                                                                                                  // 620
                                                                                                                       //
  if (cursor.count() > 0) {                                                                                            // 621
    this.added("hasAdmin", "hasAdmin", {                                                                               // 622
      hasAdmin: true                                                                                                   // 622
    });                                                                                                                // 622
  } else {                                                                                                             // 623
    var handle = cursor.observeChanges({                                                                               // 624
      added: function (id) {                                                                                           // 625
        _this3.added("hasAdmin", "hasAdmin", {                                                                         // 626
          hasAdmin: true                                                                                               // 626
        });                                                                                                            // 626
                                                                                                                       //
        handle.stop();                                                                                                 // 627
        handle = null;                                                                                                 // 628
      }                                                                                                                // 629
    });                                                                                                                // 624
    this.onStop(function () {                                                                                          // 631
      if (handle) handle.stop();                                                                                       // 632
    });                                                                                                                // 633
  }                                                                                                                    // 634
                                                                                                                       //
  this.ready();                                                                                                        // 636
});                                                                                                                    // 637
Meteor.publish("appIndexAdmin", function (token) {                                                                     // 639
  if (!authorizedAsAdmin(token, this.userId)) return [];                                                               // 640
  return globalDb.collections.appIndex.find();                                                                         // 641
});                                                                                                                    // 642
                                                                                                                       //
function observeOauthService(name) {                                                                                   // 644
  Settings.find({                                                                                                      // 645
    _id: name,                                                                                                         // 645
    value: true                                                                                                        // 645
  }).observe({                                                                                                         // 645
    added: function () {                                                                                               // 646
      // Tell the oauth library it should accept login attempts from this service.                                     // 647
      Accounts.oauth.registerService(name);                                                                            // 648
    },                                                                                                                 // 649
    removed: function () {                                                                                             // 651
      // Tell the oauth library it should deny login attempts from this service.                                       // 652
      Accounts.oauth.unregisterService(name);                                                                          // 653
    }                                                                                                                  // 654
  });                                                                                                                  // 645
}                                                                                                                      // 656
                                                                                                                       //
observeOauthService("github");                                                                                         // 658
observeOauthService("google");                                                                                         // 659
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"backup.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/backup.js                                                                                                    //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var ChildProcess = Npm.require("child_process");                                                                       // 19
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 20
                                                                                                                       //
var GrainInfo = Capnp.importSystem("sandstorm/grain.capnp").GrainInfo;                                                 // 22
var TOKEN_CLEANUP_MINUTES = 120; // Give enough time for large uploads on slow connections.                            // 24
                                                                                                                       //
var TOKEN_CLEANUP_TIMER = TOKEN_CLEANUP_MINUTES * 60 * 1000;                                                           // 25
                                                                                                                       //
function cleanupToken(tokenId) {                                                                                       // 27
  check(tokenId, String);                                                                                              // 28
  FileTokens.remove({                                                                                                  // 29
    _id: tokenId                                                                                                       // 29
  });                                                                                                                  // 29
  waitPromise(globalBackend.cap().deleteBackup(tokenId));                                                              // 30
}                                                                                                                      // 31
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 33
  // Cleanup tokens every TOKEN_CLEANUP_MINUTES                                                                        // 34
  SandstormDb.periodicCleanup(TOKEN_CLEANUP_TIMER, function () {                                                       // 35
    var queryDate = new Date(Date.now() - TOKEN_CLEANUP_TIMER);                                                        // 36
    FileTokens.find({                                                                                                  // 38
      timestamp: {                                                                                                     // 38
        $lt: queryDate                                                                                                 // 38
      }                                                                                                                // 38
    }).forEach(function (token) {                                                                                      // 38
      cleanupToken(token._id);                                                                                         // 39
    });                                                                                                                // 40
  });                                                                                                                  // 41
});                                                                                                                    // 42
Meteor.methods({                                                                                                       // 44
  backupGrain: function (grainId) {                                                                                    // 45
    check(grainId, String);                                                                                            // 46
    var grain = Grains.findOne(grainId);                                                                               // 47
                                                                                                                       //
    if (!grain || !this.userId || grain.userId !== this.userId) {                                                      // 48
      throw new Meteor.Error(403, "Unauthorized", "User is not the owner of this grain");                              // 49
    }                                                                                                                  // 50
                                                                                                                       //
    this.unblock();                                                                                                    // 52
    var token = {                                                                                                      // 54
      _id: Random.id(),                                                                                                // 55
      timestamp: new Date(),                                                                                           // 56
      name: grain.title                                                                                                // 57
    }; // TODO(soon): does the grain need to be offline?                                                               // 54
                                                                                                                       //
    var grainInfo = _.pick(grain, "appId", "appVersion", "title");                                                     // 62
                                                                                                                       //
    grainInfo.ownerIdentityId = grain.identityId;                                                                      // 63
    FileTokens.insert(token);                                                                                          // 65
    waitPromise(globalBackend.cap().backupGrain(token._id, this.userId, grainId, grainInfo));                          // 66
    return token._id;                                                                                                  // 68
  },                                                                                                                   // 69
  newRestoreToken: function () {                                                                                       // 71
    if (!isSignedUpOrDemo()) {                                                                                         // 72
      throw new Meteor.Error(403, "Unauthorized", "Only invited users can restore backups.");                          // 73
    }                                                                                                                  // 74
                                                                                                                       //
    if (isUserOverQuota(Meteor.user())) {                                                                              // 76
      throw new Meteor.Error(402, "You are out of storage space. Please delete some things and try again.");           // 77
    }                                                                                                                  // 79
                                                                                                                       //
    var token = {                                                                                                      // 81
      _id: Random.id(),                                                                                                // 82
      timestamp: new Date()                                                                                            // 83
    };                                                                                                                 // 81
    FileTokens.insert(token);                                                                                          // 86
    return token._id;                                                                                                  // 88
  },                                                                                                                   // 89
  restoreGrain: function (tokenId, obsolete) {                                                                         // 91
    check(tokenId, String);                                                                                            // 92
    var token = FileTokens.findOne(tokenId);                                                                           // 93
                                                                                                                       //
    if (!token || !isSignedUpOrDemo()) {                                                                               // 94
      throw new Meteor.Error(403, "Unauthorized", "Token was not found, or user cannot create grains");                // 95
    }                                                                                                                  // 97
                                                                                                                       //
    if (isUserOverQuota(Meteor.user())) {                                                                              // 99
      throw new Meteor.Error(402, "You are out of storage space. Please delete some things and try again.");           // 100
    }                                                                                                                  // 102
                                                                                                                       //
    this.unblock();                                                                                                    // 104
    var grainId = Random.id(22);                                                                                       // 106
                                                                                                                       //
    try {                                                                                                              // 108
      var grainInfo = waitPromise(globalBackend.cap().restoreGrain(tokenId, this.userId, grainId).catch(function (err) {
        console.error("Unzip failure:", err.message);                                                                  // 111
        throw new Meteor.Error(500, "Invalid backup file.");                                                           // 112
      })).info;                                                                                                        // 113
                                                                                                                       //
      if (!grainInfo.appId) {                                                                                          // 114
        globalBackend.deleteGrain(grainId, this.userId);                                                               // 115
        throw new Meteor.Error(500, "Metadata object for uploaded grain has no AppId");                                // 116
      }                                                                                                                // 117
                                                                                                                       //
      var action = UserActions.findOne({                                                                               // 119
        appId: grainInfo.appId,                                                                                        // 119
        userId: this.userId                                                                                            // 119
      }); // Create variables we'll use for later Mongo query.                                                         // 119
                                                                                                                       //
      var packageId = void 0;                                                                                          // 122
      var appVersion = void 0; // DevPackages are system-wide, so we do not check the user ID.                         // 123
                                                                                                                       //
      var devPackage = DevPackages.findOne({                                                                           // 126
        appId: grainInfo.appId                                                                                         // 126
      });                                                                                                              // 126
                                                                                                                       //
      if (devPackage) {                                                                                                // 127
        // If the dev app package exists, it should override the user action.                                          // 128
        packageId = devPackage.packageId;                                                                              // 129
        appVersion = devPackage.manifest.appVersion;                                                                   // 130
      } else if (action) {                                                                                             // 131
        // The app is installed, so we can continue restoring this                                                     // 132
        // grain.                                                                                                      // 133
        packageId = action.packageId;                                                                                  // 134
        appVersion = action.appVersion;                                                                                // 135
      } else {                                                                                                         // 136
        // If the package isn't installed at all, bail out.                                                            // 137
        globalBackend.deleteGrain(grainId, this.userId);                                                               // 138
        throw new Meteor.Error(500, "App id for uploaded grain not installed", "App Id: " + grainInfo.appId);          // 139
      }                                                                                                                // 142
                                                                                                                       //
      if (appVersion < grainInfo.appVersion) {                                                                         // 144
        globalBackend.deleteGrain(grainId, this.userId);                                                               // 145
        throw new Meteor.Error(500, "App version for uploaded grain is newer than any " + "installed version. You need to upgrade your app first", "New version: " + grainInfo.appVersion + ", Old version: " + appVersion);
      }                                                                                                                // 151
                                                                                                                       //
      console.log(grainInfo);                                                                                          // 153
      Grains.insert({                                                                                                  // 155
        _id: grainId,                                                                                                  // 156
        packageId: packageId,                                                                                          // 157
        appId: grainInfo.appId,                                                                                        // 158
        appVersion: appVersion,                                                                                        // 159
        userId: this.userId,                                                                                           // 160
        // For older backups that don't have the owner's identity ID, use the owner's identicon                        // 161
        // ID which is based on old-style global identity IDs.                                                         // 162
        identityId: grainInfo.ownerIdentityId || (Meteor.user().profile || {}).identicon || SandstormDb.generateIdentityId(),
        title: grainInfo.title,                                                                                        // 166
        "private": true,                                                                                               // 167
        size: 0                                                                                                        // 168
      });                                                                                                              // 155
    } finally {                                                                                                        // 170
      cleanupToken(tokenId);                                                                                           // 171
    }                                                                                                                  // 172
                                                                                                                       //
    return grainId;                                                                                                    // 174
  }                                                                                                                    // 175
});                                                                                                                    // 44
Router.map(function () {                                                                                               // 178
  this.route("downloadBackup", {                                                                                       // 179
    where: "server",                                                                                                   // 180
    path: "/downloadBackup/:tokenId",                                                                                  // 181
    action: function () {                                                                                              // 182
      var token = FileTokens.findOne(this.params.tokenId);                                                             // 183
      var response = this.response;                                                                                    // 184
                                                                                                                       //
      if (!token) {                                                                                                    // 185
        response.writeHead(404, {                                                                                      // 186
          "Content-Type": "text/plain"                                                                                 // 186
        });                                                                                                            // 186
        return response.end("File does not exist");                                                                    // 187
      }                                                                                                                // 188
                                                                                                                       //
      var started = false;                                                                                             // 190
      var encodedFilename = encodeURIComponent(token.name || "backup") + ".zip";                                       // 191
      var sawEnd = false;                                                                                              // 192
      var stream = {                                                                                                   // 194
        expectSize: function (size) {                                                                                  // 195
          if (!started) {                                                                                              // 196
            started = true;                                                                                            // 197
            response.writeHead(200, {                                                                                  // 198
              "Content-Length": size,                                                                                  // 199
              "Content-Type": "application/zip",                                                                       // 200
              "Content-Disposition": "attachment;filename*=utf-8''" + encodedFilename                                  // 201
            });                                                                                                        // 198
          }                                                                                                            // 203
        },                                                                                                             // 204
        write: function (data) {                                                                                       // 206
          if (!started) {                                                                                              // 207
            started = true;                                                                                            // 208
            response.writeHead(200, {                                                                                  // 209
              "Content-Type": "application/zip",                                                                       // 210
              "Content-Disposition": "attachment;filename*=utf-8''" + encodedFilename                                  // 211
            });                                                                                                        // 209
          }                                                                                                            // 213
                                                                                                                       //
          response.write(data);                                                                                        // 215
        },                                                                                                             // 216
        done: function (data) {                                                                                        // 218
          if (!started) {                                                                                              // 219
            started = true;                                                                                            // 220
            response.writeHead(200, {                                                                                  // 221
              "Content-Length": 0,                                                                                     // 222
              "Content-Type": "application/zip",                                                                       // 223
              "Content-Disposition": "attachment;filename*=utf-8''" + encodedFilename                                  // 224
            });                                                                                                        // 221
          }                                                                                                            // 226
                                                                                                                       //
          sawEnd = true;                                                                                               // 228
          response.end();                                                                                              // 229
        }                                                                                                              // 230
      };                                                                                                               // 194
      waitPromise(globalBackend.cap().downloadBackup(this.params.tokenId, stream));                                    // 233
                                                                                                                       //
      if (!sawEnd) {                                                                                                   // 235
        console.error("backend failed to call done() when downloading backup");                                        // 236
                                                                                                                       //
        if (!started) {                                                                                                // 237
          throw new Meteor.Error(500, "backend failed to produce data");                                               // 238
        }                                                                                                              // 239
                                                                                                                       //
        response.end();                                                                                                // 241
      }                                                                                                                // 242
                                                                                                                       //
      cleanupToken(this.params.tokenId);                                                                               // 244
    }                                                                                                                  // 245
  });                                                                                                                  // 179
  this.route("uploadBackup", {                                                                                         // 248
    where: "server",                                                                                                   // 249
    path: "/uploadBackup/:token",                                                                                      // 250
    action: function () {                                                                                              // 251
      if (this.request.method === "POST") {                                                                            // 252
        var token = FileTokens.findOne(this.params.token);                                                             // 253
                                                                                                                       //
        if (!this.params.token || !token) {                                                                            // 254
          this.response.writeHead(403, {                                                                               // 255
            "Content-Type": "text/plain",                                                                              // 256
            "Access-Control-Allow-Origin": "*"                                                                         // 257
          });                                                                                                          // 255
          this.response.write("Invalid upload token.");                                                                // 259
          this.response.end();                                                                                         // 260
          return;                                                                                                      // 261
        }                                                                                                              // 262
                                                                                                                       //
        var request = this.request;                                                                                    // 264
                                                                                                                       //
        try {                                                                                                          // 265
          var stream = globalBackend.cap().uploadBackup(this.params.token).stream;                                     // 266
          waitPromise(new Promise(function (resolve, reject) {                                                         // 268
            request.on("data", function (data) {                                                                       // 269
              stream.write(data);                                                                                      // 270
            });                                                                                                        // 271
            request.on("end", function () {                                                                            // 272
              resolve(stream.done());                                                                                  // 273
            });                                                                                                        // 274
            request.on("error", function (err) {                                                                       // 275
              stream.close();                                                                                          // 276
            });                                                                                                        // 277
          }));                                                                                                         // 278
          this.response.writeHead(204, {                                                                               // 280
            "Access-Control-Allow-Origin": "*"                                                                         // 281
          });                                                                                                          // 280
          this.response.end();                                                                                         // 283
        } catch (error) {                                                                                              // 284
          console.error(error.stack);                                                                                  // 285
          this.response.writeHead(500, {                                                                               // 286
            "Content-Type": "text/plain",                                                                              // 287
            "Access-Control-Allow-Origin": "*"                                                                         // 288
          });                                                                                                          // 286
          this.response.write(error.stack);                                                                            // 290
          this.response.end();                                                                                         // 291
        }                                                                                                              // 292
      } else if (this.request.method == "OPTIONS") {                                                                   // 293
        // Allow cross-origin posts to uploadBackup so that uploads can occur on the DDP host                          // 294
        // rather than the main host. In theory we could have Access-Control-Allow-Origin specify                      // 295
        // the main host rather than "*", but an uploadBackup request already requires a valid                         // 296
        // upload token, which is plenty of access control in itself.                                                  // 297
        var requestedHeaders = this.request.headers["access-control-request-headers"];                                 // 298
                                                                                                                       //
        if (requestedHeaders) {                                                                                        // 299
          this.response.setHeader("Access-Control-Allow-Headers", requestedHeaders);                                   // 300
        }                                                                                                              // 301
                                                                                                                       //
        this.response.writeHead(204, {                                                                                 // 303
          "Access-Control-Allow-Origin": "*",                                                                          // 304
          "Access-Control-Allow-Methods": "POST, OPTIONS",                                                             // 305
          "Access-Control-Max-Age": "3600"                                                                             // 306
        });                                                                                                            // 303
        this.response.end();                                                                                           // 308
      } else {                                                                                                         // 309
        this.response.writeHead(405, {                                                                                 // 310
          "Content-Type": "text/plain"                                                                                 // 311
        });                                                                                                            // 310
        this.response.write("You can only POST here.");                                                                // 313
        this.response.end();                                                                                           // 314
      }                                                                                                                // 315
    }                                                                                                                  // 316
  });                                                                                                                  // 248
});                                                                                                                    // 318
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"contacts-server.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/contacts-server.js                                                                                           //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
Meteor.publish("contactProfiles", function (showAll) {                                                                 // 17
  var db = this.connection.sandstormDb;                                                                                // 18
                                                                                                                       //
  var _this = this;                                                                                                    // 19
                                                                                                                       //
  var userId = this.userId; // We maintain a map from account IDs to live query handles that track profile changes.    // 20
                                                                                                                       //
  var contactAccounts = {};                                                                                            // 23
  var disallowGuests = db.getOrganizationDisallowGuests();                                                             // 24
                                                                                                                       //
  function addAccountOfContact(accountId) {                                                                            // 26
    if (!(accountId in contactAccounts)) {                                                                             // 27
      var user = Meteor.users.findOne({                                                                                // 28
        _id: accountId                                                                                                 // 28
      });                                                                                                              // 28
                                                                                                                       //
      if (disallowGuests && !showAll) {                                                                                // 30
        if (!db.isUserInOrganization(user)) {                                                                          // 31
          return;                                                                                                      // 32
        }                                                                                                              // 33
      }                                                                                                                // 34
                                                                                                                       //
      if (user) {                                                                                                      // 36
        var filteredUser = _.pick(user, "_id", "profile");                                                             // 37
                                                                                                                       //
        filteredUser.intrinsicNames = db.getAccountIntrinsicNames(user, false);                                        // 38
                                                                                                                       //
        _this.added("contactProfiles", user._id, filteredUser);                                                        // 39
      }                                                                                                                // 40
                                                                                                                       //
      contactAccounts[accountId] = Meteor.users.find({                                                                 // 42
        _id: accountId                                                                                                 // 43
      }, {                                                                                                             // 43
        fields: {                                                                                                      // 43
          profile: 1                                                                                                   // 43
        }                                                                                                              // 43
      }).observeChanges({                                                                                              // 43
        changed: function (id, fields) {                                                                               // 44
          _this.changed("contactProfiles", id, fields);                                                                // 45
        }                                                                                                              // 46
      });                                                                                                              // 43
    }                                                                                                                  // 48
  }                                                                                                                    // 49
                                                                                                                       //
  var cursor = db.collections.contacts.find({                                                                          // 51
    ownerId: userId                                                                                                    // 51
  });                                                                                                                  // 51
  var handle = cursor.observe({                                                                                        // 53
    added: function (contact) {                                                                                        // 54
      addAccountOfContact(contact.accountId);                                                                          // 55
    },                                                                                                                 // 56
    changed: function (contact) {                                                                                      // 58
      addAccountOfContact(contact.accountId);                                                                          // 59
    },                                                                                                                 // 60
    removed: function (contact) {                                                                                      // 62
      _this.removed("contactProfiles", contact.accountId);                                                             // 63
                                                                                                                       //
      var contactAccount = contactAccounts[contact.accountId];                                                         // 64
      if (contactAccount) contactAccounts[contact.accountId].stop();                                                   // 65
      delete contactAccounts[contact.accountId];                                                                       // 66
    }                                                                                                                  // 67
  });                                                                                                                  // 53
  var orgHandle = void 0;                                                                                              // 70
                                                                                                                       //
  if (db.getOrganizationShareContacts() && db.isUserInOrganization(db.collections.users.findOne({                      // 72
    _id: userId                                                                                                        // 73
  }))) {                                                                                                               // 73
    var orgCursor = db.collections.users.find({                                                                        // 74
      type: "account"                                                                                                  // 74
    }); // TODO(perf): make a mongo query that can find all accounts in an organization and add                        // 74
    // indices for it. Currently, we do some case insensitive matching which mongo can't                               // 76
    // handle well.                                                                                                    // 77
                                                                                                                       //
    orgHandle = orgCursor.observe({                                                                                    // 79
      added: function (user) {                                                                                         // 80
        if (db.isUserInOrganization(user) && user._id !== userId) {                                                    // 81
          addAccountOfContact(user._id);                                                                               // 82
        }                                                                                                              // 83
      },                                                                                                               // 84
      changed: function (user) {                                                                                       // 86
        if (db.isUserInOrganization(user) && user._id !== userId) {                                                    // 87
          addAccountOfContact(user._id);                                                                               // 88
        }                                                                                                              // 89
      },                                                                                                               // 90
      removed: function (user) {                                                                                       // 92
        if (db.isUserInOrganization(user) && user._id !== userId) {                                                    // 93
          _this.removed("contactProfiles", user._id);                                                                  // 94
                                                                                                                       //
          var contactAccount = contactAccounts[contact.accountId];                                                     // 95
          if (contactAccount) contactAccounts[contact.accountId].stop();                                               // 96
          delete contactAccounts[user._id];                                                                            // 97
        }                                                                                                              // 98
      }                                                                                                                // 99
    });                                                                                                                // 79
  }                                                                                                                    // 101
                                                                                                                       //
  this.ready();                                                                                                        // 103
  this.onStop(function () {                                                                                            // 105
    handle.stop();                                                                                                     // 106
                                                                                                                       //
    if (orgHandle) {                                                                                                   // 107
      orgHandle.stop();                                                                                                // 108
    }                                                                                                                  // 109
                                                                                                                       //
    Object.keys(contactAccounts).forEach(function (accountId) {                                                        // 111
      contactAccounts[accountId].stop();                                                                               // 112
      delete contactAccounts[accountId];                                                                               // 113
    });                                                                                                                // 114
  });                                                                                                                  // 115
});                                                                                                                    // 116
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"core.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/core.js                                                                                                      //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");                          //
                                                                                                                       //
var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);                                 //
                                                                                                                       //
var _inherits2 = require("babel-runtime/helpers/inherits");                                                            //
                                                                                                                       //
var _inherits3 = _interopRequireDefault(_inherits2);                                                                   //
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var inMeteor = void 0,                                                                                                 // 1
    waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  },                                                                                                                   // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var StaticAssetImpl = void 0,                                                                                          // 1
    IdenticonStaticAssetImpl = void 0;                                                                                 // 1
module.watch(require("/imports/server/static-asset.js"), {                                                             // 1
  StaticAssetImpl: function (v) {                                                                                      // 1
    StaticAssetImpl = v;                                                                                               // 1
  },                                                                                                                   // 1
  IdenticonStaticAssetImpl: function (v) {                                                                             // 1
    IdenticonStaticAssetImpl = v;                                                                                      // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var PersistentImpl = void 0,                                                                                           // 1
    hashSturdyRef = void 0,                                                                                            // 1
    generateSturdyRef = void 0,                                                                                        // 1
    checkRequirements = void 0,                                                                                        // 1
    fetchApiToken = void 0,                                                                                            // 1
    insertApiToken = void 0;                                                                                           // 1
module.watch(require("/imports/server/persistent.js"), {                                                               // 1
  PersistentImpl: function (v) {                                                                                       // 1
    PersistentImpl = v;                                                                                                // 1
  },                                                                                                                   // 1
  hashSturdyRef: function (v) {                                                                                        // 1
    hashSturdyRef = v;                                                                                                 // 1
  },                                                                                                                   // 1
  generateSturdyRef: function (v) {                                                                                    // 1
    generateSturdyRef = v;                                                                                             // 1
  },                                                                                                                   // 1
  checkRequirements: function (v) {                                                                                    // 1
    checkRequirements = v;                                                                                             // 1
  },                                                                                                                   // 1
  fetchApiToken: function (v) {                                                                                        // 1
    fetchApiToken = v;                                                                                                 // 1
  },                                                                                                                   // 1
  insertApiToken: function (v) {                                                                                       // 1
    insertApiToken = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 19
                                                                                                                       //
var PersistentHandle = Capnp.importSystem("sandstorm/supervisor.capnp").PersistentHandle;                              // 23
var SandstormCore = Capnp.importSystem("sandstorm/supervisor.capnp").SandstormCore;                                    // 24
var SandstormCoreFactory = Capnp.importSystem("sandstorm/backend.capnp").SandstormCoreFactory;                         // 25
var PersistentOngoingNotification = Capnp.importSystem("sandstorm/supervisor.capnp").PersistentOngoingNotification;    // 26
var PersistentUiView = Capnp.importSystem("sandstorm/persistentuiview.capnp").PersistentUiView;                        // 27
var StaticAsset = Capnp.importSystem("sandstorm/util.capnp").StaticAsset;                                              // 28
var SystemPersistent = Capnp.importSystem("sandstorm/supervisor.capnp").SystemPersistent;                              // 29
                                                                                                                       //
var SandstormCoreImpl = function () {                                                                                  //
  function SandstormCoreImpl(db, grainId) {                                                                            // 32
    (0, _classCallCheck3.default)(this, SandstormCoreImpl);                                                            // 32
    this.db = db;                                                                                                      // 33
    this.grainId = grainId;                                                                                            // 34
  }                                                                                                                    // 35
                                                                                                                       //
  SandstormCoreImpl.prototype.restore = function () {                                                                  //
    function restore(sturdyRef) {                                                                                      //
      var _this = this;                                                                                                // 37
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 38
        sturdyRef = sturdyRef.toString("utf8");                                                                        // 39
        var token = fetchApiToken(_this.db, sturdyRef, {                                                               // 40
          "owner.grain.grainId": _this.grainId                                                                         // 41
        });                                                                                                            // 41
                                                                                                                       //
        if (!token) {                                                                                                  // 42
          throw new Error("no such token");                                                                            // 43
        }                                                                                                              // 44
                                                                                                                       //
        return restoreInternal(_this.db, sturdyRef, {                                                                  // 46
          grain: Match.ObjectIncluding({                                                                               // 47
            grainId: _this.grainId                                                                                     // 47
          })                                                                                                           // 47
        }, [], token);                                                                                                 // 47
      });                                                                                                              // 49
    }                                                                                                                  // 50
                                                                                                                       //
    return restore;                                                                                                    //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.drop = function () {                                                                     //
    function drop(sturdyRef) {                                                                                         //
      var _this2 = this;                                                                                               // 52
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 53
        sturdyRef = sturdyRef.toString("utf8");                                                                        // 54
        return dropInternal(_this2.db, sturdyRef, {                                                                    // 55
          grain: Match.ObjectIncluding({                                                                               // 55
            grainId: _this2.grainId                                                                                    // 55
          })                                                                                                           // 55
        });                                                                                                            // 55
      });                                                                                                              // 56
    }                                                                                                                  // 57
                                                                                                                       //
    return drop;                                                                                                       //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.makeToken = function () {                                                                //
    function makeToken(ref, owner, requirements) {                                                                     //
      var _this3 = this;                                                                                               // 59
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 60
        var sturdyRef = insertApiToken(_this3.db, {                                                                    // 61
          grainId: _this3.grainId,                                                                                     // 62
          objectId: ref,                                                                                               // 63
          owner: owner,                                                                                                // 64
          created: new Date(),                                                                                         // 65
          requirements: requirements                                                                                   // 66
        });                                                                                                            // 61
        return {                                                                                                       // 69
          token: new Buffer(sturdyRef)                                                                                 // 70
        };                                                                                                             // 69
      });                                                                                                              // 72
    }                                                                                                                  // 73
                                                                                                                       //
    return makeToken;                                                                                                  //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.makeChildToken = function () {                                                           //
    function makeChildToken(parent, owner, requirements) {                                                             //
      var _this4 = this;                                                                                               // 75
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 76
        // Compute the save ApiToken template.                                                                         // 77
        return makeSaveTemplateForChild(_this4.db, parent.toString(), requirements);                                   // 78
      }).then(function (saveTemplate) {                                                                                // 79
        // Create a dummy PersistentImpl and invoke its own save() method.                                             // 80
        return new PersistentImpl(_this4.db, saveTemplate).save({                                                      // 81
          sealFor: owner                                                                                               // 81
        });                                                                                                            // 81
      }).then(function (saveResult) {                                                                                  // 82
        // Transform to expected result structure.                                                                     // 83
        return {                                                                                                       // 84
          token: saveResult.sturdyRef                                                                                  // 84
        };                                                                                                             // 84
      });                                                                                                              // 85
    }                                                                                                                  // 86
                                                                                                                       //
    return makeChildToken;                                                                                             //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.getOwnerNotificationTarget = function () {                                               //
    function getOwnerNotificationTarget() {                                                                            //
      var _this5 = this;                                                                                               // 88
                                                                                                                       //
      var grainId = this.grainId;                                                                                      // 89
      return {                                                                                                         // 90
        owner: {                                                                                                       // 91
          addOngoing: function (displayInfo, notification) {                                                           // 92
            return inMeteor(function () {                                                                              // 93
              var grain = _this5.db.collections.grains.findOne({                                                       // 94
                _id: grainId                                                                                           // 94
              });                                                                                                      // 94
                                                                                                                       //
              if (!grain) {                                                                                            // 95
                throw new Error("Grain not found.");                                                                   // 96
              }                                                                                                        // 97
                                                                                                                       //
              var castedNotification = notification.castAs(PersistentOngoingNotification);                             // 99
              var wakelockToken = waitPromise(castedNotification.save()).sturdyRef.toString("utf8"); // We have to close both the casted cap and the original. Perhaps this should be fixed in
              // node-capnp?                                                                                           // 103
                                                                                                                       //
              castedNotification.close();                                                                              // 104
              notification.close();                                                                                    // 105
                                                                                                                       //
              var notificationId = _this5.db.collections.notifications.insert({                                        // 106
                ongoing: wakelockToken,                                                                                // 107
                grainId: grainId,                                                                                      // 108
                userId: grain.userId,                                                                                  // 109
                text: displayInfo.caption,                                                                             // 110
                timestamp: new Date(),                                                                                 // 111
                isUnread: true                                                                                         // 112
              });                                                                                                      // 106
                                                                                                                       //
              return {                                                                                                 // 115
                handle: globalFrontendRefRegistry.create(_this5.db, {                                                  // 116
                  notificationHandle: notificationId                                                                   // 117
                })                                                                                                     // 117
              };                                                                                                       // 115
            });                                                                                                        // 119
          }                                                                                                            // 120
        }                                                                                                              // 91
      };                                                                                                               // 90
    }                                                                                                                  // 123
                                                                                                                       //
    return getOwnerNotificationTarget;                                                                                 //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.backgroundActivity = function () {                                                       //
    function backgroundActivity(event) {                                                                               //
      var _this6 = this;                                                                                               // 125
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 126
        logActivity(_this6.grainId, null, event);                                                                      // 127
      });                                                                                                              // 128
    }                                                                                                                  // 129
                                                                                                                       //
    return backgroundActivity;                                                                                         //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.reportGrainSize = function () {                                                          //
    function reportGrainSize(bytes) {                                                                                  //
      bytes = parseInt(bytes); // int64s are stringified but precision isn't critical here                             // 132
                                                                                                                       //
      var result = this.db.collections.grains.findAndModify({                                                          // 134
        query: {                                                                                                       // 135
          _id: this.grainId                                                                                            // 135
        },                                                                                                             // 135
        update: {                                                                                                      // 136
          $set: {                                                                                                      // 136
            size: bytes                                                                                                // 136
          }                                                                                                            // 136
        },                                                                                                             // 136
        fields: {                                                                                                      // 137
          _id: 1,                                                                                                      // 137
          userId: 1,                                                                                                   // 137
          size: 1                                                                                                      // 137
        }                                                                                                              // 137
      });                                                                                                              // 134
                                                                                                                       //
      if (!result.ok) {                                                                                                // 140
        throw new Error("Grain not found.");                                                                           // 141
      } // If the grain did not have a "size" field before the update, then it is a grain created                      // 142
      // before per-grain size tracking was implemented. In that case, we don't want to update the                     // 145
      // user record because it may already be counting the grain (specifically on Blackrock, where                    // 146
      // whole-user size counting has existed for some time).                                                          // 147
                                                                                                                       //
                                                                                                                       //
      if (this.db.isQuotaEnabled() && "size" in result.value) {                                                        // 148
        // Update the user record, too. Note that we periodically recompute the user's storage usage                   // 149
        // from scratch as well, so this doesn't have to be perfectly reliable.                                        // 150
        var diff = bytes - (result.value.size || 0);                                                                   // 151
        this.db.collections.users.update(result.value.userId, {                                                        // 152
          $inc: {                                                                                                      // 152
            storageUsage: diff                                                                                         // 152
          }                                                                                                            // 152
        });                                                                                                            // 152
      }                                                                                                                // 153
    }                                                                                                                  // 154
                                                                                                                       //
    return reportGrainSize;                                                                                            //
  }();                                                                                                                 //
                                                                                                                       //
  SandstormCoreImpl.prototype.getIdentityId = function () {                                                            //
    function getIdentityId(identity) {                                                                                 //
      var _this7 = this;                                                                                               // 156
                                                                                                                       //
      return unwrapFrontendCap(identity, "identity", function (accountId) {                                            // 157
        var grain = _this7.db.getGrain(_this7.grainId);                                                                // 158
                                                                                                                       //
        if (!grain) {                                                                                                  // 159
          throw new Error("Grain not found.");                                                                         // 160
        }                                                                                                              // 161
                                                                                                                       //
        var identityId = _this7.db.getOrGenerateIdentityId(accountId, grain);                                          // 162
                                                                                                                       //
        return {                                                                                                       // 163
          id: new Buffer(identityId, "hex")                                                                            // 163
        };                                                                                                             // 163
      });                                                                                                              // 164
    }                                                                                                                  // 165
                                                                                                                       //
    return getIdentityId;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  return SandstormCoreImpl;                                                                                            //
}();                                                                                                                   //
                                                                                                                       //
var makeSandstormCore = function (db, grainId) {                                                                       // 168
  return new Capnp.Capability(new SandstormCoreImpl(db, grainId), SandstormCore);                                      // 169
};                                                                                                                     // 170
                                                                                                                       //
var NotificationHandle = function (_PersistentImpl) {                                                                  //
  (0, _inherits3.default)(NotificationHandle, _PersistentImpl);                                                        //
                                                                                                                       //
  // TODO(cleanup): Move to a different file.                                                                          // 173
  function NotificationHandle(db, saveTemplate, notificationId) {                                                      // 175
    (0, _classCallCheck3.default)(this, NotificationHandle);                                                           // 175
                                                                                                                       //
    var _this8 = (0, _possibleConstructorReturn3.default)(this, _PersistentImpl.call(this, db, saveTemplate));         // 175
                                                                                                                       //
    _this8.notificationId = notificationId;                                                                            // 177
    _this8.db = db;                                                                                                    // 178
    return _this8;                                                                                                     // 175
  }                                                                                                                    // 179
                                                                                                                       //
  NotificationHandle.prototype.close = function () {                                                                   //
    function close() {                                                                                                 //
      var _this9 = this;                                                                                               // 181
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 182
        if (!_this9.isSaved()) {                                                                                       // 183
          dismissNotification(_this9.db, _this9.notificationId);                                                       // 184
        }                                                                                                              // 185
      });                                                                                                              // 186
    }                                                                                                                  // 187
                                                                                                                       //
    return close;                                                                                                      //
  }();                                                                                                                 //
                                                                                                                       //
  return NotificationHandle;                                                                                           //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
var PersistentUiViewImpl = function (_PersistentImpl2) {                                                               //
  (0, _inherits3.default)(PersistentUiViewImpl, _PersistentImpl2);                                                     //
                                                                                                                       //
  // TODO(cleanup): Move out of core.js.                                                                               // 191
  function PersistentUiViewImpl(db, saveTemplate, grainId) {                                                           // 193
    (0, _classCallCheck3.default)(this, PersistentUiViewImpl);                                                         // 193
                                                                                                                       //
    var _this10 = (0, _possibleConstructorReturn3.default)(this, _PersistentImpl2.call(this, db, saveTemplate));       // 193
                                                                                                                       //
    check(grainId, String);                                                                                            // 195
    _this10._db = db;                                                                                                  // 196
    _this10._grainId = grainId;                                                                                        // 197
    return _this10;                                                                                                    // 193
  }                                                                                                                    // 198
                                                                                                                       //
  PersistentUiViewImpl.prototype.getViewInfo = function () {                                                           //
    function getViewInfo() {                                                                                           //
      var _this11 = this;                                                                                              // 200
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 201
        var grain = _this11._db.getGrain(_this11._grainId);                                                            // 202
                                                                                                                       //
        if (!grain || grain.trashed) {                                                                                 // 203
          throw new Error("grain no longer exists");                                                                   // 204
        }                                                                                                              // 205
                                                                                                                       //
        var pkg = _this11._db.getPackage(grain.packageId) || DevPackages.findOne({                                     // 207
          appId: grain.appId                                                                                           // 208
        }) || {};                                                                                                      // 208
        var manifest = pkg.manifest || {};                                                                             // 211
        var viewInfo = grain.cachedViewInfo || {};                                                                     // 213
                                                                                                                       //
        if (!viewInfo.appTitle) {                                                                                      // 215
          viewInfo.appTitle = manifest.appTitle || {};                                                                 // 216
        }                                                                                                              // 217
                                                                                                                       //
        if (!viewInfo.grainIcon) {                                                                                     // 219
          var grainIcon = ((manifest.metadata || {}).icons || {}).grain;                                               // 220
                                                                                                                       //
          if (grainIcon) {                                                                                             // 221
            viewInfo.grainIcon = new Capnp.Capability(new StaticAssetImpl(grainIcon.assetId), StaticAsset);            // 222
          } else {                                                                                                     // 224
            var hash = Identicon.hashAppIdForIdenticon(pkg.appId);                                                     // 225
            viewInfo.grainIcon = new Capnp.Capability(new IdenticonStaticAssetImpl(hash, 24), StaticAsset);            // 226
          }                                                                                                            // 228
        }                                                                                                              // 229
                                                                                                                       //
        return viewInfo;                                                                                               // 231
      });                                                                                                              // 232
    }                                                                                                                  // 233
                                                                                                                       //
    return getViewInfo;                                                                                                //
  }(); // All other UiView methods are currently unimplemented, which, while not strictly correct,                     //
  // results in the same overall behavior, since users can't call restore() on a PersistentUiView,                     // 236
  // and grains can't call methods on UiViews because they lack the "is human" pseudopermission.                       // 237
                                                                                                                       //
                                                                                                                       //
  return PersistentUiViewImpl;                                                                                         //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
var makePersistentUiView = function (db, saveTemplate, grainId) {                                                      // 240
  check(grainId, String); // Verify that the grain exists and hasn't been trashed.                                     // 241
                                                                                                                       //
  var grain = db.getGrain(grainId);                                                                                    // 244
                                                                                                                       //
  if (!grain || grain.trashed) {                                                                                       // 245
    throw new Meteor.Error(404, "grain not found");                                                                    // 246
  }                                                                                                                    // 247
                                                                                                                       //
  return new Capnp.Capability(new PersistentUiViewImpl(db, saveTemplate, grainId), PersistentUiView);                  // 249
};                                                                                                                     // 251
                                                                                                                       //
globalFrontendRefRegistry.register({                                                                                   // 253
  frontendRefField: "notificationHandle",                                                                              // 254
  restore: function (db, saveTemplate, notificationId) {                                                               // 256
    return new Capnp.Capability(new NotificationHandle(db, saveTemplate, notificationId), PersistentHandle);           // 257
  }                                                                                                                    // 259
});                                                                                                                    // 253
                                                                                                                       //
function dismissNotification(db, notificationId, callCancel) {                                                         // 262
  var notification = db.collections.notifications.findOne({                                                            // 263
    _id: notificationId                                                                                                // 263
  });                                                                                                                  // 263
                                                                                                                       //
  if (notification) {                                                                                                  // 264
    db.collections.notifications.remove({                                                                              // 265
      _id: notificationId                                                                                              // 265
    });                                                                                                                // 265
                                                                                                                       //
    if (notification.ongoing) {                                                                                        // 266
      var id = notification.ongoing;                                                                                   // 267
                                                                                                                       //
      if (!callCancel) {                                                                                               // 269
        dropInternal(db, id, {                                                                                         // 270
          frontend: null                                                                                               // 270
        });                                                                                                            // 270
      } else {                                                                                                         // 271
        var notificationCap = restoreInternal(db, id, {                                                                // 272
          frontend: null                                                                                               // 272
        }, []).cap;                                                                                                    // 272
        var castedNotification = notificationCap.castAs(PersistentOngoingNotification);                                // 273
        dropInternal(db, id, {                                                                                         // 274
          frontend: null                                                                                               // 274
        });                                                                                                            // 274
                                                                                                                       //
        try {                                                                                                          // 275
          waitPromise(castedNotification.cancel());                                                                    // 276
          castedNotification.close();                                                                                  // 277
          notificationCap.close();                                                                                     // 278
        } catch (err) {                                                                                                // 279
          if (err.kjType !== "disconnected") {                                                                         // 280
            // ignore disconnected errors, since cancel may shutdown the grain before the supervisor                   // 281
            // responds.                                                                                               // 282
            throw err;                                                                                                 // 283
          }                                                                                                            // 284
        }                                                                                                              // 285
      }                                                                                                                // 286
    } else if (notification.appUpdates) {                                                                              // 287
      _.forEach(notification.appUpdates, function (app, appId) {                                                       // 288
        db.deleteUnusedPackages(appId);                                                                                // 289
      });                                                                                                              // 290
    }                                                                                                                  // 291
  }                                                                                                                    // 292
}                                                                                                                      // 293
                                                                                                                       //
Meteor.methods({                                                                                                       // 295
  dismissNotification: function (notificationId) {                                                                     // 296
    // This will remove notifications from the database and from view of the user.                                     // 297
    // For ongoing notifications, it will begin the process of cancelling and dropping them from                       // 298
    // the app.                                                                                                        // 299
    check(notificationId, String);                                                                                     // 301
    var db = this.connection.sandstormDb;                                                                              // 303
    var notification = db.collections.notifications.findOne({                                                          // 305
      _id: notificationId                                                                                              // 305
    });                                                                                                                // 305
                                                                                                                       //
    if (!notification) {                                                                                               // 306
      throw new Meteor.Error(404, "Notification id not found.");                                                       // 307
    } else if (notification.userId !== Meteor.userId()) {                                                              // 308
      throw new Meteor.Error(403, "Notification does not belong to current user.");                                    // 309
    } else {                                                                                                           // 310
      dismissNotification(db, notificationId, true);                                                                   // 311
    }                                                                                                                  // 312
  },                                                                                                                   // 313
  readAllNotifications: function () {                                                                                  // 315
    // Marks all notifications as read for the current user.                                                           // 316
    if (!Meteor.userId()) {                                                                                            // 317
      throw new Meteor.Error(403, "User not logged in.");                                                              // 318
    }                                                                                                                  // 319
                                                                                                                       //
    var db = this.connection.sandstormDb;                                                                              // 321
    db.collections.notifications.update({                                                                              // 322
      userId: Meteor.userId()                                                                                          // 323
    }, {                                                                                                               // 323
      $set: {                                                                                                          // 324
        isUnread: false                                                                                                // 324
      }                                                                                                                // 324
    }, {                                                                                                               // 324
      multi: true                                                                                                      // 325
    });                                                                                                                // 325
  },                                                                                                                   // 326
  acceptPowerboxOffer: function (sessionId, sturdyRef, sessionToken) {                                                 // 328
    var db = this.connection.sandstormDb;                                                                              // 329
    check(sessionId, String);                                                                                          // 330
    check(sturdyRef, String);                                                                                          // 331
    check(sessionToken, Match.OneOf(String, null, undefined));                                                         // 332
    var sessionQuery = {                                                                                               // 334
      _id: sessionId                                                                                                   // 334
    };                                                                                                                 // 334
                                                                                                                       //
    if (sessionToken) {                                                                                                // 335
      sessionQuery.hashedToken = hashSturdyRef(sessionToken);                                                          // 336
    } else {                                                                                                           // 337
      sessionQuery.userId = this.userId;                                                                               // 338
    }                                                                                                                  // 339
                                                                                                                       //
    var session = db.collections.sessions.findOne(sessionQuery);                                                       // 341
                                                                                                                       //
    if (!session) {                                                                                                    // 342
      throw new Meteor.Error(404, "No matching session found.");                                                       // 343
    }                                                                                                                  // 344
                                                                                                                       //
    var apiToken = fetchApiToken(db, sturdyRef, {                                                                      // 346
      "owner.clientPowerboxOffer.sessionId": sessionId                                                                 // 347
    });                                                                                                                // 347
                                                                                                                       //
    if (!apiToken) {                                                                                                   // 348
      throw new Meteor.Error(404, "No such token.");                                                                   // 349
    }                                                                                                                  // 350
                                                                                                                       //
    var tokenId = apiToken._id;                                                                                        // 352
    var newSturdyRef = void 0;                                                                                         // 354
                                                                                                                       //
    if (sessionToken && apiToken.parentToken) {                                                                        // 355
      // An anonymous user is being offered a child token. To avoid bloating the database,                             // 356
      // we deterministically derive the sturdyref from the session token and the hashed parent                        // 357
      // token.                                                                                                        // 358
      //                                                                                                               // 359
      // Note that an attacker with read access to the database might be able to derive                                // 360
      // `newSturdyRef` from `sessionToken` and gain access to the capability without ever having                      // 361
      // been explicitly offered it by the grain. Therefore, grains must assume that offering                          // 362
      // a capability to an anonymous user can possibly make that capability accessible to any                         // 363
      // other anonymous user who has connected through the same URL.                                                  // 364
      newSturdyRef = Crypto.createHash("sha256").update(sessionToken).update(apiToken.parentToken).digest("base64").slice(0, -1) // removing trailing "="
      .replace(/\+/g, "-").replace(/\//g, "_"); // make URL-safe                                                       // 366
                                                                                                                       //
      if (fetchApiToken(db, newSturdyRef)) {                                                                           // 373
        // We have already generated this token.                                                                       // 374
        db.removeApiTokens({                                                                                           // 375
          _id: tokenId                                                                                                 // 375
        });                                                                                                            // 375
        return newSturdyRef;                                                                                           // 376
      }                                                                                                                // 377
    } else {                                                                                                           // 378
      newSturdyRef = generateSturdyRef();                                                                              // 379
    }                                                                                                                  // 380
                                                                                                                       //
    apiToken.owner = {                                                                                                 // 382
      webkey: null                                                                                                     // 382
    };                                                                                                                 // 382
    insertApiToken(db, apiToken, newSturdyRef);                                                                        // 384
    db.removeApiTokens({                                                                                               // 385
      _id: tokenId                                                                                                     // 385
    });                                                                                                                // 385
    return newSturdyRef;                                                                                               // 386
  }                                                                                                                    // 387
});                                                                                                                    // 295
                                                                                                                       //
var makeSaveTemplateForChild = function (db, parentToken, requirements, parentTokenInfo) {                             // 391
  // Constructs (part of) an ApiToken record appropriate to be used when save()ing a capability                        // 392
  // that was originally created by restore()ing `parentToken`. This fills in everything that is                       // 393
  // appropriate to fill in based only on the parent. Some fields -- especially `owner`, `created`,                    // 394
  // and `_id` -- obviously cannot be filled in until `save()` is called.                                              // 395
  //                                                                                                                   // 396
  // `parentToken` is the raw SturdyRef of the parent.                                                                 // 397
  //                                                                                                                   // 398
  // `requirements` is a list of MembraneRequirements that should be added to any children.                            // 399
  //                                                                                                                   // 400
  // `parentTokenInfo` is the ApiToken record for `parentToken`. Provide this only if you have                         // 401
  // it handy; if omitted it will be looked up.                                                                        // 402
  parentTokenInfo = parentTokenInfo || fetchApiToken(db, parentToken);                                                 // 404
                                                                                                                       //
  if (!parentTokenInfo) {                                                                                              // 405
    throw new Error("no such token");                                                                                  // 406
  }                                                                                                                    // 407
                                                                                                                       //
  var parentOwner = parentTokenInfo.owner || {};                                                                       // 409
  var saveTemplate = void 0;                                                                                           // 411
                                                                                                                       //
  if (parentOwner.clientPowerboxRequest || parentOwner.clientPowerboxOffer) {                                          // 412
    // Saving this token should make a copy of the restored token, rather than make a child                            // 413
    // token.                                                                                                          // 414
    saveTemplate = _.clone(parentTokenInfo); // Don't copy over fields that should be determined at save() time.       // 416
                                                                                                                       //
    delete saveTemplate._id;                                                                                           // 419
    delete saveTemplate.owner;                                                                                         // 420
    delete saveTemplate.created;                                                                                       // 421
  } else {                                                                                                             // 422
    if (parentTokenInfo.accountId) {                                                                                   // 423
      // A UiView token. Need to denormalize some fields from the parent.                                              // 424
      saveTemplate = _.pick(parentTokenInfo, "grainId", "accountId"); // By default, a save()d copy should have the same permissions, so set an allAccess role
      // assignment.                                                                                                   // 428
                                                                                                                       //
      saveTemplate.roleAssignment = {                                                                                  // 429
        allAccess: null                                                                                                // 429
      };                                                                                                               // 429
    } else {                                                                                                           // 430
      // A non-UiView child token.                                                                                     // 431
      saveTemplate = {};                                                                                               // 432
    } // Saved token should be a child of the restored token.                                                          // 433
                                                                                                                       //
                                                                                                                       //
    saveTemplate.parentToken = parentTokenInfo._id; // Will be encrypted on save().                                    // 436
                                                                                                                       //
    saveTemplate.parentTokenKey = parentToken;                                                                         // 439
  }                                                                                                                    // 440
                                                                                                                       //
  if (requirements) {                                                                                                  // 442
    // Append additional requirements requested by caller.                                                             // 443
    saveTemplate.requirements = (saveTemplate.requirements || []).concat(requirements);                                // 444
  }                                                                                                                    // 445
                                                                                                                       //
  return saveTemplate;                                                                                                 // 447
};                                                                                                                     // 448
                                                                                                                       //
restoreInternal = function (db, originalToken, ownerPattern, requirements, originalTokenInfo, currentTokenId, currentTokenKey) {
  // Restores the token `originalToken`, which is a Buffer.                                                            // 452
  //                                                                                                                   // 453
  // `ownerPattern` is a match pattern (i.e. used with check()) that the token's owner must match.                     // 454
  // This is used to enforce than an entity can't use tokens owned by some other entity.                               // 455
  //                                                                                                                   // 456
  // `requirements` is a list of additional MembraneRequirements to add to the returned capability,                    // 457
  // beyond what's already stored in ApiTokens. This is often an empty list.                                           // 458
  //                                                                                                                   // 459
  // `originalTokenInfo` is optional. If specified, it should be the ApiToken record associated                        // 460
  // with `originalToken`. Provide this if you happen to have looked it up already.                                    // 461
  //                                                                                                                   // 462
  // `currentTokenId` is only provided when this function calls itself recursively. It is the                          // 463
  // _id (hashed token) of an ancestor of originalToken. The function recurses until it gets to                        // 464
  // the top-level token.                                                                                              // 465
  requirements = requirements || [];                                                                                   // 467
                                                                                                                       //
  if (!originalTokenInfo) {                                                                                            // 469
    originalTokenInfo = fetchApiToken(db, originalToken);                                                              // 470
                                                                                                                       //
    if (!originalTokenInfo) {                                                                                          // 471
      throw new Meteor.Error(403, "No token found to restore");                                                        // 472
    }                                                                                                                  // 473
  }                                                                                                                    // 474
                                                                                                                       //
  var token = !currentTokenId ? originalTokenInfo : typeof currentTokenKey === "string" ? fetchApiToken(db, currentTokenKey) : db.collections.apiTokens.findOne(currentTokenId);
                                                                                                                       //
  if (!token) {                                                                                                        // 479
    if (!originalTokenInfo) {                                                                                          // 480
      throw new Meteor.Error(403, "Couldn't restore token because parent token has been deleted");                     // 481
    }                                                                                                                  // 482
  }                                                                                                                    // 483
                                                                                                                       //
  if (token.revoked) {                                                                                                 // 485
    throw new Meteor.Error(403, "Token has been revoked");                                                             // 486
  } // The ownerPattern should specify the appropriate user or grain involved, if appropriate.                         // 487
                                                                                                                       //
                                                                                                                       //
  check(token.owner, ownerPattern); // Check requirements on the token.                                                // 490
                                                                                                                       //
  checkRequirements(db, token.requirements); // Check expiration.                                                      // 493
                                                                                                                       //
  if (token.expires && token.expires.getTime() <= Date.now()) {                                                        // 496
    throw new Meteor.Error(403, "Authorization token expired");                                                        // 497
  }                                                                                                                    // 498
                                                                                                                       //
  if (token.expiresIfUnused) {                                                                                         // 500
    if (token.expiresIfUnused.getTime() <= Date.now()) {                                                               // 501
      throw new Meteor.Error(403, "Authorization token expired");                                                      // 502
    } else {                                                                                                           // 503
      // It's getting used now, so clear the expiresIfUnused field.                                                    // 504
      db.collections.apiTokens.update(token._id, {                                                                     // 505
        $unset: {                                                                                                      // 505
          expiresIfUnused: ""                                                                                          // 505
        }                                                                                                              // 505
      });                                                                                                              // 505
    }                                                                                                                  // 506
  } // If this token has a parent, go ahead and recurse to it now.                                                     // 507
                                                                                                                       //
                                                                                                                       //
  if (token.parentToken) {                                                                                             // 510
    // A token which chains to some parent token.  Restore the parent token (possibly recursively),                    // 511
    // checking requirements on the way up.                                                                            // 512
    return restoreInternal(db, originalToken, Match.Any, requirements, originalTokenInfo, token.parentToken, token.parentTokenKey);
  } // Check the passed-in `requirements`.                                                                             // 515
                                                                                                                       //
                                                                                                                       //
  checkRequirements(db, requirements);                                                                                 // 518
                                                                                                                       //
  if (token.objectId) {                                                                                                // 520
    // A token which represents a specific capability exported by a grain.                                             // 521
    // Fix Mongo converting Buffers to Uint8Arrays.                                                                    // 523
    if (token.objectId.appRef) {                                                                                       // 524
      token.objectId.appRef = new Buffer(token.objectId.appRef);                                                       // 525
    } // Ensure the grain is running, then restore the capability.                                                     // 526
                                                                                                                       //
                                                                                                                       //
    return waitPromise(globalBackend.useGrain(token.grainId, function (supervisor) {                                   // 529
      // Note that in this case it is the supervisor's job to implement SystemPersistent, so we                        // 530
      // don't generate a saveTemplate here.                                                                           // 531
      return supervisor.restore(token.objectId, requirements, new Buffer(originalToken, "utf8"));                      // 532
    }));                                                                                                               // 533
  } else {                                                                                                             // 534
    // Construct a template ApiToken for use if the restored capability is save()d later.                              // 535
    var saveTemplate = makeSaveTemplateForChild(db, originalToken, requirements, originalTokenInfo);                   // 536
                                                                                                                       //
    if (token.frontendRef) {                                                                                           // 538
      // A token which represents a capability implemented by a pseudo-driver.                                         // 539
      var cap = globalFrontendRefRegistry.restore(db, saveTemplate, token.frontendRef);                                // 541
      return {                                                                                                         // 542
        cap: cap                                                                                                       // 542
      };                                                                                                               // 542
    } else if (token.grainId) {                                                                                        // 543
      // It's a UiView.                                                                                                // 544
      // If a grain is attempting to restore a UiView, it gets a UiView which filters out all                          // 546
      // the method calls.  In the future, we may allow grains to restore UiViews that pass along the                  // 547
      // "is human" pseudopermission (say, to allow an app to proxy all requests to some grain and                     // 548
      // do some transformation), which will return a different capability.                                            // 549
      return {                                                                                                         // 550
        cap: makePersistentUiView(db, saveTemplate, token.grainId)                                                     // 550
      };                                                                                                               // 550
    } else {                                                                                                           // 551
      throw new Meteor.Error(500, "Unknown token type. ID: " + token._id);                                             // 552
    }                                                                                                                  // 553
  }                                                                                                                    // 554
};                                                                                                                     // 555
                                                                                                                       //
function dropInternal(db, sturdyRef, ownerPattern) {                                                                   // 557
  // Drops `sturdyRef`, checking first that its owner matches `ownerPattern`.                                          // 558
  var token = fetchApiToken(db, sturdyRef);                                                                            // 560
                                                                                                                       //
  if (!token) {                                                                                                        // 561
    return;                                                                                                            // 562
  }                                                                                                                    // 563
                                                                                                                       //
  var hashedSturdyRef = token._id;                                                                                     // 565
                                                                                                                       //
  if (!Match.test(token.owner, ownerPattern)) {                                                                        // 567
    // The caller of `drop()` does not own the token. From the caller's perspective, this means                        // 568
    // that there is actually nothing to be dropped. For example, perhaps a grain got backed up                        // 569
    // and restored, and now is trying to drop one of its pre-backup tokens. The call to                               // 570
    // `SandstormApi.drop()` should succeed, behaving exactly as if the token was not found.                           // 571
    return;                                                                                                            // 572
  } // TODO(soon): Revoke OAuth tokens.                                                                                // 573
                                                                                                                       //
                                                                                                                       //
  if (token.frontendRef && token.frontendRef.notificationHandle) {                                                     // 577
    var notificationId = token.frontendRef.notificationHandle;                                                         // 578
    db.removeApiTokens({                                                                                               // 579
      _id: hashedSturdyRef                                                                                             // 579
    });                                                                                                                // 579
    var anyToken = db.collections.apiTokens.findOne({                                                                  // 580
      "frontendRef.notificationHandle": notificationId                                                                 // 581
    });                                                                                                                // 581
                                                                                                                       //
    if (!anyToken) {                                                                                                   // 582
      // No other tokens referencing this notification exist, so dismiss the notification                              // 583
      dismissNotification(db, notificationId);                                                                         // 584
    }                                                                                                                  // 585
  } else if (token.objectId) {                                                                                         // 586
    waitPromise(globalBackend.useGrain(token.grainId, function (supervisor) {                                          // 587
      return supervisor.drop(token.objectId);                                                                          // 588
    }));                                                                                                               // 589
    db.removeApiTokens({                                                                                               // 591
      _id: hashedSturdyRef                                                                                             // 591
    });                                                                                                                // 591
  } else {                                                                                                             // 592
    db.removeApiTokens({                                                                                               // 593
      _id: hashedSturdyRef                                                                                             // 593
    });                                                                                                                // 593
  }                                                                                                                    // 594
}                                                                                                                      // 595
                                                                                                                       //
var SandstormCoreFactoryImpl = function () {                                                                           //
  function SandstormCoreFactoryImpl(db) {                                                                              // 598
    (0, _classCallCheck3.default)(this, SandstormCoreFactoryImpl);                                                     // 598
    this.db = db;                                                                                                      // 599
  }                                                                                                                    // 600
                                                                                                                       //
  SandstormCoreFactoryImpl.prototype.getSandstormCore = function () {                                                  //
    function getSandstormCore(grainId) {                                                                               //
      return {                                                                                                         // 603
        core: makeSandstormCore(this.db, grainId)                                                                      // 603
      };                                                                                                               // 603
    }                                                                                                                  // 604
                                                                                                                       //
    return getSandstormCore;                                                                                           //
  }();                                                                                                                 //
                                                                                                                       //
  return SandstormCoreFactoryImpl;                                                                                     //
}();                                                                                                                   //
                                                                                                                       //
makeSandstormCoreFactory = function (db) {                                                                             // 607
  return new Capnp.Capability(new SandstormCoreFactoryImpl(db), SandstormCoreFactory);                                 // 608
};                                                                                                                     // 609
                                                                                                                       //
unwrapFrontendCap = function (cap, type, callback) {                                                                   // 611
  // Expect that `cap` is a Cap'n Proto capability implemented by the frontend as a frontendRef                        // 612
  // with the given type (the name of one of the fields of frontendRef). Unwraps the capability                        // 613
  // and then calls callback() with the `frontendRef[type]` descriptor object as                                       // 614
  // the paramater. The callback runs in a Meteor fiber, but this function can be called from                          // 615
  // anywhere. The function returns a Promise for the result of the callback.                                          // 616
  //                                                                                                                   // 617
  // (The reason for the callback, rather than just returning a Promise for the descriptor, is                         // 618
  // so that you don't have to do your own inMeteor() dance.)                                                          // 619
  // For now, we save() the capability and then dig through ApiTokens to find where it leads.                          // 621
  // TODO(cleanup): In theory we should be using something like CapabilityServerSet, but it is                         // 622
  //   not available in Javascript yet and even if it were, it wouldn't work in the case where                         // 623
  //   there are multiple front-end replicas, since the capability could be on a different                             // 624
  //   replica.                                                                                                        // 625
  return cap.castAs(SystemPersistent).save({                                                                           // 627
    frontend: null                                                                                                     // 627
  }).then(function (saveResult) {                                                                                      // 627
    return inMeteor(function () {                                                                                      // 628
      var tokenInfo = fetchApiToken(globalDb, saveResult.sturdyRef.toString("utf8")); // Delete the token now since it's not needed.
                                                                                                                       //
      ApiTokens.remove(tokenInfo._id);                                                                                 // 632
                                                                                                                       //
      for (;;) {                                                                                                       // 634
        if (!tokenInfo) throw new Error("missing token?");                                                             // 635
        if (!tokenInfo.parentToken) break;                                                                             // 636
                                                                                                                       //
        if (typeof tokenInfo.parentTokenKey === "string") {                                                            // 637
          tokenInfo = fetchApiToken(globalDb, tokenInfo.parentTokenKey);                                               // 638
        } else {                                                                                                       // 639
          // Hmm, parentTokenKey doesn't exist or is still encrypted. We can't decrypt the parent                      // 640
          // but we can still fetch it in encrypted format.                                                            // 641
          tokenInfo = ApiTokens.findOne(tokenInfo.parentToken);                                                        // 642
        }                                                                                                              // 643
      }                                                                                                                // 644
                                                                                                                       //
      if (!tokenInfo.frontendRef || !tokenInfo.frontendRef[type]) {                                                    // 646
        throw new Error("not a " + type + " capability");                                                              // 647
      }                                                                                                                // 648
                                                                                                                       //
      return callback(tokenInfo.frontendRef[type]);                                                                    // 650
    });                                                                                                                // 651
  });                                                                                                                  // 652
};                                                                                                                     // 653
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"demo-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/demo-server.js                                                                                               //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var allowDemo = void 0;                                                                                                // 1
module.watch(require("/imports/demo.js"), {                                                                            // 1
  allowDemo: function (v) {                                                                                            // 1
    allowDemo = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var DEMO_EXPIRATION_MS = 60 * 60 * 1000;                                                                               // 20
var DEMO_GRACE_MS = 10 * 60 * 1000; // time between expiration and deletion                                            // 21
                                                                                                                       //
Accounts.validateLoginAttempt(function (attempt) {                                                                     // 23
  // Enforce expiration times for demo accounts.                                                                       // 24
  if (attempt.user && attempt.user.expires) {                                                                          // 26
    var expireIn = attempt.user.expires.getTime() - Date.now() + DEMO_GRACE_MS;                                        // 27
                                                                                                                       //
    if (expireIn < 0) {                                                                                                // 28
      throw new Meteor.Error(403, "This demo account has expired.");                                                   // 29
    }                                                                                                                  // 30
                                                                                                                       //
    if (!allowDemo) {                                                                                                  // 32
      // An expiration time implies that this is a demo account, but those have been disabled.                         // 33
      // Perhaps the config was just changed.                                                                          // 34
      throw new Meteor.Error(403, "Demo accounts have been disabled.");                                                // 35
    } // Force connection close when account expires, so that the client reconnects and                                // 36
    // re-authenticates, which fails.                                                                                  // 39
                                                                                                                       //
                                                                                                                       //
    var connection = attempt.connection;                                                                               // 40
    var handle = Meteor.setTimeout(function () {                                                                       // 41
      connection.close();                                                                                              // 41
    }, expireIn);                                                                                                      // 41
    connection.onClose(function () {                                                                                   // 43
      Meteor.clearTimeout(handle);                                                                                     // 43
    });                                                                                                                // 43
  }                                                                                                                    // 44
                                                                                                                       //
  return true;                                                                                                         // 46
});                                                                                                                    // 47
                                                                                                                       //
function cleanupExpiredUsers() {                                                                                       // 49
  // Delete expired demo accounts and all their grains.                                                                // 50
  var now = new Date(Date.now() - DEMO_GRACE_MS);                                                                      // 52
  Meteor.users.find({                                                                                                  // 53
    expires: {                                                                                                         // 53
      $lt: now                                                                                                         // 53
    },                                                                                                                 // 53
    loginCredentials: {                                                                                                // 53
      $exists: true                                                                                                    // 53
    }                                                                                                                  // 53
  }, {                                                                                                                 // 53
    fields: {                                                                                                          // 54
      _id: 1,                                                                                                          // 54
      loginCredentials: 1,                                                                                             // 54
      lastActive: 1,                                                                                                   // 54
      appDemoId: 1,                                                                                                    // 54
      experiments: 1                                                                                                   // 54
    }                                                                                                                  // 54
  }).forEach(function (user) {                                                                                         // 54
    console.log("delete demo user: " + user._id);                                                                      // 56
    globalDb.deleteAccount(user._id, globalBackend); // Record stats about demo accounts.                              // 57
                                                                                                                       //
    var deleteStatsType = "demoUser";                                                                                  // 60
    var isAppDemoUser = !!user.appDemoId;                                                                              // 61
                                                                                                                       //
    if (isAppDemoUser) {                                                                                               // 62
      deleteStatsType = "appDemoUser";                                                                                 // 63
    } // Intentionally record deleted users at time of deletion to avoid miscounting users that                        // 64
    // were demoing just before the day rolled over.                                                                   // 67
                                                                                                                       //
                                                                                                                       //
    var record = {                                                                                                     // 68
      type: deleteStatsType,                                                                                           // 68
      lastActive: new Date(),                                                                                          // 68
      appId: user.appDemoId                                                                                            // 68
    };                                                                                                                 // 68
                                                                                                                       //
    if (user.experiments) {                                                                                            // 69
      record.experiments = user.experiments;                                                                           // 70
    }                                                                                                                  // 71
                                                                                                                       //
    DeleteStats.insert(record);                                                                                        // 73
  }); // All demo credentials should have been deleted as part of deleting the demo users, but just in                 // 74
  // case, check for them too.                                                                                         // 77
                                                                                                                       //
  Meteor.users.find({                                                                                                  // 78
    expires: {                                                                                                         // 78
      $lt: now                                                                                                         // 78
    },                                                                                                                 // 78
    loginCredentials: {                                                                                                // 78
      $exists: false                                                                                                   // 78
    }                                                                                                                  // 78
  }, {                                                                                                                 // 78
    fields: {                                                                                                          // 79
      _id: 1                                                                                                           // 79
    }                                                                                                                  // 79
  }).forEach(function (user) {                                                                                         // 79
    console.log("delete demo credential: " + user._id);                                                                // 81
    globalDb.deleteCredential(user._id);                                                                               // 82
  });                                                                                                                  // 83
}                                                                                                                      // 84
                                                                                                                       //
if (allowDemo) {                                                                                                       // 86
  Meteor.methods({                                                                                                     // 87
    createDemoUser: function (displayName, appDemoId) {                                                                // 88
      // This is a login method that creates a new temporary user                                                      // 89
      // every time it is used.                                                                                        // 90
      //                                                                                                               // 91
      // appDemoId is important for stats; see cleanupExpiredUsers().                                                  // 92
      check(displayName, String);                                                                                      // 93
      check(appDemoId, Match.OneOf(undefined, null, String)); // Create the new user.                                  // 94
                                                                                                                       //
      var newUser = {                                                                                                  // 97
        expires: new Date(Date.now() + DEMO_EXPIRATION_MS)                                                             // 97
      };                                                                                                               // 97
                                                                                                                       //
      if (appDemoId) {                                                                                                 // 98
        newUser.appDemoId = appDemoId;                                                                                 // 99
      }                                                                                                                // 100
                                                                                                                       //
      var userId = Accounts.insertUserDoc({                                                                            // 102
        profile: {                                                                                                     // 102
          name: displayName                                                                                            // 102
        }                                                                                                              // 102
      }, newUser); // Log them in on this connection.                                                                  // 102
                                                                                                                       //
      return Accounts._loginMethod(this, "createDemoUser", arguments, "demo", function () {                            // 105
        return {                                                                                                       // 106
          userId: userId                                                                                               // 106
        };                                                                                                             // 106
      });                                                                                                              // 106
    },                                                                                                                 // 107
    testExpireDemo: function () {                                                                                      // 109
      if (!isDemoUser()) throw new Meteor.Error(403, "not a demo user");                                               // 110
      var newExpires = new Date(Date.now() + 15000);                                                                   // 112
                                                                                                                       //
      if (Meteor.user().expires.getTime() < newExpires.getTime()) {                                                    // 113
        throw new Meteor.Error(403, "can't exend demo");                                                               // 114
      }                                                                                                                // 115
                                                                                                                       //
      Meteor.users.update(this.userId, {                                                                               // 117
        $set: {                                                                                                        // 117
          expires: newExpires                                                                                          // 117
        }                                                                                                              // 117
      });                                                                                                              // 117
    }                                                                                                                  // 118
  }); // If demo mode is enabled, we permit the client to subscribe to information about an app by                     // 87
  // appId. If this were available in non-demo mode, then anonymous users could effectively ask the                    // 122
  // server which apps are installed.                                                                                  // 123
                                                                                                                       //
  Meteor.publish("appDemoInfo", function (appId) {                                                                     // 124
    // This publishes info about an app, including the latest version of it. Once you log in, it                       // 125
    // also publishes your list of UserActions.                                                                        // 126
    check(appId, String); // Get data about this app from the app index. Note that the app index cache on this server  // 127
    // is typically 6-24 hours delayed from reality, so if an app is newly-available                                   // 130
    // in the app market, it won't be in the app index. Therefore we don't bail-out if the app                         // 131
    // is missing from the AppIndex collection.                                                                        // 132
                                                                                                                       //
    var appIndexData = globalDb.collections.appIndex.findOne({                                                         // 133
      appId: appId                                                                                                     // 133
    }); // Prepare a helper function we can use to transform a package cursor into an appropriate return               // 133
    // value for this function.                                                                                        // 136
                                                                                                                       //
    var packageCursorAndMaybeUserActions = function (userId, appId, packageCursor) {                                   // 137
      // This allows us to avoid creating duplicate UserActions.                                                       // 138
      if (userId) {                                                                                                    // 139
        return [packageCursor, UserActions.find({                                                                      // 140
          userId: userId,                                                                                              // 142
          appId: appId                                                                                                 // 142
        })];                                                                                                           // 142
      }                                                                                                                // 144
                                                                                                                       //
      return packageCursor;                                                                                            // 146
    }; // If the app is in the app index, and the current version is installed, always return that. If                 // 147
    // that package isn't installed, store the version number so we can filter on it later.                            // 150
                                                                                                                       //
                                                                                                                       //
    var packageQuery = {};                                                                                             // 151
                                                                                                                       //
    if (appIndexData) {                                                                                                // 152
      var appIndexPackageQuery = Packages.find({                                                                       // 153
        _id: appIndexData.packageId                                                                                    // 153
      });                                                                                                              // 153
                                                                                                                       //
      if (appIndexPackageQuery.count() > 0) {                                                                          // 154
        return packageCursorAndMaybeUserActions(this.userId, appId, appIndexPackageQuery);                             // 155
      } // If the app index version isn't present, insist on a lower version than the app index                        // 156
      // version. This avoids accidentally catching some development version of the app that has the                   // 159
      // same version number as the app market version but isn't the app market version.                               // 160
                                                                                                                       //
                                                                                                                       //
      packageQuery["manifest.appVersion"] = {                                                                          // 161
        $lt: appIndexData.versionNumber                                                                                // 161
      };                                                                                                               // 161
    } // If the specific package from the app index isn't installed, or the app isn't there at all, do                 // 162
    // our best.                                                                                                       // 165
                                                                                                                       //
                                                                                                                       //
    packageQuery.appId = appId;                                                                                        // 166
    var packageCursor = Packages.find(packageQuery, {                                                                  // 167
      sort: {                                                                                                          // 169
        "manifest.appVersion": -1                                                                                      // 169
      },                                                                                                               // 169
      limit: 1                                                                                                         // 170
    });                                                                                                                // 169
    return packageCursorAndMaybeUserActions(this.userId, appId, packageCursor);                                        // 172
  });                                                                                                                  // 173
  SandstormDb.periodicCleanup(DEMO_EXPIRATION_MS, cleanupExpiredUsers);                                                // 175
} else {                                                                                                               // 176
  // Just run once, in case the config just changed from allowing demos to prohibiting them.                           // 177
  Meteor.setTimeout(cleanupExpiredUsers, DEMO_EXPIRATION_MS + DEMO_GRACE_MS);                                          // 178
}                                                                                                                      // 179
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"desktop-notifications-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/desktop-notifications-server.js                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var Meteor = void 0;                                                                                                   // 1
module.watch(require("meteor/meteor"), {                                                                               // 1
  Meteor: function (v) {                                                                                               // 1
    Meteor = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
SandstormDb.periodicCleanup(120000, function () {                                                                      // 19
  // Remove old desktop notfications regularly.                                                                        // 20
  // periodicCleanup doesn't like runs more frequent than once every two minutes                                       // 21
  var now = Date.now();                                                                                                // 22
  var then = new Date(now - 30000); // Clear tokens older than 30 seconds.                                             // 23
                                                                                                                       //
  globalDb.collections.desktopNotifications.find({                                                                     // 24
    creationDate: {                                                                                                    // 25
      $lt: then                                                                                                        // 25
    }                                                                                                                  // 25
  }).forEach(function (doc) {                                                                                          // 24
    globalDb.collections.desktopNotifications.remove({                                                                 // 27
      _id: doc._id                                                                                                     // 28
    });                                                                                                                // 27
  });                                                                                                                  // 30
});                                                                                                                    // 31
Meteor.publish("desktopNotifications", function () {                                                                   // 33
  var _this = this;                                                                                                    // 33
                                                                                                                       //
  var subscribeTime = new Date();                                                                                      // 34
                                                                                                                       //
  if (!this.userId) {                                                                                                  // 36
    // No desktop notifications for anonymous users.                                                                   // 37
    return [];                                                                                                         // 38
  }                                                                                                                    // 39
                                                                                                                       //
  var db = this.connection.sandstormDb;                                                                                // 41
  var callbacks = {                                                                                                    // 42
    added: function (doc) {                                                                                            // 43
      _this.added("desktopNotifications", doc._id, doc);                                                               // 44
    },                                                                                                                 // 45
    changed: function (newDoc, oldDoc) {                                                                               // 47
      _this.changed("desktopNotifications", newDoc._id, newDoc);                                                       // 48
    },                                                                                                                 // 49
    removed: function (doc) {                                                                                          // 51
      _this.removed("desktopNotifications", doc._id);                                                                  // 52
    }                                                                                                                  // 53
  };                                                                                                                   // 42
  var sub = db.collections.desktopNotifications.find({                                                                 // 56
    userId: this.userId,                                                                                               // 57
    creationDate: {                                                                                                    // 58
      $gt: subscribeTime                                                                                               // 58
    }                                                                                                                  // 58
  }).observe(callbacks);                                                                                               // 56
  this.onStop(function () {                                                                                            // 61
    if (sub) {                                                                                                         // 62
      sub.stop();                                                                                                      // 63
    }                                                                                                                  // 64
  });                                                                                                                  // 65
  this.ready();                                                                                                        // 67
});                                                                                                                    // 68
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"dev-accounts-server.js":function(require){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/dev-accounts-server.js                                                                                       //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var Crypto = Npm.require("crypto");                                                                                    // 17
                                                                                                                       //
Meteor.methods({                                                                                                       // 19
  createDevAccount: function (displayName, isAdmin, profile, unverifiedEmail) {                                        // 20
    // This is a login method that creates or logs in a dev account with the given displayName                         // 21
    check(displayName, String);                                                                                        // 23
    check(isAdmin, Match.OneOf(undefined, null, Boolean));                                                             // 24
    check(profile, Match.OneOf(undefined, null, Object));                                                              // 25
    check(unverifiedEmail, Match.OneOf(undefined, null, String));                                                      // 26
                                                                                                                       //
    if (!this.connection.sandstormDb.allowDevAccounts()) {                                                             // 28
      throw new Meteor.Error(404, "Dev accounts are not enabled on this server");                                      // 29
    }                                                                                                                  // 30
                                                                                                                       //
    isAdmin = isAdmin || false;                                                                                        // 32
    profile = profile || {};                                                                                           // 34
    profile.name = profile.name || displayName;                                                                        // 35
    var hasCompletedSignup = !!unverifiedEmail && !!profile.pronoun && !!profile.handle;                               // 36
    var user = Meteor.users.findOne({                                                                                  // 38
      "services.dev.name": displayName                                                                                 // 38
    });                                                                                                                // 38
    var userId = void 0;                                                                                               // 39
                                                                                                                       //
    if (user) {                                                                                                        // 41
      userId = user._id;                                                                                               // 42
    } else {                                                                                                           // 43
      userId = Accounts.insertUserDoc({                                                                                // 44
        profile: profile,                                                                                              // 45
        unverifiedEmail: unverifiedEmail                                                                               // 46
      }, {                                                                                                             // 44
        services: {                                                                                                    // 48
          dev: {                                                                                                       // 49
            name: displayName,                                                                                         // 50
            isAdmin: isAdmin,                                                                                          // 51
            hasCompletedSignup: hasCompletedSignup                                                                     // 52
          }                                                                                                            // 49
        }                                                                                                              // 48
      });                                                                                                              // 47
    } // Log them in on this connection.                                                                               // 56
                                                                                                                       //
                                                                                                                       //
    return Accounts._loginMethod(this, "createDevAccount", arguments, "dev", function () {                             // 58
      return {                                                                                                         // 59
        userId: userId                                                                                                 // 59
      };                                                                                                               // 59
    });                                                                                                                // 59
  }                                                                                                                    // 60
});                                                                                                                    // 19
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"grain-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/grain-server.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _typeof2 = require("babel-runtime/helpers/typeof");                                                                //
                                                                                                                       //
var _typeof3 = _interopRequireDefault(_typeof2);                                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var sendEmail = void 0;                                                                                                // 1
module.watch(require("/imports/server/email.js"), {                                                                    // 1
  send: function (v) {                                                                                                 // 1
    sendEmail = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var Crypto = Npm.require("crypto");                                                                                    // 17
                                                                                                                       //
var ROOT_URL = process.env.ROOT_URL;                                                                                   // 20
                                                                                                                       //
var emailLinkWithInlineStyle = function (url, text) {                                                                  // 22
  return "<a href='" + url + "' style='display:inline-block;text-decoration:none;" + "font-family:sans-serif;width:200px;min-height:30px;line-height:30px;" + "border-radius:4px;text-align:center;background:#762F87;color:white'>" + text + "</a>";
};                                                                                                                     // 27
                                                                                                                       //
Meteor.publish("grainTopBar", function (grainId) {                                                                     // 29
  check(grainId, String);                                                                                              // 30
  var result = [Grains.find({                                                                                          // 32
    _id: grainId,                                                                                                      // 34
    $or: [{                                                                                                            // 35
      userId: this.userId                                                                                              // 36
    }, {                                                                                                               // 36
      "private": {                                                                                                     // 37
        $ne: true                                                                                                      // 37
      }                                                                                                                // 37
    }]                                                                                                                 // 37
  }, {                                                                                                                 // 33
    fields: {                                                                                                          // 40
      title: 1,                                                                                                        // 41
      userId: 1,                                                                                                       // 42
      "private": 1                                                                                                     // 43
    }                                                                                                                  // 40
  })];                                                                                                                 // 39
                                                                                                                       //
  if (this.userId) {                                                                                                   // 47
    result.push(ApiTokens.find({                                                                                       // 48
      grainId: grainId,                                                                                                // 49
      $or: [{                                                                                                          // 50
        "owner.user.accountId": this.userId                                                                            // 51
      }, {                                                                                                             // 51
        accountId: this.userId                                                                                         // 52
      }]                                                                                                               // 52
    }));                                                                                                               // 48
  }                                                                                                                    // 55
                                                                                                                       //
  return result;                                                                                                       // 57
}); // We allow users to learn package information about a grain they own.                                             // 58
// This is used for obtaining icon and app title information for grains                                                // 61
// you own, which is used in the sidebar. It is not a security/privacy                                                 // 62
// risk since it only exposes this information for grains the user owns.                                               // 63
                                                                                                                       //
Meteor.publish("packageByGrainId", function (grainId) {                                                                // 64
  check(grainId, String);                                                                                              // 65
  var publishThis = []; // We need to publish the packageId so that client-side code can                               // 66
  // find the right package.                                                                                           // 68
                                                                                                                       //
  var thisGrainCursor = Grains.find({                                                                                  // 69
    _id: grainId,                                                                                                      // 70
    userId: this.userId                                                                                                // 71
  }, {                                                                                                                 // 69
    fields: {                                                                                                          // 73
      packageId: 1                                                                                                     // 73
    }                                                                                                                  // 73
  });                                                                                                                  // 72
  publishThis.push(thisGrainCursor);                                                                                   // 75
                                                                                                                       //
  if (thisGrainCursor.count()) {                                                                                       // 77
    var thisGrain = thisGrainCursor.fetch()[0];                                                                        // 78
    var thisPackageCursor = Packages.find({                                                                            // 79
      _id: thisGrain.packageId                                                                                         // 79
    });                                                                                                                // 79
    publishThis.push(thisPackageCursor);                                                                               // 80
  }                                                                                                                    // 81
                                                                                                                       //
  return publishThis;                                                                                                  // 83
});                                                                                                                    // 84
Meteor.publish("tokenInfo", function (token, isStandalone) {                                                           // 86
  // Allows the client side to map a raw token to its entry in ApiTokens, and the additional                           // 87
  // metadata that it will need to display the app icon and title.  We do not care about making                        // 88
  // the metadata reactive.                                                                                            // 89
  check(token, String);                                                                                                // 90
  var hashedToken = Crypto.createHash("sha256").update(token).digest("base64");                                        // 92
  var apiToken = ApiTokens.findOne({                                                                                   // 93
    _id: hashedToken                                                                                                   // 94
  }, {                                                                                                                 // 93
    fields: {                                                                                                          // 96
      grainId: 1,                                                                                                      // 97
      accountId: 1,                                                                                                    // 98
      owner: 1,                                                                                                        // 99
      revoked: 1                                                                                                       // 100
    }                                                                                                                  // 96
  });                                                                                                                  // 95
                                                                                                                       //
  if (!apiToken) {                                                                                                     // 103
    this.added("tokenInfo", token, {                                                                                   // 104
      invalidToken: true                                                                                               // 104
    });                                                                                                                // 104
  } else if (apiToken.revoked) {                                                                                       // 105
    this.added("tokenInfo", token, {                                                                                   // 106
      revoked: true                                                                                                    // 106
    });                                                                                                                // 106
  } else {                                                                                                             // 107
    var grainId = apiToken.grainId;                                                                                    // 108
    var grain = Grains.findOne({                                                                                       // 109
      _id: grainId                                                                                                     // 110
    }, {                                                                                                               // 109
      fields: {                                                                                                        // 112
        packageId: 1,                                                                                                  // 113
        appId: 1,                                                                                                      // 114
        userId: 1                                                                                                      // 115
      }                                                                                                                // 112
    });                                                                                                                // 111
                                                                                                                       //
    if (!grain) {                                                                                                      // 118
      this.added("tokenInfo", token, {                                                                                 // 119
        invalidToken: true                                                                                             // 119
      });                                                                                                              // 119
    } else {                                                                                                           // 120
      if (apiToken.owner && apiToken.owner.user) {                                                                     // 121
        var account = Meteor.users.findOne({                                                                           // 122
          _id: apiToken.owner.user.accountId                                                                           // 122
        });                                                                                                            // 122
        var metadata = apiToken.owner.user.denormalizedGrainMetadata;                                                  // 123
                                                                                                                       //
        if (account && metadata) {                                                                                     // 124
          account.credentials = Meteor.users.find({                                                                    // 125
            _id: {                                                                                                     // 126
              $in: account.loginCredentials.map(function (cred) {                                                      // 126
                return cred.id;                                                                                        // 126
              })                                                                                                       // 126
            }                                                                                                          // 126
          }).map(function (credential) {                                                                               // 126
            return {                                                                                                   // 128
              serviceName: SandstormDb.getServiceName(credential),                                                     // 129
              intrinsicName: SandstormDb.getIntrinsicName(credential, true),                                           // 130
              loginId: SandstormDb.getLoginId(credential)                                                              // 131
            };                                                                                                         // 128
          });                                                                                                          // 133
          account.intrinsicNames = globalDb.getAccountIntrinsicNames(account, false);                                  // 135
          this.added("tokenInfo", token, {                                                                             // 137
            accountOwner: _.pick(account, "_id", "profile", "credentials", "intrinsicNames"),                          // 138
            grainId: grainId,                                                                                          // 139
            grainMetadata: metadata                                                                                    // 140
          });                                                                                                          // 137
        } else {                                                                                                       // 142
          this.added("tokenInfo", token, {                                                                             // 143
            invalidToken: true                                                                                         // 143
          });                                                                                                          // 143
        }                                                                                                              // 144
      } else if (!apiToken.owner || "webkey" in apiToken.owner) {                                                      // 145
        if (this.userId && !isStandalone) {                                                                            // 146
          var childToken = ApiTokens.findOne({                                                                         // 147
            "owner.user.accountId": this.userId,                                                                       // 148
            parentToken: apiToken._id                                                                                  // 149
          });                                                                                                          // 147
                                                                                                                       //
          if (childToken || this.userId === grain.userId || this.userId === apiToken.accountId) {                      // 151
            this.added("tokenInfo", token, {                                                                           // 153
              alreadyRedeemed: true,                                                                                   // 153
              grainId: apiToken.grainId                                                                                // 153
            });                                                                                                        // 153
            this.ready();                                                                                              // 154
            return;                                                                                                    // 155
          }                                                                                                            // 156
        }                                                                                                              // 157
                                                                                                                       //
        var pkg = Packages.findOne({                                                                                   // 159
          _id: grain.packageId                                                                                         // 159
        }, {                                                                                                           // 159
          fields: {                                                                                                    // 159
            manifest: 1                                                                                                // 159
          }                                                                                                            // 159
        });                                                                                                            // 159
        var appTitle = pkg && pkg.manifest && pkg.manifest.appTitle || {                                               // 160
          defaultText: ""                                                                                              // 160
        };                                                                                                             // 160
        var appIcon = undefined;                                                                                       // 161
                                                                                                                       //
        if (pkg && pkg.manifest && pkg.manifest.metadata && pkg.manifest.metadata.icons) {                             // 162
          var icons = pkg.manifest.metadata.icons;                                                                     // 163
          appIcon = icons.grain || icons.appGrid;                                                                      // 164
        }                                                                                                              // 165
                                                                                                                       //
        var denormalizedGrainMetadata = {                                                                              // 167
          appTitle: appTitle,                                                                                          // 168
          icon: appIcon,                                                                                               // 169
          appId: appIcon ? undefined : grain.appId                                                                     // 170
        };                                                                                                             // 167
        this.added("tokenInfo", token, {                                                                               // 172
          webkey: true,                                                                                                // 173
          grainId: grainId,                                                                                            // 174
          grainMetadata: denormalizedGrainMetadata                                                                     // 175
        });                                                                                                            // 172
      } else {                                                                                                         // 177
        this.added("tokenInfo", token, {                                                                               // 178
          invalidToken: true                                                                                           // 178
        });                                                                                                            // 178
      }                                                                                                                // 179
    }                                                                                                                  // 180
  }                                                                                                                    // 181
                                                                                                                       //
  this.ready();                                                                                                        // 183
  return;                                                                                                              // 184
});                                                                                                                    // 185
Meteor.publish("requestingAccess", function (grainId) {                                                                // 187
  check(grainId, String);                                                                                              // 188
                                                                                                                       //
  if (!this.userId) {                                                                                                  // 190
    throw new Meteor.Error(403, "Must be logged in to request access.");                                               // 191
  }                                                                                                                    // 192
                                                                                                                       //
  var grain = globalDb.getGrain(grainId);                                                                              // 194
                                                                                                                       //
  if (!grain) {                                                                                                        // 195
    throw new Meteor.Error(404, "Grain not found.");                                                                   // 196
  }                                                                                                                    // 197
                                                                                                                       //
  if (grain.userId === this.userId) {                                                                                  // 199
    this.added("grantedAccessRequests", Random.id(), {                                                                 // 200
      grainId: grainId                                                                                                 // 200
    });                                                                                                                // 200
  }                                                                                                                    // 201
                                                                                                                       //
  var _this = this;                                                                                                    // 203
                                                                                                                       //
  var query = ApiTokens.find({                                                                                         // 204
    grainId: grainId,                                                                                                  // 205
    accountId: grain.userId,                                                                                           // 206
    parentToken: {                                                                                                     // 207
      $exists: false                                                                                                   // 207
    },                                                                                                                 // 207
    "owner.user.accountId": this.userId,                                                                               // 208
    revoked: {                                                                                                         // 209
      $ne: true                                                                                                        // 209
    }                                                                                                                  // 209
  });                                                                                                                  // 204
  var handle = query.observe({                                                                                         // 211
    added: function (apiToken) {                                                                                       // 212
      _this.added("grantedAccessRequests", Random.id(), {                                                              // 213
        grainId: grainId                                                                                               // 213
      });                                                                                                              // 213
    }                                                                                                                  // 214
  });                                                                                                                  // 211
  this.onStop(function () {                                                                                            // 217
    return handle.stop();                                                                                              // 217
  });                                                                                                                  // 217
});                                                                                                                    // 218
var GRAIN_DELETION_MS = 1000 * 60 * 60 * 24 * 30; // thirty days                                                       // 220
                                                                                                                       //
SandstormDb.periodicCleanup(86400000, function () {                                                                    // 221
  var trashExpiration = new Date(Date.now() - GRAIN_DELETION_MS);                                                      // 222
  globalDb.removeApiTokens({                                                                                           // 223
    trashed: {                                                                                                         // 223
      $lt: trashExpiration                                                                                             // 223
    }                                                                                                                  // 223
  });                                                                                                                  // 223
  globalDb.deleteGrains({                                                                                              // 224
    trashed: {                                                                                                         // 224
      $lt: trashExpiration                                                                                             // 224
    }                                                                                                                  // 224
  }, globalBackend, "grain");                                                                                          // 224
});                                                                                                                    // 225
Meteor.methods({                                                                                                       // 227
  updateGrainTitle: function (grainId, newTitle, obsolete) {                                                           // 228
    check(grainId, String);                                                                                            // 229
    check(newTitle, String);                                                                                           // 230
                                                                                                                       //
    if (this.userId) {                                                                                                 // 231
      var grain = Grains.findOne(grainId);                                                                             // 232
                                                                                                                       //
      if (grain) {                                                                                                     // 233
        if (grain.userId === this.userId) {                                                                            // 234
          Grains.update({                                                                                              // 235
            _id: grainId,                                                                                              // 235
            userId: this.userId                                                                                        // 235
          }, {                                                                                                         // 235
            $set: {                                                                                                    // 235
              title: newTitle                                                                                          // 235
            }                                                                                                          // 235
          }); // Denormalize new title out to all sharing tokens.                                                      // 235
                                                                                                                       //
          ApiTokens.update({                                                                                           // 238
            grainId: grainId,                                                                                          // 238
            "owner.user": {                                                                                            // 238
              $exists: true                                                                                            // 238
            }                                                                                                          // 238
          }, {                                                                                                         // 238
            $set: {                                                                                                    // 239
              "owner.user.upstreamTitle": newTitle                                                                     // 239
            }                                                                                                          // 239
          }, {                                                                                                         // 239
            multi: true                                                                                                // 240
          });                                                                                                          // 240
        } else {                                                                                                       // 241
          var token = ApiTokens.findOne({                                                                              // 242
            grainId: grainId,                                                                                          // 243
            objectId: {                                                                                                // 244
              $exists: false                                                                                           // 244
            },                                                                                                         // 244
            "owner.user.accountId": this.userId                                                                        // 245
          }, {                                                                                                         // 242
            sort: {                                                                                                    // 247
              created: 1                                                                                               // 247
            } // The oldest token is our source of truth for the name.                                                 // 247
                                                                                                                       //
          });                                                                                                          // 246
                                                                                                                       //
          if (token && token.owner.user.title !== newTitle) {                                                          // 249
            if (token.owner.user.upstreamTitle === newTitle) {                                                         // 250
              // User renamed grain to match upstream title. Act like they never renamed it at                         // 251
              // all.                                                                                                  // 252
              ApiTokens.update({                                                                                       // 253
                grainId: grainId,                                                                                      // 254
                "owner.user.accountId": this.userId                                                                    // 255
              }, {                                                                                                     // 253
                $set: {                                                                                                // 257
                  "owner.user.title": newTitle                                                                         // 257
                },                                                                                                     // 257
                $unset: {                                                                                              // 258
                  "owner.user.upstreamTitle": 1,                                                                       // 258
                  "owner.user.renamed": 1                                                                              // 258
                }                                                                                                      // 258
              }, {                                                                                                     // 256
                multi: true                                                                                            // 260
              });                                                                                                      // 259
            } else {                                                                                                   // 262
              var modification = {                                                                                     // 263
                "owner.user.title": newTitle,                                                                          // 264
                "owner.user.renamed": true                                                                             // 265
              };                                                                                                       // 263
                                                                                                                       //
              if (!token.owner.user.upstreamTitle) {                                                                   // 268
                // If `upstreamTitle` isn't present then it is equal to the old title.                                 // 269
                modification["owner.user.upstreamTitle"] = token.owner.user.title;                                     // 270
              }                                                                                                        // 271
                                                                                                                       //
              ApiTokens.update({                                                                                       // 273
                grainId: grainId,                                                                                      // 273
                "owner.user.accountId": this.userId                                                                    // 273
              }, {                                                                                                     // 273
                $set: modification                                                                                     // 274
              }, {                                                                                                     // 274
                multi: true                                                                                            // 275
              });                                                                                                      // 275
            }                                                                                                          // 276
          }                                                                                                            // 277
        }                                                                                                              // 278
      }                                                                                                                // 279
    }                                                                                                                  // 280
  },                                                                                                                   // 281
  privatizeGrain: function (grainId) {                                                                                 // 283
    check(grainId, String);                                                                                            // 284
                                                                                                                       //
    if (this.userId) {                                                                                                 // 285
      Grains.update({                                                                                                  // 286
        _id: grainId,                                                                                                  // 286
        userId: this.userId                                                                                            // 286
      }, {                                                                                                             // 286
        $set: {                                                                                                        // 286
          "private": true                                                                                              // 286
        }                                                                                                              // 286
      });                                                                                                              // 286
    }                                                                                                                  // 287
  },                                                                                                                   // 288
  inviteUsersToGrain: function (_origin, obsolete, grainId, title, roleAssignment, contacts, message) {                // 290
    if ((typeof message === "undefined" ? "undefined" : (0, _typeof3.default)(message)) === "object") {                // 292
      // Older versions of the client passed an object here, but we only care about the `text`                         // 293
      // parameter. (This block can eventually be removed.)                                                            // 294
      message = message.text;                                                                                          // 295
    }                                                                                                                  // 296
                                                                                                                       //
    if (!this.isSimulation) {                                                                                          // 298
      check(_origin, String);                                                                                          // 299
      check(grainId, String);                                                                                          // 300
      check(title, String);                                                                                            // 301
      check(roleAssignment, roleAssignmentPattern);                                                                    // 302
      check(contacts, [{                                                                                               // 303
        _id: String,                                                                                                   // 305
        isDefault: Match.Optional(Boolean),                                                                            // 306
        profile: Match.ObjectIncluding({                                                                               // 307
          service: Match.Optional(String),                                                                             // 308
          name: String,                                                                                                // 309
          intrinsicName: Match.Optional(String)                                                                        // 310
        }),                                                                                                            // 307
        intrinsicNames: [Object]                                                                                       // 312
      }]);                                                                                                             // 304
      check(message, String);                                                                                          // 315
                                                                                                                       //
      if (!this.userId) {                                                                                              // 316
        throw new Meteor.Error(403, "Must be logged in to share by email.");                                           // 317
      }                                                                                                                // 318
                                                                                                                       //
      if (contacts.length === 0) {                                                                                     // 320
        throw new Meteor.Error(400, "No contacts were provided.");                                                     // 321
      }                                                                                                                // 322
                                                                                                                       //
      if (globalDb.isDemoUser()) {                                                                                     // 324
        throw new Meteor.Error(403, "Demo users are not allowed to share by email.");                                  // 325
      }                                                                                                                // 326
                                                                                                                       //
      var escapedMessage = message.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/\n/g, "<br>");
      var accountId = this.userId;                                                                                     // 334
      var outerResult = {                                                                                              // 335
        successes: [],                                                                                                 // 335
        failures: []                                                                                                   // 335
      };                                                                                                               // 335
      var fromEmail = globalDb.getReturnAddressWithDisplayName(accountId);                                             // 336
      var replyTo = globalDb.getPrimaryEmail(accountId);                                                               // 337
      contacts.forEach(function (contact) {                                                                            // 338
        if (contact.isDefault) {                                                                                       // 339
          var emailAddress = contact.profile.name;                                                                     // 340
          var result = SandstormPermissions.createNewApiToken(globalDb, {                                              // 341
            accountId: accountId                                                                                       // 342
          }, grainId, "email invitation for " + emailAddress, roleAssignment, {                                        // 342
            webkey: {                                                                                                  // 344
              forSharing: true                                                                                         // 344
            }                                                                                                          // 344
          });                                                                                                          // 344
          var url = ROOT_URL + "/shared/" + result.token;                                                              // 345
          var html = escapedMessage + "<br><br>" + emailLinkWithInlineStyle(url, "Open Shared Grain") + "<div style='font-size:8pt;font-style:italic;color:gray'>" + "Note: If you forward this email to other people, they will be able to access " + "the share as well. To prevent this, remove the button before forwarding.</div>";
                                                                                                                       //
          try {                                                                                                        // 351
            globalDb.incrementDailySentMailCount(accountId);                                                           // 352
            sendEmail({                                                                                                // 353
              to: emailAddress,                                                                                        // 354
              from: fromEmail,                                                                                         // 355
              replyTo: replyTo,                                                                                        // 356
              subject: title + " - Invitation to collaborate",                                                         // 357
              text: message + "\n\nFollow this link to open the shared grain:\n\n" + url + "\n\nNote: If you forward this email to other people, they will be able to " + "access the share as well. To prevent this, remove the link before forwarding.",
              html: html                                                                                               // 361
            });                                                                                                        // 353
          } catch (e) {                                                                                                // 363
            console.error(e.stack);                                                                                    // 364
            outerResult.failures.push({                                                                                // 365
              contact: contact,                                                                                        // 365
              error: e.toString()                                                                                      // 365
            });                                                                                                        // 365
          }                                                                                                            // 366
        } else {                                                                                                       // 367
          var _result = SandstormPermissions.createNewApiToken(globalDb, {                                             // 368
            accountId: accountId                                                                                       // 369
          }, grainId, "direct invitation to " + contact.profile.name, roleAssignment, {                                // 369
            user: {                                                                                                    // 371
              accountId: contact._id,                                                                                  // 371
              title: title                                                                                             // 371
            }                                                                                                          // 371
          });                                                                                                          // 371
                                                                                                                       //
          var _url = ROOT_URL + "/shared/" + _result.token;                                                            // 372
                                                                                                                       //
          try {                                                                                                        // 373
            var account = Meteor.users.findOne({                                                                       // 374
              _id: contact._id                                                                                         // 374
            });                                                                                                        // 374
                                                                                                                       //
            var email = _.findWhere(SandstormDb.getUserEmails(account), {                                              // 375
              primary: true                                                                                            // 376
            });                                                                                                        // 376
                                                                                                                       //
            if (email) {                                                                                               // 377
              var intrinsicName = contact.profile.intrinsicName;                                                       // 378
                                                                                                                       //
              var _html = escapedMessage + "<br><br>" + emailLinkWithInlineStyle(_url, "Open Shared Grain") + "<div style='font-size:8pt;font-style:italic;color:gray'>";
                                                                                                                       //
              globalDb.incrementDailySentMailCount(accountId);                                                         // 383
              sendEmail({                                                                                              // 384
                to: email.email,                                                                                       // 385
                from: fromEmail,                                                                                       // 386
                replyTo: replyTo,                                                                                      // 387
                subject: title + " - Invitation to collaborate",                                                       // 388
                text: message + "\n\nFollow this link to open the shared grain:\n\n" + _url,                           // 389
                html: _html                                                                                            // 390
              });                                                                                                      // 384
            } else {                                                                                                   // 392
              outerResult.failures.push({                                                                              // 393
                contact: contact,                                                                                      // 393
                warning: "User does not have a " + "verified email, so notification of this share was not sent to them. Please " + "manually share " + _url + " with them."
              });                                                                                                      // 393
            }                                                                                                          // 396
          } catch (e) {                                                                                                // 397
            console.error(e.stack);                                                                                    // 398
            outerResult.failures.push({                                                                                // 399
              contact: contact,                                                                                        // 399
              error: e.toString(),                                                                                     // 399
              warning: "Share succeeded, but there was an error emailing the user. Please " + "manually share " + _url + " with them."
            });                                                                                                        // 399
          }                                                                                                            // 402
        }                                                                                                              // 403
      });                                                                                                              // 404
      return outerResult;                                                                                              // 406
    }                                                                                                                  // 407
  },                                                                                                                   // 408
  requestAccess: function (_origin, grainId, obsolete) {                                                               // 410
    check(_origin, String);                                                                                            // 411
    check(grainId, String);                                                                                            // 412
                                                                                                                       //
    if (!this.isSimulation) {                                                                                          // 413
      if (!this.userId) {                                                                                              // 414
        throw new Meteor.Error(403, "Must be logged in to request access.");                                           // 415
      }                                                                                                                // 416
                                                                                                                       //
      var grain = Grains.findOne(grainId);                                                                             // 418
                                                                                                                       //
      if (!grain) {                                                                                                    // 419
        throw new Meteor.Error(404, "No such grain");                                                                  // 420
      }                                                                                                                // 421
                                                                                                                       //
      var grainOwner = globalDb.getUser(grain.userId);                                                                 // 423
                                                                                                                       //
      var email = _.findWhere(SandstormDb.getUserEmails(grainOwner), {                                                 // 424
        primary: true                                                                                                  // 424
      });                                                                                                              // 424
                                                                                                                       //
      if (!email) {                                                                                                    // 425
        throw new Meteor.Error("no email", "Grain owner has no email address.");                                       // 426
      }                                                                                                                // 427
                                                                                                                       //
      var emailAddress = email.email;                                                                                  // 429
      globalDb.addContact(grainOwner._id, this.userId);                                                                // 431
      var fromEmail = globalDb.getReturnAddressWithDisplayName(this.userId);                                           // 433
      var replyTo = globalDb.getPrimaryEmail(this.userId); // TODO(soon): In the HTML version, we should display an identity card.
                                                                                                                       //
      var identityNotes = [];                                                                                          // 437
      globalDb.getAccountIntrinsicNames(Meteor.user(), true).forEach(function (intrinsic) {                            // 438
        // TODO(cleanup): Don't switch on service here; extend getAccountIntrinsicNames or                             // 439
        //   Account.loginServices to cover what we need.                                                              // 440
        if (intrinsic.service === "google") {                                                                          // 441
          identityNotes.push(intrinsic.name);                                                                          // 442
        } else if (intrinsic.service === "github") {                                                                   // 443
          identityNotes.push(intrinsic.name + " on GitHub");                                                           // 444
        } else if (intrinsic.service === "email") {                                                                    // 445
          identityNotes.push(intrinsic.name);                                                                          // 446
        } else if (intrinsic.service === "ldap") {                                                                     // 447
          identityNotes.push(intrinsic.name + " on LDAP");                                                             // 448
        } else if (intrinsic.service === "saml") {                                                                     // 449
          identityNotes.push(intrinsic.name + " on SAML");                                                             // 450
        }                                                                                                              // 451
      });                                                                                                              // 452
      var identityNote = identityNotes.length === 0 ? "" : " (" + identityNotes.join(", ") + ")";                      // 454
      var message = Meteor.user().profile.name + identityNote + " is requesting access to your grain: " + grain.title + ".";
      var url = ROOT_URL + "/share/" + grainId + "/" + this.userId;                                                    // 460
      var html = message + "<br><br>" + emailLinkWithInlineStyle(url, "Open Sharing Menu");                            // 462
      var user = Meteor.user();                                                                                        // 465
      var ACCESS_REQUEST_LIMIT = 10;                                                                                   // 466
      var resetCount = true;                                                                                           // 467
                                                                                                                       //
      if (user.accessRequests) {                                                                                       // 468
        if (user.accessRequests.resetOn < new Date()) {                                                                // 469
          Meteor.users.update({                                                                                        // 470
            _id: user._id                                                                                              // 470
          }, {                                                                                                         // 470
            $unset: {                                                                                                  // 470
              accessRequests: 1                                                                                        // 470
            }                                                                                                          // 470
          });                                                                                                          // 470
        } else if (user.accessRequests.count >= ACCESS_REQUEST_LIMIT) {                                                // 471
          throw new Meteor.Error(403, "For spam control reasons, you are not allowed to make " + "more than " + ACCESS_REQUEST_LIMIT + " access requests per day.");
        } else {                                                                                                       // 475
          resetCount = false;                                                                                          // 476
        }                                                                                                              // 477
      }                                                                                                                // 478
                                                                                                                       //
      var modifier = {                                                                                                 // 480
        $inc: {                                                                                                        // 480
          "accessRequests.count": 1                                                                                    // 480
        }                                                                                                              // 480
      };                                                                                                               // 480
                                                                                                                       //
      if (resetCount) {                                                                                                // 481
        var tomorrow = new Date(Date.now() + 86400000);                                                                // 482
        modifier.$set = {                                                                                              // 483
          "accessRequests.resetOn": tomorrow                                                                           // 483
        };                                                                                                             // 483
      }                                                                                                                // 484
                                                                                                                       //
      Meteor.users.update({                                                                                            // 486
        _id: user._id                                                                                                  // 486
      }, modifier);                                                                                                    // 486
      sendEmail({                                                                                                      // 488
        to: emailAddress,                                                                                              // 489
        from: fromEmail,                                                                                               // 490
        replyTo: replyTo,                                                                                              // 491
        subject: grain.title + " - Request for access",                                                                // 492
        text: message + "\n\nFollow this link to share access:\n\n" + url,                                             // 493
        html: html                                                                                                     // 494
      });                                                                                                              // 488
    }                                                                                                                  // 496
  }                                                                                                                    // 497
});                                                                                                                    // 227
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"hack-session.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/hack-session.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");                          //
                                                                                                                       //
var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);                                 //
                                                                                                                       //
var _inherits2 = require("babel-runtime/helpers/inherits");                                                            //
                                                                                                                       //
var _inherits3 = _interopRequireDefault(_inherits2);                                                                   //
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var hashSturdyRef = void 0,                                                                                            // 1
    checkRequirements = void 0,                                                                                        // 1
    fetchApiToken = void 0;                                                                                            // 1
module.watch(require("/imports/server/persistent.js"), {                                                               // 1
  hashSturdyRef: function (v) {                                                                                        // 1
    hashSturdyRef = v;                                                                                                 // 1
  },                                                                                                                   // 1
  checkRequirements: function (v) {                                                                                    // 1
    checkRequirements = v;                                                                                             // 1
  },                                                                                                                   // 1
  fetchApiToken: function (v) {                                                                                        // 1
    fetchApiToken = v;                                                                                                 // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var inMeteor = void 0,                                                                                                 // 1
    waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  },                                                                                                                   // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var ssrfSafeLookup = void 0;                                                                                           // 1
module.watch(require("/imports/server/networking.js"), {                                                               // 1
  ssrfSafeLookup: function (v) {                                                                                       // 1
    ssrfSafeLookup = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var Crypto = Npm.require("crypto");                                                                                    // 17
                                                                                                                       //
var Http = Npm.require("http");                                                                                        // 18
                                                                                                                       //
var Https = Npm.require("https");                                                                                      // 19
                                                                                                                       //
var Net = Npm.require("net");                                                                                          // 20
                                                                                                                       //
var Dgram = Npm.require("dgram");                                                                                      // 21
                                                                                                                       //
var EmailRpc = Capnp.importSystem("sandstorm/email.capnp");                                                            // 26
var HackSessionContext = Capnp.importSystem("sandstorm/hack-session.capnp").HackSessionContext;                        // 27
var Supervisor = Capnp.importSystem("sandstorm/supervisor.capnp").Supervisor;                                          // 28
var SystemPersistent = Capnp.importSystem("sandstorm/supervisor.capnp").SystemPersistent;                              // 29
var IpRpc = Capnp.importSystem("sandstorm/ip.capnp");                                                                  // 30
var EmailSendPort = EmailRpc.EmailSendPort;                                                                            // 31
var Grain = Capnp.importSystem("sandstorm/grain.capnp");                                                               // 32
var Powerbox = Capnp.importSystem("sandstorm/powerbox.capnp");                                                         // 33
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 35
                                                                                                                       //
ROOT_URL = Url.parse(process.env.ROOT_URL);                                                                            // 37
HOSTNAME = ROOT_URL.hostname;                                                                                          // 38
                                                                                                                       //
SessionContextImpl = function () {                                                                                     // 40
  function SessionContextImpl(grainId, sessionId, accountId, tabId) {                                                  // 41
    (0, _classCallCheck3.default)(this, SessionContextImpl);                                                           // 41
    this.grainId = grainId;                                                                                            // 42
    this.sessionId = sessionId;                                                                                        // 43
    this.accountId = accountId;                                                                                        // 44
    this.tabId = tabId;                                                                                                // 45
  }                                                                                                                    // 46
                                                                                                                       //
  SessionContextImpl.prototype.claimRequest = function () {                                                            // 40
    function claimRequest(sturdyRef, requiredPermissions) {                                                            // 40
      var _this2 = this;                                                                                               // 48
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 49
        var token = fetchApiToken(globalDb, sturdyRef, {                                                               // 50
          "owner.clientPowerboxRequest.sessionId": _this2.sessionId                                                    // 51
        });                                                                                                            // 51
                                                                                                                       //
        if (!token) {                                                                                                  // 53
          throw new Error("no such token");                                                                            // 54
        }                                                                                                              // 55
                                                                                                                       //
        var session = Sessions.findOne({                                                                               // 57
          _id: _this2.sessionId                                                                                        // 57
        });                                                                                                            // 57
                                                                                                                       //
        if (!session) {                                                                                                // 59
          throw new Error("no such session");                                                                          // 60
        } // Honor `requiredPermissions`.                                                                              // 61
                                                                                                                       //
                                                                                                                       //
        var requirements = [];                                                                                         // 64
                                                                                                                       //
        if (session.hashedToken) {                                                                                     // 65
          // Session is authorized by token. Note that it's important to check this before accountId                   // 66
          // e.g. in the case of standalone domains.                                                                   // 67
          requirements.push({                                                                                          // 68
            permissionsHeld: {                                                                                         // 69
              permissions: requiredPermissions || [],                                                                  // 70
              tokenId: session.hashedToken,                                                                            // 71
              grainId: _this2.grainId                                                                                  // 72
            }                                                                                                          // 69
          });                                                                                                          // 68
        } else if (session.identityId) {                                                                               // 75
          // Session is authorized by identity. (We check session.identityId rather than                               // 76
          // session.userId because session.userId is set even for incognito sessions.)                                // 77
          requirements.push({                                                                                          // 78
            permissionsHeld: {                                                                                         // 79
              permissions: requiredPermissions || [],                                                                  // 80
              accountId: session.userId,                                                                               // 81
              grainId: _this2.grainId                                                                                  // 82
            }                                                                                                          // 79
          });                                                                                                          // 78
        } else {                                                                                                       // 85
          // This can only happen with old-style sharing which has been deprecated for years. If                       // 86
          // we wanted to support this, I suppose we could do so as long as requiredPermissions is                     // 87
          // all-false? But probably this will never come up.                                                          // 88
          throw new Error("Cannot accept powerbox request from anonymous session that " + "doesn't have a token.");    // 89
        }                                                                                                              // 91
                                                                                                                       //
        return restoreInternal(globalDb, sturdyRef, {                                                                  // 93
          clientPowerboxRequest: Match.ObjectIncluding({                                                               // 95
            sessionId: _this2.sessionId                                                                                // 95
          })                                                                                                           // 95
        }, requirements, token);                                                                                       // 95
      });                                                                                                              // 97
    }                                                                                                                  // 98
                                                                                                                       //
    return claimRequest;                                                                                               // 40
  }();                                                                                                                 // 40
                                                                                                                       //
  SessionContextImpl.prototype._offerOrFulfill = function () {                                                         // 40
    function _offerOrFulfill(isFulfill, cap, requiredPermissions, descriptor, displayInfo) {                           // 40
      var _this3 = this;                                                                                               // 100
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 101
        var session = Sessions.findOne({                                                                               // 103
          _id: _this3.sessionId                                                                                        // 103
        });                                                                                                            // 103
                                                                                                                       //
        if (!session.identityId && !session.hashedToken) {                                                             // 105
          throw new Error("Session has neither an identityId nor a hashedToken.");                                     // 106
        }                                                                                                              // 107
                                                                                                                       //
        if (isFulfill && !session.powerboxRequest) {                                                                   // 109
          // Not a request session, so treat fulfillRequest() same as offer().                                         // 110
          isFulfill = false;                                                                                           // 111
        }                                                                                                              // 112
                                                                                                                       //
        var castedCap = cap.castAs(SystemPersistent);                                                                  // 114
        var apiTokenOwner = {                                                                                          // 115
          clientPowerboxOffer: {                                                                                       // 115
            sessionId: _this3.sessionId                                                                                // 115
          }                                                                                                            // 115
        };                                                                                                             // 115
        var isUiView = descriptor && descriptor.tags && descriptor.tags.length === 1 && descriptor.tags[0] && descriptor.tags[0].id && descriptor.tags[0].id === Grain.UiView.typeId;
                                                                                                                       //
        if (isUiView) {                                                                                                // 119
          var tagValue = {};                                                                                           // 120
                                                                                                                       //
          if (descriptor.tags[0].value) {                                                                              // 121
            tagValue = Capnp.parse(Grain.UiView.PowerboxTag, descriptor.tags[0].value);                                // 122
          }                                                                                                            // 123
                                                                                                                       //
          if (!tagValue.title) {                                                                                       // 125
            throw new Error("No value provided for UiView.PowerboxTag.title.");                                        // 126
          }                                                                                                            // 127
                                                                                                                       //
          if (session.identityId) {                                                                                    // 129
            apiTokenOwner = {                                                                                          // 130
              user: {                                                                                                  // 131
                accountId: _this3.accountId,                                                                           // 132
                title: tagValue.title                                                                                  // 133
              }                                                                                                        // 131
            };                                                                                                         // 130
          }                                                                                                            // 136
        }                                                                                                              // 137
                                                                                                                       //
        if (isFulfill) {                                                                                               // 139
          // The capability will pass directly to the requesting grain.                                                // 140
          apiTokenOwner = {                                                                                            // 141
            clientPowerboxRequest: {                                                                                   // 142
              sessionId: session.powerboxRequest.requestingSession                                                     // 143
            }                                                                                                          // 142
          };                                                                                                           // 141
        } // TODO(soon): This will eventually use SystemPersistent.addRequirements when membranes                      // 146
        // are fully implemented for supervisors.                                                                      // 149
                                                                                                                       //
                                                                                                                       //
        var permissionsHeld = {                                                                                        // 150
          grainId: _this3.grainId,                                                                                     // 151
          permissions: requiredPermissions                                                                             // 152
        }; // Note that if both hashedToken and identityId are present, this indicates that the session                // 150
        // is authorized by a token which the user has NOT redeemed. Thus the identity does not                        // 156
        // directly have access to the grain; the token does. (This is the case in particular for                      // 157
        // standalone domains.) So clearly we want our requirement to be based on the token, not the                   // 158
        // identity.                                                                                                   // 159
                                                                                                                       //
        if (session.hashedToken) {                                                                                     // 160
          permissionsHeld.tokenId = session.hashedToken;                                                               // 161
        } else if (session.identityId) {                                                                               // 162
          permissionsHeld.accountId = session.userId;                                                                  // 163
        } else {                                                                                                       // 164
          throw new Error("Cannot offer to anonymous session that does not have a token.");                            // 165
        }                                                                                                              // 166
                                                                                                                       //
        var requirement = {                                                                                            // 168
          permissionsHeld: permissionsHeld                                                                             // 168
        };                                                                                                             // 168
        checkRequirements(globalDb, [requirement]);                                                                    // 170
        var save = castedCap.save(apiTokenOwner);                                                                      // 172
        var sturdyRef = waitPromise(save).sturdyRef;                                                                   // 173
        ApiTokens.update({                                                                                             // 174
          _id: hashSturdyRef(sturdyRef)                                                                                // 174
        }, {                                                                                                           // 174
          $push: {                                                                                                     // 174
            requirements: requirement                                                                                  // 174
          }                                                                                                            // 174
        });                                                                                                            // 174
        var powerboxView = void 0;                                                                                     // 176
                                                                                                                       //
        if (isFulfill) {                                                                                               // 177
          powerboxView = {                                                                                             // 178
            fulfill: {                                                                                                 // 179
              token: sturdyRef.toString(),                                                                             // 180
              descriptor: Capnp.serializePacked(Powerbox.PowerboxDescriptor, descriptor).toString("base64")            // 181
            }                                                                                                          // 179
          };                                                                                                           // 178
        } else if (isUiView) {                                                                                         // 185
          if (session.identityId) {                                                                                    // 186
            // Deduplicate.                                                                                            // 187
            var newApiToken = fetchApiToken(globalDb, sturdyRef.toString());                                           // 188
            var tokenId = newApiToken._id;                                                                             // 189
                                                                                                                       //
            var dupeQuery = _.pick(newApiToken, "grainId", "roleAssignment", "requirements", "parentToken", "parentTokenKey", "accountId", "accountId");
                                                                                                                       //
            dupeQuery._id = {                                                                                          // 192
              $ne: newApiToken._id                                                                                     // 192
            };                                                                                                         // 192
            dupeQuery["owner.user.accountId"] = _this3.accountId;                                                      // 193
            dupeQuery.trashed = {                                                                                      // 194
              $exists: false                                                                                           // 194
            };                                                                                                         // 194
            dupeQuery.revoked = {                                                                                      // 195
              $exists: false                                                                                           // 195
            };                                                                                                         // 195
            var dupeToken = ApiTokens.findOne(dupeQuery);                                                              // 197
                                                                                                                       //
            if (dupeToken) {                                                                                           // 198
              globalDb.removeApiTokens({                                                                               // 199
                _id: tokenId                                                                                           // 199
              });                                                                                                      // 199
              tokenId = dupeToken._id;                                                                                 // 200
            }                                                                                                          // 201
                                                                                                                       //
            powerboxView = {                                                                                           // 203
              offer: {                                                                                                 // 203
                uiView: {                                                                                              // 203
                  tokenId: tokenId                                                                                     // 203
                }                                                                                                      // 203
              }                                                                                                        // 203
            };                                                                                                         // 203
          } else {                                                                                                     // 204
            powerboxView = {                                                                                           // 205
              offer: {                                                                                                 // 205
                uiView: {                                                                                              // 205
                  token: sturdyRef.toString()                                                                          // 205
                }                                                                                                      // 205
              }                                                                                                        // 205
            };                                                                                                         // 205
          }                                                                                                            // 206
        } else {                                                                                                       // 207
          powerboxView = {                                                                                             // 208
            offer: {                                                                                                   // 209
              token: sturdyRef.toString()                                                                              // 210
            }                                                                                                          // 209
          };                                                                                                           // 208
        }                                                                                                              // 213
                                                                                                                       //
        Sessions.update({                                                                                              // 215
          _id: _this3.sessionId                                                                                        // 215
        }, {                                                                                                           // 215
          $set: {                                                                                                      // 217
            powerboxView: powerboxView                                                                                 // 218
          }                                                                                                            // 217
        });                                                                                                            // 216
      });                                                                                                              // 222
    }                                                                                                                  // 223
                                                                                                                       //
    return _offerOrFulfill;                                                                                            // 40
  }();                                                                                                                 // 40
                                                                                                                       //
  SessionContextImpl.prototype.offer = function () {                                                                   // 40
    function offer(cap, requiredPermissions, descriptor, displayInfo) {                                                // 40
      return this._offerOrFulfill(false, cap, requiredPermissions, descriptor, displayInfo);                           // 226
    }                                                                                                                  // 227
                                                                                                                       //
    return offer;                                                                                                      // 40
  }();                                                                                                                 // 40
                                                                                                                       //
  SessionContextImpl.prototype.fulfillRequest = function () {                                                          // 40
    function fulfillRequest(cap, requiredPermissions, descriptor, displayInfo) {                                       // 40
      return this._offerOrFulfill(true, cap, requiredPermissions, descriptor, displayInfo);                            // 230
    }                                                                                                                  // 231
                                                                                                                       //
    return fulfillRequest;                                                                                             // 40
  }();                                                                                                                 // 40
                                                                                                                       //
  SessionContextImpl.prototype.activity = function () {                                                                // 40
    function activity(event) {                                                                                         // 40
      var _this4 = this;                                                                                               // 233
                                                                                                                       //
      return inMeteor(function () {                                                                                    // 234
        logActivity(_this4.grainId, _this4.accountId || "anonymous", event);                                           // 235
      });                                                                                                              // 236
    }                                                                                                                  // 237
                                                                                                                       //
    return activity;                                                                                                   // 40
  }();                                                                                                                 // 40
                                                                                                                       //
  return SessionContextImpl;                                                                                           // 40
}();                                                                                                                   // 40
                                                                                                                       //
Meteor.methods({                                                                                                       // 240
  finishPowerboxRequest: function (sessionId, webkeyUrl, saveLabel, obsolete, grainId) {                               // 241
    check(sessionId, String);                                                                                          // 242
    check(webkeyUrl, String);                                                                                          // 243
    check(saveLabel, Match.OneOf(undefined, null, String));                                                            // 244
    check(grainId, String);                                                                                            // 245
    var db = this.connection.sandstormDb;                                                                              // 247
    var userId = Meteor.userId();                                                                                      // 249
                                                                                                                       //
    if (!userId) {                                                                                                     // 250
      throw new Meteor.Error(403, "Not logged in.");                                                                   // 251
    }                                                                                                                  // 252
                                                                                                                       //
    var parsedWebkey = Url.parse(webkeyUrl.trim());                                                                    // 254
    var hostId = matchWildcardHost(parsedWebkey.host);                                                                 // 255
                                                                                                                       //
    if (!hostId || !db.isApiHostId(hostId)) {                                                                          // 256
      throw new Meteor.Error(500, "Hostname does not match this server. External webkeys are not " + "supported (yet)");
    }                                                                                                                  // 259
                                                                                                                       //
    var token = parsedWebkey.hash;                                                                                     // 261
                                                                                                                       //
    if (!token) {                                                                                                      // 262
      throw new Meteor.Error(400, "Invalid webkey. You must pass in the full webkey, " + "including domain name and hash fragment");
    }                                                                                                                  // 265
                                                                                                                       //
    token = token.slice(1);                                                                                            // 267
                                                                                                                       //
    if (db.isTokenSpecificHostId(hostId) && hostId !== db.apiHostIdForToken(token)) {                                  // 268
      // Note: This case is not security-sensitive. The client could easily compute the correct                        // 269
      //   host ID for this token. But since they've passed one that doesn't match, we assume                          // 270
      //   something is wrong and stop here.                                                                           // 271
      throw new Meteor.Error(400, "Invalid webkey: token doesn't match hostname.");                                    // 272
    }                                                                                                                  // 273
                                                                                                                       //
    var cap = restoreInternal(db, token, Match.Optional({                                                              // 275
      webkey: Match.Optional(Match.Any)                                                                                // 276
    }), []).cap;                                                                                                       // 276
    var castedCap = cap.castAs(SystemPersistent);                                                                      // 277
    var owner = {                                                                                                      // 278
      clientPowerboxRequest: {                                                                                         // 279
        grainId: grainId,                                                                                              // 280
        sessionId: sessionId                                                                                           // 281
      }                                                                                                                // 279
    };                                                                                                                 // 278
                                                                                                                       //
    if (saveLabel) {                                                                                                   // 284
      grainOwner.saveLabel = {                                                                                         // 285
        defaultText: saveLabel                                                                                         // 286
      };                                                                                                               // 285
    }                                                                                                                  // 288
                                                                                                                       //
    var save = castedCap.save(owner);                                                                                  // 290
    var sturdyRef = waitPromise(save).sturdyRef;                                                                       // 291
    return sturdyRef.toString();                                                                                       // 292
  },                                                                                                                   // 293
  finishPowerboxOffer: function (sessionId) {                                                                          // 295
    check(sessionId, String);                                                                                          // 296
    Sessions.update({                                                                                                  // 298
      _id: sessionId                                                                                                   // 298
    }, {                                                                                                               // 298
      $unset: {                                                                                                        // 298
        powerboxView: null                                                                                             // 298
      }                                                                                                                // 298
    });                                                                                                                // 298
  },                                                                                                                   // 299
  getViewInfoForApiToken: function (apiTokenId) {                                                                      // 301
    check(apiTokenId, String);                                                                                         // 302
    var token = globalDb.collections.apiTokens.findOne(apiTokenId);                                                    // 303
                                                                                                                       //
    if (!token) {                                                                                                      // 304
      throw new Meteor.Error(400, "No such token.");                                                                   // 305
    }                                                                                                                  // 306
                                                                                                                       //
    var grain = globalDb.collections.grains.findOne(token.grainId);                                                    // 308
                                                                                                                       //
    if (!grain) {                                                                                                      // 309
      throw new Meteor.Error(500, "No grain found for token.");                                                        // 310
    }                                                                                                                  // 311
                                                                                                                       //
    return grain.cachedViewInfo;                                                                                       // 313
  }                                                                                                                    // 314
});                                                                                                                    // 240
                                                                                                                       //
HackSessionContextImpl = function (_SessionContextImpl) {                                                              // 317
  (0, _inherits3.default)(HackSessionContextImpl, _SessionContextImpl);                                                // 317
                                                                                                                       //
  function HackSessionContextImpl(grainId, sessionId, accountId, tabId) {                                              // 318
    (0, _classCallCheck3.default)(this, HackSessionContextImpl);                                                       // 318
    return (0, _possibleConstructorReturn3.default)(this, _SessionContextImpl.call(this, grainId, sessionId, accountId, tabId));
  }                                                                                                                    // 320
                                                                                                                       //
  HackSessionContextImpl.prototype._getPublicId = function () {                                                        // 317
    function _getPublicId() {                                                                                          // 317
      // Get the grain's public ID, assigning a new one if it doesn't yet have one.                                    // 323
      //                                                                                                               // 324
      // Must be called in a Meteor context.                                                                           // 325
      while (!this.publicId) {                                                                                         // 327
        // We haven't looked up the public ID yet.                                                                     // 328
        var grain = Grains.findOne(this.grainId, {                                                                     // 329
          fields: {                                                                                                    // 329
            publicId: 1                                                                                                // 329
          }                                                                                                            // 329
        });                                                                                                            // 329
        if (!grain) throw new Error("Grain does not exist.");                                                          // 330
                                                                                                                       //
        if (grain.publicId) {                                                                                          // 332
          this.publicId = grain.publicId;                                                                              // 333
        } else {                                                                                                       // 334
          // The grain doesn't have a public ID yet. Generate one.                                                     // 335
          var candidate = generateRandomHostname(20);                                                                  // 336
                                                                                                                       //
          if (Grains.findOne({                                                                                         // 338
            publicId: candidate                                                                                        // 338
          })) {                                                                                                        // 338
            // This should never ever happen.                                                                          // 339
            console.error("CRITICAL PROBLEM: Public ID collision. " + "CSPRNG is bad or has insufficient entropy.");   // 340
            continue;                                                                                                  // 342
          } // Carefully perform an update that becomes a no-op if anyone else has assigned a public ID                // 343
          // simultaneously.                                                                                           // 346
                                                                                                                       //
                                                                                                                       //
          if (Grains.update({                                                                                          // 347
            _id: this.grainId,                                                                                         // 347
            publicId: {                                                                                                // 347
              $exists: false                                                                                           // 347
            }                                                                                                          // 347
          }, {                                                                                                         // 347
            $set: {                                                                                                    // 348
              publicId: candidate                                                                                      // 348
            }                                                                                                          // 348
          }) > 0) {                                                                                                    // 348
            // We won the race.                                                                                        // 349
            this.publicId = candidate;                                                                                 // 350
          }                                                                                                            // 351
        }                                                                                                              // 352
      }                                                                                                                // 353
                                                                                                                       //
      return this.publicId;                                                                                            // 355
    }                                                                                                                  // 356
                                                                                                                       //
    return _getPublicId;                                                                                               // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype._getAddress = function () {                                                         // 317
    function _getAddress() {                                                                                           // 317
      // Get the grain's outgoing e-mail address.                                                                      // 359
      //                                                                                                               // 360
      // Must be called in a Meteor context.                                                                           // 361
      return this._getPublicId() + "@" + HOSTNAME;                                                                     // 363
    }                                                                                                                  // 364
                                                                                                                       //
    return _getAddress;                                                                                                // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype._getUserAddress = function () {                                                     // 317
    function _getUserAddress() {                                                                                       // 317
      // Get the user's e-mail address.                                                                                // 367
      //                                                                                                               // 368
      // Must be called in a Meteor context.                                                                           // 369
      var grain = Grains.findOne(this.grainId, {                                                                       // 371
        fields: {                                                                                                      // 371
          userId: 1                                                                                                    // 371
        }                                                                                                              // 371
      });                                                                                                              // 371
      var user = Meteor.users.findOne({                                                                                // 373
        _id: grain.userId                                                                                              // 373
      });                                                                                                              // 373
                                                                                                                       //
      var email = _.findWhere(SandstormDb.getUserEmails(user), {                                                       // 375
        primary: true                                                                                                  // 375
      });                                                                                                              // 375
                                                                                                                       //
      var result = {};                                                                                                 // 377
                                                                                                                       //
      if (email) {                                                                                                     // 378
        result.address = email.email;                                                                                  // 379
      }                                                                                                                // 380
                                                                                                                       //
      if (user.profile.name) {                                                                                         // 382
        result.name = user.profile.name;                                                                               // 383
      }                                                                                                                // 384
                                                                                                                       //
      return result;                                                                                                   // 386
    }                                                                                                                  // 387
                                                                                                                       //
    return _getUserAddress;                                                                                            // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.send = function () {                                                                // 317
    function send(email) {                                                                                             // 317
      return hackSendEmail(this, email);                                                                               // 390
    }                                                                                                                  // 391
                                                                                                                       //
    return send;                                                                                                       // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.getPublicId = function () {                                                         // 317
    function getPublicId() {                                                                                           // 317
      return inMeteor(function () {                                                                                    // 394
        var result = {};                                                                                               // 395
        result.publicId = this._getPublicId();                                                                         // 397
        result.hostname = HOSTNAME;                                                                                    // 398
        result.autoUrl = ROOT_URL.protocol + "//" + makeWildcardHost(result.publicId);                                 // 399
        var grain = Grains.findOne(this.grainId, {                                                                     // 401
          fields: {                                                                                                    // 401
            userId: 1                                                                                                  // 401
          }                                                                                                            // 401
        });                                                                                                            // 401
        result.isDemoUser = Meteor.users.findOne(grain.userId).expires ? true : false;                                 // 402
        return result;                                                                                                 // 404
      }.bind(this));                                                                                                   // 405
    }                                                                                                                  // 406
                                                                                                                       //
    return getPublicId;                                                                                                // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.httpGet = function () {                                                             // 317
    function httpGet(url) {                                                                                            // 317
      var _this = this;                                                                                                // 409
                                                                                                                       //
      var session = _this;                                                                                             // 410
      return inMeteor(function () {                                                                                    // 412
        return ssrfSafeLookup(globalDb, url);                                                                          // 413
      }).then(function (safe) {                                                                                        // 414
        return new Promise(function (resolve, reject) {                                                                // 415
          var requestMethod = Http.request;                                                                            // 416
                                                                                                                       //
          if (safe.url.indexOf("https://") === 0) {                                                                    // 417
            requestMethod = Https.request;                                                                             // 418
          } else if (safe.url.indexOf("http://") !== 0) {                                                              // 419
            err = new Error("Protocol not recognized.");                                                               // 420
            err.nature = "precondition";                                                                               // 421
            reject(err);                                                                                               // 422
          }                                                                                                            // 423
                                                                                                                       //
          var options = Url.parse(safe.url);                                                                           // 425
          options.headers = {                                                                                          // 426
            host: safe.host                                                                                            // 426
          };                                                                                                           // 426
          options.servername = safe.host.split(":")[0];                                                                // 427
          req = requestMethod(options, function (resp) {                                                               // 429
            var buffers = [];                                                                                          // 430
            var err = void 0;                                                                                          // 431
                                                                                                                       //
            switch (Math.floor(resp.statusCode / 100)) {                                                               // 433
              case 2:                                                                                                  // 434
                // 2xx response -- OK.                                                                                 // 434
                resp.on("data", function (buf) {                                                                       // 435
                  buffers.push(buf);                                                                                   // 436
                });                                                                                                    // 437
                resp.on("end", function () {                                                                           // 439
                  resolve({                                                                                            // 440
                    content: Buffer.concat(buffers),                                                                   // 441
                    mimeType: resp.headers["content-type"] || null                                                     // 442
                  });                                                                                                  // 440
                });                                                                                                    // 444
                break;                                                                                                 // 445
                                                                                                                       //
              case 3:                                                                                                  // 446
                // 3xx response -- redirect.                                                                           // 446
                resolve(session.httpGet(resp.headers.location));                                                       // 447
                break;                                                                                                 // 448
                                                                                                                       //
              case 4:                                                                                                  // 449
                // 4xx response -- client error.                                                                       // 449
                err = new Error("Status code " + resp.statusCode + " received in response.");                          // 450
                err.nature = "precondition";                                                                           // 451
                reject(err);                                                                                           // 452
                break;                                                                                                 // 453
                                                                                                                       //
              case 5:                                                                                                  // 454
                // 5xx response -- internal server error.                                                              // 454
                err = new Error("Status code " + resp.statusCode + " received in response.");                          // 455
                err.nature = "localBug";                                                                               // 456
                reject(err);                                                                                           // 457
                break;                                                                                                 // 458
                                                                                                                       //
              default:                                                                                                 // 459
                // ???                                                                                                 // 459
                err = new Error("Invalid status code " + resp.statusCode + " received in response.");                  // 460
                err.nature = "localBug";                                                                               // 461
                reject(err);                                                                                           // 462
                break;                                                                                                 // 463
            }                                                                                                          // 433
          });                                                                                                          // 465
          req.on("error", function (e) {                                                                               // 467
            e.nature = "networkFailure";                                                                               // 468
            reject(e);                                                                                                 // 469
          });                                                                                                          // 470
          req.setTimeout(15000, function () {                                                                          // 472
            req.abort();                                                                                               // 473
            err = new Error("Request timed out.");                                                                     // 474
            err.nature = "localBug";                                                                                   // 475
            err.durability = "overloaded";                                                                             // 476
            reject(err);                                                                                               // 477
          });                                                                                                          // 478
          req.end();                                                                                                   // 480
        });                                                                                                            // 481
      });                                                                                                              // 482
    }                                                                                                                  // 483
                                                                                                                       //
    return httpGet;                                                                                                    // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.getUserAddress = function () {                                                      // 317
    function getUserAddress() {                                                                                        // 317
      return inMeteor(function () {                                                                                    // 486
        return this._getUserAddress();                                                                                 // 487
      }.bind(this));                                                                                                   // 488
    }                                                                                                                  // 489
                                                                                                                       //
    return getUserAddress;                                                                                             // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.obsoleteGenerateApiToken = function () {                                            // 317
    function obsoleteGenerateApiToken(petname, userInfo, expires) {                                                    // 317
      throw new Error("generateApiToken() has been removed. Use offer templates instead.");                            // 492
    }                                                                                                                  // 493
                                                                                                                       //
    return obsoleteGenerateApiToken;                                                                                   // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.obsoleteListApiTokens = function () {                                               // 317
    function obsoleteListApiTokens() {                                                                                 // 317
      throw new Error("listApiTokens() has been removed. Use offer templates instead.");                               // 496
    }                                                                                                                  // 497
                                                                                                                       //
    return obsoleteListApiTokens;                                                                                      // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.obsoleteRevokeApiToken = function () {                                              // 317
    function obsoleteRevokeApiToken(tokenId) {                                                                         // 317
      throw new Error("revokeApiToken() has been removed. Use offer templates instead.");                              // 500
    }                                                                                                                  // 501
                                                                                                                       //
    return obsoleteRevokeApiToken;                                                                                     // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  HackSessionContextImpl.prototype.getUiViewForEndpoint = function () {                                                // 317
    function getUiViewForEndpoint(url) {                                                                               // 317
      var parsedUrl = Url.parse(url);                                                                                  // 504
                                                                                                                       //
      if (parsedUrl.hash) {                                                                                            // 506
        // Assume that anything with a fragment is a webkey                                                            // 506
        if (parsedUrl.pathname && parsedUrl.pathname !== "/") {                                                        // 507
          throw new Error("Webkey urls cannot contain a path.");                                                       // 508
        }                                                                                                              // 509
                                                                                                                       //
        var token = parsedUrl.hash.slice(1); // Get rid of # which is always the first character                       // 511
                                                                                                                       //
        var hostId = matchWildcardHost(parsedUrl.host); // Connecting to a remote server with a bearer token.          // 512
        // TODO(someday): Negotiate server-to-server Cap'n Proto connection.                                           // 514
                                                                                                                       //
        return {                                                                                                       // 515
          view: new ExternalUiView(url, token)                                                                         // 515
        };                                                                                                             // 515
      } else {                                                                                                         // 516
        return {                                                                                                       // 517
          view: new ExternalUiView(url)                                                                                // 517
        };                                                                                                             // 517
      }                                                                                                                // 518
    }                                                                                                                  // 519
                                                                                                                       //
    return getUiViewForEndpoint;                                                                                       // 317
  }();                                                                                                                 // 317
                                                                                                                       //
  return HackSessionContextImpl;                                                                                       // 317
}(SessionContextImpl);                                                                                                 // 317
                                                                                                                       //
makeHackSessionContext = function (grainId, sessionId, accountId, tabId) {                                             // 522
  return new Capnp.Capability(new HackSessionContextImpl(grainId, sessionId, accountId, tabId), HackSessionContext);   // 523
};                                                                                                                     // 525
                                                                                                                       //
var HOSTNAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789";                                                           // 527
                                                                                                                       //
generateRandomHostname = function (length) {                                                                           // 529
  // Generate a random unique name suitable for use in a hostname.                                                     // 530
  var digits = [];                                                                                                     // 532
                                                                                                                       //
  for (var i = 0; i < length; i++) {                                                                                   // 533
    digits[i] = Random.choice(HOSTNAME_CHARS);                                                                         // 534
  }                                                                                                                    // 535
                                                                                                                       //
  return digits.join("");                                                                                              // 537
};                                                                                                                     // 538
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"identity.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/identity.js                                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");                          //
                                                                                                                       //
var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);                                 //
                                                                                                                       //
var _inherits2 = require("babel-runtime/helpers/inherits");                                                            //
                                                                                                                       //
var _inherits3 = _interopRequireDefault(_inherits2);                                                                   //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var PersistentImpl = void 0;                                                                                           // 1
module.watch(require("/imports/server/persistent.js"), {                                                               // 1
  PersistentImpl: function (v) {                                                                                       // 1
    PersistentImpl = v;                                                                                                // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var StaticAssetImpl = void 0,                                                                                          // 1
    IdenticonStaticAssetImpl = void 0;                                                                                 // 1
module.watch(require("/imports/server/static-asset.js"), {                                                             // 1
  StaticAssetImpl: function (v) {                                                                                      // 1
    StaticAssetImpl = v;                                                                                               // 1
  },                                                                                                                   // 1
  IdenticonStaticAssetImpl: function (v) {                                                                             // 1
    IdenticonStaticAssetImpl = v;                                                                                      // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var IdentityRpc = Capnp.importSystem("sandstorm/identity-impl.capnp");                                                 // 17
var Identity = Capnp.importSystem("sandstorm/identity.capnp").Identity;                                                // 18
var StaticAsset = Capnp.importSystem("sandstorm/util.capnp").StaticAsset;                                              // 21
                                                                                                                       //
var IdentityImpl = function (_PersistentImpl) {                                                                        //
  (0, _inherits3.default)(IdentityImpl, _PersistentImpl);                                                              //
                                                                                                                       //
  function IdentityImpl(db, saveTemplate, accountId) {                                                                 // 24
    (0, _classCallCheck3.default)(this, IdentityImpl);                                                                 // 24
                                                                                                                       //
    var _this = (0, _possibleConstructorReturn3.default)(this, _PersistentImpl.call(this, db, saveTemplate));          // 24
                                                                                                                       //
    _this.accountId = accountId;                                                                                       // 26
    _this.db = db;                                                                                                     // 27
    return _this;                                                                                                      // 24
  }                                                                                                                    // 28
                                                                                                                       //
  IdentityImpl.prototype.getProfile = function () {                                                                    //
    function getProfile() {                                                                                            //
      var user = Meteor.users.findOne({                                                                                // 31
        _id: this.accountId                                                                                            // 31
      });                                                                                                              // 31
      var profile = {                                                                                                  // 33
        displayName: {                                                                                                 // 34
          defaultText: user.profile.name                                                                               // 34
        },                                                                                                             // 34
        preferredHandle: user.profile.handle,                                                                          // 35
        pronouns: user.profile.pronoun                                                                                 // 36
      };                                                                                                               // 33
                                                                                                                       //
      if (user.profile.picture) {                                                                                      // 39
        profile.picture = new Capnp.Capability(new StaticAssetImpl(user.profile.picture), StaticAsset);                // 40
      } else {                                                                                                         // 42
        var hash = user.profile.identicon;                                                                             // 43
        profile.picture = new Capnp.Capability(new IdenticonStaticAssetImpl(hash, 24), StaticAsset);                   // 44
      }                                                                                                                // 46
                                                                                                                       //
      return {                                                                                                         // 48
        profile: profile                                                                                               // 48
      };                                                                                                               // 48
    }                                                                                                                  // 49
                                                                                                                       //
    return getProfile;                                                                                                 //
  }();                                                                                                                 //
                                                                                                                       //
  return IdentityImpl;                                                                                                 //
}(PersistentImpl);                                                                                                     //
                                                                                                                       //
; // TODO(cleanup): Find a better home for this.                                                                       // 50
                                                                                                                       //
var MembraneRequirement = Match.OneOf({                                                                                // 53
  tokenValid: String                                                                                                   // 54
}, {                                                                                                                   // 54
  permissionsHeld: {                                                                                                   // 55
    accountId: String,                                                                                                 // 56
    grainId: String,                                                                                                   // 56
    permissions: Match.Optional([Boolean])                                                                             // 56
  }                                                                                                                    // 56
}, {                                                                                                                   // 55
  permissionsHeld: {                                                                                                   // 57
    tokenId: String,                                                                                                   // 58
    grainId: String,                                                                                                   // 58
    permissions: Match.Optional([Boolean])                                                                             // 58
  }                                                                                                                    // 58
}, {                                                                                                                   // 57
  userIsAdmin: String                                                                                                  // 59
});                                                                                                                    // 59
                                                                                                                       //
makeIdentity = function (accountId, requirements) {                                                                    // 61
  var saveTemplate = {                                                                                                 // 62
    frontendRef: {                                                                                                     // 62
      identity: accountId                                                                                              // 62
    }                                                                                                                  // 62
  };                                                                                                                   // 62
                                                                                                                       //
  if (requirements) {                                                                                                  // 63
    check(requirements, [MembraneRequirement]);                                                                        // 64
    saveTemplate.requirements = requirements;                                                                          // 65
  }                                                                                                                    // 66
                                                                                                                       //
  return new Capnp.Capability(new IdentityImpl(globalDb, saveTemplate, accountId), IdentityRpc.PersistentIdentity);    // 68
};                                                                                                                     // 70
                                                                                                                       //
globalFrontendRefRegistry.register({                                                                                   // 72
  frontendRefField: "identity",                                                                                        // 73
  typeId: Identity.typeId,                                                                                             // 74
  restore: function (db, saveTemplate, accountId) {                                                                    // 76
    return new Capnp.Capability(new IdentityImpl(db, saveTemplate, accountId), IdentityRpc.PersistentIdentity);        // 77
  },                                                                                                                   // 79
  validate: function (db, session, value) {                                                                            // 81
    check(value, {                                                                                                     // 82
      id: String,                                                                                                      // 82
      roleAssignment: SandstormDb.prototype.roleAssignmentPattern                                                      // 82
    });                                                                                                                // 82
                                                                                                                       //
    if (!session.userId) {                                                                                             // 84
      throw new Meteor.Error(403, "Not logged in.");                                                                   // 85
    }                                                                                                                  // 86
                                                                                                                       //
    var grain = db.getGrain(session.grainId);                                                                          // 88
    SandstormPermissions.createNewApiToken(db, {                                                                       // 89
      accountId: session.userId                                                                                        // 91
    }, session.grainId, "petname", value.roleAssignment, {                                                             // 91
      user: {                                                                                                          // 95
        accountId: value.id,                                                                                           // 95
        title: grain.title                                                                                             // 95
      }                                                                                                                // 95
    }); // TODO(soon): Somehow notify this user that they now have access.                                             // 95
    // TODO(perf): This permissions computation happens once here and then once again when the                         // 99
    //   `permissionsHeld` requirement is checked. Is there a way to avoid the duplicated work?                        // 100
                                                                                                                       //
    var permissions = SandstormPermissions.grainPermissions(db, {                                                      // 101
      grain: {                                                                                                         // 102
        _id: session.grainId,                                                                                          // 102
        accountId: value.id                                                                                            // 102
      }                                                                                                                // 102
    }, session.viewInfo || {}).permissions;                                                                            // 102
    return {                                                                                                           // 105
      descriptor: {                                                                                                    // 106
        tags: [{                                                                                                       // 107
          id: Identity.typeId,                                                                                         // 109
          value: Capnp.serialize(Identity.PowerboxTag, {                                                               // 110
            permissions: permissions                                                                                   // 113
          })                                                                                                           // 112
        }]                                                                                                             // 108
      },                                                                                                               // 106
      requirements: [{                                                                                                 // 118
        permissionsHeld: {                                                                                             // 120
          accountId: value.id,                                                                                         // 121
          grainId: session.grainId,                                                                                    // 122
          permissions: []                                                                                              // 123
        }                                                                                                              // 120
      }],                                                                                                              // 119
      frontendRef: value.id                                                                                            // 127
    };                                                                                                                 // 105
  },                                                                                                                   // 129
  query: function (db, userId, value) {                                                                                // 131
    var resultSet = {};                                                                                                // 132
    var requestedPermissions = [];                                                                                     // 133
                                                                                                                       //
    if (value) {                                                                                                       // 134
      requestedPermissions = Capnp.parse(Identity.PowerboxTag, value).permissions || [];                               // 135
    }                                                                                                                  // 136
                                                                                                                       //
    var resultForUser = function (user) {                                                                              // 138
      SandstormDb.fillInPictureUrl(user);                                                                              // 139
      return {                                                                                                         // 140
        _id: "frontendref-identity-" + user._id,                                                                       // 141
        frontendRef: {                                                                                                 // 142
          identity: user._id                                                                                           // 142
        },                                                                                                             // 142
        cardTemplate: "identityPowerboxCard",                                                                          // 143
        configureTemplate: "identityPowerboxConfiguration",                                                            // 144
        profile: user.profile,                                                                                         // 145
        requestedPermissions: requestedPermissions,                                                                    // 146
        searchTerms: [user.profile.name, user.profile.handle]                                                          // 147
      };                                                                                                               // 140
    };                                                                                                                 // 153
                                                                                                                       //
    db.collections.contacts.find({                                                                                     // 155
      ownerId: userId                                                                                                  // 155
    }).forEach(function (contact) {                                                                                    // 155
      var user = Meteor.users.findOne({                                                                                // 156
        _id: contact.accountId                                                                                         // 156
      });                                                                                                              // 156
                                                                                                                       //
      if (user) {                                                                                                      // 157
        resultSet[user._id] = resultForUser(user);                                                                     // 158
      }                                                                                                                // 159
    });                                                                                                                // 160
                                                                                                                       //
    if (db.getOrganizationShareContacts() && db.isUserInOrganization(db.collections.users.findOne({                    // 162
      _id: userId                                                                                                      // 163
    }))) {                                                                                                             // 163
      // TODO(perf): Add some way to efficiently fetch all members in an organization.                                 // 165
      db.collections.users.find({                                                                                      // 166
        type: "credential"                                                                                             // 166
      }).forEach(function (credential) {                                                                               // 166
        if (db.isCredentialInOrganization(credential)) {                                                               // 167
          var user = Meteor.users.findOne({                                                                            // 168
            "loginCredentials.id": credential._id                                                                      // 168
          });                                                                                                          // 168
                                                                                                                       //
          if (user) {                                                                                                  // 169
            resultSet[user._id] = resultForUser(user);                                                                 // 170
          }                                                                                                            // 171
        }                                                                                                              // 172
      });                                                                                                              // 173
    }                                                                                                                  // 174
                                                                                                                       //
    return _.values(resultSet);                                                                                        // 176
  }                                                                                                                    // 177
});                                                                                                                    // 72
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"install-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/install-server.js                                                                                            //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var allowDemo = void 0;                                                                                                // 1
module.watch(require("/imports/demo.js"), {                                                                            // 1
  allowDemo: function (v) {                                                                                            // 1
    allowDemo = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var promiseToFuture = void 0;                                                                                          // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  promiseToFuture: function (v) {                                                                                      // 1
    promiseToFuture = v;                                                                                               // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var TOKEN_CLEANUP_MINUTES = 120; // Give enough time for large uploads on slow connections.                            // 20
                                                                                                                       //
var TOKEN_CLEANUP_TIMER = TOKEN_CLEANUP_MINUTES * 60 * 1000;                                                           // 21
                                                                                                                       //
function cleanupToken(tokenId) {                                                                                       // 23
  check(tokenId, String);                                                                                              // 24
  globalDb.collections.spkTokens.remove({                                                                              // 25
    _id: tokenId                                                                                                       // 25
  });                                                                                                                  // 25
  waitPromise(globalBackend.cap().deleteBackup(tokenId));                                                              // 26
}                                                                                                                      // 27
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 29
  // Cleanup tokens every TOKEN_CLEANUP_MINUTES                                                                        // 30
  SandstormDb.periodicCleanup(TOKEN_CLEANUP_TIMER, function () {                                                       // 31
    var queryDate = new Date(Date.now() - TOKEN_CLEANUP_TIMER);                                                        // 32
    globalDb.collections.spkTokens.find({                                                                              // 34
      timestamp: {                                                                                                     // 34
        $lt: queryDate                                                                                                 // 34
      }                                                                                                                // 34
    }).forEach(function (token) {                                                                                      // 34
      cleanupToken(token._id);                                                                                         // 35
    });                                                                                                                // 36
  });                                                                                                                  // 37
});                                                                                                                    // 38
var localizedTextPattern = {                                                                                           // 40
  defaultText: String,                                                                                                 // 41
  localizations: Match.Optional([{                                                                                     // 42
    locale: String,                                                                                                    // 42
    text: String                                                                                                       // 42
  }])                                                                                                                  // 42
}; // Not all users are allowed to upload apps. We need to manually implement authorization                            // 40
// because Meteor.userId() is not available in server-side routes.                                                     // 46
                                                                                                                       //
Meteor.methods({                                                                                                       // 48
  cancelDownload: function (packageId) {                                                                               // 49
    check(packageId, String); // TODO(security):  Only let user cancel download if they initiated it.                  // 50
                                                                                                                       //
    cancelDownload(packageId);                                                                                         // 53
  },                                                                                                                   // 54
  newUploadToken: function () {                                                                                        // 56
    if (!isSignedUp()) {                                                                                               // 57
      throw new Meteor.Error(403, "Unauthorized", "Only invited users can upload apps.");                              // 58
    }                                                                                                                  // 59
                                                                                                                       //
    if (globalDb.isUninvitedFreeUser()) {                                                                              // 61
      throw new Meteor.Error(403, "Unauthorized", "Only paid users can upload apps.");                                 // 62
    }                                                                                                                  // 63
                                                                                                                       //
    var token = Random.id(22);                                                                                         // 65
    globalDb.collections.spkTokens.insert({                                                                            // 66
      _id: token,                                                                                                      // 66
      timestamp: new Date()                                                                                            // 66
    });                                                                                                                // 66
    return token;                                                                                                      // 68
  },                                                                                                                   // 69
  upgradeGrains: function (appId, version, packageId) {                                                                // 71
    this.connection.sandstormDb.upgradeGrains(appId, version, packageId, globalBackend);                               // 72
  },                                                                                                                   // 73
  ensureInstalled: function (packageId, url, isRetry) {                                                                // 75
    check(packageId, String);                                                                                          // 76
    check(url, Match.OneOf(String, undefined, null));                                                                  // 77
    check(isRetry, Boolean);                                                                                           // 78
                                                                                                                       //
    if (!packageId.match(/^[a-zA-Z0-9]*$/)) {                                                                          // 80
      throw new Meteor.Error(400, "The package ID contains illegal characters.");                                      // 81
    }                                                                                                                  // 82
                                                                                                                       //
    if (!this.userId) {                                                                                                // 84
      if (allowDemo && isSafeDemoAppUrl(url)) {// continue on                                                          // 85
      } else {                                                                                                         // 87
        // jscs:ignore disallowEmptyBlocks                                                                             // 87
        throw new Meteor.Error(403, "You must be logged in to install packages.");                                     // 88
      }                                                                                                                // 89
    } else if (!isSignedUpOrDemo()) {                                                                                  // 90
      throw new Meteor.Error(403, "This Sandstorm server requires you to get an invite before installing apps.");      // 91
    }                                                                                                                  // 93
                                                                                                                       //
    var pkg = Packages.findOne(packageId);                                                                             // 95
                                                                                                                       //
    if (!pkg || pkg.status !== "ready") {                                                                              // 97
      if (!this.userId || isDemoUser() || globalDb.isUninvitedFreeUser()) {                                            // 98
        if (!isSafeDemoAppUrl(url)) {                                                                                  // 99
          // TODO(someday): Billing prompt on client side.                                                             // 100
          throw new Meteor.Error(403, "Sorry, demo and free users cannot upload custom apps; " + "they may only install apps from apps.sandstorm.io.");
        }                                                                                                              // 103
      }                                                                                                                // 104
    }                                                                                                                  // 105
                                                                                                                       //
    if (!pkg || isRetry) {                                                                                             // 107
      globalDb.startInstall(packageId, url, isRetry);                                                                  // 108
    }                                                                                                                  // 109
  }                                                                                                                    // 110
});                                                                                                                    // 48
Meteor.publish("packageInfo", function (packageId) {                                                                   // 113
  check(packageId, String);                                                                                            // 114
  var db = this.connection.sandstormDb;                                                                                // 115
  var pkgCursor = db.collections.packages.find(packageId);                                                             // 116
  var pkg = pkgCursor.fetch()[0];                                                                                      // 117
                                                                                                                       //
  if (pkg && this.userId) {                                                                                            // 118
    return [pkgCursor, db.collections.userActions.find({                                                               // 119
      userId: this.userId,                                                                                             // 121
      appId: pkg.appId                                                                                                 // 121
    }), db.collections.grains.find({                                                                                   // 121
      userId: this.userId,                                                                                             // 122
      appId: pkg.appId                                                                                                 // 122
    })];                                                                                                               // 122
  } else {                                                                                                             // 124
    return pkgCursor;                                                                                                  // 125
  }                                                                                                                    // 126
});                                                                                                                    // 127
Router.map(function () {                                                                                               // 129
  this.route("upload", {                                                                                               // 130
    where: "server",                                                                                                   // 131
    path: "/upload/:token",                                                                                            // 132
    action: function () {                                                                                              // 134
      if (typeof this.params.token !== "string" || !globalDb.collections.spkTokens.findOne(this.params.token)) {       // 135
        this.response.writeHead(403, {                                                                                 // 137
          "Content-Type": "text/plain",                                                                                // 138
          "Access-Control-Allow-Origin": "*"                                                                           // 139
        });                                                                                                            // 137
        this.response.write("Invalid upload token.");                                                                  // 141
        this.response.end();                                                                                           // 142
      } else if (this.request.method === "POST") {                                                                     // 143
        try {                                                                                                          // 144
          var packageId = promiseToFuture(doClientUpload(this.request)).wait();                                        // 145
          this.response.writeHead(200, {                                                                               // 146
            "Content-Length": packageId.length,                                                                        // 147
            "Content-Type": "text/plain",                                                                              // 148
            "Access-Control-Allow-Origin": "*"                                                                         // 149
          });                                                                                                          // 146
          this.response.write(packageId);                                                                              // 151
          this.response.end();                                                                                         // 152
          globalDb.collections.spkTokens.remove(this.params.token);                                                    // 153
        } catch (error) {                                                                                              // 154
          console.error(error.stack);                                                                                  // 155
          this.response.writeHead(500, {                                                                               // 156
            "Content-Type": "text/plain",                                                                              // 157
            "Access-Control-Allow-Origin": "*"                                                                         // 158
          });                                                                                                          // 156
          this.response.write("Unpacking SPK failed; is it valid?");                                                   // 160
          this.response.end();                                                                                         // 161
        }                                                                                                              // 162
                                                                                                                       //
        ;                                                                                                              // 162
      } else if (this.request.method == "OPTIONS") {                                                                   // 163
        // Allow cross-origin posts to upload so that uploads can occur on the DDP host                                // 164
        // rather than the main host. In theory we could have Access-Control-Allow-Origin specify                      // 165
        // the main host rather than "*", but an upload request already requires a valid                               // 166
        // upload token, which is plenty of access control in itself.                                                  // 167
        var requestedHeaders = this.request.headers["access-control-request-headers"];                                 // 168
                                                                                                                       //
        if (requestedHeaders) {                                                                                        // 169
          this.response.setHeader("Access-Control-Allow-Headers", requestedHeaders);                                   // 170
        }                                                                                                              // 171
                                                                                                                       //
        this.response.writeHead(204, {                                                                                 // 173
          "Access-Control-Allow-Origin": "*",                                                                          // 174
          "Access-Control-Allow-Methods": "POST, OPTIONS",                                                             // 175
          "Access-Control-Max-Age": "3600"                                                                             // 176
        });                                                                                                            // 173
        this.response.end();                                                                                           // 178
      } else {                                                                                                         // 179
        this.response.writeHead(405, {                                                                                 // 180
          "Content-Type": "text/plain",                                                                                // 181
          "Access-Control-Allow-Origin": "*"                                                                           // 182
        });                                                                                                            // 180
        this.response.write("You can only POST here.");                                                                // 184
        this.response.end();                                                                                           // 185
      }                                                                                                                // 186
    }                                                                                                                  // 187
  });                                                                                                                  // 130
});                                                                                                                    // 189
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"installer.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/installer.js                                                                                                 //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var module1 = module;                                                                                                  // 1
var Fs = void 0;                                                                                                       // 1
module1.watch(require("fs"), {                                                                                         // 1
  "default": function (v) {                                                                                            // 1
    Fs = v;                                                                                                            // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var Path = void 0;                                                                                                     // 1
module1.watch(require("path"), {                                                                                       // 1
  "default": function (v) {                                                                                            // 1
    Path = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var Crypto = void 0;                                                                                                   // 1
module1.watch(require("crypto"), {                                                                                     // 1
  "default": function (v) {                                                                                            // 1
    Crypto = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var ChildProcess = void 0;                                                                                             // 1
module1.watch(require("child_process"), {                                                                              // 1
  "default": function (v) {                                                                                            // 1
    ChildProcess = v;                                                                                                  // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var Url = void 0;                                                                                                      // 1
module1.watch(require("url"), {                                                                                        // 1
  "default": function (v) {                                                                                            // 1
    Url = v;                                                                                                           // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
var inMeteor = void 0,                                                                                                 // 1
    waitPromise = void 0;                                                                                              // 1
module1.watch(require("/imports/server/async-helpers.js"), {                                                           // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  },                                                                                                                   // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 5);                                                                                                                 // 1
var ssrfSafeLookupOrProxy = void 0;                                                                                    // 1
module1.watch(require("/imports/server/networking.js"), {                                                              // 1
  ssrfSafeLookupOrProxy: function (v) {                                                                                // 1
    ssrfSafeLookupOrProxy = v;                                                                                         // 1
  }                                                                                                                    // 1
}, 6);                                                                                                                 // 1
var Request = HTTPInternals.NpmModules.request.module;                                                                 // 26
var Manifest = Capnp.importSystem("sandstorm/package.capnp").Manifest;                                                 // 28
var installers = void 0; // set to {} on main replica                                                                  // 30
// To protect against race conditions, we require that each row in the Packages                                        // 31
// collection have at most one writer at a time, as tracked by this `installers`                                       // 32
// map. Each key is a package ID and each value is either an AppInstaller object                                       // 33
// or the string 'uninstalling', indicating that some fiber is working on                                              // 34
// uninstalling the package.                                                                                           // 35
                                                                                                                       //
var verifyIsMainReplica = function () {                                                                                // 37
  if (Meteor.settings.replicaNumber) {                                                                                 // 38
    throw new Error("This can only be called on the main front-end replica.");                                         // 39
  }                                                                                                                    // 40
};                                                                                                                     // 41
                                                                                                                       //
var deletePackageInternal = function (pkg) {                                                                           // 43
  verifyIsMainReplica();                                                                                               // 44
  var packageId = pkg._id;                                                                                             // 46
                                                                                                                       //
  if (packageId in installers) {                                                                                       // 48
    return;                                                                                                            // 49
  }                                                                                                                    // 50
                                                                                                                       //
  installers[packageId] = "uninstalling";                                                                              // 52
                                                                                                                       //
  try {                                                                                                                // 54
    var action = UserActions.findOne({                                                                                 // 55
      packageId: packageId                                                                                             // 55
    });                                                                                                                // 55
    var grain = Grains.findOne({                                                                                       // 56
      packageId: packageId                                                                                             // 56
    });                                                                                                                // 56
    var notificationQuery = {};                                                                                        // 57
    notificationQuery["appUpdates." + pkg.appId + ".packageId"] = packageId;                                           // 58
                                                                                                                       //
    if (!grain && !action && !Notifications.findOne(notificationQuery) && !globalDb.getAppIdForPreinstalledPackage(packageId)) {
      Packages.update({                                                                                                // 61
        _id: packageId                                                                                                 // 62
      }, {                                                                                                             // 61
        $set: {                                                                                                        // 64
          status: "delete"                                                                                             // 64
        },                                                                                                             // 64
        $unset: {                                                                                                      // 65
          shouldCleanup: ""                                                                                            // 65
        }                                                                                                              // 65
      });                                                                                                              // 63
      waitPromise(globalBackend.cap().deletePackage(packageId));                                                       // 67
      Packages.remove(packageId); // Clean up assets (icon, etc).                                                      // 68
                                                                                                                       //
      getAllManifestAssets(pkg.manifest).forEach(function (assetId) {                                                  // 71
        globalDb.unrefStaticAsset(assetId);                                                                            // 72
      });                                                                                                              // 73
    } else {                                                                                                           // 74
      Packages.update({                                                                                                // 75
        _id: packageId                                                                                                 // 75
      }, {                                                                                                             // 75
        $unset: {                                                                                                      // 75
          shouldCleanup: ""                                                                                            // 75
        }                                                                                                              // 75
      });                                                                                                              // 75
    }                                                                                                                  // 76
                                                                                                                       //
    delete installers[packageId];                                                                                      // 78
  } catch (error) {                                                                                                    // 79
    delete installers[packageId];                                                                                      // 80
    throw error;                                                                                                       // 81
  }                                                                                                                    // 82
};                                                                                                                     // 83
                                                                                                                       //
var startInstallInternal = function (pkg) {                                                                            // 85
  verifyIsMainReplica();                                                                                               // 86
                                                                                                                       //
  if (pkg._id in installers) {                                                                                         // 88
    return;                                                                                                            // 89
  }                                                                                                                    // 90
                                                                                                                       //
  var installer = new AppInstaller(pkg._id, pkg.url, pkg.appId, pkg.isAutoUpdated);                                    // 92
  installers[pkg._id] = installer;                                                                                     // 93
  installer.start();                                                                                                   // 94
};                                                                                                                     // 95
                                                                                                                       //
cancelDownload = function (packageId) {                                                                                // 97
  Packages.remove({                                                                                                    // 98
    _id: packageId,                                                                                                    // 98
    status: "download"                                                                                                 // 98
  });                                                                                                                  // 98
};                                                                                                                     // 99
                                                                                                                       //
var cancelDownloadInternal = function (pkg) {                                                                          // 101
  verifyIsMainReplica();                                                                                               // 102
  var installer = installers[pkg._id]; // Don't do anything unless a download is in progress.                          // 104
                                                                                                                       //
  if (installer && installer.downloadRequest) {                                                                        // 107
    // OK, effect cancellation by faking an error.                                                                     // 108
    installer.wrapCallback(function () {                                                                               // 109
      throw new Error("Canceled");                                                                                     // 110
    })();                                                                                                              // 111
  }                                                                                                                    // 112
};                                                                                                                     // 113
                                                                                                                       //
if (!Meteor.settings.replicaNumber) {                                                                                  // 115
  installers = {};                                                                                                     // 116
  Meteor.startup(function () {                                                                                         // 118
    // Restart any deletions that were killed while in-progress.                                                       // 119
    Packages.find({                                                                                                    // 120
      status: "delete"                                                                                                 // 120
    }).forEach(deletePackageInternal); // Watch for new installation requests and fulfill them.                        // 120
                                                                                                                       //
    Packages.find({                                                                                                    // 123
      status: {                                                                                                        // 123
        $in: ["download", "verify", "unpack", "analyze"]                                                               // 123
      }                                                                                                                // 123
    }).observe({                                                                                                       // 123
      added: startInstallInternal,                                                                                     // 124
      removed: cancelDownloadInternal                                                                                  // 125
    }); // Watch for new cleanup requests and fulfill them.                                                            // 123
                                                                                                                       //
    Packages.find({                                                                                                    // 129
      status: "ready",                                                                                                 // 129
      shouldCleanup: true                                                                                              // 129
    }).observe({                                                                                                       // 129
      added: deletePackageInternal                                                                                     // 130
    });                                                                                                                // 129
  });                                                                                                                  // 132
}                                                                                                                      // 133
                                                                                                                       //
doClientUpload = function (stream) {                                                                                   // 135
  return new Promise(function (resolve, reject) {                                                                      // 136
    var id = Random.id();                                                                                              // 137
    var backendStream = globalBackend.cap().installPackage().stream;                                                   // 139
    var hasher = Crypto.createHash("sha256");                                                                          // 140
    stream.on("data", function (chunk) {                                                                               // 142
      try {                                                                                                            // 143
        hasher.update(chunk);                                                                                          // 144
        backendStream.write(chunk);                                                                                    // 145
      } catch (err) {                                                                                                  // 146
        reject(err);                                                                                                   // 147
      }                                                                                                                // 148
    });                                                                                                                // 149
    stream.on("end", function () {                                                                                     // 151
      try {                                                                                                            // 152
        backendStream.done();                                                                                          // 153
        var packageId = hasher.digest("hex").slice(0, 32);                                                             // 154
        resolve(backendStream.saveAs(packageId).then(function () {                                                     // 155
          return packageId;                                                                                            // 156
        }));                                                                                                           // 157
        backendStream.close();                                                                                         // 158
      } catch (err) {                                                                                                  // 159
        reject(err);                                                                                                   // 160
      }                                                                                                                // 161
    });                                                                                                                // 162
    stream.on("error", function (err) {                                                                                // 164
      // TODO(soon):  This event does't seem to fire if the user leaves the page mid-upload.                           // 165
      try {                                                                                                            // 166
        backendStream.close();                                                                                         // 167
        reject(err);                                                                                                   // 168
      } catch (err2) {                                                                                                 // 169
        reject(err2);                                                                                                  // 170
      }                                                                                                                // 171
    });                                                                                                                // 172
  });                                                                                                                  // 173
};                                                                                                                     // 174
                                                                                                                       //
AppInstaller = function () {                                                                                           // 176
  function AppInstaller(packageId, url, appId, isAutoUpdated) {                                                        // 177
    (0, _classCallCheck3.default)(this, AppInstaller);                                                                 // 177
    verifyIsMainReplica();                                                                                             // 178
    this.packageId = packageId;                                                                                        // 180
    this.url = url;                                                                                                    // 181
    this.failed = false;                                                                                               // 182
    this.appId = appId;                                                                                                // 183
    this.isAutoUpdated = isAutoUpdated; // Serializes database writes.                                                 // 184
                                                                                                                       //
    this.writeChain = Promise.resolve();                                                                               // 187
  }                                                                                                                    // 188
                                                                                                                       //
  AppInstaller.prototype.updateProgress = function () {                                                                // 176
    function updateProgress(status, progress, error, manifest) {                                                       // 176
      // TODO(security):  On error, we should actually delete the package from the database and only                   // 191
      //   display the error to whomever was watching at the time.  Otherwise it's easy to confuse                     // 192
      //   people by 'pre-failing' packages.  (Actually, perhaps if a user tries to download an                        // 193
      //   already-downloading package but specifies a different URL, we really should initiate an                     // 194
      //   entirely separate download...  but cancel it if the first download succeeds.)                               // 195
      this.status = status;                                                                                            // 197
      this.progress = progress || -1;                                                                                  // 198
      this.error = error;                                                                                              // 199
      this.manifest = manifest || null;                                                                                // 200
                                                                                                                       //
      var _this = this; // The callback passed to inMeteor() runs in a new fiber. We need to make sure database writes
      // occur in exactly the order in which we generate them, so we use a promise chain to serialize                  // 205
      // them.                                                                                                         // 206
                                                                                                                       //
                                                                                                                       //
      this.writeChain = this.writeChain.then(function () {                                                             // 207
        return inMeteor(function () {                                                                                  // 208
          if (manifest) extractManifestAssets(manifest);                                                               // 209
          Packages.update(_this.packageId, {                                                                           // 211
            $set: {                                                                                                    // 212
              status: _this.status,                                                                                    // 213
              progress: _this.progress,                                                                                // 214
              error: _this.error ? _this.error.message : null,                                                         // 215
              manifest: _this.manifest,                                                                                // 216
              appId: _this.appId,                                                                                      // 217
              authorPgpKeyFingerprint: _this.authorPgpKeyFingerprint                                                   // 218
            }                                                                                                          // 212
          });                                                                                                          // 211
                                                                                                                       //
          if (_this.authorPgpKeyFingerprint) {                                                                         // 222
            globalDb.updateKeybaseProfileAsync(_this.authorPgpKeyFingerprint);                                         // 223
          }                                                                                                            // 224
        }).catch(function (err) {                                                                                      // 225
          console.error(err.stack);                                                                                    // 226
        });                                                                                                            // 227
      });                                                                                                              // 228
    }                                                                                                                  // 229
                                                                                                                       //
    return updateProgress;                                                                                             // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  AppInstaller.prototype.wrapCallback = function () {                                                                  // 176
    function wrapCallback(method) {                                                                                    // 176
      // Note that the function below must not be an arrow function, since arrow functions do not have                 // 232
      // access to the context's arguments array.                                                                      // 233
      var _this = this;                                                                                                // 234
                                                                                                                       //
      return function () {                                                                                             // 235
        if (_this.failed) return;                                                                                      // 236
                                                                                                                       //
        try {                                                                                                          // 237
          return method.apply(_this, _.toArray(arguments));                                                            // 238
        } catch (err) {                                                                                                // 239
          _this.failed = true;                                                                                         // 240
                                                                                                                       //
          _this.cleanup();                                                                                             // 241
                                                                                                                       //
          _this.updateProgress("failed", 0, err);                                                                      // 242
                                                                                                                       //
          _this.writeChain = _this.writeChain.then(function () {                                                       // 243
            delete installers[_this.packageId];                                                                        // 244
          });                                                                                                          // 245
          console.error("Failed to install app:", err.stack);                                                          // 246
        }                                                                                                              // 247
      };                                                                                                               // 248
    }                                                                                                                  // 249
                                                                                                                       //
    return wrapCallback;                                                                                               // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  AppInstaller.prototype.cleanup = function () {                                                                       // 176
    function cleanup() {                                                                                               // 176
      if (this.uploadStream) {                                                                                         // 252
        try {                                                                                                          // 253
          this.uploadStream.close();                                                                                   // 254
        } catch (err) {}                                                                                               // 255
                                                                                                                       //
        delete this.uploadStream;                                                                                      // 257
      }                                                                                                                // 258
                                                                                                                       //
      if (this.downloadRequest) {                                                                                      // 260
        try {                                                                                                          // 261
          this.downloadRequest.abort();                                                                                // 262
        } catch (err) {}                                                                                               // 263
                                                                                                                       //
        delete this.downloadRequest;                                                                                   // 265
      }                                                                                                                // 266
    }                                                                                                                  // 267
                                                                                                                       //
    return cleanup;                                                                                                    // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  AppInstaller.prototype.start = function () {                                                                         // 176
    function start() {                                                                                                 // 176
      var _this2 = this;                                                                                               // 269
                                                                                                                       //
      return this.wrapCallback(function () {                                                                           // 270
        _this2.cleanup();                                                                                              // 271
                                                                                                                       //
        globalBackend.cap().tryGetPackage(_this2.packageId).then(_this2.wrapCallback(function (info) {                 // 273
          if (info.appId) {                                                                                            // 274
            _this2.appId = info.appId;                                                                                 // 275
            _this2.authorPgpKeyFingerprint = info.authorPgpKeyFingerprint;                                             // 276
                                                                                                                       //
            _this2.done(info.manifest);                                                                                // 277
          } else {                                                                                                     // 278
            _this2.doDownload();                                                                                       // 279
          }                                                                                                            // 280
        }), _this2.wrapCallback(function (err) {                                                                       // 281
          throw err;                                                                                                   // 282
        }));                                                                                                           // 283
      })();                                                                                                            // 284
    }                                                                                                                  // 285
                                                                                                                       //
    return start;                                                                                                      // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  AppInstaller.prototype.doDownload = function () {                                                                    // 176
    function doDownload() {                                                                                            // 176
      if (!this.url) {                                                                                                 // 288
        throw new Error("Unknown package ID, and no URL was provided.");                                               // 289
      }                                                                                                                // 290
                                                                                                                       //
      console.log("Downloading app:", this.url);                                                                       // 292
      this.updateProgress("download");                                                                                 // 293
      this.uploadStream = globalBackend.cap().installPackage().stream;                                                 // 295
      return this.doDownloadTo(this.uploadStream);                                                                     // 296
    }                                                                                                                  // 297
                                                                                                                       //
    return doDownload;                                                                                                 // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  AppInstaller.prototype.doDownloadTo = function () {                                                                  // 176
    function doDownloadTo(out) {                                                                                       // 176
      inMeteor(this.wrapCallback(function () {                                                                         // 300
        var _this3 = this;                                                                                             // 300
                                                                                                                       //
        var safe = ssrfSafeLookupOrProxy(globalDb, this.url);                                                          // 301
        var bytesExpected = undefined;                                                                                 // 303
        var bytesReceived = 0;                                                                                         // 304
        var hasher = Crypto.createHash("sha256");                                                                      // 305
        var done = false;                                                                                              // 306
                                                                                                                       //
        var updateDownloadProgress = _.throttle(this.wrapCallback(function () {                                        // 307
          if (!done) {                                                                                                 // 308
            if (bytesExpected) {                                                                                       // 309
              _this3.updateProgress("download", bytesReceived / bytesExpected);                                        // 310
            } else {                                                                                                   // 311
              _this3.updateProgress("download", bytesReceived);                                                        // 312
            }                                                                                                          // 313
          }                                                                                                            // 314
        }), 500);                                                                                                      // 315
                                                                                                                       //
        var request = safe.proxy ? Request.get(this.url, {                                                             // 317
          proxy: safe.proxy                                                                                            // 318
        }) : Request.get(safe.url, {                                                                                   // 318
          headers: {                                                                                                   // 320
            host: safe.host                                                                                            // 320
          },                                                                                                           // 320
          servername: safe.host.split(":")[0]                                                                          // 321
        });                                                                                                            // 319
        request.on("response", this.wrapCallback(function (response) {                                                 // 324
          if ("content-length" in response.headers) {                                                                  // 325
            bytesExpected = parseInt(response.headers["content-length"]);                                              // 326
          }                                                                                                            // 327
        }));                                                                                                           // 328
        request.on("data", this.wrapCallback(function (chunk) {                                                        // 330
          hasher.update(chunk);                                                                                        // 331
          out.write(chunk);                                                                                            // 332
          bytesReceived += chunk.length;                                                                               // 333
          updateDownloadProgress();                                                                                    // 334
        }));                                                                                                           // 335
        request.on("end", this.wrapCallback(function () {                                                              // 337
          out.done();                                                                                                  // 338
                                                                                                                       //
          if (hasher.digest("hex").slice(0, 32) !== _this3.packageId) {                                                // 340
            throw new Error("Package hash did not match.");                                                            // 341
          }                                                                                                            // 342
                                                                                                                       //
          done = true;                                                                                                 // 344
          delete _this3.downloadRequest;                                                                               // 345
                                                                                                                       //
          _this3.updateProgress("unpack");                                                                             // 347
                                                                                                                       //
          out.saveAs(_this3.packageId).then(_this3.wrapCallback(function (info) {                                      // 348
            _this3.appId = info.appId;                                                                                 // 349
            _this3.authorPgpKeyFingerprint = info.authorPgpKeyFingerprint;                                             // 350
                                                                                                                       //
            _this3.done(info.manifest);                                                                                // 351
          }), _this3.wrapCallback(function (err) {                                                                     // 352
            throw err;                                                                                                 // 353
          }));                                                                                                         // 354
        }));                                                                                                           // 355
        request.on("error", this.wrapCallback(function (err) {                                                         // 357
          throw err;                                                                                                   // 357
        }));                                                                                                           // 357
        this.downloadRequest = request;                                                                                // 359
      }));                                                                                                             // 360
    }                                                                                                                  // 361
                                                                                                                       //
    return doDownloadTo;                                                                                               // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  AppInstaller.prototype.done = function () {                                                                          // 176
    function done(manifest) {                                                                                          // 176
      console.log("App ready:", this.packageId);                                                                       // 364
      this.updateProgress("ready", 1, undefined, manifest);                                                            // 365
                                                                                                                       //
      var _this = this;                                                                                                // 366
                                                                                                                       //
      _this.writeChain = _this.writeChain.then(function () {                                                           // 367
        return inMeteor(function () {                                                                                  // 368
          if (_this.isAutoUpdated) {                                                                                   // 369
            globalDb.sendAppUpdateNotifications(_this.appId, _this.packageId, manifest.appTitle && manifest.appTitle.defaultText, manifest.appVersion, manifest.appMarketingVersion && manifest.appMarketingVersion.defaultText);
          }                                                                                                            // 373
                                                                                                                       //
          if (globalDb.getPackageIdForPreinstalledApp(_this.appId) && globalDb.collections.appIndex.findOne({          // 375
            _id: _this.appId,                                                                                          // 377
            packageId: _this.packageId                                                                                 // 378
          })) {                                                                                                        // 376
            // Only mark app as preinstall ready if its appId is in the preinstalledApps setting                       // 380
            // and if it's the latest package version in the appIndex. The updateAppIndex function                     // 381
            // will always trigger updates of preinstalled apps, even if a concurrent download of                      // 382
            // an older package is going on.                                                                           // 383
            globalDb.setPreinstallAppAsReady(_this.appId, _this.packageId);                                            // 384
          }                                                                                                            // 385
        });                                                                                                            // 386
      }).then(function () {                                                                                            // 387
        delete installers[_this.packageId];                                                                            // 388
      });                                                                                                              // 389
    }                                                                                                                  // 390
                                                                                                                       //
    return done;                                                                                                       // 176
  }();                                                                                                                 // 176
                                                                                                                       //
  return AppInstaller;                                                                                                 // 176
}();                                                                                                                   // 176
                                                                                                                       //
extractManifestAssets = function (manifest) {                                                                          // 393
  var metadata = manifest.metadata;                                                                                    // 394
  if (!metadata) return;                                                                                               // 395
  var icons = metadata.icons;                                                                                          // 397
                                                                                                                       //
  if (icons) {                                                                                                         // 398
    var handleIcon = function (icon) {                                                                                 // 399
      if (icon.svg) {                                                                                                  // 400
        icon.assetId = globalDb.addStaticAsset({                                                                       // 401
          mimeType: "image/svg+xml"                                                                                    // 401
        }, icon.svg);                                                                                                  // 401
        icon.format = "svg";                                                                                           // 402
        delete icon.svg;                                                                                               // 403
        return true;                                                                                                   // 404
      } else if (icon.png) {                                                                                           // 405
        // Use the 1x version for 'normal' DPI, unless 1x isn't provided, in which case use 2x.                        // 406
        var normalDpi = icon.png.dpi1x || icon.png.dpi2x;                                                              // 407
        if (!normalDpi) return false;                                                                                  // 408
        icon.format = "png";                                                                                           // 409
        icon.assetId = globalDb.addStaticAsset({                                                                       // 410
          mimeType: "image/png"                                                                                        // 410
        }, normalDpi);                                                                                                 // 410
                                                                                                                       //
        if (icon.png.dpi1x && icon.png.dpi2x) {                                                                        // 412
          // Icon specifies both resolutions, so also record a 2x DPI option.                                          // 413
          icon.assetId2xDpi = globalDb.addStaticAsset({                                                                // 414
            mimeType: "image/png"                                                                                      // 414
          }, icon.png.dpi2x);                                                                                          // 414
        }                                                                                                              // 415
                                                                                                                       //
        delete icon.png;                                                                                               // 417
        return true;                                                                                                   // 418
      } else {                                                                                                         // 419
        // Unknown icon. Filter it.                                                                                    // 420
        return false;                                                                                                  // 421
      }                                                                                                                // 422
    };                                                                                                                 // 423
                                                                                                                       //
    if (icons.appGrid && !handleIcon(icons.appGrid)) delete icons.appGrid;                                             // 425
    if (icons.grain && !handleIcon(icons.grain)) delete icons.grain; // We don't need the market icons.                // 426
                                                                                                                       //
    if (icons.market) delete icons.market;                                                                             // 429
    if (icons.marketBig) delete icons.marketBig;                                                                       // 430
  }                                                                                                                    // 431
                                                                                                                       //
  var handleLocalizedText = function (text) {                                                                          // 433
    if (text.defaultText) {                                                                                            // 434
      text.defaultTextAssetId = globalDb.addStaticAsset({                                                              // 435
        mimeType: "text/plain"                                                                                         // 435
      }, text.defaultText);                                                                                            // 435
      delete text.defaultText;                                                                                         // 436
    }                                                                                                                  // 437
                                                                                                                       //
    if (text.localizations) {                                                                                          // 439
      text.localizations.forEach(function (localization) {                                                             // 440
        if (localization.text) {                                                                                       // 441
          localization.assetId = globalDb.addStaticAsset({                                                             // 442
            mimeType: "text/plain"                                                                                     // 443
          }, localization.text);                                                                                       // 443
          delete localization.text;                                                                                    // 444
        }                                                                                                              // 445
      });                                                                                                              // 446
    }                                                                                                                  // 447
  };                                                                                                                   // 448
                                                                                                                       //
  var license = metadata.license;                                                                                      // 450
                                                                                                                       //
  if (license) {                                                                                                       // 451
    if (license.proprietary) license.proprietary = handleLocalizedText(license.proprietary);                           // 452
    if (license.publicDomain) license.publicDomain = handleLocalizedText(license.proprietary);                         // 453
    if (license.notices) license.notices = handleLocalizedText(license.notices);                                       // 454
  }                                                                                                                    // 455
                                                                                                                       //
  var author = metadata.author;                                                                                        // 457
                                                                                                                       //
  if (author) {                                                                                                        // 458
    // We remove the PGP signature since it was already verified down to a key ID in the back-end.                     // 459
    if (author.pgpSignature) delete author.pgpSignature;                                                               // 460
  } // Don't need the keyring either.                                                                                  // 461
                                                                                                                       //
                                                                                                                       //
  if (metadata.pgpKeyring) delete metadata.pgpKeyring; // Perhaps used by the 'about' page?                            // 464
                                                                                                                       //
  if (metadata.description) metadata.description = handleLocalizedText(metadata.description); // Screenshots are for app marketing; we don't use them post-install.
                                                                                                                       //
  if (metadata.screenshots) delete metadata.screenshots; // We might allow the user to view the changelog.             // 470
                                                                                                                       //
  if (metadata.changeLog) metadata.changeLog = handleLocalizedText(metadata.changeLog);                                // 473
};                                                                                                                     // 474
                                                                                                                       //
getAllManifestAssets = function (manifest) {                                                                           // 476
  // Returns a list of all asset IDs in the given manifest.                                                            // 477
  var metadata = manifest.metadata;                                                                                    // 479
  if (!metadata) return [];                                                                                            // 480
  var result = [];                                                                                                     // 482
  var icons = metadata.icons;                                                                                          // 484
                                                                                                                       //
  if (icons) {                                                                                                         // 485
    var handleIcon = function (icon) {                                                                                 // 486
      if (icon.assetId) {                                                                                              // 487
        result.push(icon.assetId);                                                                                     // 488
      }                                                                                                                // 489
    };                                                                                                                 // 490
                                                                                                                       //
    if (icons.appGrid) handleIcon(icons.appGrid);                                                                      // 492
    if (icons.grain) handleIcon(icons.grain);                                                                          // 493
  }                                                                                                                    // 494
                                                                                                                       //
  var handleLocalizedText = function (text) {                                                                          // 496
    if (text.defaultTextAssetId) {                                                                                     // 497
      result.push(defaultTextAssetId);                                                                                 // 498
    }                                                                                                                  // 499
                                                                                                                       //
    if (text.localizations) {                                                                                          // 501
      text.localizations.forEach(function (localization) {                                                             // 502
        if (localization.assetId) {                                                                                    // 503
          result.push(localization.assetId);                                                                           // 504
        }                                                                                                              // 505
      });                                                                                                              // 506
    }                                                                                                                  // 507
  };                                                                                                                   // 508
                                                                                                                       //
  var license = metadata.license;                                                                                      // 510
                                                                                                                       //
  if (license) {                                                                                                       // 511
    if (license.proprietary) handleLocalizedText(license.proprietary);                                                 // 512
    if (license.publicDomain) handleLocalizedText(license.publicDomain);                                               // 513
    if (license.notices) handleLocalizedText(license.notices);                                                         // 514
  }                                                                                                                    // 515
                                                                                                                       //
  if (metadata.description) handleLocalizedText(metadata.description);                                                 // 517
  if (metadata.changeLog) handleLocalizedText(metadata.changeLog);                                                     // 518
  return result;                                                                                                       // 520
};                                                                                                                     // 521
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"notifications-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/notifications-server.js                                                                                      //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var createAppActivityDesktopNotification = void 0;                                                                     // 1
module.watch(require("/imports/server/desktop-notifications.js"), {                                                    // 1
  createAppActivityDesktopNotification: function (v) {                                                                 // 1
    createAppActivityDesktopNotification = v;                                                                          // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var SupervisorCapnp = Capnp.importSystem("sandstorm/supervisor.capnp");                                                // 20
var SystemPersistent = SupervisorCapnp.SystemPersistent;                                                               // 21
                                                                                                                       //
logActivity = function (grainId, accountIdOrAnonymous, event) {                                                        // 23
  // accountIdOrAnonymous is the string "anonymous" for an anonymous user, or is null for a                            // 24
  // non-user-initiated ("background") activity.                                                                       // 25
  check(grainId, String);                                                                                              // 27
  check(accountIdOrAnonymous, Match.Maybe(String)); // `event` is always an ActivityEvent parsed from Cap'n Proto but that's too complicated to check
  // here.                                                                                                             // 30
                                                                                                                       //
  var accountId = accountIdOrAnonymous === "anonymous" ? null : accountIdOrAnonymous; // TODO(perf): A cached copy of the grain from when the session opened would be fine to use
  //   here, rather than looking it up every time.                                                                     // 35
                                                                                                                       //
  grain = Grains.findOne(grainId);                                                                                     // 36
                                                                                                                       //
  if (!grain) {                                                                                                        // 37
    // Shouldn't be possible since activity events come from the grain.                                                // 38
    throw new Error("no such grain");                                                                                  // 39
  } // Look up the event typedef.                                                                                      // 40
                                                                                                                       //
                                                                                                                       //
  var eventType = ((grain.cachedViewInfo || {}).eventTypes || [])[event.type];                                         // 43
                                                                                                                       //
  if (!eventType) {                                                                                                    // 44
    throw new Error("No such event type in app's ViewInfo: " + event.type);                                            // 45
  }                                                                                                                    // 46
                                                                                                                       //
  if (!eventType.suppressUnread) {                                                                                     // 48
    // Clear the "seenAllActivity" bit for all users except the acting user.                                           // 49
    // TODO(perf): Consider throttling? Or should that be the app's responsibility?                                    // 50
    if (accountId != grain.userId) {                                                                                   // 51
      Grains.update(grainId, {                                                                                         // 52
        $unset: {                                                                                                      // 52
          ownerSeenAllActivity: true                                                                                   // 52
        }                                                                                                              // 52
      });                                                                                                              // 52
    } // Also clear on ApiTokens.                                                                                      // 53
                                                                                                                       //
                                                                                                                       //
    ApiTokens.update({                                                                                                 // 56
      "grainId": grainId,                                                                                              // 57
      "owner.user.seenAllActivity": true,                                                                              // 58
      "owner.user.accountId": {                                                                                        // 59
        $ne: accountId                                                                                                 // 59
      }                                                                                                                // 59
    }, {                                                                                                               // 56
      $unset: {                                                                                                        // 60
        "owner.user.seenAllActivity": true                                                                             // 60
      }                                                                                                                // 60
    }, {                                                                                                               // 60
      multi: true                                                                                                      // 60
    });                                                                                                                // 60
  }                                                                                                                    // 61
                                                                                                                       //
  if (accountId) {                                                                                                     // 63
    // Apply auto-subscriptions.                                                                                       // 64
    if (eventType.autoSubscribeToGrain) {                                                                              // 65
      globalDb.subscribeToActivity(accountId, grainId);                                                                // 66
    }                                                                                                                  // 67
                                                                                                                       //
    if (event.thread && eventType.autoSubscribeToThread) {                                                             // 69
      globalDb.subscribeToActivity(accountId, grainId, event.thread.path || "");                                       // 70
    }                                                                                                                  // 71
  } // Figure out whom we need to notify.                                                                              // 72
                                                                                                                       //
                                                                                                                       //
  var notifyMap = {};                                                                                                  // 75
                                                                                                                       //
  var addRecipient = function (recipient) {                                                                            // 76
    // Mutes take priority over subscriptions.                                                                         // 77
    if (recipient.mute) {                                                                                              // 78
      notifyMap[recipient.accountId] = false;                                                                          // 79
    } else {                                                                                                           // 80
      if (!(recipient.accountId in notifyMap)) {                                                                       // 81
        notifyMap[recipient.accountId] = true;                                                                         // 82
      }                                                                                                                // 83
    }                                                                                                                  // 84
  };                                                                                                                   // 85
                                                                                                                       //
  if (accountId) {                                                                                                     // 87
    // Don't notify self.                                                                                              // 88
    addRecipient({                                                                                                     // 89
      accountId: accountId,                                                                                            // 89
      mute: true                                                                                                       // 89
    });                                                                                                                // 89
  } // Notify subscribers, if desired.                                                                                 // 90
                                                                                                                       //
                                                                                                                       //
  if (eventType.notifySubscribers) {                                                                                   // 93
    // The grain owner is implicitly subscribed.                                                                       // 94
    addRecipient({                                                                                                     // 95
      accountId: grain.userId                                                                                          // 95
    }); // Add everyone subscribed to the grain.                                                                       // 95
                                                                                                                       //
    globalDb.getActivitySubscriptions(grainId).forEach(addRecipient);                                                  // 98
                                                                                                                       //
    if (event.thread) {                                                                                                // 100
      // Add everyone subscribed to the thread.                                                                        // 101
      globalDb.getActivitySubscriptions(grainId, event.thread.path || "").forEach(addRecipient);                       // 102
    }                                                                                                                  // 103
  } // Add everyone who is mentioned.                                                                                  // 104
                                                                                                                       //
                                                                                                                       //
  if (event.users && event.users.length > 0) {                                                                         // 107
    var promises = [];                                                                                                 // 108
    event.users.forEach(function (user) {                                                                              // 109
      if (user.identity && (user.mentioned || user.subscribed)) {                                                      // 110
        promises.push(unwrapFrontendCap(user.identity, "identity", function (targetId) {                               // 111
          addRecipient({                                                                                               // 112
            accountId: targetId                                                                                        // 112
          });                                                                                                          // 112
        }));                                                                                                           // 113
      }                                                                                                                // 114
    });                                                                                                                // 115
    waitPromise(Promise.all(promises).then(function (junk) {                                                           // 116
      return undefined;                                                                                                // 116
    }));                                                                                                               // 116
  } // Make a list of everyone to notify.                                                                              // 117
                                                                                                                       //
                                                                                                                       //
  var notify = [];                                                                                                     // 120
                                                                                                                       //
  for (var _accountId in meteorBabelHelpers.sanitizeForInObject(notifyMap)) {                                          // 121
    if (notifyMap[_accountId]) {                                                                                       // 122
      notify.push(_accountId);                                                                                         // 123
    }                                                                                                                  // 124
  }                                                                                                                    // 125
                                                                                                                       //
  if (notify.length > 0) {                                                                                             // 127
    var notification = {                                                                                               // 128
      grainId: grainId,                                                                                                // 129
      path: event.path || ""                                                                                           // 130
    }; // Fields we'll update even if the notification already exists.                                                 // 128
                                                                                                                       //
    var update = {                                                                                                     // 134
      isUnread: true,                                                                                                  // 135
      timestamp: new Date()                                                                                            // 136
    };                                                                                                                 // 134
                                                                                                                       //
    if (event.thread) {                                                                                                // 139
      notification.threadPath = event.thread.path || "";                                                               // 140
    }                                                                                                                  // 141
                                                                                                                       //
    if (accountId) {                                                                                                   // 143
      notification.initiatingAccount = accountId;                                                                      // 144
    } else if (accountIdOrAnonymous) {                                                                                 // 145
      notification.initiatorAnonymous = true;                                                                          // 146
    }                                                                                                                  // 147
                                                                                                                       //
    notification.eventType = event.type;                                                                               // 149
    update.text = eventType.verbPhrase;                                                                                // 150
    var body = event.notification && event.notification.caption || {                                                   // 152
      defaultText: ""                                                                                                  // 152
    };                                                                                                                 // 152
    var appActivity = {                                                                                                // 154
      grainId: notification.grainId,                                                                                   // 155
      path: notification.path,                                                                                         // 156
      body: body,                                                                                                      // 157
      actionText: eventType.verbPhrase                                                                                 // 158
    };                                                                                                                 // 154
                                                                                                                       //
    if (accountId) {                                                                                                   // 161
      // Look up icon urls for the responsible user and the app                                                        // 162
      var account = Meteor.users.findOne({                                                                             // 163
        _id: accountId                                                                                                 // 163
      });                                                                                                              // 163
                                                                                                                       //
      if (!account) {                                                                                                  // 164
        throw new Error("no such user");                                                                               // 165
      }                                                                                                                // 166
                                                                                                                       //
      SandstormDb.fillInPictureUrl(account);                                                                           // 168
      appActivity.user = {                                                                                             // 170
        accountId: account._id,                                                                                        // 171
        name: account.profile.name,                                                                                    // 172
        avatarUrl: account.profile.pictureUrl || ""                                                                    // 173
      };                                                                                                               // 170
    } else if (accountIdOrAnonymous) {                                                                                 // 175
      appActivity.user = {                                                                                             // 176
        anonymous: true                                                                                                // 176
      };                                                                                                               // 176
    }                                                                                                                  // 177
                                                                                                                       //
    notify.forEach(function (targetId) {                                                                               // 179
      // Notify the account.                                                                                           // 180
      // We need to know the ID of the inserted/updated document so we can embed it in the                             // 182
      // desktop notification to bind them.                                                                            // 183
      var idIfInserted = Random.id(17);                                                                                // 184
      var result = Notifications.findAndModify({                                                                       // 185
        query: _.extend({                                                                                              // 186
          userId: targetId                                                                                             // 186
        }, notification),                                                                                              // 186
        update: {                                                                                                      // 187
          $set: update,                                                                                                // 188
          $inc: {                                                                                                      // 189
            count: 1                                                                                                   // 189
          },                                                                                                           // 189
          $setOnInsert: {                                                                                              // 190
            _id: idIfInserted                                                                                          // 191
          }                                                                                                            // 190
        },                                                                                                             // 187
        upsert: true                                                                                                   // 194
      });                                                                                                              // 185
                                                                                                                       //
      if (!result.ok) {                                                                                                // 197
        console.error("Couldn't create notification!", result.lastErrorObject);                                        // 198
        return;                                                                                                        // 199
      }                                                                                                                // 200
                                                                                                                       //
      var notificationId = result.value._id || idIfInserted;                                                           // 202
      var desktopNotification = {                                                                                      // 204
        userId: targetId,                                                                                              // 205
        notificationId: notificationId,                                                                                // 206
        appActivity: appActivity                                                                                       // 207
      };                                                                                                               // 204
      createAppActivityDesktopNotification(desktopNotification);                                                       // 210
    });                                                                                                                // 211
  }                                                                                                                    // 212
};                                                                                                                     // 213
                                                                                                                       //
Meteor.methods({                                                                                                       // 215
  testNotifications: function () {                                                                                     // 216
    // Deliver a test notification of each non-grain-initiated type to the user.                                       // 217
    if (!this.userId) return;                                                                                          // 218
    Notifications.insert({                                                                                             // 220
      admin: {                                                                                                         // 221
        action: "/admin/stats",                                                                                        // 222
        type: "reportStats"                                                                                            // 223
      },                                                                                                               // 221
      userId: this.userId,                                                                                             // 225
      timestamp: new Date(),                                                                                           // 226
      isUnread: true                                                                                                   // 227
    });                                                                                                                // 220
    Notifications.insert({                                                                                             // 230
      userId: this.userId,                                                                                             // 231
      referral: true,                                                                                                  // 232
      timestamp: new Date(),                                                                                           // 233
      isUnread: true                                                                                                   // 234
    });                                                                                                                // 230
    Notifications.insert({                                                                                             // 237
      userId: this.userId,                                                                                             // 238
      identityChanges: true,                                                                                           // 239
      timestamp: new Date(),                                                                                           // 240
      isUnread: true                                                                                                   // 241
    });                                                                                                                // 237
                                                                                                                       //
    if (global.BlackrockPayments) {                                                                                    // 244
      Notifications.insert({                                                                                           // 245
        userId: this.userId,                                                                                           // 246
        mailingListBonus: true,                                                                                        // 247
        timestamp: new Date(),                                                                                         // 248
        isUnread: true                                                                                                 // 249
      });                                                                                                              // 245
    }                                                                                                                  // 251
  }                                                                                                                    // 252
});                                                                                                                    // 215
Meteor.publish("notifications", function () {                                                                          // 255
  return Notifications.find({                                                                                          // 256
    userId: this.userId                                                                                                // 256
  });                                                                                                                  // 256
});                                                                                                                    // 257
Meteor.publish("notificationGrains", function (notificationIds) {                                                      // 259
  // Since publishes can't be reactive, we leave it to the client to subscribe to both                                 // 260
  // "notifications" and "notificationGrains" reactively.                                                              // 261
  check(notificationIds, [String]);                                                                                    // 262
  var notifications = Notifications.find({                                                                             // 263
    _id: {                                                                                                             // 264
      $in: notificationIds                                                                                             // 264
    },                                                                                                                 // 264
    userId: this.userId                                                                                                // 265
  }, {                                                                                                                 // 263
    fields: {                                                                                                          // 267
      grainId: 1,                                                                                                      // 267
      initiatingAccount: 1                                                                                             // 267
    }                                                                                                                  // 267
  });                                                                                                                  // 266
  var grainIds = notifications.map(function (row) {                                                                    // 270
    return row.grainId;                                                                                                // 271
  }).filter(function (x) {                                                                                             // 272
    return x;                                                                                                          // 272
  });                                                                                                                  // 272
  var accounts = notifications.map(function (row) {                                                                    // 274
    return row.initiatingAccount;                                                                                      // 275
  }).filter(function (x) {                                                                                             // 276
    return x;                                                                                                          // 276
  });                                                                                                                  // 276
  return [Meteor.users.find({                                                                                          // 278
    _id: {                                                                                                             // 279
      $in: accounts                                                                                                    // 279
    }                                                                                                                  // 279
  }, {                                                                                                                 // 279
    fields: {                                                                                                          // 279
      type: 1,                                                                                                         // 279
      profile: 1                                                                                                       // 279
    }                                                                                                                  // 279
  })];                                                                                                                 // 279
});                                                                                                                    // 281
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"pre-meteor.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/pre-meteor.js                                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var module1 = module;                                                                                                  // 1
var inMeteor = void 0;                                                                                                 // 1
module1.watch(require("/imports/server/async-helpers.js"), {                                                           // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 21
                                                                                                                       //
var Fs = Npm.require("fs");                                                                                            // 22
                                                                                                                       //
var Dns = Npm.require("dns");                                                                                          // 23
                                                                                                                       //
var Future = Npm.require("fibers/future");                                                                             // 24
                                                                                                                       //
var Http = Npm.require("http");                                                                                        // 25
                                                                                                                       //
var ByteStream = Capnp.importSystem("sandstorm/util.capnp").ByteStream;                                                // 26
var HOSTNAME = Url.parse(process.env.ROOT_URL).hostname;                                                               // 28
var DDP_HOSTNAME = process.env.DDP_DEFAULT_CONNECTION_URL && Url.parse(process.env.DDP_DEFAULT_CONNECTION_URL).hostname;
var CACHE_TTL_SECONDS = 30; // 30 seconds.  Cache-Control expects units of seconds, not millis.                        // 31
                                                                                                                       //
var DNS_CACHE_TTL = CACHE_TTL_SECONDS * 1000; // DNS cache is in millis.                                               // 32
                                                                                                                       //
var webPublishingHandlers = {}; // Maps grain public IDs to Connect handlers.                                          // 34
// TODO(perf): Garbage-collect this map?                                                                               // 36
// TODO(perf): Implement a more general cache not just of web publishing handlers but of all sorts                     // 38
//   of hosts, in order to avoid database lookups to determine how to handle requests. Host types                      // 39
//   to distinguish between:                                                                                           // 40
//   - Grain hosts                                                                                                     // 41
//   - Web publishing hosts                                                                                            // 42
//   - Standalone grain hosts                                                                                          // 43
                                                                                                                       //
var dnsCache = {}; // Unfortunately, node's DNS library doesn't cache results, so we do our own caching.               // 45
// Unfortunately, node's DNS library also dosen't give us TTLs. So, we'll cache for                                    // 47
// DNS_CACHE_TTL (a relatively small value) and rely on the upstream DNS server to implement                           // 48
// better caching.                                                                                                     // 49
                                                                                                                       //
function isSandstormShell(hostname) {                                                                                  // 51
  // Is this hostname mapped to the Sandstorm shell?                                                                   // 52
  return hostname === HOSTNAME || DDP_HOSTNAME && hostname === DDP_HOSTNAME;                                           // 54
} // We need to use connect. Let's make sure we're using the same version as Meteor's WebApp module                    // 55
// uses. Fortunately, they let us extract it.                                                                          // 58
                                                                                                                       //
                                                                                                                       //
var connect = WebAppInternals.NpmModules.connect.module;                                                               // 59
var mime = connect.static.mime;                                                                                        // 61
                                                                                                                       //
function wwwHandlerForGrain(grainId) {                                                                                 // 63
  return function (request, response) {                                                                                // 64
    var path = request.url; // Strip query.                                                                            // 65
                                                                                                                       //
    path = path.split("?")[0]; // If a directory, open "index.html".                                                   // 68
                                                                                                                       //
    if (path.slice(-1) === "/") {                                                                                      // 71
      path = path + "index.html";                                                                                      // 72
    } // Strip leading "/".                                                                                            // 73
                                                                                                                       //
                                                                                                                       //
    if (path[0] === "/") path = path.slice(1); // URI-decode the rest. Note that this allows filenames to contain spaces and question marks.
                                                                                                                       //
    var encoded = path;                                                                                                // 79
    path = decodeURIComponent(encoded);                                                                                // 80
    var type = mime.lookup(path);                                                                                      // 82
    var charset = mime.charsets.lookup(type);                                                                          // 83
                                                                                                                       //
    if (charset) {                                                                                                     // 84
      type = type + "; charset=" + charset;                                                                            // 85
    } else if (type === "application/json") {                                                                          // 86
      // HACK: Apparently the MIME module does not assume UTF-8 for JSON. :(                                           // 87
      type = type + "; charset=utf-8";                                                                                 // 88
    } // TODO(perf): Automatically gzip text content? Use Express's "compress" middleware for this?                    // 89
    //   Note that nginx will also auto-compress things...                                                             // 92
                                                                                                                       //
                                                                                                                       //
    response.setHeader("Content-Type", type);                                                                          // 94
    response.setHeader("Cache-Control", "public, max-age=" + CACHE_TTL_SECONDS);                                       // 95
                                                                                                                       //
    if (path === "apps/index.json" || path.match(/apps\/[a-z0-9]{52}[.]json/) || path === "experimental/index.json" || path.match(/experimental\/[a-z0-9]{52}[.]json/)) {
      // TODO(cleanup): Extra special terrible hack: The app index needs to serve these JSON files                     // 101
      //   cross-origin. We could almost just make all web sites allow cross-origin since generally                    // 102
      //   web publishing is meant to publish public content. There is one case where this is                          // 103
      //   problematic, though: sites behind a firewall. Those sites could potentially be read                         // 104
      //   by outside sites if CORS is enabled on them. Some day we should make it so apps can                         // 105
      //   explicitly opt-in to allowing cross-origin queries but that day is not today.                               // 106
      response.setHeader("Access-Control-Allow-Origin", "*");                                                          // 107
    }                                                                                                                  // 108
                                                                                                                       //
    var stream = new ResponseStream(response, 200, "OK"); // We hold our own capability to the stream becaues we don't want ResponseStream's close()
    // logic to happen if the response has no data.                                                                    // 113
    //                                                                                                                 // 114
    // TODO(cleanup): This is pretty ugly.                                                                             // 115
                                                                                                                       //
    var streamCap = new Capnp.Capability(stream, ByteStream);                                                          // 116
    globalBackend.useGrain(grainId, function (supervisor) {                                                            // 118
      var tryPath = function (path) {                                                                                  // 119
        return supervisor.getWwwFileHack(path, streamCap).then(function (result) {                                     // 119
          // jscs:disable disallowQuotedKeysInObjects                                                                  // 120
          var status = result.status;                                                                                  // 121
                                                                                                                       //
          if (status === "file") {// Nothing to do.                                                                    // 122
          } else if (status === "directory") {                                                                         // 124
            stream.sendingDirectResponse();                                                                            // 125
            response.writeHead(303, {                                                                                  // 126
              "Content-Type": "text/plain",                                                                            // 127
              "Location": "/" + path + "/",                                                                            // 128
              "Cache-Control": "public, max-age=" + CACHE_TTL_SECONDS                                                  // 129
            });                                                                                                        // 126
            response.end("redirect: /" + path + "/");                                                                  // 131
          } else if (status === "notFound") {                                                                          // 132
            if (path !== encoded) {                                                                                    // 133
              // Compatibility hack: Try getting the URL-encoded version of the path. We used to not                   // 134
              // URL-decode paths at all, hence the need for this fallback.                                            // 135
              // TODO(apibump): Move this hack to the compat layer.                                                    // 136
              return tryPath(encoded);                                                                                 // 137
            }                                                                                                          // 138
                                                                                                                       //
            stream.sendingDirectResponse();                                                                            // 140
            response.writeHead(404, {                                                                                  // 141
              "Content-Type": "text/plain"                                                                             // 142
            });                                                                                                        // 141
            response.end("404 not found: /" + path);                                                                   // 144
          } else {                                                                                                     // 145
            console.error("didn't understand result of getWwwFileHack:", status);                                      // 146
            stream.sendingDirectResponse();                                                                            // 147
            response.writeHead(500, {                                                                                  // 148
              "Content-Type": "text/plain"                                                                             // 149
            });                                                                                                        // 148
            response.end("Internal server error");                                                                     // 151
          }                                                                                                            // 152
                                                                                                                       //
          streamCap.close();                                                                                           // 154
        });                                                                                                            // 155
      };                                                                                                               // 119
                                                                                                                       //
      return tryPath(path);                                                                                            // 156
    }).catch(function (err) {                                                                                          // 157
      console.error(err.stack);                                                                                        // 158
      streamCap.close();                                                                                               // 159
    });                                                                                                                // 160
  };                                                                                                                   // 161
}                                                                                                                      // 162
                                                                                                                       //
function writeErrorResponse(res, err) {                                                                                // 164
  var status = 500;                                                                                                    // 165
                                                                                                                       //
  if (err instanceof Meteor.Error && typeof err.error === "number" && err.error >= 400 && err.error < 600) {           // 166
    status = err.error;                                                                                                // 168
  } else if (err.httpErrorCode) {                                                                                      // 169
    status = err.httpErrorCode;                                                                                        // 170
  } // Log errors that are our fault, but not errors that are the client's fault.                                      // 171
                                                                                                                       //
                                                                                                                       //
  if (status >= 500) console.error(err.stack);                                                                         // 174
  res.writeHead(status, {                                                                                              // 176
    "Content-Type": err.htmlMessage ? "text/html" : "text/plain"                                                       // 176
  });                                                                                                                  // 176
  res.end(err.htmlMessage || err.message);                                                                             // 177
}                                                                                                                      // 178
                                                                                                                       //
var PNG_MAGIC = new Buffer([0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A]);                                          // 180
var JPEG_MAGIC = new Buffer([0xFF, 0xD8, 0xFF]);                                                                       // 181
                                                                                                                       //
function checkMagic(buf, magic) {                                                                                      // 183
  if (buf.length < magic.length) return false;                                                                         // 184
                                                                                                                       //
  for (var i = 0; i < magic.length; i++) {                                                                             // 186
    if (buf[i] != magic[i]) return false;                                                                              // 187
  }                                                                                                                    // 188
                                                                                                                       //
  return true;                                                                                                         // 190
}                                                                                                                      // 191
                                                                                                                       //
function serveSelfTest(req, res) {                                                                                     // 193
  try {                                                                                                                // 194
    if (req.method === "GET" && req.url === "/") {                                                                     // 195
      var content = new Buffer("Self-test OK."); // Convert the ROOT_URL to something that is a valid origin.          // 197
                                                                                                                       //
      var rootUrlAsOrigin = process.env.ROOT_URL;                                                                      // 200
                                                                                                                       //
      if (rootUrlAsOrigin.slice(-1) === "/") {                                                                         // 201
        rootUrlAsOrigin = rootUrlAsOrigin.slice(0, -1);                                                                // 202
      }                                                                                                                // 203
                                                                                                                       //
      res.writeHead(200, {                                                                                             // 205
        "Content-Type": "text/plain",                                                                                  // 206
        "Content-Length": content.length,                                                                              // 207
        "Access-Control-Allow-Origin": rootUrlAsOrigin                                                                 // 208
      });                                                                                                              // 205
      res.end(content);                                                                                                // 210
    } else {                                                                                                           // 211
      res.writeHead(400, {                                                                                             // 212
        "Content-Type": "text/plain"                                                                                   // 212
      });                                                                                                              // 212
      res.end("Bad request to self-test subdomain.");                                                                  // 213
      return;                                                                                                          // 214
    }                                                                                                                  // 215
  } catch (err) {                                                                                                      // 216
    console.log("WARNING: An error occurred while serving self-test; proceeding anyway:", err);                        // 217
  }                                                                                                                    // 218
}                                                                                                                      // 219
                                                                                                                       //
function serveStaticAsset(req, res, hostId) {                                                                          // 221
  inMeteor(function () {                                                                                               // 222
    if (req.method === "GET") {                                                                                        // 223
      var assetCspHeader = "default-src 'none'; style-src 'unsafe-inline'; sandbox";                                   // 224
                                                                                                                       //
      if (req.headers["if-none-match"] === "permanent") {                                                              // 225
        // Cache never invalidates since we use a new URL for every resource.                                          // 226
        res.writeHead(304, {                                                                                           // 227
          "Cache-Control": "public, max-age=31536000",                                                                 // 228
          "ETag": "permanent",                                                                                         // 229
          // To be safe, send these again, although it shouldn't be necessary.                                         // 231
          "Content-Security-Policy": assetCspHeader,                                                                   // 232
          "Access-Control-Allow-Origin": "*",                                                                          // 233
          "X-Content-Type-Options": "nosniff"                                                                          // 234
        });                                                                                                            // 227
        res.end();                                                                                                     // 236
        return;                                                                                                        // 237
      }                                                                                                                // 238
                                                                                                                       //
      var url = Url.parse(req.url, true);                                                                              // 240
      var pathname = url.pathname.slice(1);                                                                            // 241
      var asset = void 0;                                                                                              // 242
                                                                                                                       //
      if (pathname.startsWith("identicon/")) {                                                                         // 243
        var size = parseInt((url.query || {}).s);                                                                      // 244
        asset = new Identicon(pathname.slice("identicon/".length), size).asAsset();                                    // 245
      } else if (pathname.indexOf("/") == -1) {                                                                        // 246
        asset = globalDb.getStaticAsset(pathname);                                                                     // 247
      }                                                                                                                // 248
                                                                                                                       //
      if (asset) {                                                                                                     // 250
        var headers = {                                                                                                // 251
          "Content-Type": asset.mimeType,                                                                              // 252
          "Content-Length": asset.content.length,                                                                      // 253
          // Assets can be cached forever because each one has a unique ID.                                            // 255
          "Cache-Control": "public, max-age=31536000",                                                                 // 256
          // Since different resources get different URLs, we can use a static etag.                                   // 258
          "ETag": "permanent",                                                                                         // 259
          // Set strict Content-Security-Policy to prevent static assets from executing any script                     // 261
          // or doing basically anything when browsed to directly. The static assets host is not                       // 262
          // intended to serve HTML. Mostly, it serves images and javascript -- note that setting                      // 263
          // the CSP header on Javascript files does not prevent other hosts from voluntarily                          // 264
          // specifying these scripts in <script> tags.                                                                // 265
          "Content-Security-Policy": assetCspHeader,                                                                   // 266
          // Allow any host to fetch these assets. This is safe since requests to this host are                        // 268
          // totally side-effect-free and the asset ID acts as a capability to prevent loading                         // 269
          // assets you're not supposed to know about.                                                                 // 270
          "Access-Control-Allow-Origin": "*",                                                                          // 271
          // Extra protection against content type trickery.                                                           // 273
          "X-Content-Type-Options": "nosniff"                                                                          // 274
        };                                                                                                             // 251
                                                                                                                       //
        if (asset.encoding) {                                                                                          // 277
          headers["Content-Encoding"] = asset.encoding;                                                                // 278
        }                                                                                                              // 279
                                                                                                                       //
        res.writeHead(200, headers);                                                                                   // 281
        res.end(asset.content);                                                                                        // 282
      } else {                                                                                                         // 283
        res.writeHead(404, {                                                                                           // 284
          "Content-Type": "text/plain"                                                                                 // 284
        });                                                                                                            // 284
        res.end("not found");                                                                                          // 285
      }                                                                                                                // 286
    } else if (req.method === "POST") {                                                                                // 287
      res.setHeader("Access-Control-Allow-Origin", "*");                                                               // 288
                                                                                                                       //
      var _url = Url.parse(req.url);                                                                                   // 290
                                                                                                                       //
      var purpose = globalDb.fulfillAssetUpload(_url.pathname.slice(1)); // Sanity check the purpose of this upload token.
                                                                                                                       //
      if (!purpose) {                                                                                                  // 294
        res.writeHead(404, {                                                                                           // 295
          "Content-Type": "text/plain"                                                                                 // 295
        });                                                                                                            // 295
        res.end("Upload token not found or expired.");                                                                 // 296
        return;                                                                                                        // 297
      } else if (purpose.profilePicture) {                                                                             // 298
        var userId = purpose.profilePicture.userId;                                                                    // 299
        check(userId, String);                                                                                         // 300
      } else if (purpose.loginLogo) {// no additional fields for loginLogo                                             // 301
      } else {                                                                                                         // 303
        res.writeHead(500, {                                                                                           // 304
          "Content-Type": "text/plain"                                                                                 // 304
        });                                                                                                            // 304
        res.end("Found invalid purpose for upload");                                                                   // 305
        return;                                                                                                        // 306
      }                                                                                                                // 307
                                                                                                                       //
      var buffers = [];                                                                                                // 309
      var totalSize = 0;                                                                                               // 310
      var done = new Future();                                                                                         // 311
      req.on("data", function (buf) {                                                                                  // 312
        totalSize += buf.length;                                                                                       // 313
                                                                                                                       //
        if (totalSize <= 64 * 1024) {                                                                                  // 314
          buffers.push(buf);                                                                                           // 315
        }                                                                                                              // 316
      });                                                                                                              // 317
      req.on("end", done.return.bind(done));                                                                           // 318
      req.on("error", done.throw.bind(done));                                                                          // 319
      done.wait();                                                                                                     // 320
                                                                                                                       //
      if (totalSize > 64 * 1024) {                                                                                     // 322
        // TODO(soon): Resize the image ourselves.                                                                     // 323
        res.writeHead(400, {                                                                                           // 324
          "Content-Type": "text/plain"                                                                                 // 324
        });                                                                                                            // 324
        res.end("Picture too large; please use an image under 64 KiB.");                                               // 325
        return;                                                                                                        // 326
      }                                                                                                                // 327
                                                                                                                       //
      var content = Buffer.concat(buffers);                                                                            // 329
      var type = void 0;                                                                                               // 330
                                                                                                                       //
      if (checkMagic(content, PNG_MAGIC)) {                                                                            // 331
        type = "image/png";                                                                                            // 332
      } else if (checkMagic(content, JPEG_MAGIC)) {                                                                    // 333
        type = "image/jpeg";                                                                                           // 334
      } else {                                                                                                         // 335
        res.writeHead(400, {                                                                                           // 336
          "Content-Type": "text/plain"                                                                                 // 336
        });                                                                                                            // 336
        res.end("Image must be PNG or JPEG.");                                                                         // 337
        return;                                                                                                        // 338
      }                                                                                                                // 339
                                                                                                                       //
      var assetId = globalDb.addStaticAsset({                                                                          // 341
        mimeType: type                                                                                                 // 341
      }, content);                                                                                                     // 341
                                                                                                                       //
      if (purpose.profilePicture) {                                                                                    // 343
        var accountId = purpose.profilePicture.userId;                                                                 // 344
        var result = Meteor.users.findAndModify({                                                                      // 345
          query: {                                                                                                     // 346
            _id: accountId                                                                                             // 346
          },                                                                                                           // 346
          update: {                                                                                                    // 347
            $set: {                                                                                                    // 347
              "profile.picture": assetId                                                                               // 347
            }                                                                                                          // 347
          },                                                                                                           // 347
          fields: {                                                                                                    // 348
            "profile.picture": 1                                                                                       // 348
          }                                                                                                            // 348
        });                                                                                                            // 345
                                                                                                                       //
        if (result.ok) {                                                                                               // 351
          var old = result.value;                                                                                      // 352
                                                                                                                       //
          if (old && old.profile && old.profile.picture) {                                                             // 353
            globalDb.unrefStaticAsset(old.profile.picture);                                                            // 354
          }                                                                                                            // 355
        } else {                                                                                                       // 356
          res.writeHead(500, {                                                                                         // 357
            "Content-Type": "text/plain"                                                                               // 357
          });                                                                                                          // 357
          res.end("Couldn't update profile picture.");                                                                 // 358
          return;                                                                                                      // 359
        }                                                                                                              // 360
      } else if (purpose.loginLogo) {                                                                                  // 361
        var _result = globalDb.collections.settings.findAndModify({                                                    // 362
          query: {                                                                                                     // 363
            _id: "whitelabelCustomLogoAssetId"                                                                         // 363
          },                                                                                                           // 363
          update: {                                                                                                    // 364
            $setOnInsert: {                                                                                            // 365
              _id: "whitelabelCustomLogoAssetId"                                                                       // 365
            },                                                                                                         // 365
            $set: {                                                                                                    // 366
              value: assetId                                                                                           // 366
            }                                                                                                          // 366
          },                                                                                                           // 364
          upsert: true,                                                                                                // 368
          fields: {                                                                                                    // 369
            value: 1                                                                                                   // 369
          }                                                                                                            // 369
        });                                                                                                            // 362
                                                                                                                       //
        if (_result.ok) {                                                                                              // 372
          var _old = _result.value;                                                                                    // 373
                                                                                                                       //
          if (_old && _old.value) {                                                                                    // 374
            globalDb.unrefStaticAsset(_old.value);                                                                     // 375
          }                                                                                                            // 376
        } else {                                                                                                       // 377
          res.writeHead(500, {                                                                                         // 378
            "Content-Type": "text/plain"                                                                               // 378
          });                                                                                                          // 378
          res.end("Couldn't update login logo.");                                                                      // 379
          return;                                                                                                      // 380
        }                                                                                                              // 381
      }                                                                                                                // 382
                                                                                                                       //
      res.writeHead(204, {});                                                                                          // 384
      res.end();                                                                                                       // 385
    } else if (req.method === "OPTIONS") {                                                                             // 386
      var requestedHeaders = req.headers["access-control-request-headers"];                                            // 387
                                                                                                                       //
      if (requestedHeaders) {                                                                                          // 388
        res.setHeader("Access-Control-Allow-Headers", requestedHeaders);                                               // 389
      }                                                                                                                // 390
                                                                                                                       //
      res.writeHead(204, {                                                                                             // 392
        "Access-Control-Allow-Origin": "*",                                                                            // 393
        "Access-Control-Allow-Methods": "GET, HEAD, POST, OPTIONS",                                                    // 394
        "Access-Control-Max-Age": "3600"                                                                               // 395
      });                                                                                                              // 392
      res.end();                                                                                                       // 397
    } else {                                                                                                           // 398
      res.writeHead(405, "Method Not Allowed", {                                                                       // 399
        "Allow": "GET, POST, OPTIONS",                                                                                 // 400
        "Content-Type": "text/plain"                                                                                   // 401
      });                                                                                                              // 399
      res.end("405 Method Not Allowed: " + req.method);                                                                // 403
    }                                                                                                                  // 404
  }).catch(function (err) {                                                                                            // 405
    writeErrorResponse(res, err);                                                                                      // 406
  });                                                                                                                  // 407
}                                                                                                                      // 408
                                                                                                                       //
var canonicalizeShellOrWildcardUrl = function (hostname, url) {                                                        // 410
  // Start with ROOT_URL, apply host & path from inbound URL, then                                                     // 411
  // redirect.                                                                                                         // 412
  var targetUrl = Url.parse(process.env.ROOT_URL); // Retain the protocol & port from ROOT_URL but use the inbound     // 413
  // hostname.                                                                                                         // 416
                                                                                                                       //
  targetUrl.host = hostname + ":" + targetUrl.port; // The following allows to avoid decoding + re-encoding query      // 417
  // string parameters, if provided.                                                                                   // 420
                                                                                                                       //
  targetUrl = Url.resolve(Url.format(targetUrl), url);                                                                 // 421
  return targetUrl;                                                                                                    // 422
};                                                                                                                     // 423
                                                                                                                       //
var handleNonMeteorRequest = function (req, res, next, redirectIfInWildcard) {                                         // 425
  // if redirectIfInWildcard is true, and if the host is part of the wildcard host, then we will                       // 426
  // prefer to redirect to the canonical url for that request.  If false, we will respond to the                       // 427
  // request directly.                                                                                                 // 428
  var hostname = req.headers.host.split(":")[0];                                                                       // 430
  var publicIdPromise = void 0; // See if the request was for a host in the wildcard.                                  // 432
                                                                                                                       //
  var id = matchWildcardHost(req.headers.host);                                                                        // 434
                                                                                                                       //
  if (id) {                                                                                                            // 435
    // Match!                                                                                                          // 436
    if (redirectIfInWildcard) {                                                                                        // 437
      res.writeHead(302, {                                                                                             // 438
        "Location": canonicalizeShellOrWildcardUrl(hostname, req.url)                                                  // 438
      });                                                                                                              // 438
      res.end();                                                                                                       // 439
      return;                                                                                                          // 440
    }                                                                                                                  // 441
                                                                                                                       //
    if (id === "static") {                                                                                             // 443
      // Static assets domain.                                                                                         // 444
      serveStaticAsset(req, res);                                                                                      // 445
      return;                                                                                                          // 446
    }                                                                                                                  // 447
                                                                                                                       //
    if (id.match(/^selftest-/)) {                                                                                      // 449
      // Self test domain pattern. Starts w/ hyphen to avoid ambiguity with grain session/static                       // 450
      // publishing wildcard hosts.                                                                                    // 451
      serveSelfTest(req, res);                                                                                         // 452
      return;                                                                                                          // 453
    } // Try to route the request to a session.                                                                        // 454
                                                                                                                       //
                                                                                                                       //
    publicIdPromise = tryProxyRequest(id, req, res).then(function (handled) {                                          // 457
      if (handled) {                                                                                                   // 458
        return null;                                                                                                   // 459
      } else {                                                                                                         // 460
        return id;                                                                                                     // 461
      }                                                                                                                // 462
    });                                                                                                                // 463
  } else {                                                                                                             // 464
    // Not a wildcard host. Perhaps it is a custom host.                                                               // 465
    publicIdPromise = lookupPublicIdFromDns(hostname);                                                                 // 466
  }                                                                                                                    // 467
                                                                                                                       //
  publicIdPromise.then(function (publicId) {                                                                           // 469
    if (publicId) {                                                                                                    // 470
      return Promise.resolve(undefined).then(function () {                                                             // 471
        var handler = webPublishingHandlers[publicId];                                                                 // 472
                                                                                                                       //
        if (handler) {                                                                                                 // 473
          return handler;                                                                                              // 474
        } else {                                                                                                       // 475
          // We don't have a handler for this publicId, so look it up in the grain DB.                                 // 476
          return inMeteor(function () {                                                                                // 477
            var grain = Grains.findOne({                                                                               // 478
              publicId: publicId                                                                                       // 478
            }, {                                                                                                       // 478
              fields: {                                                                                                // 478
                _id: 1                                                                                                 // 478
              }                                                                                                        // 478
            });                                                                                                        // 478
                                                                                                                       //
            if (!grain) {                                                                                              // 479
              throw new Meteor.Error(404, "No such grain for public ID: " + publicId);                                 // 480
            }                                                                                                          // 481
                                                                                                                       //
            var grainId = grain._id;                                                                                   // 483
            return webPublishingHandlers[publicId] = wwwHandlerForGrain(grainId);                                      // 484
          });                                                                                                          // 485
        }                                                                                                              // 486
      }).then(function (handler) {                                                                                     // 487
        handler(req, res);                                                                                             // 488
      });                                                                                                              // 489
    } else {                                                                                                           // 490
      // Already handled by tryProxyRequest above, no further action needed.                                           // 491
      return Promise.resolve(undefined);                                                                               // 492
    }                                                                                                                  // 493
  }).catch(function (err) {                                                                                            // 494
    writeErrorResponse(res, err);                                                                                      // 495
  });                                                                                                                  // 496
}; // This function serves responses on Sandstorm's main HTTP/HTTPS                                                    // 497
// port.                                                                                                               // 500
                                                                                                                       //
                                                                                                                       //
var handleNonMeteorRequestDirectly = function (req, res, next) {                                                       // 501
  handleNonMeteorRequest(req, res, next, false);                                                                       // 502
}; // "Alternate ports" are ports other than the main HTTP or HTTPS port.                                              // 503
// They are handled as follows:                                                                                        // 506
//                                                                                                                     // 507
// * Requests to the shell & grains, we redirect to the main port.                                                     // 508
// * For requests to static publishing on hosts within the wildcard, we redirect to the main port.                     // 509
// * For static publishing glued in via DNS TXT record, we serve the request as-is.                                    // 510
//                                                                                                                     // 511
// Alternate ports are bound to FD #5 and higher.                                                                      // 512
                                                                                                                       //
                                                                                                                       //
var getNumberOfAlternatePorts = function () {                                                                          // 513
  var numPorts = process.env.PORTS.split(",").length;                                                                  // 514
  var numAlternatePorts = numPorts - 1;                                                                                // 515
  return numAlternatePorts;                                                                                            // 516
};                                                                                                                     // 517
                                                                                                                       //
var handleNonMainPortRequest = function (req, res, next) {                                                             // 519
  try {                                                                                                                // 520
    if (!req.headers.host) {                                                                                           // 521
      res.writeHead(400, {                                                                                             // 522
        "Content-Type": "text/plain"                                                                                   // 522
      });                                                                                                              // 522
      res.end("Missing Host header");                                                                                  // 523
      return;                                                                                                          // 524
    } // If this request is intended for the shell, redirect to the canonical shell URL.                               // 525
                                                                                                                       //
                                                                                                                       //
    var hostname = req.headers.host.split(":")[0];                                                                     // 528
                                                                                                                       //
    if (isSandstormShell(hostname)) {                                                                                  // 529
      res.writeHead(302, {                                                                                             // 530
        "Location": canonicalizeShellOrWildcardUrl(hostname, req.url)                                                  // 530
      });                                                                                                              // 530
      res.end();                                                                                                       // 531
      return;                                                                                                          // 532
    } // TODO(someday): check for standalone hosts and do something reasonable                                         // 533
                                                                                                                       //
                                                                                                                       //
    handleNonMeteorRequest(req, res, next, true);                                                                      // 536
  } catch (e) {                                                                                                        // 537
    // This should never be reached, because all the request handlers should be catching                               // 538
    // exceptions, but you can never be too careful in a top-level request handler.                                    // 539
    console.err("Unhandled exception in request handler:", e.stack);                                                   // 540
    res.writeHead(500, {                                                                                               // 541
      "Content-Type": "text/plain"                                                                                     // 541
    });                                                                                                                // 541
    res.end("Unhandled exception: " + e.stack);                                                                        // 542
  }                                                                                                                    // 543
};                                                                                                                     // 544
                                                                                                                       //
var listenOnAlternatePorts = function () {                                                                             // 546
  for (var i = 0; i < getNumberOfAlternatePorts(); i++) {                                                              // 547
    // Call createServerForSandstorm() to skip our monkey patching.                                                    // 548
    var alternatePortServer = Http.createServerForSandstorm(handleNonMainPortRequest);                                 // 549
    alternatePortServer.listen({                                                                                       // 550
      fd: i + 5                                                                                                        // 550
    });                                                                                                                // 550
  }                                                                                                                    // 551
};                                                                                                                     // 552
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 554
  var meteorRequestListeners = WebApp.httpServer.listeners("request"); // Construct the middleware chain for requests to non-DDP, non-shell hosts.
                                                                                                                       //
  var nonMeteorRequestHandler = connect(); // BlackrockPayments is only defined in the Blackrock build of Sandstorm.   // 558
                                                                                                                       //
  if (global.BlackrockPayments) {                                                                                      // 560
    // Have to check with global, because it could be undefined.                                                       // 560
    nonMeteorRequestHandler.use(BlackrockPayments.makeConnectHandler(globalDb));                                       // 561
  }                                                                                                                    // 562
                                                                                                                       //
  nonMeteorRequestHandler.use(handleNonMeteorRequestDirectly);                                                         // 564
  WebApp.httpServer.removeAllListeners("request");                                                                     // 566
  WebApp.httpServer.on("request", function (req, res) {                                                                // 567
    Promise.resolve(undefined).then(function () {                                                                      // 568
      if (!req.headers.host) {                                                                                         // 569
        res.writeHead(400, {                                                                                           // 570
          "Content-Type": "text/plain"                                                                                 // 570
        });                                                                                                            // 570
        res.end("Missing Host header");                                                                                // 571
        return;                                                                                                        // 572
      }                                                                                                                // 573
                                                                                                                       //
      var hostname = req.headers.host.split(":")[0]; // strip port if it exists                                        // 575
                                                                                                                       //
      if (isSandstormShell(hostname)) {                                                                                // 576
        if (req.url.startsWith("/_oauth/")) {                                                                          // 577
          // Intercept oauth callbacks on the main host to check if the user was actually logging                      // 578
          // into a standalone host, in which case we need to redirect to that host instead.                           // 579
          var parsedUrl = Url.parse(req.url, true);                                                                    // 580
                                                                                                                       //
          if (parsedUrl.query && parsedUrl.query.state) {                                                              // 581
            var rawState = new Buffer(parsedUrl.query.state, "base64");                                                // 582
            var state = JSON.parse(rawState.toString());                                                               // 583
                                                                                                                       //
            if (state.redirectUrl) {                                                                                   // 584
              var parsedRedirect = Url.parse(state.redirectUrl);                                                       // 585
              var redirectHostname = parsedRedirect.hostname;                                                          // 586
                                                                                                                       //
              if (redirectHostname !== HOSTNAME) {                                                                     // 587
                return inMeteor(function () {                                                                          // 588
                  if (globalDb.hostIsStandalone(redirectHostname)) {                                                   // 589
                    res.writeHead(302, {                                                                               // 590
                      "Location": parsedRedirect.protocol + "//" + parsedRedirect.host + req.url                       // 590
                    });                                                                                                // 590
                    res.end();                                                                                         // 592
                    return;                                                                                            // 593
                  } else {                                                                                             // 594
                    throw new Meteor.Error(400, "redirectUrl in OAuth was for an unknown host: " + state.redirectUrl);
                  }                                                                                                    // 597
                });                                                                                                    // 598
              }                                                                                                        // 599
            }                                                                                                          // 600
          }                                                                                                            // 601
        } // If destined for the DDP host or the main host, pass on to Meteor                                          // 602
                                                                                                                       //
                                                                                                                       //
        for (var i = 0; i < meteorRequestListeners.length; i++) {                                                      // 604
          meteorRequestListeners[i](req, res);                                                                         // 605
        }                                                                                                              // 606
      } else {                                                                                                         // 607
        return inMeteor(function () {                                                                                  // 608
          if (globalDb.hostIsStandalone(hostname)) {                                                                   // 609
            // If it's a standalone host, also pass on to meteor                                                       // 610
            for (var _i = 0; _i < meteorRequestListeners.length; _i++) {                                               // 611
              meteorRequestListeners[_i](req, res);                                                                    // 612
            }                                                                                                          // 613
          } else {                                                                                                     // 614
            // Otherwise, dispatch to our own middleware proxy chain.                                                  // 615
            nonMeteorRequestHandler(req, res); // Adjust timeouts on proxied requests to allow apps to long-poll if needed.
                                                                                                                       //
            WebApp._timeoutAdjustmentRequestCallback(req, res);                                                        // 618
          }                                                                                                            // 619
        });                                                                                                            // 620
      }                                                                                                                // 621
    }).catch(function (e) {                                                                                            // 622
      // This should never be reached, because all the request handlers should be catching                             // 623
      // exceptions, but you can never be too careful in a top-level request handler.                                  // 624
      console.error("Unhandled exception in request handler:", e.stack);                                               // 625
      res.writeHead(500, {                                                                                             // 626
        "Content-Type": "text/plain"                                                                                   // 626
      });                                                                                                              // 626
      res.end("Unhandled exception: " + e.stack);                                                                      // 627
    });                                                                                                                // 628
  });                                                                                                                  // 629
  var meteorUpgradeListeners = WebApp.httpServer.listeners("upgrade");                                                 // 631
  WebApp.httpServer.removeAllListeners("upgrade");                                                                     // 632
  WebApp.httpServer.on("upgrade", function (req, socket, head) {                                                       // 634
    Promise.resolve(undefined).then(function () {                                                                      // 635
      if (!req.headers.host) {                                                                                         // 636
        throw new Meteor.Error(400, "Missing Host header");                                                            // 637
      } else if (isSandstormShell(req.headers.host.split(":")[0])) {                                                   // 638
        // Go on to Meteor.                                                                                            // 639
        for (var ii = 0; ii < meteorUpgradeListeners.length; ++ii) {                                                   // 640
          meteorUpgradeListeners[ii](req, socket, head);                                                               // 641
        }                                                                                                              // 642
                                                                                                                       //
        return true;                                                                                                   // 644
      } else {                                                                                                         // 645
        return inMeteor(function () {                                                                                  // 646
          // TODO(someday): This is slightly sad that some requests will now need a Meteor fiber                       // 647
          // If we disallow standalone domains from being in the wildcard host, then we can move                       // 648
          // this check to the bottom.                                                                                 // 649
          if (globalDb.hostIsStandalone(req.headers.host.split(":")[0])) {                                             // 650
            for (var _ii = 0; _ii < meteorUpgradeListeners.length; ++_ii) {                                            // 651
              meteorUpgradeListeners[_ii](req, socket, head);                                                          // 652
            }                                                                                                          // 653
                                                                                                                       //
            return true;                                                                                               // 655
          } else {                                                                                                     // 656
            var id = matchWildcardHost(req.headers.host);                                                              // 657
                                                                                                                       //
            if (id) {                                                                                                  // 658
              return tryProxyUpgrade(id, req, socket, head);                                                           // 659
            } else {                                                                                                   // 660
              return false;                                                                                            // 661
            }                                                                                                          // 662
          }                                                                                                            // 663
        });                                                                                                            // 664
      }                                                                                                                // 665
    }).then(function (handled) {                                                                                       // 666
      if (!handled) socket.destroy();                                                                                  // 667
    }).catch(function (err) {                                                                                          // 668
      console.error("WebSocket event handler failed:", err.stack);                                                     // 669
      socket.destroy();                                                                                                // 670
    });                                                                                                                // 671
  });                                                                                                                  // 672
  listenOnAlternatePorts();                                                                                            // 674
});                                                                                                                    // 675
var errorTxtMapping = {};                                                                                              // 677
errorTxtMapping[Dns.NOTFOUND] = "<p>" + "If you were trying to configure static publishing for a blog or website, powered " + "by a Sandstorm app hosted at this server, you either have not added DNS TXT records " + "correctly, or the DNS cache has not updated yet (may take a while, like 5 minutes to one " + "hour).</p>";
errorTxtMapping[Dns.NODATA] = errorTxtMapping[Dns.NOTFOUND];                                                           // 683
errorTxtMapping[Dns.TIMEOUT] = "<p>" + "The DNS query has timed out, which may be a sign of poorly configured DNS on the server.</p>";
errorTxtMapping[Dns.CONNREFUSED] = "<p>" + "The DNS server refused the connection, which means either your DNS server is down/unreachable, " + "or the server has misconfigured their DNS.</p>";
                                                                                                                       //
function lookupPublicIdFromDns(hostname) {                                                                             // 690
  // Given a hostname, determine its public ID.                                                                        // 691
  // We look for a TXT record indicating the public ID. Unfortunately, according to spec, a single                     // 692
  // hostname cannot have both a CNAME and a TXT record, because a TXT lookup on a CNAME'd host                        // 693
  // should actually be redirected to the CNAME, just like an A lookup would be. In practice lots                      // 694
  // of DNS software actually allows TXT records on CNAMEs, and it seems to work, but some software                    // 695
  // does not allow it and it's explicitly disallowed by the spec. Therefore, we instead look for                      // 696
  // the TXT record on a subdomain.                                                                                    // 697
  //                                                                                                                   // 698
  // I also considered having the CNAME itself point to <publicId>.<hostname>, where                                   // 699
  // *.<hostname> is in turn a CNAME for the Sandstorm server. This approach seemed elegant at                         // 700
  // first, but has a number of problems, the biggest being that it breaks the ability to place a                      // 701
  // CDN like CloudFlare in front of the site.                                                                         // 702
  var cache = dnsCache[hostname];                                                                                      // 704
                                                                                                                       //
  if (cache && Date.now() < cache.expiration) {                                                                        // 705
    return Promise.resolve(cache.value);                                                                               // 706
  }                                                                                                                    // 707
                                                                                                                       //
  return new Promise(function (resolve, reject) {                                                                      // 709
    Dns.resolveTxt("sandstorm-www." + hostname, function (err, records) {                                              // 710
      if (err) {                                                                                                       // 711
        var errorMsg = errorTxtMapping[err.code] || "";                                                                // 712
        var error = new Error('Error looking up DNS TXT records for host "' + hostname + '": ' + err.message);         // 713
        error.htmlMessage = '<style type="text/css">h2, h3, p { max-width: 600px; }</style>' + "<h2>Sandstorm static publishing needs further configuration (or wrong URL)</h2>" + errorMsg + "<p>To visit this Sandstorm server's main interface, go to: <a href='" + process.env.ROOT_URL + "'>" + process.env.ROOT_URL + "</a></p>" + "<h3>DNS details</h3>" + '<p>Error looking up DNS TXT records for host "' + hostname + '": ' + err.message + "</p>" + "<p>If you have the <tt>dig</tt> tool, you can run this command to learn more:</p>" + "<p><tt>dig TXT sandstorm-www." + hostname + "</tt></p>" + "<h3>Changing the server URL, or troubleshooting OAuth login</h3>" + "<p>If you are the server admin and want to use this address as the main interface, " + "edit /opt/sandstorm/sandstorm.conf, modify the BASE_URL setting, and restart " + "Sandstorm.</p>" + "<p>If you got here after trying to log in via OAuth (e.g. through GitHub or Google), " + "the problem is probably that the OAuth callback URL was set wrong. You need to " + "update it through the respective login provider's management console. The " + "easiest way to do that is to run <tt>sudo sandstorm admin-token</tt>, then " + "reconfigure the OAuth provider.</p>";
        error.httpErrorCode = _.contains(["ENOTFOUND", "ENODATA"], err.code) ? 404 : 500;                              // 734
        reject(error);                                                                                                 // 735
      } else if (records.length !== 1) {                                                                               // 736
        reject(new Error('Host "sandstorm-www.' + hostname + '" must have exactly one TXT record.'));                  // 737
      } else {                                                                                                         // 738
        // Under node 0.12+, `records` is an array-of-arrays. Each sub-array contains "chunks" of a                    // 739
        // record, which should be joined.                                                                             // 740
        var result = records[0].join("");                                                                              // 741
        dnsCache[hostname] = {                                                                                         // 742
          value: result,                                                                                               // 742
          expiration: Date.now() + DNS_CACHE_TTL                                                                       // 742
        };                                                                                                             // 742
        resolve(result);                                                                                               // 743
      }                                                                                                                // 744
    });                                                                                                                // 745
  });                                                                                                                  // 746
}                                                                                                                      // 747
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"proxy.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/proxy.js                                                                                                     //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");                                                //
                                                                                                                       //
var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);                                                       //
                                                                                                                       //
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }                      //
                                                                                                                       //
var Bignum = void 0;                                                                                                   // 1
module.watch(require("bignum"), {                                                                                      // 1
  "default": function (v) {                                                                                            // 1
    Bignum = v;                                                                                                        // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var SANDSTORM_ALTHOME = void 0;                                                                                        // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  SANDSTORM_ALTHOME: function (v) {                                                                                    // 1
    SANDSTORM_ALTHOME = v;                                                                                             // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
var SandstormBackend = void 0,                                                                                         // 1
    shouldRestartGrain = void 0;                                                                                       // 1
module.watch(require("/imports/server/backend.js"), {                                                                  // 1
  SandstormBackend: function (v) {                                                                                     // 1
    SandstormBackend = v;                                                                                              // 1
  },                                                                                                                   // 1
  shouldRestartGrain: function (v) {                                                                                   // 1
    shouldRestartGrain = v;                                                                                            // 1
  }                                                                                                                    // 1
}, 2);                                                                                                                 // 1
var inMeteor = void 0,                                                                                                 // 1
    waitPromise = void 0;                                                                                              // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  },                                                                                                                   // 1
  waitPromise: function (v) {                                                                                          // 1
    waitPromise = v;                                                                                                   // 1
  }                                                                                                                    // 1
}, 3);                                                                                                                 // 1
var REQUEST_HEADER_WHITELIST = void 0,                                                                                 // 1
    RESPONSE_HEADER_WHITELIST = void 0;                                                                                // 1
module.watch(require("/imports/server/header-whitelist.js"), {                                                         // 1
  REQUEST_HEADER_WHITELIST: function (v) {                                                                             // 1
    REQUEST_HEADER_WHITELIST = v;                                                                                      // 1
  },                                                                                                                   // 1
  RESPONSE_HEADER_WHITELIST: function (v) {                                                                            // 1
    RESPONSE_HEADER_WHITELIST = v;                                                                                     // 1
  }                                                                                                                    // 1
}, 4);                                                                                                                 // 1
                                                                                                                       //
var Crypto = Npm.require("crypto");                                                                                    // 23
                                                                                                                       //
var ChildProcess = Npm.require("child_process");                                                                       // 24
                                                                                                                       //
var Fs = Npm.require("fs");                                                                                            // 25
                                                                                                                       //
var Path = Npm.require("path");                                                                                        // 26
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 27
                                                                                                                       //
var Net = Npm.require("net");                                                                                          // 28
                                                                                                                       //
var ByteStream = Capnp.importSystem("sandstorm/util.capnp").ByteStream;                                                // 30
var ApiSession = Capnp.importSystem("sandstorm/api-session.capnp").ApiSession;                                         // 31
var WebSession = Capnp.importSystem("sandstorm/web-session.capnp").WebSession;                                         // 32
var HackSession = Capnp.importSystem("sandstorm/hack-session.capnp");                                                  // 33
var Supervisor = Capnp.importSystem("sandstorm/supervisor.capnp").Supervisor;                                          // 34
var Backend = Capnp.importSystem("sandstorm/backend.capnp").Backend;                                                   // 35
var Powerbox = Capnp.importSystem("sandstorm/powerbox.capnp");                                                         // 36
var PARSED_ROOT_URL = Url.parse(process.env.ROOT_URL);                                                                 // 38
var PROTOCOL = PARSED_ROOT_URL.protocol;                                                                               // 39
var PORT = PARSED_ROOT_URL.port;                                                                                       // 40
                                                                                                                       //
var storeReferralProgramInfoApiTokenCreated = function (db, accountId, apiTokenAccountId) {                            // 42
  // From the Referral program's perspective, if Bob's Account has no referredByComplete, then we                      // 43
  // update Bob's Account to say it's referredBy Alice's Account (which is apiTokenAccountId).                         // 44
  check(accountId, String);                                                                                            // 45
  check(apiTokenAccountId, String); // Bail out early if referrals aren't enabled                                      // 46
                                                                                                                       //
  if (!db.isReferralEnabled()) {                                                                                       // 49
    return;                                                                                                            // 50
  }                                                                                                                    // 51
                                                                                                                       //
  var aliceAccountId = apiTokenAccountId;                                                                              // 53
  var bobAccountId = accountId;                                                                                        // 54
                                                                                                                       //
  if (Meteor.users.find({                                                                                              // 56
    _id: bobAccountId,                                                                                                 // 57
    referredByComplete: {                                                                                              // 58
      $exists: true                                                                                                    // 58
    }                                                                                                                  // 58
  }).count() > 0) {                                                                                                    // 56
    return;                                                                                                            // 60
  } // Only actually update Bob's Account ID if there is no referredBy.                                                // 61
                                                                                                                       //
                                                                                                                       //
  Meteor.users.update({                                                                                                // 64
    _id: bobAccountId,                                                                                                 // 65
    referredBy: {                                                                                                      // 65
      $exists: false                                                                                                   // 65
    }                                                                                                                  // 65
  }, {                                                                                                                 // 65
    $set: {                                                                                                            // 66
      referredBy: aliceAccountId                                                                                       // 66
    }                                                                                                                  // 66
  });                                                                                                                  // 66
};                                                                                                                     // 67
                                                                                                                       //
function referralProgramLogSharingTokenUse(db, bobAccountId) {                                                         // 69
  // Hooray! The sharing token is valid! Someone (let's call them Charlie) is going to get a UiView                    // 70
  // to this grain!  This means that the user who created this apiToken knows how to use the 'share                    // 71
  // access' interface. Let's call them Bob.                                                                           // 72
  //                                                                                                                   // 73
  // If Bob himself was referred by Alice, then Alice is now eligible for referral credit, as Bob                      // 74
  // has proven he knows how to share.                                                                                 // 75
  //                                                                                                                   // 76
  // If Bob's Account.referredByComplete is not yet set, then look at Bob's referredBy -- let's                        // 77
  // call that Alice.                                                                                                  // 78
  //                                                                                                                   // 79
  // We copy Alice's account ID to Bob's Account.referredByComplete, and then update Alice's                           // 80
  // referredAccountIds to point at Bob's Account, and then remove the referredBy from Bob's                           // 81
  // Account since it has become redundant.                                                                            // 82
  //                                                                                                                   // 83
  // Implementation note: this does mean that Alice can get referral credit for Bob by sharing a                       // 84
  // link with Bob, even if Bob already had an account.                                                                // 85
  // Bail out early if referrals aren't enabled                                                                        // 87
  if (!db.isReferralEnabled()) {                                                                                       // 88
    return;                                                                                                            // 89
  }                                                                                                                    // 90
                                                                                                                       //
  var bobAccount = Meteor.users.findOne({                                                                              // 92
    _id: bobAccountId                                                                                                  // 92
  }); // Bail out if Bob is already a complete referral.                                                               // 92
                                                                                                                       //
  if (bobAccount.referredByComplete) {                                                                                 // 95
    return;                                                                                                            // 96
  } // Bail out if Bob wasn't referred by anyone.                                                                      // 97
                                                                                                                       //
                                                                                                                       //
  if (!bobAccount.referredBy) {                                                                                        // 100
    return;                                                                                                            // 101
  }                                                                                                                    // 102
                                                                                                                       //
  var aliceAccountId = bobAccount.referredBy; // Store Bob's Account.referralCompletedBy.                              // 104
                                                                                                                       //
  var now = new Date();                                                                                                // 107
  Meteor.users.update({                                                                                                // 108
    _id: bobAccountId,                                                                                                 // 109
    referredBy: {                                                                                                      // 110
      $exists: true                                                                                                    // 110
    },                                                                                                                 // 110
    referredByComplete: {                                                                                              // 111
      $exists: false                                                                                                   // 111
    }                                                                                                                  // 111
  }, {                                                                                                                 // 108
    $rename: {                                                                                                         // 113
      referredBy: "referredByComplete"                                                                                 // 114
    },                                                                                                                 // 113
    $set: {                                                                                                            // 116
      referredCompleteDate: now                                                                                        // 117
    }                                                                                                                  // 116
  }); // Update Alice's Account.referredAccountIds.                                                                    // 112
                                                                                                                       //
  Meteor.users.update({                                                                                                // 122
    _id: aliceAccountId                                                                                                // 122
  }, {                                                                                                                 // 122
    $push: {                                                                                                           // 123
      referredAccountIds: bobAccountId                                                                                 // 123
    }                                                                                                                  // 123
  });                                                                                                                  // 122
} // User-agent strings that should be allowed to use http basic authentication.                                       // 125
// These are regex matches, so ensure they are escaped properly with double                                            // 128
// backslashes. For security reasons, we MUST NOT whitelist any user-agents                                            // 129
// that may render html and execute embedded scripts.                                                                  // 130
                                                                                                                       //
                                                                                                                       //
BASIC_AUTH_USER_AGENTS = ["git\\/", "GitHub-Hookshot\\/", "mirall\\/", "Mozilla\\/5\\.0 \\([^\\\\]*\\) mirall\\/", "Mozilla\\/5\\.0 \\(iOS\\) ownCloud-iOS\\/", "Mozilla\\/5\\.0 \\(Android\\) ownCloud-android\\/", "litmus\\/"];
BASIC_AUTH_USER_AGENTS_REGEX = new RegExp("^(" + BASIC_AUTH_USER_AGENTS.join("|") + ")", "");                          // 140
var SESSION_PROXY_TIMEOUT = 60000;                                                                                     // 142
var sandstormCoreFactory = makeSandstormCoreFactory(globalDb);                                                         // 144
var backendAddress = "unix:" + (SANDSTORM_ALTHOME || "") + Backend.socketPath;                                         // 145
var sandstormBackendConnection = Capnp.connect(backendAddress, sandstormCoreFactory);                                  // 146
var sandstormBackend = sandstormBackendConnection.restore(null, Backend); // TODO(cleanup): This initilization belongs with the rest of our package initialization in
//   db-deprecated.js. We can't put it there now because we need to contruct sandstormCoreFactory first.               // 150
                                                                                                                       //
globalBackend = new SandstormBackend(globalDb, sandstormBackend);                                                      // 151
Meteor.onConnection(function (connection) {                                                                            // 152
  connection.sandstormBackend = globalBackend;                                                                         // 153
}); // We've observed a problem in production where occasionally the front-end stops talking to the                    // 154
// back-end. It happens very rarely -- like once a month -- and we've been unable to reproduce it                      // 157
// in testing, making it very hard to debug. The problem appears both on Sandstorm and Blackrock.                      // 158
// Restarting the node process (and only the node process) always fixes the problem.                                   // 159
//                                                                                                                     // 160
// Here, I've added some code that attempts to detect the problem by doing a health check                              // 161
// periodically and then remaking the connection if it seems broken. We'll see if this helps!                          // 162
                                                                                                                       //
var backendHealthy = true;                                                                                             // 163
var disconnectCount = 0;                                                                                               // 164
Meteor.setInterval(function () {                                                                                       // 165
  if (!backendHealthy) {                                                                                               // 166
    if (disconnectCount++ > 2) process.abort();                                                                        // 167
    console.error("error: Backend hasn't responded in 30 seconds! Reconnecting.");                                     // 168
    if (Capnp.enableVerboseDebugLogging) Capnp.enableVerboseDebugLogging(true);                                        // 169
    sandstormBackendConnection.close();                                                                                // 170
    sandstormBackendConnection = Capnp.connect(backendAddress, sandstormCoreFactory);                                  // 171
    sandstormBackend = sandstormBackendConnection.restore(null, Backend);                                              // 172
    globalBackend._backendCap = sandstormBackend;                                                                      // 173
  }                                                                                                                    // 174
                                                                                                                       //
  var debugLog = !backendHealthy;                                                                                      // 176
  backendHealthy = false;                                                                                              // 177
  var promise = sandstormBackend.ping().then(function () {                                                             // 178
    backendHealthy = true;                                                                                             // 179
    if (Capnp.enableVerboseDebugLogging) Capnp.enableVerboseDebugLogging(false);                                       // 180
  }, function (err) {                                                                                                  // 181
    console.error("error: Backend ping threw error!", err.stack); // The connection will be remade on the next interval. Note that we do NOT normally observe
    // exceptions being thrown for this problem; we see the connection simply stop responding.                         // 184
    // So we don't expect this branch to execute in any case.                                                          // 185
  });                                                                                                                  // 186
                                                                                                                       //
  if (debugLog) {                                                                                                      // 188
    console.log("capnp.js: outer promise:", promise);                                                                  // 189
  }                                                                                                                    // 190
}, 30000); // =======================================================================================                  // 191
// API for creating / starting grains from Meteor methods.                                                             // 194
                                                                                                                       //
var proxiesByHostId = {};                                                                                              // 196
                                                                                                                       //
function parsePowerboxDescriptorList(list) {                                                                           // 198
  return list.map(function (packedDescriptor) {                                                                        // 199
    return Capnp.parse(Powerbox.PowerboxDescriptor, new Buffer(packedDescriptor, "base64"), {                          // 199
      packed: true                                                                                                     // 201
    });                                                                                                                // 201
  });                                                                                                                  // 199
}                                                                                                                      // 202
                                                                                                                       //
Meteor.methods({                                                                                                       // 204
  newGrain: function (packageId, command, title, obsolete) {                                                           // 205
    // Create and start a new grain.                                                                                   // 206
    check(packageId, String);                                                                                          // 208
    check(command, Object); // Manifest.Command from package.capnp.                                                    // 209
                                                                                                                       //
    check(title, String);                                                                                              // 210
                                                                                                                       //
    if (!this.userId) {                                                                                                // 212
      throw new Meteor.Error(403, "Unauthorized", "Must be logged in to create grains.");                              // 213
    }                                                                                                                  // 214
                                                                                                                       //
    if (!isSignedUpOrDemo()) {                                                                                         // 216
      throw new Meteor.Error(403, "Unauthorized", "Only invited users or demo users can create grains.");              // 217
    }                                                                                                                  // 219
                                                                                                                       //
    if (isUserOverQuota(Meteor.user())) {                                                                              // 221
      throw new Meteor.Error(402, "You are out of storage space. Please delete some things and try again.");           // 222
    }                                                                                                                  // 224
                                                                                                                       //
    var pkg = Packages.findOne(packageId);                                                                             // 226
    var isDev = false;                                                                                                 // 227
    var mountProc = false;                                                                                             // 228
                                                                                                                       //
    if (!pkg) {                                                                                                        // 229
      // Maybe they wanted a dev package.  Check there too.                                                            // 230
      pkg = DevPackages.findOne(packageId);                                                                            // 231
      isDev = true;                                                                                                    // 232
      mountProc = pkg.mountProc;                                                                                       // 233
    }                                                                                                                  // 234
                                                                                                                       //
    if (!pkg) {                                                                                                        // 236
      throw new Meteor.Error(404, "Not Found", "No such package is installed.");                                       // 237
    }                                                                                                                  // 238
                                                                                                                       //
    var appId = pkg.appId;                                                                                             // 240
    var manifest = pkg.manifest;                                                                                       // 241
    var grainId = Random.id(22); // 128 bits of entropy                                                                // 242
                                                                                                                       //
    Grains.insert({                                                                                                    // 243
      _id: grainId,                                                                                                    // 244
      packageId: packageId,                                                                                            // 245
      appId: appId,                                                                                                    // 246
      appVersion: manifest.appVersion,                                                                                 // 247
      userId: this.userId,                                                                                             // 248
      identityId: SandstormDb.generateIdentityId(),                                                                    // 249
      title: title,                                                                                                    // 250
      "private": true,                                                                                                 // 251
      size: 0                                                                                                          // 252
    });                                                                                                                // 243
    globalBackend.startGrainInternal(packageId, grainId, this.userId, command, true, isDev, mountProc);                // 255
    globalBackend.updateLastActive(grainId, this.userId);                                                              // 257
    return grainId;                                                                                                    // 258
  },                                                                                                                   // 259
  openSession: function (grainId, revealIdentity, cachedSalt, options) {                                               // 261
    // Open a new UI session on an existing grain.  Starts the grain if it is not already                              // 262
    // running.                                                                                                        // 263
    options = options || {};                                                                                           // 265
    check(grainId, String);                                                                                            // 267
    revealIdentity = !!revealIdentity;                                                                                 // 268
    check(cachedSalt, Match.OneOf(undefined, null, String));                                                           // 269
    check(options, {                                                                                                   // 270
      powerboxRequest: Match.Optional({                                                                                // 271
        descriptors: [String],                                                                                         // 272
        requestingSession: String                                                                                      // 273
      })                                                                                                               // 271
    });                                                                                                                // 270
    var accountId = revealIdentity ? this.userId : null;                                                               // 277
    var grain = Grains.findOne({                                                                                       // 279
      _id: grainId                                                                                                     // 279
    });                                                                                                                // 279
                                                                                                                       //
    if (!grain) {                                                                                                      // 280
      throw new Meteor.Error(404, "Grain not found", "Grain ID: " + grainId);                                          // 281
    } else if (grain.trashed) {                                                                                        // 282
      throw new Meteor.Error("grain-is-in-trash", "Grain is in trash", "Grain ID: " + grainId);                        // 283
    } else if (grain.suspended) {                                                                                      // 284
      throw new Meteor.Error("grain-owner-suspended", "Grain's owner is suspended", "Grain ID: " + grainId);           // 285
    }                                                                                                                  // 287
                                                                                                                       //
    var db = this.connection.sandstormDb;                                                                              // 289
    check(cachedSalt, Match.OneOf(undefined, null, String));                                                           // 290
                                                                                                                       //
    if (!SandstormPermissions.mayOpenGrain(db, {                                                                       // 291
      grain: {                                                                                                         // 292
        _id: grainId,                                                                                                  // 292
        accountId: accountId                                                                                           // 292
      }                                                                                                                // 292
    })) {                                                                                                              // 292
      throw new Meteor.Error(403, "Unauthorized", "User is not authorized to open this grain.");                       // 293
    }                                                                                                                  // 294
                                                                                                                       //
    var sessionFields = {};                                                                                            // 296
                                                                                                                       //
    if (options.powerboxRequest) {                                                                                     // 297
      sessionFields.powerboxRequest = {                                                                                // 298
        descriptors: parsePowerboxDescriptorList(options.powerboxRequest.descriptors),                                 // 299
        requestingSession: options.powerboxRequest.requestingSession                                                   // 300
      };                                                                                                               // 298
    }                                                                                                                  // 302
                                                                                                                       //
    var opened = globalBackend.openSessionInternal(grain, this.userId, revealIdentity, null, null, cachedSalt, sessionFields);
    var result = opened.methodResult;                                                                                  // 306
    var proxy = new Proxy(grain, result.sessionId, result.hostId, result.tabId, accountId, result.identityId, false, null, opened.supervisor);
                                                                                                                       //
    if (sessionFields.powerboxRequest) {                                                                               // 310
      proxy.powerboxRequest = sessionFields.powerboxRequest;                                                           // 311
    }                                                                                                                  // 312
                                                                                                                       //
    proxiesByHostId[result.hostId] = proxy;                                                                            // 314
    return result;                                                                                                     // 315
  },                                                                                                                   // 316
  openSessionFromApiToken: function (params, revealIdentity, cachedSalt, neverRedeem, parentOrigin, options) {         // 318
    // Given an API token, either opens a new WebSession to the underlying grain or returns a                          // 319
    // path to which the client should redirect in order to open such a session.                                       // 320
    //                                                                                                                 // 321
    // `parentOrigin` is the origin of the parent frame of the grain iframe. This may differ from                      // 322
    // ROOT_URL for standalone domains.                                                                                // 323
    neverRedeem = neverRedeem || false;                                                                                // 325
    parentOrigin = parentOrigin || process.env.ROOT_URL;                                                               // 326
    options = options || {};                                                                                           // 327
    check(params, {                                                                                                    // 329
      token: String,                                                                                                   // 330
      incognito: Match.Optional(Boolean) // obsolete, ignored                                                          // 331
                                                                                                                       //
    });                                                                                                                // 329
    revealIdentity = !!revealIdentity;                                                                                 // 333
    check(cachedSalt, Match.OneOf(undefined, null, String));                                                           // 334
    check(neverRedeem, Boolean);                                                                                       // 335
    check(parentOrigin, String);                                                                                       // 336
    check(options, {                                                                                                   // 337
      powerboxRequest: Match.Optional({                                                                                // 338
        descriptors: [String],                                                                                         // 339
        requestingSession: String                                                                                      // 340
      })                                                                                                               // 338
    });                                                                                                                // 337
    var accountId = revealIdentity ? this.userId : null;                                                               // 344
                                                                                                                       //
    if (!accountId && globalDb.getOrganizationDisallowGuests()) {                                                      // 346
      throw new Meteor.Error("guestDisallowed", "This Sandstorm server does not allow " + "guests or anonymous users");
    }                                                                                                                  // 349
                                                                                                                       //
    var token = params.token;                                                                                          // 351
    var incognito = !accountId;                                                                                        // 352
    var hashedToken = Crypto.createHash("sha256").update(token).digest("base64");                                      // 353
    var apiToken = ApiTokens.findOne(hashedToken);                                                                     // 354
    validateWebkey(apiToken);                                                                                          // 355
    var grain = Grains.findOne({                                                                                       // 356
      _id: apiToken.grainId                                                                                            // 356
    });                                                                                                                // 356
                                                                                                                       //
    if (!grain) {                                                                                                      // 357
      throw new Meteor.Error(404, "Grain not found", "Grain ID: " + apiToken.grainId);                                 // 358
    }                                                                                                                  // 359
                                                                                                                       //
    if (apiToken.accountId) {                                                                                          // 361
      referralProgramLogSharingTokenUse(globalDb, apiToken.accountId);                                                 // 362
    }                                                                                                                  // 363
                                                                                                                       //
    var title = void 0;                                                                                                // 365
                                                                                                                       //
    if (grain.userId === apiToken.accountId) {                                                                         // 366
      title = grain.title;                                                                                             // 367
    } else {                                                                                                           // 368
      var sharerToken = apiToken.accountId && ApiTokens.findOne({                                                      // 369
        grainId: apiToken.grainId,                                                                                     // 371
        "owner.user.accountId": apiToken.accountId                                                                     // 372
      }, {                                                                                                             // 370
        sort: {                                                                                                        // 374
          lastUsed: -1                                                                                                 // 375
        }                                                                                                              // 374
      });                                                                                                              // 373
                                                                                                                       //
      if (sharerToken) {                                                                                               // 378
        title = sharerToken.owner.user.title;                                                                          // 379
      } else {                                                                                                         // 380
        title = "shared grain";                                                                                        // 381
      }                                                                                                                // 382
    }                                                                                                                  // 383
                                                                                                                       //
    if (this.userId && !incognito && !neverRedeem) {                                                                   // 385
      if (accountId != apiToken.accountId && accountId != grain.userId && !ApiTokens.findOne({                         // 386
        "owner.user.accountId": accountId,                                                                             // 387
        parentToken: hashedToken                                                                                       // 387
      })) {                                                                                                            // 387
        var owner = {                                                                                                  // 388
          user: {                                                                                                      // 388
            accountId: accountId,                                                                                      // 388
            title: title                                                                                               // 388
          }                                                                                                            // 388
        }; // Create a new API token for the account redeeming this token.                                             // 388
                                                                                                                       //
        var result = SandstormPermissions.createNewApiToken(globalDb, {                                                // 391
          rawParentToken: token                                                                                        // 392
        }, apiToken.grainId, apiToken.petname || "redeemed webkey", {                                                  // 392
          allAccess: null                                                                                              // 394
        }, owner);                                                                                                     // 394
        globalDb.addContact(apiToken.accountId, accountId); // If the parent API token is forSharing and it has an accountId, then the logged-in user (call
        // them Bob) is about to access a grain owned by someone (call them Alice) and save a reference                // 398
        // to it as a new ApiToken. (For share-by-link, this occurs when viewing the grain. For                        // 399
        // share-by-identity, this happens immediately.)                                                               // 400
                                                                                                                       //
        if (result.parentApiToken) {                                                                                   // 401
          var parentApiToken = result.parentApiToken;                                                                  // 402
                                                                                                                       //
          if (parentApiToken.forSharing && parentApiToken.accountId) {                                                 // 403
            storeReferralProgramInfoApiTokenCreated(globalDb, this.userId, parentApiToken.accountId);                  // 404
          }                                                                                                            // 406
        }                                                                                                              // 407
      }                                                                                                                // 408
                                                                                                                       //
      return {                                                                                                         // 410
        redirectToGrain: apiToken.grainId                                                                              // 410
      };                                                                                                               // 410
    } else {                                                                                                           // 411
      if (!SandstormPermissions.mayOpenGrain(globalDb, {                                                               // 412
        token: apiToken                                                                                                // 412
      })) {                                                                                                            // 412
        throw new Meteor.Error(403, "Unauthorized", "User is not authorized to open this grain.");                     // 413
      }                                                                                                                // 415
                                                                                                                       //
      var sessionFields = {};                                                                                          // 417
                                                                                                                       //
      if (options.powerboxRequest) {                                                                                   // 418
        sessionFields.powerboxRequest = {                                                                              // 419
          descriptors: parsePowerboxDescriptorList(options.powerboxRequest.descriptors),                               // 420
          requestingSession: options.powerboxRequest.requestingSession                                                 // 421
        };                                                                                                             // 419
      } // Even in incognito mode, we want to record the user ID on the session. The user ID is not                    // 423
      // leaked to the app in any way -- it is used to decide what options to show in a powerbox                       // 426
      // request. Even if the user is incognito, they should be able to offer their full range of                      // 427
      // capabilities.                                                                                                 // 428
      //                                                                                                               // 429
      // The identity ID passed here IS revealed to the app, but for incognito mode it is always                       // 430
      // null/undefined.                                                                                               // 431
                                                                                                                       //
                                                                                                                       //
      var opened = globalBackend.openSessionInternal(grain, this.userId, revealIdentity, title, apiToken, cachedSalt, sessionFields);
      var _result = opened.methodResult;                                                                               // 435
      var proxy = new Proxy(grain, _result.sessionId, _result.hostId, _result.tabId, accountId, _result.identityId, false, parentOrigin, opened.supervisor);
      proxy.apiToken = apiToken;                                                                                       // 438
                                                                                                                       //
      if (sessionFields.powerboxRequest) {                                                                             // 439
        proxy.powerboxRequest = sessionFields.powerboxRequest;                                                         // 440
      }                                                                                                                // 441
                                                                                                                       //
      proxiesByHostId[_result.hostId] = proxy;                                                                         // 443
      return _result;                                                                                                  // 444
    }                                                                                                                  // 445
  },                                                                                                                   // 446
  keepSessionAlive: function (sessionId) {                                                                             // 448
    // TODO(security):  Prevent draining someone else's quota by holding open several grains shared                    // 449
    //   by them.                                                                                                      // 450
    check(sessionId, String);                                                                                          // 451
    var result = Sessions.findAndModify({                                                                              // 453
      query: {                                                                                                         // 454
        _id: sessionId                                                                                                 // 454
      },                                                                                                               // 454
      update: {                                                                                                        // 455
        $set: {                                                                                                        // 455
          timestamp: new Date().getTime()                                                                              // 455
        }                                                                                                              // 455
      },                                                                                                               // 455
      fields: {                                                                                                        // 456
        grainId: 1,                                                                                                    // 456
        identityId: 1,                                                                                                 // 456
        hostId: 1                                                                                                      // 456
      }                                                                                                                // 456
    });                                                                                                                // 453
                                                                                                                       //
    if (!result.ok) {                                                                                                  // 459
      return false;                                                                                                    // 460
    }                                                                                                                  // 461
                                                                                                                       //
    var session = result.value;                                                                                        // 463
                                                                                                                       //
    if (session) {                                                                                                     // 464
      // Session still present in database, so send keep-alive to backend.                                             // 465
      try {                                                                                                            // 466
        var grainId = session.grainId;                                                                                 // 467
        var hostId = session.hostId;                                                                                   // 468
        var supervisor = proxiesByHostId[hostId] && proxiesByHostId[hostId].supervisor;                                // 469
                                                                                                                       //
        if (!supervisor) {                                                                                             // 470
          supervisor = globalBackend.continueGrain(grainId).supervisor;                                                // 471
        }                                                                                                              // 472
                                                                                                                       //
        waitPromise(supervisor.keepAlive());                                                                           // 474
        globalBackend.updateLastActive(grainId, this.userId);                                                          // 475
      } catch (err) {                                                                                                  // 476
        // Ignore disconnects, which imply that the grain shut down already. It'll start back up on                    // 477
        // the next request, so whatever.                                                                              // 478
        if (err.kjType !== "disconnected") {                                                                           // 479
          throw err;                                                                                                   // 480
        }                                                                                                              // 481
      }                                                                                                                // 482
                                                                                                                       //
      return true;                                                                                                     // 484
    } else {                                                                                                           // 485
      return false;                                                                                                    // 486
    }                                                                                                                  // 487
  },                                                                                                                   // 488
  shutdownGrain: function (grainId) {                                                                                  // 490
    check(grainId, String);                                                                                            // 491
    var grain = Grains.findOne(grainId);                                                                               // 492
                                                                                                                       //
    if (!grain || !this.userId || grain.userId !== this.userId) {                                                      // 493
      throw new Meteor.Error(403, "Unauthorized", "User is not the owner of this grain");                              // 494
    }                                                                                                                  // 495
                                                                                                                       //
    waitPromise(globalBackend.shutdownGrain(grainId, grain.userId, true));                                             // 497
  }                                                                                                                    // 498
});                                                                                                                    // 204
                                                                                                                       //
var validateWebkey = function (apiToken, refreshedExpiration) {                                                        // 501
  // Validates that `apiToken` is a valid UiView webkey, throwing an exception if it is not. If                        // 502
  // `refreshedExpiration` is set and if the token has an `expiresIfUnused` field, then the                            // 503
  // `expiresIfUnused` field is reset to `refreshedExpiration`.                                                        // 504
  if (!apiToken) {                                                                                                     // 506
    throw new Meteor.Error(403, "Invalid authorization token");                                                        // 507
  }                                                                                                                    // 508
                                                                                                                       //
  if (apiToken.revoked) {                                                                                              // 510
    throw new Meteor.Error(403, "Authorization token has been revoked");                                               // 511
  }                                                                                                                    // 512
                                                                                                                       //
  if (apiToken.owner && !("webkey" in apiToken.owner)) {                                                               // 514
    throw new Meteor.Error(403, "Unauthorized to open non-webkey token.");                                             // 515
  }                                                                                                                    // 516
                                                                                                                       //
  if (apiToken.expires && apiToken.expires.getTime() <= Date.now()) {                                                  // 518
    throw new Meteor.Error(403, "Authorization token expired");                                                        // 519
  }                                                                                                                    // 520
                                                                                                                       //
  if (apiToken.expiresIfUnused) {                                                                                      // 522
    if (apiToken.expiresIfUnused.getTime() <= Date.now()) {                                                            // 523
      throw new Meteor.Error(403, "Authorization token expired");                                                      // 524
    } else if (refreshedExpiration) {                                                                                  // 525
      ApiTokens.update(apiToken._id, {                                                                                 // 526
        $set: {                                                                                                        // 526
          expiresIfUnused: refreshedExpiration                                                                         // 526
        }                                                                                                              // 526
      });                                                                                                              // 526
    } else {                                                                                                           // 527
      // It's getting used now, so clear the expiresIfUnused field.                                                    // 528
      ApiTokens.update(apiToken._id, {                                                                                 // 529
        $set: {                                                                                                        // 529
          expiresIfUnused: null                                                                                        // 529
        }                                                                                                              // 529
      });                                                                                                              // 529
    }                                                                                                                  // 530
  }                                                                                                                    // 531
                                                                                                                       //
  if (apiToken.objectId || apiToken.frontendRef) {                                                                     // 533
    throw new Meteor.Error(403, "ApiToken refers to a non-webview Capability.");                                       // 534
  }                                                                                                                    // 535
};                                                                                                                     // 536
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 538
  var shutdownApp = function (appId) {                                                                                 // 539
    Grains.find({                                                                                                      // 540
      appId: appId                                                                                                     // 540
    }).forEach(function (grain) {                                                                                      // 540
      waitPromise(globalBackend.shutdownGrain(grain._id, grain.userId));                                               // 541
    });                                                                                                                // 542
  };                                                                                                                   // 543
                                                                                                                       //
  DevPackages.find().observe({                                                                                         // 545
    removed: function (devPackage) {                                                                                   // 546
      shutdownApp(devPackage.appId);                                                                                   // 546
    },                                                                                                                 // 546
    changed: function (newDevPackage, oldDevPackage) {                                                                 // 548
      shutdownApp(oldDevPackage.appId);                                                                                // 549
                                                                                                                       //
      if (oldDevPackage.appId !== newDevPackage.appId) {                                                               // 550
        shutdownApp(newDevPackage.appId);                                                                              // 551
      }                                                                                                                // 552
    },                                                                                                                 // 553
    added: function (devPackage) {                                                                                     // 555
      shutdownApp(devPackage.appId);                                                                                   // 555
    }                                                                                                                  // 555
  });                                                                                                                  // 545
  Sessions.find().observe({                                                                                            // 558
    removed: function (session) {                                                                                      // 559
      var proxy = proxiesByHostId[session.hostId];                                                                     // 560
      delete proxiesByHostId[session.hostId];                                                                          // 561
                                                                                                                       //
      if (proxy) {                                                                                                     // 562
        proxy.close();                                                                                                 // 563
      }                                                                                                                // 564
    }                                                                                                                  // 565
  });                                                                                                                  // 558
}); // Kill off sessions idle for >~3 minutes.                                                                         // 567
                                                                                                                       //
var TIMEOUT_MS = 180000;                                                                                               // 570
                                                                                                                       //
var gcSessions = function () {                                                                                         // 571
  var now = new Date().getTime();                                                                                      // 572
  Sessions.remove({                                                                                                    // 573
    timestamp: {                                                                                                       // 573
      $lt: now - TIMEOUT_MS                                                                                            // 573
    }                                                                                                                  // 573
  });                                                                                                                  // 573
};                                                                                                                     // 574
                                                                                                                       //
SandstormDb.periodicCleanup(TIMEOUT_MS, gcSessions);                                                                   // 576
                                                                                                                       //
var getProxyForHostId = function (hostId, isAlreadyOpened) {                                                           // 578
  // Get the Proxy corresponding to the given grain session host, possibly (re)creating it if it                       // 579
  // doesn't already exist. The first request on the session host will always create a new proxy.                      // 580
  // Later requests may create a proxy if they go to a different front-end replica or if the                           // 581
  // front-end was restarted.                                                                                          // 582
  check(hostId, String);                                                                                               // 583
  return Promise.resolve(undefined).then(function () {                                                                 // 585
    var proxy = proxiesByHostId[hostId];                                                                               // 586
                                                                                                                       //
    if (proxy) {                                                                                                       // 587
      return proxy;                                                                                                    // 588
    } else {                                                                                                           // 589
      // Set table entry to null for now so that we can detect if it is concurrently deleted.                          // 590
      proxiesByHostId[hostId] = null;                                                                                  // 591
      return inMeteor(function () {                                                                                    // 593
        var session = Sessions.findOne({                                                                               // 594
          hostId: hostId                                                                                               // 594
        });                                                                                                            // 594
                                                                                                                       //
        if (!session) {                                                                                                // 595
          if (isAlreadyOpened) {                                                                                       // 596
            var observer = void 0;                                                                                     // 597
            return new Promise(function (resolve, reject) {                                                            // 598
              var task = Meteor.setTimeout(function () {                                                               // 599
                reject(new Meteor.Error(504, "Requested session that no longer exists, and " + "timed out waiting for client to restore it. This can happen if you have " + "opened an app's content in a new window and then closed it in the " + "UI. If you see this error *inside* the Sandstorm UI, please report a " + "bug and describe the circumstances of the error."));
              }, SESSION_PROXY_TIMEOUT);                                                                               // 605
              observer = Sessions.find({                                                                               // 606
                hostId: hostId                                                                                         // 606
              }).observe({                                                                                             // 606
                added: function () {                                                                                   // 607
                  Meteor.clearTimeout(task);                                                                           // 608
                  resolve(getProxyForHostId(hostId, false));                                                           // 609
                }                                                                                                      // 610
              });                                                                                                      // 606
            }).then(function (v) {                                                                                     // 612
              observer.stop();                                                                                         // 613
              return v;                                                                                                // 614
            }, function (e) {                                                                                          // 615
              observer.stop();                                                                                         // 616
              throw e;                                                                                                 // 617
            });                                                                                                        // 618
          } else {                                                                                                     // 619
            // Does not appear to be a valid session host.                                                             // 620
            return undefined;                                                                                          // 621
          }                                                                                                            // 622
        }                                                                                                              // 623
                                                                                                                       //
        var apiToken = void 0;                                                                                         // 625
                                                                                                                       //
        if (session.hashedToken) {                                                                                     // 626
          apiToken = ApiTokens.findOne({                                                                               // 627
            _id: session.hashedToken                                                                                   // 627
          }); // We don't have to fully validate the API token here because if it changed the session                  // 627
          // would have been deleted.                                                                                  // 629
                                                                                                                       //
          if (!apiToken) {                                                                                             // 630
            throw new Meteor.Error(410, "ApiToken has been deleted");                                                  // 631
          }                                                                                                            // 632
        }                                                                                                              // 633
                                                                                                                       //
        var grain = Grains.findOne(session.grainId);                                                                   // 635
                                                                                                                       //
        if (!grain) {                                                                                                  // 636
          // Grain was deleted, I guess.                                                                               // 637
          throw new Meteor.Error(410, "Resource has been deleted");                                                    // 638
        } // Note that we don't need to call mayOpenGrain() because the existence of a session                         // 639
        // implies this check was already performed.                                                                   // 642
                                                                                                                       //
                                                                                                                       //
        var proxy = new Proxy(grain, session._id, hostId, session.tabId, session.userId, session.identityId, false);   // 644
        if (apiToken) proxy.apiToken = apiToken;                                                                       // 646
                                                                                                                       //
        if (session.powerboxRequest) {                                                                                 // 648
          proxy.powerboxRequest = session.powerboxRequest;                                                             // 649
        } // Only add the proxy to the table if it was not concurrently deleted (which could happen                    // 650
        // e.g. if the user's access was revoked).                                                                     // 653
                                                                                                                       //
                                                                                                                       //
        if (hostId in proxiesByHostId) {                                                                               // 654
          proxiesByHostId[hostId] = proxy;                                                                             // 655
        } else {                                                                                                       // 656
          throw new Meteor.Error(403, "Session was concurrently closed.");                                             // 657
        }                                                                                                              // 658
                                                                                                                       //
        return proxy;                                                                                                  // 660
      });                                                                                                              // 661
    }                                                                                                                  // 662
  });                                                                                                                  // 663
}; // =======================================================================================                          // 664
// API tokens                                                                                                          // 667
                                                                                                                       //
                                                                                                                       //
var ApiSessionProxies = function () {                                                                                  //
  // Class that caches proxies for requests that come in through HTTP API endpoints. Such                              // 670
  // requests are not associated with any entry in the `Sessions` collection, so we need                               // 671
  // some other means of detecting unused proxies and closing them. This class works by keeping                        // 672
  // the proxies in two buckets. When a proxy is used, it is put in the new bucket. Periodically,                      // 673
  // the old bucket is emptied and the contents of new bucket are moved to the old bucket.                             // 674
  //                                                                                                                   // 675
  // Each bucket is a two-level map, keyed by hashes of the tokens and their `ApiSession.Params`                       // 676
  // structs.                                                                                                          // 677
  function ApiSessionProxies(intervalMillis) {                                                                         // 679
    var _this2 = this;                                                                                                 // 679
                                                                                                                       //
    (0, _classCallCheck3.default)(this, ApiSessionProxies);                                                            // 679
    check(intervalMillis, Number);                                                                                     // 680
    this.newBucket = {};                                                                                               // 681
    this.oldBucket = {};                                                                                               // 682
    this.interval = Meteor.setInterval(function () {                                                                   // 683
      for (var oldHashedToken in meteorBabelHelpers.sanitizeForInObject(_this2.oldBucket)) {                           // 684
        for (var oldHashedParams in meteorBabelHelpers.sanitizeForInObject(_this2.oldBucket[oldHashedToken])) {        // 685
          var newProxy = _this2.newBucket[oldHashedToken] && _this2.newBucket[oldHashedToken][oldHashedParams];        // 686
          var oldProxy = _this2.oldBucket[oldHashedToken][oldHashedParams];                                            // 688
                                                                                                                       //
          if (oldProxy && !newProxy) {                                                                                 // 689
            if (Object.keys(oldProxy.websockets).length > 0 || Object.keys(oldProxy.streamingRequests).length > 0 || Object.keys(oldProxy.streamingResponses).length > 0) {
              // A client has an open stream. Keep the proxy around.                                                   // 693
              _this2.put(oldHashedToken, oldHashedParams, oldProxy);                                                   // 694
            } else {                                                                                                   // 695
              // We can close this proxy and forget about it.                                                          // 696
              oldProxy.close();                                                                                        // 697
            }                                                                                                          // 698
          }                                                                                                            // 699
        }                                                                                                              // 700
      }                                                                                                                // 701
                                                                                                                       //
      _this2.oldBucket = _this2.newBucket;                                                                             // 703
      _this2.newBucket = {};                                                                                           // 704
      var keepAlives = [];                                                                                             // 706
                                                                                                                       //
      for (var hashedToken in meteorBabelHelpers.sanitizeForInObject(_this2.oldBucket)) {                              // 707
        for (var hashedParams in meteorBabelHelpers.sanitizeForInObject(_this2.oldBucket[hashedToken])) {              // 708
          var proxy = _this2.oldBucket[hashedToken][hashedParams];                                                     // 709
          keepAlives.push(proxy.supervisor.keepAlive().catch(function (e) {                                            // 710
            console.error("supervisor.keepAlive() failed.", e);                                                        // 711
          }));                                                                                                         // 712
        }                                                                                                              // 713
      }                                                                                                                // 714
                                                                                                                       //
      waitPromise(Promise.all(keepAlives));                                                                            // 716
    }, intervalMillis);                                                                                                // 717
  }                                                                                                                    // 718
                                                                                                                       //
  ApiSessionProxies.prototype.get = function () {                                                                      //
    function get(hashedToken, hashedParams) {                                                                          //
      if (this.newBucket[hashedToken] && this.newBucket[hashedToken][hashedParams]) {                                  // 721
        return this.newBucket[hashedToken][hashedParams];                                                              // 722
      } else if (this.oldBucket[hashedToken] && this.oldBucket[hashedToken][hashedParams]) {                           // 723
        var proxy = this.oldBucket[hashedToken][hashedParams];                                                         // 724
        delete this.oldBucket[hashedToken][hashedParams];                                                              // 725
        this.put(hashedToken, hashedParams, proxy);                                                                    // 726
        return proxy;                                                                                                  // 727
      } else {                                                                                                         // 728
        return null;                                                                                                   // 729
      }                                                                                                                // 730
    }                                                                                                                  // 731
                                                                                                                       //
    return get;                                                                                                        //
  }();                                                                                                                 //
                                                                                                                       //
  ApiSessionProxies.prototype.put = function () {                                                                      //
    function put(hashedToken, hashedParams, proxy) {                                                                   //
      if (!this.newBucket[hashedToken]) {                                                                              // 734
        this.newBucket[hashedToken] = {};                                                                              // 735
      }                                                                                                                // 736
                                                                                                                       //
      this.newBucket[hashedToken][hashedParams] = proxy;                                                               // 738
    }                                                                                                                  // 739
                                                                                                                       //
    return put;                                                                                                        //
  }();                                                                                                                 //
                                                                                                                       //
  ApiSessionProxies.prototype.removeProxiesOfToken = function () {                                                     //
    function removeProxiesOfToken(hashedToken) {                                                                       //
      if (hashedToken in this.oldBucket) {                                                                             // 742
        for (var hashedParams in meteorBabelHelpers.sanitizeForInObject(this.oldBucket[hashedToken])) {                // 743
          this.oldBucket[hashedToken][hashedParams].close();                                                           // 744
        }                                                                                                              // 745
                                                                                                                       //
        delete this.oldBucket[hashedToken];                                                                            // 747
      }                                                                                                                // 748
                                                                                                                       //
      if (hashedToken in this.newBucket) {                                                                             // 750
        for (var _hashedParams in meteorBabelHelpers.sanitizeForInObject(this.newBucket[hashedToken])) {               // 751
          this.newBucket[hashedToken][_hashedParams].close();                                                          // 752
        }                                                                                                              // 753
                                                                                                                       //
        delete this.newBucket[hashedToken];                                                                            // 755
      }                                                                                                                // 756
    }                                                                                                                  // 757
                                                                                                                       //
    return removeProxiesOfToken;                                                                                       //
  }();                                                                                                                 //
                                                                                                                       //
  return ApiSessionProxies;                                                                                            //
}(); // The interval of 60 seconds here deliberately matches the `keepSessionAlive()` interval                         //
// in the case of non-API proxies.                                                                                     // 761
                                                                                                                       //
                                                                                                                       //
var apiSessionProxies = new ApiSessionProxies(60 * 1000);                                                              // 762
Meteor.startup(function () {                                                                                           // 764
  Grains.find().observe({                                                                                              // 765
    changed: function (newGrain, oldGrain) {                                                                           // 766
      if (oldGrain.private != newGrain.private) {                                                                      // 767
        Sessions.remove({                                                                                              // 768
          grainId: oldGrain._id,                                                                                       // 768
          userId: {                                                                                                    // 768
            $ne: oldGrain.userId                                                                                       // 768
          }                                                                                                            // 768
        });                                                                                                            // 768
        ApiTokens.find({                                                                                               // 769
          grainId: oldGrain._id                                                                                        // 769
        }).forEach(function (apiToken) {                                                                               // 769
          apiSessionProxies.removeProxiesOfToken(apiToken._id);                                                        // 770
        });                                                                                                            // 771
      }                                                                                                                // 772
    }                                                                                                                  // 773
  });                                                                                                                  // 765
});                                                                                                                    // 775
                                                                                                                       //
function getApiSessionParams(request) {                                                                                // 777
  var params = {};                                                                                                     // 778
                                                                                                                       //
  if ("x-sandstorm-passthrough" in request.headers) {                                                                  // 779
    var optIns = request.headers["x-sandstorm-passthrough"].split(",").map(function (s) {                              // 780
      return s.trim();                                                                                                 // 782
    }); // The only currently supported passthrough value is 'address', but others could be useful in                  // 782
    // the future.                                                                                                     // 784
                                                                                                                       //
    if (optIns.indexOf("address") !== -1) {                                                                            // 786
      // Sadly, we can't use request.socket.remoteFamily because it's not available in the                             // 787
      // (rather-old) version of node that comes in the Meteor bundle we're using. Hence this                          // 788
      // hackery.                                                                                                      // 789
      var addressToPass = request.socket.remoteAddress;                                                                // 790
                                                                                                                       //
      if (isRfc1918OrLocal(addressToPass) && "x-real-ip" in request.headers) {                                         // 791
        // Allow overriding the socket's remote address with X-Real-IP header if the request comes                     // 792
        // from either localhost or an RFC1918 address. These are not useful for geolocation anyway.                   // 793
        addressToPass = request.headers["x-real-ip"];                                                                  // 794
      }                                                                                                                // 795
                                                                                                                       //
      if (Net.isIPv4(addressToPass)) {                                                                                 // 797
        // Map IPv4 addresses in IPv6.                                                                                 // 798
        // This conveniently comes out to a 48-bit number, which is precisely representable in a                       // 799
        // double (which has 53 mantissa bits). Thus we can avoid using Bignum/strings, which we                       // 800
        // might otherwise need to precisely represent 64-bit fields.                                                  // 801
        var v4Int = 0xFFFF00000000 + addressToPass.split(".").map(function (x) {                                       // 802
          return parseInt(x, 10);                                                                                      // 803
        }).reduce(function (a, b) {                                                                                    // 803
          return 256 * a + b;                                                                                          // 804
        });                                                                                                            // 804
        params.remoteAddress = {                                                                                       // 805
          lower64: v4Int,                                                                                              // 806
          upper64: 0                                                                                                   // 807
        };                                                                                                             // 805
      } else if (Net.isIPv6(addressToPass)) {                                                                          // 809
        // TODO(test): Unit test this                                                                                  // 810
        // Parse a valid v6 address.                                                                                   // 811
        // Split into groups, then insert an appropriate number of 0's if :: was used.                                 // 812
        var groups = addressToPass.split(":"); // Strip extra empty group in the case of a leading or trailing '::'.   // 813
                                                                                                                       //
        if (groups[0] === "") {                                                                                        // 816
          groups.shift();                                                                                              // 817
        }                                                                                                              // 818
                                                                                                                       //
        if (groups[groups.length - 1] === "") {                                                                        // 820
          groups.pop();                                                                                                // 821
        }                                                                                                              // 822
                                                                                                                       //
        var lastGroup = groups[groups.length - 1]; // Handle IPv4-mapped IPv6 addresses.  These end in a dotted-quad IPv4 address, which we
        // should expand into two groups of 4-character hex strings, like the rest of the address.                     // 826
                                                                                                                       //
        if (Net.isIPv4(lastGroup)) {                                                                                   // 827
          groups.pop();                                                                                                // 828
          var quad = lastGroup.split(".").map(function (x) {                                                           // 829
            return parseInt(x, 10);                                                                                    // 829
          });                                                                                                          // 829
          groups.push((quad[0] * 256 + quad[1]).toString(16));                                                         // 830
          groups.push((quad[2] * 256 + quad[3]).toString(16));                                                         // 831
        }                                                                                                              // 832
                                                                                                                       //
        var groupsToAdd = 8 - groups.length;                                                                           // 834
        var emptyGroupIndex = groups.indexOf("");                                                                      // 835
        var cleanGroups = void 0;                                                                                      // 836
                                                                                                                       //
        if (emptyGroupIndex !== -1) {                                                                                  // 837
          var head = groups.slice(0, emptyGroupIndex); // groupsToAdd + 1 because we sliced out the empty element      // 838
                                                                                                                       //
          var mid = Array(groupsToAdd + 1);                                                                            // 840
                                                                                                                       //
          for (var i = 0; i < groupsToAdd + 1; i++) {                                                                  // 841
            mid[i] = "0";                                                                                              // 842
          }                                                                                                            // 843
                                                                                                                       //
          var tail = groups.slice(emptyGroupIndex + 1, groups.length);                                                 // 845
          cleanGroups = [].concat(head, mid, tail);                                                                    // 846
        } else {                                                                                                       // 847
          cleanGroups = groups;                                                                                        // 848
        }                                                                                                              // 849
                                                                                                                       //
        var ints = cleanGroups.map(function (x) {                                                                      // 851
          return parseInt(x, 16);                                                                                      // 851
        }); // We use strings because we'd lose data from loss of precision casting the 64-bit uints                   // 851
        // into 53-bit-mantissa doubles.                                                                               // 853
                                                                                                                       //
        params.remoteAddress = {                                                                                       // 854
          upper64: quadToIntString(ints.slice(0, 4)),                                                                  // 855
          lower64: quadToIntString(ints.slice(4, 8))                                                                   // 856
        };                                                                                                             // 854
      }                                                                                                                // 858
    }                                                                                                                  // 859
  }                                                                                                                    // 860
                                                                                                                       //
  return Capnp.serialize(ApiSession.Params, params);                                                                   // 862
} // Used by server/drivers/external-ui/view.js                                                                        // 863
                                                                                                                       //
                                                                                                                       //
getProxyForApiToken = function (token, request) {                                                                      // 866
  check(token, String);                                                                                                // 867
  var hashedToken = Crypto.createHash("sha256").update(token).digest("base64");                                        // 868
  var tabId = Crypto.createHash("sha256").update("tab:").update(hashedToken).digest("hex").slice(0, 32);               // 869
  var serializedParams = getApiSessionParams(request);                                                                 // 871
  var hashedParams = Crypto.createHash("sha256").update(serializedParams).digest("base64");                            // 872
  return Promise.resolve(undefined).then(function () {                                                                 // 873
    var proxy = apiSessionProxies.get(hashedToken, hashedParams);                                                      // 874
                                                                                                                       //
    if (proxy) {                                                                                                       // 875
      if (proxy.expires && proxy.expires.getTime() <= Date.now()) {                                                    // 876
        throw new Meteor.Error(403, "Authorization token expired");                                                    // 877
      }                                                                                                                // 878
                                                                                                                       //
      return proxy;                                                                                                    // 880
    } else {                                                                                                           // 881
      return inMeteor(function () {                                                                                    // 882
        var tokenInfo = ApiTokens.findOne(hashedToken);                                                                // 883
        validateWebkey(tokenInfo);                                                                                     // 884
        var grain = Grains.findOne(tokenInfo.grainId);                                                                 // 886
                                                                                                                       //
        if (!grain) {                                                                                                  // 887
          // Grain was deleted, I guess.                                                                               // 888
          throw new Meteor.Error(410, "Resource has been deleted");                                                    // 889
        }                                                                                                              // 890
                                                                                                                       //
        var proxy = void 0;                                                                                            // 892
                                                                                                                       //
        if (tokenInfo.userInfo) {                                                                                      // 893
          throw new Error("API tokens created with arbitrary userInfo no longer supported");                           // 894
        } else {                                                                                                       // 895
          var identityId = null;                                                                                       // 896
          var accountId = null;                                                                                        // 897
                                                                                                                       //
          if (tokenInfo.accountId && !tokenInfo.forSharing) {                                                          // 898
            accountId = tokenInfo.accountId;                                                                           // 899
            identityId = globalDb.getOrGenerateIdentityId(tokenInfo.accountId, grain);                                 // 900
          }                                                                                                            // 901
                                                                                                                       //
          proxy = new Proxy(grain, null, null, tabId, accountId, identityId, true);                                    // 903
          proxy.apiToken = tokenInfo;                                                                                  // 904
          proxy.apiSessionParams = serializedParams;                                                                   // 905
        }                                                                                                              // 906
                                                                                                                       //
        if (!SandstormPermissions.mayOpenGrain(globalDb, {                                                             // 908
          token: tokenInfo                                                                                             // 908
        })) {                                                                                                          // 908
          // Note that only public grains may be opened without a user ID.                                             // 909
          throw new Meteor.Error(403, "Unauthorized.");                                                                // 910
        }                                                                                                              // 911
                                                                                                                       //
        if (tokenInfo.expires) {                                                                                       // 913
          proxy.expires = tokenInfo.expires;                                                                           // 914
        }                                                                                                              // 915
                                                                                                                       //
        apiSessionProxies.put(hashedToken, hashedParams, proxy);                                                       // 917
        return proxy;                                                                                                  // 919
      });                                                                                                              // 920
    }                                                                                                                  // 921
  });                                                                                                                  // 922
};                                                                                                                     // 923
                                                                                                                       //
var apiUseBasicAuth = function (req, hostId) {                                                                         // 925
  // If the request was to a token-specific host *and* the request has no Origin header (meaning                       // 926
  // it is not an XHR from a web site), then we permit the use of HTTP basic auth. The reason we                       // 927
  // prohibit XHRs is because we want to discourage people from using basic auth in a browser,                         // 928
  // because the browser will cache the credentials, which exposes the hostname to XSRF attack                         // 929
  // (which is mostly, but not completely, mitigated by the unguessable hostname). The reason we                       // 930
  // want to allow basic auth at all is because a lot of existing client apps support only basic                       // 931
  // auth. New clients and web-based clients should use `Authorization: bearer <token>` instead.                       // 932
  if (globalDb.isTokenSpecificHostId(hostId) && !req.headers.origin) {                                                 // 933
    return true;                                                                                                       // 934
  } // Historically, all API tokens were served from the same host rather than have token-specific                     // 935
  // hosts. In this model, basic auth in browsers was far more dangerous since the hostname was                        // 938
  // not a secret -- in fact, even benign attempts to use two APIs on the same server from the same                    // 939
  // browser could interfere if using basic auth. Hence, basic auth was prohibited except when                         // 940
  // coming from certain whitelisted clients known not to be browsers, e.g. Mirall, the ownCloud                       // 941
  // client app.                                                                                                       // 942
  //                                                                                                                   // 943
  // Since many clients in the wild have already been configured to use the shared API host, we                        // 944
  // must continue to support them, so this logic remains.                                                             // 945
                                                                                                                       //
                                                                                                                       //
  var agent = req.headers["user-agent"];                                                                               // 946
  return agent && agent.match(BASIC_AUTH_USER_AGENTS_REGEX);                                                           // 947
};                                                                                                                     // 948
                                                                                                                       //
var apiTokenForRequest = function (req, hostId) {                                                                      // 950
  // Extract the API token from the request.                                                                           // 951
  var auth = req.headers.authorization;                                                                                // 953
  var token = void 0;                                                                                                  // 954
                                                                                                                       //
  if (auth && auth.slice(0, 7).toLowerCase() === "bearer ") {                                                          // 955
    token = auth.slice(7).trim();                                                                                      // 956
  } else if (auth && auth.slice(0, 6).toLowerCase() === "basic " && apiUseBasicAuth(req, hostId)) {                    // 957
    token = new Buffer(auth.slice(6).trim(), "base64").toString().split(":")[1];                                       // 959
  } else if (hostId !== "api" && req.url.startsWith("/.sandstorm-token/")) {                                           // 960
    // Disallow clients that use the old "api" hostId from using tokens in the path.                                   // 961
    // Allowing them would be a security problem in browsers that ignore CSP.                                          // 962
    var parts = req.url.slice(1).split("/"); // remove leading / and split                                             // 963
                                                                                                                       //
    if (parts.length < 2) {                                                                                            // 964
      token = undefined;                                                                                               // 965
    } else {                                                                                                           // 966
      token = parts[1];                                                                                                // 967
      req.url = "/" + parts.slice(2).join("/"); // remove .sandstorm-token/$TOKEN from path                            // 968
    }                                                                                                                  // 970
  } else {                                                                                                             // 971
    token = undefined;                                                                                                 // 972
  }                                                                                                                    // 973
                                                                                                                       //
  if (token && hostId !== "api") {                                                                                     // 975
    // Verify that the token matches the specific host.                                                                // 976
    if (hostId !== globalDb.apiHostIdForToken(token)) {                                                                // 977
      token = undefined;                                                                                               // 978
    }                                                                                                                  // 979
  }                                                                                                                    // 980
                                                                                                                       //
  return token;                                                                                                        // 982
}; // =======================================================================================                          // 983
// Routing to proxies.                                                                                                 // 986
//                                                                                                                     // 987
// pre-meteor.js calls this                                                                                            // 989
                                                                                                                       //
                                                                                                                       //
tryProxyUpgrade = function (hostId, req, socket, head) {                                                               // 990
  // Attempt to handle a WebSocket upgrade by dispatching it to a grain. Returns a promise that                        // 991
  // resolves true if an appropriate grain is found, false if there was no match (but the caller                       // 992
  // should consider other host types, like static web publishing), or throws an error if the                          // 993
  // request is definitely invalid.                                                                                    // 994
  if (globalDb.isApiHostId(hostId)) {                                                                                  // 996
    var token = apiTokenForRequest(req, hostId);                                                                       // 997
                                                                                                                       //
    if (token) {                                                                                                       // 998
      return getProxyForApiToken(token, req).then(function (proxy) {                                                   // 999
        // Meteor sets the timeout to five seconds. Change that back to two                                            // 1000
        // minutes, which is the default value.                                                                        // 1001
        socket.setTimeout(120000);                                                                                     // 1002
        proxy.upgradeHandler(req, socket, head);                                                                       // 1004
        return true;                                                                                                   // 1005
      });                                                                                                              // 1006
    } else {                                                                                                           // 1007
      return Promise.resolve(false);                                                                                   // 1008
    }                                                                                                                  // 1009
  } else {                                                                                                             // 1010
    var isAlreadyOpened = req.headers.cookie && req.headers.cookie.indexOf("sandstorm-sid=") !== -1;                   // 1011
    return getProxyForHostId(hostId, isAlreadyOpened).then(function (proxy) {                                          // 1012
      if (proxy) {                                                                                                     // 1013
        // Cross-origin requests are not allowed on UI session hosts.                                                  // 1014
        var origin = req.headers.origin;                                                                               // 1015
                                                                                                                       //
        if (origin !== PROTOCOL + "//" + req.headers.host) {                                                           // 1016
          throw new Meteor.Error(403, "Detected illegal cross-origin WebSocket from: " + origin);                      // 1017
        } // Meteor sets the timeout to five seconds. Change that back to two                                          // 1018
        // minutes, which is the default value.                                                                        // 1021
                                                                                                                       //
                                                                                                                       //
        socket.setTimeout(120000);                                                                                     // 1022
        proxy.upgradeHandler(req, socket, head);                                                                       // 1024
        return true;                                                                                                   // 1025
      } else {                                                                                                         // 1026
        return false;                                                                                                  // 1027
      }                                                                                                                // 1028
    });                                                                                                                // 1029
  }                                                                                                                    // 1030
}; // pre-meteor.js calls this                                                                                         // 1031
                                                                                                                       //
                                                                                                                       //
tryProxyRequest = function (hostId, req, res) {                                                                        // 1034
  // Attempt to handle an HTTP request by dispatching it to a grain. Returns a promise that                            // 1035
  // resolves true if an appropriate grain is found, false if there was no match (but the caller                       // 1036
  // should consider other host types, like static web publishing), or throws an error if the                          // 1037
  // request is definitely invalid.                                                                                    // 1038
  var hostIdHash = globalDb.isApiHostId(hostId);                                                                       // 1040
                                                                                                                       //
  if (hostIdHash) {                                                                                                    // 1041
    // This is a request for the API host.                                                                             // 1042
    res.setHeader("Access-Control-Allow-Origin", "*");                                                                 // 1044
                                                                                                                       //
    if (req.method === "OPTIONS") {                                                                                    // 1046
      // Reply to CORS preflight request.                                                                              // 1047
      // All we want to do is permit APIs to be accessed from arbitrary origins. Since clients                         // 1049
      // must send a valid Authorization header, and since cookies are not used for                                    // 1050
      // authorization, this is perfectly safe. In a sane world, we would only need to send back                       // 1051
      // 'Access-Control-Allow-Origin: *' and be done with it.                                                         // 1052
      //                                                                                                               // 1053
      // However, CORS demands that we explicitly whitelist individual methods and headers for                         // 1054
      // use cross-origin, as if this is somehow useful for implementing any practical security                        // 1055
      // policy (it isn't). To make matters worse, we are REQUIRED to enumerate each one                               // 1056
      // individually. We cannot just write '*' for these lists. WTF, CORS?                                            // 1057
      //                                                                                                               // 1058
      // Luckily, the request tells us exactly what method and headers are being requested, so we                      // 1059
      // only need to copy those over, rather than create an exhaustive list. But this is still                        // 1060
      // overly complicated.                                                                                           // 1061
      var accessControlHeaders = {                                                                                     // 1063
        "Access-Control-Allow-Methods": "GET, HEAD, POST, PUT, PATCH, DELETE",                                         // 1064
        "Access-Control-Max-Age": "3600"                                                                               // 1065
      }; // Copy all requested headers to the allowed headers list.                                                    // 1063
                                                                                                                       //
      var requestedHeaders = req.headers["access-control-request-headers"];                                            // 1069
                                                                                                                       //
      if (requestedHeaders) {                                                                                          // 1070
        accessControlHeaders["Access-Control-Allow-Headers"] = requestedHeaders;                                       // 1071
      } // Add the requested method to the allowed methods list, if it's not there already.                            // 1072
                                                                                                                       //
                                                                                                                       //
      var requestedMethod = req.headers["access-control-request-method"];                                              // 1075
                                                                                                                       //
      if (requestedMethod && !_.contains(["GET", "HEAD", "POST", "PUT", "PATCH", "DELETE"], requestedMethod)) {        // 1076
        accessControlHeaders["Access-Control-Allow-Methods"] += ", " + requestedMethod;                                // 1078
      }                                                                                                                // 1079
                                                                                                                       //
      for (header in meteorBabelHelpers.sanitizeForInObject(accessControlHeaders)) {                                   // 1081
        res.setHeader(header, accessControlHeaders[header]);                                                           // 1082
      }                                                                                                                // 1083
    }                                                                                                                  // 1084
                                                                                                                       //
    var errorHandler = function (err) {                                                                                // 1086
      if (err instanceof Meteor.Error) {                                                                               // 1087
        console.log("error: " + err);                                                                                  // 1088
        res.writeHead(err.error, err.reason, {                                                                         // 1089
          "Content-Type": "text/plain"                                                                                 // 1089
        });                                                                                                            // 1089
      } else {                                                                                                         // 1090
        res.writeHead(500, "Internal Server Error", {                                                                  // 1091
          "Content-Type": "text/plain"                                                                                 // 1091
        });                                                                                                            // 1091
      }                                                                                                                // 1092
                                                                                                                       //
      res.end(err.stack);                                                                                              // 1094
    };                                                                                                                 // 1095
                                                                                                                       //
    var token = apiTokenForRequest(req, hostId);                                                                       // 1097
                                                                                                                       //
    if (token && req.headers["x-sandstorm-token-keepalive"]) {                                                         // 1098
      inMeteor(function () {                                                                                           // 1099
        var keepaliveDuration = parseInt(req.headers["x-sandstorm-token-keepalive"]);                                  // 1100
        check(keepaliveDuration, Match.Integer);                                                                       // 1101
        var hashedToken = Crypto.createHash("sha256").update(token).digest("base64");                                  // 1102
        validateWebkey(ApiTokens.findOne(hashedToken), new Date(Date.now() + keepaliveDuration));                      // 1103
      }).then(function () {                                                                                            // 1104
        res.writeHead(200, {});                                                                                        // 1105
        res.end();                                                                                                     // 1106
      }, errorHandler);                                                                                                // 1107
    } else if (token) {                                                                                                // 1108
      getProxyForApiToken(token, req).then(function (proxy) {                                                          // 1109
        proxy.requestHandler(req, res);                                                                                // 1110
      }, errorHandler);                                                                                                // 1111
    } else {                                                                                                           // 1112
      // No token. Look up static API host info.                                                                       // 1113
      inMeteor(function () {                                                                                           // 1114
        var apiHost = globalDb.collections.apiHosts.findOne(hostIdHash);                                               // 1115
                                                                                                                       //
        if (req.method === "OPTIONS") {                                                                                // 1117
          // OPTIONS request with no authorization token.                                                              // 1118
          var dav = ((apiHost || {}).options || {}).dav || [];                                                         // 1120
                                                                                                                       //
          if (dav.length > 0) {                                                                                        // 1121
            res.setHeader("DAV", dav.join(", "));                                                                      // 1122
            res.setHeader("Access-Control-Expose-Headers", "DAV");                                                     // 1123
          }                                                                                                            // 1124
                                                                                                                       //
          res.writeHead(200, {});                                                                                      // 1126
          res.end();                                                                                                   // 1127
        } else {                                                                                                       // 1128
          var resources = (apiHost || {}).resources || {};                                                             // 1129
          var path = SandstormDb.escapeMongoKey(req.url.split("?")[0]);                                                // 1130
                                                                                                                       //
          if (path in resources) {                                                                                     // 1131
            // Serve a static resource.                                                                                // 1132
            var resource = resources[path];                                                                            // 1133
            console.log(path, resources, resource);                                                                    // 1134
            if (resource.language) res.setHeader("Content-Language", resource.language);                               // 1135
            if (resource.encoding) res.setHeader("Content-Encoding", resource.encoding);                               // 1136
            res.writeHead(200, {                                                                                       // 1137
              "Content-Type": resource.type                                                                            // 1138
            });                                                                                                        // 1137
            res.end(resource.body);                                                                                    // 1140
          } else {                                                                                                     // 1141
            if (apiUseBasicAuth(req, hostId)) {                                                                        // 1142
              res.writeHead(401, {                                                                                     // 1143
                "Content-Type": "text/plain",                                                                          // 1144
                "WWW-Authenticate": "Basic realm='Sandstorm API'"                                                      // 1145
              });                                                                                                      // 1143
            } else {                                                                                                   // 1147
              // TODO(someday): Display some sort of nifty API browser.                                                // 1148
              res.writeHead(403, {                                                                                     // 1149
                "Content-Type": "text/plain"                                                                           // 1149
              });                                                                                                      // 1149
            }                                                                                                          // 1150
                                                                                                                       //
            res.end("Missing or invalid authorization header.\n\n" + "This address serves APIs, which allow external apps (such as a phone app) to\n" + "access data on your Sandstorm server. This address is not meant to be opened\n" + "in a regular browser.");
          }                                                                                                            // 1156
        }                                                                                                              // 1157
      }).then(function () {}, errorHandler);                                                                           // 1158
    }                                                                                                                  // 1159
                                                                                                                       //
    return Promise.resolve(true);                                                                                      // 1161
  } else {                                                                                                             // 1162
    var isAlreadyOpened = req.headers.cookie && req.headers.cookie.indexOf("sandstorm-sid=") !== -1;                   // 1163
    return getProxyForHostId(hostId, isAlreadyOpened).then(function (proxy) {                                          // 1166
      if (proxy) {                                                                                                     // 1167
        // Let's do some CSRF defense.                                                                                 // 1168
        var expectedOrigin = PROTOCOL + "//" + req.headers.host;                                                       // 1169
        var mainUrl = proxy.parentOrigin;                                                                              // 1170
        var origin = req.headers.origin;                                                                               // 1171
        var referer = req.headers.referer;                                                                             // 1172
                                                                                                                       //
        if (origin) {                                                                                                  // 1173
          // If an origin header was provided, then it must be accurate. Note that Chrome and                          // 1174
          // Safari always send an Origin header on non-GET requests (even same-origin), and                           // 1175
          // therefore on those browsers this rule will block all CSRF attacks. Unfortunately,                         // 1176
          // a patch to add this behavior to Firefox has stalled:                                                      // 1177
          //     https://bugzilla.mozilla.org/show_bug.cgi?id=446344                                                   // 1178
          // I haven't found any information on IE/Edge's behavior.                                                    // 1179
          if (origin !== expectedOrigin) {                                                                             // 1180
            // TODO(security): Alas, it turns out we have apps that have:                                              // 1181
            //   <meta name="referrer" content="no-referrer">                                                          // 1182
            // as Chrome sends "Origin: null" in these cases. :( These apps need to switch to:                         // 1183
            //   <meta name="referrer" content="same-origin">                                                          // 1184
            // It's important that we don't break apps, so we will accept null origins for now,                        // 1185
            // which of course completely defeats any CSRF protection. We should get the apps to                       // 1186
            // update or apply a whitelist soon.                                                                       // 1187
            if (origin === "null") {                                                                                   // 1188
              if (!proxy.wroteCsrfWarning) {                                                                           // 1189
                console.warn("Note: Observed null Origin header. App needs to be updated so that " + "we can apply automatic CSRF protection. grain ID:", proxy.grainId);
                proxy.wroteCsrfWarning = true;                                                                         // 1193
              }                                                                                                        // 1194
            } else {                                                                                                   // 1195
              throw new Meteor.Error(403, "Blocked illegal cross-origin request from: " + origin);                     // 1196
            }                                                                                                          // 1197
          }                                                                                                            // 1198
        } else if (referer) {                                                                                          // 1199
          // Mark that we've seed a Referer header on this host, which indicates that the user's                       // 1200
          // browser is not suppressing them.                                                                          // 1201
          proxy.seenReferer = true; // Deny requests coming from an external referer, since there is no legitimate use case
          // for these. Note of course that an attacker can trivially suppress the referer header                      // 1205
          // in practice, so this check alone does not give us any security benefit if we accept                       // 1206
          // requests that lack a referer.                                                                             // 1207
                                                                                                                       //
          if (referer != expectedOrigin && !referer.startsWith(expectedOrigin + "/") && referer != mainUrl && !referer.startsWith(mainUrl + "/")) {
            throw new Meteor.Error(403, "Blocked illegal cross-origin referral from: " + referer);                     // 1210
          }                                                                                                            // 1211
        } else {                                                                                                       // 1212
          // We saw neither an Origin nor a Referer header.                                                            // 1213
          if (req.method !== "GET" && req.method !== "HEAD") {                                                         // 1214
            // This is possibly a side-effecting request, meaning we should worry about CSRF.                          // 1215
            //                                                                                                         // 1216
            // Note that we'll never get to this branch for Chrome or Safari since they would have                     // 1217
            // sent an Origin header.                                                                                  // 1218
            if (!proxy.wroteCsrfWarning) {                                                                             // 1220
              console.warn("Note: Observed session for which auto-CSRF-protection couldn't be applied.", "seenReferer:", !!proxy.seenReferer, "method:", req.method, "path:", req.url, "UA:", req.headers["user-agent"]);
              proxy.wroteCsrfWarning = true;                                                                           // 1225
            }                                                                                                          // 1226
          }                                                                                                            // 1227
        }                                                                                                              // 1228
                                                                                                                       //
        proxy.requestHandler(req, res);                                                                                // 1230
        return true;                                                                                                   // 1231
      } else {                                                                                                         // 1232
        return false;                                                                                                  // 1233
      }                                                                                                                // 1234
    });                                                                                                                // 1235
  }                                                                                                                    // 1236
}; // =======================================================================================                          // 1237
// Proxy class                                                                                                         // 1240
//                                                                                                                     // 1241
// Connects to a grain and exports it on a wildcard host.                                                              // 1242
//                                                                                                                     // 1243
                                                                                                                       //
                                                                                                                       //
var Proxy = function () {                                                                                              //
  function Proxy(grain, sessionId, hostId, tabId, accountId, identityId, isApi, parentOrigin, supervisor) {            // 1246
    var _this3 = this;                                                                                                 // 1247
                                                                                                                       //
    (0, _classCallCheck3.default)(this, Proxy);                                                                        // 1247
    // `grain` is an entry in the `Grains` collection.                                                                 // 1248
    this.grainId = grain._id;                                                                                          // 1249
    this.ownerId = grain.userId;                                                                                       // 1250
    this.accountId = accountId;                                                                                        // 1251
    this.identityId = identityId;                                                                                      // 1252
    this.supervisor = supervisor; // note: optional parameter; we can reconnect                                        // 1253
                                                                                                                       //
    this.sessionId = sessionId;                                                                                        // 1254
    this.tabId = tabId;                                                                                                // 1255
    this.isApi = isApi;                                                                                                // 1256
    this.hasLoaded = false;                                                                                            // 1257
    this.websockets = {};                                                                                              // 1258
    this.websocketCounter = 0; // Used for generating unique socket IDs.                                               // 1259
                                                                                                                       //
    this.streamingRequests = {};                                                                                       // 1260
    this.streamingRequestCounter = 0; // Used for generating unique request IDs.                                       // 1261
                                                                                                                       //
    this.streamingResponses = {};                                                                                      // 1262
    this.streamingResponseCounter = 0; // Used for generating unique response IDs.                                     // 1263
                                                                                                                       //
    this.parentOrigin = parentOrigin || process.env.ROOT_URL;                                                          // 1264
                                                                                                                       //
    if (sessionId) {                                                                                                   // 1266
      if (!hostId) throw new Error("sessionId must come with hostId");                                                 // 1267
      if (isApi) throw new Error("API proxy shouldn't have sessionId");                                                // 1268
      this.hostId = hostId;                                                                                            // 1269
    } else {                                                                                                           // 1270
      if (!isApi) throw new Error("non-API proxy requires sessionId");                                                 // 1271
      if (hostId) throw new Error("API proxy sholudn't have hostId");                                                  // 1272
    }                                                                                                                  // 1273
                                                                                                                       //
    if (this.accountId) {                                                                                              // 1275
      var user = Meteor.users.findOne({                                                                                // 1276
        _id: this.accountId                                                                                            // 1276
      });                                                                                                              // 1276
                                                                                                                       //
      if (!user) {                                                                                                     // 1277
        throw new Error("user not found: " + this.accountId);                                                          // 1278
      }                                                                                                                // 1279
                                                                                                                       //
      SandstormDb.fillInPictureUrl(user); // The identity cap becomes invalid if the user no longer has access to the grain.
                                                                                                                       //
      var idCapRequirement = {                                                                                         // 1284
        permissionsHeld: {                                                                                             // 1285
          accountId: user._id,                                                                                         // 1285
          grainId: this.grainId                                                                                        // 1285
        }                                                                                                              // 1285
      };                                                                                                               // 1284
      this.userInfo = {                                                                                                // 1288
        displayName: {                                                                                                 // 1289
          defaultText: user.profile.name                                                                               // 1289
        },                                                                                                             // 1289
        preferredHandle: user.profile.handle,                                                                          // 1290
        identityId: new Buffer(this.identityId, "hex"),                                                                // 1291
        identity: makeIdentity(user._id, [idCapRequirement]),                                                          // 1292
        pictureUrl: user.profile.pictureUrl                                                                            // 1293
      };                                                                                                               // 1288
      if (user.profile.pronoun) this.userInfo.pronouns = user.profile.pronoun;                                         // 1296
    } else {                                                                                                           // 1297
      this.userInfo = {                                                                                                // 1298
        displayName: {                                                                                                 // 1299
          defaultText: "Anonymous User"                                                                                // 1299
        },                                                                                                             // 1299
        preferredHandle: "anonymous"                                                                                   // 1300
      };                                                                                                               // 1298
    }                                                                                                                  // 1302
                                                                                                                       //
    this.requestHandler = function (request, response) {                                                               // 1304
      if (_this3.sessionId) {                                                                                          // 1305
        // Implement /_sandstorm-init for setting the session cookie.                                                  // 1306
        var url = Url.parse(request.url, true);                                                                        // 1307
                                                                                                                       //
        if (url.pathname === "/_sandstorm-init" && url.query.sessionid === _this3.sessionId) {                         // 1308
          _this3.doSessionInit(request, response, url.query.path);                                                     // 1309
                                                                                                                       //
          return;                                                                                                      // 1310
        }                                                                                                              // 1311
      }                                                                                                                // 1312
                                                                                                                       //
      Promise.resolve(undefined).then(function () {                                                                    // 1314
        var contentLength = request.headers["content-length"];                                                         // 1315
                                                                                                                       //
        if ((request.method === "POST" || request.method === "PUT") && (contentLength === undefined || contentLength > 1024 * 1024)) {
          // The input is either very long, or we don't know how long it is, so use streaming mode.                    // 1318
          return _this3.handleRequestStreaming(request, response, contentLength, 0);                                   // 1319
        } else {                                                                                                       // 1320
          return readAll(request).then(function (data) {                                                               // 1321
            return _this3.handleRequest(request, data, response, 0);                                                   // 1322
          });                                                                                                          // 1323
        }                                                                                                              // 1324
      }).catch(function (err) {                                                                                        // 1325
        _this3.setHasLoaded();                                                                                         // 1326
                                                                                                                       //
        var body = err.stack;                                                                                          // 1328
                                                                                                                       //
        if (err.cppFile) {                                                                                             // 1329
          body += "\nC++ location:" + err.cppFile + ":" + (err.line || "??");                                          // 1330
        }                                                                                                              // 1331
                                                                                                                       //
        if (err.kjType) {                                                                                              // 1333
          body += "\ntype: " + err.kjType;                                                                             // 1334
        }                                                                                                              // 1335
                                                                                                                       //
        if (response.headersSent) {                                                                                    // 1337
          // Unfortunately, it's too late to tell the client what happened.                                            // 1338
          console.error("HTTP request failed after response already sent:", body);                                     // 1339
          response.end();                                                                                              // 1340
        } else {                                                                                                       // 1341
          if (err instanceof Meteor.Error) {                                                                           // 1342
            response.writeHead(err.error, err.reason, {                                                                // 1343
              "Content-Type": "text/plain"                                                                             // 1343
            });                                                                                                        // 1343
          } else {                                                                                                     // 1344
            response.writeHead(500, "Internal Server Error", {                                                         // 1345
              "Content-Type": "text/plain"                                                                             // 1345
            });                                                                                                        // 1345
          }                                                                                                            // 1346
                                                                                                                       //
          response.end(body);                                                                                          // 1348
        }                                                                                                              // 1349
      });                                                                                                              // 1350
    };                                                                                                                 // 1351
                                                                                                                       //
    this.upgradeHandler = function (request, socket, head) {                                                           // 1353
      _this3.handleWebSocket(request, socket, head, 0).catch(function (err) {                                          // 1354
        console.error("WebSocket setup failed:", err.stack); // TODO(cleanup):  Manually send back a 500 response?     // 1355
                                                                                                                       //
        socket.destroy();                                                                                              // 1357
      });                                                                                                              // 1358
    };                                                                                                                 // 1359
  }                                                                                                                    // 1360
                                                                                                                       //
  Proxy.prototype.close = function () {                                                                                //
    function close() {                                                                                                 //
      for (var streamingResponseIdx in meteorBabelHelpers.sanitizeForInObject(this.streamingResponses)) {              // 1363
        this.streamingResponses[streamingResponseIdx].abort();                                                         // 1364
      }                                                                                                                // 1365
                                                                                                                       //
      this.streamingResponses = {};                                                                                    // 1367
                                                                                                                       //
      for (var streamingRequestIdx in meteorBabelHelpers.sanitizeForInObject(this.streamingRequests)) {                // 1369
        this.streamingRequests[streamingRequestIdx].destroy();                                                         // 1370
      }                                                                                                                // 1371
                                                                                                                       //
      this.streamingRequests = {};                                                                                     // 1373
                                                                                                                       //
      for (var socketIdx in meteorBabelHelpers.sanitizeForInObject(this.websockets)) {                                 // 1375
        this.websockets[socketIdx].destroy();                                                                          // 1376
      }                                                                                                                // 1377
                                                                                                                       //
      ;                                                                                                                // 1377
      this.websockets = {};                                                                                            // 1379
                                                                                                                       //
      if (this.session) {                                                                                              // 1381
        this.session.close();                                                                                          // 1382
        delete this.session;                                                                                           // 1383
      }                                                                                                                // 1384
                                                                                                                       //
      if (this.uiView) {                                                                                               // 1386
        this.uiView.close();                                                                                           // 1387
        delete this.uiView;                                                                                            // 1388
      }                                                                                                                // 1389
                                                                                                                       //
      if (this.supervisor) {                                                                                           // 1391
        this.supervisor.close();                                                                                       // 1392
        delete this.supervisor;                                                                                        // 1393
      }                                                                                                                // 1394
                                                                                                                       //
      if (this.permissionsObserver) {                                                                                  // 1396
        this.permissionsObserver.stop();                                                                               // 1397
        delete this.permissionsObserver;                                                                               // 1398
      }                                                                                                                // 1399
    }                                                                                                                  // 1400
                                                                                                                       //
    return close;                                                                                                      //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.getConnection = function () {                                                                        //
    function getConnection() {                                                                                         //
      if (!this.supervisor) {                                                                                          // 1403
        this.supervisor = globalBackend.cap().getGrain(this.ownerId, this.grainId).supervisor;                         // 1404
        this.uiView = null;                                                                                            // 1405
      }                                                                                                                // 1406
                                                                                                                       //
      if (!this.uiView) {                                                                                              // 1408
        this.uiView = this.supervisor.getMainView().view;                                                              // 1409
      }                                                                                                                // 1410
    }                                                                                                                  // 1411
                                                                                                                       //
    return getConnection;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype._callNewWebSession = function () {                                                                   //
    function _callNewWebSession(request, userInfo) {                                                                   //
      var params = Capnp.serialize(WebSession.Params, {                                                                // 1414
        basePath: PROTOCOL + "//" + request.headers.host,                                                              // 1415
        userAgent: "user-agent" in request.headers ? request.headers["user-agent"] : "UnknownAgent/0.0",               // 1416
        acceptableLanguages: "accept-language" in request.headers ? request.headers["accept-language"].split(",").map(function (s) {
          return s.trim();                                                                                             // 1420
        }) : ["en-US", "en"]                                                                                           // 1420
      });                                                                                                              // 1414
      return this.uiView.newSession(userInfo, makeHackSessionContext(this.grainId, this.sessionId, this.accountId, this.tabId), WebSession.typeId, params, new Buffer(this.tabId, "hex")).session;
    }                                                                                                                  // 1426
                                                                                                                       //
    return _callNewWebSession;                                                                                         //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype._callNewApiSession = function () {                                                                   //
    function _callNewApiSession(request, userInfo) {                                                                   //
      var _this4 = this;                                                                                               // 1428
                                                                                                                       //
      var serializedParams = this.apiSessionParams;                                                                    // 1429
                                                                                                                       //
      if (!serializedParams) {                                                                                         // 1430
        throw new Meteor.Error(500, "Should have already computed apiSessionParams.");                                 // 1431
      } // TODO(someday): We are currently falling back to WebSession if we get any kind of error upon                 // 1432
      // calling newSession with an ApiSession._id.                                                                    // 1435
      // Eventually we'll remove this logic once we're sure apps have updated.                                         // 1436
                                                                                                                       //
                                                                                                                       //
      return this.uiView.newSession(userInfo, makeHackSessionContext(this.grainId, this.sessionId, this.accountId, this.tabId), ApiSession.typeId, serializedParams, new Buffer(this.tabId, "hex")).then(function (session) {
        return session.session;                                                                                        // 1441
      }, function (err) {                                                                                              // 1442
        return _this4._callNewWebSession(request, userInfo);                                                           // 1443
      });                                                                                                              // 1444
    }                                                                                                                  // 1445
                                                                                                                       //
    return _callNewApiSession;                                                                                         //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype._callNewRequestSession = function () {                                                               //
    function _callNewRequestSession(request, userInfo) {                                                               //
      var params = Capnp.serialize(WebSession.Params, {                                                                // 1448
        basePath: PROTOCOL + "//" + request.headers.host,                                                              // 1449
        userAgent: "user-agent" in request.headers ? request.headers["user-agent"] : "UnknownAgent/0.0",               // 1450
        acceptableLanguages: "accept-language" in request.headers ? request.headers["accept-language"].split(",").map(function (s) {
          return s.trim();                                                                                             // 1454
        }) : ["en-US", "en"]                                                                                           // 1454
      });                                                                                                              // 1448
      return this.uiView.newRequestSession(userInfo, makeHackSessionContext(this.grainId, this.sessionId, this.accountId, this.tabId), WebSession.typeId, params, this.powerboxRequest.descriptors, new Buffer(this.tabId, "hex")).session;
    }                                                                                                                  // 1461
                                                                                                                       //
    return _callNewRequestSession;                                                                                     //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype._callNewSession = function () {                                                                      //
    function _callNewSession(request, viewInfo) {                                                                      //
      var _this5 = this;                                                                                               // 1463
                                                                                                                       //
      var userInfo = _.clone(this.userInfo);                                                                           // 1464
                                                                                                                       //
      var promise = inMeteor(function () {                                                                             // 1465
        var vertex = void 0;                                                                                           // 1466
                                                                                                                       //
        if (_this5.apiToken) {                                                                                         // 1467
          vertex = {                                                                                                   // 1468
            token: _this5.apiToken                                                                                     // 1468
          };                                                                                                           // 1468
        } else {                                                                                                       // 1469
          // (this.accountId might be null; this is fine)                                                              // 1470
          vertex = {                                                                                                   // 1471
            grain: {                                                                                                   // 1471
              _id: _this5.grainId,                                                                                     // 1471
              accountId: _this5.accountId                                                                              // 1471
            }                                                                                                          // 1471
          };                                                                                                           // 1471
        }                                                                                                              // 1472
                                                                                                                       //
        var onInvalidated = function () {                                                                              // 1474
          Sessions.remove({                                                                                            // 1475
            _id: _this5.sessionId                                                                                      // 1475
          });                                                                                                          // 1475
        };                                                                                                             // 1476
                                                                                                                       //
        if (!_this5.sessionId) {                                                                                       // 1478
          onInvalidated = function () {                                                                                // 1479
            apiSessionProxies.removeProxiesOfToken(_this5.apiToken._id);                                               // 1480
          };                                                                                                           // 1481
        }                                                                                                              // 1482
                                                                                                                       //
        var permissions = SandstormPermissions.grainPermissions(globalDb, vertex, viewInfo, onInvalidated);            // 1484
        if (_this5.permissionsObserver) _this5.permissionsObserver.stop();                                             // 1487
        _this5.permissionsObserver = permissions.observeHandle;                                                        // 1488
                                                                                                                       //
        if (!permissions.permissions) {                                                                                // 1490
          throw new Meteor.Error(403, "Unauthorized", "User is not authorized to open this grain.");                   // 1491
        }                                                                                                              // 1492
                                                                                                                       //
        Sessions.update({                                                                                              // 1494
          _id: _this5.sessionId                                                                                        // 1495
        }, {                                                                                                           // 1494
          $set: {                                                                                                      // 1497
            viewInfo: viewInfo,                                                                                        // 1498
            permissions: permissions.permissions                                                                       // 1499
          }                                                                                                            // 1497
        });                                                                                                            // 1496
        return permissions.permissions;                                                                                // 1503
      });                                                                                                              // 1504
      return promise.then(function (permissions) {                                                                     // 1506
        userInfo.permissions = permissions;                                                                            // 1507
        var numBytes = Math.ceil(permissions.length / 8);                                                              // 1509
        var buf = new Buffer(numBytes);                                                                                // 1510
                                                                                                                       //
        for (var ii = 0; ii < numBytes; ++ii) {                                                                        // 1511
          buf[ii] = 0;                                                                                                 // 1512
        }                                                                                                              // 1513
                                                                                                                       //
        for (var _ii = 0; _ii < permissions.length; ++_ii) {                                                           // 1515
          var byteNum = Math.floor(_ii / 8);                                                                           // 1516
          var bitNum = _ii % 8;                                                                                        // 1517
                                                                                                                       //
          if (permissions[_ii]) {                                                                                      // 1518
            buf[byteNum] = buf[byteNum] | 1 << bitNum;                                                                 // 1519
          }                                                                                                            // 1520
        }                                                                                                              // 1521
                                                                                                                       //
        userInfo.deprecatedPermissionsBlob = buf;                                                                      // 1523
                                                                                                                       //
        if (_this5.isApi) {                                                                                            // 1525
          return _this5._callNewApiSession(request, userInfo);                                                         // 1526
        } else if (_this5.powerboxRequest) {                                                                           // 1527
          return _this5._callNewRequestSession(request, userInfo);                                                     // 1528
        } else {                                                                                                       // 1529
          return _this5._callNewWebSession(request, userInfo);                                                         // 1530
        }                                                                                                              // 1531
      });                                                                                                              // 1532
    }                                                                                                                  // 1533
                                                                                                                       //
    return _callNewSession;                                                                                            //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.getSession = function () {                                                                           //
    function getSession(request) {                                                                                     //
      var _this6 = this;                                                                                               // 1535
                                                                                                                       //
      if (!this.session) {                                                                                             // 1536
        this.getConnection(); // make sure we're connected                                                             // 1537
                                                                                                                       //
        var promise = this.uiView.getViewInfo().then(function (viewInfo) {                                             // 1538
          return inMeteor(function () {                                                                                // 1539
            // For now, we don't allow grains to set `appTitle` or `grainIcon`.                                        // 1540
            var cachedViewInfo = _.omit(viewInfo, "appTitle", "grainIcon");                                            // 1541
                                                                                                                       //
            Grains.update(_this6.grainId, {                                                                            // 1542
              $set: {                                                                                                  // 1542
                cachedViewInfo: cachedViewInfo                                                                         // 1542
              }                                                                                                        // 1542
            });                                                                                                        // 1542
          }).then(function () {                                                                                        // 1543
            return _this6._callNewSession(request, viewInfo);                                                          // 1544
          });                                                                                                          // 1545
        }, function (error) {                                                                                          // 1546
          if (error.kjType === "failed" || error.kjType === "unimplemented") {                                         // 1547
            // Method not implemented.                                                                                 // 1548
            // TODO(apibump): Don't treat 'failed' as 'unimplemented'. Unfortunately, old apps built                   // 1549
            //   with old versions of Cap'n Proto don't throw 'unimplemented' exceptions, so we have                   // 1550
            //   to accept 'failed' here at least until the next API bump.                                             // 1551
            return _this6._callNewSession(request, {});                                                                // 1552
          } else {                                                                                                     // 1553
            return Promise.reject(error);                                                                              // 1554
          }                                                                                                            // 1555
        });                                                                                                            // 1556
        this.session = new Capnp.Capability(promise, WebSession);                                                      // 1557
      }                                                                                                                // 1558
                                                                                                                       //
      return this.session;                                                                                             // 1560
    }                                                                                                                  // 1561
                                                                                                                       //
    return getSession;                                                                                                 //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.keepAlive = function () {                                                                            //
    function keepAlive() {                                                                                             //
      this.getConnection();                                                                                            // 1564
      return this.supervisor.keepAlive();                                                                              // 1565
    }                                                                                                                  // 1566
                                                                                                                       //
    return keepAlive;                                                                                                  //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.resetConnection = function () {                                                                      //
    function resetConnection() {                                                                                       //
      if (this.session) {                                                                                              // 1569
        this.session.close();                                                                                          // 1570
        delete this.session;                                                                                           // 1571
      }                                                                                                                // 1572
                                                                                                                       //
      if (this.uiView) {                                                                                               // 1574
        this.uiView.close();                                                                                           // 1575
        delete this.uiView;                                                                                            // 1576
      }                                                                                                                // 1577
                                                                                                                       //
      if (this.supervisor) {                                                                                           // 1579
        this.supervisor.close();                                                                                       // 1580
        delete this.supervisor;                                                                                        // 1581
      }                                                                                                                // 1582
                                                                                                                       //
      if (this.permissionsObserver) {                                                                                  // 1584
        this.permissionsObserver.stop();                                                                               // 1585
        delete this.permissionsObserver;                                                                               // 1586
      }                                                                                                                // 1587
    }                                                                                                                  // 1588
                                                                                                                       //
    return resetConnection;                                                                                            //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.maybeRetryAfterError = function () {                                                                 //
    function maybeRetryAfterError(error, retryCount) {                                                                 //
      var _this7 = this;                                                                                               // 1590
                                                                                                                       //
      // If the error may be caused by the grain dying or a network failure, try to restart it,                        // 1591
      // returning a promise that resolves once restarted. Otherwise, just rethrow the error.                          // 1592
      // `retryCount` should be incremented for every successful retry as part of the same request;                    // 1593
      // we only want to retry once.                                                                                   // 1594
      if (shouldRestartGrain(error, retryCount)) {                                                                     // 1595
        this.resetConnection();                                                                                        // 1596
        return inMeteor(function () {                                                                                  // 1597
          _this7.supervisor = globalBackend.continueGrain(_this7.grainId).supervisor;                                  // 1598
        });                                                                                                            // 1599
      } else {                                                                                                         // 1600
        throw error;                                                                                                   // 1601
      }                                                                                                                // 1602
    }                                                                                                                  // 1603
                                                                                                                       //
    return maybeRetryAfterError;                                                                                       //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.doSessionInit = function () {                                                                        //
    function doSessionInit(request, response, requestPath) {                                                           //
      // jscs:disable disallowQuotedKeysInObjects                                                                      // 1606
      var path = requestPath || "/"; // Check that the path is relative (ie. starts with a /).                         // 1607
      // Also ensure that it doesn't start with 2 /, because that is interpreted as non-relative                       // 1610
                                                                                                                       //
      if (path.lastIndexOf("/", 0) !== 0 || path.lastIndexOf("//", 0) === 0) {                                         // 1611
        response.writeHead(400, "Invalid path supplied", {                                                             // 1612
          "Content-Type": "text/plain"                                                                                 // 1612
        });                                                                                                            // 1612
        response.end("Invalid path supplied.");                                                                        // 1613
        return;                                                                                                        // 1614
      } // Set the session ID.                                                                                         // 1615
                                                                                                                       //
                                                                                                                       //
      response.setHeader("Set-Cookie", ["sandstorm-sid=", this.sessionId, "; Max-Age=31536000; HttpOnly"].join(""));   // 1618
      response.setHeader("Cache-Control", "no-cache, private"); // Redirect to the app's root URL.                     // 1620
      // Note:  All browsers support relative locations and the next update to HTTP/1.1 will officially                // 1623
      //   make them valid.  http://tools.ietf.org/html/draft-ietf-httpbis-p2-semantics-26#page-67                     // 1624
                                                                                                                       //
      response.writeHead(303, "See Other", {                                                                           // 1625
        "Location": encodeURI(path)                                                                                    // 1625
      });                                                                                                              // 1625
      response.end();                                                                                                  // 1626
    }                                                                                                                  // 1627
                                                                                                                       //
    return doSessionInit;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.makeContext = function () {                                                                          //
    function makeContext(request, response, callback) {                                                                //
      // Parses the cookies from the request, checks that the session ID is present and valid, then                    // 1630
      // synchronously calls callback() passing it the request context object which contains the                       // 1631
      // other cookies. Throws an exception if the session ID is missing or invalid. The callback                      // 1632
      // must immediately use the context in an RPC. Once the callback returns, the context object                     // 1633
      // will be torn down.                                                                                            // 1634
      //                                                                                                               // 1635
      // `makeContext()` returns the result of `callback()`.                                                           // 1636
      var context = {};                                                                                                // 1638
                                                                                                                       //
      if (this.hostId) {                                                                                               // 1640
        var parseResult = parseCookies(request);                                                                       // 1641
                                                                                                                       //
        if (!parseResult.sessionId || parseResult.sessionId !== this.sessionId) {                                      // 1642
          throw new Meteor.Error(403, "Unauthorized");                                                                 // 1643
        }                                                                                                              // 1644
                                                                                                                       //
        if (parseResult.cookies.length > 0) {                                                                          // 1646
          context.cookies = parseResult.cookies;                                                                       // 1647
        }                                                                                                              // 1648
      } else {// jscs:disable disallowEmptyBlocks                                                                      // 1649
        // This is an API request. Cookies are not supported.                                                          // 1651
      }                                                                                                                // 1652
                                                                                                                       //
      context.accept = parseAcceptHeader(request);                                                                     // 1654
      context.acceptEncoding = parseAcceptEncodingHeader(request);                                                     // 1656
      context.eTagPrecondition = parsePreconditionHeader(request);                                                     // 1658
      context.additionalHeaders = [];                                                                                  // 1660
                                                                                                                       //
      for (var headerName in meteorBabelHelpers.sanitizeForInObject(request.headers)) {                                // 1661
        if (REQUEST_HEADER_WHITELIST.matches(headerName)) {                                                            // 1662
          context.additionalHeaders.push({                                                                             // 1663
            name: headerName,                                                                                          // 1664
            value: request.headers[headerName]                                                                         // 1665
          });                                                                                                          // 1663
        }                                                                                                              // 1667
      }                                                                                                                // 1668
                                                                                                                       //
      if (response) {                                                                                                  // 1670
        var promise = new Promise(function (resolve, reject) {                                                         // 1671
          response.resolveResponseStream = resolve;                                                                    // 1672
          response.rejectResponseStream = reject;                                                                      // 1673
        });                                                                                                            // 1674
        context.responseStream = new Capnp.Capability(promise, ByteStream);                                            // 1676
                                                                                                                       //
        try {                                                                                                          // 1678
          return callback(context);                                                                                    // 1679
        } finally {                                                                                                    // 1680
          context.responseStream.close();                                                                              // 1681
        }                                                                                                              // 1682
      } else {                                                                                                         // 1683
        // WebSocket -- no response object.                                                                            // 1684
        return callback(context);                                                                                      // 1685
      }                                                                                                                // 1686
    }                                                                                                                  // 1687
                                                                                                                       //
    return makeContext;                                                                                                //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.translateResponse = function () {                                                                    //
    function translateResponse(rpcResponse, response, request) {                                                       //
      var _this8 = this;                                                                                               // 1689
                                                                                                                       //
      if (this.hostId) {                                                                                               // 1690
        if (rpcResponse.setCookies && rpcResponse.setCookies.length > 0) {                                             // 1691
          response.setHeader("Set-Cookie", rpcResponse.setCookies.map(makeSetCookieHeader));                           // 1692
        }                                                                                                              // 1693
                                                                                                                       //
        var mainUrl = this.parentOrigin;                                                                               // 1695
        var grainHost = PROTOCOL + "//" + request.headers.host;                                                        // 1696
        response.setHeader("Content-Security-Policy", "frame-ancestors " + mainUrl + " " + grainHost);                 // 1697
        response.setHeader("X-Frame-Options", "ALLOW-FROM " + mainUrl); // TODO(security): Add a Content-Security-Policy header which:
        // (1) Prevents the app from initiating HTTP requests to third parties.                                        // 1701
        // (2) Prevents the app from navigating the parent frame.                                                      // 1702
        // (3) Prevents the app from opening popups.                                                                   // 1703
        // (4) Prohibits anyone other than the Sandstorm shell from framing the app (as a backup                       // 1704
        //   defense vs. clickjacking, though unguessable hostnames already mostly prevent this).                      // 1705
      } else {                                                                                                         // 1706
        // This is an API request. Cookies are not supported.                                                          // 1707
        // We need to make sure caches know that different bearer tokens get totally different results.                // 1709
        response.setHeader("Vary", "Authorization"); // APIs can be called from any origin. Because we ignore cookies, there is no security problem.
                                                                                                                       //
        response.setHeader("Access-Control-Allow-Origin", "*"); // Add a Content-Security-Policy as a backup in case someone finds a way to load this resource
        // in a browser context. This policy should thoroughly neuter it.                                              // 1716
                                                                                                                       //
        response.setHeader("Content-Security-Policy", "default-src 'none'; sandbox");                                  // 1717
      }                                                                                                                // 1718
                                                                                                                       //
      if (rpcResponse.additionalHeaders && rpcResponse.additionalHeaders.length > 0) {                                 // 1720
        // Add any additional prefixed/whitelisted headers to the response.                                            // 1721
        // We cannot trust that the headers in the RPC response are actually                                           // 1722
        // allowed, so we check if they are prefixed or whitelisted.                                                   // 1723
        rpcResponse.additionalHeaders.forEach(function (header) {                                                      // 1724
          if (RESPONSE_HEADER_WHITELIST.matches(header.name)) {                                                        // 1725
            // If header is prefixed with appHeaderPrefix,                                                             // 1726
            // or if the header name is in whitelist, set it.                                                          // 1727
            response.setHeader(header.name, header.value);                                                             // 1728
          }                                                                                                            // 1729
        });                                                                                                            // 1730
      } // On first response, update the session to have hasLoaded=true                                                // 1731
                                                                                                                       //
                                                                                                                       //
      this.setHasLoaded(); // TODO(security): Set X-Content-Type-Options: nosniff?                                     // 1734
                                                                                                                       //
      if ("content" in rpcResponse) {                                                                                  // 1738
        var content = rpcResponse.content;                                                                             // 1739
        var code = successCodes[content.statusCode];                                                                   // 1740
                                                                                                                       //
        if (!code) {                                                                                                   // 1741
          throw new Error("Unknown status code: ", content.statusCode);                                                // 1742
        }                                                                                                              // 1743
                                                                                                                       //
        if (content.mimeType) {                                                                                        // 1745
          response.setHeader("Content-Type", content.mimeType);                                                        // 1746
        }                                                                                                              // 1747
                                                                                                                       //
        if (content.encoding) {                                                                                        // 1749
          response.setHeader("Content-Encoding", content.encoding);                                                    // 1750
        }                                                                                                              // 1751
                                                                                                                       //
        if (content.language) {                                                                                        // 1753
          response.setHeader("Content-Language", content.language);                                                    // 1754
        }                                                                                                              // 1755
                                                                                                                       //
        if (content.eTag) {                                                                                            // 1757
          response.setHeader("ETag", composeETag(content.eTag));                                                       // 1758
        }                                                                                                              // 1759
                                                                                                                       //
        if ("disposition" in content && "download" in content.disposition) {                                           // 1761
          response.setHeader("Content-Disposition", 'attachment; filename="' + content.disposition.download.replace(/([\\"\n])/g, "\\$1") + '"');
        }                                                                                                              // 1764
                                                                                                                       //
        if ("stream" in content.body) {                                                                                // 1766
          if (request.method === "HEAD") {                                                                             // 1767
            content.body.stream.close();                                                                               // 1768
            response.rejectResponseStream(new Error("HEAD request; content doesn't matter."));                         // 1769
          } else if (response.socket.destroyed) {                                                                      // 1770
            // The client has already closed the stream. No additional "close" event will be                           // 1771
            // emitted. Annoyingly, we cannot rely on `response.write()` throwing an exception                         // 1772
            // in this case, so if we don't handle it specially we could end up with a                                 // 1773
            // `ResponseStream` that gets stuck waiting for a "drain" event that never arrives.                        // 1774
            content.body.stream.close();                                                                               // 1775
            response.rejectResponseStream(new Error("client disconnected"));                                           // 1776
          } else {                                                                                                     // 1777
            var idx = this.streamingResponseCounter;                                                                   // 1778
            this.streamingResponseCounter += 1;                                                                        // 1779
            var streamHandle = content.body.stream;                                                                    // 1780
            var rawResponseStream = new ResponseStream(response, code.id, code.title, streamHandle, function () {      // 1781
              delete _this8.streamingResponses[idx];                                                                   // 1783
            });                                                                                                        // 1783
            this.streamingResponses[idx] = rawResponseStream;                                                          // 1784
            var responseStream = new Capnp.Capability(rawResponseStream, ByteStream);                                  // 1785
            response.resolveResponseStream(responseStream);                                                            // 1786
            return {                                                                                                   // 1787
              streamHandle: responseStream                                                                             // 1787
            };                                                                                                         // 1787
          }                                                                                                            // 1788
        } else {                                                                                                       // 1789
          response.rejectResponseStream(new Error("Response content body was not a stream."));                         // 1790
                                                                                                                       //
          if ("bytes" in content.body) {                                                                               // 1793
            response.setHeader("Content-Length", content.body.bytes.length);                                           // 1794
          } else {                                                                                                     // 1795
            throw new Error("Unknown content body type.");                                                             // 1796
          }                                                                                                            // 1797
        }                                                                                                              // 1798
                                                                                                                       //
        response.writeHead(code.id, code.title);                                                                       // 1800
                                                                                                                       //
        if ("bytes" in content.body && request.method !== "HEAD") {                                                    // 1802
          response.write(content.body.bytes);                                                                          // 1803
        }                                                                                                              // 1804
                                                                                                                       //
        response.end();                                                                                                // 1806
      } else if ("noContent" in rpcResponse) {                                                                         // 1807
        var noContent = rpcResponse.noContent;                                                                         // 1808
        var noContentCode = noContentSuccessCodes[noContent.shouldResetForm * 1];                                      // 1809
                                                                                                                       //
        if (noContent.eTag) {                                                                                          // 1810
          response.setHeader("ETag", composeETag(noContent.eTag));                                                     // 1811
        }                                                                                                              // 1812
                                                                                                                       //
        response.writeHead(noContentCode.id, noContentCode.title);                                                     // 1814
        response.end();                                                                                                // 1815
      } else if ("preconditionFailed" in rpcResponse) {                                                                // 1816
        var preconditionFailed = rpcResponse.preconditionFailed;                                                       // 1817
                                                                                                                       //
        if (request.method === "GET" && "if-none-match" in request.headers) {                                          // 1818
          if (preconditionFailed.matchingETag) {                                                                       // 1819
            response.setHeader("ETag", composeETag(preconditionFailed.matchingETag));                                  // 1820
          }                                                                                                            // 1821
                                                                                                                       //
          response.writeHead(304, "Not Modified");                                                                     // 1823
        } else {                                                                                                       // 1824
          response.writeHead(412, "Precondition Failed");                                                              // 1825
        }                                                                                                              // 1826
                                                                                                                       //
        response.end();                                                                                                // 1828
      } else if ("redirect" in rpcResponse) {                                                                          // 1829
        var redirect = rpcResponse.redirect;                                                                           // 1830
        var redirectCode = redirectCodes[redirect.switchToGet * 2 + redirect.isPermanent];                             // 1831
        response.writeHead(redirectCode.id, redirectCode.title, {                                                      // 1832
          "Location": encodeURI(redirect.location)                                                                     // 1833
        });                                                                                                            // 1832
        response.end();                                                                                                // 1835
      } else if ("clientError" in rpcResponse) {                                                                       // 1836
        var clientError = rpcResponse.clientError;                                                                     // 1837
        var errorCode = errorCodes[clientError.statusCode];                                                            // 1838
                                                                                                                       //
        if (!errorCode) {                                                                                              // 1839
          throw new Error("Unknown status code: ", clientError.statusCode);                                            // 1840
        }                                                                                                              // 1841
                                                                                                                       //
        response.writeHead(errorCode.id, errorCode.title, {                                                            // 1843
          "Content-Type": "text/html"                                                                                  // 1844
        });                                                                                                            // 1843
                                                                                                                       //
        if (request.method !== "HEAD") {                                                                               // 1847
          if (clientError.descriptionHtml) {                                                                           // 1848
            response.write(clientError.descriptionHtml);                                                               // 1849
          } else {                                                                                                     // 1850
            // TODO(someday):  Better default error page.                                                              // 1851
            response.write("<html><body><h1>" + errorCode.id + ": " + errorCode.title + "</h1></body></html>");        // 1852
          }                                                                                                            // 1854
        }                                                                                                              // 1855
                                                                                                                       //
        response.end();                                                                                                // 1857
      } else if ("serverError" in rpcResponse) {                                                                       // 1858
        response.writeHead(500, "Internal Server Error", {                                                             // 1859
          "Content-Type": "text/html"                                                                                  // 1860
        });                                                                                                            // 1859
                                                                                                                       //
        if (request.method !== "HEAD") {                                                                               // 1863
          if (rpcResponse.serverError.descriptionHtml) {                                                               // 1864
            response.write(rpcResponse.serverError.descriptionHtml);                                                   // 1865
          } else {                                                                                                     // 1866
            // TODO(someday):  Better default error page.                                                              // 1867
            response.write("<html><body><h1>500: Internal Server Error</h1></body></html>");                           // 1868
          }                                                                                                            // 1869
        }                                                                                                              // 1870
                                                                                                                       //
        response.end();                                                                                                // 1872
      } else {                                                                                                         // 1873
        throw new Error("Unknown HTTP response type:\n" + JSON.stringify(rpcResponse));                                // 1874
      }                                                                                                                // 1875
                                                                                                                       //
      return {};                                                                                                       // 1877
    }                                                                                                                  // 1878
                                                                                                                       //
    return translateResponse;                                                                                          //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.handleRequest = function () {                                                                        //
    function handleRequest(request, data, response, retryCount) {                                                      //
      var _this9 = this;                                                                                               // 1880
                                                                                                                       //
      return Promise.resolve(undefined).then(function () {                                                             // 1881
        return _this9.makeContext(request, response, function (context) {                                              // 1882
          // jscs:disable requireDotNotation                                                                           // 1883
          // Send the RPC.                                                                                             // 1884
          var path = request.url.slice(1); // remove leading '/'                                                       // 1885
                                                                                                                       //
          var session = _this9.getSession(request);                                                                    // 1886
                                                                                                                       //
          var requestContent = function () {                                                                           // 1888
            return {                                                                                                   // 1889
              content: data,                                                                                           // 1890
              encoding: request.headers["content-encoding"],                                                           // 1891
              mimeType: request.headers["content-type"]                                                                // 1892
            };                                                                                                         // 1889
          };                                                                                                           // 1894
                                                                                                                       //
          var xmlContent = function () {                                                                               // 1896
            var type = request.headers["content-type"] || "application/xml;charset=utf-8";                             // 1897
            var match = type.match(/[^/]*\/xml(; *charset *= *([^ ;]*))?/);                                            // 1898
                                                                                                                       //
            if (!match) {                                                                                              // 1899
              response.writeHead(415, "Unsupported media type.", {                                                     // 1900
                "Content-Type": "text/plain"                                                                           // 1901
              });                                                                                                      // 1900
              response.end("expected XML request body");                                                               // 1903
              throw new Error("expected XML request body");                                                            // 1904
            }                                                                                                          // 1905
                                                                                                                       //
            var charset = match[2] || "ISO-8859-1";                                                                    // 1907
            var encoding = request.headers["content-encoding"];                                                        // 1909
                                                                                                                       //
            if (encoding && encoding !== "identity") {                                                                 // 1910
              if (encoding !== "gzip") throw new Error("unknown Content-Encoding: " + encoding);                       // 1911
              data = gunzipSync(data);                                                                                 // 1912
            }                                                                                                          // 1913
                                                                                                                       //
            return data.toString(charset.toLowerCase() === "utf-8" ? "utf8" : "binary");                               // 1915
          };                                                                                                           // 1916
                                                                                                                       //
          var propfindDepth = function () {                                                                            // 1918
            var depth = request.headers["depth"];                                                                      // 1919
            return depth === "0" ? "zero" : depth === "1" ? "one" : "infinity";                                        // 1920
          };                                                                                                           // 1923
                                                                                                                       //
          var shallow = function () {                                                                                  // 1925
            return request.headers["depth"] === "0";                                                                   // 1926
          };                                                                                                           // 1927
                                                                                                                       //
          var noOverwrite = function () {                                                                              // 1929
            return (request.headers["overwrite"] || "").toLowerCase() === "f";                                         // 1930
          };                                                                                                           // 1931
                                                                                                                       //
          var destination = function () {                                                                              // 1933
            var result = request.headers["destination"];                                                               // 1934
            if (!result) throw new Error("missing destination");                                                       // 1935
            return Url.parse(result).path.slice(1); // remove leading '/'                                              // 1936
          };                                                                                                           // 1937
                                                                                                                       //
          if (request.method === "GET" || request.method === "HEAD") {                                                 // 1939
            return session.get(path, context, request.method === "HEAD");                                              // 1940
          } else if (request.method === "POST") {                                                                      // 1941
            return session.post(path, requestContent(), context);                                                      // 1942
          } else if (request.method === "PUT") {                                                                       // 1943
            return session.put(path, requestContent(), context);                                                       // 1944
          } else if (request.method === "PATCH") {                                                                     // 1945
            return session.patch(path, requestContent(), context);                                                     // 1946
          } else if (request.method === "DELETE") {                                                                    // 1947
            return session.delete(path, context);                                                                      // 1948
          } else if (request.method === "PROPFIND") {                                                                  // 1949
            return session.propfind(path, xmlContent(), propfindDepth(), context);                                     // 1950
          } else if (request.method === "PROPPATCH") {                                                                 // 1951
            return session.proppatch(path, xmlContent(), context);                                                     // 1952
          } else if (request.method === "MKCOL") {                                                                     // 1953
            return session.mkcol(path, requestContent(), context);                                                     // 1954
          } else if (request.method === "COPY") {                                                                      // 1955
            return session.copy(path, destination(), noOverwrite(), shallow(), context);                               // 1956
          } else if (request.method === "MOVE") {                                                                      // 1957
            return session.move(path, destination(), noOverwrite(), context);                                          // 1958
          } else if (request.method === "LOCK") {                                                                      // 1959
            return session.lock(path, xmlContent(), shallow(), context);                                               // 1960
          } else if (request.method === "UNLOCK") {                                                                    // 1961
            return session.unlock(path, request.headers["lock-token"], context);                                       // 1962
          } else if (request.method === "ACL") {                                                                       // 1963
            return session.acl(path, xmlContent(), context);                                                           // 1964
          } else if (request.method === "REPORT") {                                                                    // 1965
            return session.report(path, requestContent(), context);                                                    // 1966
          } else if (request.method === "OPTIONS") {                                                                   // 1967
            return session.options(path, context).then(function (options) {                                            // 1968
              var dav = [];                                                                                            // 1969
              if (options.davClass1) dav.push("1");                                                                    // 1970
              if (options.davClass2) dav.push("2");                                                                    // 1971
              if (options.davClass3) dav.push("3");                                                                    // 1972
                                                                                                                       //
              if (options.davExtensions) {                                                                             // 1973
                options.davExtensions.forEach(function (token) {                                                       // 1974
                  if (token.match(/^([a-zA-Z0-9!#$%&'*+.^_`|~-]+|<[\x21-\x7E]*>)$/)) {                                 // 1975
                    dav.push(token);                                                                                   // 1976
                  }                                                                                                    // 1977
                });                                                                                                    // 1978
              }                                                                                                        // 1979
                                                                                                                       //
              if (dav.length > 0) {                                                                                    // 1981
                response.setHeader("DAV", dav.join(", "));                                                             // 1982
                response.setHeader("Access-Control-Expose-Headers", "DAV");                                            // 1983
              }                                                                                                        // 1984
                                                                                                                       //
              response.end(); // Return no response; we already handled everything.                                    // 1986
            }, function (err) {                                                                                        // 1988
              if (err.kjType !== "unimplemented") throw err;                                                           // 1989
              response.end(); // Return no response; we already handled everything.                                    // 1990
            });                                                                                                        // 1992
          } else {                                                                                                     // 1993
            throw new Error("Sandstorm only supports the following methods: GET, POST, PUT, PATCH, DELETE, HEAD, PROPFIND, PROPPATCH, MKCOL, COPY, MOVE, LOCK, UNLOCK, ACL, REPORT, and OPTIONS.");
          }                                                                                                            // 1995
        });                                                                                                            // 1996
      }).then(function (rpcResponse) {                                                                                 // 1997
        if (rpcResponse !== undefined) {                                                                               // 1998
          // Will be undefined for OPTIONS request.                                                                    // 1998
          return _this9.translateResponse(rpcResponse, response, request);                                             // 1999
        }                                                                                                              // 2000
      }).catch(function (error) {                                                                                      // 2001
        return _this9.maybeRetryAfterError(error, retryCount).then(function () {                                       // 2002
          return _this9.handleRequest(request, data, response, retryCount + 1);                                        // 2003
        });                                                                                                            // 2004
      });                                                                                                              // 2005
    }                                                                                                                  // 2006
                                                                                                                       //
    return handleRequest;                                                                                              //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.handleRequestStreaming = function () {                                                               //
    function handleRequestStreaming(request, response, contentLength, retryCount) {                                    //
      var _this10 = this;                                                                                              // 2008
                                                                                                                       //
      return this.makeContext(request, response, function (context) {                                                  // 2009
        var path = request.url.slice(1); // remove leading '/'                                                         // 2010
                                                                                                                       //
        var session = _this10.getSession(request);                                                                     // 2011
                                                                                                                       //
        var mimeType = request.headers["content-type"] || "application/octet-stream";                                  // 2013
        var encoding = request.headers["content-encoding"];                                                            // 2014
        var requestStreamPromise = void 0;                                                                             // 2016
                                                                                                                       //
        if (request.method === "POST") {                                                                               // 2017
          requestStreamPromise = session.postStreaming(path, mimeType, context, encoding);                             // 2018
        } else if (request.method === "PUT") {                                                                         // 2019
          requestStreamPromise = session.putStreaming(path, mimeType, context, encoding);                              // 2020
        } else {                                                                                                       // 2021
          throw new Error("Sandstorm only supports streaming POST and PUT requests.");                                 // 2022
        } // TODO(perf): We ought to be pipelining the body, but we can't currently, because we have to                // 2023
        //   handle the case where the app doesn't actually support streaming. We could pipeline while                 // 2026
        //   also buffering the data on the side in case we need it again later, but that's kind of                    // 2027
        //   complicated. We should fix the whole protocol to make streaming the standard.                             // 2028
                                                                                                                       //
                                                                                                                       //
        return requestStreamPromise.then(function (requestStreamResult) {                                              // 2029
          var requestStream = requestStreamResult.stream; // Initialized when getResponse() returns, if the response is streaming.
                                                                                                                       //
          var downloadStreamHandle = void 0; // Initialized if an upload-stream method throws.                         // 2033
                                                                                                                       //
          var uploadStreamError = void 0; // We call `getResponse()` immediately so that the app can start streaming data down even
          // while data is still being streamed up. This theoretically allows apps to perform                          // 2039
          // bidirectional streaming, though probably very few actually do that.                                       // 2040
          //                                                                                                           // 2041
          // Note that we need to be able to cancel `responsePromise` below, so it's important that                    // 2042
          // it is the raw Cap'n Proto promise. Hence `translateResponsePromise` is a separate                         // 2043
          // variable.                                                                                                 // 2044
                                                                                                                       //
          var responsePromise = requestStream.getResponse();                                                           // 2045
                                                                                                                       //
          var reportUploadStreamError = function (err) {                                                               // 2047
            // Called when an upload-stream method throws.                                                             // 2048
            if (!uploadStreamError) {                                                                                  // 2050
              uploadStreamError = err; // If we're still waiting on any response stuff, cancel it.                     // 2051
                                                                                                                       //
              responsePromise.cancel();                                                                                // 2054
              requestStream.close();                                                                                   // 2055
                                                                                                                       //
              if (downloadStreamHandle) {                                                                              // 2056
                downloadStreamHandle.close();                                                                          // 2057
              }                                                                                                        // 2058
            }                                                                                                          // 2059
          }; // If we have a Content-Length, pass it along to the app by calling `expectSize()`.                       // 2060
                                                                                                                       //
                                                                                                                       //
          if (contentLength !== undefined) {                                                                           // 2063
            requestStream.expectSize(contentLength).catch(function (err) {                                             // 2064
              // expectSize() is allowed to be unimplemented.                                                          // 2065
              if (err.kjType !== "unimplemented") {                                                                    // 2066
                reportUploadStreamError(err);                                                                          // 2067
              }                                                                                                        // 2068
            });                                                                                                        // 2069
          } // Pipe the input stream to the app.                                                                       // 2070
                                                                                                                       //
                                                                                                                       //
          var writesInFlight = 0;                                                                                      // 2073
          var readingPaused = false;                                                                                   // 2074
          request.on("data", function (buf) {                                                                          // 2075
            // TODO(someday): Coalesce small writes.                                                                   // 2076
            if (buf.length > 4194304) {                                                                                // 2078
              // Whoa, buffer is more than 4MB. We should be splitting it. But does this actually                      // 2079
              // happen? Let's try to detect this problem before we try to do anything about it.                       // 2080
              console.log("whoa, huge upload buffer:", buf.length);                                                    // 2081
              reportUploadStreamError(new Error("upload buffer too big (bug in Sandstorm)"));                          // 2082
              destructor();                                                                                            // 2083
              return;                                                                                                  // 2084
            }                                                                                                          // 2085
                                                                                                                       //
            if (!uploadStreamError) {                                                                                  // 2087
              requestStream.write(buf).then(function () {                                                              // 2088
                writesInFlight--;                                                                                      // 2089
                                                                                                                       //
                if (readingPaused) {                                                                                   // 2090
                  request.resume();                                                                                    // 2091
                  readingPaused = false;                                                                               // 2092
                }                                                                                                      // 2093
              }).catch(reportUploadStreamError);                                                                       // 2094
                                                                                                                       //
              if (writesInFlight++ > 16) {                                                                             // 2096
                request.pause();                                                                                       // 2097
                readingPaused = true;                                                                                  // 2098
              }                                                                                                        // 2099
            }                                                                                                          // 2100
          });                                                                                                          // 2101
          var idx = _this10.streamingRequestCounter;                                                                   // 2103
          _this10.streamingRequestCounter += 1;                                                                        // 2104
          _this10.streamingRequests[idx] = request;                                                                    // 2105
                                                                                                                       //
          destructor = function () {                                                                                   // 2106
            delete _this10.streamingRequests[idx];                                                                     // 2106
          };                                                                                                           // 2106
                                                                                                                       //
          request.on("end", function () {                                                                              // 2108
            if (!uploadStreamError) requestStream.done().catch(reportUploadStreamError); // We're all done making calls to requestStream.
                                                                                                                       //
            requestStream.close();                                                                                     // 2112
            destructor();                                                                                              // 2113
          });                                                                                                          // 2114
          request.on("close", function () {                                                                            // 2116
            reportUploadStreamError(new Error("HTTP connection unexpectedly closed during request."));                 // 2117
            destructor();                                                                                              // 2118
          });                                                                                                          // 2119
          request.on("error", function (err) {                                                                         // 2121
            reportUploadStreamError(err);                                                                              // 2122
            destructor();                                                                                              // 2123
          });                                                                                                          // 2124
          return responsePromise.then(function (rpcResponse) {                                                         // 2126
            // Stop here if the upload stream has already failed.                                                      // 2127
            if (uploadStreamError) throw uploadStreamError;                                                            // 2128
            downloadStreamHandle = _this10.translateResponse(rpcResponse, response, request).streamHandle;             // 2129
          });                                                                                                          // 2130
        }, function (err) {                                                                                            // 2131
          if (err.kjType === "failed" && err.message.indexOf("not implemented") !== -1) {                              // 2132
            // Hack to work around old apps using an old version of Cap'n Proto, before the                            // 2133
            // 'unimplemented' exception type was introduced. :(                                                       // 2134
            // TODO(cleanup): When we transition to API version 2, we can move this into the                           // 2135
            //   compatibility layer.                                                                                  // 2136
            err.kjType = "unimplemented";                                                                              // 2137
          }                                                                                                            // 2138
                                                                                                                       //
          if (shouldRestartGrain(err, 0)) {                                                                            // 2140
            // This is the kind of error that indicates we should retry. Note that we passed 0 for                     // 2141
            // the retry count above because we were just checking if this is a retriable error (vs.                   // 2142
            // possibly a method-not-implemented error); maybeRetryAfterError() will check again with                  // 2143
            // the proper retry count.                                                                                 // 2144
            return _this10.maybeRetryAfterError(err, retryCount).then(function () {                                    // 2145
              return _this10.handleRequestStreaming(request, response, contentLength, retryCount + 1);                 // 2146
            });                                                                                                        // 2147
          } else if (err.kjType === "unimplemented") {                                                                 // 2148
            // Streaming is not implemented. Fall back to non-streaming version.                                       // 2149
            return readAll(request).then(function (data) {                                                             // 2150
              return _this10.handleRequest(request, data, response, 0);                                                // 2151
            });                                                                                                        // 2152
          } else {                                                                                                     // 2153
            throw err;                                                                                                 // 2154
          }                                                                                                            // 2155
        });                                                                                                            // 2156
      });                                                                                                              // 2157
    }                                                                                                                  // 2158
                                                                                                                       //
    return handleRequestStreaming;                                                                                     //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.handleWebSocket = function () {                                                                      //
    function handleWebSocket(request, socket, head, retryCount) {                                                      //
      var _this11 = this;                                                                                              // 2160
                                                                                                                       //
      return Promise.resolve(undefined).then(function () {                                                             // 2161
        return _this11.makeContext(request, undefined, function (context) {                                            // 2162
          var path = request.url.slice(1); // remove leading '/'                                                       // 2163
                                                                                                                       //
          var session = _this11.getSession(request);                                                                   // 2164
                                                                                                                       //
          if (!("sec-websocket-key" in request.headers)) {                                                             // 2166
            throw new Error("Missing Sec-WebSocket-Accept header.");                                                   // 2167
          }                                                                                                            // 2168
                                                                                                                       //
          var magic = request.headers["sec-websocket-key"] + "258EAFA5-E914-47DA-95CA-C5AB0DC85B11";                   // 2170
          var acceptKey = Crypto.createHash("sha1").update(magic).digest("base64");                                    // 2171
          var protocols = [];                                                                                          // 2173
                                                                                                                       //
          if ("sec-websocket-protocol" in request.headers) {                                                           // 2174
            protocols = request.headers["sec-websocket-protocol"].split(",").map(function (s) {                        // 2175
              return s.trim();                                                                                         // 2176
            });                                                                                                        // 2176
          }                                                                                                            // 2177
                                                                                                                       //
          var receiver = new WebSocketReceiver(socket);                                                                // 2179
          var promise = session.openWebSocket(path, context, protocols, receiver);                                     // 2181
                                                                                                                       //
          if (head.length > 0) {                                                                                       // 2183
            promise.serverStream.sendBytes(head);                                                                      // 2184
          }                                                                                                            // 2185
                                                                                                                       //
          var socketIdx = _this11.websocketCounter;                                                                    // 2187
          _this11.websockets[socketIdx] = socket;                                                                      // 2188
          _this11.websocketCounter += 1;                                                                               // 2189
          pumpWebSocket(socket, promise.serverStream, function () {                                                    // 2190
            delete _this11.websockets[socketIdx];                                                                      // 2190
          });                                                                                                          // 2190
          return promise.then(function (response) {                                                                    // 2192
            var headers = ["HTTP/1.1 101 Switching Protocols", "Upgrade: websocket", "Connection: Upgrade", "Sec-WebSocket-Accept: " + acceptKey];
                                                                                                                       //
            if (response.protocol && response.protocol.length > 0) {                                                   // 2199
              headers.push("Sec-WebSocket-Protocol: " + response.protocol.join(", "));                                 // 2200
            }                                                                                                          // 2201
                                                                                                                       //
            headers.push("");                                                                                          // 2203
            headers.push("");                                                                                          // 2204
            socket.write(headers.join("\r\n"));                                                                        // 2206
            receiver.go(); // Note:  At this point errors are out of our hands.                                        // 2207
          });                                                                                                          // 2210
        });                                                                                                            // 2211
      }).catch(function (error) {                                                                                      // 2212
        return _this11.maybeRetryAfterError(error, retryCount).then(function () {                                      // 2213
          return _this11.handleWebSocket(request, socket, head, retryCount + 1);                                       // 2214
        });                                                                                                            // 2215
      });                                                                                                              // 2216
    }                                                                                                                  // 2217
                                                                                                                       //
    return handleWebSocket;                                                                                            //
  }();                                                                                                                 //
                                                                                                                       //
  Proxy.prototype.setHasLoaded = function () {                                                                         //
    function setHasLoaded() {                                                                                          //
      if (!this.hasLoaded) {                                                                                           // 2220
        this.hasLoaded = true;                                                                                         // 2221
        var sessionId = this.sessionId;                                                                                // 2222
        inMeteor(function () {                                                                                         // 2223
          Sessions.update({                                                                                            // 2224
            _id: sessionId                                                                                             // 2224
          }, {                                                                                                         // 2224
            $set: {                                                                                                    // 2224
              hasLoaded: true                                                                                          // 2224
            }                                                                                                          // 2224
          });                                                                                                          // 2224
        });                                                                                                            // 2225
      }                                                                                                                // 2226
    }                                                                                                                  // 2227
                                                                                                                       //
    return setHasLoaded;                                                                                               //
  }();                                                                                                                 //
                                                                                                                       //
  return Proxy;                                                                                                        //
}();                                                                                                                   //
                                                                                                                       //
var isRfc1918OrLocal = function (address) {                                                                            // 2230
  if (Net.isIPv4(address)) {                                                                                           // 2231
    var quad = address.split(".").map(function (x) {                                                                   // 2232
      return parseInt(x, 10);                                                                                          // 2232
    });                                                                                                                // 2232
    return quad[0] === 127 || quad[0] === 10 || quad[0] === 192 && quad[1] === 168 || quad[0] === 172 && quad[1] >= 16 && quad[1] < 32;
  } else if (Net.isIPv6(address)) {                                                                                    // 2236
    // IPv6 specifies ::1 as localhost and fd:: as reserved for private networks                                       // 2237
    return address === "::1" || address.lastIndexOf("fd", 0) === 0;                                                    // 2238
  } else {                                                                                                             // 2239
    // Ignore things that are neither IPv4 nor IPv6                                                                    // 2240
    return false;                                                                                                      // 2241
  }                                                                                                                    // 2242
};                                                                                                                     // 2243
                                                                                                                       //
var quadToIntString = function (quad) {                                                                                // 2245
  var num = Bignum(quad[0]).shiftLeft(16).add(quad[1]).shiftLeft(16).add(quad[2]).shiftLeft(16).add(quad[3]);          // 2246
  return num.toString();                                                                                               // 2250
}; // -----------------------------------------------------------------------------                                    // 2251
// Session cookie management                                                                                           // 2254
                                                                                                                       //
                                                                                                                       //
var parseCookies = function (request) {                                                                                // 2256
  // jscs:disable requireDotNotation                                                                                   // 2257
  var header = request.headers["cookie"];                                                                              // 2258
  var result = {                                                                                                       // 2260
    cookies: []                                                                                                        // 2260
  };                                                                                                                   // 2260
                                                                                                                       //
  if (header) {                                                                                                        // 2261
    var reqCookies = header.split(";");                                                                                // 2262
                                                                                                                       //
    for (var i in meteorBabelHelpers.sanitizeForInObject(reqCookies)) {                                                // 2263
      var reqCookie = reqCookies[i];                                                                                   // 2264
      var equalsPos = reqCookie.indexOf("=");                                                                          // 2265
      var cookie = void 0;                                                                                             // 2266
                                                                                                                       //
      if (equalsPos === -1) {                                                                                          // 2267
        cookie = {                                                                                                     // 2268
          key: reqCookie.trim(),                                                                                       // 2268
          value: ""                                                                                                    // 2268
        };                                                                                                             // 2268
      } else {                                                                                                         // 2269
        cookie = {                                                                                                     // 2270
          key: reqCookie.slice(0, equalsPos).trim(),                                                                   // 2270
          value: reqCookie.slice(equalsPos + 1)                                                                        // 2270
        };                                                                                                             // 2270
      }                                                                                                                // 2271
                                                                                                                       //
      if (cookie.key === "sandstorm-sid") {                                                                            // 2273
        if (result.sessionId) {                                                                                        // 2274
          throw new Error("Multiple sandstorm session IDs?");                                                          // 2275
        }                                                                                                              // 2276
                                                                                                                       //
        result.sessionId = cookie.value;                                                                               // 2278
      } else {                                                                                                         // 2279
        result.cookies.push(cookie);                                                                                   // 2280
      }                                                                                                                // 2281
    }                                                                                                                  // 2282
  }                                                                                                                    // 2283
                                                                                                                       //
  return result;                                                                                                       // 2285
};                                                                                                                     // 2286
                                                                                                                       //
var parsePreconditionHeader = function (request) {                                                                     // 2288
  if (request.headers["if-match"]) {                                                                                   // 2289
    if (request.headers["if-match"].trim() === "*") {                                                                  // 2290
      return {                                                                                                         // 2291
        exists: null                                                                                                   // 2291
      };                                                                                                               // 2291
    }                                                                                                                  // 2292
                                                                                                                       //
    var matches = parseETagList(request.headers["if-match"]);                                                          // 2294
                                                                                                                       //
    if (matches.length > 0) {                                                                                          // 2295
      return {                                                                                                         // 2296
        matchesOneOf: matches                                                                                          // 2296
      };                                                                                                               // 2296
    }                                                                                                                  // 2297
  }                                                                                                                    // 2298
                                                                                                                       //
  if (request.headers["if-none-match"]) {                                                                              // 2300
    if (request.headers["if-none-match"].trim() === "*") {                                                             // 2301
      return {                                                                                                         // 2302
        doesntExist: null                                                                                              // 2302
      };                                                                                                               // 2302
    }                                                                                                                  // 2303
                                                                                                                       //
    var noneMatches = parseETagList(request.headers["if-none-match"]);                                                 // 2305
                                                                                                                       //
    if (noneMatches.length > 0) {                                                                                      // 2306
      return {                                                                                                         // 2307
        matchesNoneOf: noneMatches                                                                                     // 2307
      };                                                                                                               // 2307
    }                                                                                                                  // 2308
  }                                                                                                                    // 2309
                                                                                                                       //
  return {                                                                                                             // 2311
    none: null                                                                                                         // 2311
  };                                                                                                                   // 2311
};                                                                                                                     // 2312
                                                                                                                       //
var parseETagList = function (input) {                                                                                 // 2314
  // An ETag is a quoted, \-escaped string, possibly prefixed with W/ (outside the quotes) to                          // 2315
  // indicate that it is weak. We are parsing a list of comma-delimited etags.                                         // 2316
  input = input.trim();                                                                                                // 2318
  var results = [];                                                                                                    // 2319
                                                                                                                       //
  while (input.length > 0) {                                                                                           // 2321
    var match = input.match(/^\s*(W\/)?"(([^"\\]|\\.)*)"\s*($|,)/);                                                    // 2322
    if (!match) throw new Meteor.Error(400, "invalid etag");                                                           // 2323
    input = input.slice(match[0].length).trim();                                                                       // 2325
    results.push({                                                                                                     // 2326
      weak: !!match[1],                                                                                                // 2326
      value: match[2].replace(/\\(.)/g, "$1")                                                                          // 2326
    });                                                                                                                // 2326
  }                                                                                                                    // 2327
                                                                                                                       //
  return results;                                                                                                      // 2329
};                                                                                                                     // 2330
                                                                                                                       //
var composeETag = function (tag) {                                                                                     // 2332
  var result = '"' + (tag.value || "").replace(/([\\"])/g, "\\$1") + '"';                                              // 2333
  if (tag.weak) result = "W/" + result;                                                                                // 2334
  return result;                                                                                                       // 2335
};                                                                                                                     // 2336
                                                                                                                       //
var parseAcceptHeaderString = function (header, field) {                                                               // 2338
  var result = [];                                                                                                     // 2339
                                                                                                                       //
  if (header) {                                                                                                        // 2340
    var acceptList = header.split(",");                                                                                // 2341
                                                                                                                       //
    for (var i in meteorBabelHelpers.sanitizeForInObject(acceptList)) {                                                // 2342
      var acceptStr = acceptList[i];                                                                                   // 2343
      var tokensList = acceptStr.split(";");                                                                           // 2344
      var temp = {};                                                                                                   // 2346
      temp[field] = tokensList[0].trim();                                                                              // 2347
      var tokensListRest = tokensList.slice(1);                                                                        // 2349
                                                                                                                       //
      for (var j in meteorBabelHelpers.sanitizeForInObject(tokensListRest)) {                                          // 2350
        var token = tokensListRest[j];                                                                                 // 2351
        var equalsPos = token.indexOf("=");                                                                            // 2352
                                                                                                                       //
        if (equalsPos) {                                                                                               // 2353
          var key = token.slice(0, equalsPos).trim();                                                                  // 2354
          var value = token.slice(equalsPos + 1).trim();                                                               // 2355
                                                                                                                       //
          if (key === "q") {                                                                                           // 2357
            temp.qValue = +value;                                                                                      // 2358
          }                                                                                                            // 2359
        }                                                                                                              // 2360
      }                                                                                                                // 2361
                                                                                                                       //
      result.push(temp);                                                                                               // 2363
    }                                                                                                                  // 2364
  }                                                                                                                    // 2365
                                                                                                                       //
  return result;                                                                                                       // 2367
};                                                                                                                     // 2368
                                                                                                                       //
var parseAcceptHeader = function (request) {                                                                           // 2370
  // jscs:disable requireDotNotation                                                                                   // 2371
  var header = request.headers["accept"];                                                                              // 2372
  return parseAcceptHeaderString(header, "mimeType");                                                                  // 2373
};                                                                                                                     // 2374
                                                                                                                       //
var parseAcceptEncodingHeader = function (request) {                                                                   // 2376
  // jscs:disable requireDotNotation                                                                                   // 2377
  var header = request.headers["accept-encoding"];                                                                     // 2378
  return parseAcceptHeaderString(header, "contentCoding");                                                             // 2379
}; // -----------------------------------------------------------------------------                                    // 2380
// Regular HTTP request handling                                                                                       // 2383
                                                                                                                       //
                                                                                                                       //
var readAll = function (stream) {                                                                                      // 2385
  return new Promise(function (resolve, reject) {                                                                      // 2386
    var buffers = [];                                                                                                  // 2387
    stream.on("data", function (buf) {                                                                                 // 2388
      buffers.push(buf);                                                                                               // 2389
    });                                                                                                                // 2390
    stream.on("end", function () {                                                                                     // 2392
      resolve(Buffer.concat(buffers));                                                                                 // 2393
    });                                                                                                                // 2394
    stream.on("error", reject);                                                                                        // 2396
  });                                                                                                                  // 2397
};                                                                                                                     // 2398
                                                                                                                       //
var makeSetCookieHeader = function (cookie) {                                                                          // 2400
  var result = [cookie.name, "=", cookie.value];                                                                       // 2401
                                                                                                                       //
  if ("absolute" in cookie.expires) {                                                                                  // 2403
    result.push("; Expires=");                                                                                         // 2404
    result.push(new Date(cookie.expires.absolute * 1000).toUTCString());                                               // 2405
  } else if ("relative" in cookie.expires) {                                                                           // 2406
    result.push("; Max-Age=" + cookie.expires.relative);                                                               // 2407
  }                                                                                                                    // 2408
                                                                                                                       //
  if (cookie.path) {                                                                                                   // 2410
    result.push("; Path=" + cookie.path);                                                                              // 2411
  }                                                                                                                    // 2412
                                                                                                                       //
  if (cookie.httpOnly) {                                                                                               // 2414
    result.push("; HttpOnly");                                                                                         // 2415
  }                                                                                                                    // 2416
                                                                                                                       //
  return result.join("");                                                                                              // 2418
}; // TODO(cleanup):  Auto-generate based on annotations in web-session.capnp.                                         // 2419
                                                                                                                       //
                                                                                                                       //
var successCodes = {                                                                                                   // 2422
  ok: {                                                                                                                // 2423
    id: 200,                                                                                                           // 2423
    title: "OK"                                                                                                        // 2423
  },                                                                                                                   // 2423
  created: {                                                                                                           // 2424
    id: 201,                                                                                                           // 2424
    title: "Created"                                                                                                   // 2424
  },                                                                                                                   // 2424
  accepted: {                                                                                                          // 2425
    id: 202,                                                                                                           // 2425
    title: "Accepted"                                                                                                  // 2425
  },                                                                                                                   // 2425
  multiStatus: {                                                                                                       // 2426
    id: 207,                                                                                                           // 2426
    title: "Multi-Status"                                                                                              // 2426
  }                                                                                                                    // 2426
};                                                                                                                     // 2422
var noContentSuccessCodes = [// Indexed by shouldResetForm * 1                                                         // 2428
{                                                                                                                      // 2430
  id: 204,                                                                                                             // 2430
  title: "No Content"                                                                                                  // 2430
}, {                                                                                                                   // 2430
  id: 205,                                                                                                             // 2431
  title: "Reset Content"                                                                                               // 2431
}];                                                                                                                    // 2431
var redirectCodes = [// Indexed by switchToGet * 2 + isPermanent                                                       // 2433
{                                                                                                                      // 2435
  id: 307,                                                                                                             // 2435
  title: "Temporary Redirect"                                                                                          // 2435
}, {                                                                                                                   // 2435
  id: 308,                                                                                                             // 2436
  title: "Permanent Redirect"                                                                                          // 2436
}, {                                                                                                                   // 2436
  id: 303,                                                                                                             // 2437
  title: "See Other"                                                                                                   // 2437
}, {                                                                                                                   // 2437
  id: 301,                                                                                                             // 2438
  title: "Moved Permanently"                                                                                           // 2438
}];                                                                                                                    // 2438
var errorCodes = {                                                                                                     // 2440
  badRequest: {                                                                                                        // 2441
    id: 400,                                                                                                           // 2441
    title: "Bad Request"                                                                                               // 2441
  },                                                                                                                   // 2441
  forbidden: {                                                                                                         // 2442
    id: 403,                                                                                                           // 2442
    title: "Forbidden"                                                                                                 // 2442
  },                                                                                                                   // 2442
  notFound: {                                                                                                          // 2443
    id: 404,                                                                                                           // 2443
    title: "Not Found"                                                                                                 // 2443
  },                                                                                                                   // 2443
  methodNotAllowed: {                                                                                                  // 2444
    id: 405,                                                                                                           // 2444
    title: "Method Not Allowed"                                                                                        // 2444
  },                                                                                                                   // 2444
  notAcceptable: {                                                                                                     // 2445
    id: 406,                                                                                                           // 2445
    title: "Not Acceptable"                                                                                            // 2445
  },                                                                                                                   // 2445
  conflict: {                                                                                                          // 2446
    id: 409,                                                                                                           // 2446
    title: "Conflict"                                                                                                  // 2446
  },                                                                                                                   // 2446
  gone: {                                                                                                              // 2447
    id: 410,                                                                                                           // 2447
    title: "Gone"                                                                                                      // 2447
  },                                                                                                                   // 2447
  requestEntityTooLarge: {                                                                                             // 2448
    id: 413,                                                                                                           // 2448
    title: "Request Entity Too Large"                                                                                  // 2448
  },                                                                                                                   // 2448
  requestUriTooLong: {                                                                                                 // 2449
    id: 414,                                                                                                           // 2449
    title: "Request-URI Too Long"                                                                                      // 2449
  },                                                                                                                   // 2449
  unsupportedMediaType: {                                                                                              // 2450
    id: 415,                                                                                                           // 2450
    title: "Unsupported Media Type"                                                                                    // 2450
  },                                                                                                                   // 2450
  imATeapot: {                                                                                                         // 2451
    id: 418,                                                                                                           // 2451
    title: "I'm a teapot"                                                                                              // 2451
  },                                                                                                                   // 2451
  unprocessableEntity: {                                                                                               // 2452
    id: 422,                                                                                                           // 2452
    title: "Unprocessable Entity"                                                                                      // 2452
  }                                                                                                                    // 2452
};                                                                                                                     // 2440
                                                                                                                       //
ResponseStream = function () {                                                                                         // 2455
  // Note: This class is used in pre-meteor.js as well as in this file.                                                // 2456
  function ResponseStream(response, httpCode, httpStatus, upstreamHandle, idempotentCleanup) {                         // 2458
    var _this12 = this;                                                                                                // 2458
                                                                                                                       //
    (0, _classCallCheck3.default)(this, ResponseStream);                                                               // 2458
    // This is stupidly complicated, because:                                                                          // 2459
    // - New versions of sandstorm-http-bridge (and other well-behaved apps) wait on write()                           // 2460
    //   completion for flow control, so we want write() to complete only when there is space                          // 2461
    //   available for more writes.                                                                                    // 2462
    // - Old versions of sandstorm-http-bridge would make a zillion simultaneous calls to write()                      // 2463
    //   with all of the data they wanted to send without waiting for previous calls to complete.                      // 2464
    //   While in-flight, these writes can consume a lot of RAM at multiple points in the system.                      // 2465
    // - Old versions of sandstorm-http-bridge would print any thrown error to stderr and then                         // 2466
    //   keep going. So if canceling a download early on causes all future write()s to fail, this                      // 2467
    //   can lead to very excessive log spam.                                                                          // 2468
    // - If a write() fails to complete (it returns a promise that never resolves), then later                         // 2469
    //   upon garbage collection the RPC will actually throw an exception about a PromiseFulfiller                     // 2470
    //   never having been fulfilled. Hence, if we simply stop responding to write()s after some                       // 2471
    //   point, we again get log spam.                                                                                 // 2472
    // - If the connection closes during a write, Node never calls the completion callback at all.                     // 2473
    //   We have to listen for the "close" event.                                                                      // 2474
    this.response = response;                                                                                          // 2476
    this.httpCode = httpCode;                                                                                          // 2477
    this.httpStatus = httpStatus;                                                                                      // 2478
    this.upstreamHandle = upstreamHandle;                                                                              // 2479
    this.idempotentCleanup = idempotentCleanup;                                                                        // 2480
    this.started = false;                                                                                              // 2482
    this.ended = false; // Invariant: if `ended` is true, then `started` is also true.                                 // 2483
                                                                                                                       //
    this.waiting = [];                                                                                                 // 2484
    response.on("drain", function () {                                                                                 // 2486
      _this12.waiting.forEach(function (f) {                                                                           // 2487
        return f();                                                                                                    // 2487
      });                                                                                                              // 2487
                                                                                                                       //
      _this12.waiting = [];                                                                                            // 2488
    });                                                                                                                // 2489
    response.on("close", function () {                                                                                 // 2491
      _this12.abort();                                                                                                 // 2491
    });                                                                                                                // 2491
  }                                                                                                                    // 2492
                                                                                                                       //
  ResponseStream.prototype.abort = function () {                                                                       // 2455
    function abort() {                                                                                                 // 2455
      this.aborted = true; // Resolve all outstanding writes. This is OK even if they didn't actually go through because
      // an incomplete download is the client's problem, not the server's. The important thing is                      // 2498
      // to cancel the stream if it is still being generated; to that end, the next new write() call                   // 2499
      // after this point will throw.                                                                                  // 2500
                                                                                                                       //
      this.waiting.forEach(function (f) {                                                                              // 2501
        return f();                                                                                                    // 2501
      });                                                                                                              // 2501
      this.waiting = [];                                                                                               // 2502
      this.close();                                                                                                    // 2504
    }                                                                                                                  // 2505
                                                                                                                       //
    return abort;                                                                                                      // 2455
  }();                                                                                                                 // 2455
                                                                                                                       //
  ResponseStream.prototype.expectSize = function () {                                                                  // 2455
    function expectSize(size) {                                                                                        // 2455
      if (!this.started) {                                                                                             // 2508
        this.started = true;                                                                                           // 2509
        this.response.writeHead(this.httpCode, this.httpStatus, {                                                      // 2510
          "Content-Length": size                                                                                       // 2510
        });                                                                                                            // 2510
      }                                                                                                                // 2511
    }                                                                                                                  // 2512
                                                                                                                       //
    return expectSize;                                                                                                 // 2455
  }();                                                                                                                 // 2455
                                                                                                                       //
  ResponseStream.prototype.write = function () {                                                                       // 2455
    function write(data) {                                                                                             // 2455
      var _this13 = this;                                                                                              // 2514
                                                                                                                       //
      if (!this.started) {                                                                                             // 2515
        this.started = true;                                                                                           // 2516
        this.response.writeHead(this.httpCode, this.httpStatus);                                                       // 2517
      }                                                                                                                // 2518
                                                                                                                       //
      if (this.aborted) {                                                                                              // 2520
        // Connection has been aborted.                                                                                // 2521
        if (this.alreadyThrew) {                                                                                       // 2522
          // We already threw an error from a previous write(). Do not throw more errors, because                      // 2523
          // older versions of sandstorm-http-bridge do not stop on the first error and will print                     // 2524
          // every error to the log, wasting space. Newer versions stop on the first failure.                          // 2525
          return;                                                                                                      // 2526
        } else {                                                                                                       // 2527
          this.alreadyThrew = true;                                                                                    // 2528
          throw new Error("client disconnected");                                                                      // 2529
        }                                                                                                              // 2530
      } else {                                                                                                         // 2531
        if (this.response.write(data)) {                                                                               // 2532
          // All written.                                                                                              // 2533
          return;                                                                                                      // 2534
        } else {                                                                                                       // 2535
          // Write buffer is full. Don't complete until it has more space.                                             // 2536
          return new Promise(function (resolve, reject) {                                                              // 2537
            _this13.waiting.push(resolve);                                                                             // 2538
          });                                                                                                          // 2539
        }                                                                                                              // 2540
      }                                                                                                                // 2541
    }                                                                                                                  // 2542
                                                                                                                       //
    return write;                                                                                                      // 2455
  }();                                                                                                                 // 2455
                                                                                                                       //
  ResponseStream.prototype.done = function () {                                                                        // 2455
    function done() {                                                                                                  // 2455
      if (!this.started) {                                                                                             // 2545
        this.started = true;                                                                                           // 2546
        this.response.writeHead(this.httpCode, this.httpStatus, {                                                      // 2547
          "Content-Length": 0                                                                                          // 2547
        });                                                                                                            // 2547
      }                                                                                                                // 2548
                                                                                                                       //
      if (!this.ended) {                                                                                               // 2550
        this.ended = true;                                                                                             // 2551
        this.response.end();                                                                                           // 2552
      }                                                                                                                // 2553
                                                                                                                       //
      this.close();                                                                                                    // 2555
    }                                                                                                                  // 2556
                                                                                                                       //
    return done;                                                                                                       // 2455
  }();                                                                                                                 // 2455
                                                                                                                       //
  ResponseStream.prototype.close = function () {                                                                       // 2455
    function close() {                                                                                                 // 2455
      if (this.upstreamHandle) {                                                                                       // 2559
        this.upstreamHandle.close();                                                                                   // 2560
      }                                                                                                                // 2561
                                                                                                                       //
      if (this.idempotentCleanup) {                                                                                    // 2563
        this.idempotentCleanup();                                                                                      // 2564
      } // If `this.done()` has not been called, we should report an error to the client. Unfortunately,               // 2565
      // the only case in which we can actually report the error is when we haven't done any writes                    // 2568
      // yet. So we handle that case and ignore the error otherwise. Note that `!this.started`                         // 2569
      // implies `!this.ended`.                                                                                        // 2570
                                                                                                                       //
                                                                                                                       //
      if (!this.started) {                                                                                             // 2571
        this.started = true;                                                                                           // 2572
        this.response.writeHead(500, "done() never called on response stream", {                                       // 2573
          "Content-Type": "text/plain"                                                                                 // 2574
        });                                                                                                            // 2574
      }                                                                                                                // 2575
                                                                                                                       //
      if (!this.ended) {                                                                                               // 2577
        this.ended = true;                                                                                             // 2578
        this.response.end();                                                                                           // 2579
      }                                                                                                                // 2580
    }                                                                                                                  // 2581
                                                                                                                       //
    return close;                                                                                                      // 2455
  }();                                                                                                                 // 2455
                                                                                                                       //
  ResponseStream.prototype.sendingDirectResponse = function () {                                                       // 2455
    function sendingDirectResponse() {                                                                                 // 2455
      // For internal front-end use: Indicates that the caller is going to fill in the response                        // 2584
      // instead. If a response has already been started, this throws an exception.                                    // 2585
      //                                                                                                               // 2586
      // This is used in particular in pre-meteor.js, when dealing with static web publishing.                         // 2587
      if (this.started) {                                                                                              // 2589
        throw new Error("can't sendDirectResponse() because response already started");                                // 2590
      } else if (this.ended) {                                                                                         // 2591
        throw new Error("can't sendDirectResponse() because response already sent");                                   // 2592
      } else {                                                                                                         // 2593
        this.started = true;                                                                                           // 2594
        this.ended = true;                                                                                             // 2595
      }                                                                                                                // 2596
    }                                                                                                                  // 2597
                                                                                                                       //
    return sendingDirectResponse;                                                                                      // 2455
  }();                                                                                                                 // 2455
                                                                                                                       //
  return ResponseStream;                                                                                               // 2455
}(); // TODO(cleanup): Node 0.12 has a `gunzipSync` but 0.10 (which Meteor still uses) does not.                       // 2455
                                                                                                                       //
                                                                                                                       //
var Zlib = Npm.require("zlib");                                                                                        // 2601
                                                                                                                       //
var gunzipSync = Meteor.wrapAsync(Zlib.gunzip, Zlib); // -----------------------------------------------------------------------------
// WebSocket handling                                                                                                  // 2605
                                                                                                                       //
WebSocketReceiver = function () {                                                                                      // 2607
  function WebSocketReceiver(socket) {                                                                                 // 2608
    (0, _classCallCheck3.default)(this, WebSocketReceiver);                                                            // 2608
    this.socket = socket;                                                                                              // 2609
    this.queue = [];                                                                                                   // 2610
  }                                                                                                                    // 2611
                                                                                                                       //
  WebSocketReceiver.prototype.go = function () {                                                                       // 2607
    function go() {                                                                                                    // 2607
      for (var i in meteorBabelHelpers.sanitizeForInObject(this.queue)) {                                              // 2614
        this.socket.write(this.queue[i]);                                                                              // 2615
      }                                                                                                                // 2616
                                                                                                                       //
      this.queue = null;                                                                                               // 2618
    }                                                                                                                  // 2619
                                                                                                                       //
    return go;                                                                                                         // 2607
  }();                                                                                                                 // 2607
                                                                                                                       //
  WebSocketReceiver.prototype.sendBytes = function () {                                                                // 2607
    function sendBytes(message) {                                                                                      // 2607
      // TODO(someday):  Flow control of some sort?                                                                    // 2622
      if (this.queue === null) {                                                                                       // 2623
        this.socket.write(message);                                                                                    // 2624
      } else {                                                                                                         // 2625
        this.queue.push(message);                                                                                      // 2626
      }                                                                                                                // 2627
    }                                                                                                                  // 2628
                                                                                                                       //
    return sendBytes;                                                                                                  // 2607
  }();                                                                                                                 // 2607
                                                                                                                       //
  WebSocketReceiver.prototype.close = function () {                                                                    // 2607
    function close() {                                                                                                 // 2607
      this.socket.end();                                                                                               // 2631
    }                                                                                                                  // 2632
                                                                                                                       //
    return close;                                                                                                      // 2607
  }();                                                                                                                 // 2607
                                                                                                                       //
  return WebSocketReceiver;                                                                                            // 2607
}();                                                                                                                   // 2607
                                                                                                                       //
function pumpWebSocket(socket, rpcStream, destructor) {                                                                // 2635
  var writesInFlight = 0;                                                                                              // 2636
  var readingPaused = false;                                                                                           // 2637
  socket.on("data", function (chunk) {                                                                                 // 2638
    rpcStream.sendBytes(chunk).then(function () {                                                                      // 2639
      writesInFlight--;                                                                                                // 2640
                                                                                                                       //
      if (readingPaused) {                                                                                             // 2641
        socket.resume();                                                                                               // 2642
        readingPaused = false;                                                                                         // 2643
      }                                                                                                                // 2644
    }).catch(function (err) {                                                                                          // 2645
      if (err.kjType !== "disconnected") {                                                                             // 2646
        console.error("WebSocket sendBytes failed: " + err.stack);                                                     // 2647
      }                                                                                                                // 2648
                                                                                                                       //
      socket.destroy();                                                                                                // 2650
    });                                                                                                                // 2651
                                                                                                                       //
    if (writesInFlight++ > 16) {                                                                                       // 2653
      socket.pause();                                                                                                  // 2654
      readingPaused = true;                                                                                            // 2655
    }                                                                                                                  // 2656
  });                                                                                                                  // 2657
  socket.on("end", function (chunk) {                                                                                  // 2659
    rpcStream.close();                                                                                                 // 2660
  });                                                                                                                  // 2661
  socket.on("close", function () {                                                                                     // 2663
    destructor();                                                                                                      // 2664
  });                                                                                                                  // 2665
}                                                                                                                      // 2666
                                                                                                                       //
; // =======================================================================================                           // 2666
// Debug log access                                                                                                    // 2669
                                                                                                                       //
Meteor.publish("grainLog", function (grainId) {                                                                        // 2671
  check(grainId, String);                                                                                              // 2672
  var id = 0;                                                                                                          // 2673
  var grain = Grains.findOne(grainId);                                                                                 // 2674
                                                                                                                       //
  if (!grain || !this.userId || grain.userId !== this.userId) {                                                        // 2675
    this.added("grainLog", id++, {                                                                                     // 2676
      text: "Only the grain owner can view the debug log."                                                             // 2676
    });                                                                                                                // 2676
    this.ready();                                                                                                      // 2677
    return;                                                                                                            // 2678
  }                                                                                                                    // 2679
                                                                                                                       //
  var connected = false;                                                                                               // 2681
                                                                                                                       //
  var _this = this;                                                                                                    // 2682
                                                                                                                       //
  var receiver = {                                                                                                     // 2684
    write: function (data) {                                                                                           // 2685
      connected = true;                                                                                                // 2686
                                                                                                                       //
      _this.added("grainLog", id++, {                                                                                  // 2687
        text: data.toString("utf8")                                                                                    // 2687
      });                                                                                                              // 2687
    },                                                                                                                 // 2688
    close: function () {                                                                                               // 2690
      if (connected) {                                                                                                 // 2691
        _this.added("grainLog", id++, {                                                                                // 2692
          text: "*** lost connection to grain (probably because it shut down) ***"                                     // 2693
        });                                                                                                            // 2692
      }                                                                                                                // 2695
    }                                                                                                                  // 2696
  };                                                                                                                   // 2684
                                                                                                                       //
  try {                                                                                                                // 2699
    var handle = waitPromise(globalBackend.useGrain(grainId, function (supervisor) {                                   // 2700
      return supervisor.watchLog(8192, receiver);                                                                      // 2701
    })).handle;                                                                                                        // 2702
    connected = true;                                                                                                  // 2703
    this.onStop(function () {                                                                                          // 2704
      handle.close();                                                                                                  // 2705
    });                                                                                                                // 2706
  } catch (err) {                                                                                                      // 2707
    if (!connected) {                                                                                                  // 2708
      this.added("grainLog", id++, {                                                                                   // 2709
        text: "*** couldn't connect to grain (" + err + ") ***"                                                        // 2710
      });                                                                                                              // 2709
    }                                                                                                                  // 2712
  } // Notify ready.                                                                                                   // 2713
                                                                                                                       //
                                                                                                                       //
  this.ready();                                                                                                        // 2716
});                                                                                                                    // 2717
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"sandcats.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/sandcats.js                                                                                                  //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var pki = void 0,                                                                                                      // 1
    asn1 = void 0;                                                                                                     // 1
module.watch(require("node-forge"), {                                                                                  // 1
  pki: function (v) {                                                                                                  // 1
    pki = v;                                                                                                           // 1
  },                                                                                                                   // 1
  asn1: function (v) {                                                                                                 // 1
    asn1 = v;                                                                                                          // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var SANDSTORM_ALTHOME = void 0;                                                                                        // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  SANDSTORM_ALTHOME: function (v) {                                                                                    // 1
    SANDSTORM_ALTHOME = v;                                                                                             // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
Sandcats = {};                                                                                                         // 17
                                                                                                                       //
var querystring = Npm.require("querystring");                                                                          // 20
                                                                                                                       //
var https = Npm.require("https");                                                                                      // 21
                                                                                                                       //
var fs = Npm.require("fs");                                                                                            // 22
                                                                                                                       //
var dgram = Npm.require("dgram");                                                                                      // 23
                                                                                                                       //
var Url = Npm.require("url");                                                                                          // 24
                                                                                                                       //
var SANDCATS_HOSTNAME = Meteor.settings && Meteor.settings.public && Meteor.settings.public.sandcatsHostname;          // 28
var SANDCATS_VARDIR = (SANDSTORM_ALTHOME || "") + "/var/sandcats"; // Figure out what IP address to send Sandcats requests from. For machines with multiple IPs, it
// is important to use the IP to which we're binding. However, some people set BIND_IP to 127.0.0.1                    // 33
// and put sniproxy in front of Sandstorm. In those cases, it won't work to send from BIND_IP;                         // 34
// we'll have to let the system choose.                                                                                // 35
                                                                                                                       //
var BIND_IP = process.env.BIND_IP && process.env.BIND_IP.startsWith("127.") ? null : process.env.BIND_IP;              // 36
var ROOT_URL = Url.parse(process.env.ROOT_URL);                                                                        // 39
var HOSTNAME = ROOT_URL.hostname;                                                                                      // 40
var SANDCATS_NAME = void 0; // Look at `startup` below to see where this is set                                        // 41
                                                                                                                       //
var updateSandcatsIp = function () {                                                                                   // 43
  var responseCallback = function (res) {                                                                              // 44
    if (res.statusCode === 200) {                                                                                      // 45
      console.log("Successfully updated sandcats IP");                                                                 // 46
    } else {                                                                                                           // 47
      console.error("Failed to update sandcats IP:", res.headers);                                                     // 48
    }                                                                                                                  // 49
  };                                                                                                                   // 50
                                                                                                                       //
  var errorCallback = function (err) {                                                                                 // 52
    console.error("Couldn't send update sandcats hostname", err);                                                      // 53
  };                                                                                                                   // 54
                                                                                                                       //
  performSandcatsRequest("/update", SANDCATS_HOSTNAME, {                                                               // 56
    rawHostname: SANDCATS_NAME                                                                                         // 56
  }, errorCallback, responseCallback);                                                                                 // 56
};                                                                                                                     // 58
                                                                                                                       //
var pingUdp = function () {                                                                                            // 60
  var socket = dgram.createSocket("udp4");                                                                             // 61
  var secret = Random.secret(16);                                                                                      // 62
  message = new Buffer(SANDCATS_NAME + " " + secret);                                                                  // 64
  socket.on("message", function (buf) {                                                                                // 65
    if (buf.toString() === secret) {                                                                                   // 66
      updateSandcatsIp();                                                                                              // 67
    } else {                                                                                                           // 68
      console.error("Received unexpected response in UDP sandcats ping:", buf.toString());                             // 69
    }                                                                                                                  // 70
  });                                                                                                                  // 71
  socket.on("error", function (err) {                                                                                  // 73
    throw err;                                                                                                         // 74
  });                                                                                                                  // 75
                                                                                                                       //
  var callback = function () {                                                                                         // 77
    socket.send(message, 0, message.length, 8080, SANDCATS_HOSTNAME, function (err) {                                  // 78
      if (err) {                                                                                                       // 79
        console.error("Couldn't send UDP sandcats ping", err);                                                         // 80
      }                                                                                                                // 81
    });                                                                                                                // 82
    setTimeout(function () {                                                                                           // 84
      socket.close();                                                                                                  // 85
    }, 10 * 1000);                                                                                                     // 86
  };                                                                                                                   // 87
                                                                                                                       //
  if (BIND_IP) {                                                                                                       // 89
    socket.bind({                                                                                                      // 90
      address: BIND_IP                                                                                                 // 90
    }, callback);                                                                                                      // 90
  } else {                                                                                                             // 91
    callback();                                                                                                        // 92
  }                                                                                                                    // 93
};                                                                                                                     // 94
                                                                                                                       //
var performSandcatsRequest = function (path, hostname, postData, errorCallback, responseCallback) {                    // 96
  var options = {                                                                                                      // 97
    hostname: hostname,                                                                                                // 98
    path: path,                                                                                                        // 99
    method: "POST",                                                                                                    // 100
    agent: false,                                                                                                      // 101
    key: fs.readFileSync(SANDCATS_VARDIR + "/id_rsa"),                                                                 // 102
    cert: fs.readFileSync(SANDCATS_VARDIR + "/id_rsa.pub"),                                                            // 103
    headers: {                                                                                                         // 104
      "X-Sand": "cats",                                                                                                // 105
      "Content-Type": "application/x-www-form-urlencoded"                                                              // 106
    }                                                                                                                  // 104
  };                                                                                                                   // 97
                                                                                                                       //
  if (BIND_IP) {                                                                                                       // 110
    options.localAddress = BIND_IP;                                                                                    // 111
  }                                                                                                                    // 112
                                                                                                                       //
  if (postData.certificateSigningRequest) {                                                                            // 114
    console.log("Submitting certificate request for host", postData.rawHostname, "where the request has length", postData.certificateSigningRequest.length);
  }                                                                                                                    // 118
                                                                                                                       //
  var newPostData = querystring.stringify(postData);                                                                   // 120
                                                                                                                       //
  if (SANDCATS_HOSTNAME === "sandcats-dev.sandstorm.io" && !process.env.NODE_TLS_REJECT_UNAUTHORIZED) {                // 121
    console.log("You are using the Sandcats dev server but have full HTTPS " + "certificate checking enabled. Probably you will get an HTTPS " + "error. Proceeding anyway.");
  }                                                                                                                    // 126
                                                                                                                       //
  var req = https.request(options, responseCallback);                                                                  // 128
  req.write(newPostData);                                                                                              // 129
  req.end();                                                                                                           // 130
                                                                                                                       //
  if (errorCallback) {                                                                                                 // 132
    req.on("error", errorCallback);                                                                                    // 133
  }                                                                                                                    // 134
                                                                                                                       //
  return req;                                                                                                          // 136
};                                                                                                                     // 137
                                                                                                                       //
var generateKeyAndCsr = function (commonName) {                                                                        // 139
  check(commonName, String); // Generate key pair. Using Meteor.wrapAsync because forge supports                       // 140
  // a synchronous as well as an asynchronous API, and the synchronous                                                 // 143
  // one blocks for a while.                                                                                           // 144
                                                                                                                       //
  var wrappedGenerateKeyPair = Meteor.wrapAsync(pki.rsa.generateKeyPair); // I could pick an `e`[xponent] value for the resulting RSA key, but
  // I will refrain.                                                                                                   // 148
                                                                                                                       //
  var keys = wrappedGenerateKeyPair({                                                                                  // 149
    bits: 2048                                                                                                         // 149
  }); // Create a certificate request (CSR).                                                                           // 149
                                                                                                                       //
  var csr = pki.createCertificationRequest();                                                                          // 152
  csr.publicKey = keys.publicKey;                                                                                      // 153
  csr.setSubject([{                                                                                                    // 154
    name: "commonName",                                                                                                // 156
    value: commonName,                                                                                                 // 157
    valueTagClass: asn1.Type.UTF8 // We specify UTF8 to encode a UTF8String (rather than the default of PRINTABLESTRING) in the
    // commonName so that GlobalSign does not report a warning, and also because that happens to                       // 160
    // be what openssl(1) does when asked to create a CSR.                                                             // 161
                                                                                                                       //
  }]);                                                                                                                 // 155
  csr.sign(keys.privateKey);                                                                                           // 164
  console.log("generateKeyAndCsr created new key & certificate request for", commonName);                              // 165
  return {                                                                                                             // 166
    privateKeyAsPem: pki.privateKeyToPem(keys.privateKey),                                                             // 167
    csrAsPem: pki.certificationRequestToPem(csr)                                                                       // 168
  };                                                                                                                   // 166
};                                                                                                                     // 170
                                                                                                                       //
Sandcats.storeNewKeyAndCsr = function (hostname, basePath) {                                                           // 172
  // We use the current JS time (like UNIX timestamp but in                                                            // 173
  // milliseconds) as the key number. Note that the keyNumber is                                                       // 174
  // intended to be an opaque identifier; the only important thing is                                                  // 175
  // that it increases numerically over time. We use the current time                                                  // 176
  // just as a simplistic way to pick a filename that probably no one                                                  // 177
  // else has created yet.                                                                                             // 178
  var keyNumber = new Date().getTime();                                                                                // 179
  var keyFilename = basePath + "/" + keyNumber;                                                                        // 180
  var csrFilename = keyFilename + ".csr";                                                                              // 181
  var responseFilename = keyFilename + ".response-json";                                                               // 182
  var withWildcard = "*." + hostname;                                                                                  // 183
  var keyAndCsr = generateKeyAndCsr(withWildcard);                                                                     // 184
  fs.writeFileSync(keyFilename, keyAndCsr.privateKeyAsPem, {                                                           // 185
    mode: 0400                                                                                                         // 186
  });                                                                                                                  // 186
  fs.writeFileSync(csrFilename, keyAndCsr.csrAsPem);                                                                   // 187
  console.log("storeNewKeyAndCsr successfully saved key and certificate request to", keyFilename, "and", csrFilename, "respectively of length", keyAndCsr.privateKeyAsPem.length, "and", keyAndCsr.csrAsPem.length);
  return {                                                                                                             // 192
    csrFilename: csrFilename,                                                                                          // 193
    keyFilename: keyFilename,                                                                                          // 194
    responseFilename: responseFilename                                                                                 // 195
  };                                                                                                                   // 192
};                                                                                                                     // 197
                                                                                                                       //
Sandcats.renewHttpsCertificateIfNeeded = function () {                                                                 // 199
  var renewHttpsCertificate = function () {                                                                            // 200
    var hostname = Url.parse(process.env.ROOT_URL).hostname;                                                           // 201
    var basePath = "/var/sandcats/https/" + hostname;                                                                  // 202
    var filenames = Sandcats.storeNewKeyAndCsr(hostname, basePath);                                                    // 203
                                                                                                                       //
    var errorCallback = function (err) {                                                                               // 205
      console.error("Error while renewing HTTPS certificate (will continue to retry)", err);                           // 206
    };                                                                                                                 // 207
                                                                                                                       //
    var responseCallback = function (res) {                                                                            // 209
      if (res.statusCode == 200) {                                                                                     // 210
        // Save the response, chunk by chunk, then store it on disk                                                    // 211
        // for later use.                                                                                              // 212
        var responseBody = "";                                                                                         // 213
        res.on("data", function (chunk) {                                                                              // 214
          responseBody += chunk;                                                                                       // 215
        });                                                                                                            // 216
        res.on("end", function () {                                                                                    // 218
          // For sanity, make sure it parses as JSON, since we're                                                      // 219
          // going to need it to do that.                                                                              // 220
          try {                                                                                                        // 221
            JSON.parse(responseBody);                                                                                  // 222
          } catch (e) {                                                                                                // 223
            console.error("JSON parse error receiving new HTTPS certificate. Discarding:", responseBody, "due to exception:", e); // Overwrite the responseBody with the empty JSON object
            // and continue with the process of saving it to disk.                                                     // 229
                                                                                                                       //
            responseBody = "{}";                                                                                       // 230
          }                                                                                                            // 231
                                                                                                                       //
          try {                                                                                                        // 233
            fs.writeFileSync(filenames.responseFilename, responseBody, "utf-8");                                       // 234
          } catch (err) {                                                                                              // 235
            return console.error("Failure while saveing new HTTPS certificate to", filenames.responseFilename, "will continue to retry. Exception was:", err);
          } // Tell Sandcats that now is a good time to update its info                                                // 240
          // about which keys are available.                                                                           // 243
                                                                                                                       //
                                                                                                                       //
          if (!global.sandcats) {                                                                                      // 244
            console.error("When getting new certificate, could not find callback to request re-keying! Certs will probably become invalid soon.");
          } else {                                                                                                     // 246
            // Call the sandcats rekeying function.                                                                    // 247
            global.sandcats.rekey(); // That's that.                                                                   // 248
                                                                                                                       //
            console.log("Successfully renewed HTTPS certificate into", filenames.responseFilename);                    // 250
          }                                                                                                            // 252
        });                                                                                                            // 253
      } else {                                                                                                         // 254
        console.log("Received HTTP error while renewing certificate (will keep retrying)", res.statusCode);            // 255
        res.on("data", function (chunk) {                                                                              // 257
          console.log("Error response contained information", chunk.toString("utf-8"));                                // 258
        });                                                                                                            // 259
      }                                                                                                                // 260
    };                                                                                                                 // 261
                                                                                                                       //
    var wrappedReadFile = Meteor.wrapAsync(fs.readFile);                                                               // 263
    performSandcatsRequest("/getcertificate", SANDCATS_HOSTNAME, {                                                     // 265
      rawHostname: SANDCATS_NAME,                                                                                      // 266
      certificateSigningRequest: wrappedReadFile(filenames.csrFilename, "utf-8")                                       // 267
    }, errorCallback, responseCallback);                                                                               // 265
  };                                                                                                                   // 269
                                                                                                                       //
  if (global.sandcats.shouldGetAnotherCertificate()) {                                                                 // 271
    console.log("renewHttpsCertificateIfNeeded: Happily choosing to renew certificate.");                              // 272
    return renewHttpsCertificate();                                                                                    // 273
  }                                                                                                                    // 274
};                                                                                                                     // 275
                                                                                                                       //
Sandcats.initializeSandcats = function () {                                                                            // 277
  var i = HOSTNAME.lastIndexOf(SANDCATS_HOSTNAME);                                                                     // 278
                                                                                                                       //
  if (i < 0) {                                                                                                         // 279
    console.error("SANDCATS_BASE_DOMAIN is configured but your HOSTNAME doesn't appear to contain it:", SANDCATS_HOSTNAME, HOSTNAME);
  } else {                                                                                                             // 282
    var oneMinute = 60 * 1000;                                                                                         // 283
    var oneHour = 60 * oneMinute;                                                                                      // 284
    var randomIntervalZeroToOneHour = Math.random() * oneHour; // All Sandcats installs need dyndns updating.          // 285
                                                                                                                       //
    SANDCATS_NAME = HOSTNAME.slice(0, i - 1);                                                                          // 287
    Meteor.setInterval(pingUdp, oneMinute); // If process.env.HTTPS_PORT is set, we need to auto-refresh our HTTPS certificate.
                                                                                                                       //
    if (process.env.HTTPS_PORT) {                                                                                      // 290
      // Always do a HTTPS certificate update check on Sandstorm start.                                                // 291
      Sandcats.renewHttpsCertificateIfNeeded(); // After that's done, schedule it for every approx 1-2 hours in        // 292
      // the future.                                                                                                   // 295
                                                                                                                       //
      Meteor.setInterval(Sandcats.renewHttpsCertificateIfNeeded, oneHour + randomIntervalZeroToOneHour);               // 296
    }                                                                                                                  // 298
  }                                                                                                                    // 299
};                                                                                                                     // 300
                                                                                                                       //
if (SANDCATS_HOSTNAME) {                                                                                               // 302
  Meteor.startup(Sandcats.initializeSandcats);                                                                         // 303
}                                                                                                                      // 304
                                                                                                                       //
;                                                                                                                      // 304
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"shell-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/shell-server.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var inMeteor = void 0;                                                                                                 // 1
module.watch(require("/imports/server/async-helpers.js"), {                                                            // 1
  inMeteor: function (v) {                                                                                             // 1
    inMeteor = v;                                                                                                      // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var staticAssetHost = void 0;                                                                                          // 1
module.watch(require("/imports/server/constants.js"), {                                                                // 1
  staticAssetHost: function (v) {                                                                                      // 1
    staticAssetHost = v;                                                                                               // 1
  }                                                                                                                    // 1
}, 1);                                                                                                                 // 1
BrowserPolicy.framing.disallow(); // Disallow framing of the UI.                                                       // 38
                                                                                                                       //
Meteor.startup(function () {                                                                                           // 39
  var frameSetter = function () {                                                                                      // 40
    BrowserPolicy.content.disallowFrame(); // This clears all the old rules                                            // 41
                                                                                                                       //
    BrowserPolicy.content.allowFrameOrigin(getWildcardOrigin());                                                       // 42
    var billingPromptUrl = globalDb.getBillingPromptUrl();                                                             // 43
                                                                                                                       //
    if (billingPromptUrl) {                                                                                            // 44
      BrowserPolicy.content.allowFrameOrigin(billingPromptUrl);                                                        // 45
    }                                                                                                                  // 46
  };                                                                                                                   // 47
                                                                                                                       //
  frameSetter(); // Call once on startup                                                                               // 49
                                                                                                                       //
  globalDb.collections.settings.find({                                                                                 // 50
    _id: "billingPromptUrl"                                                                                            // 50
  }).observe({                                                                                                         // 50
    added: frameSetter,                                                                                                // 51
    changed: frameSetter,                                                                                              // 52
    removed: frameSetter                                                                                               // 53
  });                                                                                                                  // 50
}); // Allow anything to be loaded from the static asset host.                                                         // 55
                                                                                                                       //
BrowserPolicy.content.allowImageOrigin(staticAssetHost);                                                               // 59
BrowserPolicy.content.allowScriptOrigin(staticAssetHost);                                                              // 60
BrowserPolicy.content.allowFontOrigin(staticAssetHost);                                                                // 61
BrowserPolicy.content.allowConnectOrigin(staticAssetHost);                                                             // 62
BrowserPolicy.content.allowConnectOrigin("wss:");                                                                      // 63
BrowserPolicy.content.allowConnectOrigin("ws:");                                                                       // 64
Meteor.publish("grainsMenu", function () {                                                                             // 66
  if (this.userId) {                                                                                                   // 67
    if (globalDb.isQuotaEnabled()) {                                                                                   // 68
      // Hack: Fire off an asynchronous update to the user's storage usage whenever they open the                      // 69
      //   front page.                                                                                                 // 70
      // TODO(someday): Implement the ability to reactively subscribe to storage usage from the                        // 71
      //   back-end?                                                                                                   // 72
      var userId = this.userId;                                                                                        // 73
      globalBackend.cap().getUserStorageUsage(userId).then(function (results) {                                        // 74
        inMeteor(function () {                                                                                         // 75
          Meteor.users.update(userId, {                                                                                // 76
            $set: {                                                                                                    // 76
              storageUsage: parseInt(results.size)                                                                     // 76
            }                                                                                                          // 76
          });                                                                                                          // 76
        });                                                                                                            // 77
      }).catch(function (err) {                                                                                        // 78
        if (err.kjType === "unimplemented") {                                                                          // 79
          // Compute based on sum of grain sizes instead.                                                              // 80
          var total = 0;                                                                                               // 81
          Grains.find({                                                                                                // 82
            userId: userId                                                                                             // 82
          }, {                                                                                                         // 82
            fields: {                                                                                                  // 82
              size: 1                                                                                                  // 82
            }                                                                                                          // 82
          }).forEach(function (grain) {                                                                                // 82
            return total += grain.size || 0;                                                                           // 83
          });                                                                                                          // 83
          Meteor.users.update(userId, {                                                                                // 84
            $set: {                                                                                                    // 84
              storageUsage: total                                                                                      // 84
            }                                                                                                          // 84
          });                                                                                                          // 84
        } else {                                                                                                       // 85
          console.error(err.stack);                                                                                    // 86
        }                                                                                                              // 87
      });                                                                                                              // 88
    }                                                                                                                  // 89
                                                                                                                       //
    return [UserActions.find({                                                                                         // 91
      userId: this.userId                                                                                              // 92
    }), Grains.find({                                                                                                  // 92
      userId: this.userId                                                                                              // 93
    }), ApiTokens.find({                                                                                               // 93
      "owner.user.accountId": this.userId                                                                              // 94
    })];                                                                                                               // 94
  } else {                                                                                                             // 96
    return [];                                                                                                         // 97
  }                                                                                                                    // 98
});                                                                                                                    // 99
Meteor.publish("sessions", function (sessionId) {                                                                      // 101
  // sessionId itself should be secret enough, but they are also not meant to be shared, so as                         // 102
  // a backup we only publish the session to its owner. Note that `userId` can be null if the                          // 103
  // user is not logged in or is using incognito mode.                                                                 // 104
  check(sessionId, String); // We exclude powerboxRequest because the client already has the descriptor list in packed
  // format, and the parsed format can be kind of large.                                                               // 108
                                                                                                                       //
  return Sessions.find({                                                                                               // 109
    _id: sessionId,                                                                                                    // 109
    $or: [{                                                                                                            // 109
      userId: this.userId                                                                                              // 109
    }, {                                                                                                               // 109
      userId: null                                                                                                     // 109
    }]                                                                                                                 // 109
  }, {                                                                                                                 // 109
    fields: {                                                                                                          // 110
      powerboxRequest: 0                                                                                               // 110
    }                                                                                                                  // 110
  });                                                                                                                  // 110
});                                                                                                                    // 111
Meteor.publish("devPackages", function () {                                                                            // 113
  return DevPackages.find();                                                                                           // 114
});                                                                                                                    // 115
Meteor.publish("hasUsers", function () {                                                                               // 117
  var _this = this;                                                                                                    // 117
                                                                                                                       //
  // Publish pseudo-collection which tells the client if there are any users at all.                                   // 118
  //                                                                                                                   // 119
  // TODO(cleanup):  This seems overcomplicated.  Does Meteor have a better way?                                       // 120
  var cursor = Meteor.users.find();                                                                                    // 121
                                                                                                                       //
  if (cursor.count() > 0) {                                                                                            // 122
    this.added("hasUsers", "hasUsers", {                                                                               // 123
      hasUsers: true                                                                                                   // 123
    });                                                                                                                // 123
  } else {                                                                                                             // 124
    var handle = cursor.observeChanges({                                                                               // 125
      added: function (id) {                                                                                           // 126
        _this.added("hasUsers", "hasUsers", {                                                                          // 127
          hasUsers: true                                                                                               // 127
        });                                                                                                            // 127
                                                                                                                       //
        handle.stop();                                                                                                 // 128
        handle = null;                                                                                                 // 129
      }                                                                                                                // 130
    });                                                                                                                // 125
    this.onStop(function () {                                                                                          // 132
      if (handle) handle.stop();                                                                                       // 133
    });                                                                                                                // 134
  }                                                                                                                    // 135
                                                                                                                       //
  this.ready();                                                                                                        // 137
});                                                                                                                    // 138
Meteor.publish("referralInfoPseudo", function () {                                                                     // 140
  var _this2 = this;                                                                                                   // 140
                                                                                                                       //
  // This publishes a pseudo-collection called referralInfo whose documents have the following                         // 141
  // form:                                                                                                             // 142
  //                                                                                                                   // 143
  // - id: (String) same as the User._id of an account this user has referred                                          // 144
  // - name: (String) the profile.name from that account                                                               // 145
  // - completed: (Boolean) if this referral is complete                                                               // 146
  //  If the user is not logged in, then we have no referralInfo.                                                      // 148
  if (!this.userId) {                                                                                                  // 149
    return [];                                                                                                         // 150
  } // Implementation note:                                                                                            // 151
  //                                                                                                                   // 154
  // This pseudo-collection is populated very differently for (1) the completed: false case versus                     // 155
  // the (2) completed: true case.                                                                                     // 156
  // Case 1. Publish information about not-yet-complete referrals.                                                     // 158
                                                                                                                       //
                                                                                                                       //
  var notCompletedReferralAccountsCursor = Meteor.users.find({                                                         // 159
    referredBy: this.userId,                                                                                           // 160
    referredByComplete: {                                                                                              // 161
      $exists: false                                                                                                   // 161
    },                                                                                                                 // 161
    type: "account",                                                                                                   // 162
    "profile.name": {                                                                                                  // 163
      $exists: true                                                                                                    // 163
    }                                                                                                                  // 163
  }, {                                                                                                                 // 159
    fields: {                                                                                                          // 165
      _id: 1,                                                                                                          // 166
      referredBy: 1,                                                                                                   // 167
      "profile.name": 1                                                                                                // 168
    }                                                                                                                  // 165
  });                                                                                                                  // 164
  var notCompletedReferralAccountsHandle = notCompletedReferralAccountsCursor.observeChanges({                         // 171
    // The added function gets called with the id of Bob when Alice refers Bob.                                        // 172
    added: function (id, fields) {                                                                                     // 173
      _this2.added("referralInfo", id, {                                                                               // 174
        name: fields.profile.name,                                                                                     // 174
        completed: false                                                                                               // 174
      });                                                                                                              // 174
    },                                                                                                                 // 175
    // The removed function gets called when Bob is no longer an uncompleted referral.  Note that                      // 176
    // this will get more complicated once we support sending completed referrals to the client.                       // 177
    removed: function (id) {                                                                                           // 178
      _this2.removed("referralInfo", id);                                                                              // 179
    },                                                                                                                 // 180
    // The modified function gets called when Bob's profile.name changed.                                              // 181
    modified: function (id, fields) {                                                                                  // 182
      _this2.modified("referralInfo", id, {                                                                            // 183
        name: fields.profile.name,                                                                                     // 183
        completed: false                                                                                               // 183
      });                                                                                                              // 183
    }                                                                                                                  // 184
  }); // Case 2. Handle completed referrals.                                                                           // 171
  //                                                                                                                   // 188
  // - Do a query for the current list of completed accounts.                                                          // 189
  //                                                                                                                   // 190
  // - Every time we see a new such account, we create a query that watches that one account in                        // 191
  //   case its profile.name changes.                                                                                  // 192
  //                                                                                                                   // 193
  // - Also watch the first query, since the list of completed accounts might change.                                  // 194
                                                                                                                       //
  var handleForProfileNameByAccountId = {};                                                                            // 195
                                                                                                                       //
  var stopWatchingAllAccounts = function () {                                                                          // 196
    Object.keys(handleForProfileNameByAccountId).forEach(function (accountId) {                                        // 197
      stopWatchingAccount(accountId);                                                                                  // 198
    });                                                                                                                // 199
  };                                                                                                                   // 200
                                                                                                                       //
  var stopWatchingAccount = function (accountId) {                                                                     // 202
    var handleForProfileName = handleForProfileNameByAccountId[accountId];                                             // 203
                                                                                                                       //
    if (handleForProfileName) {                                                                                        // 204
      _this2.removed("referralInfo", accountId);                                                                       // 205
                                                                                                                       //
      handleForProfileName.stop(); // delete is safe because we iterate across `Object.keys()` which returns a copy.   // 206
                                                                                                                       //
      delete handleForProfileNameByAccountId[accountId];                                                               // 208
    }                                                                                                                  // 209
  };                                                                                                                   // 210
                                                                                                                       //
  var watchAccountAndPublishReferralSuccess = function (accountId) {                                                   // 212
    var handleForProfileName = handleForProfileNameByAccountId[accountId];                                             // 213
                                                                                                                       //
    if (handleForProfileName) {                                                                                        // 214
      return;                                                                                                          // 215
    }                                                                                                                  // 216
                                                                                                                       //
    handleForProfileName = Meteor.users.find({                                                                         // 218
      _id: accountId                                                                                                   // 219
    }, {                                                                                                               // 218
      fields: {                                                                                                        // 221
        "profile.name": 1                                                                                              // 222
      }                                                                                                                // 221
    }).observeChanges({                                                                                                // 220
      added: function (id, fields) {                                                                                   // 225
        _this2.added("referralInfo", id, {                                                                             // 226
          name: fields.profile.name,                                                                                   // 226
          completed: true                                                                                              // 226
        });                                                                                                            // 226
      },                                                                                                               // 227
      changed: function (id, fields) {                                                                                 // 229
        _this2.changed("referralInfo", id, {                                                                           // 230
          name: fields.profile.name,                                                                                   // 230
          completed: true                                                                                              // 230
        });                                                                                                            // 230
      },                                                                                                               // 231
      removed: function (id) {                                                                                         // 233
        stopWatchingAccount(id);                                                                                       // 234
      }                                                                                                                // 235
    });                                                                                                                // 224
    handleForProfileNameByAccountId[accountId] = handleForProfileName;                                                 // 238
  };                                                                                                                   // 239
                                                                                                                       //
  var completedAccountIdsHandle = Meteor.users.find({                                                                  // 241
    _id: this.userId,                                                                                                  // 242
    referredAccountIds: {                                                                                              // 243
      $exists: true                                                                                                    // 243
    }                                                                                                                  // 243
  }, {                                                                                                                 // 241
    fields: {                                                                                                          // 245
      referredAccountIds: true                                                                                         // 246
    }                                                                                                                  // 245
  }).observeChanges({                                                                                                  // 244
    // `added` gets called when a user gets their first completed referral.                                            // 249
    added: function (id, fields) {                                                                                     // 250
      for (var i = 0; i < fields.referredAccountIds.length; i++) {                                                     // 251
        // Unconditionally mark these as successful referrals and start watching.                                      // 252
        watchAccountAndPublishReferralSuccess(fields.referredAccountIds[i]);                                           // 253
      }                                                                                                                // 255
    },                                                                                                                 // 256
    // `changed` gets called when a user adds/removes referredAccountIds, usually when a                               // 257
    // referral becomes complete.                                                                                      // 258
    changed: function (id, fields) {                                                                                   // 259
      // Two major tasks.                                                                                              // 260
      //                                                                                                               // 261
      // 1. Look for accountIds to unsubscribe from & send removed notices to the client.                              // 262
      //                                                                                                               // 263
      // 2. Look for accountIds to subscribe to.                                                                       // 264
      // Task 1. Unsubscribe where needed.                                                                             // 266
      var referredAccountIdsAsObject = {};                                                                             // 267
      fields.referredAccountIds.forEach(function (i) {                                                                 // 268
        referredAccountIdsAsObject[i] = true;                                                                          // 268
      });                                                                                                              // 268
      Object.keys(handleForProfileNameByAccountId).forEach(function (accountId) {                                      // 270
        // If the handle doesn't show up in the new list of referredAccountIds, then remove                            // 271
        // info from the client & stop it on the server & make it null.                                                // 272
        var handleForProfileName = handleForProfileNameByAccountId[accountId];                                         // 273
                                                                                                                       //
        if (referredAccountIdsAsObject.hasOwnProperty(accountId)) {                                                    // 274
          stopWatchingAccount(accountId);                                                                              // 275
        }                                                                                                              // 276
      }); // Task 2. Subscribe where needed.                                                                           // 277
                                                                                                                       //
      for (var i = 0; i < fields.referredAccountIds.length; i++) {                                                     // 280
        // The watch... function will avoid double-creating subscriptions, so this is safe.                            // 281
        watchAccountAndPublishReferralSuccess(fields.referredAccountIds[i]);                                           // 282
      }                                                                                                                // 283
    },                                                                                                                 // 284
    // `removed` gets called when a User suddenly has no referredAccountIds.                                           // 285
    removed: function () {                                                                                             // 286
      // Remove all data from client; stop all handles.                                                                // 287
      stopWatchingAllAccounts();                                                                                       // 288
    }                                                                                                                  // 289
  }); // With cases 1 and 2 handled, register a cleanup function, then declare victory.                                // 248
                                                                                                                       //
  this.onStop(function () {                                                                                            // 293
    stopWatchingAllAccounts();                                                                                         // 294
    notCompletedReferralAccountsHandle.stop();                                                                         // 295
    completedAccountIdsHandle.stop();                                                                                  // 296
  });                                                                                                                  // 297
  this.ready();                                                                                                        // 299
});                                                                                                                    // 300
Meteor.publish("backers", function () {                                                                                // 302
  var _this3 = this;                                                                                                   // 302
                                                                                                                       //
  var backers = Assets.getText("backers.txt");                                                                         // 303
  var anonCount = 0;                                                                                                   // 304
  var counter = 0;                                                                                                     // 305
  backers.split("\n").forEach(function (name) {                                                                        // 307
    name = name.trim();                                                                                                // 308
                                                                                                                       //
    if (name === "") {                                                                                                 // 309
      ++anonCount;                                                                                                     // 310
    } else {                                                                                                           // 311
      _this3.added("backers", counter++, {                                                                             // 312
        name: name                                                                                                     // 312
      });                                                                                                              // 312
    }                                                                                                                  // 313
  }); // Text file ends in \n but that shouldn't count.                                                                // 314
                                                                                                                       //
  --anonCount;                                                                                                         // 317
  this.added("backers", "anonymous", {                                                                                 // 319
    count: anonCount - 1                                                                                               // 319
  });                                                                                                                  // 319
  this.ready();                                                                                                        // 321
});                                                                                                                    // 322
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"signup-server.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/signup-server.js                                                                                             //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
// This file covers creation and consumption of invite keys (i.e. to invite people to become                           // 17
// users of the Sandstorm server).                                                                                     // 18
Meteor.publish("signupKey", function (key) {                                                                           // 20
  check(key, String);                                                                                                  // 21
  return SignupKeys.find(key);                                                                                         // 22
});                                                                                                                    // 23
Meteor.methods({                                                                                                       // 25
  useSignupKey: function (key) {                                                                                       // 26
    check(key, String);                                                                                                // 27
                                                                                                                       //
    if (!this.userId) {                                                                                                // 29
      throw new Meteor.Error(403, "Must be signed in.");                                                               // 30
    }                                                                                                                  // 31
                                                                                                                       //
    var user = Meteor.user();                                                                                          // 33
                                                                                                                       //
    if (user.signupKey) {                                                                                              // 34
      // Don't waste it.                                                                                               // 35
      return;                                                                                                          // 36
    }                                                                                                                  // 37
                                                                                                                       //
    if (isDemoUser()) {                                                                                                // 39
      throw new Meteor.Error(403, "Demo users cannot accept invite keys. Please sign in as a real user.");             // 40
    }                                                                                                                  // 42
                                                                                                                       //
    if (!user.loginCredentials) {                                                                                      // 44
      // Don't consume signup key as identity user.                                                                    // 45
      return;                                                                                                          // 46
    }                                                                                                                  // 47
                                                                                                                       //
    var keyInfo = SignupKeys.findOne(key);                                                                             // 49
                                                                                                                       //
    if (!keyInfo || keyInfo.used) {                                                                                    // 50
      throw new Meteor.Error(403, "Invalid key or already used.");                                                     // 51
    }                                                                                                                  // 52
                                                                                                                       //
    if (isSignedUp() && user.payments && user.payments.id) {                                                           // 54
      // This user is already signed up with a payment account. Possibly, they signed up before                        // 55
      // using their invite, and then went back and clicked on the invite. As a result they                            // 56
      // probably now have two payment accounts. Mark this invite as used but also add a special                       // 57
      // flag so we can find it later and cancel the dupe payment account. Record who tried to                         // 58
      // use it so that we can transfer credits over if needed.                                                        // 59
      SignupKeys.update(key, {                                                                                         // 60
        $set: {                                                                                                        // 60
          used: true,                                                                                                  // 60
          rejectedBy: this.userId                                                                                      // 60
        }                                                                                                              // 60
      });                                                                                                              // 60
      return;                                                                                                          // 61
    }                                                                                                                  // 62
                                                                                                                       //
    var userFields = {                                                                                                 // 64
      signupKey: key,                                                                                                  // 65
      signupNote: keyInfo.note                                                                                         // 66
    };                                                                                                                 // 64
                                                                                                                       //
    if (keyInfo.email) {                                                                                               // 68
      userFields.signupEmail = keyInfo.email;                                                                          // 69
    }                                                                                                                  // 70
                                                                                                                       //
    if ("quota" in keyInfo) {                                                                                          // 72
      userFields.quota = keyInfo.quota;                                                                                // 73
    }                                                                                                                  // 74
                                                                                                                       //
    if (keyInfo.plan) {                                                                                                // 76
      userFields.plan = keyInfo.plan;                                                                                  // 77
    }                                                                                                                  // 78
                                                                                                                       //
    if (keyInfo.payments) {                                                                                            // 80
      userFields["payments.id"] = keyInfo.payments.id;                                                                 // 81
    }                                                                                                                  // 82
                                                                                                                       //
    Meteor.users.update(this.userId, {                                                                                 // 84
      $set: userFields                                                                                                 // 84
    });                                                                                                                // 84
    SignupKeys.update(key, {                                                                                           // 85
      $set: {                                                                                                          // 85
        used: true                                                                                                     // 85
      }                                                                                                                // 85
    });                                                                                                                // 85
  }                                                                                                                    // 86
});                                                                                                                    // 25
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"standalone-server.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/standalone-server.js                                                                                         //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
Meteor.publish("standaloneDomain", function (domain) {                                                                 // 17
  check(domain, String);                                                                                               // 18
  return globalDb.collections.standaloneDomains.find({                                                                 // 20
    _id: domain                                                                                                        // 20
  });                                                                                                                  // 20
});                                                                                                                    // 21
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"startup.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/startup.js                                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
// This file is for various startup code that doesn't fit neatly anywhere else                                         // 17
var ROOT_URL = process.env.ROOT_URL;                                                                                   // 19
Meteor.startup(function () {                                                                                           // 21
  var baseUrlRow = Misc.findOne({                                                                                      // 22
    _id: "BASE_URL"                                                                                                    // 22
  });                                                                                                                  // 22
                                                                                                                       //
  if (!baseUrlRow) {                                                                                                   // 24
    // Fill data with current value in the case this is a first run                                                    // 25
    baseUrlRow = {                                                                                                     // 26
      _id: "BASE_URL",                                                                                                 // 26
      value: ROOT_URL                                                                                                  // 26
    };                                                                                                                 // 26
    Misc.insert(baseUrlRow);                                                                                           // 27
  } else if (baseUrlRow.value !== ROOT_URL) {                                                                          // 28
    // If the only thing that happened is some trailing slashes got removed, then don't reset OAuth                    // 29
    // providers.                                                                                                      // 30
    var resetOAuth = true;                                                                                             // 31
                                                                                                                       //
    if (baseUrlRow.value.replace(/[/]*$/, "") === ROOT_URL) {                                                          // 32
      console.log("Not resetting OAuth because old ROOT_URL and new ROOT_URL differ only by trailing slashes.");       // 33
      resetOAuth = false;                                                                                              // 34
    }                                                                                                                  // 35
                                                                                                                       //
    if (resetOAuth) {                                                                                                  // 37
      console.log("resetting oauth");                                                                                  // 38
      Settings.find({                                                                                                  // 39
        _id: {                                                                                                         // 39
          $in: ["google", "github"]                                                                                    // 39
        }                                                                                                              // 39
      }).forEach(function (setting) {                                                                                  // 39
        if (!!setting.value) {                                                                                         // 40
          Settings.update({                                                                                            // 41
            _id: setting._id                                                                                           // 41
          }, {                                                                                                         // 41
            $set: {                                                                                                    // 42
              value: false,                                                                                            // 42
              automaticallyReset: {                                                                                    // 43
                baseUrlChangedFrom: baseUrlRow.value                                                                   // 43
              }                                                                                                        // 43
            }                                                                                                          // 42
          });                                                                                                          // 42
        }                                                                                                              // 44
      });                                                                                                              // 45
    }                                                                                                                  // 46
                                                                                                                       //
    baseUrlRow = {                                                                                                     // 48
      _id: "BASE_URL",                                                                                                 // 48
      value: ROOT_URL                                                                                                  // 48
    };                                                                                                                 // 48
    Misc.update({                                                                                                      // 49
      _id: "BASE_URL"                                                                                                  // 49
    }, {                                                                                                               // 49
      $set: {                                                                                                          // 49
        value: ROOT_URL                                                                                                // 49
      }                                                                                                                // 49
    });                                                                                                                // 49
  }                                                                                                                    // 50
});                                                                                                                    // 51
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"stats-server.js":function(require,exports,module){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// server/stats-server.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var allowDemo = void 0;                                                                                                // 1
module.watch(require("/imports/demo.js"), {                                                                            // 1
  allowDemo: function (v) {                                                                                            // 1
    allowDemo = v;                                                                                                     // 1
  }                                                                                                                    // 1
}, 0);                                                                                                                 // 1
var DAY_MS = 24 * 60 * 60 * 1000;                                                                                      // 19
                                                                                                                       //
if (Mongo.Collection.prototype.aggregate) {                                                                            // 21
  throw new Error("Looks like Meteor wrapped the Collection.aggregate() call. Make sure it " + "works then delete our own wrapper.");
}                                                                                                                      // 24
                                                                                                                       //
Mongo.Collection.prototype.aggregate = function () {                                                                   // 26
  // Meteor doesn't wrapp Mongo's aggregate() method.                                                                  // 27
  var raw = this.rawCollection();                                                                                      // 28
  return Meteor.wrapAsync(raw.aggregate, raw).apply(raw, arguments);                                                   // 29
};                                                                                                                     // 30
                                                                                                                       //
computeStats = function (since) {                                                                                      // 32
  // We'll need this for a variety of queries.                                                                         // 33
  var timeConstraint = {                                                                                               // 34
    $gt: since                                                                                                         // 34
  }; // This calculates the number of user accounts that have been used                                                // 34
  // during the requested time period.                                                                                 // 37
                                                                                                                       //
  var currentlyActiveUsersCount = Meteor.users.find({                                                                  // 38
    expires: {                                                                                                         // 39
      $exists: false                                                                                                   // 39
    },                                                                                                                 // 39
    loginCredentials: {                                                                                                // 40
      $exists: true                                                                                                    // 40
    },                                                                                                                 // 40
    lastActive: timeConstraint                                                                                         // 41
  }).count(); // This calculates the number of grains that have been used during                                       // 38
  // the requested time period.                                                                                        // 45
                                                                                                                       //
  var activeGrainsCount = Grains.find({                                                                                // 46
    lastUsed: timeConstraint                                                                                           // 46
  }).count(); // If Meteor.settings.allowDemoAccounts is true, DeleteStats                                             // 46
  // contains records of type `user` and `appDemoUser`, indicating                                                     // 49
  // the number of those types of accounts that were created and                                                       // 50
  // then auto-expired through the demo mode's auto-account-expiry.                                                    // 51
                                                                                                                       //
  var deletedDemoUsersCount = DeleteStats.find({                                                                       // 52
    type: "demoUser",                                                                                                  // 53
    lastActive: timeConstraint                                                                                         // 53
  }).count();                                                                                                          // 53
  var deletedAppDemoUsersCount = DeleteStats.find({                                                                    // 54
    type: "appDemoUser",                                                                                               // 55
    lastActive: timeConstraint                                                                                         // 55
  }).count(); // Similarly, if the demo is enabled, we auto-delete grains; we store that                               // 55
  // fact in DeleteStats with type: "grain".                                                                           // 58
                                                                                                                       //
  var deletedGrainsCount = DeleteStats.find({                                                                          // 59
    type: "grain",                                                                                                     // 60
    lastActive: timeConstraint                                                                                         // 60
  }).count();                                                                                                          // 60
  var apps = Grains.aggregate([{                                                                                       // 62
    $match: {                                                                                                          // 63
      lastUsed: timeConstraint                                                                                         // 63
    }                                                                                                                  // 63
  }, {                                                                                                                 // 63
    $group: {                                                                                                          // 65
      _id: "$appId",                                                                                                   // 66
      grains: {                                                                                                        // 67
        $sum: 1                                                                                                        // 67
      },                                                                                                               // 67
      userIds: {                                                                                                       // 68
        $addToSet: "$userId"                                                                                           // 68
      }                                                                                                                // 68
    }                                                                                                                  // 65
  }, {                                                                                                                 // 64
    $project: {                                                                                                        // 72
      grains: 1,                                                                                                       // 73
      owners: {                                                                                                        // 74
        $size: "$userIds"                                                                                              // 74
      }                                                                                                                // 74
    }                                                                                                                  // 72
  }]);                                                                                                                 // 71
  apps = _.indexBy(apps, "_id");                                                                                       // 78
                                                                                                                       //
  for (var appId in meteorBabelHelpers.sanitizeForInObject(apps)) {                                                    // 80
    // We need to count ApiTokens, which don't have appId denormalized into them. We therefore                         // 81
    // have to fetch a list of grainIds first.                                                                         // 82
    // TODO(perf): If stats are getting slow, denormalize appId into ApiTokens. Note that it is                        // 83
    //   already denormalized for the specific case of apps that don't have icons, but the data                        // 84
    //   for that use case is NOT safe to use here because it's intended that we might allow an                        // 85
    //   app to mimic another app's icon by spoofing that app ID. In other words, the existing                         // 86
    //   denormalization of appId should be considered "app ID for identicon purposes only". We'll                     // 87
    //   need to add a new denormalization for stats purposes -- and make sure that it is not                          // 88
    //   revealed to the user.                                                                                         // 89
    var app = apps[appId];                                                                                             // 90
    delete app._id;                                                                                                    // 91
    var grains = Grains.find({                                                                                         // 92
      lastUsed: timeConstraint,                                                                                        // 93
      appId: appId                                                                                                     // 94
    }, {                                                                                                               // 92
      fields: {                                                                                                        // 96
        _id: 1                                                                                                         // 96
      }                                                                                                                // 96
    }).fetch();                                                                                                        // 95
                                                                                                                       //
    var grainIds = _.pluck(grains, "_id");                                                                             // 98
                                                                                                                       //
    var counts = ApiTokens.aggregate([{                                                                                // 100
      $match: {                                                                                                        // 102
        "owner.user": {                                                                                                // 103
          $exists: true                                                                                                // 103
        },                                                                                                             // 103
        lastUsed: timeConstraint,                                                                                      // 104
        grainId: {                                                                                                     // 105
          $in: grainIds                                                                                                // 105
        }                                                                                                              // 105
      }                                                                                                                // 102
    }, {                                                                                                               // 101
      $group: {                                                                                                        // 108
        _id: "$owner.user.accountId"                                                                                   // 108
      }                                                                                                                // 108
    }, {                                                                                                               // 108
      $group: {                                                                                                        // 110
        _id: "count",                                                                                                  // 111
        count: {                                                                                                       // 112
          $sum: 1                                                                                                      // 112
        }                                                                                                              // 112
      }                                                                                                                // 110
    }]);                                                                                                               // 109
                                                                                                                       //
    if (counts.length > 0) {                                                                                           // 117
      if (counts.length !== 1) {                                                                                       // 118
        console.error("error: sharedUsers aggregation returned multiple rows");                                        // 119
      }                                                                                                                // 120
                                                                                                                       //
      app.sharedUsers = counts[0].count;                                                                               // 122
    }                                                                                                                  // 123
  } // Count per-app appdemo users and deleted grains.                                                                 // 124
                                                                                                                       //
                                                                                                                       //
  DeleteStats.aggregate([{                                                                                             // 127
    $match: {                                                                                                          // 129
      lastActive: timeConstraint,                                                                                      // 130
      appId: {                                                                                                         // 131
        $exists: true                                                                                                  // 131
      }                                                                                                                // 131
    }                                                                                                                  // 129
  }, {                                                                                                                 // 128
    $group: {                                                                                                          // 135
      _id: {                                                                                                           // 136
        appId: "$appId",                                                                                               // 137
        type: "$type"                                                                                                  // 138
      },                                                                                                               // 136
      count: {                                                                                                         // 140
        $sum: 1                                                                                                        // 140
      }                                                                                                                // 140
    }                                                                                                                  // 135
  }]).forEach(function (deletion) {                                                                                    // 134
    var app = apps[deletion._id.appId];                                                                                // 144
                                                                                                                       //
    if (!app) {                                                                                                        // 145
      app = apps[deletion.appId] = {};                                                                                 // 146
    }                                                                                                                  // 147
                                                                                                                       //
    if (deletion._id.type === "appDemoUser") {                                                                         // 149
      app.appDemoUsers = deletion.count;                                                                               // 150
    } else if (deletion._id.type === "grain") {                                                                        // 151
      app.deleted = deletion.count;                                                                                    // 152
    } else if (deletion._id.type === "demoGrain") {                                                                    // 153
      app.demoed = deletion.count;                                                                                     // 154
    }                                                                                                                  // 155
  });                                                                                                                  // 156
  return {                                                                                                             // 158
    activeUsers: currentlyActiveUsersCount,                                                                            // 159
    demoUsers: deletedDemoUsersCount,                                                                                  // 160
    appDemoUsers: deletedAppDemoUsersCount,                                                                            // 161
    activeGrains: activeGrainsCount + deletedGrainsCount,                                                              // 162
    apps: apps                                                                                                         // 163
  };                                                                                                                   // 158
};                                                                                                                     // 165
                                                                                                                       //
function recordStats() {                                                                                               // 167
  var postStats = function (record) {                                                                                  // 168
    HTTP.post("https://alpha-api.sandstorm.io/data", {                                                                 // 169
      data: record,                                                                                                    // 170
      headers: {                                                                                                       // 171
        Authorization: "Bearer aT-mGyNwsgwZBbZvd5FWr0Ma79O9IehI4NiEO94y_oR",                                           // 172
        "Content-Type": "application/json"                                                                             // 173
      }                                                                                                                // 171
    });                                                                                                                // 169
  };                                                                                                                   // 176
                                                                                                                       //
  var now = new Date();                                                                                                // 178
                                                                                                                       //
  var planStats = _.countBy(Meteor.users.find({                                                                        // 180
    expires: {                                                                                                         // 181
      $exists: false                                                                                                   // 181
    },                                                                                                                 // 181
    "payments.id": {                                                                                                   // 181
      $exists: true                                                                                                    // 181
    }                                                                                                                  // 181
  }, {                                                                                                                 // 181
    fields: {                                                                                                          // 182
      plan: 1                                                                                                          // 182
    }                                                                                                                  // 182
  }).fetch(), "plan");                                                                                                 // 182
                                                                                                                       //
  var record = {                                                                                                       // 186
    timestamp: now,                                                                                                    // 187
    daily: computeStats(new Date(now.getTime() - DAY_MS)),                                                             // 188
    weekly: computeStats(new Date(now.getTime() - 7 * DAY_MS)),                                                        // 189
    monthly: computeStats(new Date(now.getTime() - 30 * DAY_MS)),                                                      // 190
    forever: computeStats(new Date(0)),                                                                                // 191
    plans: planStats                                                                                                   // 192
  };                                                                                                                   // 186
  record.computeTime = Date.now() - now;                                                                               // 194
                                                                                                                       //
  if (global.BlackrockPayments && global.BlackrockPayments.getTotalCharges) {                                          // 195
    // This only exists under Blackrock                                                                                // 196
    record.totalCharges = global.BlackrockPayments.getTotalCharges();                                                  // 197
  }                                                                                                                    // 198
                                                                                                                       //
  ActivityStats.insert(record);                                                                                        // 200
  var age = ActivityStats.find().count(); // The stats page which the user agreed we can send actually displays the whole history
  // of the server, but we're only sending stats from the last day. Let's also throw in the                            // 203
  // length of said history. This is still strictly less information than what the user said                           // 204
  // we're allowed to send.                                                                                            // 205
                                                                                                                       //
  record.serverAge = age;                                                                                              // 206
                                                                                                                       //
  if (age > 3) {                                                                                                       // 208
    var reportSetting = Settings.findOne({                                                                             // 209
      _id: "reportStats"                                                                                               // 209
    });                                                                                                                // 209
                                                                                                                       //
    if (!reportSetting) {                                                                                              // 211
      // Setting not set yet, send out notifications and set it to false                                               // 212
      globalDb.sendAdminNotification("reportStats", "/admin/stats");                                                   // 213
      Settings.insert({                                                                                                // 214
        _id: "reportStats",                                                                                            // 214
        value: "unset"                                                                                                 // 214
      });                                                                                                              // 214
    } else if (reportSetting.value === true) {                                                                         // 215
      postStats(record);                                                                                               // 216
    }                                                                                                                  // 217
  }                                                                                                                    // 218
}                                                                                                                      // 219
                                                                                                                       //
if (!Meteor.settings.replicaNumber) {                                                                                  // 221
  // Wait until 10:00 UTC (2:00 PST / 5:00 EST), then start recording stats every 24 hours.                            // 222
  // (Only on the first replica to avoid conflicts.)                                                                   // 223
  Meteor.setTimeout(function () {                                                                                      // 224
    Meteor.setInterval(function () {                                                                                   // 225
      recordStats();                                                                                                   // 226
    }, DAY_MS);                                                                                                        // 227
    recordStats();                                                                                                     // 229
  }, DAY_MS - (Date.now() - 10 * 60 * 60 * 1000) % DAY_MS);                                                            // 230
  Meteor.startup(function () {                                                                                         // 232
    if (StatsTokens.find().count() === 0) {                                                                            // 233
      StatsTokens.remove({});                                                                                          // 234
      StatsTokens.insert({                                                                                             // 235
        _id: Random.id(22)                                                                                             // 235
      });                                                                                                              // 235
    }                                                                                                                  // 236
  });                                                                                                                  // 237
}                                                                                                                      // 238
                                                                                                                       //
Meteor.methods({                                                                                                       // 240
  regenerateStatsToken: function () {                                                                                  // 241
    if (!isAdmin()) {                                                                                                  // 242
      throw new Meteor.Error(403, "Unauthorized", "User must be admin");                                               // 243
    }                                                                                                                  // 244
                                                                                                                       //
    StatsTokens.remove({});                                                                                            // 246
    var token = StatsTokens.insert({                                                                                   // 247
      _id: Random.id(22)                                                                                               // 247
    });                                                                                                                // 247
    return token._id;                                                                                                  // 248
  }                                                                                                                    // 249
});                                                                                                                    // 240
Router.map(function () {                                                                                               // 252
  this.route("fetchStats", {                                                                                           // 253
    where: "server",                                                                                                   // 254
    path: "/fetchStats/:tokenId",                                                                                      // 255
    action: function () {                                                                                              // 256
      var token = StatsTokens.findOne({                                                                                // 257
        _id: this.params.tokenId                                                                                       // 257
      });                                                                                                              // 257
                                                                                                                       //
      if (!token) {                                                                                                    // 259
        this.response.writeHead(404, {                                                                                 // 260
          "Content-Type": "text/plain"                                                                                 // 261
        });                                                                                                            // 260
        this.response.write("Token not found");                                                                        // 263
        return this.response.end();                                                                                    // 264
      }                                                                                                                // 265
                                                                                                                       //
      try {                                                                                                            // 267
        var stats = ActivityStats.find().fetch();                                                                      // 268
        var statsString = JSON.stringify(stats);                                                                       // 269
        this.response.writeHead(200, {                                                                                 // 271
          "Content-Type": "application/json"                                                                           // 272
        });                                                                                                            // 271
        this.response.write(statsString);                                                                              // 274
      } catch (error) {                                                                                                // 275
        console.error(error.stack);                                                                                    // 276
        this.response.writeHead(500, {                                                                                 // 277
          "Content-Type": "text/plain"                                                                                 // 278
        });                                                                                                            // 277
        this.response.write(error.stack);                                                                              // 280
      }                                                                                                                // 281
                                                                                                                       //
      return this.response.end();                                                                                      // 283
    }                                                                                                                  // 284
  });                                                                                                                  // 253
});                                                                                                                    // 286
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"i18n":{"en.i18n.json":function(require){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// i18n/en.i18n.json                                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _ = Package.underscore._,
    package_name = "project",
    namespace = "project";

if (package_name != "project") {
    namespace = TAPi18n.packages[package_name].namespace;
}
TAPi18n._enable({"helper_name":"_","supported_languages":null,"i18n_files_route":"/tap-i18n","preloaded_langs":[],"cdn_path":null});
TAPi18n.languages_names["en"] = ["English","English"];
// integrate the fallback language translations 
translations = {};
translations[namespace] = {"error":"Error: %s","shell":{"layout":{"demoExpired":{"title":"Demo Expired","canUpgrade":"Thanks for trying the Sandstorm Demo! Your demo account has expired. If you'd like to continue using Sandstorm and make your data permanent, please sign in.","cannotUpgrade":"Thanks for trying the Sandstorm Demo! Your demo account has expired.","signOut":"Sign out"},"adminAlert":{"icon":"!","title":"Notice from Admin"},"demoModal":{"header":"Sandstorm is a self-hostable web productivity suite.","withAppDemo":"with apps like %s.","withoutAppDemo":"with all the apps you need in one place.","startButton":{"withAppDemo":"Try %s »","withoutAppDemo":"Start »"},"explanation":"<p>Sandstorm makes it easy to run apps on your own server. In addition to improving privacy and control, <a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\" target=\"_blank\">this brings Open Source apps to a wide audience.</a></p>","securityOverview":"<p>Sandstorm is also ridiculously secure. If security interests you, check out <a href=\"https://docs.sandstorm.io/en/latest/developing/security-practices/\" target=\"_blank\">Sandstorm's security model overview</a>.</p>"}},"notifications":{"hint":"Notifications","text":"Notifications"},"notificationsPopup":{"title":"Notifications","empty":"No notifications..."},"loading":{"title":"Loading...","text":"Loading..."},"notFound":{"title":"Not Found","text":"404 Not Found :("},"root":{"title":"Home"},"grainLogContents":{"prefix":"..."},"grainLog":{"title":"Debug log: %s"},"signup":{"title":"Claim Invite","signedUp":{"keyIsUsedWithOrigin":"You're all signed up!  Now go <a href=\"https://apps.sandstorm.io/?host=%s\">install some apps</a>.","keyIsNotUsed":"It looks like you already have an account, so you don't need this signup key."},"keyIsUsed":"Sorry, this signup key has already been used.","processing":"Please wait...","signInGuide":"First, you have to sign in. Click \"Sign in\" in the upper-right.","invalidKey":"Sorry, this is not a valid signup key."},"uploadStatus":{"title":"Uploading...","uploadFailedWithError":"Upload failed: %s %s","statusWithProgress":"%s... %s","status":"%s...","uploadFailed":"Upload failed. Please try again."},"uploadTest":{"title":"Upload test page","explanation":"This page is intended for use by the test scripts.","uploadApp":"Upload app:","uploadBackup":"Upload backup:"},"referrals":{"title":"Referral Program","header":{"isPaid":{"title":"Get up to <span class=\"reward-amount\">30 GB</span> of free space when your friends share on Oasis!","explanation":"For each friend that creates a sharing link, we'll reward you with 2 GB."},"isNotPaid":{"title":"Get unlimited grains &amp; up to <span class=\"reward-amount\">2 GB</span> of free space when your friends share on Oasis!","explanation":"For each friend that creates a sharing link, we'll reward you with 50 MB. You'll also get unlimited grains."}},"howTo":{"title":"Sharing is referring","explanation":"To get started, open any grain and click Share access.","getStart":{"step1":"You share a grain with your friend (who isn't already an Oasis user).","step2":"Your friend signs in.","step3":"Your friend shares a grain with any other person (including you)."}},"status":{"title":"Your referrals & status","explanation":"Once you send a sharing link out, people will appear in this list after they sign in.","pending":{"title":"Still needs to share","explanation":"Tell these people to make a grain and share it with you. You must actually view the grain they shared with you!"},"success":{"title":"Successful referrals","explanation":"You've successfully referred these people. Congratulations!"}},"finePrint":{"title":"The fine print","warningWithAccountLink":"Your referral bonus lasts until 1 year after the referral. The terms of this referral program may change without warning; however, such changes will not affect bonuses you have already received. If you need even more space, <a href=\"%s\">upgrade your account.</a>","explanation":"Paid subscribers can get up to 30 GB of free storage, 2 GB per person they refer, in addition to the storage from their account level. Free account holders can earn unlimited grains and 50 MB of free storage per person they refer, up to 2 GB total."}},"about":{"title":"About Sandstorm","intro":{"explanation":"<a href=\"https://sandstorm.io\">Sandstorm.io</a> is an open source hosting platform for personal/private instances of web apps. Users can upload and install arbitrary software through a simple app-store-like web interface. In addition to better privacy, <a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\">this is the only way to make Open Source web apps viable.</a>","version":"Version: %s","unofficialWarning":"<strong>WARNING:</strong> This server is running an unofficial build of Sandstorm which may contain modifications not reviewed by the developers."},"changelog":{"title":"Recent changes"},"developers":{"title":"Core developers"},"advisors":{"title":"Advisors"},"corporate":{"title":"Corporate sponsors","explanation":"These companies contributed heavily to Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaign</a>."},"individual":{"title":"Key indivisual sponsors","explanation":"These individuals contributed heavily to Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaign</a>."},"backers":{"title":"Backers","explanation":"from <a href=\"http://igg.me/at/sandstorm\">our Indiegogo campaign &gt;</a>","anonymous":"and %s anonymous contributors","loading":"loading..."},"dependencies":{"title":"Third-party works used","license":"license","explanation":"These works are copyright the respective authors and used under the terms of the respective licenses."},"copyright":{"title":"Copyright","license":"The Sandstorm implementation is copyright by Sandstorm Development Group, Inc. and contributors. The implementation is licensed under the Apache License 2.0.","trademark":"\"Sandstorm\" and the Sandstorm logo are trademarks of Sandstorm Development Group, Inc., and may not be used to advertise derivative works nor commercial services without written consent from Sandstorm Development Group, Inc."},"terms":{"license":"License","terms":"Terms of service","privacy":"Privacy policy"}}},"grains":{"title":"Grains","grainView":{"notFoundWithId":"No grain found with ID %s.","openWithWhichIdentity":"Would you like to reveal your identity to this grain, or remain anonymous?","openInIncognitoMode":"Open in Incognito Mode","revealMyIdentity":"Reveal my identity"},"grainInMyTrash":{"explanation":"This grain is in your trash. You cannot access it unless you restore it to your main list.","restore":"Restore to main list"},"grainInOwnersTrash":{"explanation":"You can no longer access this grain because its owner has moved it to the trash."},"grainOwnerSuspended":{"explanation":"You can no longer access this grain because its owner's account has been suspended."},"wrongIdentity":{"explanation":"This link was intended for the user:"},"requestAccess":{"permissionDenied":"You do not have permission to access this grain.","requestAccessHint":"Request access","requestAccess":"Request Access","warnIdentityRevealing":"Requesting access will reveal your identity to the grain owner.","sendRequest":"Sending your request...","waitForApproving":"Your request has been sent. You will receive an email if your request is approved.","requestFailed":"Your request failed because: %s","signInPlease":"Please sign in to request access."},"invalidToken":{"invalidToken":"Invalid token: %s"},"revokedShareLink":{"linkRevoked":"Sorry, this link has been revoked."},"grainTitle":{"was":"was: %s","renamedFrom":"renamed from: %s"},"grainDeleteButton":{"hint":"Move to trash","text":"Move to trash"},"grainDebugLogButton":{"hint":"Show Debug Log","text":"Log"},"grainBackupButton":{"hint":"Download Backup","text":"Backup"},"grainBackupPopup":{"title":"Download Backup","warning":"Warning! Restoring this backup might not work how you expect it to.","processing":"Preparing backup...","cancelHint":"cancel","cancel":"Cancel","confirmHint":"download backup","confirm":"Download backup"},"grainRestartButton":{"hint":"Restart App","text":"Restart"},"grainApiTokenButton":{"hint":"Get webkey","text":"Get&nbsp;Webkey"},"grainApiTokenPopup":{"title":"Webkeys","processing":"Generating...","cancel":"Cancel","copyForExternalApp":"Copy this webkey and give it to the external app:","back":"Go back","description":"A webkey allows you to connect an external app (e.g. a mobile app) to this app.","labelTitle":"Label:","labelPlaceholder":"e.g. 'Android app'","roleTitle":"Role:","permissionTitle":"all access","create":"Create","webkeyListTitle":"Your existing webkeys:","revokeHint":"Revoke","revoke":"Revoke"},"grainShareButton":{"hint":"Share access to this grain","text":"Share access"},"whoHasAccessPopup":{"title":"Who has access","noLoggedInUsers":"No logged-in users have visited your links yet.","nobody":"You have granted access to nobody.","addedBy":"added by","revokeHint":"Revoke","revoke":"Revoke","hasCustomPermissions":"[has custom permissions]"},"selectRole":{"hint":"select a role"},"emailInviteTab":{"demoModeAlert":"Sharing emails cannot be sent while in demo mode. Create an account to send sharing emails, or get a shareable link instead.","personalMessagePlaceholder":"Type a personal message (optional)...","send":"Send","sending":"Sending...","success":"Successfully sent.","reset":"+ Send more invites","startOver":"<< Start over"},"shareableLinkTab":{"withRoles":"Anyone with this link:","withoutRoles":"Anyone with this link can access the grain.","labelPlaceholder":"Label (optional)","labelExplanation":"Use a label to remind yourself to whom you sent this unique link.","create":"Create","creating":"Generating...","success":"Copy this link:","reset":"+ Create another link","cancel":"Cancel"},"shareWithOthers":{"title":"Share with others","sendInviteTab":"Send an invite","shareableLinkTab":"Get shareable link","whoHasAccess":"See who has access"},"grainSharePopup":{"oldSharingModelDescription":"This grain uses the old sharing model. You can share it by copy/pasting the URL from the location bar","upgradeDescription":"You can upgrade this grain to the new model, but anyone with whom you shared this grain previously will lose access; you will have to share with them again.","upgradeWarning":"Upgrading cannot be undone.","upgrade":"upgrade","openInIncognitoMode":"You are viewing this grain anonymously.","toShareAccess":"To share access, copy this link:","openNonAnonymouslyHead":"For more sharing options, you must","openNonAnonymouslyLink":"open this grain non-anonymously","openNonAnonymouslyTail":".","signIn":"For more sharing options, please sign in."},"grainPowerboxRequest":{"powerboxRequestHint":"Powerbox request","powerboxRequest":"P"},"grainPowerboxOffer":{"powerboxOfferHint":"Powerbox offer","powerboxOffer":"P"},"grainPowerboxOfferPopup":{"title":"Powerbox Offer","dismiss":"Dismiss"},"share":{"grainNotFoundWithId":"No grain found with ID %s. Maybe you need to sign in to a different account?","signIn":"You must be signed in to grant access to a grain."},"grainlist":{"sandstormGrainListPage":{"introButton":{"hint":"What is a grain?"},"mainListButton":"View main list","trashButton":"View trash (%s)","restoreButton":"Restore backup...","search":{"placeholder":"search"},"usageInfo":{"quota":"__count__ of __quota__ grains","size":"__size__ of __total__ used"},"trash":{"emptyButton":"Empty all trash","explanation":"(Grains will be permanently removed after 30 days in the trash. To keep a grain, restore it to the Main list.)"},"noGrains":{"notFound":"<strong>No matching grains found.</strong>","inTrash":"Some grains in your trash match your search.","trashButton":"View trash","empty":"<strong>Trash is empty.</strong>","explanation":"You don't have any grains yet.","hasApps":"Select an app and create your first grain.","hasNoApp":"Install an app to create your first grain."},"moveToTrash":"Move to trash"},"sandstormGrainTable":{"bulkAction":{"disabledHint":"To perform bulk actions, you must first select some grains in the list."},"tableHeader":{"title":"Name","size":"Size","lastUsed":"Last activity","ownership":"Mine/Shared","collaborators":"Collaborators"},"tableBody":{"was":"(was: %s)","renamedFrom":"(renamed from: %s)","new":"Unopened","mine":"My grain","shared":"Shared with me"}},"contactAutocomplete":{"contactInputBox":{"placeholder":"Enter email addresses or names"}}}},"notifications":{"appUpdateNotificationItem":{"title":"App updates are available:","appUpdates":"<strong>__name__</strong>: version __version__ is available","submitButton":{"hint":"Update all of the above apps","text":"Apply updates"},"dismissButton":{"hint":"Ignore the above updates","text":"Dismiss"}},"mailingListBonusNotificationItem":{"explanationWithBonus":"Subscribe to our announcement list and get <strong>%s bonus storage</strong> (1-2 emails per month).","submitButton":"Subscribe","dismissButton":"No thanks"},"referralNotificationItem":{"paidUser":"Get up to 30 GB bonus with the","unpaidUser":"Unlimited grains & up to 2 GB bonus with the","link":"referral program »","dismissButton":"Dismiss"},"adminNotificationItem":{"explanation":"You can help Sandstorm by sending us some anonymous usage stats. Click here for more info."},"backgroundedGrainNotificationItem":{"title":"<strong>__grain__</strong> is backgrounded:","cancelButton":{"hint":"Stop the background app","text":"Cancel"}},"identityChangesNotificationItem":{"explanation":"We're changing the way identities work in Sandstorm. Your account may be affected. Click here to learn more."},"grainActivityNotificationItem":{"title":"<strong>__sender__</strong> in <strong>__grain__</strong>:","dismissButton":{"hint":"Dismiss this notification","text":"Dismiss"}}},"powerboxBuiltins":{"uiViewPowerboxConfiguration":{"explanation":"with permissions:","fullAccess":"Full access","connectButton":"Connect","chooseHostedObjectButton":"Browse grain contents »"},"ipNetworkPowerboxCard":{"explanation":"Admin: grant all outgoing network access","encryptedWith":"(encrypted with %s)"},"ipInterfacePowerboxCard":{"explanation":"Admin: grant all incoming network access"},"emailVerifierPowerboxCard":{"verifyWith":"Verify e-mail addresses using %s","verifyWithAnyService":"Verify e-mail addresses using any login service"},"identityPowerboxConfiguration":{"explanation":"with permissions:","fullAccess":"Full access","connectButton":"Connect"},"addNewVerifiedEmailPowerboxCard":{"explanation":"Add a new address"},"httpUrlPowerboxCard":{"useUrl":"Use %s"},"httpArbitraryPowerboxCard":{"explanation":"Specify URL..."},"httpArbitraryPowerboxConfiguration":{"explanation":"Enter URL:","placeholder":"https://example.com","connectButton":"Connect"},"httpOAuthPowerboxCard":{"useService":"Use %s"},"httpOAuthPowerboxConfiguration":{"requstPermissionsFromService":"Requesting permissions from %s (see popup)."}},"styleguide":{"title":"Sandstorm style guide","note":"(note: this is a work in progress.)","copyRules":{"title":"Copy rules","rule1":"Be terse but clear.  Avoid technical terms where less-technical ones will do.","rule2":"Capitalize the first letter of the first word of a section heading or button.  Do not capitalize the second word, or anything else, unless referring to a proper noun like Google or GitHub.","rule3":"Use verbs for text on buttons. The button text should describe what clicking that button will do.","rule4":"Use nouns for text which is a link.  The link text should indicate what the link will take you to.","rule5":"Use nouns for basically everything else, too, to communicate the information architecture. Section headings should be nouns which describe the content of the section.  Labels should be nouns which describe the content of the field."},"styleRules":{"title":"Style rules","rule1":"Stick to a limited color palette.  (TODO: add specific colors)","rule2":"Use typefaces no smaller than 13px for text content.  Readability is important and not everyone has young eyes.","rule3":"Make click targets no smaller than 32px square or 26px tall if appreciably wider.  Remember that due to Fitts' Law, larger click targets are easier to interact with."},"forms":{"title":"Forms","rule1":"Use the <code>standard-form</code> class where appropriate.","rule2":"The primary action button of a form should be at the bottom right of the form. Secondary buttons should line up to the left of that primary action button.","rule3":"When providing text input fields, place labels above the input, not side-by-side or inside the field as placeholder text.  Place additional explanation or instructions below the input. Placeholder text should be used for examples, not instructions.","rule4":"Forms are generally used for transactional interactions, where the user cares about the success or failure of the application.  Avoid optimistic UI in favor of multi-state UI which communicates network activity, success, and failure.  A common pattern is:","rule4-1":"Have the default state of the form.  Disable a save button until the user makes a change.","rule4-2":"When the user clicks an action button, start the RPC, and disable the entire form to indicate that the action is underway.","rule4-3":"When the RPC completes, if the RPC failed, show an error message at the top of the context over which the action applies.  If the RPC succeeded, either indicate a success message (again, at the top of the context over which the action applies) or if this was a modal interaction and the completion of the RPC indicates successful completion of the workflow, dismiss the form with no additional message."},"example":{"title":"An example:","label":"Email","placeholder":"alice@example.com","subtext":"You might want to receive notifications.","submitButton":"Save","discardButton":"Discard"}},"admin":{"title":"Admin","admin":{"newAdmin":{"title":"Admin panel","wildcardHostBroken":"WARNING: This server seems to have its WILDCARD_HOST misconfigured.  Until you fix it, you will not be able to use any apps. You can read <a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#why-do-i-see-an-error-when-i-try-to-launch-an-app-even-when-the-sandstorm-interface-works-fine\"> more info in the Sandstorm docs</a> and in your browser's Javascript console. You'll need to adjust DNS, SSL/TLS certificates, or edit the sandstorm.conf file. If you see no information in the JS console, or wish to test if you have fixed the problem, reload this page to re-run the test. If you're still having problems, <a href=\"https://github.com/sandstorm-io/sandstorm/issues\">please file an issue.</a>","websocketBroken":"WARNING: This server seems to be unable to create a WebSocket. Sandstorm may appear to run fine, but many apps will fail to run properly. <a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#how-do-i-enable-websockets-proxying-or-why-do-some-apps-seem-to-crash-reload\"> Learn more.</a>  Usually this is caused by a reverse proxy like nginx or Apache that needs to be reconfigured. If you think you are seeing this message in error, please reload this page or <a href=\"https://github.com/sandstorm-io/sandstorm/issues\">file an issue.</a>","hasSetupToken":"You are not logged in as an admin, but you do have a setup token. You can either <a href=\"/setup\">visit the setup wizard</a> or <a href=\"/admin/status\">view the system log</a>.","hasNoSetupToken":"You are not logged in as an admin. To access the admin panel, log in as an admin user or generate a token from the command line by running `sandstorm admin-token`."},"newAdminRoot":{"configuration":{"title":"Configuration"},"management":{"title":"Management"}}},"identityProviders":{"name":"Login providers","providers":{"emailPassLess":"E-mail (passwordless)"},"subtext":"How users log in.","newAdminIdentity":{"explanation":"Choose which services users may use to identify themselves.  Anyone can log in, but only users you invite will be able to install apps or create grains.","organizationLinkPrefix":"To configure organization membership and settings, see","organizationLink":"organization settings","organizationLinkPostfix":"."},"adminIdentityProviderConfigureEmail":{"title":"Email login configuration","error":"Failed to save changes: %s","warning":"Note: to use email login, you'll need to configure email delivery after this.","explanation":"Email login has no additional configuration options.","saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"googleLoginSetupInstructions":{"explanation":"First, you'll need to get a Google Client ID. Follow these steps:","step1":"Visit <a href=\"https://console.developers.google.com/\" target=\"blank\">https://console.developers.google.com/</a>","step2":"\"Create Project\", if needed. Wait for Google to finish provisioning.","step3":"On the left sidebar, go to \"Credentials\" and, on the right, \"OAuth consent screen\". Make sure to enter an email address and a product name, and save.","step4":"On the left sidebar, go to \"Credentials\".  Click the \"Create credentials\" button, then select \"OAuth client ID\" as the type.","step5":"Select \"Web application\" as your application type.","step6":"Set Authorized Javascript Origins to: <span class=\"idp-config-url\">%s</span>","step7":"Set Authorized Redirect URI to: <span class=\"idp-config-url\">%s/_oauth/google</span>","step8":"Finish by clicking \"Create\"."},"adminIdentityProviderConfigureGoogle":{"title":"Google login configuration","formerBaseUrlWarning":"This server moved from <code>__former__</code> to <code>__current__</code>. You'll need to visit <a target=\"_blank\" href=\"https://console.developers.google.com/\">https://console.developers.google.com/</a> and update the \"Authorized Javascript Origins\" and \"Authorized Redirect URI\" for this Client ID to include the values below.","error":"Failed to save changes: %s","explanation":"Then, copy over your Client ID and Client secret below:","clientId":"Client ID:","clientSecret":"Client secret:","saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"githubLoginSetupInstructions":{"explanation":"First, you'll need to get a Github Client ID and Client secret. Follow these steps:","step1":"Visit <a href=\"https://github.com/settings/applications/new\" target=\"blank\">https://github.com/settings/applications/new</a>","step2":"Set Homepage URL to: <span class=\"idp-config-url\">%s</span>","step3":"Set Authorization callback URL to: <span class=\"idp-config-url\">%s_oauth/github</span>"},"adminIdentityProviderConfigureGitHub":{"title":"GitHub login configuration","formerBaseUrlWarning":"This server moved from <code>__former__</code> to <code>__current__</code>. You'll need to visit <a target=\"_blank\" href=\"https://github.com/settings/developers\">https://github.com/settings/developers</a> and update the Homepage URL and Authorization callback URL for this application to the values below.","explanation":"Then, copy over your Client ID and Client secret below:","clientId":"Client ID:","clientSecret":"Client secret:","saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"adminIdentityProviderConfigureLdap":{"title":"LDAP login configuration","error":"Failed to save changes: %s","url":{"label":"LDAP server URL:","subtext":"e.g. <code>ldap://ldap.example.com:389</code>.  Use port 636 for SSL."},"searchBindDn":{"label":"Bind user DN","subtext":"The DN to bind as when searching for a user in the tree, e.g. <code>cn=admin,ou=users,dc=example,dc=com</code>.  Optional, but needed if your server doesn't allow unauthenticated searches."},"searchBindPassword":{"label":"Bind user password","subtext":"Optional, needed if specifying bind DN above."},"base":{"label":"Base DN","subtext":"The root object under which to search for user account nodes, e.g. <code>ou=users,dc=example,dc=com</code>"},"searchUsername":{"label":"LDAP username attribute","subtext":"The LDAP attribute on a user object which represents the user's username (what they will enter in the username field when signing in)."},"nameField":{"label":"LDAP given name attribute","subtext":"The LDAP attribute on a user object which represents the user's full display name."},"emailField":{"label":"LDAP email attribute","subtext":"The LDAP attribute on a user object which represents the user's email address."},"filter":{"label":"Additional LDAP filter criteria","subtext":"An optional LDAP query fragment that will be included in the user search. This is not commonly used. See <a target=\"_blank\" href=\"http://www.ietf.org/rfc/rfc2254.txt\">RFC2254</a>."},"caCert":{"label":"LDAP CA certificate","placeholder":"-----BEGIN CERTIFICATE-----\nMIID...\n-----END CERTIFICATE-----","subtext":"This is only necessary if your server supports SSL and is using a private CA not in the common trusted set. Note that it does NOT work to enter the LDAP server's own certificate here -- it must be the certificate of a CA which has signed the LDAP server's certificate."},"saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"adminIdentityProviderConfigureSaml":{"title":"SAML login configuration","explanation":"Your SAML IDP should be configured to return a persistent nameID. In addition, if you are not using Active Directory, you <strong>must</strong> configure your IDP to provide two extra attributes, <code>email</code> and <code>displayName</code>.","serviceUrl":"The Service URL of this server is: <code>%s</code>","logoutUrl":"The Service Logout URL of this server is: <code>%s</code>","metadataUrlAt":"If your SAML IDP supports reading the service provider metadata from a URL, then you can point it at: <code><a href=\"__href__\" target=\"_blank\">__text__</a></code>","error":"Failed to save changes: %s","entryPoint":{"label":"SAML provider entry point URL:","placeholder":"https://YOUR_SAML_PROVIDER.com/sso/saml"},"logout":{"label":"SAML provider logout URL (optional):","placeholder":"https://YOUR_SAML_PROVIDER.com/sso/saml/SingleLogoutService"},"publicCert":{"label":"SAML cert for above provider:","placeholder":"base64 encoded cert (e.g. MIID...8F==)"},"entityId":{"label":"Entity ID:"},"saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"adminIdentityProviderTable":{"title":"Login provider","status":"Status"},"adminIdentityRow":{"enabled":"Enabeld","disabled":"Not enabled","baseUrlChanged":{"explanation":"Disabled due to BASE_URL change","reconfigureButton":"reconfigure"},"configureButton":{"hint":"Configure %s login","text":"Configure"}}},"organization":{"name":"Organization settings","subtext":"Smoother onboarding and collaboration for larger teams.","organizationSettingsBlurb":{"explanation":"Sandstorm allows you to define an organization. You can automatically apply some settings to all members of your organization. Users within the organization will automatically be able to log in, install apps, and create grains."},"newAdminOrganization":{"success":"Saved changes.","error":"Failed to save changes: %s","membership":{"title":"Organization membership","emailToggle":{"label":"Users authenticated via email address","subtext":"Note: disabled because email login is not configured."},"emailDomain":{"label":"Domain:","subtext":"Users with an email address at this domain will be members of this server's organization."},"gappsToggle":{"label":"Users authenticated via Google Apps for Work","subtext":"Note: disabled because Google login is not configured."},"gappsDomain":{"label":"Domain:","subtext":"Users with a Google Apps for Work account under this domain will be members of this server's organization."},"ldapToggle":{"label":"Users authenticated via LDAP","subtext":"Note: disabled because LDAP login is not configured."},"samlToggle":{"label":"Users authenticated via SAML","subtext":"Note: disabled because SAML login is not configured."}},"settings":{"title":"Organization settings","disallowGuests":{"label":"Disallow collaboration with users outside the organization.","subtext":"Users will be required to log in as a member of your organization to view sharing links. Guest access will be disabled."},"shareContacts":{"label":"Make all organization users visible to each other","subtext":"This setting automatically adds users within the organization to each other's contact list so that they can share grains with each other.  Disable this if you have some users whose identity should stay hidden from other users."}},"saveButton":"Save"}},"emailConfig":{"name":"Email configuration","subtext":"How email gets sent to users.","newAdminEmailConfig":{"outbound":{"title":"Outbound email","warning":"<strong>Email is unconfigured.</strong> Email-related features like email login, notifications, sharing invitations, and requesting access will be unavailable until you configure email."},"howItWorks":{"title":"How Sandstorm uses email","explanation":"Sandstorm sends email notifications, and Sandstorm apps can send email notifications, too. Every app receives a unique email address, so Sandstorm needs to send email from a variety of \"From:\" addresses. Therefore, Gmail SMTP won't work, but there are other free services. See <a href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">more info in the docs</a>."},"success":"Saved changes.","error":"Failed to save changes: %s","address":{"hostname":"SMTP host","port":"Port","subtext":"Use port 25 for SMTP (unencrypted or with StartTLS), port 465 for TLS-encrypted SMTPS."},"username":{"label":"Username (optional)"},"password":{"label":"Password (optional)"},"returnAddress":{"label":"Sandstorm server's own email address","subtext":"Sandstorm will send login emails, notifications of granted access requests, and other emails from this address. It will also use this address in the SMTP envelope."},"submitButton":"Save Configuration","testButton":"Test","disableButton":"Disable email"},"emailTestPopup":{"title":"Send a test email","testAddress":{"label":"Send test mail to:"},"pending":"Sending...","submitButton":"Send test email","closeButton":"Close"},"emailDisablePopup":{"title":"Really disable email?","explanation":"This will disable all outbound email delivery for this server, including sharing via email, email token login (if enabled), and email notifications to users.","disableButton":"Disable email","closeButton":"Close"}},"personalization":{"name":"Personalization","subtext":"Customize a splash page, terms of service, privacy policy, etc.","newAdminPersonalization":{"serverTitle":{"label":"Server title:","subtext":"The name of this server, as used when communicating with users."},"splashUrl":{"label":"Splash URL (experimental):","subtext":"This web page will be displayed in the background behind the login dialog (the home page when logged out). For security reasons, the page must be hosted within this Sandstorm server's wildcard host (otherwise it will be blocked by <code>Content-Security-Policy</code>). We suggest using a static web publishing app like <a target=\"_blank\" href=\"https://apps.sandstorm.io/app/nqmcqs9spcdpmqyuxemf0tsgwn8awfvswc58wgk375g4u25xv6yh/?host=%s\">Hacker CMS</a> to host content.<strong>WARNING: This feature is experimental; in particular the style and positioning of the login box is subject to change without notice.</strong> Please let us know if you'd like to see it stabilize."},"signupDialog":{"label":"Signup dialog:","subtext":"This text will be displayed to users who click on signup links."},"termsOfService":{"label":"Terms of service URL:","placeholder":"https://example.com/tos","subtext":"If provided, new users will be required to agree to the linked terms of service."},"privacyPolicy":{"label":"Privacy policy URL:","placeholder":"https://example.com/privacy","subtext":"If provided, new users will be required to agree to the linked privacy policy."},"whitelabeling":{"title":"Whitelabeling","logo":{"label":"Custom logo on sign-in dialog","resetButton":"Use default","uploadButton":"Upload logo"},"hideTroubleshooting":{"label":"Hide troubleshooting button"},"hideAbout":{"label":"Hide \"About Sandstorm\" links"},"hideSendFeedback":{"label":"Hide \"Send feedback\" link"},"useServerTitleForHomeText":{"label":"Use server title (above) for home link","subtext":"If enabled, changes the default label \"Sandstorm\" on the link at the very top left to match whatever you specified for the Server title above."},"customLoginProviderName":{"label":"Custom login button text","subtext":"Replaces the text \"with LDAP\" and \"with SAML\" on the login page with the specified text."}},"saveButton":"Save"}},"preinstalledApps":{"name":"Pre-installed apps","subtext":"Choose apps to pre-install for new users.","newAdminPreinstalledApps":{"explanation":"Apps you configure on this page to be pre-installed will automatically be installed (show under /apps) for new users.","productivitySuite":{"label":"Productivity Suite","subtext":"We recommend pre-installing these core apps that are useful to most users."},"systemSuite":{"label":"System apps","subtext":"We recommend pre-installing these system apps that greatly enhance the abilities of Sandstorm."},"allApps":{"label":"All apps on market","subtext":"The checked apps below will be installed by default for all new users.","lessButton":"Show fewer apps","moreButton":"Show %s more apps"},"saveButton":"Save"},"_appRow":{"download":{"success":"Downloaded","failed":"Failed"}}},"appSources":{"name":"App sources","subtext":"Where to look for apps and app updates.","newAdminAppSources":{"appMarketUrl":{"label":"App market URL","subtext":"The \"Install...\" button on the Apps page will link to this URL."},"enableAppUpdates":{"label":"Enable app update notifications","subtext":"Updates to apps will be automatically downloaded by the Sandstorm server daily and notifications sent to users asking if they want to update their installed apps."},"appIndexUrl":{"label":"App index URL","subtext":"This is the URL that the Sandstorm server will check to learn about app updates."},"saveButton":"Save","restoreButton":"Restore defaults"}},"networking":{"name":"Networking","altName":"Security","subtext":"Control how the network is accessed.","newAdminNetworking":{"ipBlacklist":{"label":"Server-side request IP blacklist:","subtext":"Users will be prohibited from making requests to these IP addresses. This includes making a request from an app, downloading an SPK file from a user-provided URL, etc. You may specify one IP address or network (in CIDR notation, e.g. \"127.0.0.0/8\") per line. The default value includes standard local and private network addresses. Note that when an HTTP proxy is in use, this setting may be ignored; the proxy must implement its own blacklist."},"saveButton":"Save","restoreButton":"Restore defaults"}},"users":{"name":"Users","subtext":"Manage all users on this Sandstorm installation."},"userAccounts":{"newAdminUserTableRow":{"admin":{"hint":"Admin","cap":"A"},"user":{"hint":"User","cap":"U"},"visitor":{"hint":"Visitor","cap":"V"},"unknownIdentity":"Unknown credential","unknownIdentifier":"unknown identifier","manageButton":"Manage"},"newAdminUserTable":{"header":{"name":"Name","credentials":"Credentials","created":"Created","lastActive":"Last Active","actions":"Actions"},"inviteLink":"Invite new user","notFound":"No users matched your filter.","processing":"Loading users..."},"newAdminUsers":{"search":{"placeholder":"search"},"filter":{"explanation":"Show:","admin":"Admin","user":"User","visitor":"Visitor"}}},"userDetails":{"newAdminUserDetailsIdentityTableRow":{"profilePicture":{"label":"Profile picture"},"handle":{"title":"Handle","emails":"Emails"},"pronouns":{"hint":"Pronouns","label":"Pronouns"},"emails":{"hint":"Emails","label":"Emails","primary":"Primary","verified":"Verified","unverified":"Unverified","notFound":"No known emails."},"properties":{"login":"Can log in","org":"Org. member"},"unkonwn":{"profilePicture":{"cell":"(unknown name)"},"service":{"hint":"unknown service","label":"(unknown service)","cell":"(unknown name)"},"handle":{"cell":"(unknown handle)"},"pronouns":{"cell":"(unknown pronouns)"},"created":{"cell":"unknown creation date"},"lastActive":{"cell":"unknown activity date"}}},"newAdminUserDetailsIdentityTable":{"header":{"credential":"Credential","created":"Created","lastActive":"Last Active","properties":"Properties"}},"newAdminUserDetails":{"role":{"title":"Role","admin":{"label":"Admin","explanation":"Manages all accounts.  Can install apps, create grains, and administer the server.","isAdmin":"Current role","isNotAdmin":"Promote to admin"},"user":{"label":"User","explanation":"Can install apps and create grains.","isUser":"Current role","isAdmin":"Demote to user","isNotUser":"Promote to user"},"visitor":{"label":"Visitor","explanation":"Can use and re-share shared grains, but cannot create grains of their own nor install apps.","isVisitor":"Current role","isNotVisitor":"Demote to visitor"}},"credentials":{"title":"Credentials"},"profile":{"title":"Profile"},"status":{"title":"Status","deleting":"Suspended - Pending deletion %s","suspended":"Suspended","active":"Active","unsuspendButton":"Unsuspend account","suspendButton":"Suspend account","deleteButton":"Delete account"},"deletePopup":{"title":"Delete %s","explanation":"You are about to mark %s's account for deletion. Once marked for deletion, this account will be suspended, and admins will have 7 days to cancel the pending deletion by returning to this page and clicking the \"Unsuspend Account\" button. %s will <strong>not</strong> be able to stop this suspension/deletion.","detail":"After 7 days, %s's account will be permanently deleted. Deleting this account will:","detail1":"Permanently delete any grains %s created","detail2":"Revoke all sharing links %s created","detail3":"Remove %s from all users' contact lists","detail4":"Delete all credentials owned by this account (except credentials also associated with another account).","deleteButton":"Delete account","cancelButton":"Cancel"},"suspendPopup":{"title":"Suspend %s","detail":"Suspending this user account will:","detail1":"Prevent any grains created by %s from being opened","detail2":"Disable any sharing links created by %s","detail3":"This also applies to all credentials owned by this account (except credentials associated with another account)","suspendButton":"Suspend account","cancelButton":"Cancel"},"loading":"Loading user data..."}},"userInvite":{"newAdminUserInviteLink":{"explanation":"You can invite others to create accounts on your server by giving them an account creation link. People you invite will be granted User permission and be able to install apps and create grains of their own. Alternatively, if you want to share a specific grain with someone, you can do that within the grain's \"Share Access\" menu.","title":"Create a link","invite":{"explanation":"This will create an account creation link which you can send to someone.","createButton":"Create link"}},"newAdminUserInviteEmail":{"title":"Send an email","emailWarning":{"explanation":"Email has not been configured, so sending invites via email is disabled.","configureLink":"Configure email"},"explanation":"This will create a link and send it in an email.","to":{"label":"To:","placeholder":"alice@email.com\nbob@example.com","subtext":"Email addresses, one per line."},"subject":{"label":"Subject:"},"body":{"label":"Body:","subtext":"<code>$KEY</code> will be replaced with the newly-generated link."},"sendButton":"Send invite"},"newAdminUserInvite":{"title":"Invite"}},"systemStatus":{"name":"System log","subtext":"View Sandstorm's debug log.","newAdminLogContents":{"ellipsis":"..."},"newAdminStatus":{"downloadButton":"Download full log"}},"stats":{"name":"Statistics","subtext":"View usage statistics for this server.","header":{"date":"date","users":"users","grains":"grains","appName":"App name","owners":"owners","sharedUsers":"sharedUsers","daily":"daily","weekly":"weekly","monthly":"monthly","forever":"forever"},"body":{"notFound":"There are no stats recorded yet.","loading":"Loading..."},"newAdminStats":{"share":{"title":"Publish anonymously to Sandstorm dev team","explanation":"We'd like to know how many Sandstorm users there are, to make graphs and such.  Is it okay to send the numbers on this page once a day to the Sandstorm team, so that we can add them to the totals?  Any stats you send are anonymous.","enableButton":"Send anonymous stats","disableButton":"Don't send","control":{"explanation":"The developers of Sandstorm like to know how many Sandstorm users there are, to make graphs and such.  You can help us by opting in to sharing the numbers on this page once a day to the Sandstorm team, so that we can add them to our totals. Any stats shared are submitted anonymously.","label":"Anonymous stats reporting to sandstorm.io:","enabled":"Enabled","disabled":"Disabled","disableButton":"Disable","enableButton":"Enable"}},"json":{"title":"View as JSON","explanation":"If you want to integrate Sandstorm stats with an internal stats tool, use this secret link to retrieve the tables below as JSON. You can revoke the current link at any time by generating a new link.","label":"Stats JSON link:","regenerateButton":"Generate new link"},"userGrains":{"title":"Users and grains","current":"Right now: __users__ users, __grains__ grains","today":"Today: __users__ users, __grains__ grains"},"apps":{"title":"Apps","label":"Select a date:"}}},"maintenanceMessage":{"name":"Maintenance message","subtext":"Communicate potential downtime to users.","newAdminMaintenance":{"explanation":"You can set a maintenance message which will be shown to all users in the menu bar, on the top right. This is useful for announcing scheduled maintenance so users can be prepared to tolerate downtime. If the message text is empty, no message will be displayed.","variables":"You may use the following variables in the \"Message text\" field, which will be automatically substituted with useful values:","variable1":"<code>$TIME</code> : The time component of \"Maintenance time\", converted to the user's local time (eg. \"1:15:45 PM\")","variable2":"<code>$DATE</code> : The date component of \"Maintenance time\", converted to the user's local time (eg. \"6/19/2015\")","variable3":"<code>$IN_COUNTDOWN</code> : An active countdown until \"Maintenance time\" (eg. \"in 3 days\" or \"in 1 minute\"). Displays \"any moment\" at the time specified in \"Maintenance time\" and then disappears after an hour.","messageText":{"label":"Message text:","subtext":""},"maintenanceTime":{"label":"Maintenance time:","subtext":"The time you intend to start performing maintenance, formatted like \"%s\"."},"url":{"label":"Link URL (optional):","subtext":"If given, the maintenance message will render as a clickable link to the provided URL."},"saveButton":"Set message"}},"networkCapabilities":{"name":"Network capabilities","subtext":"Manage direct network access for grains.","newAdminNetworkCapabilitiesTableCapabilityRow":{"adminAccount":"Admin account %s","unknownGranter":"Unknown granter","anyone":"Anyone with the token","unknownRecipient":"Unknown recipient","revoked":"Revoked","revokeButton":"Revoke"},"newAdminNetworkCapabilitiesTable":{"header":{"grantBy":"Granted by","grantTo":"Granted to","grantTime":"Time","actions":"Actions"},"notFound":"No capabilities found."},"newAdminNetworkCapabilitiesSection":{"filter":{"label":"Showing:","active":"Active (%s)","revoked":"Revoked (%s)"}},"newAdminNetworkCapabilities":{"networkCaps":"Outbound network access","networkCapsTls":"TLS-encrypted outbound network access","interfaceCaps":"Inbound network access"}},"hostingManagement":{"name":"Hosting management","newAdminHostingManagement":{"explanation":"This feature is experiemental and only works with the LDAP login provider at the moment. Use with caution.","quotaEnabled":{"label":"Enable quota for users","subtext":"Enable Quota management on this server. This requires both SAML and LDAP to be configured, and email addresses will be used to match between the two."},"quotaLdapAttribute":{"label":"LDAP quota attribute","subtext":"This is the attribute name that will be looked up from LDAP to determine quota for a user."},"billingPromptUrl":{"label":"Billing prompt url","subtext":"This is the url that will be used in the billing prompt."},"saveButton":"Save","restoreButton":"Restore defaults"}}},"people":{"title":"People"},"sharingLinks":{"title":"Sharing Links"},"apps":{"title":"Apps","appList":{"uninstallingHint":"Done uninstalling","doneUninstalling":"Done uninstalling","uninstallHint":"Uninstall an app","uninstallButton":"Uninstall...","uploadApp":"Upload app...","searchHint":"Search","search":"search","mostUsed":"Most used","allApps":"All apps","install":"Install...","fromMarket":"from app market","uninstall":"Uninstall","noMatchApp":"You have no installed apps with that name. Try <a href=\"__url__\">searching on the app market &raquo;</a>","unauthorized":"Unauthorized! :(","unauthorizedText":"This is a private Sandstorm server. In order to install apps, you need to be invited by the admin. The admin can invite you from the admin panel.","intro":"<strong>Welcome!</strong><br>Create documents, chat rooms, blogs, and more.<br><br>Get started by installing an app.","doneButton":"Got it"},"install":{"sandstormAppInstallPage":{"title":"Install app","error":{"title":"Failed to install app","retryButton":"Try Again"},"stepWait":{"title":"Waiting for write access..."},"stepDownload":{"title":"Downloading...","cancelButton":"Cancel"},"stepVerify":{"title":"Verifying..."},"stepUnpack":{"title":"Unpacking..."},"stepAnalyze":{"title":"Analyzing..."},"stepConfirm":{"upgrade":{"explanation":"You have an older version of this app installed.  Do you want to upgrade?","button":"Upgrade %s"},"downgrade":{"explanation":"You already have a newer version of this app installed.  Are you sure you want to revert to this older version?","button":"Downgrade %s"},"install":{"button":"Install %s"}},"stepRun":{"title":"%s installed","explanation":"Installation complete.","appDetailsLink":"Go to %s"},"stepDelete":{"title":"Deleting","explanation":"Server is currently deleting this app..."},"stepFailed":{"title":"Failed","explanation":"Installation failed: %s"}}},"appDetails":{"sandstormAppDetailsPage":{"newerVersion":{"explanation":"Some of your files were made with a newer version of this app.  They will keep using the newer version, as going backwards could break them."},"olderVersion":{"explanation":"Some of your grains were made with an older version of this app.  Upgrade them?","upgradeButton":"Upgrade Grains"},"search":{"hint":"Search","placeholder":"search"},"trash":{"explanation":"Grains will be permanently removed after 30 days in the trash. To keep a grain, restore it to the Main list."},"noGrains":{"notFound":"<strong>No matching grains found.</strong>","inTrash":"Some grains in your trash match your search.","trashButton":"View trash"}},"sandstormAppDetails":{"websiteLink":"Website","appMarketLink":"App market","sourceCodeLink":"Source","bugReportLink":"Report issue","mainListButton":"View main list","trashButton":"View trash","uninstallButton":"Uninstall","restoreButton":"Restore grain backup...","version":{"label":"Version"},"lastUpdate":{"label":"Last Updated","unknown":"<unknown>"},"publisher":{"label":"Signed by"},"hasFingerprint":"The app publisher has proven using PGP that they control these accounts.","noFingerprint":"This app has no signature from its creator; its author cannot be verified."}}},"accounts":{"title":"Account settings","accountSettings":{"sandstormAccountSettings":{"title":"Account","identities":{"title":"My Profile","success":"Success: %s","linkedCredentials":"Linked credentials","linkNewIdentity":{"title":"Link new credential:","cancelButton":"Cancel","linkButton":"Link new credential","logoutAllButton":"Log out all other sessions"},"email":{"title":"Primary e-mail address for service-related notifications","primary":"Primary","primaryButton":"Set as primary","notFound":" No verified emails! ","addNew":"Add new e-mail address"}},"billing":{"title":"Billing"},"deleteAccount":{"title":"Delete account","button":"Delete your account"},"deletePopup":{"title":"Delete your account","explanation":"You are about to mark your account for deletion. Once marked for deletion, your account will be suspended, and you will have 7 days to cancel the pending deletion by logging back into your account.","results":"After 7 days, your account will be permanently deleted. Deleting your account will:","result1":"Permanently delete any grains you created","result2":"Revoke all sharing links you created","result3":"Remove you from all users' contact lists","result4":"Delete all credentials owned by your account (except credentials also associated with another account).","confirm":"Please type \"delete my account\" below to confirm.","goodbye":"We're sorry to see you go. Feel free to share any feedback you have with the server admin.","feedbackPlaceholder":"Optional","deleteButton":"Delete account","cancelButton":"Cancel"}},"sandstormAccountsFirstSignIn":{"title":"Confirm your profile","success":"Success: %s"},"accountSuspended":{"title":"This account has been suspended by you.","deletedIn":"This account will be deleted in %s. Would you like to cancel deletion and restore this account?","suspended":"This account is suspended and all of its grains and shares are disabled. Would you like to restore this account?","restoreButton":"Restore account","logoutButton":"Logout"},"_accountProfileEditor":{"picture":{"label":"Profile picture:","uploadButton":"Upload new picture","subtext":"This picture will be shown to other users to help identify you.","maxSize":"512x512px, 64KiB max."},"fullName":{"label":"Full name:","placeholder":"full name","subtext":"Your regular first and last name, as you would like it displayed to others. This does not have to be your real name."},"handle":{"label":"Handle:","placeholder":"handle","hint":"Handles must start with a lower-case letter and contain only lower-case letters, digits, and underscores.","subtext":"To be displayed in apps that use handles. Your handle must start with a letter and contain only lower-case letters, digits, and underscores."},"emailList":{"label":"E-mail:"},"pronoun":{"label":"Preferred pronoun:","neutral":"they (unspecified)","male":"he/him (male)","female":"she/her (female)","robot":"it (robot)"},"agreedToTerms":{"agreeTo":" I have read and agree to the","termsOfServiceLink":"Terms of Service","and":" and ","privacyLink":"Privacy policy"},"saveButton":"Save","continueButton":"Continue","cancelButton":"Cancel"}},"loginButtons":{"icon":{"info":"i"},"loginButtons":{"createButton":"Create account","signInButton":"Sign in"},"accountButtons":{"loading":"Loading..."},"_loginButtonsLoggedInDropdown":{"revealIdentity":"Reveal identity","title":"Account","sendFeedback":"Send feedback","referralProgram":"Referral program","signOutButton":"Sign out"},"_loginButtonsLoggedOutDropdown":{"demo":{"title":"Create account","explanation":"Your demo data will be transferred to your new account."},"title":"Sign in","shared":{"explanation":"Sign in so that collaborators know who you are."},"signInButton":"Sign in","anonymousButton":"Stay anonymous"},"oauthLoginButton":{"with":"with "},"devLoginForm":{"label":"with a Dev account","hint":"unexpand","alice":"Alice (admin)","bob":"Bob","carol":"Carol","dave":"Dave","customPlaceholder":"Eve","customButton":"Custom"},"loginButtonsList":{"configure":"configure login","troubleshooting":"troubleshooting","about":"about Sandstorm"},"emailLoginForm":{"description":"with e-mail","sendButtonText":"send Login Email"},"emailAuthenticationForm":{"hint":"Passwordless E-mail Login allows you to log in using an e-mail address. No password is required; instead, every time you log in, we send you a new email confirmation code. You will only need to log in once per device.","token":{"explanation":"We've sent a login e-mail to %s. It may take a few moments to show up in your inbox.","label":"Enter your token:","confirmButton":"Confirm","cancelButton":"Cancel"},"disabled":"A server admin must enable email authentication before you can confirm your email."},"ldapLoginForm":{"username":{"label":"username","hint":"This is your LDAP username. It will be the name that you normally use to log in to other internal services at your company."},"password":{"label":"password","hint":"This is your LDAP password. It will be the password that you normally use to log in to other internal services at your company."},"loginButton":"Log in"},"_loginButtonsSeparator":{"or":"or"}},"identity":{"identityLoginInterstitial":{"title":"Logging in...","loading":"loading...","logoutButton":"Log out","cancelButton":"Cancel","goBackButton":"Go back","create":"creating account...","createFailed":"Failed to create account.","loginAccount":"logging in to account...","loginAccountFailed":"Account login failed","loginToken":"logging back in to account...","loginTokenFailed":"Failed to resume session.","link":"linking credential to account...","success":"Unlinked credential from %s","explanation":"You have authenticated as this credential","warning":"but that is not a login credential for any account.","moreIdentities":"The following credentials can log in to your credential's accounts:","unlinkButton":{"hint":"Unlink your credential from this account.","text":"unlink"},"unlink":{"explanation":"Are you sure you want to unlink your credential from the account of %s?","unlinkButton":"Unlink","cancelButton":"Cancel"}},"identityManagementButtons":{"toggleLogin":{"label":"Allow this credential to log in to your account.","unlinkButton":"Unlink"}},"loginIdentitiesOfLinkedAccounts":{"show":{"explanation":"This credential is shared with other accounts. Those accounts have the following login credentials:","unlinkButton":"unlink","hideButton":"hide other accounts"},"hide":{"explanation":"This credential is shared with at least one other account.","showButton":"show other accounts"}},"identityCardSignInButton":{"hint":"sign in as %s","title":"Sign in"},"accountCardSignInButton":{"reSign":"To sign out of your current account and sign in as this user, choose how to sign in:","howToSignIn":"Choose how to sign in:"}},"tokenTemplates":{"_emailTokenError":{"title":"Invalid authentication code. If it has been more than 15 minutes, the token has expired, otherwise you may have simply mistyped it."},"_emailLinkIdentityError":{"title":"To confirm your email address, you must be logged in as the account that created this link."},"addNewVerifiedEmailPowerboxConfiguration":{"email":{"placeholder":"enter email address","selectButton":"Select","sendButton":"Send confirmation email"},"sendingTo":"Sending to %s","resetButton":"Try a different address","token":{"placeholder":"enter your token","confirmButton":"Confirm"},"verifying":"Verifying token..."}}},"wizard":{"setupWizardLoading":{"loading":"Loading..."},"setupWizardProgressBar":{"identityProviders":"Login providers","organization":"Organization","email":"Email","preinstalledApps":"Pre-installed apps","adminAccount":"Admin account"},"setupWizardHelpFooter":{"needHelp":"Need help?","documentation":"Documentation","emailSupport":"Email support","systemLog":"System Log"},"setupWizardSystemLog":{"header":{"label":"System log","downloadButton":"Download log"},"permissionDenied":"You cannot view the system log because you are not logged in as an admin and you do not have a setup token."},"setupWizardIntro":{"title":"Welcome to Sandstorm!","revisitWizardLink":"Revisit setup wizard","noIdpEnabled":{"explanation":"This Sandstorm server has been set up previously, but due to a change in <code>sandstorm.conf</code>, you'll need to configure login providers again."},"idpEnabled":{"explanation":"This Sandstorm server is ready to roll. You can adjust server settings, including which providers are enabled and how Sandstorm sends email.","turnUserIntoAdmin":"You can also turn an existing user into an administrator.","isAdmin":{"adminPanelLink":"Proceed to admin panel","suButton":"Give yourself admin privileges "},"isNotAdmin":{"explanation":"To make an account into an admin account, please sign in:","signInLabel":"Sign in","signInButton":"Sign in to become admin"}},"setupButton":"Begin Sandstorm Setup"},"setupWizardIdentity":{"title":"Login providers","explanation1":"To use Sandstorm, you need to create a user account.  Every user account on Sandstorm is backed by an login provider.  You'll use this login provider to authenticate as the first administrator of this Sandstorm install.","explanation2":"Configure the login provider or providers you wish to enable.","nextButton":"Next","backButton":"Back"},"setupWizardOrganization":{"title":"Organization settings","error":"Failed to save change: %s","membership":{"title":"Organization membership","emailToggle":{"label":"Users authenticated via email address","subtext":"Note: disabled because email login is not configured."},"emailDomain":{"label":"Domain:","subtext":"Users with an email address at this domain will be members of this server's organization."},"gappsToggle":{"label":"Users authenticated via Google Apps for Work","subtext":"Note: disabled because Google login is not configured."},"gappsDomain":{"label":"Domain:","subtext":"Users with a Google Apps for Work account under this domain will be members of this server's organization."},"ldapToggle":{"label":"Users authenticated via LDAP","subtext":"Note: disabled because LDAP login is not configured."},"samlToggle":{"label":"Users authenticated via SAML","subtext":"Note: disabled because SAML login is not configured."}},"settings":{"title":"Organization settings","disallowGuests":{"label":"Disallow collaboration with users outside the organization.","subtext":"Users will be required to log in as a member of your organization to view sharing links. Guest access will be disabled."},"shareContacts":{"label":"Make all organization users visible to each other","subtext":"This setting automatically adds users within the organization to each other's contact list so that they can share grains with each other.  Disable this if you have some users whose identity should stay hidden from other users."}},"saveButton":"Save and continue","backButton":"Back"},"setupWizardEmailConfig":{"title":"Email delivery","explanation":"Sandstorm needs a way to send email.  You can skip this step (unless you're using email login), but email-related features will be unavailable until you configure email in the future.","moreDetail":"<strong>How Sandstorm uses email:</strong> Sandstorm sends email notifications, and Sandstorm apps can send email notifications, too. Every app receives a unique email address, so Sandstorm needs to send email from a variety of \"From:\" addresses. Therefore, Gmail SMTP won't work, but there are other free services. See more info in <a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">the docs</a>.","error":"Failed to save changes: %s","hostPort":{"host":"SMTP host","port":"Port","subtext":"Use port 25 for SMTP (unencrypted or with StartTLS), port 465 for TLS-encrypted SMTPS."},"username":{"label":"Username (optional)"},"password":{"label":"Password (optional)"},"email":{"label":"Sandstorm server's own email address","subtext":"Sandstorm will send login emails, notifications of granted access requests, and other emails from this address. It will also use this address in the SMTP envelope."},"saveButton":"Save and continue","skipButton":"Skip for now","testButton":"Test","backButton":"Back"},"setupWizardPreinstalled":{"title":"Pre-installed Apps Setup","explanation":"Please wait while Sandstorm installs the following Productivity Suite apps that are useful for most users. You will be able to configure all pre-installed apps in the Admin Settings panel after setup.","downloaded":"Downloaded","failed":"Failed","loading":"Please wait, loading app index...","nextButton":"Next","skipButton":"Skip for now","backButton":"Back"},"setupWizardLoginUser":{"title":"Create admin account","firstLogin":{"title":"Confirm your profile"},"isAdmin":"Great! Your account has admin privileges.","hasAdmin":{"explanation":"This Sandstorm server already has an admin user, but if you want, you can also make yourself an admin.","adminButton":"Become an admin"},"makeYouAnAdmin":"Making you an admin on this server...","loggedOut":{"explanation":"You can now create an admin account on this Sandstorm server.","description":"Log in to create your admin account","dialogLabel":"Create account"},"finishButton":"Finish","backButton":"Back"},"setupWizardSuccess":{"title":"Ready to roll!","serverIsReady":"Your Sandstorm server is now ready to use.","whatsNext":"What would you like to do next?","addUsersLink":"Add users","editSettingsLink":"Edit other settings","sandstormLink":"Start using Sandstorm","backButton":"Back"},"setupWizardVerifyToken":{"title":"Verifying token...","explanation":"Sorry, it looks like your setup token has expired.","howToFix":"Generate a new one now from the command line by running <code>sandstorm admin-token</code> to access the setup wizard."},"setupWizardTokenExpired":{"title":"Setup token required","alreadySetup":"This server is already set up, so we recommend administering it via the main","adminRootLink":"administration panel","explanation":"To use this server's setup page, you need a valid setup token.","howToFix":"Generate a new one now from the command line by running <code>sandstorm admin-token</code> and following the link it outputs."}},"packages":{"topbar":{"updated":"Sandstorm has been updated — Click here to reload","apps":"Apps","grains":"Grains","oasis":"Start collaborating with others by using Sandstorm on Oasis:","createAccount":"Create free account »","demoHint":"Your demo data will be transferred.","selfHostHint":"Or, install Sandstorm for your team (it's open source):","signUp":"Sign up within:","close":"Close"},"powerbox":{"title":"Powerbox request","inputToken":"Please input an api token:","importToken":"Import token","selected":"Selected:","selectOne":"Select one:","noGrain":"No grains can provide the requested interface.","loading":"Loading..."}}};
TAPi18n._loadLangFileObject("en", translations);
TAPi18n._registerServerTranslator("en", namespace);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"nl.i18n.json":function(require){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// i18n/nl.i18n.json                                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _ = Package.underscore._,
    package_name = "project",
    namespace = "project";

if (package_name != "project") {
    namespace = TAPi18n.packages[package_name].namespace;
}
TAPi18n.languages_names["nl"] = ["Dutch","Nederlands"];
if(_.isUndefined(TAPi18n.translations["nl"])) {
  TAPi18n.translations["nl"] = {};
}

if(_.isUndefined(TAPi18n.translations["nl"][namespace])) {
  TAPi18n.translations["nl"][namespace] = {};
}

_.extend(TAPi18n.translations["nl"][namespace], {"error":"Fout: %s","shell":{"layout":{"demoExpired":{"title":"Demo verlopen","canUpgrade":"Bedankt voor het uitproberen van de Sandstorm Demo! Uw demo accaunt is verlopen. Als u Sandstorm wilt blijven gebruiken en uw data permanent op wilt slaan, log dan alstublieft in.","cannotUpgrade":"Bedankt voor het uitproberen van de Sandstorm Demo! Uw demo accaunt is verlopen.","signOut":"Uitloggen"},"adminAlert":{"icon":"!","title":"Bericht van de administrator"},"demoModal":{"header":"Sandstorm is en zelf hostbare online werkomgeving.","withAppDemo":"met apps zoals %s.","withoutAppDemo":"Met alle apps die u nodig heeft op een plaats.","startButton":{"withAppDemo":"Probeer %s »","withoutAppDemo":"Start »"},"explanation":"<p>Sandstorm maakt het makkelijk om apps op uw eigen server te draaien. Naast het verbeteren van privacy en controle, <a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\" target=\"_blank\">brengt dit Open Source apps naar een groter publiek.</a></p>","securityOverview":"<p>Sandstorm is daarnaast ook belachelijk veilig. Als u beveiliging interessant vindt, kijk dan eens naar<a href=\"https://docs.sandstorm.io/en/latest/developing/security-practices/\" target=\"_blank\">een overzicht van Sandstorm's beveiligingsmodel.</a>.</p>"}},"notifications":{"hint":"Meldingen","text":"Meldingen"},"notificationsPopup":{"title":"Meldingen","empty":"Geen meldingen..."},"loading":{"title":"Laden...","text":"Laden..."},"notFound":{"title":"Niet gevonden","text":"404 Niet gevonden :("},"root":{"title":"Home"},"grainLogContents":{"prefix":"..."},"grainLog":{"title":"Debug log: %s"},"signup":{"title":"Uitnodiging opeisen","signedUp":{"keyIsUsedWithOrigin":"U bent helemaal ingeschreven!  <a href=\"https://apps.sandstorm.io/?host=%s\">Installeer wat apps</a>.","keyIsNotUsed":"Het ziet ernaar uit dat u al een account heeft, dus u heeft deze inschrijvingscode niet nodig."},"keyIsUsed":"Sorry, deze inschrijvingscode is al gebruikt.","processing":"Een ogenblik geduld alstublieft...","signInGuide":"U moet eerst nog inloggen. Klik rechtsbovenin op \"Inloggen\".","invalidKey":"Sorry, dit is niet een geldige inscrhijvingscode."},"uploadStatus":{"title":"Uploaden...","uploadFailedWithError":"Upload mislukt: %s %s","statusWithProgress":"%s... %s","status":"%s...","uploadFailed":"Upload mislukt. Probeer het alstublieft opnieuw."},"uploadTest":{"title":"Upload test pagina","explanation":"Deze pagina is bedoeld voor gebruik door de test scripts.","uploadApp":"Upload app:","uploadBackup":"Upload backup:"},"referrals":{"title":"Verwijzingsprogramma","header":{"isPaid":{"title":"Krijg tot wel <span class=\"reward-amount\">30 GB</span> aan gratis opslagruimte wanneer uw vrienden delen op Oasis!","explanation":"Voor elke vriend die een deellink aanmaakt krijgt u 2 GB"},"isNotPaid":{"title":"Krijg een onbeperkte hoeveelheid korrels &amp; tot wel <span class=\"reward-amount\">2 GB</span> aan gratis opslagruimte wanneer uw vrienden delen op Oasis!","explanation":"Voor elke vriend die een deellink aanmaakt krijgt u 50 MB en daarnaast een onbeperkte hoeveelheid korrels."}},"howTo":{"title":"Delen is verwijzen","explanation":"Om te beginnen, open een korrel en klik op Toegang delen.","getStart":{"step1":"U deelt een korrel met een vriend (die nog geen Oasis gebruiker is).","step2":"Uw vriend logt in.","step3":"Uw vriend deelt een korrel met een ander (waaronder uzelf)."}},"status":{"title":"Uw verwijzingen & status","explanation":"Wanneer u een deel link verzend verschijnen mensen hieronder wanneer ze inloggen.","pending":{"title":"Moet nog delen.","explanation":"Vertel deze mensen om een korrel aan te maken en die met u te delen. U moet de korrel ook bekijken!"},"success":{"title":"Succesvolle verwijzingen","explanation":"U heeft deze mensen met succes verwezen, gefeliciteerd!"}},"finePrint":{"title":"De kleine lettertjes","warningWithAccountLink":"Uw verwijzingsbonus is voor 1 jaar na de verwijzing geldig. De voorwaarden van dit verwijzingsprogramma kunnen veranderen zonder waarschuwing, bonussen die u al heeft ontvangen worden daar niet door beïnvloed. Als u nog meer ruimte nodig heeft, <a href=\"%s\">upgrade dan uw account.</a>","explanation":"Betaalde abonnees kunnen tot 30 GB gratis opslagruimte krijgen, 2 GB per persoon die ze verwijzen, naast de hoeveelheid opslag van hun account niveau. Eigenaren van gratis accounts kunnen een onbeperkte hoeveelheid korrels en 50MB opslagruimte per persoon die ze verwijzen verdienen, tot in totaal 2GB."}},"about":{"title":"Over Sandstorm","intro":{"explanation":"<a href=\"https://sandstorm.io\">Sandstorm.io</a> is een open source hosting platform voor persoonlijke/privé instanties van web apps. Gebruikers kunnen software uploaden en installeren via een simpele app-store achtige interface. Naast betere privacy <a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\">is dit de enige manier om Open Source web apps haalbaar te maken.</a>","version":"Versie: %s","unofficialWarning":"<strong>WAARSCHUWING:</strong> deze server draait een onofficiële build van Sandstorm die aanpassingen kan bevatten die niet zijn gecontroleerd door de ontwikkelaars."},"changelog":{"title":"Recente aanpassingen"},"developers":{"title":"Hoofdontwikkelaars"},"advisors":{"title":"Adviseurs"},"corporate":{"title":"Zakelijke sponsors","explanation":"Deze bedrijven hebben veel bijgedragen aan Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaigne</a>."},"individual":{"title":"Belangrijke individuele sponsors","explanation":"Deze mensen hebben een grote bijdrage geleverd aan Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaigne</a>."},"backers":{"title":"Backers","explanation":"van <a href=\"http://igg.me/at/sandstorm\">onze Indiegogo campaigne &gt;</a>","anonymous":"en %s anonieme bijdragers","loading":"Laden..."},"dependencies":{"title":"Gebruikte werken van derde partijen","license":"licentie","explanation":"Deze werken zijn eigendom van de respectieve auters en worden gebruikt onder de voorwaarden van hun respectieve licenties"},"copyright":{"title":"Copyright","license":"De Sandstorm implementatie is copyright van Sandstorm Development Group, Inc. en bijdragers. De implementatie is uitgebracht onder de Apache License versie 2.0","trademark":"\"Sandstorm\" en het Sandstorm logo zijn trademarks van Sandstorm Development Group, Inc., en mogen niet gebruikt worden voor het adverteren van afgeleide werken of commerciële diensten zonder geschreven toestemming van Sandstorm Development Group, Inc."},"terms":{"license":"Licentie","terms":"Gebruiksvoorwaarden","privacy":"Privacy beleid"}}},"grains":{"title":"Korrels","grainView":{"notFoundWithId":"Geen korrels gevonden met ID %s.","openWithWhichIdentity":"Wilt u uw identiteit openbaren aan deze korrel of anoniem blijven?","openInIncognitoMode":"Open in Incognito Mode","revealMyIdentity":"Maak mijn identiteit bekend"},"grainInMyTrash":{"explanation":"Deze korrel zit in uw vuilnisbak. U kunt er geen toegang tot krijgen tenzij u deze hersteld naar uw hoofdlijst","restore":"Herstellen naar hoofdlijst"},"grainInOwnersTrash":{"explanation":"U kunt geen toegang meer krijgen tot deze korrel omdat de eigenaar het naar de vuilnisbak heeft verplaatst."},"grainOwnerSuspended":{"explanation":"U kunt geen toegang meer krijgen tot deze korrel omdat het account van de eigenaar is geschorst."},"wrongIdentity":{"explanation":"Deze link was bedoeld voor de gebruiker: "},"requestAccess":{"permissionDenied":"U heeft geen toestemming om toegang te krijgen tot deze korrel","requestAccessHint":"Toegang aanvragen","requestAccess":"Toegang aanvragen","warnIdentityRevealing":"Toegang aanvragen maakt uw identiteit bekend aan de eigenaar van deze korrel.","sendRequest":"Uw aanvraag verzenden...","waitForApproving":"Uw aanvraag is verzonden, u zult een email ontvangen als uw aanvraag wordt goedgekeurd.","requestFailed":"Uw aanvraag is mislukt omdat: %s","signInPlease":"Log alstublieft in om toegang aan te vragen"},"invalidToken":{"invalidToken":"Ongeldig kenteken: %s"},"revokedShareLink":{"linkRevoked":"Sorry, deze link is ingetrokken."},"grainTitle":{"was":"was: %s","renamedFrom":"hernoemd van: %s"},"grainDeleteButton":{"hint":"Verplaatsen naar vuilnisbak","text":"Verplaatsen naar vuilnisbak"},"grainDebugLogButton":{"hint":"Laat debug log zien","text":"Log"},"grainBackupButton":{"hint":"Download Backup","text":"Backup"},"grainBackupPopup":{"title":"Download Backup","warning":"Waarschuwing! Het herstellen van deze backup werkt misschien niet op de manier hoe u verwacht.","processing":"Backup voorbereiden...","cancelHint":"annuleren","cancel":"Annuleren","confirmHint":"download backup","confirm":"Download backup"},"grainRestartButton":{"hint":"Herstart App","text":"Herstart"},"grainApiTokenButton":{"hint":"Websleutel opvragen","text":"Websleutel&nbsp;opvragen"},"grainApiTokenPopup":{"title":"Websleutels","processing":"Genereren...","cancel":"Annuleren","copyForExternalApp":"Kopieer deze websleutel en geeft het aan de externe app:","back":"Ga terug","description":"Een websleutel stelt u in staat om een externe app (bijvoorbeeld een mobile app) te verbinden met deze app","labelTitle":"Label:","labelPlaceholder":"b.v. 'Android app'","roleTitle":"Rol:","permissionTitle":"volledige toegang","create":"Aanmaken","webkeyListTitle":"Uw bestaande websleutels:","revokeHint":"Intrekken","revoke":"Intrekken"},"grainShareButton":{"hint":"Toegang tot deze korrel delen","text":"Toegang delen"},"whoHasAccessPopup":{"title":"Wie heeft er toegang","noLoggedInUsers":"Er zijn nog geen ingelogde gebruikers die uw links hebben bezocht.","nobody":"U heeft aan niemand toegang verleend.","addedBy":"toegevoegd door","revokeHint":"Intrekken","revoke":"Intrekken","hasCustomPermissions":"[heeft aangepaste rechten]"},"selectRole":{"hint":"selecteer een rol"},"emailInviteTab":{"demoModeAlert":"Deel emails kunnen niet worden verzonden in demo mode. Maak een account aan om deel emails te versturen, of maak een deellink aan.","personalMessagePlaceholder":"Type een persoonlijk bericht (optioneel)...","send":"Versturen","sending":"Versturen...","success":"Succesvol verstuurd.","reset":"Stuur meer uitnodigingen","startOver":"<< Begin opnieuw"},"shareableLinkTab":{"withRoles":"Iedereen met deze link:","withoutRoles":"Iedereen met deze link heeft toegang tot deze korrel","labelPlaceholder":"Label (optioneel)","labelExplanation":"Gebruik een label om uzelf te herinneren naar wie u deze unieke link hebt verstuurd.","create":"Aanmaken","creating":"Genereren...","success":"Kopieer deze link","reset":"Maak een andere link aan","cancel":"Annuleren"},"shareWithOthers":{"title":"Deel met anderen","sendInviteTab":"Stuur een uitnodiging","shareableLinkTab":"Maak een deellink","whoHasAccess":"Zie in wie toegang heeft"},"grainSharePopup":{"oldSharingModelDescription":"Deze korrel maakt gebruik van het oude deel model. U kunt het delen door de URL uit de locatiebalk te kopieren en plakken.","upgradeDescription":"U kunt deze korrel upgraden naar het nieuwe model, maar de mensen met wie u deze korrel eerder heeft gedeeld zullen hun toegang verliezen; u zult de korrel opnieuw met hen moeten delen.","upgradeWarning":"Upgraden kan niet ongedaan worden gemaakt.","upgrade":"upgrade","openInIncognitoMode":"U kijkt anoniem naar deze korrel","toShareAccess":"Kopieer deze link om toegang te delen:","openNonAnonymouslyHead":"Voor meer deel opties moet u","openNonAnonymouslyLink":"deze korrel niet-anoniem openen","openNonAnonymouslyTail":".","signIn":"Voor meer deel opties, log alstublieft in."},"grainPowerboxRequest":{"powerboxRequestHint":"Powerbox aanvraag","powerboxRequest":"P"},"grainPowerboxOffer":{"powerboxOfferHint":"Powerbox aanbod","powerboxOffer":"P"},"grainPowerboxOfferPopup":{"title":"Powerbox Aanbod","dismiss":"Sluiten"},"share":{"grainNotFoundWithId":"Geen korrel gevonden met ID %s. Misschien moet u met een ander account inloggen?","signIn":"U moet ingelogd zijn om toegang te verlenen tot deze korrel"},"grainlist":{"sandstormGrainListPage":{"introButton":{"hint":"Wat is een korrel?"},"mainListButton":"Bekijk hoofdlist","trashButton":"Bekijk vuilnisbak (%s)","restoreButton":"Backup herstellen...","search":{"placeholder":"zoeken"},"usageInfo":{"quota":"__count__ van __quota__ korrels","size":"__size__ van __total__ gebruikt"},"trash":{"emptyButton":"Vuilnisbak leegmaken","explanation":"(Korrels worden na 30 dagen permanent verwijdert uit de vuilnisbak. Herstel een korrel naar de Hoofdlijst om deze te behouden.)"},"noGrains":{"notFound":"<strong>Geen overeenkomende korrels gevonden.</strong>","inTrash":"Sommige korrels in uw vuilnisbak komen overeen met uw zoekopdracht.","trashButton":"Vuilnisbak bekijken","empty":"<strong>Vuilnisbak is leeg.</strong>","explanation":"U heeft nog geen korrels.","hasApps":"Selecteer een app en maak uw eerste korrel aan.","hasNoApp":"Installeer een app om uw eerste korrel aan te maken."},"moveToTrash":"Verplaats naar vuilnisbak"},"sandstormGrainTable":{"bulkAction":{"disabledHint":"Om bulkacties te verrichten moet u eerst een aantal korrels in de lijst selecteren."},"tableHeader":{"title":"Naam","size":"Grootte","lastUsed":"Laatste activiteit","ownership":"Van mij/Gedeeld","collaborators":"Bijdragers"},"tableBody":{"was":"(was: %s)","renamedFrom":"(hernoemd van: %s)","new":"Ongeopend","mine":"Mijn korrel","shared":"Met mij gedeeld"}},"contactAutocomplete":{"contactInputBox":{"placeholder":"Voer email adressen of namen in"}}}},"notifications":{"appUpdateNotificationItem":{"title":"App updates zijn beschikbaar","appUpdates":"<strong>__name__</strong>: versie __version__ is beschikbaar","submitButton":{"hint":"Update alle bovenstaande apps","text":"Updates toepassen"},"dismissButton":{"hint":"Negeer de bovenstaande updates","text":"Sluiten"}},"mailingListBonusNotificationItem":{"explanationWithBonus":"Abbonneer op onze nieuwsbrief en krijg <strong>%s bonus opslagruimte</strong> (1 tot 2 emails per maand).","submitButton":"Abboneren","dismissButton":"Nee bedankt"},"referralNotificationItem":{"paidUser":"Krijg tot wel 30 GB extra met het","unpaidUser":"Onbeperkte korrels & tot 2 GB extra met het","link":"verwijzingsprogramma »","dismissButton":"Sluiten"},"adminNotificationItem":{"explanation":"U kunt Sandstorm helpen door ons een aantal gebruikstatistieken te versturen. Klik hier voor meer informatie."},"backgroundedGrainNotificationItem":{"title":"<strong>__grain__</strong> draait in de achtergrond:","cancelButton":{"hint":"Stop de app in de achtergrond","text":"Annuleren"}},"identityChangesNotificationItem":{"explanation":"We zijn de manier waarop identiteiten in Sandstorm werken. Uw account zou hier door beïnvloed kunnen zijn. klik hier voor meer informatie."},"grainActivityNotificationItem":{"title":"<strong>__sender__</strong> in <strong>__grain__</strong>:","dismissButton":{"hint":"Sluit deze melding","text":"Sluiten"}}},"powerboxBuiltins":{"uiViewPowerboxConfiguration":{"explanation":"met rechten:","fullAccess":"Volledige toegang","connectButton":"Verbinden","chooseHostedObjectButton":"Verken korrel inhoud »"},"ipNetworkPowerboxCard":{"explanation":"Admin: sta al het uitgaande netwerkverkeer toe","encryptedWith":"(versleuteld met %s)"},"ipInterfacePowerboxCard":{"explanation":"Admin: sta al het binnenkomende netwerkverkeer toe"},"emailVerifierPowerboxCard":{"verifyWith":"Verifieer e-mail adressen door middel van %s","verifyWithAnyService":"Verifieer e-mail adressen door middel van elke login service"},"identityPowerboxConfiguration":{"explanation":"met rechten:","fullAccess":"Volledige toegang","connectButton":"Verbinden"},"addNewVerifiedEmailPowerboxCard":{"explanation":"Voeg een nieuw adres toe"},"httpUrlPowerboxCard":{"useUrl":"Gebruik %s"},"httpArbitraryPowerboxCard":{"explanation":"Specificeer URL..."},"httpArbitraryPowerboxConfiguration":{"explanation":"Voer URL in:","placeholder":"https://example.com","connectButton":"Verbinden"},"httpOAuthPowerboxCard":{"useService":"Gebruik %s"},"httpOAuthPowerboxConfiguration":{"requstPermissionsFromService":"Rechten aanvragen van %s (zie popup)."}},"styleguide":{"title":"Sandstorm style guide","note":"(note: this is a work in progress.)","copyRules":{"title":"Copy rules","rule1":"Be terse but clear.  Avoid technical terms where less-technical ones will do.","rule2":"Capitalize the first letter of the first word of a section heading or button.  Do not capitalize the second word, or anything else, unless referring to a proper noun like Google or GitHub.","rule3":"Use verbs for text on buttons. The button text should describe what clicking that button will do.","rule4":"Use nouns for text which is a link.  The link text should indicate what the link will take you to.","rule5":"Use nouns for basically everything else, too, to communicate the information architecture. Section headings should be nouns which describe the content of the section.  Labels should be nouns which describe the content of the field."},"styleRules":{"title":"Style rules","rule1":"Stick to a limited color palette.  (TODO: add specific colors)","rule2":"Use typefaces no smaller than 13px for text content.  Readability is important and not everyone has young eyes.","rule3":"Make click targets no smaller than 32px square or 26px tall if appreciably wider.  Remember that due to Fitts' Law, larger click targets are easier to interact with."},"forms":{"title":"Forms","rule1":"Use the <code>standard-form</code> class where appropriate.","rule2":"The primary action button of a form should be at the bottom right of the form. Secondary buttons should line up to the left of that primary action button.","rule3":"When providing text input fields, place labels above the input, not side-by-side or inside the field as placeholder text.  Place additional explanation or instructions below the input. Placeholder text should be used for examples, not instructions.","rule4":"Forms are generally used for transactional interactions, where the user cares about the success or failure of the application.  Avoid optimistic UI in favor of multi-state UI which communicates network activity, success, and failure.  A common pattern is:","rule4-1":"Have the default state of the form.  Disable a save button until the user makes a change.","rule4-2":"When the user clicks an action button, start the RPC, and disable the entire form to indicate that the action is underway.","rule4-3":"When the RPC completes, if the RPC failed, show an error message at the top of the context over which the action applies.  If the RPC succeeded, either indicate a success message (again, at the top of the context over which the action applies) or if this was a modal interaction and the completion of the RPC indicates successful completion of the workflow, dismiss the form with no additional message."},"example":{"title":"An example:","label":"Email","placeholder":"alice@example.com","subtext":"You might want to receive notifications.","submitButton":"Save","discardButton":"Discard"}},"admin":{"title":"Admin","admin":{"newAdmin":{"title":"Admin panel","wildcardHostBroken":"WAARSCHUWING: het lijkt erop dat de WILDCARD_HOST van deze server niet goed is ingesteld. Totdat u dit probleem oplost kunt u geen gebruik maken van apps. U kunt meer informatie lezen in <a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#why-do-i-see-an-error-when-i-try-to-launch-an-app-even-when-the-sandstorm-interface-works-fine\"> de Sandstorm documentatie</a> en in de JavaScript console van uw browser. U moet DNS en SSL/TLS certificaten aanpassen, of het sandstorm.conf bestand bewerken. Als u geen informatie in de JS console kunt zien of wilt testen of u het probleem hebt opgelost, herlaad dan deze pagina om de test opnieuw uit te voeren. als u nogsteeds problemen heeft <a href=\"https://github.com/sandstorm-io/sandstorm/issues\">open dan alstublieft een issue..</a>","websocketBroken":"WAARSCHUWING: deze server lijkt niet in staat te zijn om een WebSocket aan te maken. Sandstorm zal lijken te werken zonder problemen, maar veel apps zullen niet goed worden uitgevoerd. <a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#how-do-i-enable-websockets-proxying-or-why-do-some-apps-seem-to-crash-reload\">Meer informatie.</a> Meestal wordt dit veroorzaakt door een reverse proxy zoals nginx of Apache die opnieuw geconfigureerd moet worden. Als u denkt dat dit een fout is, herlaad dan deze pagina of <a href=\"https://github.com/sandstorm-io/sandstorm/issues\">open een issue.</a>","hasSetupToken":"U bent niet ingelogd als een administrator, maar u heeft wel een setup token. U kunt <a href=\"/setup\">de setup wizard bezoeken</a> of <a href=\"/admin/status\">de systeemlog bekijken</a>.","hasNoSetupToken":"U bent niet ingelogd als een administrator. Log in als een administrator om het administrator panel te bezoeken of genereer een token vanaf de opdrachtregel door `sandstorm admin-token` uit te voeren."},"newAdminRoot":{"configuration":{"title":"Configuratie"},"management":{"title":"Beheer"}}},"identityProviders":{"name":"Inlogproviders","providers":{"emailPassLess":"E-mail (wachtwoordloos)"},"subtext":"Hoe gebruikers inloggen.","newAdminIdentity":{"explanation":"Kies welke diensten gebruikers mogen gebruiken om zich te identificeren. Iedereen kan inloggen, maar alleen de gebruikers die u uitnodigt kunnen apps installeren of korrels aanmaken.","organizationLinkPrefix":"Om organisatie lidmaatschap en instellingen te configureren, bekijk","organizationLink":"organisatie instellingen","organizationLinkPostfix":"."},"adminIdentityProviderConfigureEmail":{"title":"Email login configuratie","error":"Opslaan van veranderingen mislukt: %s","warning":"Opmerking: om gebruik te maken van email login moet u hierna email bezorging configureren.","explanation":"Email login heeft erder geen configuratie opties.","saveButton":"Opslaan","enableButton":"Inschakelen","cancelButton":"Annuleren","disableButton":"Uitschakelen"},"googleLoginSetupInstructions":{"explanation":"Als eerste heeft u een Google Client ID nodig. Volg de volgende stappen:","step1":"Ga naar <a href=\"https://console.developers.google.com/\" target=\"blank\">https://console.developers.google.com/</a>","step2":"Klik op \"Project maken\", wacht tot Google klaar is met provisioning indien nodig.","step3":"In de linker zijbalk, ga naar \"Inloggegevens\" en aan de rechter kant \"OAuth toestemmingsscherm\". Voer een email adres en productnaam in en klik op opslaan.","step4":"Ga in de linker zijbalk naar \"Inloggegevens\". Klik op de \"Inloggegevens maken\" knop, en kies dan voor \"Client-ID OAuth\" als het type.","step5":"Selecteer \"Webapp\" als het applicatie type.","step6":"Stel Geautoriseerde JavaScript bronnen in op: <span class=\"idp-config-url\">%s</span>","step7":"Stel Geautoriseerde omleidings-URI's in op: <span class=\"idp-config-url\">%s/_oauth/google</span>","step8":"Rond af door te klikken op \"Maken\"."},"adminIdentityProviderConfigureGoogle":{"title":"Google login configuratie","formerBaseUrlWarning":"Deze server is verplaatst van <code>__former__</code> naar <code>__current__</code>. U zult <a target=\"_blank\" href=\"https://console.developers.google.com/\">https://console.developers.google.com/</a> moeten bezoeken en \"Authorized Javascript Origins\" en \"Authorized Redirect URI\" voor deze Client ID bij moeten werken om de waarden hieronder te bevatten.","error":"Opslaan van veranderingen mislukt: %s","explanation":"Kopieer daarna uw Client ID en Client secret naar hieronder:","clientId":"Client ID:","clientSecret":"Client secret:","saveButton":"Opslaan","enableButton":"Inschakelen","cancelButton":"Annuleren","disableButton":"Uitschakelen"},"githubLoginSetupInstructions":{"explanation":"Eerst heeft u een Github Client ID en Client secret nodig, volg deze stappen:","step1":"Ga naar <a href=\"https://github.com/settings/applications/new\" target=\"blank\">https://github.com/settings/applications/new</a>","step2":"Stel Homepage URL in op: <span class=\"idp-config-url\">%s</span>","step3":"Stel Authorization callback URL in op: <span class=\"idp-config-url\">%s_oauth/github</span>"},"adminIdentityProviderConfigureGitHub":{"title":"GitHub login configuratie","formerBaseUrlWarning":"Deze server is verplaatst van <code>__former__</code> naar <code>__current__</code>. U zult moeten gaan naar <a target=\"_blank\" href=\"https://github.com/settings/developers\">https://github.com/settings/developers</a> en de Homepage URL en Authorization callback URL voor deze applicatie bijwerken naar de waardes hieronder.","explanation":"Kopieer daarna uw Client ID en Client secret hieronder","clientId":"Client ID:","clientSecret":"Client secret:","saveButton":"Opslaan","enableButton":"Inschakelen","cancelButton":"Annuleren","disableButton":"Uitschakelen"},"adminIdentityProviderConfigureLdap":{"title":"LDAP login configuratie","error":"Opslaan van veranderingen mislukt: %s","url":{"label":"LDAP server URL:","subtext":"bv. <code>ldap://ldap.example.com:389</code>.  Gebruik poort 636 voor SSL."},"searchBindDn":{"label":"Bind user DN","subtext":"De DN om aan te binden tijdens het zoeken naar een gebruiker, b.v. <code>cn=admin,ou=users,dc=example,dc=com</code>.  Optioneel, maar nodig wanneer uw server geen ongeautoriseerde zoekopdrachten toestaat."},"searchBindPassword":{"label":"Bind user wachtwoord","subtext":"Optioneel, nodig voor het specificeren van de bind DN zoals hierboven."},"base":{"label":"Base DN","subtext":"Het hoofd object om onder te zoeken naar gebruikersaccount nodes, bv. <code>ou=users,dc=example,dc=com</code>"},"searchUsername":{"label":"LDAP gebruikersnaam attribuut","subtext":"Het LDAP attribuut op een gebruikersobject dat de gebruikersnaam representeert (wet gebruikers invullen als gebruikersnaam tijdens het inloggen)."},"nameField":{"label":"LDAP gegeven naam attribuut","subtext":"Het LDAP attribuut op een gebruikersobject dat de volledige schermnaam van een gebruiker representeert."},"emailField":{"label":"LDAP email attribuut","subtext":"Het LDAP attribuut op een gebruikersobject dat het email adres representeert."},"filter":{"label":"Aanvullende LDAP filter criteria","subtext":"Een optioneel LDAP query fragment dat mee wordt gestuurd tijdens het zoeken naar gebruikers. Dit wordt niet vaak gebruikt. Zie <a target=\"_blank\" href=\"http://www.ietf.org/rfc/rfc2254.txt\">RFC2254</a>."},"caCert":{"label":"LDAP CA certificaat","placeholder":"-----BEGIN CERTIFICATE-----\nMIID...\n-----END CERTIFICATE-----","subtext":"Dit is alleen maar nodig als uw server SSL ondersteund en gebruik maakt van een prive CA die zich niet in de gemeenschappelijke vertrouwde set bevindt. Het werkt NIET om hier het certificaat van de LDAP server zelf in te voeren -- het moet een certificaat zijn van een CA die het certificaat van de LDAP server heeft ondertekend."},"saveButton":"Opslaan","enableButton":"Inschakelen","cancelButton":"Annuleren","disableButton":"Uitschakelen"},"adminIdentityProviderConfigureSaml":{"title":"SAML login configuratie","explanation":"Uw SAML IDP moet geconfigureerd zijn om een persistente nameID terug te geven. Als u daarnaast geen gebruik makt van Active Directory <strong>moet</strong> u uw IDP configureren om twee extra attributen te voorzien, <code>email</code> en <code>displayName</code>","serviceUrl":"De Service URL van deze server is: <code>%s</code>","logoutUrl":"De Service Logout URL van deze server is: <code>%s</code>","metadataUrlAt":"Als uw SAML IDP in staat is om service provider metadata uit te lezen van een URL, dan kunt u deze wijzen naar: <code><a href=\"__href__\" target=\"_blank\">__text__</a></code>","error":"Opslaan van veranderingen mislukt: %s","entryPoint":{"label":"SAML provider entry point URL:","placeholder":"https://YOUR_SAML_PROVIDER.com/sso/saml"},"logout":{"label":"SAML provider logout URL (optioneel):","placeholder":"https://YOUR_SAML_PROVIDER.com/sso/saml/SingleLogoutService"},"publicCert":{"label":"SAML certificaat de bovenstaande provider:","placeholder":"base64 gecodeerd certificaat (bv. MIID...8F==)"},"entityId":{"label":"Entity ID:"},"saveButton":"Opslaan","enableButton":"Inschakelen","cancelButton":"Annuleren","disableButton":"Uitschakelen"},"adminIdentityProviderTable":{"title":"Login provider","status":"Status"},"adminIdentityRow":{"enabled":"Ingeschakeld","disabled":"Niet ingeschakeld","baseUrlChanged":{"explanation":"Uitgeschakeld vanwege BASE_URL verandering","reconfigureButton":"herconfigureren"},"configureButton":{"hint":"Configureer %s login","text":"Configureer"}}},"organization":{"name":"Organisatie instellingen","subtext":"Snellere onboarding en samenwerking voor grote teams.","organizationSettingsBlurb":{"explanation":"Sandstorm maakt het mogelijk om een organisatie te definiëren. U kunt automatisch dezelfde regels voor alle leden van uw organisatie instellen. Gebruikers in de organisatie kunnen automatisch inloggen, apps installeren en korrels aanmaken."},"newAdminOrganization":{"success":"Veranderingen opgeslagen.","error":"Veranderingen opslaan mislukt: %s","membership":{"title":"Organisatie lidmaatschap","emailToggle":{"label":"Gebruikers geverifieerd met email adres.","subtext":"Opmerking: uitgeschakeld omdat email login niet is geconfigureerd."},"emailDomain":{"label":"Domein:","subtext":"Gebruikers met een email adres op dit domein zullen lid zijn van deze servers organisatie."},"gappsToggle":{"label":"Gebruikers geverifieerd via Google Apps voor Werk","subtext":"Opmerking: uitgeschakeld omdat Google login niet is geconfigureerd."},"gappsDomain":{"label":"Domein:","subtext":"Gebruikers met een Google Apps voor Werk account onder dit domein zullen lid zijn van deze servers organisatie."},"ldapToggle":{"label":"Gebruikers geverifieerd via LDAP","subtext":"Opmerking: uitgeschakeld omdat LDAP login niet is geconfigureerd."},"samlToggle":{"label":"Gebruikers geverifieerd via SAML","subtext":"Opmerking: uitgeschakeld omdat SAML login niet is geconfigureerd."}},"settings":{"title":"Organisatie instellingen","disallowGuests":{"label":"Verbied samenwerking met gebruikers buiten de organisatie.","subtext":"Gebruikers zullen in moeten loggen als een gebruiker van uw organisatie om deel links te bekijken. Gast toegang wordt uitgeschakeld."},"shareContacts":{"label":"Maak alle organisaties zichtbaar voor elkaar.","subtext":"Deze instelling voegt gebruikers binnen een organisatie automatisch toe aan elkaars contactenlijst zodat ze korrels met elkaar kunnen delen. Schakel dit uit als u gebruikers heeft waarvan de entiteit verborgen moet blijven voor anderen."}},"saveButton":"Opslaan"}},"emailConfig":{"name":"Email configuratie","subtext":"Hoe email wordt verzonden naar gebruikers.","newAdminEmailConfig":{"outbound":{"title":"Uitgaande email","warning":"<strong>Email is niet geconfigureerd</strong> Email-gerelateerde functionaliteit zoals emal login, meldingen, deel uitnodiging en het aanvragen van toegang zullen niet beschikbaar blijven totdat u email configureert."},"howItWorks":{"title":"Hoe Sandstorm email gebruikt","explanation":"Sandstorm en Sandstorm apps kunnen email notificaties verzenden. Elke app kriijgt een uniek email adres, dus Sandstorm moet email kunnen verzenden van meerdere \"Van:\" adressen. Gmail SMTP zal daarom niet werken, maar er zijn andere gratis diensten beschikbaar. Bekijk voor meer informatie <a href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">de documentatie</a>."},"success":"Veranderingen opgeslagen.","error":"Veranderingen opslaan mislukt: %s","address":{"hostname":"SMTP host","port":"Poort","subtext":"Gebruik poort 25 voor SMTP (on-versleuteld of met StartTLS), port 465 voor met TLS versleutelde SMTPS."},"username":{"label":"Gebruikersnaam (optioneel)"},"password":{"label":"Wachtwoord (optioneel)"},"returnAddress":{"label":"Email adres van de Sandstorm server","subtext":"Sandstorm zal login emails, notificatie van goedgekeurde toegangsverzoeken en andere emails vanaf dit adres versturen. Dit adres wordt ook gebruikt in de SMTP envelop."},"submitButton":"Configuratie opslaan","testButton":"Testen","disableButton":"Email uitschakelen"},"emailTestPopup":{"title":"Stuur een test email","testAddress":{"label":"Stuur test mail naar:"},"pending":"Versturen...","submitButton":"Verstuur test email","closeButton":"Sluiten"},"emailDisablePopup":{"title":"Email echt uitschakelen?","explanation":"Dit zal alle bezorging van uitgaande mail voor deze server uitschakelen, inclusief delen via email, email login (indien ingeschakeld) en email notificaties naar gebruikers.","disableButton":"Email uitschakelen","closeButton":"Sluiten"}},"personalization":{"name":"Personalisatie","subtext":"Stel een splash pagina in, servicevoorwaarden, privacybeleid, etc.","newAdminPersonalization":{"serverTitle":{"label":"Server titel:","subtext":"De naam van deze server, zoals gebruikt tijdens het communiceren met gebruikers."},"splashUrl":{"label":"Splash URL (experimenteel):","subtext":"Deze pagina wordt achter het login dialoogvenster weergegeven (de homepagina wanneer u niet bent ingelogd). Om veiligheidsredenen moet deze pagina binnen de Sandstorm servers wildcard host worden gehost, anders wordt de pagina geblokkeerd door <code>Content-Security-Policy</code>). We suggereren een statische web publising app zoals <a target=\"_blank\" href=\"https://apps.sandstorm.io/app/nqmcqs9spcdpmqyuxemf0tsgwn8awfvswc58wgk375g4u25xv6yh/?host=%s\">Hacker CMS</a> om de inhoud te hosten.<strong>WAARSCHUWING: deze functie is experimenteel, de stijl en positionering van het login scherm kunnen veranderen zonder kennisgeving.</strong> Laat het ons alstublieft weten als u het graag gestabiliseerd zou willen hebben."},"signupDialog":{"label":"Aanmeldingsdialoog:","subtext":"Deze tekst wordt weergegeven aan gebruikers die op aanmeldingslinks klikken."},"termsOfService":{"label":"Servicevoorwaarden URL:","placeholder":"https://example.com/tos","subtext":"Indien ingevuld worden nieuwe gebruikers verplicht om akkoord te gaan met de gelinkte servicevoorwaarden."},"privacyPolicy":{"label":"Privacybeleid URL:","placeholder":"https://example.com/privacy","subtext":"Indien ingevuld worden nieuwe gebruikers verplicht om akkoord te gaan met het gelinkte privacybeleid."},"whitelabeling":{"title":"Whitelabeling","logo":{"label":"Aangepast logo op het inlogdialoog","resetButton":"Standaard gebruiken","uploadButton":"Logo uploaden"},"hideTroubleshooting":{"label":"Verberg probleemoplossing knop"},"hideAbout":{"label":"Verberg \"Over Sandstorm\" links"},"hideSendFeedback":{"label":"Verberg \"Stuur feedback\" link"},"useServerTitleForHomeText":{"label":"Gebruik server titel (hierboven) voor home link","subtext":"Vervang het standaard \"Sandstorm\" label op de link helemaal linksbovenin met wat u hierboven als Server titel heeft opgegeven."},"customLoginProviderName":{"label":"Aangepaste tekst op de inlog knop","subtext":"Vervangt de tekst \"met LDAP\" en \"met SAML\" op de inlogpagina met de opgegeven tekst."}},"saveButton":"Opslaan"}},"preinstalledApps":{"name":"Vooraf geïnstalleerde apps","subtext":"Kies apps om voor gebruikers te voorinstalleren.","newAdminPreinstalledApps":{"explanation":"Apps die up op deze pagina configureert om vooraf te installeren worden automatisch geïnstalleerd (onder /apps geplaatst) voor nieuwe gebruikers.","productivitySuite":{"label":"Productiviteit","subtext":"We raden het aan om deze kern apps te installeren die voor de meeste gebruikers nuttig zijn."},"systemSuite":{"label":"Systeem apps","subtext":"We raden het aan om deze apps te installeren die de mogelijkheden van Sandstorm veel uitbreiden."},"allApps":{"label":"Alle apps op de markt","subtext":"De aangevinkte apps hieronder worden standaard geïnstalleerd voor alle nieuwe gebruikers.","lessButton":"Laat minder apps zien","moreButton":"Laat %s meer apps zien"},"saveButton":"Opslaan"},"_appRow":{"download":{"success":"Gedownload","failed":"Mislukt"}}},"appSources":{"name":"App bronnen","subtext":"Waar te zoeken naar apps and updates.","newAdminAppSources":{"appMarketUrl":{"label":"App markt URL","subtext":"De \"Install...\" knop op de Apps pagina zal naar deze URL linken."},"enableAppUpdates":{"label":"Schakel app update notificaties in","subtext":"Updates voor apps zullen dagelijks door de Sandstorm server worden gedownload en notificaties worden gestuurd naar de gebruikers met de vraag of ze hun geïnstalleerde apps willen updaten."},"appIndexUrl":{"label":"App index URL","subtext":"Dit is de URL die de Sandstorm server zal controleren om te leren over app updates."},"saveButton":"Opslaan","restoreButton":"Standaardinstellingen herstellen"}},"networking":{"name":"Netwerk","altName":"Beveiliging","subtext":"Beheer hoe er toegang wordt verkregen tot het netwerk.","newAdminNetworking":{"ipBlacklist":{"label":"Server-side request IP blacklist:","subtext":"Gebruikers worden verboden om aanvragen naar deze IP adressen te doen. Hieronder valt het maken van een aavraag vanuit een app, het downloaden van een SPK van door de gebruiker opgegeven URL, etc. U kunt een IP adres of netwerk (In CIDR notatie, bv. \"127.0.0.0/8\" per regel. De standaardwaarden bevatten locale en privé netwerkadressen. Onthoud dat deze instelling genegeerd kan worden wanneer er gebruik wordt gemaakt van een proxy, de proxy moet zijn eigen blacklist implementeren."},"saveButton":"Opslaan","restoreButton":"Standaardinstellingen herstellen"}},"users":{"name":"Gebruikers","subtext":"Beheer alle gebruikers op deze Sandstorm installatie."},"userAccounts":{"newAdminUserTableRow":{"admin":{"hint":"Administrator","cap":"A"},"user":{"hint":"Gebruiker","cap":"G"},"visitor":{"hint":"Bezoeker","cap":"B"},"unknownIdentity":"Onbekende credential","unknownIdentifier":"onbekende identifier","manageButton":"Beheren"},"newAdminUserTable":{"header":{"name":"Naam","credentials":"Credentials","created":"Aangemaakt","lastActive":"Laatst actief","actions":"Acties"},"inviteLink":"Nieuwe gebruiker uitnodigen","notFound":"Er kwamen geen gebruikers overeen met uw filter.","processing":"Gebruikers laden..."},"newAdminUsers":{"search":{"placeholder":"zoeken"},"filter":{"explanation":"Laat zien:","admin":"Administrator","user":"Gebruiker","visitor":"Bezoeker"}}},"userDetails":{"newAdminUserDetailsIdentityTableRow":{"profilePicture":{"label":"Profielafbeelding"},"handle":{"title":"Handle","emails":"E-mails"},"pronouns":{"hint":"Voornaamwoord","label":"Voornaamwoord"},"emails":{"hint":"E-mails","label":"E-mails","primary":"Primair","verified":"Geverifieerd","unverified":"On-geverifieerd","notFound":"Geen bekende e-mails"},"properties":{"login":"Kan inloggen","org":"Org. lid"},"unkonwn":{"profilePicture":{"cell":"(onbekende naam)"},"service":{"hint":"onbekende dienst","label":"(onbekende dienst)","cell":"(onbekende naam)"},"handle":{"cell":"(onbekende handle)"},"pronouns":{"cell":"(onbekende voornaamwoorden)"},"created":{"cell":"onbekende aanmaakdatum"},"lastActive":{"cell":"onbekende activiteit datum"}}},"newAdminUserDetailsIdentityTable":{"header":{"credential":"Credential","created":"Aangemaakt","lastActive":"Laatst actief","properties":"Eigenschappen"}},"newAdminUserDetails":{"role":{"title":"Rol","admin":{"label":"Administrator","explanation":"Beheert alle accounts. Kan apps installeren, kan korrels aanmaken en de server beheren.","isAdmin":"Huidige rol","isNotAdmin":"Promoveren naar administrator"},"user":{"label":"Gebruiker","explanation":"Kan apps installeren en korrels aanmaken.","isUser":"Huidige rol","isAdmin":"Demoveren naar gebruiker","isNotUser":"Promoveren naar gebruiker"},"visitor":{"label":"Bezoeker","explanation":"Kan gedeelde korrels bekijken en verder delen, maar kan niet zelf korrels aanmaken of apps installeren.","isVisitor":"Huidige rol","isNotVisitor":"Demoveren naar bezoeker"}},"credentials":{"title":"Credentials"},"profile":{"title":"Profiel"},"status":{"title":"Status","deleting":"Geschorst - In afwachting van verwijdering %s","suspended":"Geschorst","active":"Actief","unsuspendButton":"Schorsing ongedaan maken","suspendButton":"Account schorsen","deleteButton":"Account verwijderen"},"deletePopup":{"title":"Verwijder %s","explanation":"U staat op het punt om %s's account te markeren voor verwijdering. Eenmaal gemarkeerd wordt dit account geschorst, en admininstrators zullen 7 dagen hebben om de geplande verwijdering te stoppen door terug te keren naar deze pagina en te klikken op \"Schorsing ongedaan maken\". %s kan zelf <strong>niet</strong> de schorsing / verwijdering ongedaan maken.","detail":"Na 7 dagen wordt %s's account permanent verwijderd. Het verwijderen van dit account zal:","detail1":"Permanent korrels verwijderen aangemaakt door %s","detail2":"Alle deellinks intrekken die %s heeft aangemaakt","detail3":"%s verwijderen uit alle gebruikers contactenlijsten","detail4":"Alle credentials die eigendom zijn van dit account verwijderen (behalve credentials die ook geassocieerd zijn met een ander account).","deleteButton":"Account verwijderen","cancelButton":"Annuleren"},"suspendPopup":{"title":"Schors %s","detail":"Het schorsen van dit gebruikersaccount zal ervoor zorgen dat:","detail1":"Korrels aangemaakt door %s niet kunnen worden geopend","detail2":"Deellinks aangemaakt door %s uitschakelen","detail3":"Dit geldt ook voor alle credentials die eigendom zijn van diet account (behalve credentials die ook geassocieerd zijn met een ander account)","suspendButton":"Account schorsen","cancelButton":"Annuleren"},"loading":"Gebruikersdata laden..."}},"userInvite":{"newAdminUserInviteLink":{"explanation":"U kunt anderen uitnodigen om accounts aan te maken op uw server door hen een account aanmaaklink te geven. Mensen die u uitnodigt krijgen Gebruiker permissie en kunnen apps installeren en zelf korrels aanmaken. Als u alleen een specifieke korrel met iemand wilt delen kunt u dat doen via de korrels \"Toegang Delen\" menu.","title":"Maak een link","invite":{"explanation":"Dit zal een account aanmaaklink aanmaken die u naar iemand kunt versturen.","createButton":"Link aanmaken"}},"newAdminUserInviteEmail":{"title":"Stuur een email","emailWarning":{"explanation":"Email is niet geconfigureerd, het versturen van uitnodiging via email is daarom uitgeschakeld.","configureLink":"Email configureren"},"explanation":"Dit zal een link aanmaken en het versturen in een email.","to":{"label":"Naar:","placeholder":"alice@email.com\nbob@example.com","subtext":"Email adressen, een per regel."},"subject":{"label":"Onderwerp:"},"body":{"label":"Body:","subtext":"<code>$KEY</code> zal worden vervangen door de gegenereerde link."},"sendButton":"Stuur uitnodiging"},"newAdminUserInvite":{"title":"Uitnodigen"}},"systemStatus":{"name":"Systeem log","subtext":"Bekijk Sandstorms debug log","newAdminLogContents":{"ellipsis":"..."},"newAdminStatus":{"downloadButton":"Volledige log downloaden"}},"stats":{"name":"Statistieken","subtext":"Gebruiksstatistieken voor deze server bekijken.","header":{"date":"datum","users":"gebruikers","grains":"korrels","appName":"App naam","owners":"eigenaren","sharedUsers":"sharedUsers","daily":"dagelijks","weekly":"wekelijks","monthly":"maandelijks","forever":"voor altijd"},"body":{"notFound":"Er zijn nog geen statistieken opgenomen.","loading":"Laden..."},"newAdminStats":{"share":{"title":"Anoniem publiceren naar het Sandstorm dev team","explanation":"We zouden graag willen weten hoeveel Sandstorm gebruikers er zijn om grafieken en dergelijken maken. Is het goed om de gegevens op deze pagina een keer per dag naar het Sandstorm team te sturen zodat we ze bij de totalen kunnen optellen? Statistieken die u verzend zijn anoniem.","enableButton":"Verstuur anonieme statistieken","disableButton":"Niet versturen","control":{"explanation":"De ontwikkelaars van Sandstorm zouden graag willen weten hoeveel Sandstorm gebruikers er zijn om grafieken te maken en dergelijke.  U kunt helpen door te kiezen voor het versturen van de gegevens op deze pagina een kker per dag naar het Sandstorm team te versturen zodat we deze aan onze totalen kunnen toevoegen. Statistieken die worden gedeeld zijn allemaal anoniem.","label":"Anonieme statistieken rapportage naar sandstorm.io:","enabled":"Ingeschakeld","disabled":"Uitgeschakeld","disableButton":"Uitschakelen","enableButton":"Inschakelen"}},"json":{"title":"Als JSON bekijken","explanation":"Als u Sandstorm statistieken met een intern gereedschap wilt integreren dan kunt u deze geheime link gebruiken om de onderstaande tabellen als JSON op te vragen. U kunt deze link op elk moment intrekken door een nieuwe link aan te maken.","label":"Statistieken JSON link:","regenerateButton":"Nieuwe link genereren"},"userGrains":{"title":"Gebruikers en korrels","current":"Op dit moment: __users__ gebruikers, __grains__ korrels","today":"Vandaag: __users__ gebruikers, __grains__ korrels"},"apps":{"title":"Apps","label":"Selecteer een datum:"}}},"maintenanceMessage":{"name":"Onderhoudsboodschap","subtext":"Laat mogelijke uitvaltijd weten aan gebruikers.","newAdminMaintenance":{"explanation":"U kunt een onderhoudsboodschap instelling die aan alle gebruikers in de menubalk rechtsbovenin wordt getoond. Dit is nuttig voor het aankondigen van gepland onderhoud zodat gebruikers voorberijd kunnen zijn op eventuele uitvaltijd. Als de bericht tekst leeg is wordt er geen bericht getoond.","variables":"U kunt de volgende variabelen gebruiken in het \"Bericht tekst\" veld, deze worden automatisch vervangen door nuttige waarden:","variable1":"<code>$TIME</code> : Het tijd onderdeel van \"Onderhoudtijd\", omgezet naar de gebruikers lokale tijd (bv. \"1:15:45 PM\")","variable2":"<code>$DATE</code> : Het datum onderdeel \"Onderhoudtijd\", omgezet naar de gebruikers lokale tijd (bv. \"6/19/2015\")","variable3":"<code>$IN_COUNTDOWN</code> : Een aftelling tot \"Onderhoudtijd\" (bv. \"over 3 dagen\" of \"over 1 minuut\"). Laat \"any moment\" zien op de tijd opgegeven in  \"Onderhoudtijd\" en verdwijnt na een uur.","messageText":{"label":"Berichttekst:","subtext":""},"maintenanceTime":{"label":"Onderhoudtijd:","subtext":"De tijd waarop u van plan bent om onderhoud te plegen, opgemaakt als \"%s\"."},"url":{"label":"Link URL (optioneel):","subtext":"Indien ingesteld wordt het onderhoudbericht weergegeven als een klikbare link naar de opgegeven URL."},"saveButton":"Bericht instellen"}},"networkCapabilities":{"name":"Netwerk mogelijkheden","subtext":"Directe netwerktoegang voor korrels beheren.","newAdminNetworkCapabilitiesTableCapabilityRow":{"adminAccount":"Administrator account %s","unknownGranter":"Onbekende toestemming gever","anyone":"Iedereen met het token","unknownRecipient":"Onbekende ontvanger","revoked":"Ingetrokken","revokeButton":"Intrekken"},"newAdminNetworkCapabilitiesTable":{"header":{"grantBy":"Toestemming gegeven door","grantTo":"Toestemming gegeven aan","grantTime":"Tijd","actions":"Acties"},"notFound":"Geen mogelijkheden gevonden."},"newAdminNetworkCapabilitiesSection":{"filter":{"label":"Weergeven:","active":"Actieve (%s)","revoked":"Ingetrokken (%s)"}},"newAdminNetworkCapabilities":{"networkCaps":"Uitgaande netwerktoegang","networkCapsTls":"TLS-versleutelde uitgaande netwerktoegang","interfaceCaps":"Ingaande netwerktoegang"}},"hostingManagement":{"name":"Hosting management","newAdminHostingManagement":{"explanation":"Deze functie is experimenteel en werkt op dit moment alleen met de LDAP login provider. Gebruik dit voorzichtig.","quotaEnabled":{"label":"Quota instellen voor gebruikers","subtext":"Quota beheer inschakelen op deze server. Hiervoor moeten SAML en LDAP zijn geconfigureerd, en email adressen worden gebruikt om te matchen tussen de twee."},"quotaLdapAttribute":{"label":"LDAP quota attribuut","subtext":"Dit is de attribuutnaam die wordt opgezocht van LDAP om de gebruikersquota vast te stellen."},"billingPromptUrl":{"label":"Facturatie venster URL","subtext":"Dit is de URL de wordt gebruikt in het facturatie venster."},"saveButton":"Opslaan","restoreButton":"Standaardinstellingen herstellen"}}},"people":{"title":"Mensen"},"sharingLinks":{"title":"Deellinks"},"apps":{"title":"Apps","appList":{"uninstallingHint":"Klaar met verwijderen","doneUninstalling":"Klaar met verwijderen","uninstallHint":"Verwijder een app","uninstallButton":"Verwijderen...","uploadApp":"Upload app...","searchHint":"Zoeken","search":"zoeken","mostUsed":"Meest gebruikt","allApps":"Alle apps","install":"Installeren...","fromMarket":"vanaf de app markt","uninstall":"Verwijderen","noMatchApp":"U heeft geen geïnstalleerde apps met die naam. Probeer <a href=\"__url__\">te zoeken &raquo;</a>","unauthorized":"Niet geautoriseerd! :(","unauthorizedText":"Dit is een privé Sandstorm server. Om apps te installeren moet u zijn uitgenodigd door een administrator. De administrator kan u uitnodigen vanuit het admin panel.","intro":"<strong>Welkom!</strong><br>Maak documenten, chatrooms, blogs en meer.<br><br>Begin door een app te installeren.","doneButton":"Begrepen"},"install":{"sandstormAppInstallPage":{"title":"Installeer app","error":{"title":"App installeren mislukt","retryButton":"Opnieuw proberen"},"stepWait":{"title":"Wachten op schrijftoegang..."},"stepDownload":{"title":"Downloaden...","cancelButton":"Annuleren"},"stepVerify":{"title":"Verifiëren..."},"stepUnpack":{"title":"Uitpakken..."},"stepAnalyze":{"title":"Analyseren..."},"stepConfirm":{"upgrade":{"explanation":"U heeft een oudere versie van deze app geïnstalleerd. Wilt u upgraden?","button":"Upgrade %s"},"downgrade":{"explanation":"U heeft al een nieuwere versie van deze app geïnstalleerd.  Wete u zeker dat u terug wilt gaan naar deze oudere versie?","button":"Downgrade %s"},"install":{"button":"Installeer %s"}},"stepRun":{"title":"%s geïnstalleerd","explanation":"Installatie voltooid.","appDetailsLink":"Ga naar %s"},"stepDelete":{"title":"Verwijderen","explanation":"Server is bezig met het verwijderen van deze app..."},"stepFailed":{"title":"Mislukt","explanation":"Installatie mislukt: %s"}}},"appDetails":{"sandstormAppDetailsPage":{"newerVersion":{"explanation":"Sommige van uw bestanden zijn aangemaakt met een nieuwere versie van deze app. Deze zullen de nieuwere versie blijven gebruiken aangezien achteruit gaan ze kapot zou kunnen maken."},"olderVersion":{"explanation":"Sommige van uw korrels zijn gemaakt met een oudere versie van deze app.  Wilt u deze upgraden?","upgradeButton":"Upgrade Korrels"},"search":{"hint":"Zoeken","placeholder":"zoeken"},"trash":{"explanation":"Korrels worden permanent verwijdern da 30 dagen in de vuilnisbak. Om een korrel te behouden, herstel deze naar de Hoofdlijst."},"noGrains":{"notFound":"<strong>Geen overeenkomende korrels gevonden.</strong>","inTrash":"Er zijn korrels in uw vuilnisbak die overeenkomen met uw zoekopdracht.","trashButton":"Vuilnis bekijken"}},"sandstormAppDetails":{"websiteLink":"Website","appMarketLink":"App markt","sourceCodeLink":"Bron","bugReportLink":"Probleem melden","mainListButton":"Hoofdlijst bekijken","trashButton":"Vuilnisbak bekijken","uninstallButton":"Verwijderen","restoreButton":"Korrel backup herstellen...","version":{"label":"Versie"},"lastUpdate":{"label":"Laatst bijgewerkt","unknown":"<onbekend>"},"publisher":{"label":"Ondertekend door"},"hasFingerprint":"De uitgever van deze app heeft met PGP bewezen dat ze deze accounts beheren.","noFingerprint":"Deze app heeft geen handtekening van de maker; de auteur ervan kan niet worden geverifieerd."}}},"accounts":{"title":"Account instellingen","accountSettings":{"sandstormAccountSettings":{"title":"Account","identities":{"title":"Mijn profiel","success":"Succes: %s","linkedCredentials":"Gelinkte credentials","linkNewIdentity":{"title":"Nieuwe credential linken:","cancelButton":"Annuleren","linkButton":"Nieuwe credential linken","logoutAllButton":"Alle andere sessies uitloggen"},"email":{"title":"Primair email adres voor dienstgerelateerde notificaties","primary":"Primair","primaryButton":"Stel in als primair","notFound":"Geen geverifieerde emails! ","addNew":"Nieuw email adres toevoegen"}},"billing":{"title":"Facturering"},"deleteAccount":{"title":"Verwijder account","button":"Verwijder uw account"},"deletePopup":{"title":"Verwijder uw account","explanation":"U staat op het punt om uw account te markeren voor verwijdering. Eenmaal gemarkeerd wordt uw account geschorst, en u krijgt 7 dagen om de geplande verwijdering te annuleren door op uw account in te loggen.","results":"Na 7 dagen wordt uw account permanent verwijderd. Dit zal:","result1":"Permanent korrels verwijderen die door u zijn aangemaakt","result2":"Alle deellinks die u heeft gemaakt intrekken","result3":"U verwijderen van alle gebruikers contactenlijsten","result4":"Alle credentials die eigendom zijn van uw account verwijderen (behalve credentials die geassocieerd zijn met een ander account).","confirm":"Type alstublieft \"delete my account\" hieronder om te bevestigen.","goodbye":"We vinden het jammer om u te zien gaan. Voel u vrij om feedback die u heeft te delen met de server administrator.","feedbackPlaceholder":"Optioneel","deleteButton":"Account verwijderen","cancelButton":"Annuleren"}},"sandstormAccountsFirstSignIn":{"title":"Bevestig uw profiel","success":"Succes: %s"},"accountSuspended":{"title":"Dit account is door u geschorst.","deletedIn":"Dit account wordt verwijderd over %s. Wilt u verwijdering annuleren en uw dit account herstellen?","suspended":"Dit account is geschorst en alle korrels en alle delingen zijn uitgeschakeld. Wilt u dit account herstellen?","restoreButton":"Account herstellen","logoutButton":"Uitloggen"},"_accountProfileEditor":{"picture":{"label":"Profielafbeelding:","uploadButton":"Nieuwe afbeelding uploaden","subtext":"Deze afbeelding wordt aan andere gebruikers getoond om hen te helpen u te herkennen.","maxSize":"512x512px, maximaal 64KiB."},"fullName":{"label":"Volledige naam:","placeholder":"volledige naam","subtext":"Uw gewone voor en achter naam zoals u deze graag wilt hebben weergegeven aan andere gebruikers. Dit hoeft niet uw echte naam te zijn."},"handle":{"label":"Handle:","placeholder":"handle","hint":"Handles moeten beginnen met een kleine letter en mogen alleen kleine letters, getallen en lage streepjes bevatten.","subtext":"Om weer te geven in apps die gebruik maken van handles. Uw handle moet beginnen met een letter en mag alleeen kleine letters, getallen en lage streepjes bevatten."},"emailList":{"label":"E-mail:"},"pronoun":{"label":"Voorkeurvoornaamwoord:","neutral":"hen (ongespecificeerd)","male":"hij/hem (mannelijk)","female":"zij/haar (vrouwelijk)","robot":"het (robot)"},"agreedToTerms":{"agreeTo":" Ik heb gelezen en ben akkoord gegaan met de ","termsOfServiceLink":"Servicevoorwaarden","and":" en het ","privacyLink":"Privacybeleid"},"saveButton":"Opslaan","continueButton":"Doorgaan","cancelButton":"Annuleren"}},"loginButtons":{"icon":{"info":"i"},"loginButtons":{"createButton":"Account maken","signInButton":"Aanmelden"},"accountButtons":{"loading":"Laden..."},"_loginButtonsLoggedInDropdown":{"revealIdentity":"Identiteit openbaren","title":"Account","sendFeedback":"Feedback versturen","referralProgram":"Verwijzingsprogramma","signOutButton":"Afmelden"},"_loginButtonsLoggedOutDropdown":{"demo":{"title":"Account maken","explanation":"Uw demo data wordt overgebracht naar uw nieuwe account."},"title":"Aanmelden","shared":{"explanation":"Log in zodat andere mensen weten wie u bent."},"signInButton":"Aanmelden","anonymousButton":"Anoniem blijven"},"oauthLoginButton":{"with":"met "},"devLoginForm":{"label":"met een Dev account","hint":"verkleinen","alice":"Alice (administrator)","bob":"Bob","carol":"Carol","dave":"Dave","customPlaceholder":"Eve","customButton":"Aangepast"},"loginButtonsList":{"configure":"login configureren","troubleshooting":"probleemoplossing","about":"over Sandstorm"},"emailLoginForm":{"description":"met e-mail","sendButtonText":"verstuur Inlog Email"},"emailAuthenticationForm":{"hint":"Wachtwoordloze E-mail login stelt u in staat om in te loggen met een e-mail adres. Een wachtwoord is niet nodig; in plaats daarvan sturen we u een nieuwe email bevestigingscode elke keer wanneer u inlogt. Per apparaat hoeft u maar een keer in te loggen.","token":{"explanation":"We hebben een inlog e-mail verstuurd naar %s. Het kan even duren voordat deze verschijnt in uw inbox.","label":"Voer uw token in:","confirmButton":"Bevestigen","cancelButton":"Annuleren"},"disabled":"Een server administrator moet email authenticatie inschakelen voordat u uw email kan bevestigen."},"ldapLoginForm":{"username":{"label":"gebruikersnaam","hint":"Dit is uw LDAP gebruikersnaam. Het is dezelfde naam die u normaal gebruikt op in te loggen op andere interne diensten bij uw bedrijf."},"password":{"label":"wachtwoord","hint":"Dit is uw LDAP wachtwoord. Het is hetzelfde wachtwoord dat u normaal gebruikt voor het inloggen op interne diensten bij uw bedrijf."},"loginButton":"Inloggen"},"_loginButtonsSeparator":{"or":"of"}},"identity":{"identityLoginInterstitial":{"title":"Inloggen","loading":"Laden...","logoutButton":"Uitloggen","cancelButton":"Annuleren","goBackButton":"Ga terug","create":"account aanmaken...","createFailed":"Account aanmaken mislukt.","loginAccount":"Inloggen op account...","loginAccountFailed":"Account login mislukt.","loginToken":"Opnieuw inloggen op een account...","loginTokenFailed":"Sessie herstellen mislukt.","link":"credential aan account vastmaken...","success":"Credential losgemaakt van %s","explanation":"U bent geverifieerd als deze credential","warning":"maar dat is niet een login credential voor een account.","moreIdentities":"De volgende credentials kunnen gebruikt worden om in te loggen up uw credentials account:","unlinkButton":{"hint":"Maak uw credential van dit account los.","text":"losmaken"},"unlink":{"explanation":"Weet u zeker dat u uw credential van dit account wilt losmaken?","unlinkButton":"Losmaken","cancelButton":"Annuleren"}},"identityManagementButtons":{"toggleLogin":{"label":"Stel deze credential in staat om in te loggen op uw account.","unlinkButton":"Losmaken"}},"loginIdentitiesOfLinkedAccounts":{"show":{"explanation":"Deze credential is gedeeld met andere accounts. Die account hebben de volgende login credentials:","unlinkButton":"losmaken","hideButton":"verberg andere accounts"},"hide":{"explanation":"Deze credential is gedeeld met tenminste een ander account.","showButton":"laat andere accounts zien"}},"identityCardSignInButton":{"hint":"log in als %s","title":"Log in"},"accountCardSignInButton":{"reSign":"Om uit te loggen van uw andere account en in te loggen als deze gebruiker, kies hoe u wilt inloggen:","howToSignIn":"Kies hoe u wilt inloggen:"}},"tokenTemplates":{"_emailTokenError":{"title":"Ongeldige verificatie code. Als er meer dan 15 minuten zijn verstreken dan is het token verlopen, anders heeft u misschien een typefout gemaakt."},"_emailLinkIdentityError":{"title":"Om uw email adres te bevestigen moet u zijn ingelogd als het account dat deze link heeft aangemaakt."},"addNewVerifiedEmailPowerboxConfiguration":{"email":{"placeholder":"voer email adres in","selectButton":"Selecteer","sendButton":"Stuur bevestigingsmail"},"sendingTo":"Versturen naar %s","resetButton":"Probeer een ander adres","token":{"placeholder":"voer uw token in","confirmButton":"Bevestigen"},"verifying":"Token verifiëren..."}}},"wizard":{"setupWizardLoading":{"loading":"Laden..."},"setupWizardProgressBar":{"identityProviders":"Login providers","organization":"Organisatie","email":"Email","preinstalledApps":"Vooraf geïnstalleerde apps","adminAccount":"Administrator account"},"setupWizardHelpFooter":{"needHelp":"Hulp nodig?","documentation":"Documentatie","emailSupport":"Email ondersteuning","systemLog":"Systeem Log"},"setupWizardSystemLog":{"header":{"label":"Systeem log","downloadButton":"Log Downloaden"},"permissionDenied":"U kunt de systeem log niet bekijken omdat u niet bent ingelogd als een administrator en u heeft geen setup token."},"setupWizardIntro":{"title":"Welkom bij Sandstorm!","revisitWizardLink":"Installatiewizard opnieuw bezoeken","noIdpEnabled":{"explanation":"Deze Sandstorm server is al eerder opgezet, maar vanwege een verandering in <code>sandstorm.conf</code> moet u opnieuw inlog providers instellen."},"idpEnabled":{"explanation":"Deze server is klaar voor gebruik. U kunt serverinstellingen aanpassen waaronder welke providers zijn ingeschakeld en de manier waarop Sandstorm email verstuurd.","turnUserIntoAdmin":"U kunt ook een bestaande gebruiker in een administrator.","isAdmin":{"adminPanelLink":"Doorgaan naar administrator panel.","suButton":"Geef uzelf administrator privileges."},"isNotAdmin":{"explanation":"Om een account te veronderen in een administrator account, log alstublieft in:","signInLabel":"Aanmelden","signInButton":"Meld u aan om een administrator te worden"}},"setupButton":"Begin Sandstorm Setup"},"setupWizardIdentity":{"title":"Inlog providers","explanation1":"Om gebruik te maken van Sandstorm moet u een gebruikersaccount aanmaken. Elk gebruikersaccount op Sandstorm wordt gesteund door een inlogprovider.  U zult deze inlogprovider moeten gebruiken om als de eerste administrator van deze Sandstorm installatie geverifieerd te worden.","explanation2":"Configureerde de inlogprovider of providers die u wilt inschakelen.","nextButton":"Volgende","backButton":"Terug"},"setupWizardOrganization":{"title":"Organisatie instellingen","error":"Opslaan van veranderingen mislukt: %s","membership":{"title":"Organisatie lidmaatschap","emailToggle":{"label":"geverifieerde gebruikers via email adres","subtext":"Opmerking: uitgeschakeld omdat email login niet is geconfigureerd."},"emailDomain":{"label":"Domein:","subtext":"Gebruikers met een email adres op dit domein zullen lid zijn van deze servers organisatie."},"gappsToggle":{"label":"Gebruikers geverifieerd via Google Apps for Work","subtext":"Opmerking: uitgeschakeld omdat Google login niet is geconfigureerd."},"gappsDomain":{"label":"Domein:","subtext":"Gebruikers met een Google Apps for Work account onder dit domein zullen lid zijn van deze servers organiasatie."},"ldapToggle":{"label":"Gebruikers geverifieerd via LDAP","subtext":"Opmerking: uitgeschakeld omdat LDAP login niet is geconfigureerd."},"samlToggle":{"label":"Gebruikers geverifieerd via SAML","subtext":"Opmerking: uitgeschakeld omdat SAML login niet is geconfigureerd."}},"settings":{"title":"Organization instellingen","disallowGuests":{"label":"Verbied samenwerking met gebruikers buiten de organisatie.","subtext":"Gebruikers worden verplicht om in te loggen als lid van uw organisatie om deellinks te bekijken. Gast toegang zal worden uitgeschakeld."},"shareContacts":{"label":"Maak alle organisatie gebruikers zichtbaar voor elkaar","subtext":"Deze instelling voegt gebruikers binnen een organisatie automatisch toe aan elkaars contactenlijst zodat ze korrels met elkaar kunnen delen. Schakel dit uit als u gebruikers heeft wiens identiteit verborgen moet blijven voor anderen."}},"saveButton":"Save and continue","backButton":"Back"},"setupWizardEmailConfig":{"title":"Email bezorging","explanation":"Sandstorm heeft een manier nodig om email te verzenden.  U kunt deze stap overslaan (tenzij u gebruik maakt van email login), maar email gerelateerde functionaliteit zal niet beschikbaar zijn totdat u email in de toekomst configureerd.","moreDetail":"<strong>Hoe Sandstorm email gebruikt:</strong> Sandstorm en Sandstorm apps kunnen email notificaties verzenden. Elke app kriijgt een uniek email adres, dus Sandstorm moet email kunnen verzenden van meerdere \"Van:\" adressen. Gmail SMTP zal daarom niet werken, maar er zijn andere gratis diensten beschikbaar. Bekijk voor meer informatie <a href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">de documentatie</a>.","error":"Opslaan van veranderingen mislukt: %s","hostPort":{"host":"SMTP host","port":"Poort","subtext":"Gebruik poort 25 voor SMTP (on-versleuteld of met StartTLS), port 465 voor met TLS versleutelde SMTPS."},"username":{"label":"Gebruikersnaam (optioneel)"},"password":{"label":"Wachtwoord (optioneel)"},"email":{"label":"Email adres van de Sandstorm server","subtext":"Sandstorm zal login emails, notificatie van goedgekeurde toegangsverzoeken en andere emails vanaf dit adres versturen. Dit adres wordt ook gebruikt in de SMTP envelop."},"saveButton":"Opslaan en doorgaan","skipButton":"Overslaan voor nu","testButton":"Test","backButton":"Terug"},"setupWizardPreinstalled":{"title":"Vooraf geïnstalleerde apps setup","explanation":"Wacht alstublieft terwijl Sandstorm de volgende Productiviteitssuite apps installeert die nuttig zijn voor de meeste gebruikers. U kunt alle vooraf geïnstalleerde apps configureren in het Administrator panel na de installatie.","downloaded":"Gedownload","failed":"Mislukt","loading":"Een ogenblik geduld alstublieft, app index laden...","nextButton":"Volgende","skipButton":"Overslaan voor nu","backButton":"Terug"},"setupWizardLoginUser":{"title":"Administrator account aanmaken","firstLogin":{"title":"Bevestig uw profiel"},"isAdmin":"Goed! Uw account heeft beheerdersrechten.","hasAdmin":{"explanation":"Deze Sandstorm server heeft al een administrator gebruiker, maar als u wilt kunt u ook uzelf een administrator maken.","adminButton":"Word een administrator"},"makeYouAnAdmin":"Bezig met u een administrator maken op deze server...","loggedOut":{"explanation":"U kunt nu een administratoraccount aanmaken op deze Sandstorm server.","description":"Log in om uw administratoraccount aan te maken","dialogLabel":"Account aanmaken"},"finishButton":"Voltooien","backButton":"Terug"},"setupWizardSuccess":{"title":"Klaar voor gebruik!","serverIsReady":"Uw Sandstorm server is nu klaar voor gebruik.","whatsNext":"Wat wilt u als volgende doen?","addUsersLink":"Gebruikers toevoegen","editSettingsLink":"Andere instellingen bewerken","sandstormLink":"Sandstorm gaan gebruiken","backButton":"Terug"},"setupWizardVerifyToken":{"title":"Token verifiëren...","explanation":"Sorry, het lijkt dat uw setup token is verlopen.","howToFix":"Genereer een nieuwe vanaf de opdrachtregel door <code>sandstorm admin-token</code> uit te voeren om toegang te krijgen tot de installatiewizard."},"setupWizardTokenExpired":{"title":"Setup token benodigd","alreadySetup":"Deze server is al ingesteld, we raden het dus aan om deze te beheren via het hoofd","adminRootLink":"administratie panel","explanation":"Om deze servers setup pagina te gebruiken heeft u een geldig setup token nodig.","howToFix":"Genereer een nieuwe vanaf de opdrachtregel door <code>sandstorm admin-token</code> uit te voeren en de link te volgen die het uitgeeft."}},"packages":{"topbar":{"updated":"Sandstorm is bijgewerkt - Klik hier om te herladen","apps":"Apps","grains":"Korrels","oasis":"Begin met anderen samen te werken door Sandstorm op Oasis te gebruiken:","createAccount":"Maak een gratis account aan »","demoHint":"Uw demo data zal worden overgebracht.","selfHostHint":"Of installeer Sandstorm voor uw team (het is open source):","signUp":"Meld hierbinnen aan:","close":"Sluiten"},"powerbox":{"title":"Powerbox aanvraag","inputToken":"Voer alstublieft een api token in:","importToken":"Importeer token","selected":"Geselecteerd:","selectOne":"Selecteer er een:","noGrain":"Er zijn geen korrels die de gevraagde interface kunnen leveren.","loading":"Laden..."}}});
TAPi18n._registerServerTranslator("nl", namespace);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"zh-CN.i18n.json":function(require){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// i18n/zh-CN.i18n.json                                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _ = Package.underscore._,
    package_name = "project",
    namespace = "project";

if (package_name != "project") {
    namespace = TAPi18n.packages[package_name].namespace;
}
TAPi18n.languages_names["zh-CN"] = ["Chinese (China)","简体中文"];
if(_.isUndefined(TAPi18n.translations["zh-CN"])) {
  TAPi18n.translations["zh-CN"] = {};
}

if(_.isUndefined(TAPi18n.translations["zh-CN"][namespace])) {
  TAPi18n.translations["zh-CN"][namespace] = {};
}

_.extend(TAPi18n.translations["zh-CN"][namespace], {"error":"错误：%s","shell":{"layout":{"demoExpired":{"title":"Demo Expired","canUpgrade":"Thanks for trying the Sandstorm Demo! Your demo account has expired. If you'd like to continue using Sandstorm and make your data permanent, please sign in.","cannotUpgrade":"Thanks for trying the Sandstorm Demo! Your demo account has expired.","signOut":"Sign out"},"adminAlert":{"icon":"!","title":"管理员的信息"},"demoModal":{"header":"Sandstorm is a self-hostable web productivity suite.","withAppDemo":"with apps like %s.","withoutAppDemo":"with all the apps you need in one place.","startButton":{"withAppDemo":"Try %s »","withoutAppDemo":"Start »"},"explanation":"<p>Sandstorm makes it easy to run apps on your own server. In addition to improving privacy and control, <a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\" target=\"_blank\">this brings Open Source apps to a wide audience.</a></p>","securityOverview":"<p>Sandstorm is also ridiculously secure. If security interests you, check out <a href=\"https://docs.sandstorm.io/en/latest/developing/security-practices/\" target=\"_blank\">Sandstorm's security model overview</a>.</p>"}},"notifications":{"hint":"通知","text":"通知"},"notificationsPopup":{"title":"通知","empty":"没有通知……"},"loading":{"title":"正在读取……","text":"正在读取……"},"notFound":{"title":"查无此沙","text":"404 无法找到该沙粒 :("},"root":{"title":"首页"},"grainLogContents":{"prefix":"……"},"grainLog":{"title":"故障排查记录: %s"},"signup":{"title":"申请注册","signedUp":{"keyIsUsedWithOrigin":"注册成功！请前往<a href=\"https://apps.sandstorm.io/?host=%s\">下载应用程序</a>。","keyIsNotUsed":"您已经成功注册，不再需要该密钥。"},"keyIsUsed":"抱歉，该注册密钥已被使用。","processing":"请稍候……","signInGuide":"请点击右上角“登入”按钮。","invalidKey":"抱歉，该注册密钥无效。"},"uploadStatus":{"title":"正在上传……","uploadFailedWithError":"上传失败：%s %s","statusWithProgress":"%s……%s","status":"%s……","uploadFailed":"上传失败，请重试。"},"uploadTest":{"title":"Upload test page","explanation":"This page is intended for use by the test scripts.","uploadApp":"Upload app:","uploadBackup":"Upload backup:"},"referrals":{"title":"Referral Program","header":{"isPaid":{"title":"Get up to <span class=\"reward-amount\">30 GB</span> of free space when your friends share on Oasis!","explanation":"For each friend that creates a sharing link, we'll reward you with 2 GB."},"isNotPaid":{"title":"Get unlimited grains &amp; up to <span class=\"reward-amount\">2 GB</span> of free space when your friends share on Oasis!","explanation":"For each friend that creates a sharing link, we'll reward you with 50 MB. You'll also get unlimited grains."}},"howTo":{"title":"Sharing is referring","explanation":"To get started, open any grain and click Share access.","getStart":{"step1":"You share a grain with your friend (who isn't already an Oasis user).","step2":"Your friend signs in.","step3":"Your friend shares a grain with any other person (including you)."}},"status":{"title":"Your referrals & status","explanation":"Once you send a sharing link out, people will appear in this list after they sign in.","pending":{"title":"Still needs to share","explanation":"Tell these people to make a grain and share it with you. You must actually view the grain they shared with you!"},"success":{"title":"Successful referrals","explanation":"You've successfully referred these people. Congratulations!"}},"finePrint":{"title":"The fine print","warningWithAccountLink":"Your referral bonus lasts until 1 year after the referral. The terms of this referral program may change without warning; however, such changes will not affect bonuses you have already received. If you need even more space, <a href=\"%s\">upgrade your account.</a>","explanation":"Paid subscribers can get up to 30 GB of free storage, 2 GB per person they refer, in addition to the storage from their account level. Free account holders can earn unlimited grains and 50 MB of free storage per person they refer, up to 2 GB total."}},"about":{"title":"关于沙聚云","intro":{"explanation":"<a href=\"https://sandstorm.io\">沙聚云(Sandstorm.io)</a>是让用户运行私有网络应用程序的一个开源平台。用户可以上传并安装各种应用，就像手机的应用商店一样。<a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\">开源网络应用程序既优雅又正确的打开方式</a>应该是这样。","version":"版本：%s","unofficialWarning":"<strong>WARNING:</strong> This server is running an unofficial build of Sandstorm which may contain modifications not reviewed by the developers."},"changelog":{"title":"最近更新"},"developers":{"title":"核心开发者"},"advisors":{"title":"顾问"},"corporate":{"title":"赞助者","explanation":"These companies contributed heavily to Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaign</a>."},"individual":{"title":"Key indivisual sponsors","explanation":"These individuals contributed heavily to Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaign</a>."},"backers":{"title":"Backers","explanation":"from <a href=\"http://igg.me/at/sandstorm\">our Indiegogo campaign &gt;</a>","anonymous":"and %s anonymous contributors","loading":"loading..."},"dependencies":{"title":"Third-party works used","license":"license","explanation":"These works are copyright the respective authors and used under the terms of the respective licenses."},"copyright":{"title":"著作权","license":"The Sandstorm implementation is copyright by Sandstorm Development Group, Inc. and contributors. The implementation is licensed under the Apache License 2.0.","trademark":"\"Sandstorm\" and the Sandstorm logo are trademarks of Sandstorm Development Group, Inc., and may not be used to advertise derivative works nor commercial services without written consent from Sandstorm Development Group, Inc."},"terms":{"license":"许可","terms":"服务条款","privacy":"隐私声明"}}},"grains":{"title":"沙粒","grainView":{"notFoundWithId":"找不到 ID 为 %s 的沙粒。","openInIncognitoMode":"以无痕模式打开"},"grainInMyTrash":{"explanation":"这颗沙粒正在回收站中，需要捞回主清单方可重新使用。","restore":"捞回主清单"},"grainInOwnersTrash":{"explanation":"无法使用这颗沙粒，因其已被丢进回收站。"},"grainOwnerSuspended":{"explanation":"无法使用这颗沙粒，因其拥有者已被停权。"},"wrongIdentity":{},"requestAccess":{"permissionDenied":"无权存取此沙粒。","requestAccessHint":"要求存取权","requestAccess":"要求存取权","warnIdentityRevealing":"要求存取权的话，需要将您的完整身份向沙粒拥有者明示。","sendRequest":"正在送出您的要求……","waitForApproving":"要求已送出，若获核准则会以电子邮件通知您。","requestFailed":"要求失败，原因：%s","signInPlease":"若要要求存取权，请先登入。"},"invalidToken":{"invalidToken":"无效的 token：%s"},"revokedShareLink":{"linkRevoked":"非常抱歉，该链接已经被取消。"},"grainTitle":{"was":"原标题：%s","renamedFrom":"原名：%s"},"grainDeleteButton":{"hint":"移到回收站","text":"移到回收站"},"grainDebugLogButton":{"hint":"显示除错记录","text":"纪录"},"grainBackupButton":{"hint":"下载备份","text":"备份"},"grainBackupPopup":{"title":"下载备份","warning":"注意！恢复该备份会产生无法预料的后果。","processing":"正在准备备份……","cancelHint":"取消","cancel":"取消","confirmHint":"下载备份","confirm":"下载备份"},"grainRestartButton":{"hint":"重新启动应用","text":"重新启动"},"grainApiTokenButton":{"hint":"取得外网密钥(webkey)","text":"取得外网密钥(webkey)"},"grainApiTokenPopup":{"title":"外网密钥","processing":"生成中……","cancel":"取消","copyForExternalApp":"将该密钥复制到外部程序：","back":"返回","description":"外网密钥可以令外部程序存取此应用的数据。","labelTitle":"标签：","labelPlaceholder":"例如「Android app」","roleTitle":"角色：","permissionTitle":"完整存取","create":"建立","webkeyListTitle":"现有的外网密钥：","revokeHint":"取消","revoke":"取消"},"grainShareButton":{"hint":"分享此沙粒","text":"分享"},"whoHasAccessPopup":{"title":"可存取者","noLoggedInUsers":"其他用户不曾访问过该链接。","nobody":"未授予任何人权限。","addedBy":"分享人：","revokeHint":"取消","revoke":"取消","hasCustomPermissions":"[拥有特殊权限]"},"selectRole":{"hint":"选择角色"},"emailInviteTab":{"demoModeAlert":"Sharing emails cannot be sent while in demo mode. Create an account to send sharing emails, or get a shareable link instead.","personalMessagePlaceholder":"（选填）附加个人信息……","send":"发送","sending":"正在发送……","success":"成功发送。","reset":"+ 邀请更多人","startOver":"<< 重新开始"},"shareableLinkTab":{"withRoles":"任何知道链接的人：","withoutRoles":"任何知道该链接的人都可存取此沙粒。","labelPlaceholder":"标签（选填）","labelExplanation":"标签可备注该链接是为何人创建的。","create":"创建","creating":"正在创建……","success":"请复制这个链接：","reset":"+ 建立另一个链接","cancel":"取消"},"shareWithOthers":{"title":"分享给别人","sendInviteTab":"发送邀请","shareableLinkTab":"获得可分享的链接","whoHasAccess":"查阅权限"},"grainSharePopup":{"oldSharingModelDescription":"This grain uses the old sharing model. You can share it by copy/pasting the URL from the location bar","upgradeDescription":"You can upgrade this grain to the new model, but anyone with whom you shared this grain previously will lose access; you will have to share with them again.","upgradeWarning":"Upgrading cannot be undone.","upgrade":"upgrade","openInIncognitoMode":"您正以无痕模式浏览此沙粒。","toShareAccess":"请复制以下链接，进行分享：","openNonAnonymouslyHead":"更多的分享选项","openNonAnonymouslyLink":"需要登入方可使用","openNonAnonymouslyTail":"。","signIn":"其他分享选项需要登入方可使用。"},"grainPowerboxRequest":{"powerboxRequestHint":"Powerbox 要求","powerboxRequest":"P"},"grainPowerboxOffer":{"powerboxOfferHint":"Powerbox 命令","powerboxOffer":"P"},"grainPowerboxOfferPopup":{"title":"Powerbox 命令","dismiss":"隐藏"},"share":{"grainNotFoundWithId":"找不到 ID 为 %s 的沙粒，请尝试以其他账号登入。","signIn":"必须登入方可分享。"},"grainlist":{"sandstormGrainListPage":{"introButton":{"hint":"什么是沙粒？"},"mainListButton":"查阅主列表","trashButton":"查阅回收站(%s)","restoreButton":"恢复备份……","search":{"placeholder":"搜索"},"usageInfo":{"quota":"__quota__ 沙粒中 __count__ 个","size":"共 __total__，已用 __size__"},"trash":{"emptyButton":"清空回收站","explanation":"（回收站内的沙粒会在 30 天后被自动移除。若要保留沙粒，请将其移回主列表。）"},"noGrains":{"notFound":"__<strong>找不到相符的沙粒。</strong>","inTrash":"回收站内有沙粒符合搜索条件。","trashButton":"查阅回收站","empty":"__<strong>回收站是空的。</strong>","explanation":"您没有任何沙粒。","hasApps":"选择应用程序来创建您的第一粒沙。","hasNoApp":"安装应用程序来创建您的第一粒沙。"}},"sandstormGrainTable":{"bulkAction":{"disabledHint":"请从列表中选择沙粒，再进行批量操作。"},"tableHeader":{"title":"沙粒名称","size":"大小","lastUsed":"最后操作","ownership":"拥有者","collaborators":"合作者"},"tableBody":{"was":"（原标题：%s）","renamedFrom":"（原名：%s）","new":"尚未开启","mine":"自己","shared":"别人分享"}},"contactAutocomplete":{"contactInputBox":{"placeholder":"输入电邮地址或姓名"}}}},"notifications":{"appUpdateNotificationItem":{"title":"应用程序有新版本：","appUpdates":"__<strong>__name__</strong>: 版本 __version__ 已发布","submitButton":{"hint":"更新以上所有应用程序","text":"应用更新"},"dismissButton":{"hint":"忽略以上更新","text":"隐藏"}},"mailingListBonusNotificationItem":{"explanationWithBonus":"Subscribe to our announcement list and get <strong>%s bonus storage</strong> (1-2 emails per month).","submitButton":"Subscribe","dismissButton":"No thanks"},"referralNotificationItem":{"paidUser":"Get up to 30 GB bonus with the","unpaidUser":"Unlimited grains & up to 2 GB bonus with the","link":"referral program »","dismissButton":"Dismiss"},"adminNotificationItem":{"explanation":"您提供的匿名资料可以帮助我们改善沙聚云。详情请点击此处。"},"backgroundedGrainNotificationItem":{"title":"__<strong>__grain__</strong> 正在后台执行：","cancelButton":{"hint":"停止后台应用程序","text":"取消"}},"identityChangesNotificationItem":{"explanation":"我们正在改变沙聚云的鉴权机制，您的账户可能会被影响。详情请点击此处。"},"grainActivityNotificationItem":{"title":"<strong>__grain__</strong> 的 __<strong>__sender__</strong>：","dismissButton":{"hint":"隐藏此通知","text":"隐藏"}}},"powerboxBuiltins":{"uiViewPowerboxConfiguration":{"explanation":"权限：","fullAccess":"完整存取","connectButton":"联机","chooseHostedObjectButton":"浏览沙粒内容 »"},"ipNetworkPowerboxCard":{"explanation":"管理者：允许所有出网连接","encryptedWith":"(以 %s 加密)"},"ipInterfacePowerboxCard":{"explanation":"管理者：允许所有入网连接"},"emailVerifierPowerboxCard":{"verifyWith":"用 %s 验证电邮地址","verifyWithAnyService":"无论何种登入方式均验证电邮地址"},"identityPowerboxConfiguration":{"explanation":"权限：","fullAccess":"完整存取","connectButton":"联机"},"addNewVerifiedEmailPowerboxCard":{"explanation":"新增地址"},"httpUrlPowerboxCard":{"useUrl":"使用 %s"},"httpArbitraryPowerboxCard":{"explanation":"输入网址……"},"httpArbitraryPowerboxConfiguration":{"explanation":"输入网址：","placeholder":"https://example.com","connectButton":"联机"},"httpOAuthPowerboxCard":{"useService":"使用 %s"},"httpOAuthPowerboxConfiguration":{"requstPermissionsFromService":"从 %s 要求权限(请参照弹出窗口)。"}},"styleguide":{"title":"Sandstorm style guide","note":"(note: this is a work in progress.)","copyRules":{"title":"Copy rules","rule1":"Be terse but clear.  Avoid technical terms where less-technical ones will do.","rule2":"Capitalize the first letter of the first word of a section heading or button.  Do not capitalize the second word, or anything else, unless referring to a proper noun like Google or GitHub.","rule3":"Use verbs for text on buttons. The button text should describe what clicking that button will do.","rule4":"Use nouns for text which is a link.  The link text should indicate what the link will take you to.","rule5":"Use nouns for basically everything else, too, to communicate the information architecture. Section headings should be nouns which describe the content of the section.  Labels should be nouns which describe the content of the field."},"styleRules":{"title":"Style rules","rule1":"Stick to a limited color palette.  (TODO: add specific colors)","rule2":"Use typefaces no smaller than 13px for text content.  Readability is important and not everyone has young eyes.","rule3":"Make click targets no smaller than 32px square or 26px tall if appreciably wider.  Remember that due to Fitts' Law, larger click targets are easier to interact with."},"forms":{"title":"Forms","rule1":"Use the <code>standard-form</code> class where appropriate.","rule2":"The primary action button of a form should be at the bottom right of the form. Secondary buttons should line up to the left of that primary action button.","rule3":"When providing text input fields, place labels above the input, not side-by-side or inside the field as placeholder text.  Place additional explanation or instructions below the input. Placeholder text should be used for examples, not instructions.","rule4":"Forms are generally used for transactional interactions, where the user cares about the success or failure of the application.  Avoid optimistic UI in favor of multi-state UI which communicates network activity, success, and failure.  A common pattern is:","rule4-1":"Have the default state of the form.  Disable a save button until the user makes a change.","rule4-2":"When the user clicks an action button, start the RPC, and disable the entire form to indicate that the action is underway.","rule4-3":"When the RPC completes, if the RPC failed, show an error message at the top of the context over which the action applies.  If the RPC succeeded, either indicate a success message (again, at the top of the context over which the action applies) or if this was a modal interaction and the completion of the RPC indicates successful completion of the workflow, dismiss the form with no additional message."},"example":{"title":"An example:","label":"Email","placeholder":"alice@example.com","subtext":"You might want to receive notifications.","submitButton":"Save","discardButton":"Discard"}},"admin":{"title":"管理员","admin":{"newAdmin":{"wildcardHostBroken":"警告：本服务器的泛域名设置可能有误。在修复此问题前，您无法使用任何应用程序。请参照<a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#why-do-i-see-an-error-when-i-try-to-launch-an-app-even-when-the-sandstorm-interface-works-fine\">本篇沙聚云文件</a>和您浏览器的 Javascript 记录取得更多信息。您必须调整 DNS、SSL/TLS 证书，或编辑 sandstorm.conf 文件。如果您想要测试问题是否已修复，请重新整理此页面。如果您仍然有问题，请<a href=\"https://github.com/sandstorm-io/sandstorm/issues\">在此提出 issue</a>。","websocketBroken":"警告：本服务器似乎无法建立 WebSocket。沙聚云也许看起来没有问题，但许多应用程序会无法正常执行。<a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#how-do-i-enable-websockets-proxying-or-why-do-some-apps-seem-to-crash-reload\">更多信息在此。</a>通常这是因为上层的 reverse proxy （比如 nginx 或 Apache）需要被重新设定。如果您认为不是，请重新整理此页面或<a href=\"https://github.com/sandstorm-io/sandstorm/issues\">在此开一个 issue</a>。","hasSetupToken":"您并非以管理者身份登入，但您的设置 token 是正确的。您现在可以<a href=\"/setup\">启动设置精灵</a>或<a href=\"/admin/status\">检视系统记录</a>。","hasNoSetupToken":"您并非以管理者身份登入，无法使用管理面板。请以管理者身份登入，或在主机终端上执行`sandstorm admin-token`来取得管理 token。"},"newAdminRoot":{"configuration":{"title":"设定"},"management":{"title":"管理"}}},"identityProviders":{"name":"身份提供者","subtext":"設定使用者可以如何登入。","newAdminIdentity":{"explanation":"选择使用者可以用哪些方法进行身份认证。所有人都能登入，但只有您邀请的用户可以安装应用程序或建立沙粒。","organizationLinkPrefix":"要设定组织内容，请到","organizationLink":"组织设置","organizationLinkPostfix":"."},"adminIdentityProviderConfigureEmail":{"title":"电子邮件登入设定","error":"无法储存变更：%s","warning":"备注：若要使用电子邮件登入，您也需要设定服务器的电子邮件寄送功能。","explanation":"电子邮件登入没有额外的设定选项。","saveButton":"储存","enableButton":"启用","cancelButton":"取消","disableButton":"停用"},"googleLoginSetupInstructions":{"explanation":"首先，您需要取得 Google Client ID。遵循以下步骤：","step1":"造访 <a href=\"https://console.developers.google.com/\" target=\"blank\">https://console.developers.google.com/</a>","step2":"__\"Create Project\", if needed. Wait for Google to finish provisioning.","step3":"__On the left sidebar, go to \"Credentials\" and, on the right, \"OAuth consent screen\". Make sure to enter an email address and a product name, and save.","step4":"__On the left sidebar, go to \"Credentials\".  Click the \"Create credentials\" button, then select \"OAuth client ID\" as the type.","step5":"__Select \"Web application\" as your application type.","step6":"__Set Authorized Javascript Origins to: <span class=\"idp-config-url\">%s</span>","step7":"__Set Authorized Redirect URI to: <span class=\"idp-config-url\">%s/_oauth/google</span>","step8":"__Finish by clicking \"Create\"."},"adminIdentityProviderConfigureGoogle":{"title":"__Google login configuration","formerBaseUrlWarning":"__This server moved from <code>__former__</code> to <code>__current__</code>. You'll need to visit <a target=\"_blank\" href=\"https://console.developers.google.com/\">https://console.developers.google.com/</a> and update the \"Authorized Javascript Origins\" and \"Authorized Redirect URI\" for this Client ID to include the values below.","error":"__Failed to save changes: %s","explanation":"__Then, copy over your Client ID and Client secret below:","clientId":"__Client ID:","clientSecret":"__Client secret:","saveButton":"__Save","enableButton":"__Enable","cancelButton":"__Cancel","disableButton":"__Disable"},"githubLoginSetupInstructions":{"explanation":"__First, you'll need to get a Github Client ID and Client secret. Follow these steps:","step1":"__Visit <a href=\"https://github.com/settings/applications/newi\" target=\"blank\">https://github.com/settings/applications/new</a>","step2":"__Set Homepage URL to: <span class=\"idp-config-url\">%s</span>","step3":"__Set Authorization callback URL to: <span class=\"idp-config-url\">%s_oauth/github</span>"},"adminIdentityProviderConfigureGitHub":{"title":"__GitHub login configuration","formerBaseUrlWarning":"__This server moved from <code>__former__</code> to <code>__current__</code>. You'll need to visit <a target=\"_blank\" href=\"https://github.com/settings/developers\">https://github.com/settings/developers</a> and update the Homepage URL and Authorization callback URL for this application to the values below.","explanation":"__Then, copy over your Client ID and Client secret below:","clientId":"__Client ID:","clientSecret":"__Client secret:","saveButton":"__Save","enableButton":"__Enable","cancelButton":"__Cancel","disableButton":"__Disable"},"adminIdentityProviderConfigureLdap":{"title":"__LDAP login configuration","error":"__Failed to save changes: %s","url":{"label":"__LDAP server URL:","subtext":"__e.g. <code>ldap://ldap.example.com:389</code>.  Use port 636 for SSL."},"searchBindDn":{"label":"__Bind user DN","subtext":"__The DN to bind as when searching for a user in the tree, e.g. <code>cn=admin,ou=users,dc=example,dc=com</code>.  Optional, but needed if your server doesn't allow unauthenticated searches."},"searchBindPassword":{"label":"__Bind user password","subtext":"__Optional, needed if specifying bind DN above."},"base":{"label":"__Base DN","subtext":"__The root object under which to search for user account nodes, e.g. <code>ou=users,dc=example,dc=com</code>"},"searchUsername":{"label":"__LDAP username attribute","subtext":"__The LDAP attribute on a user object which represents the user's username (what they will enter in the username field when signing in)."},"nameField":{"label":"__LDAP given name attribute","subtext":"__The LDAP attribute on a user object which represents the user's full display name."},"emailField":{"label":"__LDAP email attribute","subtext":"__The LDAP attribute on a user object which represents the user's email address."},"filter":{"label":"__Additional LDAP filter criteria","subtext":"__An optional LDAP query fragment that will be included in the user search. This is not commonly used. See <a target=\"_blank\" href=\"http://www.ietf.org/rfc/rfc2254.txt\">RFC2254</a>."},"caCert":{"label":"__LDAP CA certificate","placeholder":"__-----BEGIN CERTIFICATE-----\nMIID...\n-----END CERTIFICATE-----","subtext":"__This is only necessary if your server supports SSL and is using a private CA not in the common trusted set. Note that it does NOT work to enter the LDAP server's own certificate here -- it must be the certificate of a CA which has signed the LDAP server's certificate."},"saveButton":"__Save","enableButton":"__Enable","cancelButton":"__Cancel","disableButton":"__Disable"},"adminIdentityProviderConfigureSaml":{"title":"__SAML login configuration","explanation":"__Your SAML IDP should be configured to return a persistent nameID. In addition, if you are not using Active Directory, you <strong>must</strong> configure your IDP to provide two extra attributes, <code>email</code> and <code>displayName</code>.","serviceUrl":"__The Service URL of this server is: <code>%s</code>","logoutUrl":"__The Service Logout URL of this server is: <code>%s</code>","metadataUrlAt":"__If your SAML IDP supports reading the service provider metadata from a URL, then you can point it at: <code><a href=\"__href__\" target=\"_blank\">__text__</a></code>","error":"__Failed to save changes: %s","entryPoint":{"label":"__SAML provider entry point URL:","placeholder":"__https://YOUR_SAML_PROVIDER.com/sso/saml"},"logout":{"label":"__SAML provider logout URL (optional):","placeholder":"__https://YOUR_SAML_PROVIDER.com/sso/saml/SingleLogoutService"},"publicCert":{"label":"__SAML cert for above provider:","placeholder":"__base64 encoded cert (e.g. MIID...8F==)"},"entityId":{"label":"__Entity ID:"},"saveButton":"__Save","enableButton":"__Enable","cancelButton":"__Cancel","disableButton":"__Disable"},"adminIdentityProviderTable":{"title":"__Identity provider","status":"__Status"},"adminIdentityRow":{"enabled":"__Enabeld","disabled":"__Not enabled","baseUrlChanged":{"explanation":"__Disabled due to BASE_URL change","reconfigureButton":"__reconfigure"},"configureButton":{"hint":"__Configure %s login","text":"__Configure"}}},"organization":{"name":"組織設定","subtext":"讓大團隊可以圓滑地上手及進行合作。","organizationSettingsBlurb":{"explanation":"__Sandstorm allows you to define an organization. You can automatically apply some settings to all members of your organization. Users within the organization will automatically be able to log in, install apps, and create grains."},"newAdminOrganization":{"success":"__Saved changes.","error":"__Failed to save changes: %s","membership":{"title":"__Organization membership","emailToggle":{"label":"__Users authenticated via email address","subtext":"__Note: disabled because email login is not configured."},"emailDomain":{"label":"__Domain:","subtext":"__Users with an email address at this domain will be members of this server's organization."},"gappsToggle":{"label":"__Users authenticated via Google Apps for Work","subtext":"__Note: disabled because Google login is not configured."},"gappsDomain":{"label":"__Domain:","subtext":"__Users with a Google Apps for Work account under this domain will be members of this server's organization."},"ldapToggle":{"label":"__Users authenticated via LDAP","subtext":"__Note: disabled because LDAP login is not configured."},"samlToggle":{"label":"__Users authenticated via SAML","subtext":"__Note: disabled because SAML login is not configured."}},"settings":{"title":"__Organization settings","disallowGuests":{"label":"__Disallow collaboration with users outside the organization.","subtext":"__Users will be required to log in as a member of your organization to view sharing links. Guest access will be disabled."},"shareContacts":{"label":"__Make all organization users visible to each other","subtext":"__This setting automatically adds users within the organization to each other's contact list so that they can share grains with each other.  Disable this if you have some users whose identity should stay hidden from other users."}},"saveButton":"__Save"}},"emailConfig":{"name":"電郵寄送設定","subtext":"設定沙聚雲如何寄送電郵。","newAdminEmailConfig":{"outbound":{"title":"__Outbound email"},"howItWorks":{"title":"__How Sandstorm uses email","explanation":"__Sandstorm sends email notifications, and Sandstorm apps can send email notifications, too. Every app receives a unique email address, so Sandstorm needs to send email from a variety of \"From:\" addresses. Therefore, Gmail SMTP won't work, but there are other free services. See <a href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">more info in the docs</a>."},"success":"__Saved changes.","error":"__Failed to save changes: %s","address":{"hostname":"__SMTP host","port":"__Port","subtext":"__Use port 25 for SMTP (unencrypted or with StartTLS), port 465 for TLS-encrypted SMTPS."},"username":{"label":"__Username (optional)"},"password":{"label":"__Password (optional)"},"returnAddress":{"label":"__Sandstorm server's own email address","subtext":"__Sandstorm will send login emails, notifications of granted access requests, and other emails from this address. It will also use this address in the SMTP envelope."},"submitButton":"__Save Configuration","testButton":"__Test","disableButton":"__Disable email"},"emailTestPopup":{"title":"__Send a test email","testAddress":{"label":"__Send test mail to:"},"pending":"__Sending...","submitButton":"__Send test email","closeButton":"__Close"},"emailDisablePopup":{"title":"__Really disable email?","explanation":"__This will disable all outbound email delivery for this server, including sharing via email, email token login (if enabled), and email notifications to users.","disableButton":"__Disable email","closeButton":"__Close"}},"personalization":{"name":"個人化","subtext":"客製獨自的啟動畫面、服務條款、隱私權聲明等等。","newAdminPersonalization":{"serverTitle":{"label":"__Server title:","subtext":"__The name of this server, as used when communicating with users."},"splashUrl":{"label":"__Splash URL (experimental):","subtext":"__This web page will be displayed in the background behind the login dialog (the home page when logged out). For security reasons, the page must be hosted within this Sandstorm server's wildcard host (otherwise it will be blocked by <code>Content-Security-Policy</code>). We suggest using a static web publishing app like <a target=\"_blank\" href=\"https://apps.sandstorm.io/app/nqmcqs9spcdpmqyuxemf0tsgwn8awfvswc58wgk375g4u25xv6yh/?host=%s\">Hacker CMS</a> to host content.<strong>WARNING: This feature is experimental; in particular the style and positioning of the login box is subject to change without notice.</strong> Please let us know if you'd like to see it stabilize."},"signupDialog":{"label":"__Signup dialog:","subtext":"__This text will be displayed to users who click on signup links."},"termsOfService":{"label":"__Terms of service URL:","placeholder":"__https://example.com/tos","subtext":"__If provided, new users will be required to agree to the linked terms of service."},"privacyPolicy":{"label":"__Privacy policy URL:","placeholder":"__https://example.com/privacy","subtext":"__If provided, new users will be required to agree to the linked privacy policy."},"whitelabeling":{"title":"__Whitelabeling","logo":{"label":"__Custom logo on sign-in dialog","resetButton":"__Use default","uploadButton":"__Upload logo"},"hideTroubleshooting":{"label":"__Hide troubleshooting button"},"hideAbout":{"label":"__Hide \"About Sandstorm\" links"},"hideSendFeedback":{"label":"__Hide \"Send feedback\" link"},"useServerTitleForHomeText":{"label":"__Use server title (above) for home link","subtext":"__If enabled, changes the default label \"Sandstorm\" on the link at the very top left to match whatever you specified for the Server title above."},"customLoginProviderName":{"label":"__Custom login button text","subtext":"__Replaces the text \"with LDAP\" and \"with SAML\" on the login page with the specified text."}},"saveButton":"__Save"}},"preinstalledApps":{"name":"預裝應用程式","subtext":"選擇新使用者可以直接使用的預裝應用程式。","newAdminPreinstalledApps":{"explanation":"__Apps you configure on this page to be pre-installed will automatically be installed (show under /apps) for new users.","productivitySuite":{"label":"__Productivity Suite","subtext":"__We recommend pre-installing these core apps that are useful to most users."},"systemSuite":{"label":"__System apps","subtext":"__We recommend pre-installing these system apps that greatly enhance the abilities of Sandstorm."},"allApps":{"label":"__All apps on market","subtext":"__The checked apps below will be installed by default for all new users.","lessButton":"__Show fewer apps","moreButton":"__Show %s more apps"},"saveButton":"__Save"},"_appRow":{"download":{"success":"__Downloaded","failed":"__Failed"}}},"appSources":{"name":"應用程式來源","subtext":"取得應用程式和更新的位置。","newAdminAppSources":{"appMarketUrl":{"label":"__App market URL","subtext":"__The \"Install...\" button on the Apps page will link to this URL."},"enableAppUpdates":{"label":"__Enable app update notifications","subtext":"__Updates to apps will be automatically downloaded by the Sandstorm server daily and notifications sent to users asking if they want to update their installed apps."},"appIndexUrl":{"label":"__App index URL","subtext":"__This is the URL that the Sandstorm server will check to learn about app updates."},"saveButton":"__Save","restoreButton":"__Restore defaults"}},"networking":{"name":"網路","altName":"安全性","subtext":"控制如何存取網路。","newAdminNetworking":{"title":"__Security","ipBlacklist":{"label":"__Server-side request IP blacklist:","subtext":"__Users will be prohibited from making requests to these IP addresses. This includes making a request from an app, downloading an SPK file from a user-provided URL, etc. You may specify one IP address or network (in CIDR notation, e.g. \"127.0.0.0/8\") per line. The default value includes standard local and private network addresses. Note that when an HTTP proxy is in use, this setting may be ignored; the proxy must implement its own blacklist."},"saveButton":"__Save","restoreButton":"__Restore defaults"}},"users":{"name":"使用者","subtext":"管理此沙聚雲上的使用者。"},"userAccounts":{"newAdminUserTableRow":{"admin":{"hint":"__Admin","cap":"__A"},"user":{"hint":"__User","cap":"__U"},"visitor":{"hint":"__Visitor","cap":"__V"},"unknownIdentity":"__Unknown identity","unknownName":"__unknown name","unknownIdentifier":"__unknown identifier","manageButton":"__Manage"},"newAdminUserTable":{"header":{"created":"__Created","lastActive":"__Last Active","actions":"__Actions"},"inviteLink":"__Invite new user","notFound":"__No users matched your filter.","processing":"__Loading users..."},"newAdminUsers":{"search":{"placeholder":"__search"},"filter":{"explanation":"__Show:","admin":"__Admin","user":"__User","visitor":"__Visitor"}}},"userDetails":{"newAdminUserDetailsIdentityTableRow":{"profilePicture":{"label":"__Profile picture"},"handle":{},"pronouns":{"hint":"__Pronouns","label":"__Pronouns"},"emails":{"hint":"__Emails","label":"__Emails","primary":"__Primary","verified":"__Verified","unverified":"__Unverified","notFound":"__No known emails."},"properties":{"login":"__Can log in","org":"__Org. member"},"unkonwn":{"profilePicture":{"cell":"__(unknown name)"},"service":{"hint":"__unknown service","label":"__(unknown service)","cell":"__(unknown name)"},"handle":{"cell":"__(unknown handle)"},"pronouns":{"cell":"__(unknown pronouns)"},"created":{"cell":"__unknown creation date"},"lastActive":{"cell":"__unknown activity date"}}},"newAdminUserDetailsIdentityTable":{"header":{"created":"__Created","lastActive":"__Last Active","properties":"__Properties"}},"newAdminUserDetails":{"role":{"title":"__Role","admin":{"label":"__Admin","explanation":"__Manages all accounts.  Can install apps, create grains, and administer the server.","isAdmin":"__Current role","isNotAdmin":"__Promote to admin"},"user":{"label":"__User","explanation":"__Can install apps and create grains.","isUser":"__Current role","isAdmin":"__Demote to user","isNotUser":"__Promote to user"},"visitor":{"label":"__Visitor","explanation":"__Can use and re-share shared grains, but cannot create grains of their own nor install apps.","isVisitor":"__Current role","isNotVisitor":"__Demote to visitor"}},"identities":{"title":"__Identities"},"status":{"title":"狀態","deleting":"停權 - 即將刪除 %s","suspended":"停權","active":"活躍","unsuspendButton":"解除停權","suspendButton":"將帳號停權","deleteButton":"刪除帳號"},"deletePopup":{"title":"刪除 %s","explanation":"您正在刪除 %s 的帳號。被標記為刪除中的帳號會被停權，管理者有七天的時間取消刪除動作。（在本頁點擊「解除停權」按鈕即可）%s <strong>無法</strong>阻止停權和刪除。","detail":"七天後， %s 的帳號將被永遠刪除。刪除帳號將：","detail1":"永遠刪除所有 %s 建立的沙粒","detail2":"撤銷所有 %s 建立的共享連結","detail3":"將 %s 從所有人的聯絡人名單中移除","detail4":"刪除所有本帳號的識別資訊（除非該識別資訊亦被別的帳號使用）","deleteButton":"刪除帳號","cancelButton":"取消"},"suspendPopup":{"title":"將 %s 停權","detail":"將這個帳號停權將會：","detail1":"阻止所有 %s 建立的沙粒被開啟","detail2":"停用所有 %s 建立的共享連結","detail3":"本帳號的識別資訊亦等同辦理（除非該識別資訊亦被別的帳號使用）","suspendButton":"將帳號停權","cancelButton":"取消"},"loading":"正在讀取使用者資料……"}},"userInvite":{"newAdminUserInviteLink":{"explanation":"您可以建立連結來邀請其他人創建他們的帳號。您邀請的人將直接擁有使用者權限，可以安裝應用程式及建立沙粒。如果您只想要分享一顆沙粒，請使用該沙粒畫面內的「分享存取」選單。","title":"建立連結","invite":{"explanation":"建立可以寄送給別人的帳號創建連結。","createButton":"建立連結"}},"newAdminUserInviteEmail":{"title":"寄送電郵","emailWarning":{"explanation":"電郵尚未設定完成，故此功能已被停用。","configureLink":"設定電郵"},"explanation":"建立連結並以電郵寄出。","to":{"label":"寄給：","placeholder":"alice@email.com\nbob@example.com","subtext":"每行輸入一個電郵位址。"},"subject":{"label":"標題："},"body":{"label":"內容：","subtext":"<code>$KEY</code> 會被置換成產生出來的連結。"},"sendButton":"寄送邀請"},"newAdminUserInvite":{"title":"邀請"}},"systemStatus":{"name":"系統記錄","subtext":"檢視沙聚雲的除錯記錄。","newAdminLogContents":{"ellipsis":"……"},"newAdminStatus":{"downloadButton":"下載完整記錄"}},"stats":{"name":"統計","subtext":"檢視伺服器上的各種統計數據。","header":{"date":"日期","users":"使用者","grains":"沙粒","appName":"應用程式名","owners":"擁有者","sharedUsers":"共享使用者","daily":"每日","weekly":"每週","monthly":"每月","forever":"所有時間"},"body":{"notFound":"目前還沒有統計資料。","loading":"讀取中……"},"newAdminStats":{"share":{"title":"Publish anonymously to Sandstorm dev team","explanation":"We'd like to know how many Sandstorm users there are, to make graphs and such.  Is it okay to send the numbers on this page once a day to the Sandstorm team, so that we can add them to the totals?  Any stats you send are anonymous.","enableButton":"Send anonymous stats","disableButton":"Don't send","control":{"explanation":"The developers of Sandstorm like to know how many Sandstorm users there are, to make graphs and such.  You can help us by opting in to sharing the numbers on this page once a day to the Sandstorm team, so that we can add them to our totals. Any stats shared are submitted anonymously.","label":"Anonymous stats reporting to sandstorm.io:","enabled":"Enabled","disabled":"Disabled","disableButton":"Disable","enableButton":"Enable"}},"json":{"title":"以 JSON 檢視","explanation":"If you want to integrate Sandstorm stats with an internal stats tool, use this secret link to retrieve the tables below as JSON. You can revoke the current link at any time by generating a new link.","label":"統計 JSON 連結：","regenerateButton":"生成新連結"},"userGrains":{"title":"使用者和沙粒","current":"現在： __users__ 名使用者， __grains__ 顆沙粒","today":"今天： __users__ 名使用者， __grains__ 顆沙粒"},"apps":{"title":"應用程式","label":"選擇日期："}}},"maintenanceMessage":{"name":"維修訊息","subtext":"告知使用者維修所需時間。","newAdminMaintenance":{"explanation":"You can set a maintenance message which will be shown to all users in the menu bar, on the top right. This is useful for announcing scheduled maintenance so users can be prepared to tolerate downtime. If the message text is empty, no message will be displayed.","variables":"You may use the following variables in the \"Message text\" field, which will be automatically substituted with useful values:","variable1":"<code>$TIME</code> : The time component of \"Maintenance time\", converted to the user's local time (eg. \"1:15:45 PM\")","variable2":"<code>$DATE</code> : The date component of \"Maintenance time\", converted to the user's local time (eg. \"6/19/2015\")","variable3":"<code>$IN_COUNTDOWN</code> : An active countdown until \"Maintenance time\" (eg. \"in 3 days\" or \"in 1 minute\"). Displays \"any moment\" at the time specified in \"Maintenance time\" and then disappears after an hour.","messageText":{"label":"Message text:","subtext":""},"maintenanceTime":{"label":"Maintenance time:","subtext":"The time you intend to start performing maintenance, formatted like \"%s\"."},"url":{"label":"Link URL (optional):","subtext":"If given, the maintenance message will render as a clickable link to the provided URL."},"saveButton":"Set message"}},"networkCapabilities":{"name":"網路容量","subtext":"管理沙粒的網路直接存取量。","newAdminNetworkCapabilitiesTableCapabilityRow":{"adminAccount":"Admin account %s","unknownGranter":"Unknown granter","anyone":"Anyone with the token","unknownRecipient":"Unknown recipient","revoked":"已撤銷","revokeButton":"撤銷"},"newAdminNetworkCapabilitiesTable":{"header":{"grantBy":"Granted by","grantTo":"Granted to","grantTime":"Time","actions":"Actions"},"notFound":"No capabilities found."},"newAdminNetworkCapabilitiesSection":{"filter":{"label":"Showing:","active":"Active (%s)","revoked":"Revoked (%s)"}},"newAdminNetworkCapabilities":{"networkCaps":"Outbound network access","networkCapsTls":"TLS-encrypted outbound network access","interfaceCaps":"Inbound network access"}},"hostingManagement":{"name":"Hosting management","newAdminHostingManagement":{"explanation":"This feature is experiemental and only works with the LDAP identity provider at the moment. Use with caution.","quotaEnabled":{"label":"Enable quota for users","subtext":"Enable Quota management on this server. This requires both SAML and LDAP to be configured, and email addresses will be used to match between the two."},"quotaLdapAttribute":{"label":"LDAP quota attribute","subtext":"This is the attribute name that will be looked up from LDAP to determine quota for a user."},"billingPromptUrl":{"label":"Billing prompt url","subtext":"This is the url that will be used in the billing prompt."},"saveButton":"Save","restoreButton":"Restore defaults"}}},"people":{"title":"使用者"},"sharingLinks":{"title":"分享连结"},"apps":{"title":"应用程序","appList":{"uninstallingHint":"移除完成","doneUninstalling":"移除完成","uninstallHint":"移除應用程式","uninstallButton":"批次移除...","uploadApp":"上传应用程序","searchHint":"搜尋名稱","search":"搜寻名称","mostUsed":"最常使用","allApps":"所有应用程序","install":"安装新程序","fromMarket":"从市集下载","uninstall":"移除","noMatchApp":"找不到名称含有该字符串的应用程序。请<a href=\"__url__\">试试搜寻市集 &raquo;</a>","unauthorized":"无权限！:(","unauthorizedText":"只有管理者给予权限的的用户才能安装应用程序。若需权限，请联络管理者。"},"install":{"sandstormAppInstallPage":{"title":"安裝應用程式","error":{"title":"無法安裝應用程式","retryButton":"再試一次"},"stepWait":{"title":"正在等待寫入存取……"},"stepDownload":{"title":"正在下載……","cancelButton":"取消"},"stepVerify":{"title":"正在驗證……"},"stepUnpack":{"title":"正在拆包……"},"stepAnalyze":{"title":"正在分析……"},"stepConfirm":{"upgrade":{"explanation":"您已經有安裝此程式的較舊版本。是否要更新？","button":"更新 %s"},"downgrade":{"explanation":"您已經有安裝此程式的較新版本。是否要降版？","button":"降版 %s"},"install":{"button":"安裝 %s"}},"stepRun":{"title":"已安裝 %s","explanation":"安裝完成","appDetailsLink":"開啟 %s"},"stepDelete":{"title":"正在刪除","explanation":"伺服器正在刪除這個應用程式……"},"stepFailed":{"title":"失敗","explanation":"安裝失敗：%s"}}},"appDetails":{"sandstormAppDetailsPage":{"newerVersion":{"explanation":"有些檔案是用較新版本的程式建立的。因為回復舊版本可能會破壞資料，這些檔案將會繼續使用新版本程式。"},"olderVersion":{"explanation":"有些沙粒是舊版本程式建立的。要更新嗎？","upgradeButton":"更新沙粒"},"search":{"hint":"搜尋","placeholder":"搜尋"},"trash":{"explanation":"回收桶裡的沙粒會在 30 天後被永久刪除。若要保留沙粒，請將其移回主列表。"},"noGrains":{"notFound":"<strong>找不到相符的沙粒。</strong>","inTrash":"有些回收桶裡的沙粒符合您的搜尋。","trashButton":"檢視回收桶"}},"sandstormAppDetails":{"websiteLink":"網站","appMarketLink":"應用程式市集","sourceCodeLink":"原始碼","bugReportLink":"回報問題","mainListButton":"檢視主列表","trashButton":"檢視回收桶","uninstallButton":"移除","restoreButton":"回復沙粒備份……","version":{"label":"版本"},"lastUpdate":{"label":"最後更新","unknown":"未知"},"publisher":{"label":"簽名者"},"hasFingerprint":"本程式的發行者已用 PGP 簽章證明其控制權。","noFingerprint":"本程式沒有作者的簽章，無法辨認作者。"}}}});
TAPi18n._registerServerTranslator("zh-CN", namespace);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"zh-TW.i18n.json":function(require){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// i18n/zh-TW.i18n.json                                                                                                //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
var _ = Package.underscore._,
    package_name = "project",
    namespace = "project";

if (package_name != "project") {
    namespace = TAPi18n.packages[package_name].namespace;
}
TAPi18n.languages_names["zh-TW"] = ["Chinese (Taiwan)","繁体中文（台湾）"];
if(_.isUndefined(TAPi18n.translations["zh-TW"])) {
  TAPi18n.translations["zh-TW"] = {};
}

if(_.isUndefined(TAPi18n.translations["zh-TW"][namespace])) {
  TAPi18n.translations["zh-TW"][namespace] = {};
}

_.extend(TAPi18n.translations["zh-TW"][namespace], {"error":"錯誤：%s","shell":{"layout":{"demoExpired":{"title":"Demo Expired","canUpgrade":"Thanks for trying the Sandstorm Demo! Your demo account has expired. If you'd like to continue using Sandstorm and make your data permanent, please sign in.","cannotUpgrade":"Thanks for trying the Sandstorm Demo! Your demo account has expired.","signOut":"Sign out"},"adminAlert":{"icon":"!","title":"管理者送來的訊息"},"demoModal":{"header":"Sandstorm is a self-hostable web productivity suite.","withAppDemo":"with apps like %s.","withoutAppDemo":"with all the apps you need in one place.","startButton":{"withAppDemo":"Try %s »","withoutAppDemo":"Start »"},"explanation":"<p>Sandstorm makes it easy to run apps on your own server. In addition to improving privacy and control, <a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\" target=\"_blank\">this brings Open Source apps to a wide audience.</a></p>","securityOverview":"<p>Sandstorm is also ridiculously secure. If security interests you, check out <a href=\"https://docs.sandstorm.io/en/latest/developing/security-practices/\" target=\"_blank\">Sandstorm's security model overview</a>.</p>"}},"notifications":{"hint":"通知","text":"通知"},"notificationsPopup":{"title":"通知","empty":"沒有通知……"},"loading":{"title":"正在讀取……","text":"正在讀取……"},"notFound":{"title":"查無此沙","text":"404 查無此沙 :("},"root":{"title":"首頁"},"grainLogContents":{"prefix":"……"},"grainLog":{"title":"除錯記錄: %s"},"signup":{"title":"要求邀請","signedUp":{"keyIsUsedWithOrigin":"註冊成功！來去<a href=\"https://apps.sandstorm.io/?host=%s\">下載應用程式吧</a>。","keyIsNotUsed":"您已經有帳號了，不需要這把註冊金鑰。"},"keyIsUsed":"抱歉，這把註冊金鑰已經被用掉了。","processing":"請稍候……","signInGuide":"您需要先登入。__First, you have to sign in. Click \"Sign in\" in the upper-right.","invalidKey":"抱歉，這個註冊金鑰無效。"},"uploadStatus":{"title":"正在上傳……","uploadFailedWithError":"上傳失敗：%s %s","statusWithProgress":"%s……%s","status":"%s……","uploadFailed":"上傳失敗，請重試。"},"uploadTest":{"title":"Upload test page","explanation":"This page is intended for use by the test scripts.","uploadApp":"Upload app:","uploadBackup":"Upload backup:"},"referrals":{"title":"Referral Program","header":{"isPaid":{"title":"Get up to <span class=\"reward-amount\">30 GB</span> of free space when your friends share on Oasis!","explanation":"For each friend that creates a sharing link, we'll reward you with 2 GB."},"isNotPaid":{"title":"Get unlimited grains &amp; up to <span class=\"reward-amount\">2 GB</span> of free space when your friends share on Oasis!","explanation":"For each friend that creates a sharing link, we'll reward you with 50 MB. You'll also get unlimited grains."}},"howTo":{"title":"Sharing is referring","explanation":"To get started, open any grain and click Share access.","getStart":{"step1":"You share a grain with your friend (who isn't already an Oasis user).","step2":"Your friend signs in.","step3":"Your friend shares a grain with any other person (including you)."}},"status":{"title":"Your referrals & status","explanation":"Once you send a sharing link out, people will appear in this list after they sign in.","pending":{"title":"Still needs to share","explanation":"Tell these people to make a grain and share it with you. You must actually view the grain they shared with you!"},"success":{"title":"Successful referrals","explanation":"You've successfully referred these people. Congratulations!"}},"finePrint":{"title":"The fine print","warningWithAccountLink":"Your referral bonus lasts until 1 year after the referral. The terms of this referral program may change without warning; however, such changes will not affect bonuses you have already received. If you need even more space, <a href=\"%s\">upgrade your account.</a>","explanation":"Paid subscribers can get up to 30 GB of free storage, 2 GB per person they refer, in addition to the storage from their account level. Free account holders can earn unlimited grains and 50 MB of free storage per person they refer, up to 2 GB total."}},"about":{"title":"關於沙聚雲","intro":{"explanation":"<a href=\"https://sandstorm.io\">沙聚雲(Sandstorm.io)</a>是可以執行私人或個人網路應用程式的開源平台。使用者可以上傳並安裝任何程式，就像手機的應用程式商店一樣。為了保護隱私，<a href=\"https://blog.sandstorm.io/news/2014-07-21-open-source-web-apps-require-federated-hosting.html\">這是唯一公開開源網路應用程式的方式。</a>","version":"版本：%s","unofficialWarning":"<strong>WARNING:</strong> This server is running an unofficial build of Sandstorm which may contain modifications not reviewed by the developers."},"changelog":{"title":"最新變更點"},"developers":{"title":"核心開發者"},"advisors":{"title":"顧問"},"corporate":{"title":"贊助者","explanation":"These companies contributed heavily to Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaign</a>."},"individual":{"title":"Key indivisual sponsors","explanation":"These individuals contributed heavily to Sandstorm's <a href=\"http://igg.me/at/sandstorm\">crowdfunding campaign</a>."},"backers":{"title":"Backers","explanation":"from <a href=\"http://igg.me/at/sandstorm\">our Indiegogo campaign &gt;</a>","anonymous":"and %s anonymous contributors","loading":"loading..."},"dependencies":{"title":"Third-party works used","license":"license","explanation":"These works are copyright the respective authors and used under the terms of the respective licenses."},"copyright":{"title":"著作權","license":"The Sandstorm implementation is copyright by Sandstorm Development Group, Inc. and contributors. The implementation is licensed under the Apache License 2.0.","trademark":"\"Sandstorm\" and the Sandstorm logo are trademarks of Sandstorm Development Group, Inc., and may not be used to advertise derivative works nor commercial services without written consent from Sandstorm Development Group, Inc."},"terms":{"license":"授權","terms":"服務條款","privacy":"隱私權政策"}}},"grains":{"title":"沙粒","grainView":{"notFoundWithId":"找不到 ID 為 %s 的沙粒。","openWithWhichIdentity":"您要對此沙粒出示您的身份，或是保持匿名？","openInIncognitoMode":"以無痕模式開啟","revealMyIdentity":"出示身份"},"grainInMyTrash":{"explanation":"這顆沙粒沈在回收桶裡，您要把他撈回主清單才能重新使用。","restore":"撈回主清單"},"grainInOwnersTrash":{"explanation":"這顆沙粒被擁有者丟進回收桶了，無法使用。"},"grainOwnerSuspended":{"explanation":"這顆沙粒的擁有者被停權了，無法使用。"},"wrongIdentity":{"explanation":"本連結為以下使用者專用："},"requestAccess":{"permissionDenied":"您無權存取此沙粒。","requestAccessHint":"要求存取權","requestAccess":"要求存取權","warnIdentityRevealing":"要求存取權將把您的完整身份告知沙粒擁有者。","sendRequest":"正在送出您的要求……","waitForApproving":"要求已送出，若獲核准則會以電郵通知您。","requestFailed":"要求失敗，原因：%s","signInPlease":"若要要求存取權，請先登入。"},"invalidToken":{"invalidToken":"無效的 token：%s"},"revokedShareLink":{"linkRevoked":"非常抱歉，這個連結已經被撤銷。"},"grainTitle":{"was":"原標題：%s","renamedFrom":"原名：%s"},"grainDeleteButton":{"hint":"移到回收桶","text":"移到回收桶"},"grainDebugLogButton":{"hint":"顯示除錯記錄","text":"紀錄"},"grainBackupButton":{"hint":"下載備份檔","text":"備份"},"grainBackupPopup":{"title":"下載備份檔","warning":"注意！復原這個備份檔會發生的事可能和你想的不一樣。","processing":"正在準備備份……","cancelHint":"取消","cancel":"取消","confirmHint":"下載備份檔","confirm":"下載備份檔"},"grainRestartButton":{"hint":"重新啟動應用程式","text":"重新啟動"},"grainApiTokenButton":{"hint":"取得外網金鑰(webkey)","text":"取得外網金鑰(webkey)"},"grainApiTokenPopup":{"title":"外網金鑰","processing":"生成中……","cancel":"取消","copyForExternalApp":"複製這個金鑰到外部程式中：","back":"返回","description":"外網金鑰讓您可以把外部程式（比如手機 App 等）連結到這個應用程式。","labelTitle":"標籤：","labelPlaceholder":"比如「Android app」","roleTitle":"角色：","permissionTitle":"完整存取","create":"建立","webkeyListTitle":"現存的外網金鑰：","revokeHint":"撤銷","revoke":"撤銷"},"grainShareButton":{"hint":"分享此沙粒給其他人","text":"分享"},"whoHasAccessPopup":{"title":"可存取者","noLoggedInUsers":"未曾有已登入的使用者造訪這個連結過。","nobody":"您未允許任何人存取。","addedBy":"分享人：","revokeHint":"撤銷","revoke":"撤銷","hasCustomPermissions":"[擁有特殊權限]"},"selectRole":{"hint":"選擇角色"},"emailInviteTab":{"demoModeAlert":"Sharing emails cannot be sent while in demo mode. Create an account to send sharing emails, or get a shareable link instead.","personalMessagePlaceholder":"（選填）附加個人訊息……","send":"送出","sending":"正在送出……","success":"成功送出。","reset":"+ 邀請更多人","startOver":"<< 重新開始"},"shareableLinkTab":{"withRoles":"任何知道連結的人：","withoutRoles":"任何知道這個連結的人都能存取此沙粒。","labelPlaceholder":"標籤（選填）","labelExplanation":"標籤可用來備註這個連結是創來給誰用的。","create":"建立","creating":"正在建立……","success":"請複製這個連結：","reset":"+ 建立另一個連結","cancel":"取消"},"shareWithOthers":{"title":"分享給別人","sendInviteTab":"寄送邀請","shareableLinkTab":"取得分享連結","whoHasAccess":"檢視誰能存取"},"grainSharePopup":{"oldSharingModelDescription":"This grain uses the old sharing model. You can share it by copy/pasting the URL from the location bar","upgradeDescription":"You can upgrade this grain to the new model, but anyone with whom you shared this grain previously will lose access; you will have to share with them again.","upgradeWarning":"Upgrading cannot be undone.","upgrade":"upgrade","openInIncognitoMode":"您正以無痕模式瀏覽此沙粒。","toShareAccess":"若要進行分享，請複製以下連結：","openNonAnonymouslyHead":"更多的分享選項","openNonAnonymouslyLink":"需要登入才能使用","openNonAnonymouslyTail":"。","signIn":"其他分享選項需要登入才能使用。"},"grainPowerboxRequest":{"powerboxRequestHint":"Powerbox 要求","powerboxRequest":"P"},"grainPowerboxOffer":{"powerboxOfferHint":"Powerbox 命令","powerboxOffer":"P"},"grainPowerboxOfferPopup":{"title":"Powerbox 命令","dismiss":"隱藏"},"share":{"grainNotFoundWithId":"找不到 ID 為 %s 的沙粒。也許您需要用別的帳號登入？","signIn":"您必須登入才能進行分享。"},"grainlist":{"sandstormGrainListPage":{"introButton":{"hint":"什麼是沙粒？"},"mainListButton":"檢視主列表","trashButton":"檢視回收桶(%s)","restoreButton":"復原備份……","search":{"placeholder":"搜尋 "},"usageInfo":{"quota":"__quota__ 沙粒中 __count__ 個","size":"共 __total__，已用 __size__"},"trash":{"emptyButton":"清空所有回收桶","explanation":"（回收桶內的沙粒會在 30 天後被自動移除。若要保留沙粒，請將其移回主列表。）"},"noGrains":{"notFound":"<strong>找不到相符的沙粒。</strong>","inTrash":"回收桶內有沙粒符合您的搜尋。","trashButton":"檢視回收桶","empty":"<strong>回收桶是空的。</strong>","explanation":"您沒有任何沙粒。","hasApps":"選擇應用程式來創建您的第一粒沙。","hasNoApp":"安裝應用程式來創建您的第一粒沙。"},"moveToTrash":"移至回收桶"},"sandstormGrainTable":{"bulkAction":{"disabledHint":"若要進行擴張，您必須先從列表中選擇沙粒。"},"tableHeader":{"title":"沙粒名","size":"容量","lastUsed":"最後使用","ownership":"擁有者","collaborators":"合作人"},"tableBody":{"was":"（原標題：%s）","renamedFrom":"（原名：%s）","new":"尚未開啟","mine":"自己","shared":"別人分享"}},"contactAutocomplete":{"contactInputBox":{"placeholder":"輸入電郵信箱或姓名"}}}},"notifications":{"appUpdateNotificationItem":{"title":"應用程式有新版本：","appUpdates":"<strong>__name__</strong>: 版本 __version__ 已推出","submitButton":{"hint":"更新所有上記應用程式","text":"套用更新"},"dismissButton":{"hint":"略過上記更新","text":"隱藏"}},"mailingListBonusNotificationItem":{"explanationWithBonus":"Subscribe to our announcement list and get <strong>%s bonus storage</strong> (1-2 emails per month).","submitButton":"Subscribe","dismissButton":"No thanks"},"referralNotificationItem":{"paidUser":"Get up to 30 GB bonus with the","unpaidUser":"Unlimited grains & up to 2 GB bonus with the","link":"referral program »","dismissButton":"Dismiss"},"adminNotificationItem":{"explanation":"您提供的匿名資料可以幫助我們改善沙聚雲。詳情請點擊此處。"},"backgroundedGrainNotificationItem":{"title":"__<strong>__grain__</strong> 正在後台執行：","cancelButton":{"hint":"停止後台應用程式","text":"取消"}},"identityChangesNotificationItem":{"explanation":"我們正在改變沙聚雲的身份制度，您的帳戶可能會被影響。詳情請點擊此處。"},"grainActivityNotificationItem":{"title":"<strong>__grain__</strong> 的 __<strong>__sender__</strong>：","dismissButton":{"hint":"隱藏此通知","text":"隱藏"}}},"powerboxBuiltins":{"uiViewPowerboxConfiguration":{"explanation":"權限：","fullAccess":"完整存取","connectButton":"連線","chooseHostedObjectButton":"瀏覽沙粒內容 »"},"ipNetworkPowerboxCard":{"explanation":"管理者：授權給所有連出存取","encryptedWith":"(以 %s 加密)"},"ipInterfacePowerboxCard":{"explanation":"管理者：授權給所有連入存取"},"emailVerifierPowerboxCard":{"verifyWith":"用 %s 驗證電郵信箱","verifyWithAnyService":"用任何登入服務驗證電郵信箱"},"identityPowerboxConfiguration":{"explanation":"權限：","fullAccess":"完整存取","connectButton":"連線"},"addNewVerifiedEmailPowerboxCard":{"explanation":"新增位址"},"httpUrlPowerboxCard":{"useUrl":"使用 %s"},"httpArbitraryPowerboxCard":{"explanation":"輸入網址……"},"httpArbitraryPowerboxConfiguration":{"explanation":"輸入網址：","placeholder":"https://example.com","connectButton":"連線"},"httpOAuthPowerboxCard":{"useService":"使用 %s"},"httpOAuthPowerboxConfiguration":{"requstPermissionsFromService":"從 %s 要求權限(請參照彈出視窗)。"}},"styleguide":{"title":"Sandstorm style guide","note":"(note: this is a work in progress.)","copyRules":{"title":"Copy rules","rule1":"Be terse but clear.  Avoid technical terms where less-technical ones will do.","rule2":"Capitalize the first letter of the first word of a section heading or button.  Do not capitalize the second word, or anything else, unless referring to a proper noun like Google or GitHub.","rule3":"Use verbs for text on buttons. The button text should describe what clicking that button will do.","rule4":"Use nouns for text which is a link.  The link text should indicate what the link will take you to.","rule5":"Use nouns for basically everything else, too, to communicate the information architecture. Section headings should be nouns which describe the content of the section.  Labels should be nouns which describe the content of the field."},"styleRules":{"title":"Style rules","rule1":"Stick to a limited color palette.  (TODO: add specific colors)","rule2":"Use typefaces no smaller than 13px for text content.  Readability is important and not everyone has young eyes.","rule3":"Make click targets no smaller than 32px square or 26px tall if appreciably wider.  Remember that due to Fitts' Law, larger click targets are easier to interact with."},"forms":{"title":"Forms","rule1":"Use the <code>standard-form</code> class where appropriate.","rule2":"The primary action button of a form should be at the bottom right of the form. Secondary buttons should line up to the left of that primary action button.","rule3":"When providing text input fields, place labels above the input, not side-by-side or inside the field as placeholder text.  Place additional explanation or instructions below the input. Placeholder text should be used for examples, not instructions.","rule4":"Forms are generally used for transactional interactions, where the user cares about the success or failure of the application.  Avoid optimistic UI in favor of multi-state UI which communicates network activity, success, and failure.  A common pattern is:","rule4-1":"Have the default state of the form.  Disable a save button until the user makes a change.","rule4-2":"When the user clicks an action button, start the RPC, and disable the entire form to indicate that the action is underway.","rule4-3":"When the RPC completes, if the RPC failed, show an error message at the top of the context over which the action applies.  If the RPC succeeded, either indicate a success message (again, at the top of the context over which the action applies) or if this was a modal interaction and the completion of the RPC indicates successful completion of the workflow, dismiss the form with no additional message."},"example":{"title":"An example:","label":"Email","placeholder":"alice@example.com","subtext":"You might want to receive notifications.","submitButton":"Save","discardButton":"Discard"}},"admin":{"title":"管理者","admin":{"newAdmin":{"title":"管理面板","wildcardHostBroken":"警告：本伺服器似乎把 WILDCARD_HOST 設定錯了。在修復此問題前，您無法使用任何應用程式。您可以參照<a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#why-do-i-see-an-error-when-i-try-to-launch-an-app-even-when-the-sandstorm-interface-works-fine\">本篇沙聚雲文件</a>和您瀏覽器的 Javascript 記錄取得更多資訊。您必須調整 DNS、SSL/TLS certificates，或編輯 sandstorm.conf 檔案。如果您想要測試問題是否已修復，請重新整理此頁面。如果您仍然有問題，請<a href=\"https://github.com/sandstorm-io/sandstorm/issues\">在此開一個 issue</a>。","websocketBroken":"警告：本伺服器似乎無法建立 WebSocket。沙聚雲也許看起來沒有問題，但許多應用程式會無法正常執行。<a target=\"_blank\" href=\"https://docs.sandstorm.io/en/latest/administering/faq/#how-do-i-enable-websockets-proxying-or-why-do-some-apps-seem-to-crash-reload\">更多資訊在此。</a>通常這是因為上層的 reverse proxy （比如 nginx 或 Apache）需要被重新設定。如果您認為不是，請重新整理此頁面或<a href=\"https://github.com/sandstorm-io/sandstorm/issues\">在此開一個 issue</a>。","hasSetupToken":"您並非以管理者身份登入，但您的設置 token 是正確的。您現在可以<a href=\"/setup\">啟動設置精靈</a>或<a href=\"/admin/status\">檢視系統記錄</a>。","hasNoSetupToken":"您並非以管理者身份登入，無法使用管理面板。請以管理者身份登入，或在主機終端上執行`sandstorm admin-token`來取得管理 token。"},"newAdminRoot":{"configuration":{"title":"設定"},"management":{"title":"管理"}}},"identityProviders":{"name":"登入提供者","providers":{"emailPassLess":"電子郵件（無密碼登入）"},"subtext":"設定使用者可以如何登入。","newAdminIdentity":{"explanation":"選擇使用者可以用哪些方法進行身份認證。所有人都能登入，但只有您邀請的使用者可以安裝應用程式或建立沙粒。","organizationLinkPrefix":"要設定組織內容，請到","organizationLink":"組織設置","organizationLinkPostfix":"."},"adminIdentityProviderConfigureEmail":{"title":"電郵登入設定","error":"無法儲存變更：%s","warning":"備註：若要使用電郵登入，您也需要設定伺服器的電郵寄送功能。","explanation":"電郵登入沒有額外的設定選項。","saveButton":"儲存","enableButton":"啟用","cancelButton":"取消","disableButton":"停用"},"googleLoginSetupInstructions":{"explanation":"First, you'll need to get a Google Client ID. Follow these steps:","step1":"Visit <a href=\"https://console.developers.google.com/\" target=\"blank\">https://console.developers.google.com/</a>","step2":"\"Create Project\", if needed. Wait for Google to finish provisioning.","step3":"On the left sidebar, go to \"Credentials\" and, on the right, \"OAuth consent screen\". Make sure to enter an email address and a product name, and save.","step4":"On the left sidebar, go to \"Credentials\".  Click the \"Create credentials\" button, then select \"OAuth client ID\" as the type.","step5":"Select \"Web application\" as your application type.","step6":"Set Authorized Javascript Origins to: <span class=\"idp-config-url\">%s</span>","step7":"Set Authorized Redirect URI to: <span class=\"idp-config-url\">%s/_oauth/google</span>","step8":"Finish by clicking \"Create\"."},"adminIdentityProviderConfigureGoogle":{"title":"Google login configuration","formerBaseUrlWarning":"This server moved from <code>__former__</code> to <code>__current__</code>. You'll need to visit <a target=\"_blank\" href=\"https://console.developers.google.com/\">https://console.developers.google.com/</a> and update the \"Authorized Javascript Origins\" and \"Authorized Redirect URI\" for this Client ID to include the values below.","error":"Failed to save changes: %s","explanation":"Then, copy over your Client ID and Client secret below:","clientId":"Client ID:","clientSecret":"Client secret:","saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"githubLoginSetupInstructions":{"explanation":"First, you'll need to get a Github Client ID and Client secret. Follow these steps:","step1":"Visit <a href=\"https://github.com/settings/applications/newi\" target=\"blank\">https://github.com/settings/applications/new</a>","step2":"Set Homepage URL to: <span class=\"idp-config-url\">%s</span>","step3":"Set Authorization callback URL to: <span class=\"idp-config-url\">%s_oauth/github</span>"},"adminIdentityProviderConfigureGitHub":{"title":"GitHub login configuration","formerBaseUrlWarning":"This server moved from <code>__former__</code> to <code>__current__</code>. You'll need to visit <a target=\"_blank\" href=\"https://github.com/settings/developers\">https://github.com/settings/developers</a> and update the Homepage URL and Authorization callback URL for this application to the values below.","explanation":"Then, copy over your Client ID and Client secret below:","clientId":"Client ID:","clientSecret":"Client secret:","saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"adminIdentityProviderConfigureLdap":{"title":"LDAP login configuration","error":"Failed to save changes: %s","url":{"label":"LDAP server URL:","subtext":"e.g. <code>ldap://ldap.example.com:389</code>.  Use port 636 for SSL."},"searchBindDn":{"label":"Bind user DN","subtext":"The DN to bind as when searching for a user in the tree, e.g. <code>cn=admin,ou=users,dc=example,dc=com</code>.  Optional, but needed if your server doesn't allow unauthenticated searches."},"searchBindPassword":{"label":"Bind user password","subtext":"Optional, needed if specifying bind DN above."},"base":{"label":"Base DN","subtext":"The root object under which to search for user account nodes, e.g. <code>ou=users,dc=example,dc=com</code>"},"searchUsername":{"label":"LDAP username attribute","subtext":"The LDAP attribute on a user object which represents the user's username (what they will enter in the username field when signing in)."},"nameField":{"label":"LDAP given name attribute","subtext":"The LDAP attribute on a user object which represents the user's full display name."},"emailField":{"label":"LDAP email attribute","subtext":"The LDAP attribute on a user object which represents the user's email address."},"filter":{"label":"Additional LDAP filter criteria","subtext":"An optional LDAP query fragment that will be included in the user search. This is not commonly used. See <a target=\"_blank\" href=\"http://www.ietf.org/rfc/rfc2254.txt\">RFC2254</a>."},"caCert":{"label":"LDAP CA certificate","placeholder":"-----BEGIN CERTIFICATE-----\nMIID...\n-----END CERTIFICATE-----","subtext":"This is only necessary if your server supports SSL and is using a private CA not in the common trusted set. Note that it does NOT work to enter the LDAP server's own certificate here -- it must be the certificate of a CA which has signed the LDAP server's certificate."},"saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"adminIdentityProviderConfigureSaml":{"title":"SAML login configuration","explanation":"Your SAML IDP should be configured to return a persistent nameID. In addition, if you are not using Active Directory, you <strong>must</strong> configure your IDP to provide two extra attributes, <code>email</code> and <code>displayName</code>.","serviceUrl":"The Service URL of this server is: <code>%s</code>","logoutUrl":"The Service Logout URL of this server is: <code>%s</code>","metadataUrlAt":"If your SAML IDP supports reading the service provider metadata from a URL, then you can point it at: <code><a href=\"__href__\" target=\"_blank\">__text__</a></code>","error":"Failed to save changes: %s","entryPoint":{"label":"SAML provider entry point URL:","placeholder":"https://YOUR_SAML_PROVIDER.com/sso/saml"},"logout":{"label":"SAML provider logout URL (optional):","placeholder":"https://YOUR_SAML_PROVIDER.com/sso/saml/SingleLogoutService"},"publicCert":{"label":"SAML cert for above provider:","placeholder":"base64 encoded cert (e.g. MIID...8F==)"},"entityId":{"label":"Entity ID:"},"saveButton":"Save","enableButton":"Enable","cancelButton":"Cancel","disableButton":"Disable"},"adminIdentityProviderTable":{"title":"登入提供者","status":"狀態"},"adminIdentityRow":{"enabled":"已啟用","disabled":"未啟用","baseUrlChanged":{"explanation":"因為 BASE_URL 而被停用","reconfigureButton":"重新設定"},"configureButton":{"hint":"設定 %s 登入","text":"設定"}}},"organization":{"name":"組織設定","subtext":"讓大團隊可以圓滑地上手及進行合作。","organizationSettingsBlurb":{"explanation":"沙聚雲讓您可以建立組織。您可以將設定自動套用在所有組織成員身上。所有組織成員都可以登入、安裝應用程式、以及建立沙粒。"},"newAdminOrganization":{"success":"變更已儲存。","error":"無法儲存變更：%s","membership":{"title":"組織成員","emailToggle":{"label":"自動認證電郵信箱","subtext":"注意：因為電郵登入設定未完成，所以無法使用。"},"emailDomain":{"label":"網域：","subtext":"使用上記網域信箱的人會自動認證為組織成員。"},"gappsToggle":{"label":"G Suite 使用者認證","subtext":"注意：因為 Google 登入設定未完成，所以無法使用。"},"gappsDomain":{"label":"網域：","subtext":"使用上記網域 G Suite 的人會自動認證為組織成員。"},"ldapToggle":{"label":"LDAP 使用者認證","subtext":"注意：因為 LDAP 登入設定未完成，所以無法使用。"},"samlToggle":{"label":"SAML 使用者認證","subtext":"注意：因為 SAML 登入設定未完成，所以無法使用。"}},"settings":{"title":"組織設定","disallowGuests":{"label":"禁止外部訪客進行協作。","subtext":"只有組織成員可以開啟共享連結。__Users will be required to log in as a member of your organization to view sharing links. Guest access will be disabled."},"shareContacts":{"label":"讓所有使用者可以互相看見","subtext":"讓組織內所有使用者自動互相加進聯絡人清單，以便共享沙粒。若使用者身份應該被隱蔽，請取消此選項。"}},"saveButton":"儲存"}},"emailConfig":{"name":"電郵寄送設定","subtext":"設定沙聚雲如何寄送電郵。","newAdminEmailConfig":{"outbound":{"title":"外寄郵件","warning":"<strong>外寄郵件尚未設定完成。</strong>在您設定完成前，和電郵有關的功能（比如郵件登入、通知、分享邀請或要求存取）都無法使用。"},"howItWorks":{"title":"沙聚雲如何使用電郵","explanation":"沙聚雲寄送通知郵件，並且也允許應用程式寄送。每個應用程式都有一個獨自的電郵位址，所以沙聚雲需要能從可變的「來自」位置寄信。所以 Gmail SMTP 不管用，請採用別的免費服務。更多資訊請<a href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">參見此處</a>。"},"success":"變更已儲存。","error":"無法儲存變更：%s","address":{"hostname":"SMTP 主機位址","port":"連接埠","subtext":"SMTP（無加密或 StartTLS 加密）用 port 25，TLS-加密 SMTPS 用 port 465。"},"username":{"label":"使用者名稱（選填）"},"password":{"label":"密碼（選填）"},"returnAddress":{"label":"沙聚雲伺服器自己的收信位址","subtext":"沙聚雲會用這個位址寄出登入用郵件、要求存取及共享的通知郵件等。此位置也會用在 SMTP envelope。"},"submitButton":"儲存設定","testButton":"測試","disableButton":"停用電郵"},"emailTestPopup":{"title":"寄送測試郵件","testAddress":{"label":"寄送測試郵件到："},"pending":"正在寄送……","submitButton":"寄送測試郵件","closeButton":"關閉"},"emailDisablePopup":{"title":"真的要停用電郵？","explanation":"將會停用所有本沙聚雲的外寄郵件功能，包括電郵、郵件登入、以及郵件通知。","disableButton":"停用電郵","closeButton":"關閉"}},"personalization":{"name":"個人化","subtext":"客製獨自的啟動畫面、服務條款、隱私權政策等等。","newAdminPersonalization":{"serverTitle":{"label":"伺服器標題：","subtext":"本伺服器和使用者溝通時使用的名稱。"},"splashUrl":{"label":"Splash URL (experimental):","subtext":"This web page will be displayed in the background behind the login dialog (the home page when logged out). For security reasons, the page must be hosted within this Sandstorm server's wildcard host (otherwise it will be blocked by <code>Content-Security-Policy</code>). We suggest using a static web publishing app like <a target=\"_blank\" href=\"https://apps.sandstorm.io/app/nqmcqs9spcdpmqyuxemf0tsgwn8awfvswc58wgk375g4u25xv6yh/?host=%s\">Hacker CMS</a> to host content.<strong>WARNING: This feature is experimental; in particular the style and positioning of the login box is subject to change without notice.</strong> Please let us know if you'd like to see it stabilize."},"signupDialog":{"label":"註冊對話框：","subtext":"當點擊註冊連結時顯示的文字。"},"termsOfService":{"label":"服務條款網址：","placeholder":"https://example.com/tos","subtext":"若有填寫，新使用者將會被要求同意您提供的服務條款。"},"privacyPolicy":{"label":"隱私權政策網址：","placeholder":"https://example.com/privacy","subtext":"若有填寫，新使用者將會被要求同意您提供的隱私權政策。"},"whitelabeling":{"title":"白標","logo":{"label":"登入對話框顯示圖案","resetButton":"使用預設值","uploadButton":"上傳圖案"},"hideTroubleshooting":{"label":"隱藏故障排除按鈕"},"hideAbout":{"label":"隱藏「關於沙聚雲」連結"},"hideSendFeedback":{"label":"隱藏「傳送意見」連結"},"useServerTitleForHomeText":{"label":"以伺服器標題作為首頁連結","subtext":"將畫面左上角的「Sandstorm」文字改為伺服器標題。"},"customLoginProviderName":{"label":"自訂登入按鈕文字","subtext":"將登入畫面的「LDAP 登入」和「SAML 登入」的改成您自訂的文字。"}},"saveButton":"儲存"}},"preinstalledApps":{"name":"預裝應用程式","subtext":"選擇新使用者可以直接使用的預裝應用程式。","newAdminPreinstalledApps":{"explanation":"在本頁設定為預裝的應用程式將會自動安裝在所有新使用者的沙聚雲中。","productivitySuite":{"label":"生產力套件","subtext":"大部分使用者都會用到這些核心應用程式。"},"systemSuite":{"label":"系統程式","subtext":"預裝這些系統程式可以大幅強化沙聚雲的功能性。"},"allApps":{"label":"市集上的應用程式","subtext":"沙聚雲會幫新使用者自動安裝所有被勾選的程式。","lessButton":"收合","moreButton":"顯示另外 %s 個應用程式"},"saveButton":"儲存"},"_appRow":{"download":{"success":"已下載","failed":"失敗"}}},"appSources":{"name":"應用程式來源","subtext":"取得應用程式和更新的位置。","newAdminAppSources":{"appMarketUrl":{"label":"應用程式市集網址","subtext":"設定應用程式頁面的「安裝應用程式」按鈕打開的連結。"},"enableAppUpdates":{"label":"啟用程式更新通知","subtext":"沙聚雲伺服器每天都會下載應用程式的更新檔，並且詢問每個使用者是否要進行更新。"},"appIndexUrl":{"label":"程式索引網址","subtext":"設定沙聚雲伺服器檢查更新用的網址。"},"saveButton":"儲存","restoreButton":"回復預設值"}},"networking":{"name":"網路","altName":"安全性","subtext":"控制如何存取網路。","newAdminNetworking":{"ipBlacklist":{"label":"伺服器端請求 IP 黑名單：","subtext":"這些 IP 將無法被使用者請求，包含從應用程式內請求、從使用者指定的網址下載 SPK 檔案等。您每行可以指定一個 IP 或網路（使用 CIDR notation，比如\"127.0.0.0/8\"）。預設值包含標準的本地端及私人網路位址。另外當使用 HTTP 代理時他可能有自己的黑名單設定，所以這項設定可能會被忽略。"},"saveButton":"儲存","restoreButton":"回復預設值"}},"users":{"name":"使用者","subtext":"管理此沙聚雲上的使用者。"},"userAccounts":{"newAdminUserTableRow":{"admin":{"hint":"管理者","cap":"A"},"user":{"hint":"使用者","cap":"U"},"visitor":{"hint":"訪客","cap":"V"},"unknownIdentity":"不明憑證","unknownIdentifier":"不明的憑證","manageButton":"管理"},"newAdminUserTable":{"header":{"name":"名稱","credentials":"憑證","created":"建立日期","lastActive":"最近活躍","actions":"動作"},"inviteLink":"邀請新使用者","notFound":"沒有使用者符合您的搜尋條件","processing":"正在讀取使用者資料……"},"newAdminUsers":{"search":{"placeholder":"搜尋"},"filter":{"explanation":"顯示：","admin":"管理者","user":"使用者","visitor":"訪客"}}},"userDetails":{"newAdminUserDetailsIdentityTableRow":{"profilePicture":{"label":"個人檔案照片"},"handle":{"title":"網名","emails":"電郵"},"pronouns":{"hint":"代名詞","label":"代名詞"},"emails":{"hint":"電郵信箱","label":"電郵信箱","primary":"主要","verified":"已驗證","unverified":"尚未驗證","notFound":"沒有不明電郵。"},"properties":{"login":"可登入","org":"組織成員"},"unkonwn":{"profilePicture":{"cell":"（名稱不明)"},"service":{"hint":"不明服務","label":"不明服務","cell":"名稱不明"},"handle":{"cell":"稱呼不明"},"pronouns":{"cell":"代名詞不明"},"created":{"cell":"建立日期不明"},"lastActive":{"cell":"最近活躍不明"}}},"newAdminUserDetailsIdentityTable":{"header":{"credential":"憑證","created":"建立日期","lastActive":"最近活躍","properties":"內容"}},"newAdminUserDetails":{"role":{"title":"角色","admin":{"label":"管理者","explanation":"管理所有帳號。可以安裝應用程式，建立沙粒，以及管理伺服器。","isAdmin":"目前角色","isNotAdmin":"升級為管理者"},"user":{"label":"使用者","explanation":"可以安裝應用程式及建立沙粒。","isUser":"目前角色","isAdmin":"降級為使用者","isNotUser":"升級為使用者"},"visitor":{"label":"訪客","explanation":"可以二度分享別人分享的沙粒，可是不能安裝應用程式或建立自己的沙粒。","isVisitor":"目前角色","isNotVisitor":"降級為訪客"}},"credentials":{"title":"憑證"},"profile":{"title":"個人資料"},"status":{"title":"狀態","deleting":"停權 - 即將刪除 %s","suspended":"停權","active":"活躍","unsuspendButton":"解除停權","suspendButton":"將帳號停權","deleteButton":"刪除帳號"},"deletePopup":{"title":"刪除 %s","explanation":"您正在刪除 %s 的帳號。被標記為刪除中的帳號會被停權，管理者有七天的時間取消刪除動作。（在本頁點擊「解除停權」按鈕即可）%s <strong>無法</strong>阻止停權和刪除。","detail":"七天後， %s 的帳號將被永遠刪除。刪除帳號將：","detail1":"永遠刪除所有 %s 建立的沙粒","detail2":"撤銷所有 %s 建立的共享連結","detail3":"將 %s 從所有人的聯絡人名單中移除","detail4":"刪除所有本帳號的憑證（除非該憑證亦被別的帳號使用）","deleteButton":"刪除帳號","cancelButton":"取消"},"suspendPopup":{"title":"將 %s 停權","detail":"將這個帳號停權將會：","detail1":"阻止所有 %s 建立的沙粒被開啟","detail2":"停用所有 %s 建立的共享連結","detail3":"本帳號的憑證亦等同辦理（除非該憑證亦被別的帳號使用）","suspendButton":"將帳號停權","cancelButton":"取消"},"loading":"正在讀取使用者資料……"}},"userInvite":{"newAdminUserInviteLink":{"explanation":"您可以建立連結來邀請其他人創建他們的帳號。您邀請的人將直接擁有使用者權限，可以安裝應用程式及建立沙粒。如果您只想要分享一顆沙粒，請使用該沙粒畫面內的「分享存取」選單。","title":"建立連結","invite":{"explanation":"建立可以寄送給別人的帳號創建連結。","createButton":"建立連結"}},"newAdminUserInviteEmail":{"title":"寄送電郵","emailWarning":{"explanation":"電郵尚未設定完成，故此功能已被停用。","configureLink":"設定電郵"},"explanation":"建立連結並以電郵寄出。","to":{"label":"寄給：","placeholder":"alice@email.com\nbob@example.com","subtext":"每行輸入一個電郵位址。"},"subject":{"label":"標題："},"body":{"label":"內容：","subtext":"<code>$KEY</code> 會被置換成產生出來的連結。"},"sendButton":"寄送邀請"},"newAdminUserInvite":{"title":"邀請"}},"systemStatus":{"name":"系統記錄","subtext":"檢視沙聚雲的除錯記錄。","newAdminLogContents":{"ellipsis":"……"},"newAdminStatus":{"downloadButton":"下載完整記錄"}},"stats":{"name":"統計","subtext":"檢視伺服器上的各種統計數據。","header":{"date":"日期","users":"使用者","grains":"沙粒","appName":"應用程式名","owners":"擁有者","sharedUsers":"共享使用者","daily":"每日","weekly":"每週","monthly":"每月","forever":"所有時間"},"body":{"notFound":"目前還沒有統計資料。","loading":"讀取中……"},"newAdminStats":{"share":{"title":"Publish anonymously to Sandstorm dev team","explanation":"We'd like to know how many Sandstorm users there are, to make graphs and such.  Is it okay to send the numbers on this page once a day to the Sandstorm team, so that we can add them to the totals?  Any stats you send are anonymous.","enableButton":"Send anonymous stats","disableButton":"Don't send","control":{"explanation":"The developers of Sandstorm like to know how many Sandstorm users there are, to make graphs and such.  You can help us by opting in to sharing the numbers on this page once a day to the Sandstorm team, so that we can add them to our totals. Any stats shared are submitted anonymously.","label":"Anonymous stats reporting to sandstorm.io:","enabled":"Enabled","disabled":"Disabled","disableButton":"Disable","enableButton":"Enable"}},"json":{"title":"以 JSON 檢視","explanation":"If you want to integrate Sandstorm stats with an internal stats tool, use this secret link to retrieve the tables below as JSON. You can revoke the current link at any time by generating a new link.","label":"統計 JSON 連結：","regenerateButton":"生成新連結"},"userGrains":{"title":"使用者和沙粒","current":"現在： __users__ 名使用者， __grains__ 顆沙粒","today":"今天： __users__ 名使用者， __grains__ 顆沙粒"},"apps":{"title":"應用程式","label":"選擇日期："}}},"maintenanceMessage":{"name":"維修訊息","subtext":"告知使用者維修所需時間。","newAdminMaintenance":{"explanation":"You can set a maintenance message which will be shown to all users in the menu bar, on the top right. This is useful for announcing scheduled maintenance so users can be prepared to tolerate downtime. If the message text is empty, no message will be displayed.","variables":"You may use the following variables in the \"Message text\" field, which will be automatically substituted with useful values:","variable1":"<code>$TIME</code> : The time component of \"Maintenance time\", converted to the user's local time (eg. \"1:15:45 PM\")","variable2":"<code>$DATE</code> : The date component of \"Maintenance time\", converted to the user's local time (eg. \"6/19/2015\")","variable3":"<code>$IN_COUNTDOWN</code> : An active countdown until \"Maintenance time\" (eg. \"in 3 days\" or \"in 1 minute\"). Displays \"any moment\" at the time specified in \"Maintenance time\" and then disappears after an hour.","messageText":{"label":"Message text:","subtext":""},"maintenanceTime":{"label":"Maintenance time:","subtext":"The time you intend to start performing maintenance, formatted like \"%s\"."},"url":{"label":"Link URL (optional):","subtext":"If given, the maintenance message will render as a clickable link to the provided URL."},"saveButton":"Set message"}},"networkCapabilities":{"name":"網路容量","subtext":"管理沙粒的網路直接存取量。","newAdminNetworkCapabilitiesTableCapabilityRow":{"adminAccount":"Admin account %s","unknownGranter":"Unknown granter","anyone":"Anyone with the token","unknownRecipient":"Unknown recipient","revoked":"已撤銷","revokeButton":"撤銷"},"newAdminNetworkCapabilitiesTable":{"header":{"grantBy":"Granted by","grantTo":"Granted to","grantTime":"Time","actions":"Actions"},"notFound":"No capabilities found."},"newAdminNetworkCapabilitiesSection":{"filter":{"label":"Showing:","active":"Active (%s)","revoked":"Revoked (%s)"}},"newAdminNetworkCapabilities":{"networkCaps":"Outbound network access","networkCapsTls":"TLS-encrypted outbound network access","interfaceCaps":"Inbound network access"}},"hostingManagement":{"name":"Hosting management","newAdminHostingManagement":{"explanation":"This feature is experiemental and only works with the LDAP login provider at the moment. Use with caution.","quotaEnabled":{"label":"Enable quota for users","subtext":"Enable Quota management on this server. This requires both SAML and LDAP to be configured, and email addresses will be used to match between the two."},"quotaLdapAttribute":{"label":"LDAP quota attribute","subtext":"This is the attribute name that will be looked up from LDAP to determine quota for a user."},"billingPromptUrl":{"label":"Billing prompt url","subtext":"This is the url that will be used in the billing prompt."},"saveButton":"Save","restoreButton":"Restore defaults"}}},"people":{"title":"使用者"},"sharingLinks":{"title":"分享連結"},"apps":{"title":"應用程式","appList":{"uninstallingHint":"移除完成","doneUninstalling":"移除完成","uninstallHint":"移除應用程式","uninstallButton":"批次移除...","uploadApp":"上傳應用程式","searchHint":"搜尋名稱","search":"搜尋名稱","mostUsed":"最常使用","allApps":"所有應用程式","install":"安裝新程式","fromMarket":"從市集下載","uninstall":"移除","noMatchApp":"找不到名稱含有該字串的應用程式。請<a href=\"__url__\">試試搜尋市集 &raquo;</a>","unauthorized":"無權限！:(","unauthorizedText":"只有管理者給予權限的的使用者才能安裝應用程式。若需權限，請聯絡管理者。","intro":"<strong>歡迎！</strong><br>新增文件、聊天室、部落格……。<br><br>從安裝新程式開始。","doneButton":"瞭解"},"install":{"sandstormAppInstallPage":{"title":"安裝應用程式","error":{"title":"無法安裝應用程式","retryButton":"再試一次"},"stepWait":{"title":"正在等待寫入存取……"},"stepDownload":{"title":"正在下載……","cancelButton":"取消"},"stepVerify":{"title":"正在驗證……"},"stepUnpack":{"title":"正在拆包……"},"stepAnalyze":{"title":"正在分析……"},"stepConfirm":{"upgrade":{"explanation":"您已經有安裝此程式的較舊版本。是否要更新？","button":"更新 %s"},"downgrade":{"explanation":"您已經有安裝此程式的較新版本。是否要降版？","button":"降版 %s"},"install":{"button":"安裝 %s"}},"stepRun":{"title":"已安裝 %s","explanation":"安裝完成","appDetailsLink":"開啟 %s"},"stepDelete":{"title":"正在刪除","explanation":"伺服器正在刪除這個應用程式……"},"stepFailed":{"title":"失敗","explanation":"安裝失敗：%s"}}},"appDetails":{"sandstormAppDetailsPage":{"newerVersion":{"explanation":"有些檔案是用較新版本的程式建立的。因為回復舊版本可能會破壞資料，這些檔案將會繼續使用新版本程式。"},"olderVersion":{"explanation":"有些沙粒是舊版本程式建立的。要更新嗎？","upgradeButton":"更新沙粒"},"search":{"hint":"搜尋","placeholder":"搜尋"},"trash":{"explanation":"回收桶裡的沙粒會在 30 天後被永久刪除。若要保留沙粒，請將其移回主列表。"},"noGrains":{"notFound":"<strong>找不到相符的沙粒。</strong>","inTrash":"有些回收桶裡的沙粒符合您的搜尋。","trashButton":"檢視回收桶"}},"sandstormAppDetails":{"websiteLink":"網站","appMarketLink":"應用程式市集","sourceCodeLink":"原始碼","bugReportLink":"回報問題","mainListButton":"檢視主列表","trashButton":"檢視回收桶","uninstallButton":"移除","restoreButton":"回復沙粒備份……","version":{"label":"版本"},"lastUpdate":{"label":"最後更新","unknown":"未知"},"publisher":{"label":"簽名者"},"hasFingerprint":"本程式的發行者已用 PGP 簽章證明其控制權。","noFingerprint":"本程式沒有作者的簽章，無法辨認作者。"}}},"accounts":{"title":"帳號設定","accountSettings":{"sandstormAccountSettings":{"title":"帳號","identities":{"title":"*我的資料","success":"成功：%s","linkedCredentials":"已連結的憑證","linkNewIdentity":{"title":"連結新憑證：","cancelButton":"取消","linkButton":"連結新憑證","logoutAllButton":"登出所有其他 Session"},"email":{"title":"服務、通知等使用的主要電郵位址","primary":"主要","primaryButton":"設為主要","notFound":"沒有已驗證的電郵！","addNew":"新增新的電郵地址"}},"billing":{"title":"帳單"},"deleteAccount":{"title":"刪除帳號","button":"刪除您的帳號"},"deletePopup":{"title":"刪除您的帳號","explanation":"您將會把自己的帳號標記為刪除中，被標記為刪除中的帳號會被停權。若要取消刪除，請在七天內重新用您的帳號登入。","results":"七天後，您的帳號將被永遠刪除。刪除帳號將：","result1":"永遠刪除所有您建立的沙粒，","result2":"撤銷所有您建立的共享連結，","result3":"將您從所有人的聯絡人名單中移除，","result4":"刪除所有本帳號的憑證（除非該憑證亦被別的帳號使用）。","confirm":"請在下方輸入「delete my account」進行刪除。","goodbye":"我們很遺憾您的離去。伺服器管理者歡迎您的任何意見。","feedbackPlaceholder":"選填","deleteButton":"刪除帳號","cancelButton":"取消"}},"sandstormAccountsFirstSignIn":{"title":"確認您的個人資料","success":"成功：%s"},"accountSuspended":{"title":"您已將本帳號停權。","deletedIn":"本帳號將在 %s 內被刪除。您是否要取消刪除？","suspended":"本帳號已被停權，所有沙粒及共用皆已停用。您是否要解除停權？","restoreButton":"解除停權","logoutButton":"登出"},"_accountProfileEditor":{"picture":{"label":"頭像：","uploadButton":"上傳圖片","subtext":"此圖片將作為您的頭像，讓別人可以更容易認出您。","maxSize":"最大 512x512px，64KiB。"},"fullName":{"label":"全名：","placeholder":"全名","subtext":"您公開給他人看的全名，但並不一定要是真名。"},"handle":{"label":"網名","placeholder":"網名","hint":"網名必須以小寫英文字母開頭，並只包含小寫英文字母、數字、以及底線。","subtext":"有些應用程式會使用網名作為您的公開名稱。網名必須以小寫英文字母開頭，並只包含小寫英文字母、數字、以及底線。"},"emailList":{"label":"電郵："},"pronoun":{"label":"性別代名詞：","neutral":"他（不指定）","male":"他（男性）","female":"她（女性）","robot":"它（機器人）"},"agreedToTerms":{"agreeTo":"我已經閱讀且同意","termsOfServiceLink":"服務條款","and":"以及","privacyLink":"隱私權政策"},"saveButton":"儲存","continueButton":"繼續","cancelButton":"取消"}},"loginButtons":{"icon":{"info":"i"},"loginButtons":{"createButton":"建立帳號","signInButton":"登入"},"accountButtons":{"loading":"正在讀取……"},"_loginButtonsLoggedInDropdown":{"revealIdentity":"出示身份","title":"帳號","sendFeedback":"提供意見","referralProgram":"推薦計畫","signOutButton":"登出"},"_loginButtonsLoggedOutDropdown":{"demo":{"title":"Create account","explanation":"Your demo data will be transferred to your new account."},"title":"登入","shared":{"explanation":"您需要登入才能讓夥伴們知道您是誰。"},"signInButton":"登入","anonymousButton":"維持匿名"},"oauthLoginButton":{"with":"以"},"devLoginForm":{"label":"以管理者帳號","hint":"收合","alice":"Alice（管理者）","bob":"Bob","carol":"Carol","dave":"Dave","customPlaceholder":"Eve","customButton":"自訂"},"loginButtonsList":{"configure":"設定登入","troubleshooting":"遇到問題嗎？","about":"關於沙聚雲"},"emailLoginForm":{"description":"以電子郵件","sendButtonText":"寄送登入郵件"},"emailAuthenticationForm":{"hint":"電郵登入讓您可以不需記憶密碼。相對的，每次您登入的時候都會收到一封記載有認證碼的郵件。您在每一台裝置上只需要登入一次即可。","token":{"explanation":"已將登入郵件寄送至 %s，可能需要數分鐘才會送達。","label":"輸入認證碼：","confirmButton":"確定","cancelButton":"取消"},"disabled":"郵件登入功能已被停用。若有需求，請聯絡管理者。"},"ldapLoginForm":{"username":{"label":"username","hint":"This is your LDAP username. It will be the name that you normally use to log in to other internal services at your company."},"password":{"label":"password","hint":"This is your LDAP password. It will be the password that you normally use to log in to other internal services at your company."},"loginButton":"Log in"},"_loginButtonsSeparator":{"or":"或"}},"identity":{"identityLoginInterstitial":{"title":"正在登入……","loading":"正在讀取……","logoutButton":"返回","cancelButton":"取消","goBackButton":"返回","create":"正在建立帳號……","createFailed":"無法建立帳號。","loginAccount":"正在登入帳號……","loginAccountFailed":"無法登入帳號。","loginToken":"正在重新登入帳號……","loginTokenFailed":"無法復原登入狀況。","link":"正在連結憑證和帳號……","success":"成功從 %s 將憑證斷開。","explanation":"您已經認證了這個身份","warning":"但這個身份無法拿來登入任何帳號。","moreIdentities":"以下的身份可以拿來登入您的帳號：","unlinkButton":{"hint":"將您的身份從這個帳號斷開。","text":"斷開"},"unlink":{"explanation":"您是否確定要把您的身份從 %s 的帳號斷開？","unlinkButton":"斷開","cancelButton":"取消"}},"identityManagementButtons":{"toggleLogin":{"label":"此身份可以用來登入您的帳號。","unlinkButton":"斷開"}},"loginIdentitiesOfLinkedAccounts":{"show":{"explanation":"有其他帳號共享此身份，並擁有以下登入身份：","unlinkButton":"斷開","hideButton":"隱藏其他帳號"},"hide":{"explanation":"有其他帳號共享此身份。","showButton":"顯示其他帳號"}},"identityCardSignInButton":{"hint":"以 %s 登入","title":"登入"},"accountCardSignInButton":{"reSign":"要登出並改用其他帳號登入，請選擇登入方式：","howToSignIn":"選擇登入方式："}},"tokenTemplates":{"_emailTokenError":{"title":"無效的認證碼。您可能有打錯字，或是已經超過認證碼有效期限的 15 分鐘。"},"_emailLinkIdentityError":{"title":"若要驗證電郵信箱，您需要用用建立這個連結的帳號登入。"},"addNewVerifiedEmailPowerboxConfiguration":{"email":{"placeholder":"輸入電郵地址","selectButton":"選擇","sendButton":"寄送驗證郵件"},"sendingTo":"寄送到 %s","resetButton":"嘗試別的地址","token":{"placeholder":"輸入您的認證碼","confirmButton":"確定"},"verifying":"正在檢驗認證碼……"}}},"wizard":{"setupWizardLoading":{"loading":"正在讀取……"},"setupWizardProgressBar":{"identityProviders":"登入提供者","organization":"組織","email":"電郵","preinstalledApps":"預裝應用程式","adminAccount":"管理者帳號"},"setupWizardHelpFooter":{"needHelp":"需要幫助嗎？","documentation":"文件","emailSupport":"電郵支援","systemLog":"系統記錄"},"setupWizardSystemLog":{"header":{"label":"系統記錄","downloadButton":"下載記錄"},"permissionDenied":"因為您不是管理者，也沒有設置用 token，所以無法察看系統記錄。"},"setupWizardIntro":{"title":"歡迎使用沙聚雲！","revisitWizardLink":"重新開始設定精靈","noIdpEnabled":{"explanation":"這個沙聚雲尚未被設定完成，可是因為 <code>sandstorm.conf</code> 的變更，您需要重新設定登入提供者。"},"idpEnabled":{"explanation":"本沙聚雲伺服器已經可以上線。您可以調整伺服器設定，包括使用哪些登入提供者以及沙聚雲如何寄送郵件等。","turnUserIntoAdmin":"您也可以把現存的使用者提升為管理者。","isAdmin":{"adminPanelLink":"前往管理面板","suButton":"給予自己管理者權限"},"isNotAdmin":{"explanation":"要建立管理者帳號，請登入：","signInLabel":"登入","signInButton":"登入以成為管理者"}},"setupButton":"開始設置沙聚雲"},"setupWizardIdentity":{"title":"登入提供者","explanation1":"您需要建立帳號才能使用沙聚雲。所有沙聚雲帳號皆有綁定登入提供者。您將會使用該提供者給予的登入資訊認證為本伺服器的首位管理者。","explanation2":"設定您想要使用的登入提供者。","nextButton":"下一步","backButton":"上一步"},"setupWizardOrganization":{"title":"組織設定","error":"無法儲存變更：%s","membership":{"title":"組織成員","emailToggle":{"label":"以電郵信箱認證的使用者","subtext":"因為電郵登入尚未設定，故無法使用。"},"emailDomain":{"label":"網域：","subtext":"使用此網域電郵註冊的使用者將自動提升權限為組織成員（使用者）。"},"gappsToggle":{"label":"以 G Suite 認證的使用者","subtext":"因為 Google 登入尚未設定，故無法使用。"},"gappsDomain":{"label":"網域：","subtext":"使用此網域 G Suite 註冊的使用者將自動提升權限為組織成員（使用者）。"},"ldapToggle":{"label":"以 LDAP 認證的使用者","subtext":"因為 LDAP 登入尚未設定，故無法使用。"},"samlToggle":{"label":"以 SAML 認證的使用者","subtext":"因為 SAML 登入尚未設定，故無法使用。"}},"settings":{"title":"組織設定","disallowGuests":{"label":"禁止組織成員以外的使用者進行協作。","subtext":"只有以組織成員身份登入的使用者可以打開您的共享連結。訪客存取將被停用。"},"shareContacts":{"label":"讓所有組織成員能夠看見彼此","subtext":"此功能會讓組織成員自動把其他組織成員都加入聯絡人清單，以便共享沙粒。若您有使用者需要隱藏身份，請停用本功能。"}},"saveButton":"儲存並繼續","backButton":"上一步"},"setupWizardEmailConfig":{"title":"電郵寄送","explanation":"沙聚雲需要知道如何寄送電郵。除非您使用電郵登入，否則可以跳過此步驟。若您跳過此步驟，直到您設定完成為止電郵相關的功能皆無法使用。","moreDetail":"<strong>沙聚雲使用電郵的方法：</strong>沙聚雲寄送通知郵件，並且也允許應用程式寄送。每個應用程式都有一個獨自的電郵位址，所以沙聚雲需要能從可變的「來自」位置寄信。所以 Gmail SMTP 不管用，請採用別的免費服務。更多資訊請<a href=\"https://docs.sandstorm.io/en/latest/administering/email/#outgoing-smtp\">參見此處</a>。","error":"無法儲存變更：%s","hostPort":{"host":"SMTP 主機位址","port":"連接埠","subtext":"SMTP（未加密或使用 StartTLS）使用 25 埠，TLS 加密 SMTPS 使用 465 埠。"},"username":{"label":"使用者名聲（選填）"},"password":{"label":"密碼（選填）"},"email":{"label":"沙聚雲伺服器本身的電郵位址","subtext":"沙聚雲會用這個位址來寄送登入郵件或授予存取請求的通知等郵件。在 SMTP envelope 中也會使用此位址。"},"saveButton":"儲存並繼續","skipButton":"現在不要","testButton":"測試","backButton":"上一步"},"setupWizardPreinstalled":{"title":"預裝應用程式設定","explanation":"請稍等，沙聚雲正在安裝下列應用程式（生產力套件，對大部分使用者都有用）。在初期設定結束後，您可以在管理者介面設定要預裝哪些應用程式。","downloaded":"已下載","failed":"失敗","loading":"正在讀取應用程式索引，請稍候……","nextButton":"下一步","skipButton":"現在不要","backButton":"上一步"},"setupWizardLoginUser":{"title":"建立管理者帳號","firstLogin":{"title":"確認您的個人資料"},"isAdmin":"太棒了！您的帳號現已擁有管理者權限。","hasAdmin":{"explanation":"本沙聚雲伺服器已有管理者，但若您有需要還是可以將自己設定為管理者。","adminButton":"成為管理者"},"makeYouAnAdmin":"正在將您設定為本伺服器的管理者……","loggedOut":{"explanation":"您現在可以建立本伺服器的管理者帳號了。","description":"登入並建立您的管理者帳號","dialogLabel":"建立帳號"},"finishButton":"完成","backButton":"上一步"},"setupWizardSuccess":{"title":"大功告成！","serverIsReady":"您的沙聚雲伺服器現在可供使用。","whatsNext":"您皆下來想做什麼呢？","addUsersLink":"新增使用者","editSettingsLink":"編輯其他設定","sandstormLink":"開始使用沙聚雲","backButton":"上一步"},"setupWizardVerifyToken":{"title":"正在檢查驗證碼……","explanation":"抱歉，您的設置用驗證碼過期了。","howToFix":"請在伺服器的命令列執行 <code>sandstorm admin-token</code> 來產生新的驗證碼進行設置。"},"setupWizardTokenExpired":{"title":"需要設置驗證碼","alreadySetup":"本伺服器已經設置完成，建議您從管理者介面","adminRootLink":"來進行設定。","explanation":"要使用本伺服器的設置頁面，您需要產生新的設置驗證碼。","howToFix":"請在伺服器的命令列執行 <code>sandstorm admin-token</code> 並遵循顯示步驟處理。"}},"packages":{"topbar":{"updated":"沙聚雲已有更新 &mdash; 點擊此處重新整理頁面","apps":"應用程式","grains":"沙粒","oasis":"在 Oasis 上使用沙聚雲進行協作：","createAccount":"建立免費帳號 »","demoHint":"您的試用帳號資料會被保留。","selfHostHint":"或是為您自己的團隊安裝沙聚雲（沙聚雲是開源的）:","signUp":"進行註冊：","close":"關閉"},"powerbox":{"title":"Powerbox 請求","inputToken":"請輸入 api token：","importToken":"匯入 token","selected":"已選定：","selectOne":"選擇一項：","noGrain":"沒有沙粒可以提供符合請求的介面。","loading":"正在讀取……"}}});
TAPi18n._registerServerTranslator("zh-TW", namespace);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}},"shared":{"admin.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// shared/admin.js                                                                                                     //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
// TODO(someday): split out fields by locus                                                                            // 17
// isEnabled is available on both client and server                                                                    // 18
// getLoginId is only used on server                                                                                   // 19
// initiateLogin and loginTemplate are only used on client                                                             // 20
function serviceEnabled(name) {                                                                                        // 22
  var setting = Settings.findOne({                                                                                     // 23
    _id: name                                                                                                          // 23
  });                                                                                                                  // 23
  return setting && !!setting.value;                                                                                   // 24
}                                                                                                                      // 25
                                                                                                                       //
Accounts.loginServices.github = {                                                                                      // 27
  isEnabled: function () {                                                                                             // 28
    return serviceEnabled("github");                                                                                   // 29
  },                                                                                                                   // 30
  getLoginId: function (identity) {                                                                                    // 32
    return identity.services.github.username;                                                                          // 33
  },                                                                                                                   // 34
  initiateLogin: function (loginId) {                                                                                  // 36
    Meteor.loginWithGithub();                                                                                          // 37
    return {                                                                                                           // 38
      oneClick: true                                                                                                   // 38
    };                                                                                                                 // 38
  },                                                                                                                   // 39
  loginTemplate: {                                                                                                     // 41
    name: "oauthLoginButton",                                                                                          // 42
    priority: 1,                                                                                                       // 43
    data: {                                                                                                            // 44
      method: "loginWithGithub",                                                                                       // 45
      name: "github",                                                                                                  // 46
      displayName: "GitHub",                                                                                           // 47
      linkingNewCredential: false                                                                                      // 48
    }                                                                                                                  // 44
  }                                                                                                                    // 41
};                                                                                                                     // 27
Accounts.loginServices.google = {                                                                                      // 53
  isEnabled: function () {                                                                                             // 54
    return serviceEnabled("google");                                                                                   // 55
  },                                                                                                                   // 56
  getLoginId: function (identity) {                                                                                    // 58
    return identity.services.google.email;                                                                             // 59
  },                                                                                                                   // 60
  initiateLogin: function (loginId) {                                                                                  // 62
    Meteor.loginWithGoogle({                                                                                           // 63
      loginHint: loginId                                                                                               // 63
    });                                                                                                                // 63
    return {                                                                                                           // 64
      oneClick: true                                                                                                   // 64
    };                                                                                                                 // 64
  },                                                                                                                   // 65
  loginTemplate: {                                                                                                     // 67
    name: "oauthLoginButton",                                                                                          // 68
    priority: 2,                                                                                                       // 69
    data: {                                                                                                            // 70
      method: "loginWithGoogle",                                                                                       // 71
      name: "google",                                                                                                  // 72
      displayName: "Google",                                                                                           // 73
      linkingNewCredential: false                                                                                      // 74
    }                                                                                                                  // 70
  }                                                                                                                    // 67
};                                                                                                                     // 53
Accounts.loginServices.email = {                                                                                       // 79
  isEnabled: function () {                                                                                             // 80
    return serviceEnabled("emailToken");                                                                               // 81
  },                                                                                                                   // 82
  getLoginId: function (identity) {                                                                                    // 84
    return identity.services.email.email;                                                                              // 85
  },                                                                                                                   // 86
  initiateLogin: function (loginId) {                                                                                  // 88
    return {                                                                                                           // 89
      form: true                                                                                                       // 89
    };                                                                                                                 // 89
  },                                                                                                                   // 90
  loginTemplate: {                                                                                                     // 92
    name: "emailLoginForm",                                                                                            // 93
    priority: 10                                                                                                       // 94
  }                                                                                                                    // 92
};                                                                                                                     // 79
Accounts.loginServices.ldap = {                                                                                        // 98
  isEnabled: function () {                                                                                             // 99
    return serviceEnabled("ldap");                                                                                     // 100
  },                                                                                                                   // 101
  getLoginId: function (identity) {                                                                                    // 103
    return identity.services.ldap.username;                                                                            // 104
  },                                                                                                                   // 105
  initiateLogin: function (loginId) {                                                                                  // 107
    return {                                                                                                           // 108
      form: true                                                                                                       // 108
    };                                                                                                                 // 108
  },                                                                                                                   // 109
  loginTemplate: {                                                                                                     // 111
    name: "ldapLoginForm",                                                                                             // 112
    priority: 21 // Put it at the bottom of the list.                                                                  // 113
                                                                                                                       //
  }                                                                                                                    // 111
};                                                                                                                     // 98
Accounts.loginServices.saml = {                                                                                        // 117
  isEnabled: function () {                                                                                             // 118
    return serviceEnabled("saml");                                                                                     // 119
  },                                                                                                                   // 120
  getLoginId: function (identity) {                                                                                    // 122
    return identity.services.saml.id;                                                                                  // 123
  },                                                                                                                   // 124
  initiateLogin: function (loginId) {                                                                                  // 126
    Meteor.loginWithSaml();                                                                                            // 127
    return {                                                                                                           // 128
      oneClick: true                                                                                                   // 128
    };                                                                                                                 // 128
  },                                                                                                                   // 129
  loginTemplate: {                                                                                                     // 131
    name: "samlLoginForm",                                                                                             // 132
    priority: 20 // Put it at the bottom of the list.                                                                  // 133
                                                                                                                       //
  }                                                                                                                    // 131
};                                                                                                                     // 117
Accounts.loginServices.demo = {                                                                                        // 137
  isEnabled: function () {                                                                                             // 138
    return false; // You can never authenticate as a demo user.                                                        // 139
  },                                                                                                                   // 140
  getLoginId: function (identity) {                                                                                    // 142
    return identity._id;                                                                                               // 143
  },                                                                                                                   // 144
  initiateLogin: function (loginId) {                                                                                  // 146
    return {                                                                                                           // 147
      demoCannotLogin: true                                                                                            // 147
    };                                                                                                                 // 147
  },                                                                                                                   // 148
  loginTemplate: {}                                                                                                    // 150
};                                                                                                                     // 137
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"dev-accounts.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// shared/dev-accounts.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
Accounts.loginServices.dev = {                                                                                         // 17
  isEnabled: function () {                                                                                             // 18
    return globalDb.allowDevAccounts();                                                                                // 19
  },                                                                                                                   // 20
  getLoginId: function (identity) {                                                                                    // 22
    return identity.services.dev.name;                                                                                 // 23
  },                                                                                                                   // 24
  initiateLogin: function (loginId) {                                                                                  // 26
    loginDevAccount(loginId);                                                                                          // 27
    return {                                                                                                           // 28
      oneClick: true                                                                                                   // 28
    };                                                                                                                 // 28
  },                                                                                                                   // 29
  loginTemplate: {                                                                                                     // 31
    name: "devLoginForm",                                                                                              // 32
    priority: -10 // Put it at the top.                                                                                // 33
                                                                                                                       //
  }                                                                                                                    // 31
};                                                                                                                     // 17
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"grain-shared.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// shared/grain-shared.js                                                                                              //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
Meteor.methods({                                                                                                       // 17
  // Methods defined in this file have meaningful latency compensation (client-side prediction)                        // 18
  // potential.                                                                                                        // 19
  //                                                                                                                   // 20
  // Methods for which latency compensation makes no sense are defined in grain-server.js.                             // 21
  markActivityReadByOwner: function (grainId) {                                                                        // 23
    check(grainId, String);                                                                                            // 24
    check(this.userId, String);                                                                                        // 25
    Grains.update({                                                                                                    // 27
      _id: grainId,                                                                                                    // 27
      userId: this.userId                                                                                              // 27
    }, {                                                                                                               // 27
      $set: {                                                                                                          // 28
        "ownerSeenAllActivity": true                                                                                   // 28
      }                                                                                                                // 28
    });                                                                                                                // 28
  },                                                                                                                   // 29
  markActivityRead: function (grainId, obsolete) {                                                                     // 31
    check(grainId, String);                                                                                            // 32
                                                                                                                       //
    if (!this.userId) {                                                                                                // 34
      throw new Meteor.Error(403, "Not logged in.");                                                                   // 35
    }                                                                                                                  // 36
                                                                                                                       //
    ApiTokens.update({                                                                                                 // 38
      "grainId": grainId,                                                                                              // 38
      "owner.user.accountId": Meteor.userId()                                                                          // 38
    }, {                                                                                                               // 38
      $set: {                                                                                                          // 39
        "owner.user.seenAllActivity": true                                                                             // 39
      }                                                                                                                // 39
    }, {                                                                                                               // 39
      multi: true                                                                                                      // 39
    });                                                                                                                // 39
  },                                                                                                                   // 40
  moveGrainsToTrash: function (grainIds) {                                                                             // 42
    var _this = this;                                                                                                  // 42
                                                                                                                       //
    check(grainIds, [String]);                                                                                         // 43
                                                                                                                       //
    if (this.userId) {                                                                                                 // 45
      Grains.update({                                                                                                  // 46
        userId: {                                                                                                      // 46
          $eq: this.userId                                                                                             // 46
        },                                                                                                             // 46
        _id: {                                                                                                         // 47
          $in: grainIds                                                                                                // 47
        },                                                                                                             // 47
        trashed: {                                                                                                     // 48
          $exists: false                                                                                               // 48
        }                                                                                                              // 48
      }, {                                                                                                             // 46
        $set: {                                                                                                        // 49
          trashed: new Date()                                                                                          // 49
        }                                                                                                              // 49
      }, {                                                                                                             // 49
        multi: true                                                                                                    // 50
      });                                                                                                              // 50
      ApiTokens.update({                                                                                               // 52
        grainId: {                                                                                                     // 52
          $in: grainIds                                                                                                // 52
        },                                                                                                             // 52
        "owner.user.accountId": Meteor.userId(),                                                                       // 53
        trashed: {                                                                                                     // 54
          $exists: false                                                                                               // 54
        }                                                                                                              // 54
      }, {                                                                                                             // 52
        $set: {                                                                                                        // 55
          "trashed": new Date()                                                                                        // 55
        }                                                                                                              // 55
      }, {                                                                                                             // 55
        multi: true                                                                                                    // 56
      });                                                                                                              // 56
                                                                                                                       //
      if (!this.isSimulation) {                                                                                        // 58
        var grainsOwned = Grains.find({                                                                                // 59
          userId: {                                                                                                    // 60
            $eq: this.userId                                                                                           // 60
          },                                                                                                           // 60
          _id: {                                                                                                       // 61
            $in: grainIds                                                                                              // 61
          }                                                                                                            // 61
        }, {                                                                                                           // 59
          fields: {                                                                                                    // 62
            _id: 1                                                                                                     // 62
          }                                                                                                            // 62
        });                                                                                                            // 62
        grainsOwned.forEach(function (grain) {                                                                         // 64
          Sessions.remove({                                                                                            // 65
            grainId: grain._id                                                                                         // 65
          });                                                                                                          // 65
                                                                                                                       //
          try {                                                                                                        // 66
            _this.connection.sandstormBackend.shutdownGrain(grain._id, _this.userId).await();                          // 67
          } catch (err) {                                                                                              // 68
            console.error("Failed to shutdown trashed grain", grain._id, err);                                         // 69
          }                                                                                                            // 70
        });                                                                                                            // 71
      }                                                                                                                // 72
    }                                                                                                                  // 73
  },                                                                                                                   // 74
  moveGrainsOutOfTrash: function (grainIds) {                                                                          // 76
    check(grainIds, [String]);                                                                                         // 77
                                                                                                                       //
    if (this.userId) {                                                                                                 // 79
      Grains.update({                                                                                                  // 80
        userId: {                                                                                                      // 80
          $eq: this.userId                                                                                             // 80
        },                                                                                                             // 80
        _id: {                                                                                                         // 81
          $in: grainIds                                                                                                // 81
        },                                                                                                             // 81
        trashed: {                                                                                                     // 82
          $exists: true                                                                                                // 82
        }                                                                                                              // 82
      }, {                                                                                                             // 80
        $unset: {                                                                                                      // 83
          trashed: 1                                                                                                   // 83
        }                                                                                                              // 83
      }, {                                                                                                             // 83
        multi: true                                                                                                    // 84
      });                                                                                                              // 84
      ApiTokens.update({                                                                                               // 86
        grainId: {                                                                                                     // 86
          $in: grainIds                                                                                                // 86
        },                                                                                                             // 86
        "owner.user.accountId": Meteor.userId(),                                                                       // 87
        "trashed": {                                                                                                   // 88
          $exists: true                                                                                                // 88
        }                                                                                                              // 88
      }, {                                                                                                             // 86
        $unset: {                                                                                                      // 89
          "trashed": 1                                                                                                 // 89
        }                                                                                                              // 89
      }, {                                                                                                             // 89
        multi: true                                                                                                    // 90
      });                                                                                                              // 90
    }                                                                                                                  // 91
  },                                                                                                                   // 92
  deleteGrain: function (grainId) {                                                                                    // 94
    check(grainId, String);                                                                                            // 95
                                                                                                                       //
    if (this.userId) {                                                                                                 // 97
      var grainsQuery = {                                                                                              // 98
        _id: grainId,                                                                                                  // 99
        userId: this.userId,                                                                                           // 100
        trashed: {                                                                                                     // 101
          $exists: true                                                                                                // 101
        }                                                                                                              // 101
      };                                                                                                               // 98
      var numDeleted = 0;                                                                                              // 104
                                                                                                                       //
      if (this.isSimulation) {                                                                                         // 105
        numDeleted = Grains.remove(grainsQuery);                                                                       // 106
      } else {                                                                                                         // 107
        numDeleted = globalDb.deleteGrains(grainsQuery, globalBackend, isDemoUser() ? "demoGrain" : "grain");          // 108
      } // Usually we don't automatically remove user-owned tokens that have become invalid,                           // 110
      // because if we did their owner might become confused as to why they have mysteriously                          // 113
      // disappeared. In this particular case, however, for tokens held by the grain owner,                            // 114
      // there should be no confusion. Indeed, it would be more confusing *not* to remove these                        // 115
      // tokens, because then the grain could still show up in the trash bin as a "shared with me"                     // 116
      // grain after the owner clicks "delete permanently".                                                            // 117
                                                                                                                       //
                                                                                                                       //
      var apiTokensQuery = {                                                                                           // 118
        grainId: grainId,                                                                                              // 119
        "owner.user.accountId": Meteor.userId(),                                                                       // 120
        "trashed": {                                                                                                   // 121
          $exists: true                                                                                                // 121
        }                                                                                                              // 121
      };                                                                                                               // 118
                                                                                                                       //
      if (numDeleted > 0) {                                                                                            // 124
        if (this.isSimulation) {                                                                                       // 125
          ApiTokens.remove(apiTokensQuery);                                                                            // 126
        } else {                                                                                                       // 127
          globalDb.removeApiTokens(apiTokensQuery);                                                                    // 128
        }                                                                                                              // 129
      }                                                                                                                // 130
    }                                                                                                                  // 131
  },                                                                                                                   // 132
  forgetGrain: function (grainId, obsolete) {                                                                          // 134
    check(grainId, String);                                                                                            // 135
                                                                                                                       //
    if (!this.userId) {                                                                                                // 137
      throw new Meteor.Error(403, "Must be logged in to forget a grain.");                                             // 138
    }                                                                                                                  // 139
                                                                                                                       //
    var query = {                                                                                                      // 141
      grainId: grainId,                                                                                                // 142
      "owner.user.accountId": this.userId,                                                                             // 143
      "trashed": {                                                                                                     // 144
        $exists: true                                                                                                  // 144
      }                                                                                                                // 144
    };                                                                                                                 // 141
                                                                                                                       //
    if (this.isSimulation) {                                                                                           // 147
      ApiTokens.remove(query);                                                                                         // 148
    } else {                                                                                                           // 149
      globalDb.removeApiTokens(query);                                                                                 // 150
    }                                                                                                                  // 151
  }                                                                                                                    // 152
});                                                                                                                    // 17
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"install.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// shared/install.js                                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
isSafeDemoAppUrl = function () {                                                                                       // 17
    function isSafeDemoAppUrl(url) {                                                                                   // 17
        // For demo accounts, we allow using a bare hash with no URL (which will never upload a new app)               // 18
        // and we allow specifying a sandstorm.io URL.                                                                 // 19
        return !url || url.lastIndexOf("http://sandstorm.io/", 0) === 0 || url.lastIndexOf("https://sandstorm.io/", 0) === 0 || url.lastIndexOf("https://alpha-j7uny7u376jnimcsx34c.sandstorm.io/", 0) === 0 || url.lastIndexOf("https://app-index.sandstorm.io/", 0) === 0;
    }                                                                                                                  // 25
                                                                                                                       //
    return isSafeDemoAppUrl;                                                                                           // 17
}();                                                                                                                   // 17
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

},"testing.js":function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// shared/testing.js                                                                                                   //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                                               // 2
// All rights reserved.                                                                                                // 3
//                                                                                                                     // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                                     // 5
// you may not use this file except in compliance with the License.                                                    // 6
// You may obtain a copy of the License at                                                                             // 7
//                                                                                                                     // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        // 9
//                                                                                                                     // 10
// Unless required by applicable law or agreed to in writing, software                                                 // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                                   // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            // 13
// See the License for the specific language governing permissions and                                                 // 14
// limitations under the License.                                                                                      // 15
var isTesting = Meteor.settings && Meteor.settings.public && Meteor.settings.public.isTesting; // jscs:disable requireCamelCaseOrUpperCaseIdentifiers
                                                                                                                       //
if (isTesting) {                                                                                                       // 21
  if (Meteor.isServer) {                                                                                               // 22
    var clearUser = function (id) {                                                                                    // 22
      UserActions.remove({                                                                                             // 24
        userId: id                                                                                                     // 24
      });                                                                                                              // 24
      globalDb.removeApiTokens({                                                                                       // 25
        userId: id                                                                                                     // 25
      });                                                                                                              // 25
      Grains.find({                                                                                                    // 26
        userId: id                                                                                                     // 26
      }).forEach(function (grain) {                                                                                    // 26
        globalBackend.deleteGrain(grain._id);                                                                          // 27
      });                                                                                                              // 28
      Grains.remove({                                                                                                  // 30
        userId: id                                                                                                     // 30
      });                                                                                                              // 30
      Meteor.users.remove({                                                                                            // 31
        _id: id                                                                                                        // 31
      });                                                                                                              // 31
    };                                                                                                                 // 32
                                                                                                                       //
    Meteor.methods({                                                                                                   // 34
      createMockGithubUser: function () {                                                                              // 35
        Meteor.users.update({                                                                                          // 36
          _id: "Py8fwsaryQNGBuiXb"                                                                                     // 36
        }, {                                                                                                           // 36
          $set: {                                                                                                      // 37
            createdAt: new Date("2014-08-11T21:44:04.147Z"),                                                           // 37
            isAdmin: true,                                                                                             // 37
            lastActive: new Date("2014-08-19T09:58:39.676Z"),                                                          // 37
            profile: {                                                                                                 // 37
              name: "Github User"                                                                                      // 37
            },                                                                                                         // 37
            services: {                                                                                                // 37
              github: {                                                                                                // 37
                accessToken: "sometoken",                                                                              // 37
                id: 1595880,                                                                                           // 37
                username: "testuser"                                                                                   // 37
              },                                                                                                       // 37
              resume: {                                                                                                // 37
                loginTokens: [{                                                                                        // 37
                  when: new Date("2099-08-13T05:16:02.356Z"),                                                          // 37
                  hashedToken: "GriUSDp+uN/K4HptwSl1wsdWfHEpS8c9KjjdqwKNo0k="                                          // 37
                }]                                                                                                     // 37
              }                                                                                                        // 37
            },                                                                                                         // 37
            signupKey: "admin"                                                                                         // 37
          }                                                                                                            // 37
        }, {                                                                                                           // 37
          upsert: true                                                                                                 // 38
        });                                                                                                            // 38
      },                                                                                                               // 39
      clearMockGithubUser: function () {                                                                               // 41
        clearUser("Py8fwsaryQNGBuiXb");                                                                                // 42
      },                                                                                                               // 43
      createMockGoogleUser: function () {                                                                              // 45
        Meteor.users.update({                                                                                          // 46
          _id: "6WJcRo2gg2Ysuxsok"                                                                                     // 46
        }, {                                                                                                           // 46
          $set: {                                                                                                      // 47
            createdAt: new Date("2014-08-21T07:52:55.581Z"),                                                           // 47
            profile: {                                                                                                 // 47
              name: "Google User"                                                                                      // 47
            },                                                                                                         // 47
            services: {                                                                                                // 47
              google: {                                                                                                // 47
                accessToken: "sometoken",                                                                              // 47
                expiresAt: 4562182723000,                                                                              // 47
                id: "116893057283177439912",                                                                           // 47
                verified_email: true,                                                                                  // 47
                name: "Google User",                                                                                   // 47
                given_name: "Google",                                                                                  // 47
                family_name: "User",                                                                                   // 47
                picture: "https://lh3.googleusercontent.com/-XdUIqdMkCWA/AAAAAAAAAAI/AAAAAAAAAAA/4252rscbv5M/photo.jpg",
                locale: "en",                                                                                          // 47
                gender: "male"                                                                                         // 47
              },                                                                                                       // 47
              resume: {                                                                                                // 47
                loginTokens: [{                                                                                        // 47
                  when: new Date("2099-08-21T07:52:55.592Z"),                                                          // 47
                  hashedToken: "cbJGxLGKW3f0j7Ehit77hdK58W7xuPjzZhGHgKhyddo="                                          // 47
                }]                                                                                                     // 47
              }                                                                                                        // 47
            },                                                                                                         // 47
            signupKey: "admin"                                                                                         // 47
          }                                                                                                            // 47
        }, {                                                                                                           // 47
          upsert: true                                                                                                 // 48
        });                                                                                                            // 48
      },                                                                                                               // 49
      clearMockGoogleUser: function () {                                                                               // 51
        clearUser("6WJcRo2gg2Ysuxsok");                                                                                // 52
      },                                                                                                               // 53
      fetchAppIndexTest: function () {                                                                                 // 55
        globalDb.collections.appIndex.remove({});                                                                      // 56
        SandstormAutoupdateApps.updateAppIndex(this.connection.sandstormDb);                                           // 57
      }                                                                                                                // 58
    });                                                                                                                // 34
  }                                                                                                                    // 60
                                                                                                                       //
  mockLoginGithub = function () {                                                                                      // 62
    Meteor.call("createMockGithubUser", function (err) {                                                               // 63
      if (err) {                                                                                                       // 64
        console.log(err);                                                                                              // 65
      }                                                                                                                // 66
                                                                                                                       //
      window.localStorage.setItem("Meteor.loginToken", "F0S6luPxIJV--y_GAkBKOMuWMYnnqgG3UMv9M-DIs2f");                 // 68
      window.localStorage.setItem("Meteor.loginTokenExpires", "Mon Nov 10 2099 21:16:02 GMT-0800 (PST)");              // 69
      window.localStorage.setItem("Meteor.userId", "Py8fwsaryQNGBuiXb");                                               // 70
      window.location.reload();                                                                                        // 71
    });                                                                                                                // 72
  };                                                                                                                   // 73
                                                                                                                       //
  clearMockGithubUser = function () {                                                                                  // 75
    Meteor.call("clearMockGithubUser", function (err) {                                                                // 76
      if (err) {                                                                                                       // 77
        console.log(err);                                                                                              // 78
      }                                                                                                                // 79
    });                                                                                                                // 80
  };                                                                                                                   // 81
                                                                                                                       //
  mockLoginGoogle = function () {                                                                                      // 83
    Meteor.call("createMockGoogleUser", function (err) {                                                               // 84
      if (err) {                                                                                                       // 85
        console.log(err);                                                                                              // 86
      }                                                                                                                // 87
                                                                                                                       //
      window.localStorage.setItem("Meteor.loginToken", "P3ffUfVJtptyVX2IPUfNDZY0F3b-GIZ-WQf7w3GdL21");                 // 89
      window.localStorage.setItem("Meteor.loginTokenExpires", "Tue Nov 18 2099 23:52:55 GMT-0800 (PST)");              // 90
      window.localStorage.setItem("Meteor.userId", "6WJcRo2gg2Ysuxsok");                                               // 91
      window.location.reload();                                                                                        // 92
    });                                                                                                                // 93
  };                                                                                                                   // 94
                                                                                                                       //
  clearMockGoogleUser = function () {                                                                                  // 96
    Meteor.call("clearMockGoogleUser", function (err) {                                                                // 97
      if (err) {                                                                                                       // 98
        console.log(err);                                                                                              // 99
      }                                                                                                                // 100
    });                                                                                                                // 101
  };                                                                                                                   // 102
}                                                                                                                      // 103
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}}},{
  "extensions": [
    ".js",
    ".json"
  ]
});
require("./server/accounts/credentials/credentials-server.js");
require("./server/accounts/email-token/token-server.js");
require("./server/accounts/ldap/ldap-server.js");
require("./server/accounts/saml/saml-server.js");
require("./server/accounts/accounts-server.js");
require("./server/accounts/accounts-ui-methods.js");
require("./server/accounts/accounts-ui-server.js");
require("./server/admin/admin-alert-server.js");
require("./server/admin/admin-user-invite.js");
require("./server/admin/network-capabilities-server.js");
require("./server/admin/personalization-server.js");
require("./server/admin/system-status-server.js");
require("./server/drivers/external-ui-view.js");
require("./server/drivers/ip.js");
require("./server/drivers/mail.js");
require("./i18n/en.i18n.json");
require("./i18n/nl.i18n.json");
require("./i18n/zh-CN.i18n.json");
require("./i18n/zh-TW.i18n.json");
require("./server/00-startup.js");
require("./server/account-suspension.js");
require("./server/admin-server.js");
require("./server/backup.js");
require("./server/contacts-server.js");
require("./server/core.js");
require("./server/demo-server.js");
require("./server/desktop-notifications-server.js");
require("./server/dev-accounts-server.js");
require("./server/grain-server.js");
require("./server/hack-session.js");
require("./server/identity.js");
require("./server/install-server.js");
require("./server/installer.js");
require("./server/notifications-server.js");
require("./server/pre-meteor.js");
require("./server/proxy.js");
require("./server/sandcats.js");
require("./server/shell-server.js");
require("./server/signup-server.js");
require("./server/standalone-server.js");
require("./server/startup.js");
require("./server/stats-server.js");
require("./shared/admin.js");
require("./shared/dev-accounts.js");
require("./shared/grain-shared.js");
require("./shared/install.js");
require("./shared/testing.js");
//# sourceMappingURL=app.js.map
